Package: tfrmt
Title: Applies Display Metadata to Analysis Results Datasets
Version: 0.4.0
Authors@R: c(
    person("Alanah", "Jonas", , "alanah.x.jonas@gsk.com", role = c("aut", "cre")),
    person("Becca", "Krouse", , "becca.z.krouse@gsk.com", role = "aut"),
    person("Christina", "Fillmore", , "christina.e.fillmore@gsk.com", role = "aut",
           comment = c(ORCID = "0000-0003-0595-2302")),
    person("Ellis", "Hughes", , "ellis.h.hughes@gsk.com", role = "aut",
           comment = c(ORCID = "0000-0003-0637-4436")),
    person("Karima", "Ahmad", , "karima.j.ahmad@gsk.com", role = "aut",
           comment = c(ORCID = "0000-0002-8784-1712")),
    person("Shannon", "Haughton", , "shannon.l.haughton@gsk.com", role = "aut"),
    person("Dragoș", "Moldovan-Grünfeld", , "dragos.v.moldovan-grunfeld@gsk.com", role = "aut"),
    person("GlaxoSmithKline Research & Development Limited", role = c("cph", "fnd")),
    person("Atorus Research LLC", role = c("cph", "fnd"))
  )
Description: Creates a framework to store and apply display metadata to
    Analysis Results Datasets (ARDs). The use of 'tfrmt' allows users to
    define table format and styling without the data, and later apply the
    format to the data.
License: Apache License (>= 2)
URL: https://GSK-Biostatistics.github.io/tfrmt/,
        https://github.com/GSK-Biostatistics/tfrmt
BugReports: https://github.com/GSK-Biostatistics/tfrmt/issues
Depends: R (>= 4.2.0)
Imports: cli, dplyr, forcats, ggplot2, glue, gt (>= 1.1.0), jsonlite,
        magrittr, purrr, rlang, stringi, stringr, tibble, tidyr,
        tidyselect
Suggests: cards (>= 0.8.0), covr, ggfortify, knitr, patchwork,
        pharmaverseadam, rmarkdown, survival, testthat (>= 3.0.0),
        withr
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-GB
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-07-10 08:50:58 UTC; aj641247
Author: Alanah Jonas [aut, cre],
  Becca Krouse [aut],
  Christina Fillmore [aut] (ORCID:
    <https://orcid.org/0000-0003-0595-2302>),
  Ellis Hughes [aut] (ORCID: <https://orcid.org/0000-0003-0637-4436>),
  Karima Ahmad [aut] (ORCID: <https://orcid.org/0000-0002-8784-1712>),
  Shannon Haughton [aut],
  Dragoș Moldovan-Grünfeld [aut],
  GlaxoSmithKline Research & Development Limited [cph, fnd],
  Atorus Research LLC [cph, fnd]
Maintainer: Alanah Jonas <alanah.x.jonas@gsk.com>
Repository: CRAN
Date/Publication: 2026-07-10 09:30:02 UTC
Built: R 4.6.0; ; 2026-07-10 09:52:08 UTC; unix

Package: tfrmt
Title: Applies Display Metadata to Analysis Results Datasets
Version: 0.3.0
Authors@R: c(
    person("Becca", "Krouse", , "becca.z.krouse@gsk.com", role = c("aut", "cre")),
    person("Christina", "Fillmore", , "christina.e.fillmore@gsk.com", role = "aut",
           comment = c(ORCID = "0000-0003-0595-2302")),
    person("Ellis", "Hughes", , "ellis.h.hughes@gsk.com", role = "aut",
           comment = c(ORCID = "0000-0003-0637-4436")),
    person("Karima", "Ahmad", , "karima.j.ahmad@gsk.com", role = "aut",
           comment = c(ORCID = "0000-0002-8784-1712")),
    person("Shannon", "Haughton", , "shannon.l.haughton@gsk.com", role = "aut"),
    person("Dragoș", "Moldovan-Grünfeld", , "dragos.v.moldovan-grunfeld@gsk.com", role = "aut"),
    person("Alanah", "Jonas", , "alanah.x.jonas@gsk.com", role = "aut"),
    person("GlaxoSmithKline Research & Development Limited", role = c("cph", "fnd")),
    person("Atorus Research LLC", role = c("cph", "fnd"))
  )
Description: Creates a framework to store and apply display metadata to
    Analysis Results Datasets (ARDs). The use of 'tfrmt' allows users to
    define table format and styling without the data, and later apply the
    format to the data.
License: Apache License (>= 2)
URL: https://GSK-Biostatistics.github.io/tfrmt/,
        https://github.com/GSK-Biostatistics/tfrmt
BugReports: https://github.com/GSK-Biostatistics/tfrmt/issues
Depends: R (>= 4.2.0)
Imports: cli, dplyr, forcats, ggplot2, glue, gt (>= 1.1.0), jsonlite,
        magrittr, purrr, rlang, stringi, stringr (>= 1.6.0), tibble,
        tidyr, tidyselect
Suggests: cards (>= 0.6.0), covr, ggfortify, knitr, patchwork,
        pharmaverseadam, rmarkdown, survival, testthat (>= 3.0.0),
        withr
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-GB
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-01-23 21:12:06 UTC; bkrouse
Author: Becca Krouse [aut, cre],
  Christina Fillmore [aut] (ORCID:
    <https://orcid.org/0000-0003-0595-2302>),
  Ellis Hughes [aut] (ORCID: <https://orcid.org/0000-0003-0637-4436>),
  Karima Ahmad [aut] (ORCID: <https://orcid.org/0000-0002-8784-1712>),
  Shannon Haughton [aut],
  Dragoș Moldovan-Grünfeld [aut],
  Alanah Jonas [aut],
  GlaxoSmithKline Research & Development Limited [cph, fnd],
  Atorus Research LLC [cph, fnd]
Maintainer: Becca Krouse <becca.z.krouse@gsk.com>
Repository: CRAN
Date/Publication: 2026-01-24 06:10:02 UTC
Built: R 4.6.0; ; 2026-04-25 13:28:32 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(apply_frmt,frmt)
S3method(apply_frmt,frmt_combine)
S3method(apply_frmt,frmt_when)
S3method(as.character,frmt)
S3method(as.character,frmt_combine)
S3method(as.character,frmt_when)
S3method(as.character,span_structure)
S3method(as_json,body_plan)
S3method(as_json,col_plan)
S3method(as_json,col_style_plan)
S3method(as_json,col_style_structure)
S3method(as_json,default)
S3method(as_json,frmt)
S3method(as_json,frmt_combine)
S3method(as_json,frmt_structure)
S3method(as_json,frmt_when)
S3method(as_json,page_plan)
S3method(as_json,quosure)
S3method(as_json,quosures)
S3method(as_json,span_structure)
S3method(as_json,tfrmt)
S3method(as_length_one_quo,character)
S3method(as_length_one_quo,quosure)
S3method(as_length_one_quo,quosures)
S3method(as_vars,character)
S3method(as_vars,quosure)
S3method(as_vars,quosures)
S3method(cleaned_data_to_gt,default)
S3method(cleaned_data_to_gt,list)
S3method(format,body_plan)
S3method(format,frmt)
S3method(format,frmt_combine)
S3method(format,frmt_structure)
S3method(format,frmt_when)
S3method(print,frmt)
S3method(print,frmt_combine)
S3method(print,frmt_structure)
S3method(print,frmt_when)
export("%>%")
export(apply_frmt)
export(as_json)
export(big_n_structure)
export(body_plan)
export(cleaned_data_to_gt)
export(col_plan)
export(col_style_plan)
export(col_style_structure)
export(display_row_frmts)
export(display_val_frmts)
export(element_block)
export(element_row_grp_loc)
export(extract_data)
export(footnote_plan)
export(footnote_structure)
export(frmt)
export(frmt_combine)
export(frmt_structure)
export(frmt_when)
export(is_frmt)
export(is_frmt_combine)
export(is_frmt_structure)
export(is_frmt_when)
export(is_row_grp_structure)
export(json_to_tfrmt)
export(layer_tfrmt)
export(make_mock_data)
export(page_plan)
export(page_structure)
export(param_set)
export(prep_big_n)
export(prep_combine_vars)
export(prep_hierarchical_fill)
export(prep_label)
export(print_mock_gt)
export(print_to_ggplot)
export(print_to_gt)
export(reset_component)
export(row_grp_plan)
export(row_grp_structure)
export(shuffle_card)
export(span_structure)
export(tfrmt)
export(tfrmt_n_pct)
export(tfrmt_sigdig)
export(tfrmt_to_json)
export(update_group)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,as_tibble)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,coalesce)
importFrom(dplyr,cur_column)
importFrom(dplyr,cur_group_id)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,expr)
importFrom(dplyr,filter)
importFrom(dplyr,first)
importFrom(dplyr,group_by)
importFrom(dplyr,group_map)
importFrom(dplyr,group_split)
importFrom(dplyr,if_else)
importFrom(dplyr,inner_join)
importFrom(dplyr,lag)
importFrom(dplyr,last)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,n_distinct)
importFrom(dplyr,na_if)
importFrom(dplyr,pull)
importFrom(dplyr,reframe)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,rename_with)
importFrom(dplyr,row_number)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,starts_with)
importFrom(dplyr,summarise)
importFrom(dplyr,tally)
importFrom(dplyr,tibble)
importFrom(dplyr,ungroup)
importFrom(dplyr,vars)
importFrom(forcats,fct_inorder)
importFrom(ggplot2,.data)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,unit)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(glue,glue)
importFrom(glue,glue_collapse)
importFrom(gt,cell_borders)
importFrom(gt,cell_text)
importFrom(gt,cells_body)
importFrom(gt,cells_column_labels)
importFrom(gt,cells_column_spanners)
importFrom(gt,cells_row_groups)
importFrom(gt,cells_stub)
importFrom(gt,cols_hide)
importFrom(gt,cols_label)
importFrom(gt,default_fonts)
importFrom(gt,gt)
importFrom(gt,gt_group)
importFrom(gt,md)
importFrom(gt,opt_footnote_marks)
importFrom(gt,opt_table_font)
importFrom(gt,px)
importFrom(gt,sub_missing)
importFrom(gt,tab_footnote)
importFrom(gt,tab_header)
importFrom(gt,tab_options)
importFrom(gt,tab_source_note)
importFrom(gt,tab_spanner)
importFrom(gt,tab_stubhead)
importFrom(gt,tab_style)
importFrom(gt,text_transform)
importFrom(jsonlite,parse_json)
importFrom(jsonlite,read_json)
importFrom(jsonlite,toJSON)
importFrom(jsonlite,validate)
importFrom(magrittr,"%>%")
importFrom(purrr,discard)
importFrom(purrr,keep)
importFrom(purrr,list_flatten)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map2_chr)
importFrom(purrr,map2_dfr)
importFrom(purrr,map2_lgl)
importFrom(purrr,map_chr)
importFrom(purrr,map_dbl)
importFrom(purrr,map_dfc)
importFrom(purrr,map_dfr)
importFrom(purrr,map_lgl)
importFrom(purrr,pmap)
importFrom(purrr,pmap_chr)
importFrom(purrr,quietly)
importFrom(purrr,reduce)
importFrom(purrr,safely)
importFrom(rlang,"!!!")
importFrom(rlang,"!!")
importFrom(rlang,":=")
importFrom(rlang,"f_rhs<-")
importFrom(rlang,`!!`)
importFrom(rlang,`%||%`)
importFrom(rlang,abort)
importFrom(rlang,as_function)
importFrom(rlang,as_label)
importFrom(rlang,as_name)
importFrom(rlang,caller_call)
importFrom(rlang,caller_env)
importFrom(rlang,check_dots_empty0)
importFrom(rlang,eval_bare)
importFrom(rlang,eval_tidy)
importFrom(rlang,expr_text)
importFrom(rlang,f_lhs)
importFrom(rlang,f_rhs)
importFrom(rlang,frame_call)
importFrom(rlang,inform)
importFrom(rlang,is_empty)
importFrom(rlang,is_quosure)
importFrom(rlang,is_quosures)
importFrom(rlang,list2)
importFrom(rlang,parse_expr)
importFrom(rlang,parse_exprs)
importFrom(rlang,quo)
importFrom(rlang,quo_get_expr)
importFrom(rlang,quo_is_missing)
importFrom(rlang,quo_name)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(rlang,warn)
importFrom(stats,as.formula)
importFrom(stats,setNames)
importFrom(stringi,stri_wrap)
importFrom(stringr,str_c)
importFrom(stringr,str_count)
importFrom(stringr,str_detect)
importFrom(stringr,str_dup)
importFrom(stringr,str_extract)
importFrom(stringr,str_extract_all)
importFrom(stringr,str_glue)
importFrom(stringr,str_match)
importFrom(stringr,str_pad)
importFrom(stringr,str_remove)
importFrom(stringr,str_remove_all)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_sub)
importFrom(stringr,str_trim)
importFrom(stringr,str_which)
importFrom(tibble,add_row)
importFrom(tibble,as_tibble_row)
importFrom(tibble,is_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,crossing)
importFrom(tidyr,expand)
importFrom(tidyr,fill)
importFrom(tidyr,nest)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,replace_na)
importFrom(tidyr,separate)
importFrom(tidyr,unite)
importFrom(tidyr,unnest)
importFrom(tidyr,unnest_longer)
importFrom(utils,capture.output)

# tfrmt <a href='https://gsk-biostatistics.github.io/tfrmt/'><img src="https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/main/man/figures/tfrmt.png" align="right" alt = "tfrmt logo" style="height:139px;"/></a>

<!-- badges: start -->

[![R-CMD-check](https://github.com/GSK-Biostatistics/tfrmt/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/GSK-Biostatistics/tfrmt/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/GSK-Biostatistics/tfrmt/branch/main/graph/badge.svg)](https://app.codecov.io/gh/GSK-Biostatistics/tfrmt?branch=main)
[![status:
experimental](https://github.com/GIScience/badges/raw/master/status/experimental.svg)](https://github.com/GIScience/badges#experimental)
[![CRAN
status](https://www.r-pkg.org/badges/version/tfrmt)](https://CRAN.R-project.org/package=tfrmt)
<!-- badges: end -->

The tfrmt package provides a language for defining display-related
metadata, which can then be used to automate and easily update output
formats.

In clinical trials, displays are generally quite standard, but frequent,
highly specific formatting tweaks (e.g., rounding, footnotes, headers)
are very common. Prior to data analysis, study teams often generate mock
displays to represent the desired end product for sponsors to approve or
programmers to replicate. This process is typically highly manual and
separate from the programming itself. There is also a high importance
placed on verifying the accuracy of the results via a QC (Quality
Control) process such as double programming. Finally, there is a
movement toward an industry standard data structure for Analysis Results
Data “ARD”, which means analysis results datasets will have consistent
structures and column names. Specifically, the ARD is long, with 1
record per computed value. For more information about ARDs click
[here](https://pharmasug.org/download/sde/rtp2021/PharmaSUG-NCSDE_2021-08.pdf).

tfrmt supports a vision where:

- Mock displays are integrated with the programming workflow
- Results are QC’ed prior to formatting to reduce rework
- Standard formatting styles can be applied in as little as one line of
  code
- The ARD structure can be leveraged to accommodate a variety of tables

By reducing the amount of repetitive tasks, study teams can focus on the
quality and interpretation of the results themselves.

# Why tfrmt?

While there are many existing table-making packages in the R ecosystem,
they typically fall into one of two categories:

- Table packages that perform analyses and format the results
- Table packages that format and output existing data

By design, tfrmt is more of the latter, as it is intended to be used
after the results have been computed. What makes tfrmt unique, however,
is that it offers an intuitive interface for defining and layering
standard or custom formats that are often specific to clinical trials.
It also offers the novel ability to easily generate mock displays using
metadata that will be used for the actual displays. tfrmt is built on
top of the powerful gt package, which is intended to support a variety
of output formats in the future.

# Installation

The tfrmt package can be installed from CRAN with:

``` r
install.packages("tfrmt")
```

The development version of tfrmt can be installed with:

``` r
devtools::install_github("GSK-Biostatistics/tfrmt")
```

# Input data structure

We expect an input dataset that is long, with 1 record per computed
value. Required columns include:

- \[Optional\] 1 or more **group** columns, containing grouping values
- A single **label** column, containing row label values
- 1 or more **column** columns, containing column values
- A single **param** column, which provides a label for distinct types
  of values
- A single **value** column, containing the computed, raw data values
- \[Optional\] 1 or more **sorting_cols** columns, containing numeric
  values to be used in the row ordering

# Functionality

Here is an overview of what is possible with tfrmt:

- Create a “tfrmt” metadata object containing all formatting and
  labelling for the display
- Create mock displays based on existing sample data or no prior data
- ARD-standard compliant facilitates reuse and automation

Other benefits of tfrmt:

- Provides a tidyverse-friendly, pipeable interface
- Leverages gt as output engine, which allows for further customizations
  within gt itself

# More Info

For more information about how to build your own tfrmt mocks/tables
(like the one below!), please explore the
[vignettes](https://gsk-biostatistics.github.io/tfrmt/articles/examples.html).

<figure>
<img
src="https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/main/man/figures/gt_readme.png"
alt="Example GT Demography Table" />
<figcaption aria-hidden="true">Example GT Demography Table</figcaption>
</figure>

#------------------------------ Demography Table -------------------------------

tfrmt_demog <- tfrmt(
  group = c(rowlbl1,grp),
  label = rowlbl2,
  column = column,
  param = param,
  value = value,
  sorting_cols = c(ord1, ord2),
  body_plan = body_plan(
    frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                n = frmt("xxx"),
                                                                                pct = frmt_when("==100" ~ "",
                                                                                                "==0" ~ "",
                                                                                                TRUE ~ frmt("(xx.x %)")))),
    frmt_structure(group_val = ".default", label_val = "n", frmt("xxx")),
    frmt_structure(group_val = ".default", label_val = c("Mean", "Median", "Min","Max"), frmt("xxx.x")),
    frmt_structure(group_val = ".default", label_val = "SD", frmt("xxx.xx")),
    frmt_structure(group_val = ".default", label_val = ".default", p = frmt("")),
    frmt_structure(group_val = ".default", label_val = c("n","<65 yrs","<12 months","<25"), p = frmt_when(">0.99" ~ ">0.99",
                                                                                                          "<0.001" ~ "<0.001",
                                                                                                          TRUE ~ frmt("x.xxx", missing = "")))
  ),

  col_plan = col_plan(-grp,
                      -starts_with("ord"),
                      rowlbl1,
                      rowlbl2,
                      "Placebo", "Xanomeline Low Dose",
                      "Xanomeline High Dose", "Total", "p-value"
  )
  ,
  col_style_plan = col_style_plan(
    col_style_structure(align = c(".",","," "), col = c("Placebo", "Xanomeline Low Dose",
                                                        "Xanomeline High Dose", "Total", "p-value")),
    col_style_structure(align = "left", col = c("rowlbl1","rowlbl2"))
  ),

  row_grp_plan = row_grp_plan(
    row_grp_structure(group_val = ".default", element_block(post_space = " ")),
    label_loc = element_row_grp_loc(location = "column")
  )
)

#---------------------------------- AE Table -----------------------------------

tfrmt_ae <- tfrmt(
  group = AEBODSYS,
  label = AETERM,
  column = c(col2, col1),
  param = param,
  value = value,
  sorting_cols = c(ord1, ord2),
  body_plan = body_plan(
    frmt_structure(group_val = ".default", label_val = ".default",
                   frmt_combine("{n} {pct}",
                                n = frmt("XXX"),
                                pct = frmt_when(
                                  "==100" ~ "",
                                  "==0" ~ "",
                                  TRUE ~ frmt("(xx.x %)")))),
    frmt_structure(group_val = ".default", label_val = ".default",
                   AEs = frmt("[XXX]")),
    frmt_structure(group_val = ".default", label_val = ".default",
                   pval = frmt_when(">0.99" ~ ">0.99",
                                    "<0.001" ~ "<0.001",
                                    "<0.05" ~ frmt("x.xxx*"),
                                    TRUE ~ frmt("x.xxx", missing ="--")))
  ),

  row_grp_plan = row_grp_plan(
    label_loc = element_row_grp_loc(location = "indented")
  ),

  col_style_plan = col_style_plan(
    col_style_structure(align = c(".",","," "), col = c(p_low, p_high))
  ),

  col_plan = col_plan(
    -starts_with("ord"),
    span_structure(
      col2 = c(
        "Xanomeline High Dose (N=84)" = `Xanomeline High Dose`,
        "Xanomeline Low Dose (N=84)" = `Xanomeline Low Dose`,
        "Placebo (N=86)" = Placebo
      ),
      col1 = c(`n (%)` = `n_pct` ,
               `[AEs]` = `AEs`)
    ),
    span_structure(
      col2 = c("Fisher's Exact p-values" = fisher_pval),
      col1 = c(
        # add a line break to help with table formatting
        `Placebo vs.\n Low Dose` = `p_low` ,
        `Placebo vs.\n High Dose` = `p_high`
      )
    ))
)

#------------------------------ Efficacy Table -------------------------------

tfrmt_efficacy <- tfrmt(
  group = group,
  label = label,
  column = column,
  param = param,
  value = value,
  sorting_cols = c(ord1, ord2),
  body_plan = body_plan(
    frmt_structure(group_val = ".default", label_val = "n",
                   frmt("xx")),  # we could also do: label_val = ".default", n = frmt("xx")
    frmt_structure(group_val = ".default", label_val = ".default",
                   p.value = frmt_when("<0.001" ~ "<0.001",
                                       ">0.99" ~ ">0.99",
                                       TRUE ~ frmt("x.xxx", missing = " "))),
    frmt_structure(group_val = ".default", label_val = "Median (Range)",
                   frmt_combine("{median} ({min};{max})",
                                median = frmt("xx.x"),
                                min = frmt("xx"),
                                max = frmt("xx"), missing = " ")),
    frmt_structure(group_val = ".default", label_val = "Mean (SD)",
                   frmt_combine("{mean} ({sd})",
                                mean = frmt("xx.x"),
                                sd = frmt("xx.xx"), missing = " ")),
    frmt_structure(group_val = ".default", label_val = "Diff of LS Means (SE)",
                   frmt_combine("{diff} ({diff_se})",
                                diff = frmt("xx.x"),
                                diff_se = frmt("xx.xx"), missing = " ")),
    frmt_structure(group_val = ".default", label_val = "95% CI",
                   frmt_combine("({diff_lcl};{diff_ucl})",
                                diff_lcl = frmt("xx.x"),
                                diff_ucl = frmt("xx.x"), missing = " "))
  ),
  col_plan = col_plan(
    group, label,
    Placebo,
    contains("Low"),
    contains("High"),
    -starts_with("ord")
  ),
  row_grp_plan = row_grp_plan(
    row_grp_structure(group_val = list(group="Change from Baseline"),
                      element_block(post_space = " ")),
    row_grp_structure(group_val = list(group="p-value (Dose Response)"),
                      element_block(post_space = " ")),
    row_grp_structure(group_val = list(group="p-value (Xan - Placebo)"),
                      element_block(post_space = " ")),
    label_loc = element_row_grp_loc(location = "indented")
  )
)

#--------------------------- Print Examples To {gt} ----------------------------

data_ae2 <- data_ae %>%
  group_by(AEBODSYS, AETERM) %>%
  mutate(pct_high = value[col2=="Xanomeline High Dose" & param=="pct"]) %>%
  ungroup %>%
  filter(pct_high >10) %>%
  select(-pct_high)


tfrmt_demog %>% print_to_gt(data_demog)
tfrmt_ae %>% print_to_gt(data_ae2)
tfrmt_efficacy %>% print_to_gt(data_efficacy)

#------------------------------ As JSON Examples -------------------------------

tfrmt_demog %>% as_json()
tfrmt_ae %>% as_json()
tfrmt_efficacy %>% as_json()

#----------------------------- Make JSON Examples ------------------------------

tfrmt_demog %>% tfrmt_to_json(path = "inst/json_examples/tfrmt_demog.json")
tfrmt_ae %>% tfrmt_to_json(path = "inst/json_examples/tfrmt_ae.json")
tfrmt_efficacy %>% tfrmt_to_json(path = "inst/json_examples/tfrmt_efficacy.json")

#----------------------------- Read JSON Examples ------------------------------

json_to_tfrmt(path = "inst/json_examples/tfrmt_demog.json") %>% print_to_gt(data_demog)

json_to_tfrmt(path = "inst/json_examples/tfrmt_ae.json") %>% print_to_gt(data_ae2)

json_to_tfrmt(path = "inst/json_examples/tfrmt_efficacy.json") %>% print_to_gt(data_efficacy)

#' Matching the formatting to all values in the dataset
#'
#' @param .data data
#' @param table_frmt_plan styling element needed
#' @param group symbolic list of grouping
#' @param label symbolic label
#' @param param symbolic parameter
#'
#' @importFrom tidyr unnest
#' @importFrom dplyr mutate slice pull tibble ungroup group_by left_join arrange
#' @importFrom purrr map
#' @noRd
match_frmt_to_rows <- function(.data, table_frmt_plan, group, label, param){
  .data <- .data %>%
    ungroup() %>%
    mutate(TEMP_row = row_number())

  TEMP_appl_row <- table_frmt_plan %>%
    map(fmt_test_data, .data, label, group, param)

  TEMP_fmt_to_apply <- table_frmt_plan %>% map(~.$frmt_to_apply[[1]])

  dat_plus_fmt <- tibble(
    TEMP_appl_row,
    TEMP_fmt_to_apply) %>%
    # TODO add a warning if a format isn't applied anywhere
    mutate(TEMP_fmt_rank = row_number()) %>%
    unnest(cols = c(TEMP_appl_row)) %>%
    group_by(TEMP_appl_row) %>%
    # TODO add warning if there are rows not covered
    arrange(TEMP_appl_row, desc(.data$TEMP_fmt_rank)) %>%
    slice(1) %>%
    left_join(.data, ., by = c("TEMP_row" = "TEMP_appl_row"))
}

#' Display formatting applied to each row
#'
#' Used when debugging formatting, it is an easy way to allow you to see which
#' formats are applied to each row in your dataset.
#'
#' @param tfrmt tfrmt object to apply to the data
#' @param .data Data to apply the tfrmt to
#' @param convert_to_txt Logical value converting formatting to text, by default
#'   `TRUE`
#'
#' @importFrom dplyr mutate rename select case_when
#' @importFrom purrr map_chr
#' @importFrom stringr str_remove str_remove_all str_sub str_trim
#' @return formatted tibble
#' @export
#' @examples
#'  library(dplyr)
#'  library(tidyr)
#'
#'  tfrmt_spec <- tfrmt(
#'  label = label,
#'  column = column,
#'  param = param,
#'  value=value,
#'  body_plan = body_plan(
#'    frmt_structure(group_val = ".default", label_val = ".default",
#'                   frmt_combine(
#'                     "{count} {percent}",
#'                     count = frmt("xxx"),
#'                     percent = frmt_when("==100"~ frmt(""),
#'                                         "==0"~ "",
#'                                         "TRUE" ~ frmt("(xx.x%)"))))
#'  ))
#'
#'  # Create data
#'  df <- tidyr::crossing(label = c("label 1", "label 2"),
#'                 column = c("placebo", "trt1"),
#'                 param = c("count", "percent")) |>
#'    dplyr::mutate(value=c(24,19,2400/48,1900/38,5,1,500/48,100/38))
#'
#'  display_row_frmts(tfrmt_spec,df)
display_row_frmts <- function(tfrmt, .data, convert_to_txt = TRUE){
  if (convert_to_txt == FALSE){
    output <- match_frmt_to_rows(.data ,
                                 tfrmt$body_plan,
                                 tfrmt$group,
                                 tfrmt$label,
                                 tfrmt$param) %>%
      rename(frmt_applied = "TEMP_fmt_to_apply") %>%
      select(
        -tidyselect::starts_with(
          "TEMP"
        )
      ) %>%
      mutate(frmt_type = map_chr(.data$frmt_applied, function(x) unlist(class(x)[1])))

  } else if (convert_to_txt == TRUE) {
    output <- match_frmt_to_rows(.data ,
                                 tfrmt$body_plan,
                                 tfrmt$group,
                                 tfrmt$label,
                                 tfrmt$param) %>%
      rename(frmt_applied = "TEMP_fmt_to_apply") %>%
      select(
        -tidyselect::starts_with(
          "TEMP"
        )
      ) %>%
      mutate(frmt_type = map_chr(.data$frmt_applied, function(x) unlist(class(x)[1])),
             frmt_details = map_chr(.data$frmt_applied, format)) %>%
      select(-"frmt_applied")

    # extract < frmt > type from frmt_details
    output <- output %>%
      mutate(frmt_details = case_when(frmt_type == "frmt" ~ frmt_details %>%
                                        str_remove("< frmt \\| Expression: ") %>% str_remove(" >"),
                                      frmt_type == "frmt_combine" ~ frmt_details %>%
                                        str_remove("< frmt_combine \\| Expression: ") %>% str_remove(" >"),
                                      frmt_type == "frmt_when" ~ frmt_details %>%
                                        str_remove("< frmt_when \\| ") %>%
                                        str_sub(end = -2L) %>%
                                        str_remove_all("[\n]") %>%
                                        str_trim() %>%
                                        gsub(pattern = "\\s\\s", replacement = ", "))
      )

  } else {
    stop("Please pass a boolean value into the `convert_to_txt` parameter")
  }

  output
}

#' Display formatted values
#'
#' A helper for creating positional-alignment specifications for the col_style_plan.
#' Returns all unique formatted values to appear in the column(s) specified. Numeric values are represented by x's.
#'
#'
#' @param tfrmt tfrmt object to apply to the data
#' @param .data Data to apply the tfrmt to
#' @param mock Mock table? TRUE or FALSE (default)
#' @param col Column value to align on from `column` variable. May be a quoted
#'   or unquoted column name, a tidyselect semantic, or a span_structure.
#'
#' @importFrom glue glue glue_collapse
#' @importFrom dplyr select mutate across arrange pull
#' @importFrom rlang is_empty as_name as_label
#' @importFrom stringr str_replace_all
#' @importFrom purrr map_chr map
#' @importFrom tidyr pivot_longer
#'
#' @return text representing character vector of formatted values to be copied and modified in the col_style_plan
#' @export
#' @examples
#'  tf_spec <- tfrmt(
#'  group = c(rowlbl1,grp),
#'  label = rowlbl2,
#'  column = column,
#'  param = param,
#'  value = value,
#'  sorting_cols = c(ord1, ord2),
#'  body_plan = body_plan(
#'  frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} ({pct} %)",
#'                                                                              n = frmt("xxx"),
#'                                                                              pct = frmt("xx.x"))),
#'  frmt_structure(group_val = ".default", label_val = "n", frmt("xxx")),
#'  frmt_structure(group_val = ".default", label_val = c("Mean", "Median", "Min","Max"),
#'                              frmt("xxx.x")),
#'  frmt_structure(group_val = ".default", label_val = "SD", frmt("xxx.xx")),
#'  frmt_structure(group_val = ".default", label_val = ".default",
#'                             p = frmt_when(">0.99" ~ ">0.99",
#'                                           "<0.15" ~ "<0.15",
#'                                           TRUE ~ frmt("x.xxx", missing = "")))
#' ))
#'
#'  display_val_frmts(tf_spec, data_demog, col = vars(everything()))
#'  display_val_frmts(tf_spec, data_demog, col = "p-value")
display_val_frmts <- function(tfrmt, .data, mock = FALSE, col = NULL) {


  tbl_dat <- .data %>%
    remove_big_ns(param = tfrmt$param,
                  big_n_structure = tfrmt$big_n) %>%
    apply_table_frmt_plan(
      .data = .,
      table_frmt_plan = tfrmt$body_plan,
      group = tfrmt$group,
      label = tfrmt$label,
      param = tfrmt$param,
      value = tfrmt$value,
      column = tfrmt$column,
      mock = mock
    )

  tbl_dat_wide <- tbl_dat %>%
    pivot_wider_tfrmt(tfrmt, mock) %>%
    select(
      -tidyselect::any_of(
        c(
          map_chr(tfrmt$group, as_name),
          as_name(tfrmt$label)
        )
      )
    ) %>%
    mutate(
      across(
        tidyselect::everything(),
        ~str_replace_all(., "[0-9]","x")
      )
    )

  col_plan_vars <- as_vars.character(colnames(tbl_dat_wide))
  if(is_empty(tfrmt$column)){
    # create placeholder
    column_names <- "col"
  }else{
    column_names <- map_chr(tfrmt$column, as_label)
  }
  selection <- as.list(substitute(substitute(col)))[-1] %>%
    map(trim_vars_quo_c) %>%
    do.call('c',.) %>%
    check_col_plan_dots()

  col_selection <- col_style_selections(selection, column_names, col_plan_vars)

  vec_prep <- tbl_dat_wide%>%
    select(
      tidyselect::any_of(
        col_selection
      )
    ) %>%
    pivot_longer(
      tidyselect::everything()
    ) %>%
    arrange(nchar(.data$value)) %>%
    filter(!is.na(.data$value)) %>%
    pull(.data$value) %>%
    unique %>%
    paste0("\"", ., "\"") %>%
    glue_collapse(., ",\n  ")

  glue("c({vec_prep})")
}

#' Apply page plan to data
#'
#' @param .data data
#' @param page_plan tfrmt page_plan
#' @param group symbolic list of grouping
#' @param label symbolic label column
#' @param row_grp_plan_label_loc row_grp_plan label location
#'
#' @noRd
#' @importFrom rlang is_empty
#' @importFrom tibble is_tibble
#' @importFrom purrr list_flatten
apply_page_plan <- function(.data, page_plan, group, label, row_grp_plan_label_loc = "indented"){

  # first apply page structures
  if (!is_empty(page_plan$struct_list)){
    .data <- apply_page_struct(
      .data,
      page_struct_list = page_plan$struct_list,
      group = group,
      label = label,
      note_loc = page_plan$note_loc,
      transform = page_plan$transform
    )
  }

  # then apply max rows splits
  if (!is.null(page_plan$max_rows)){

    if (is_tibble(.data)){
      .data <- apply_page_max_rows(.data, page_plan$max_rows, group, label, row_grp_plan_label_loc)
    } else {
      .data <- structure(
        map(.data, ~ apply_page_max_rows(.x, page_plan$max_rows, group, label, row_grp_plan_label_loc)) %>%
        list_flatten(),
        .page_grp_vars = attr(.data, ".page_grp_vars")
      )
    }

  }

  .data
}

#' Apply page plan splits based on row #s
#'
#' @param .data data
#' @param max_rows max number of rows per page / # of rows to split after
#' @param row_grp_plan_label_loc row_grp_plan label location
#'
#' @noRd
#' @importFrom dplyr  slice bind_rows filter pull
#' @importFrom tibble tibble
#' @importFrom forcats fct_inorder
apply_page_max_rows <- function(.data, max_rows, group, label, row_grp_plan_label_loc){

  # Ensure empty strings in grouping columns are treated as literal spaces
  # so pagination math doesn't break
  group_cols <- map_chr(group, rlang::as_label)

  .data <- .data %>%
    mutate(across(all_of(group_cols), \(x) {
      if (is.character(x)) if_else(x == "", " ", x) else x
    })) %>%
    mutate(TEMP_row = row_number())

  # determine # of rows to be added for the group during row grp lbl formatting
  # only proceed if the # of group rows to be added < max_rows
  #
  #      - "gtdefault": +1
  #      - "spanning" / "indented" : # of grouping vars
  #      - "noprint", "column": +0
  n_grp_vars <- length(group)
  n_grp_rows <- switch(row_grp_plan_label_loc,
                       gtdefault = 1,
                       spanning = n_grp_vars,
                       indented = n_grp_vars,
                       column = 0,
                       noprint = 0)

  if (!n_grp_rows<max_rows){
    message("Unable to complete pagination because `max_rows` specified in `page_plan` is smaller than the number of rows dedicated to group labels. Suggest increasing `max_rows` and trying again.")
    return(.data)
  }

  # 2) split the data by row number, accounting for row groups to be added

  # start with top row, then add rows 1 by 1, testing # of rows each time
  remain_dat <- .data
  cur_dat <- tibble()
  all_summ_row <- numeric(0)
  tbl_list <- vector(mode = "list")
  i <- 1

  while(nrow(remain_dat)>0){

    # candidate row to consider adding to tbl
    next_dat <- remain_dat %>% slice(1)

    # add the previous summary row from previous table if needed
    # (only applies to the start of new page)
    if(length(all_summ_row)>0){

      # get the data from the summary row
      prev_summ_row <- .data[all_summ_row,]

      next_dat <- next_dat %>%
        add_summary_rows(prev_summ_row, group, label)

      all_summ_row <- numeric(0) # reset until current tbl is finished
    }

    # if this row is added, how many rows will be in table
    # cur_dat_new <- bind_rows(cur_dat, next_dat) %>%
    #   combine_group_cols_mod(group, label, row_grp_plan_label_loc)

    if(length(group)==0 || is_empty(label) || row_grp_plan_label_loc %in% c("gtdefault", "noprint", "column")){
      cur_dat_new <- bind_rows(cur_dat, next_dat)
    } else{
      #  combine any grouping columns that need combining into label
      cur_dat_new <- bind_rows(cur_dat, next_dat) %>%
        combine_group_cols_mod(group, label, row_grp_plan_label_loc)

    }

    # if the table is within our limit, keep it
    if (nrow(cur_dat_new) <= max_rows){
      cur_dat <- bind_rows(cur_dat, next_dat)
      remain_dat <- remain_dat %>% slice(-1)
    }

    # if we have hit or exceeded the limit, save the table & move to next
    if (nrow(cur_dat_new) >= max_rows | nrow(remain_dat)==0){

      # summary row groups to carry forward to next tbl
      if ("..tfrmt_summary_row" %in% names(cur_dat_new)){
        all_summ_row <- cur_dat_new %>%
          filter(.data$`..tfrmt_summary_row`) %>%
          pull(.data$TEMP_row)
      }

      # save tbl to list
      tbl_list[[i]] <- structure(cur_dat %>% select(-"TEMP_row"),
                                 .page_note = attr(.data, ".page_note"))
      i <- i + 1
      cur_dat <- tibble()
    }
  }

  # return single tbl or list of tbls
  if (length(tbl_list)==1) {
    tbl_list[[1]]
  } else {
    tbl_list
  }


}

#' Apply page plan splits based on page_structures
#'
#' @param .data data
#' @param page_struct_list list of page structure objects
#' @param group symbolic list of grouping
#' @param label symbolic label column
#' @param note_loc location of page note
#' @param transform optional, a function or formula to transform the page label.
#'
#' @noRd
#' @importFrom purrr map map2 map_dbl
#' @importFrom dplyr tibble row_number mutate group_by left_join select filter summarise  lag last
#' @importFrom tidyr  pivot_longer
apply_page_struct <- function(
  .data,
  page_struct_list,
  group,
  label,
  note_loc,
  transform
) {
  .data <- .data %>%
    mutate(TEMP_row = row_number())

  # 1. check that only 1 page_structure contains a .default, drop extras
  struct_defaults_idx <- which(map_lgl(page_struct_list, detect_default))
  if (length(struct_defaults_idx)>1){
    struct_defaults_idx_drop <- struct_defaults_idx[-last(struct_defaults_idx)]
    page_struct_list <- page_struct_list[-struct_defaults_idx_drop]
    message("`page_plan` contains multiple `page_structures` with values set to \".default\". \n",
            "Only the last one specified will be used.")
  }

  # 2. get all the subsets of data

  # a) If applicable, split by any group/label vars set to ".default"
  struct_defaults_idx <- which(map_lgl(page_struct_list, detect_default)) # do this again post-dropping duplicates

  if (length(struct_defaults_idx)>0){

    # split on all set to .default
    grping <- expr_to_grouping(page_struct_list[[struct_defaults_idx]], group, label)

    dat_split_1 <- .data %>%
      nest(
        `..tfrmt_data` = tidyselect::everything(),
        .by = tidyselect::all_of(grping)
      ) %>%
      mutate(
        `..tfrmt_split_num` = row_number()
      )

  } else {
    # no default - just nest to get in same structure for next step
    dat_split_1 <- tibble(`..tfrmt_data` = list(!!.data))
  }

  # b) Further split the sub-data based on specific values (loop over all combinations of page_structure & data)

  # find indices of specific values in data
  dat_split_2_idx <- dat_split_1 %>%
    mutate(split_idx = map(.data$`..tfrmt_data`, function(x){
      map(page_struct_list, function(y){
        struct_val_idx(y, x, group, label) %>% # returns all indices in the block of data
          map_dbl(last) # keep just the last one to split after
      }) %>% unlist()
    }))

  # determine where the splits should occur in data
  dat_split_2 <- dat_split_2_idx%>%
    mutate(`..tfrmt_data` = map2(.data$`..tfrmt_data`, .data$split_idx, function(x, y){

      x %>%
        mutate(`..tfrmt_split_idx` = .data$TEMP_row %in% y,
               # carry it forward to denote start of next table,
               `..tfrmt_start_idx` = lag(.data$`..tfrmt_split_idx`, default = TRUE),
               `..tfrmt_split_after` = cumsum(.data$`..tfrmt_start_idx`)) %>%
        select(- c("..tfrmt_start_idx","..tfrmt_split_idx")) %>%
        group_by(.data$`..tfrmt_split_after`) %>%
        group_split(.keep = FALSE)
    })) %>%
    select(-"split_idx") %>%
    unnest(cols = "..tfrmt_data")

  # 3. create the page_notes as applicable
  if ("..tfrmt_split_num" %in% names(dat_split_2)){
    # create the page_notes to be carried forward as names for now
    tbl_nms <- dat_split_2 %>%
      select(-"..tfrmt_data") %>%
      pivot_longer(cols = -"..tfrmt_split_num", names_to = "grouping_col", values_to = "grouping_val") %>%
      group_by(.data$`..tfrmt_split_num`) %>%
      filter(!is.na(.data$grouping_val)) %>%
      unique() %>%
      summarise(`..tfrmt_page_note` = paste0(.data$grouping_col, ": ", .data$grouping_val) %>% paste0(collapse = ", "))

    page_grp_vars <- setdiff(names(dat_split_2), c("..tfrmt_data", "..tfrmt_split_num"))

    dat_split_2 <- left_join(dat_split_2, tbl_nms, by = "..tfrmt_split_num") %>%
      select(c("..tfrmt_data", "..tfrmt_page_note"))
  } else {
    page_grp_vars <- NULL
  }


  # 4. return the values
  # prep list of tbsl
  dat_out <- dat_split_2 %>%
    mutate(`..tfrmt_data` = map(.data$`..tfrmt_data`, ~select(.x, - "TEMP_row"))) %>%
    pull(.data$`..tfrmt_data`)

  # add pg_note to individual tbls as applicable
  if ("..tfrmt_page_note" %in% names(dat_split_2)){
    pg_note <- dat_split_2$`..tfrmt_page_note`

    if (!is.null(transform)) {
      transform <- rlang::as_function(transform)
      pg_note <- transform(pg_note)
    }

    dat_out <- purrr::map2(
      dat_out,
      pg_note,
      ~ structure(
        .x,
        .page_note = .y
      )
    )
  }

  structure(
    dat_out,
    .page_grp_vars = page_grp_vars
  )

}


#' adapted from the row_grp_plan to remove anything unnecessary but keep the logic
#' @noRd
#' @importFrom forcats fct_inorder
#' @importFrom dplyr mutate select across group_by group_split distinct last bind_rows
#' @importFrom tibble tibble
#' @importFrom stringr str_trim
combine_group_cols_mod <- function(.data, group, label, element_row_grp_loc = NULL){

  top_grouping <- group #used for splitting in case of spanning label

  # to retain the order of the data when splitting by group
  .data <- .data %>%
    select(c(!!!group, !!label, "TEMP_row")) %>%
    mutate(across(c(!!!group), ~fct_inorder(.x)),
           ..tfrmt_row_grp_lbl = FALSE)%>%
    mutate(`..tfrmt_summary_row` = str_trim(!!label, side = "left") == str_trim(!!last(group), side = "left"))

  if(element_row_grp_loc %in% c("spanning") & length(group) > 0){
    group = group[-1]
  }

  while(length(group) > 0 & !is.null(label)){

    split_dat <- .data %>%
      group_by(!!!top_grouping) %>%
      group_split()

    .data<- split_dat %>%
      map_dfr(function(lone_dat){

        lone_dat_summ <- lone_dat %>%
          mutate(`..tfrmt_summary_row_cur` = str_trim(!!label, side = "left") == str_trim(!!last(group), side = "left"))

        if (any(lone_dat_summ$`..tfrmt_summary_row_cur`)==FALSE){

          # if the set of rows contains NO group-level summary data, create an extra row to be added
          new_row <- lone_dat %>%
            select(!!!top_grouping, !!label) %>%
            mutate(!!label := !!last(group)) %>%
            distinct()%>%
            mutate(..tfrmt_row_grp_lbl = TRUE)
        } else {
          new_row <- tibble()
        }

        lone_dat_summ  %>%
          bind_rows(new_row, .)
      })
    group = group[-length(group)]
    top_grouping = top_grouping[-length(top_grouping)]
  }

  .data %>%
    select(
      -tidyselect::any_of(
        "..tfrmt_summary_row_cur"
      )
    )
}

#' add any related summary rows from previous tbl to next tbl
#' @noRd
#' @importFrom dplyr mutate slice pull filter across select bind_rows
#' @importFrom purrr map_chr map map2_lgl
#' @importFrom tidyr pivot_longer
add_summary_rows <- function(next_dat, prev_summ, group, label){

  #get grouping values from the summary row
  prev_summ_top_grp <- prev_summ %>%
    mutate(across(c(!!!group), ~ .x==!!label)) %>%
    pivot_longer(map_chr(group, as_label), names_to = "..tfrmt_summ_grp_num", values_to = "..tfrmt_summ_row") %>%
    filter(.data$`..tfrmt_summ_row`) %>%
    group_by(.data$TEMP_row) %>%
    slice(1)  %>%
    mutate(`..tfrmt_summ_grp_num` = which(.data$`..tfrmt_summ_grp_num`==map_chr(group, as_label))) %>%
    pull(.data$`..tfrmt_summ_grp_num`)

  prev_summ_top_grp_vars <- map(
    seq_len(nrow(prev_summ)),
    ~prev_summ[.x,] %>%
      select(c(!!!group)) %>%
      select(
        1:tidyselect::all_of(
          prev_summ_top_grp[.x]
        )
      )
  )

  # get the grouping values from the next row
  next_summ_top_grp_vars <- map(prev_summ_top_grp_vars,
                                ~next_dat %>% select(names(.x)))

  # check which are identical & add all that match to the data
  check_eq <- map2_lgl(prev_summ_top_grp_vars, next_summ_top_grp_vars, function(x,y){
    identical(x,y)
  })
  if(any(check_eq)){
    to_add <- which(check_eq)

    next_dat <- bind_rows(
      prev_summ[to_add,],
      next_dat)
  }
  next_dat
}

#' Column Style Plan
#'
#' Define how the columns of the table body should be aligned, whether left,
#' right or on a specific character(s).
#'
#' @param ... series of col_style_structure objects
#'
#' @return col_style_plan object
#'
#' @examples
#'
#'  plan <- col_style_plan(
#'     col_style_structure(col = "my_var", align = "left", width = 100),
#'     col_style_structure(col = vars(four), align = "right"),
#'     col_style_structure(col = vars(two, three), align = c(".", ",", " "))
#'    )
#'
#'
#'
#' @seealso [col_style_structure()] for more information on how to specify how to and which columns to align.
#'
#'   \href{https://gsk-biostatistics.github.io/tfrmt/articles/col_style_plan.html}{Link to related article}
#'
#' @export
#'
col_style_plan <- function(...){

  structure_list <- list(...)

  for(el_idx in seq_along(structure_list)){
    if(!is_col_style_structure(structure_list[[el_idx]])){
      stop(paste0("Entry number ",el_idx," is not an object of class `col_style_structure`."))
    }
  }

  structure(
    structure_list,
    class = c("col_style_plan", "frmt_table")
  )
}


#' Column Style Structure
#'
#' @param col Column value to align on from `column` variable. May be a quoted
#'   or unquoted column name, a tidyselect semantic, or a span_structure.
#' @param align Alignment to be applied to column. Defaults to `left` alignment. See details for acceptable values.
#' @param type Type of alignment: "char" or "pos", for character alignment (default), and positional alignment, respectively. Positional alignment allows for aligning over multiple positions in the column.
#' @param width Width to apply to the column in number of characters. Acceptable values include a
#'   numeric value, or a character string of a number.
#' @param ... These dots are for future extensions and must be empty
#'
#'
#' @details Supports alignment and width setting of data value columns (values found in the `column` column). Row group and label columns are left-aligned by default. Acceptable input values for `align` differ by type = "char" or "pos":
#'
#' ## Character alignment (type = "char"):
#'   - "left" for left alignment
#'   - "right" for right alignment"
#'   - supply a vector of character(s) to align on. If more than
#'   one character is provided, alignment will be based on the first occurrence
#'   of any of the characters. For alignment based on white space, leading white
#'   spaces will be ignored.
#'
#' ## Positional alignment (type = "pos"):
#'  supply a vector of strings covering all formatted cell values, with numeric values represented as x's. These values can be created manually or obtained by utilizing the helper `display_val_frmts()`. Alignment positions will be represented by vertical bars. For example, with starting values: c("12.3", "(5%)", "2.35 (10.23)") we can align all of the first sets of decimals and parentheses by providing align = c("xx|.x", "||(x%)", "x|.xx |")
#'
#' @importFrom purrr map
#' @importFrom rlang check_dots_empty0
#'
#' @seealso [col_style_plan()] for more information on how to combine
#'   col_style_structure()'s together to form a plan.
#'
#'   \href{https://gsk-biostatistics.github.io/tfrmt/articles/col_style_plan.html}{Link to related article}
#'
#' @return col_style_structure object
#' @export
#' @examples
#'
#'  plan <- col_style_plan(
#'     col_style_structure(col = "my_var",
#'                         align = c("xx| |(xx%)",
#'                                   "xx|.x |(xx.x - xx.x)"),
#'                         type = "pos", width = 100),
#'     col_style_structure(col = vars(four), align = "right", width = 200),
#'     col_style_structure(col = vars(two, three), align = c(".", ",", " ")),
#'     col_style_structure(col = c(two, three), width = 25),
#'     col_style_structure(col = two, width = 25),
#'     col_style_structure(col = span_structure(span = value, col = val2),
#'                         width = 25)
#'    )
#'
#' @rdname theme_element
col_style_structure <- function(col, align = NULL, type = c("char", "pos"), width = NULL, ...){

  check_dots_empty0(...)

  type <- match.arg(type)

  if(missing(col)){
    abort(
      "Column element is missing from col_style_structure. Note: col here refers to the values within the column variable in your data, rather than the variable name itself",
      class = "missing_col_style_structure_value"
    )
  }

  cols <- as.list(substitute(substitute(col)))[-1] %>%
    map(trim_vars_quo_c) %>%
    do.call('c',.) %>%
    check_col_plan_dots()

  if(is.null(width) & is.null(align)){
    abort("`align` or `width` must be applied to create this col_style_structure",
          class = "missing_col_style_structure_value")
  }

  if(!is.null(align)){

    if (type=="char"){
      if(!is.character(align) & length(align) > 0){
        abort("`align` must be a character vector", class = "invalid_col_style_structure_value")
      }

      if(!all(align %in% c("left","right"))){

        if (!all(nchar(align)==1)){
          message( "Alignment specified contains strings with >1 characters. Only the first character will be used.")
          align <- str_sub(align, start=1, end=1)
        }

        if (any(str_detect(align, "[[:alnum:]]"))){
          message( "Alignment specified contains one or more alphanumeric characters. Results may not be as expected.")
        }
      }
    }

  }

  if(!is.null(width)){
    if(is.character(width)){
      suppressWarnings(width <- as.numeric(width))
      if(is.na(width)){
        abort("`width` must be a value that can be converted into a number greater than 0", class = "invalid_col_style_structure_value")
      }
    }
    if(any(!is.numeric(width),width < 1, length(width) > 1)){
      abort("`width` must be a valid number greater than 0", class = "invalid_col_style_structure_value")
    }
  }


  structure(
    list(
      cols = cols,
      align = align,
      type = type,
      width = width
    ),
    class = c("col_style_structure", "structure")
  )
}


#' Check if input is an col_style_structure object
#'
#' @param x Object to check
#'
#' @noRd
is_col_style_structure <- function(x){
  inherits(x, "col_style_structure")
}

test_that("inputs are as expected",{

  # create test data
  risk<-tibble(time=c(rep(c(0,1000,2000,3000),3)),
                 label=c(rep("Obs",4),rep("Lev",4),rep("Lev+5FU",4)),
                 value=c(630,372,256,11,620,360,266,8,608,425,328,14),
                 param=rep("n",12))

  riska<- risk %>%
    mutate(group="A")

  riskb<- risk %>%
    mutate(group="B",
           value=value+10)

  test_data<-riska %>%
    rbind(riskb)

  tfrmt_1<-tfrmt(
    # specify columns in the data
    group = group,
    label = label ,
    column = time,
    param = param,
    value = value)

  # label loc not indented
  tfrmt_2<-tfrmt(
    # specify columns in the data
    group = group,
    label = label ,
    column = time,
    param = param,
    value = value,
    row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc(location = "column")))

  # two column variables
  tfrmt_4<-tfrmt(
    # specify columns in the data
    group = group,
    label = label ,
    column = c(time,time2),
    param = param,
    value = value
  )

  # col style plan
  tfrmt_5<-tfrmt(
    # specify columns in the data
    label = label ,
    column = time,
    param = param,
    value = value,
    col_style_plan =  col_style_plan(
      col_style_structure(align = "right", col = `1000`))
  )

  expect_error(print_to_ggplot(tfrmt_1,"test"),"Requires data")
  expect_error(print_to_ggplot(tfrmt="test",.data=test_data),"Requires a tfrmt object")
  expect_error(print_to_ggplot(tfrmt_2,risk),"print_to_ggplot must have label location 'indented' if row_group_plan is present")
  expect_error(print_to_ggplot(tfrmt_4,risk),"print_to_ggplot does not support multiple column variables")
  expect_error(print_to_ggplot(tfrmt_5,risk),"print_to_ggplot does not support col_style_plan elements")



})

test_that("group columns are created correctly",{
  tfrmt<-tfrmt(
    # specify columns in the data
    group = group,
    label = label ,
    column = time,
    param = param,
    value = value,
    row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc("gtdefault")))

  test_data<-tibble(time=c(rep(c(0,1000,2000,3000),3)),
                    label=c(rep("Obs",4),rep("Lev",4),rep("Lev+5FU",4)),
                    value=c(630,372,256,11,620,360,266,8,608,425,328,14),
                    param=rep("n",12),
                    group="A")
  expected_data<-tibble(label=c("A",rep("    Obs",4),rep("    Lev",4),rep("    Lev+5FU",4)),
                        time=c(NA,rep(c(0,1000,2000,3000),3)),
                        value=c(NA,630,372,256,11,620,360,266,8,608,425,328,14),
                        param=c(NA,rep("n",12)),
                        `..tfrmt_row_grp_lbl`=c(TRUE,rep(FALSE,12)))


  expect_equal(apply_grp_ggplot(test_data,tfrmt),expected_data)
})

test_that("tfrmt is as expected",{

  # create test data
  test_data<-tibble(time=c(rep(c(0,1000,2000,3000),3)),
               label=c(rep("Obs",4),rep("Lev",4),rep("Lev+5FU",4)),
               value=c(630,372,256,11,620,360,266,8,608,425,328,14),
               param=rep("n",12))



  tfrmt_1<-tfrmt(
    group = group,
    label = label ,
    param = param,
    value = value)

  tfrmt_2<-tfrmt(
    group = group,
    column = time,
    param = param,
    value = value)

  tfrmt_3<-tfrmt(
    group = group,
    label = label ,
    column = time,
    value = value)

  tfrmt_4<-tfrmt(
    group = group,
    label = label ,
    column = time,
    param= param)

  expect_error(print_to_ggplot(tfrmt_1,test_data),"column variable required for print_to_ggplot")
  expect_error(print_to_ggplot(tfrmt_2,test_data),"label variable required for print_to_ggplot")
  expect_error(print_to_ggplot(tfrmt_3,test_data),"param variable required for print_to_ggplot")
  expect_error(print_to_ggplot(tfrmt_4,test_data),"value variable required for print_to_ggplot")


})


test_that("column type has been preserved",{
  # make discrete mock data
  df <- structure(
    list(
      City = structure(
        c(2L,
          3L, 1L),
        .Label = c("Minneapolis", "Phoenix",
                   "Raleigh"),
        class = "factor"
      ),
      January = c(52.1,
                  40.5, 12.2),
      February = c(55.1, 42.2, 16.5),
      March = c(59.7, 49.2, 28.3),
      April = c(67.7,
                59.5, 45.1),
      May = c(76.3, 67.4, 57.1),
      June = c(84.6, 74.4, 66.9),
      July = c(91.2,
               77.5, 71.9),
      August = c(89.1, 76.5,
                 70.2),
      September = c(83.8, 70.6, 60),
      October = c(72.2, 60.2, 50),
      November = c(59.8,
                   50, 32.4),
      December = c(52.5, 41.2,
                   18.6)
    ),
    .Names = c(
      "City",
      "January",
      "February",
      "March",
      "April",
      "May",
      "June",
      "July",
      "August",
      "September",
      "October",
      "November",
      "December"
    ),
    class = "data.frame",
    row.names = c(NA,-3L)
  )

  dfm <- df %>%
    pivot_longer(January:December) %>%
    rename("month" = name) %>%
    mutate(month=substr(month,1,3))

  dfm$month <- factor(dfm$month,levels=c("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"))

  table_data<- dfm %>%
    mutate(param="n") %>%
    rename("label"=City,"column"=month)

  x2<-tfrmt(
    # specify columns in the data
    label = label ,
    column = column,
    param = param,
    value = value) %>%
    print_to_ggplot(table_data)

  # continuous mock data
  risk<-tibble(time=c(rep(c(0,1000,2000,3000),3)),
               label=c(rep("Obs",4),rep("Lev",4),rep("Lev+5FU",4)),
               value=c(630,372,256,11,620,360,266,8,608,425,328,14),
               param=rep("n",12))

  p2<-tfrmt(
    # specify columns in the data
    label = label ,
    column = time,
    param = param,
    value = value) %>%
    print_to_ggplot(risk)

  expect_equal(class(table_data$column),class(x2$data$column))
  expect_equal(class(risk$time),class(p2$data$column))


})

#' Check if input is a frmt
#'
#' @param x Object to check
#' @export
#' @examples
#' x1 <- frmt("XXX.XX")
#' is_frmt(x1)
#'
#' @return 'TRUE' if yes, 'FALSE' if no
#'
#' @rdname frmt_utils
is_frmt <- function(x){
  inherits(x, "frmt")
}

#' Check if input is a frmt_combine
#'
#' @param x Object to check
#' @export
#' @examples
#' x2 <- frmt_combine("XXX %","XX,XXX")
#' is_frmt_combine(x2)
#'
#' @rdname frmt_utils
is_frmt_combine <- function(x){
  inherits(x, "frmt_combine")
}

#' Check if input is a frmt_when
#'
#' @param x Object to check
#' @export
#' @examples
#' x2 <- frmt_when(
#' ">3" ~ frmt("(X.X%)"),
#' "<=3" ~ frmt("Undetectable")
#' )
#' is_frmt_when(x2)
#'
#' @rdname frmt_utils
is_frmt_when <- function(x){
  inherits(x, "frmt_when")
}
#' Check if input is a frmt_structure
#'
#' @param x Object to check
#' @export
#' @examples
#' x3 <- frmt_structure(
#'  group_val = c("group1"),
#'  label_val = ".default",
#' frmt("XXX")
#' )
#'is_frmt_structure(x3)
#'
#' @rdname frmt_utils
is_frmt_structure <- function(x){
  inherits(x, "frmt_structure")
}

#' Check if input is a row_grp_structure
#'
#' @param x Object to check
#' @export
#' @examples
#' x4 <- row_grp_structure(group_val = c("A","C"), element_block(post_space = "---"))
#' is_row_grp_structure(x4)
#'
#' @rdname frmt_utils
is_row_grp_structure <- function(x){
  inherits(x, "row_grp_structure")
}

#' @export
format.frmt <- function(x, ...){
  paste0(
    "< frmt | Expression: `",
    x$expression,
    "` >"
  )
}

#' @export
print.frmt <- function(x,...){
  cat(format(x),sep = "\n")
}

#' @export
format.frmt_combine <- function(x, ...){
  frmt_str <- paste0(
    "< frmt_combine | Expression: `",
    x$expression,
    "` >"
  )
  frmt_str
}

#' @export
print.frmt_combine <- function(x,...){
  cat(format(x),sep = "\n")
}

#' @export
format.frmt_when <- function(x, ...){
  lhs <- map_chr(x$frmt_ls, f_lhs_as_char)
  rhs <- map(x$frmt_ls, f_rhs) %>% map_chr(format)

  frmt_str <- paste(
    "< frmt_when | ", "\n ",
    paste0(
      map2_chr(lhs, rhs, paste, sep = " ~ "),
      collapse = "\n  "),
    paste0("\n", "  Missing: ", x$missing),
    "\n >"
    )
  frmt_str
}

#' @export
print.frmt_when <- function(x,...){
  cat(format(x),sep = "\n")
}

#' @export
#' @importFrom purrr map
format.frmt_structure <- function(x,...){

  if (is.list(x$group_val)){
    groups <- x$group_val %>% map(unique)
  } else {
    groups <- unique(x$group_val)
  }
  labels <- unique(x$label_val)
  param <- unique(x$param_val)
  fmts <- x$frmt_to_apply[[1]]

  if(is.list(groups)){
    group_string <- paste0(
      sapply(names(groups), function(y) {
        paste0(" `",y,"` - ", paste0("\"", x$group_val[[y]], "\"", collapse = ", "))
      }),
      collapse = ";"
    )
  }else{
    group_string <- paste0(" \"",groups,"\"", collapse=",")
  }


  frmt_struct_str <- c(
    "Format Structure",
    paste0("  Group Values:",group_string),
    paste0("  Label Values: ",paste0("\"",labels,"\"", collapse=", "))
  )


  if(!identical(param,".default")){
    frmt_struct_str <- c(
      frmt_struct_str,
      paste0("  Param Values: ",paste0("\"",param,"\"", collapse=", "))
    )
  }

  frmt_struct_str <- c(
    frmt_struct_str,
    paste0("  Format: ",format(fmts))
  )


  frmt_struct_str

}

#' @method print frmt_structure
#' @export
print.frmt_structure <- function(x, ...){
  cat(format(x, ...), sep = "\n")
}

#' @export
#' @keywords internal
format.body_plan <- function(x,...){

  table_body_plan_str <- c(
    "Table Body Plan",
    paste0(" ",length(x)," Format Structures:")
  )

  frmt_str_seq_len <- seq_len(length(x))
  frmt_str_num <- format(frmt_str_seq_len)

  tabin <- max(nzchar(frmt_str_num))

  for(frmt_struct_idx in frmt_str_seq_len){

    frmt_struct_fmt <- format(x[[frmt_struct_idx]])

    table_body_plan_str <- c(
      table_body_plan_str,
      "",
      paste0("  [[",frmt_str_num[frmt_struct_idx],"]] ",frmt_struct_fmt[1]),
      paste0(paste0(" ",rep("",tabin)),frmt_struct_fmt[-1])
    )

  }

  table_body_plan_str

}

print.body_plan <- function(x, ...){
  cat(format(x, ...), sep = "\n")
}



# Constructing frmt from strings ------------------------------------------

#' Build frmt for a given parameter
#'
#' @param param `param` value
#' @param frmt_string formatted expression
#' @param missing missing option to be included in all `frmt`s
#'
#' @return character string representing `frmt` object with `param` value as name
#' @noRd
#' @importFrom purrr map2
#' @importFrom stats setNames
frmt_builder <- function(param, frmt_string, missing = NULL) {

  if(!missing(param)){
    frmt_string <- setNames(frmt_string, param)

  }

  map(frmt_string, function(x, missing_val) {
    do.call(frmt, list(expression = x, missing = missing_val ))
  }, missing_val = missing)

}

#' Build frmt_combine for a given set of parameters
#'
#' @param param_combine character string representing how `param` values will be
#'   combined using `glue::glue()` syntax
#' @param param vector of `param` values
#' @param frmt_string vector of formatted expressions
#' @param missing missing option to be included in all `frmt`s
#'
#' @return character string representing `frmt_combine` object
#' @noRd
frmt_combine_builder <- function(param_combine, param, frmt_string, missing = NULL){

  frmts <- frmt_builder(param, frmt_string, missing)

  list(do.call(frmt_combine, c(expression = param_combine, frmts, missing = missing)))
}

#' Build format structure from a list of `frmt` and `frmt_combine` objects
#'
#' @param group_val A string or a named list of strings which represent the value of group should be when the given frmt is implemented
#' @param label_val A string which represent the value of label should be when the given frmt is implemented
#' @param frmt_vec Character vector of `frmt` and/or `frmt_combine` objects to be applied to the group_val/label_val combination
#'
#' @return list of `frmt_structure` objects
#' @noRd
#' @importFrom purrr pmap
#' @importFrom rlang `%||%`
frmt_structure_builder <- function(group_val, label_val, frmt_vec){

  grp_lbl_list <- list(list(group_val = group_val, label_val = label_val))
  frmt_vec_list <- map2(names(frmt_vec), frmt_vec, ~list(param = .x %||% "", frmt = .y))

  crossing(frmt_vec_list,
           grp_lbl_list) %>%
    pmap(function(frmt_vec_list, grp_lbl_list){

      if(is.list(grp_lbl_list$group_val) & length(grp_lbl_list$group_val) == 1 & is.null(names(grp_lbl_list$group_val))){
        grp_lbl_list$group_val <- grp_lbl_list$group_val[[1]]
      }

      if(is.list(grp_lbl_list$label_val) & length(grp_lbl_list$label_val) == 1& is.null(names(grp_lbl_list$label_val))){
        grp_lbl_list$label_val <- grp_lbl_list$label_val[[1]]
      }

      arg_list <- list(grp_lbl_list$group_val,  grp_lbl_list$label_val, frmt_vec_list$frmt)
      names(arg_list) <- c("group_val","label_val",frmt_vec_list$param)

      do.call(frmt_structure, arg_list)
    }) %>%
    unname()

}


missing_to_chr <- function(x){
  if(!is.null(x) ){
    paste0("'", x, "'")
  } else {
    x
  }
}


#' @method as.character frmt
#' @export
as.character.frmt <- function(x, ...){
  paste0("frmt('", x$expression, "'",
         if_else(!is.null(x$missing), paste0(", missing = ", missing_to_chr(x$missing)), ""),
         if_else(!is.null(x$scientific), paste0(", scientific = ", x$scientific), ""),
         if_else(!is.null(x$transform), paste0(", transform = ", deparse(x$transform) %>% str_c(collapse = "")), ""),
         ")"
         )
}

#' @method as.character frmt_when
#' @importFrom rlang quo `!!` f_rhs f_lhs eval_tidy as_label
#' @importFrom stringr str_c
#' @importFrom dplyr if_else
#' @export
as.character.frmt_when <- function(x, ...){

  right <- x$frmt_ls %>%
    map_chr(function(x){
      val <- quo(!!f_rhs(x))
      val_eval <- eval_tidy(val)
      if (!is_frmt(val_eval)){
        as_label(val)
      } else {
        as.character(val_eval)
      }

    })

  left <- x$frmt_ls %>%
    map_chr(~f_lhs(.x)) %>%
    str_c("'", ., "'")
  params <- str_c(left, " ~ ", right) %>%
    str_c(collapse = ", ")

  paste0("frmt_when(",
         params,
         if_else(!is.null(x$missing), paste0(", missing = ", missing_to_chr(x$missing)), ""),
         ")"
  )
}


#' @method as.character frmt_combine
#' @export
as.character.frmt_combine <- function(x, ...){
  params <- x$frmt_ls %>%
    map_chr(~as.character(.x)) %>%
    str_c(names(x$frmt_ls), " = ", .) %>%
    str_c(collapse = ", ")
  paste0("frmt_combine('", x$expression, "', ",
         params,
         if_else(!is.null(x$missing), paste0(", missing = ", missing_to_chr(x$missing)), ""),
         ")"
  )
}


#' @method as.character span_structure
#' @export
as.character.span_structure <- function(x, ...){
  values <- x %>%
    map(function(val){
      elements <- map_chr(val, as_label) %>%
        str_replace_all("\\\"", "'")

      # Detect function calls. Matches valid R functions i.e, my_function()
      # Valid column names containing parenthesis i.e., "n (%)" are not captured
      not_fxs <-elements %>%
        str_which("^[A-Za-z_.][A-Za-z0-9_.]*\\(", negate = TRUE)
      elements[not_fxs] <- elements[not_fxs] %>%
        str_c("'", ., "'")

      if(rlang::is_named(val)){
        elements = str_c("`", names(val), "`", " = ", elements)
      }

      elements %>%
        str_c(collapse = ", ") %>%
        str_c("c(", ., ")")
      }
      )

  paste0("span_structure(",
         str_c(names(values), " = ", values) %>% str_c(collapse = ", "),
         ")"
  )
}

#' helper for rlang::f_lhs with character conversion, in accordance with
#' purrr::map_chr no longer coercing to character
#' @noRd
f_lhs_as_char <- function(x) as.character(rlang::f_lhs(x))

test_that("setting param sigdig defaults", {

  defaults <- list(min = 1, max = 1, median = 1, "{mean} ({sd})" = c(1,2), n = NA)
  expect_equal(param_set(), defaults)

  expect_equal(param_set(max = 2, "{mean} ({sd})" = c(2,3)),
              list(min = 1, median = 1, n = NA, max = 2, "{mean} ({sd})" = c(2,3)))

  expect_equal(param_set(new_prm = 4),
              list(min = 1, max = 1, median = 1, "{mean} ({sd})" = c(1,2), n = NA, new_prm = 4))

  expect_equal(param_set(mean = 0),
               list(min = 1, max = 1, median = 1, n = NA, mean = 0))

  expect_equal(param_set("{n} ({pct}%)" = c(NA, 1)),
               list(min = 1, max = 1, median = 1, "{mean} ({sd})" = c(1,2), "{n} ({pct}%)" = c(NA, 1)))
})

test_that("build frmt objects",{

  #frmt
  frmt_spec <- frmt_builder(param = c("mean","sd"), frmt_string = c("xx.x","xx.xx"))
  expect_equal(frmt_spec, list( mean = frmt("xx.x"), sd = frmt('xx.xx')))

  frmt_spec <- frmt_builder(param = c("mean","sd"), frmt_string = c("xx.x","xx.xx"), missing = "--")
  expect_equal(frmt_spec, list(mean = frmt('xx.x', missing = '--'), sd = frmt('xx.xx', missing = '--')))

  frmt_spec <- frmt_builder(frmt_string = c("xx.x","xx.xx"), missing = "--")
  expect_equal(frmt_spec, list(frmt('xx.x', missing = '--'), frmt('xx.xx', missing = '--')))

  # frmt_combine
  frmt_spec <- frmt_combine_builder(param_combine = "{mean} ({sd})",
                                    param = c("mean","sd"), frmt_string = c("xx.x","xx.xx"), missing = "-")
  frmt_string <- list(frmt_combine('{mean} ({sd})', mean = frmt('xx.x', missing = '-'), sd = frmt('xx.xx', missing = '-'), missing = '-'))
  expect_equal(frmt_spec, frmt_string)



  # frmt_structure
  frmt_list <- list(
    frmt_combine('{mean} ({sd})', mean = frmt('xx.x'), sd = frmt('xx.xx')),
    median = frmt('xx.x'),
    n = frmt('xxx')
  )
  frmt_spec <- frmt_structure_builder(group_val = ".default", label_val = "ige", frmt_list)
  frmt_string <- list(
    frmt_structure(group_val = ".default", label_val = "ige", frmt_combine('{mean} ({sd})', mean = frmt('xx.x'), sd = frmt('xx.xx'))),
    frmt_structure(group_val = ".default", label_val = "ige", median = frmt('xx.x')),
    frmt_structure(group_val = ".default", label_val = "ige", n = frmt('xxx')))

  expect_equal(frmt_spec, frmt_string)


  frmt_spec <- frmt_structure_builder(group_val = ".default", label_val = c("ige","igg"), frmt_list)
  frmt_string <- list(
    frmt_structure(group_val = ".default", label_val = c("ige","igg"), frmt_combine('{mean} ({sd})', mean = frmt('xx.x'), sd = frmt('xx.xx'))),
    frmt_structure(group_val = ".default", label_val = c("ige","igg"), median = frmt('xx.x'))  ,
    frmt_structure(group_val = ".default", label_val = c("ige","igg"), n = frmt('xxx')))
  expect_equal(frmt_spec, frmt_string)


  # build contents of body_plan for a given sigdig value
  dat_sigdig <- tibble::tribble(
    ~group1,  ~ group2, ~ sigdig,
    "CHEM",  "ALANINE AMINOTRANSFERASE", 1,
    "CHEM", "CHOLESTEROL", 1
  )
  # 1 group, 1 label
  bp_1grp_1lbl <- body_plan_builder(dat_sigdig, group = vars(group1), label = quo(group2), param_defaults = param_set())
  bp_1grp_1lbl_man <- list(frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL"), min = frmt('x.xx')),
                           frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL"), max = frmt('x.xx')),
                           frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL"), median = frmt('x.xx')),
                           frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL"), frmt_combine('{mean} ({sd})', mean = frmt('x.xx'), sd = frmt('x.xxx'))),
                           frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL"), n = frmt('x')))
  expect_equal(bp_1grp_1lbl,
               bp_1grp_1lbl_man)

  # 2 groups, no label
  bp_2grp_0lbl <- body_plan_builder(dat_sigdig, group = vars(group1, group2), label = quo(), param_defaults = param_set())
  bp_2grp_0lbl_man <- list(frmt_structure(group_val = list(group1 = "CHEM", group2 = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL")), label_val = c(".default"), min = frmt('x.xx')),
                           frmt_structure(group_val = list(group1 = "CHEM", group2 = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL")), label_val = c(".default"), max = frmt('x.xx')),
                           frmt_structure(group_val = list(group1 = "CHEM", group2 = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL")), label_val = c(".default"), median = frmt('x.xx')),
                           frmt_structure(group_val = list(group1 = "CHEM", group2 = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL")), label_val = c(".default"), frmt_combine('{mean} ({sd})', mean = frmt('x.xx'), sd = frmt('x.xxx'))),
                           frmt_structure(group_val = list(group1 = "CHEM", group2 = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL")), label_val = c(".default"), n = frmt('x')) )
  expect_equal(bp_2grp_0lbl,
               bp_2grp_0lbl_man)


  # custom params
  bp_prm <- body_plan_builder(dat_sigdig, group = vars(group1), label = quo(group2), param_defaults = param_set(max = 0, "{pct}%" = 0))
  bp_prm_man <- list(frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL"), min = frmt('x.xx')),
                     frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL"), median = frmt('x.xx')),
                     frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL"), frmt_combine('{mean} ({sd})', mean = frmt('x.xx'), sd = frmt('x.xxx'))),
                     frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL"), n = frmt('x')),
                     frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL"), max = frmt('x.x')),
                     frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL"), pct = frmt('x.x%')))
  expect_equal(bp_prm,
               bp_prm_man)


  # custom params
  bp_prm <- body_plan_builder(dat_sigdig, group = vars(group1), label = quo(group2), param_defaults = param_set(max = 0, "{n} ({pct}%)" = c(NA, 0)))
  bp_prm_man <- list(frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL"), min = frmt('x.xx')),
                     frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL"), median = frmt('x.xx')),
                     frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL"), frmt_combine('{mean} ({sd})', mean = frmt('x.xx'), sd = frmt('x.xxx'))),
                     frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL"), max = frmt('x.x')),
                     frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "CHOLESTEROL"), frmt_combine('{n} ({pct}%)', n = frmt('x'), pct = frmt('x.x')))
                     )

  expect_equal(bp_prm,
               bp_prm_man)
})

test_that("no redundant frmt_structures",{

  dat_sigdig <- tibble::tribble(
    ~group1,  ~ group2, ~ sigdig,
    ".default", ".default", 1,
    "CHEM",  "ALANINE AMINOTRANSFERASE", 1,
    "CHEM", "BILIRUBIN", 1,
    "CHEM", "CHOLESTEROL", 2,
    "HEM", "EOSINOPHILS", 2
  )
  bp <- tfrmt_sigdig(dat_sigdig, group = vars(group1), label = quo(group2), param_defaults = param_set())$body_plan
  bp_man <- body_plan(frmt_structure(group_val = list(group1 = ".default"), label_val = c(".default"), min = frmt('x.xx')),
                      frmt_structure(group_val = list(group1 = ".default"), label_val = c(".default"), max = frmt('x.xx')),
                      frmt_structure(group_val = list(group1 = ".default"), label_val = c(".default"), median = frmt('x.xx')),
                      frmt_structure(group_val = list(group1 = ".default"), label_val = c(".default"), frmt_combine('{mean} ({sd})', mean = frmt('x.xx'), sd = frmt('x.xxx'))),
                      frmt_structure(group_val = list(group1 = ".default"), label_val = c(".default"), n = frmt('x')),
                      frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "BILIRUBIN"), min = frmt('x.xx')),
                      frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "BILIRUBIN"), max = frmt('x.xx')),
                      frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "BILIRUBIN"), median = frmt('x.xx')),
                      frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "BILIRUBIN"), frmt_combine('{mean} ({sd})', mean = frmt('x.xx'), sd = frmt('x.xxx'))),
                      frmt_structure(group_val = list(group1 = "CHEM"), label_val = c("ALANINE AMINOTRANSFERASE", "BILIRUBIN"), n = frmt('x')),
                      frmt_structure(group_val = list(group1 = c("CHEM", "HEM")), label_val = c("CHOLESTEROL", "EOSINOPHILS"), min = frmt('x.xxx')),
                      frmt_structure(group_val = list(group1 = c("CHEM", "HEM")), label_val = c("CHOLESTEROL", "EOSINOPHILS"), max = frmt('x.xxx')),
                      frmt_structure(group_val = list(group1 = c("CHEM", "HEM")), label_val = c("CHOLESTEROL", "EOSINOPHILS"), median = frmt('x.xxx')),
                      frmt_structure(group_val = list(group1 = c("CHEM", "HEM")), label_val = c("CHOLESTEROL", "EOSINOPHILS"), frmt_combine('{mean} ({sd})', mean = frmt('x.xxx'), sd = frmt('x.xxxx'))),
                      frmt_structure(group_val = list(group1 = c("CHEM", "HEM")), label_val = c("CHOLESTEROL", "EOSINOPHILS"), n = frmt('x')))
  expect_equal(bp,
               bp_man)

  dat_sigdig <- tibble::tribble(
    ~group1,  ~ group2, ~ lbl, ~ sigdig,
    "CHEM", "BILIRUBIN", "v1", 1,
    "CHEM",  ".default", "v2",1,
    "HEM", "CHOLESTEROL", "v1",2,
    ".default", "EOSINOPHILS", "v1",2
  )
  bp <- tfrmt_sigdig(dat_sigdig, group = vars(group1, group2), label = lbl, param_defaults = param_set())$body_plan
  bp_man <-  body_plan(
    frmt_structure(group_val = list(group1 = "CHEM", group2 = ".default"), label_val = c("v2"), min = frmt('x.xx')),
    frmt_structure(group_val = list(group1 = "CHEM", group2 = ".default"), label_val = c("v2"), max = frmt('x.xx')),
    frmt_structure(group_val = list(group1 = "CHEM", group2 = ".default"), label_val = c("v2"), median = frmt('x.xx')),
    frmt_structure(group_val = list(group1 = "CHEM", group2 = ".default"), label_val = c("v2"), frmt_combine('{mean} ({sd})', mean = frmt('x.xx'), sd = frmt('x.xxx'))),
    frmt_structure(group_val = list(group1 = "CHEM", group2 = ".default"), label_val = c("v2"), n = frmt('x')),
    frmt_structure(group_val = list(group1 = ".default", group2 = c("EOSINOPHILS")), label_val = c("v1"), min = frmt('x.xxx')),
    frmt_structure(group_val = list(group1 = ".default", group2 = c("EOSINOPHILS")), label_val = c("v1"), max = frmt('x.xxx')),
    frmt_structure(group_val = list(group1 = ".default", group2 = c("EOSINOPHILS")), label_val = c("v1"), median = frmt('x.xxx')),
    frmt_structure(group_val = list(group1 = ".default", group2 = c("EOSINOPHILS")), label_val = c("v1"), frmt_combine('{mean} ({sd})', mean = frmt('x.xxx'), sd = frmt('x.xxxx'))),
    frmt_structure(group_val = list(group1 = ".default", group2 = c("EOSINOPHILS")), label_val = c("v1"), n = frmt('x')),
    frmt_structure(group_val = list(group1 = "CHEM", group2 = "BILIRUBIN"), label_val = c("v1"), min = frmt('x.xx')),
    frmt_structure(group_val = list(group1 = "CHEM", group2 = "BILIRUBIN"), label_val = c("v1"), max = frmt('x.xx')),
    frmt_structure(group_val = list(group1 = "CHEM", group2 = "BILIRUBIN"), label_val = c("v1"), median = frmt('x.xx')),
    frmt_structure(group_val = list(group1 = "CHEM", group2 = "BILIRUBIN"), label_val = c("v1"), frmt_combine('{mean} ({sd})', mean = frmt('x.xx'), sd = frmt('x.xxx'))),
    frmt_structure(group_val = list(group1 = "CHEM", group2 = "BILIRUBIN"), label_val = c("v1"), n = frmt('x')),
    frmt_structure(group_val = list(group1 = "HEM", group2 = "CHOLESTEROL"), label_val = c("v1"), min = frmt('x.xxx')),
    frmt_structure(group_val = list(group1 = "HEM", group2 = "CHOLESTEROL"), label_val = c("v1"), max = frmt('x.xxx')),
    frmt_structure(group_val = list(group1 = "HEM", group2 = "CHOLESTEROL"), label_val = c("v1"), median = frmt('x.xxx')),
    frmt_structure(group_val = list(group1 = "HEM", group2 = "CHOLESTEROL"), label_val = c("v1"), frmt_combine('{mean} ({sd})', mean = frmt('x.xxx'), sd = frmt('x.xxxx'))),
    frmt_structure(group_val = list(group1 = "HEM", group2 = "CHOLESTEROL"), label_val = c("v1"), n = frmt('x'))
    )
  expect_equal(bp,
               bp_man)
})


test_that("tfrmt_sigdig returns a tfrmt", {

  dat_sigdig <- tibble::tribble(
    ~group1,  ~ group2, ~ sigdig,
    "CHEM",  "ALANINE AMINOTRANSFERASE", 1,
    "CHEM", "BILIRUBIN", 1

  )
  t_frmt <- tfrmt_sigdig(
    sigdig_df = dat_sigdig,
    group = group1,
    label = group2
  )

  expect_s3_class(t_frmt,"tfrmt")

  expect_equal( t_frmt$group, vars(group1), ignore_attr = TRUE)
  expect_equal( t_frmt$label, quo(group2), ignore_attr = TRUE)

  # body plan contains 1 frmt_structure per default param per # of rows in sigddig spec
  expect_equal(length(t_frmt$body_plan),
               length(param_set()))

  myprms <- param_set(newprm=2)
  t_frmt <- tfrmt_sigdig(
    sigdig_df = dat_sigdig,
    group = group1,
    label = group2,
    param_defaults = myprms
  )
  expect_equal(length(t_frmt$body_plan),
               length(param_set())+1)

})


test_that("varying group/label inputs",{

  dat_sigdig <- tibble::tribble(
    ~group1,  ~ group2, ~ lbl, ~ sigdig,
    "CHEM", "BILIRUBIN", "v1", 1,
    "CHEM",  ".default", "v2",1,
    "HEM", "CHOLESTEROL", "v1",2,
    ".default", "EOSINOPHILS", "v1",2
  )

  # if no group or label, assume all non-sigdig columns are groups
  t_out <- tfrmt_sigdig(sigdig_df = dat_sigdig)

  expect_equal(t_out$group, vars(group1, group2, lbl), ignore_attr = TRUE)

  expect_equal(t_out$label, quo())

  # if only some are specified, assume the rest are groups
  t_out <- tfrmt_sigdig(dat_sigdig, label = group2)

  expect_equal(t_out$group, vars(group1, lbl), ignore_attr = TRUE)
  expect_equal(t_out$label, quo(group2), ignore_attr = TRUE)

  # including a group or label that is not present in the data
  expect_warning(
    t_out <- tfrmt_sigdig(dat_sigdig, group = vars(lbl, grp3), label = mylab),
    paste0("`sigdig_df` input does not contain the following group params: grp3")
  )

  expect_equal(t_out$group, vars(lbl, grp3), ignore_attr = TRUE)
  expect_equal(t_out$label, quo(mylab), ignore_attr = TRUE)

  expect_error(
    tfrmt_sigdig(dat_sigdig, group = group4, label = group3),
    paste0("`sigdig_df` input does not contain any of the specified group/label params:\n",
           "group: group4\n",
           "label: group3")
  )


})




test_that("group vars specified in tfrmt but not sigdig data are represented in body_plan",{

  dat_sigdig <- tibble::tribble(
    ~group1,  ~ group2, ~ sigdig,
    "test1", ".default", 1
  )
  expect_warning(
    bp <- tfrmt_sigdig(dat_sigdig, group = vars(group1, newgrp), label = quo(group2), param_defaults = param_set())$body_plan,
    paste0("`sigdig_df` input does not contain the following group params: newgrp")
  )

    bp_man <- body_plan(frmt_structure(group_val = list(group1 = "test1", newgrp = ".default"), label_val = c(".default"), min = frmt('x.xx')),
                      frmt_structure(group_val = list(group1 = "test1", newgrp = ".default"), label_val = c(".default"), max = frmt('x.xx')),
                      frmt_structure(group_val = list(group1 = "test1", newgrp = ".default"), label_val = c(".default"), median = frmt('x.xx')),
                      frmt_structure(group_val = list(group1 = "test1", newgrp = ".default"), label_val = c(".default"), frmt_combine('{mean} ({sd})', mean = frmt('x.xx'), sd = frmt('x.xxx'))),
                      frmt_structure(group_val = list(group1 = "test1", newgrp = ".default"), label_val = c(".default"), n = frmt('x')))

  expect_equal(bp, bp_man)


  expect_warning(
    bp <- tfrmt_sigdig(dat_sigdig, group = vars(group1, newgrp, group2), label = quo(mylab), param_defaults = param_set())$body_plan,
    paste0("`sigdig_df` input does not contain the following group params: newgrp")
  )
  bp_man <- body_plan(frmt_structure(group_val = list(group1 = "test1", newgrp = ".default", group2 = ".default"), label_val = c(".default"), min = frmt('x.xx')),
                      frmt_structure(group_val = list(group1 = "test1", newgrp = ".default", group2 = ".default"), label_val = c(".default"), max = frmt('x.xx')),
                      frmt_structure(group_val = list(group1 = "test1", newgrp = ".default", group2 = ".default"), label_val = c(".default"), median = frmt('x.xx')),
                      frmt_structure(group_val = list(group1 = "test1", newgrp = ".default", group2 = ".default"), label_val = c(".default"), frmt_combine('{mean} ({sd})', mean = frmt('x.xx'), sd = frmt('x.xxx'))),
                      frmt_structure(group_val = list(group1 = "test1", newgrp = ".default", group2 = ".default"), label_val = c(".default"), n = frmt('x')))

  expect_equal(bp, bp_man)
})

test_that("tfrmt_sigdig can be layered onto another tfrmt",{

  prev_tfrmt <- tfrmt(group = vars(group1, group2),
                      label = lblvar,
                      body_plan = body_plan(
                        frmt_structure(group_val = ".default", label_val = ".default", frmt("xx.xx"))
                      ))
  dat_sigdig <- tibble::tribble(
    ~group1,  ~ group2, ~ sigdig,
    "test1", "test2", 1
  )
  new_tfrmt <- tfrmt_sigdig(dat_sigdig,
                            tfrmt_obj =
                              prev_tfrmt)

  expect_equal(new_tfrmt$group, prev_tfrmt$group)
  expect_equal(new_tfrmt$lblvar, prev_tfrmt$lblvar)

  bp_man <- body_plan(frmt_structure(group_val = ".default", label_val = ".default", frmt('xx.xx')),
                      frmt_structure(group_val = list(group1 = "test1", group2 = "test2"), label_val = c(".default"), min = frmt('x.xx')),
                      frmt_structure(group_val = list(group1 = "test1", group2 = "test2"), label_val = c(".default"), max = frmt('x.xx')),
                      frmt_structure(group_val = list(group1 = "test1", group2 = "test2"), label_val = c(".default"), median = frmt('x.xx')),
                      frmt_structure(group_val = list(group1 = "test1", group2 = "test2"), label_val = c(".default"), frmt_combine('{mean} ({sd})', mean = frmt('x.xx'), sd = frmt('x.xxx'))),
                      frmt_structure(group_val = list(group1 = "test1", group2 = "test2"), label_val = c(".default"), n = frmt('x')))

  expect_equal(new_tfrmt$body_plan, bp_man)



  prev_tfrmt <- tfrmt(group = somegrp,
                      label = group2)
  dat_sigdig <- tibble::tribble(
    ~group1,  ~ group2, ~ sigdig,
    "test1", "test2", 1
  )
  new_tfrmt <- tfrmt_sigdig(dat_sigdig,
                            group = group1,
                            tfrmt_obj = prev_tfrmt)

  expect_equal(new_tfrmt$group, vars(group1), ignore_attr = TRUE)
  expect_equal(new_tfrmt$label, quo(group2), ignore_attr = TRUE)

})


test_that("tfrmt_sigdig correctly passes the 'missing' argument to the body_plan", {

  # Setup minimal input data
  sig_input <- tibble::tribble(
    ~group1,   ~sigdig,
    "CHEM",    1
  )

  target_missing <- "MISSING_DATA"

  # Create the tfrmt via tfrmt_sigdig
  my_tfrmt <- tfrmt_sigdig(
    sigdig_df = sig_input,
    group = group1,
    param_defaults = param_set(),
    missing = target_missing
  )

  bp_actual <- my_tfrmt$body_plan

  # Manually construct the expected body_plan
  bp_man <- body_plan(
    frmt_structure(group_val = list(group1 = "CHEM"), label_val = c(".default"), min = frmt('x.xx', missing = target_missing)),
    frmt_structure(group_val = list(group1 = "CHEM"), label_val = c(".default"), max = frmt('x.xx', missing = target_missing)),
    frmt_structure(group_val = list(group1 = "CHEM"), label_val = c(".default"), median = frmt('x.xx', missing = target_missing)),
    frmt_structure(group_val = list(group1 = "CHEM"), label_val = c(".default"),
                   frmt_combine('{mean} ({sd})',
                                mean = frmt('x.xx', missing = target_missing),
                                sd = frmt('x.xxx', missing = target_missing),
                                missing = target_missing)),
    frmt_structure(group_val = list(group1 = "CHEM"), label_val = c(".default"), n = frmt('x', missing = target_missing))
  )

  expect_equal(bp_actual, bp_man)
})

#' Apply Footnote Plan
#'
#' @param gt gt object  to potentially add a footnote to
#' @param tfrmt tfrmt object
#' @param footnote_loc list containing footnote location
#'
#' @return gt object
#' @noRd
#'
#' @importFrom gt tab_footnote md opt_footnote_marks tab_options
apply_footnote_plan <- function(gt, tfrmt,footnote_loc){
  if(is.null(tfrmt$footnote_plan)){
    gt
  } else {
    for (i in 1:length(tfrmt$footnote_plan$struct_list)) {

      gt <- gt %>%
        apply_general_footnote(footnote_loc[[i]]) %>%
        apply_cells_column_labels(footnote_loc[[i]]) %>%
        apply_cells_column_spanners(footnote_loc[[i]]) %>%
        apply_cells_stub(tfrmt,footnote_loc[[i]]) %>%
        apply_cells_row_groups(tfrmt,footnote_loc[[i]]) %>%
        apply_cells_body(footnote_loc[[i]])

    }
    gt %>%
      opt_footnote_marks(marks = tfrmt$footnote_plan$marks ) %>%
      tab_options(footnotes.order = tfrmt$footnote_plan$order)

  }
}


#' Apply footnote
#'
#' @param gt gt object  to potentially add a footnote to
#' @param loc  list containing footnote text
#'
#' @return gt object
#' @noRd
#'
#' @importFrom gt tab_footnote
apply_general_footnote <- function(gt,loc){
  if(length(loc$row)==0 && length(loc$col)==0){
    gt <- gt %>%
      tab_footnote(
        footnote =loc$note)


  }
  gt

}

#' Apply Cells Column Labels
#'
#' @param gt gt object to potentially add a footnote to
#' @param loc list containing location of footnote and footnote text
#'
#'
#' @return gt object
#' @noRd
#'
#' @importFrom gt tab_footnote md opt_footnote_marks
apply_cells_column_labels <- function(gt,loc){
  # check row is empty - therefore a column footnote, and not a spanning column

  if(is.null(loc$row) && loc$spanning ==FALSE){

      gt<- gt %>%
        tab_footnote(
          footnote = loc$note,
          locations = cells_column_labels(columns = loc$col)
        )

  }
  gt


}


#' Apply Cells Column Spanners
#'
#' @param gt gt object to potentially add a footnote to
#' @param loc list containing location of footnote and footnote text
#'
#' @return gt object
#' @noRd
#'
#' @importFrom gt tab_footnote md opt_footnote_marks
apply_cells_column_spanners <- function(gt,loc){
  # check row is empty - therefore a column footnote
  if(!is.null(loc) && is.null(loc$row) && loc$spanning ==TRUE){

    gt<- gt %>%
      tab_footnote(
        footnote = loc$note,
        locations = cells_column_spanners(spanners = loc$col
        )
      )


  }
  gt
}


#' Apply Cells Stub
#'
#' @param gt gt object to potentially add a footnote to
#' @param tfrmt tfrmt object
#' @param loc list containing location of footnote and footnote text
#'
#' @return gt object
#' @noRd
#'
#' @importFrom gt tab_footnote md opt_footnote_marks
#' @importFrom rlang quo_get_expr
apply_cells_stub <-  function(gt,tfrmt,loc){
  if(length(loc$col)>0){

  in_stub <- (
    #column row grp label (all stubs)
    (!is.null(tfrmt$row_grp_plan) &&
                tfrmt$row_grp_plan$label_loc$location=="column") &&
    all(loc$col %in% map_chr(c(tfrmt$group, tfrmt$label), as_label)) ||
      # in the label
      (all(loc$col==as_label(tfrmt$label)))
    )

  if(in_stub){

    gt<- gt %>%
      tab_footnote(
        footnote = loc$note,
        locations = cells_stub(rows = loc$row, columns = loc$col)
      )


  }

  }
  gt

}



#' Apply Cells Row Groups
#'
#' @param gt gt object to potentially add a footnote to
#' @param tfrmt tfrmt object
#' @param loc list containing location of footnote and footnote text
#'
#' @return gt object
#' @noRd
#'
#' @importFrom gt tab_footnote md opt_footnote_marks
apply_cells_row_groups <- function(gt,tfrmt,loc){
  if(length(loc$col)>0){
    if(all(loc$col %in% map_chr(tfrmt$group, as_label) )){

      gt<- gt %>%
        tab_footnote(
          footnote = loc$note,
          locations = cells_row_groups(groups = loc$row)
          )

     }

  }
  gt
}


#' Apply Cells Body
#'
#' @param gt gt object to potentially add a footnote to
#' @param loc list containing location of footnote and footnote text
#'
#' @return gt object
#' @noRd
#'
#' @importFrom gt tab_footnote md opt_footnote_marks
apply_cells_body<- function(gt,loc){
  if(!is.null(loc$col) && !is.null(loc$row)){
    gt<- gt %>%
      tab_footnote(
        footnote = loc$note,
        locations = cells_body(columns = loc$col, rows = loc$row
        )
      )
  }
  gt

}

#' Print mock table to GT
#'
#' @param tfrmt tfrmt the mock table will be based off of
#' @param .data Optional data. If this is missing, group values, labels values
#'   and parameter values will be estimated based on the tfrmt
#' @param .default sequence to replace the default values if a dataset isn't
#'   provided
#' @param n_cols the number of columns. This will only be used if mock data isn't
#'   provided. If not supplied, it will default to using the `col_plan` from the
#'   `tfrmt`. If neither are available it will use 3.
#' @param .unicode_ws Whether to convert white space to unicode in preparation for output
#'
#' @return a stylized gt object
#' @export
#'
#' @section Examples:
#'```r
#'
#'   # Create tfrmt specification
#'   tfrmt_spec <- tfrmt( label = label, column =
#'   column, param = param, body_plan = body_plan( frmt_structure(group_val =
#'   ".default", label_val = ".default", frmt_combine( "{count} {percent}",
#'   count = frmt("xxx"), percent = frmt_when("==100"~ frmt(""), "==0"~ "",
#'   "TRUE" ~ frmt("(xx.x%)")))) ))
#'
#'   # Print mock table using default
#'   print_mock_gt(tfrmt = tfrmt_spec)
#'
#'```
#'
#'   \if{html}{\out{ `r "<img src=\"https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/master/images/example_print_mock_gt1.png\" alt = \"Simple 3 by 3 table without values\" style=\"width:50\\%;\">"` }}
#'
#'```r
#'   # Create mock data
#'   df <- tidyr::crossing(label = c("label 1", "label 2",
#'   "label 3"), column = c("placebo", "trt1", "trt2"), param = c("count",
#'   "percent"))
#'
#'   # Print mock table using mock data
#'   print_mock_gt(tfrmt_spec, df)
#'
#'```
#'
#'   \if{html}{\out{ `r "<img src=\"https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/master/images/example_print_mock_gt2.png\" alt = \"Simple 3 by 3 table without values, but with column names\" style=\"width:50\\%;\">"` }}
#'
#'
#' @importFrom gt gt tab_header tab_style cell_text cells_body px
#' @importFrom rlang quo_is_missing sym quo is_empty
#' @importFrom dplyr vars
#' @importFrom purrr quietly safely
print_mock_gt <- function(tfrmt,
                          .data = NULL,
                          .default = 1:3,
                          n_cols = NULL,
                          .unicode_ws = TRUE) {

  # fill param, column if not provided
  if (quo_is_missing(tfrmt$param)){
    message("`tfrmt` will need a `param` value to `print_to_gt` when data is available")
    tfrmt$param <- quo(!!sym("__tfrmt__param"))
  }
  if (is_empty(tfrmt$column)){
    message("`tfrmt` will need `column` value(s) to `print_to_gt` when data is available")
    tfrmt$column <- vars(!!sym("__tfrmt__column"))
  }

  if(quo_is_missing(tfrmt$value)){
    message("Message: `tfrmt` will need `value` value to `print_to_gt` when data is available")
    tfrmt$value <- quo(!!sym("__tfrmt__val"))
  }

  if(is.null(tfrmt$body_plan)){
    tfrmt$body_plan <- body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("X.X"))
    )
  }

  if(is.null(.data)){
    .data <- make_mock_data(tfrmt, .default, n_cols)
  }else{
    ## check that if value column exists in data, remove it for mocking
    select_try <- safely(quietly(tidyselect::eval_select))(tfrmt$value, data = .data)
    if(!is.null(select_try$result)){
      message(" Removing `",as_label(tfrmt$value),"` from input data for mocking.")
      .data <- .data[,-select_try$result$result]
    }
  }

  apply_tfrmt(.data, tfrmt, mock = TRUE) %>%
    cleaned_data_to_gt(tfrmt, .unicode_ws)

}

#' Print to gt
#'
#' @param tfrmt tfrmt object that will dictate the structure of the table
#' @param .data Data to style in order to make the table
#' @param .unicode_ws Whether to convert white space to unicode in preparation for output
#'
#' @return a stylized gt object
#' @export
#'
#' @section Examples:
#'
#' ```r
#' library(dplyr)
#' # Create tfrmt specification
#' tfrmt_spec <- tfrmt(
#'   label = label,
#'   column = column,
#'   param = param,
#'   value=value,
#'   body_plan = body_plan(
#'     frmt_structure(group_val = ".default", label_val = ".default",
#'                    frmt_combine(
#'                      "{count} {percent}",
#'                      count = frmt("xxx"),
#'                      percent = frmt_when("==100"~ frmt(""),
#'                                          "==0"~ "",
#'                                          "TRUE" ~ frmt("(xx.x%)"))))
#'   ))
#'
#' # Create data
#' df <- tidyr::crossing(label = c("label 1", "label 2"),
#'                column = c("placebo", "trt1"),
#'                param = c("count", "percent")) |>
#'       dplyr::mutate(value=c(24,19,2400/48,1900/38,5,1,500/48,100/38))
#'
#' print_to_gt(tfrmt_spec,df)
#'
#' ```
#' \if{html}{\out{
#' `r "<img src=\"https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/master/images/example_print_to_gt.png\" alt = \"2 by 2 table with labels down the side and placebo and trt1 across the top\" style=\"width:50\\%;\">"`
#' }}
#'
#' @importFrom gt gt tab_header tab_style cell_text cells_body tab_options
print_to_gt <- function(tfrmt, .data, .unicode_ws = TRUE){
  if(!is_tfrmt(tfrmt)){
    stop("Requires a tfrmt object")
  }

  if(!is.data.frame(.data)){
    stop("Requires data, if not available please use `print_mock_gt()`")
  }
  apply_tfrmt(.data, tfrmt, mock = FALSE) %>%
    cleaned_data_to_gt(tfrmt, .unicode_ws)

}


#' Do all the formatting for the GT
#' @rdname cleaned_data_to_gt
#' @export
cleaned_data_to_gt <- function(.data, tfrmt, .unicode_ws){
  UseMethod("cleaned_data_to_gt", .data)
}

#' Apply formatting to a list of tables
#'
#' @param .data list of cleaned datasets
#' @param tfrmt tfrmt
#' @param .unicode_ws Whether to convert white space to unicode in preparation for output
#'
#' @return gt_group object
#' @rdname cleaned_data_to_gt
#' @export
#'
#' @keywords internal
#' @importFrom gt gt_group
#' @importFrom purrr map2
cleaned_data_to_gt.list <- function(.data, tfrmt, .unicode_ws){

  map(.data, ~cleaned_data_to_gt.default(.x, tfrmt, .unicode_ws)) %>%
    gt_group(.list = .)
}
#' Apply formatting to a single table
#'
#' @param .data cleaned dataset
#' @param tfrmt tfrmt
#' @param .unicode_ws Whether to convert white space to unicode in preparation for output
#'
#' @return GT object
#' @rdname cleaned_data_to_gt
#' @export
#'
#' @keywords internal
#' @importFrom gt cells_stub cells_row_groups default_fonts cell_borders
#'   opt_table_font tab_options tab_style cell_text px cells_column_spanners
#'   cells_body cells_column_labels md cols_hide sub_missing tab_stubhead tab_source_note
cleaned_data_to_gt.default <- function(.data, tfrmt, .unicode_ws){

  existing_grp <- tfrmt$group %>%
    keep(function(x){
      as_label(x) %in% names(.data)
    })
  rowname_col <- NULL

  if (length(existing_grp)>0){
    if (!is.null(tfrmt$row_grp_plan) && !tfrmt$row_grp_plan$label_loc$location=="column") {

      .data <- .data %>%
        group_by(!!!existing_grp)

    } else {

      # drop groups into row names
      rowname_col <- existing_grp
    }
  }

  if (!"..tfrmt_row_grp_lbl" %in% names(.data)) {
    # keep attribute for footnotes
    attr_footnote <- attr(.data,".footnote_locs")
    attr_stub_header <- attr(.data,".stub_header")
    .data <- mutate(.data, ..tfrmt_row_grp_lbl = FALSE)
    attr(.data,".footnote_locs") <- attr_footnote
    attr(.data,".stub_header") <- attr_stub_header
  }

  if (!rlang::quo_is_missing(tfrmt$label) &&
        rlang::as_label(tfrmt$label) %in% names(.data)) {

    rowname_col <- c(rowname_col, tfrmt$label)
  }

  # convert to character if not null
  rowname_col <- if (!is.null(rowname_col)) map_chr(rowname_col, rlang::as_label)

  gt_out <- .data %>%
    gt(
      rowname_col = rowname_col
    )  %>%
    sub_missing(
      rows = .data$..tfrmt_row_grp_lbl==TRUE,
      missing_text = ""
    ) %>%
    cols_hide(columns = "..tfrmt_row_grp_lbl") %>%
    format_gt_column_labels(.data)

  # group label in its own column
  if(!is.null(tfrmt$row_grp_plan) && tfrmt$row_grp_plan$label_loc$location == "column"){
    gt_out <- gt_out %>%
      tab_options(row_group.as_column = TRUE)
  }

  # stub header
  if (!is.null(attr(.data, ".stub_header"))){
    gt_out <- gt_out %>%
      tab_stubhead(label = do.call(
        md,
        list(attr(.data, ".stub_header"))
      )
      )
  }

  gt_out_final  <- gt_out %>%
    tab_style(
      style = list(
        cell_text(whitespace = "pre-wrap", align = "left")
      ),
      locations = list(cells_stub(columns = rowname_col), cells_row_groups())
    ) %>%
    tab_options(
      table.font.size = 14,
      data_row.padding = px(1),
      summary_row.padding = px(1),
      grand_summary_row.padding = px(1),
      footnotes.padding = px(1),
      source_notes.padding = px(1),
      row_group.padding = px(1),
      stub.border.width = px(0),
      stub.border.color = "transparent",
      stub_row_group.border.width = px(0),
      stub_row_group.border.color = "transparent",
      row_group.border.bottom.width = px(0),
      row_group.border.bottom.color = "transparent",
      row_group.border.top.color = "transparent",
      table.font.names = c("Courier", default_fonts()),
      page.numbering = TRUE,
      page.header.use_tbl_headings = FALSE,
      page.footer.use_tbl_notes = TRUE,
      page.orientation = "landscape") %>%

    tab_style(
      style = cell_text(whitespace = "pre-wrap", align = "center"),
      locations = list(cells_column_spanners(),
                       cells_column_labels(),
                       cells_body(
                         columns = tidyselect::everything()
                       ))
    ) %>%

    tab_style(
      style = cell_borders(
        sides = c("top","bottom"),
        color = "transparent"
      ),
      locations= list(
        cells_body(
          columns = tidyselect::everything(),
          rows = tidyselect::everything()
        ),
        cells_stub(),
        cells_row_groups()
      )
    )  %>%

    tab_style(
      style = cell_borders(
        sides = c("top"),
        color = "transparent",
        weight = px(0),
      ),
      locations= list(
        cells_column_labels()
      )) %>%

    tab_style(
      style = cell_borders(
        sides = c("bottom"),
        weight = px(0),
        color = "transparent"
      ),
      locations= list(
        cells_column_spanners()
      )) %>%
    tab_style(
      style = cell_text(font = c("Courier", default_fonts())),
      locations = list(cells_body(), cells_row_groups(), cells_stub(),
                       cells_column_labels(), cells_column_spanners())
    )

  # remove vertical line
  if (utils::packageVersion("gt") >= "1.3.0"){
    gt_out_final <- gt_out_final %>%
      tab_options(
        stub.separate = FALSE
      )
  }

  # add page note if applicable
  if (!is.null(attr(.data, ".page_note")) &&
      !is.null(tfrmt$page_plan) &&
      !tfrmt$page_plan$note_loc=="noprint"){

    if (tfrmt$page_plan$note_loc=="preheader"){

      gt_out_final <- gt_out_final  %>%
        tab_header(title = tfrmt$title,
                   subtitle = tfrmt$subtitle,
                   preheader = attr(.data, ".page_note"))

    } else if (tfrmt$page_plan$note_loc=="subtitle"){

      title <- tfrmt$title %||% ""
      subtitle <- paste0(tfrmt$subtitle, attr(.data, ".page_note"), collapse = "\n")

      gt_out_final <- gt_out_final  %>%
        tab_header(title = title,
                   subtitle = subtitle)

    } else {
      gt_out_final <- gt_out_final  %>%
        tab_header(title = tfrmt$title,
                   subtitle = tfrmt$subtitle)

      if (tfrmt$page_plan$note_loc=="source_note"){
        gt_out_final <- gt_out_final %>%
          tab_source_note(attr(.data, ".page_note"))
      }

    }
  } else {
    gt_out_final <- gt_out_final  %>%
      tab_header(title = tfrmt$title,
                 subtitle = tfrmt$subtitle)
  }

  # convert white space to unicode
  if (.unicode_ws){
    gt_out_final <- gt_out_final %>%
      convert_ws_unicode()
  }

  # add footnotes and output
  gt_out_final %>%
    apply_footnote_plan(tfrmt,attr(.data,".footnote_locs"))

}



#' Format gt column labels
#'
#' Makes a gt object and applies the spanning labels if relevant, as well as
#' markdown formatting of all column labels
#'
#' @param gt_table gt object
#' @param tfrmt tfrmt object
#'
#' @return gt object
#' @noRd
#' @importFrom tidyr pivot_longer
#' @importFrom stringr str_split
#' @importFrom gt cols_label tab_spanner md
#' @importFrom dplyr as_tibble desc coalesce left_join mutate
#' @importFrom purrr keep
#'
format_gt_column_labels <- function(gt_table, .data){

  spanning <- names(.data) %>% keep(str_detect, .tlang_delim)
  if(length(spanning) > 0){

    work_df<- names(.data) %>%
      keep(str_detect, .tlang_delim) %>%
      str_split(.tlang_delim, simplify = TRUE) %>%
      as_tibble( .name_repair = ~paste0("V", 1:length(.))) %>%
      mutate(cols = spanning) %>%
      pivot_longer(-"cols")

    lowest_lvl <- work_df %>% filter(.data$name == max(.data$name))

    spans_to_apply <- work_df %>%
      filter(.data$name != max(.data$name)) %>%
      arrange(desc(.data$name)) %>%
      group_by(.data$value) %>%
      nest(set = "cols") %>%
      mutate(set = map(.data$set, ~pull(.,.data$cols))) %>%
      filter(.data$value != "NA")

    for(i in 1:nrow(spans_to_apply)) {

      # convert column spanning labels to markdown format
      gt_table <- gt_table %>%
        tab_spanner(
          md(spans_to_apply$value[i]),
          columns = tidyselect::all_of(
            spans_to_apply$set[[i]]
          )
        )
    }

    # ensure all columns are represented
    lowest_lvl <- names(.data) %>%
      tibble(cols = .) %>%
      left_join(lowest_lvl, by = "cols") %>%
      mutate(value = coalesce(.data$value, .data$cols))

    renm_vals <- lowest_lvl %>%
      pull(.data$value)
    names(renm_vals) <-lowest_lvl %>%
      pull(.data$cols)


  } else {

    renm_vals <- names(.data)
    names(renm_vals) <- renm_vals

  }

  # convert lowest level column labels to markdown format
  gt_table %>%
    cols_label(.list=
                 lapply(renm_vals,md))


}

#' Convert gt whitespace to unicode text
#'
#' @param gt_table gt object
#'
#' @return gt object
#' @noRd
#' @importFrom gt text_transform cells_body cells_stub cells_column_labels cells_column_spanners
#' @importFrom stringr str_match str_c str_dup str_trim
#'
convert_ws_unicode <- function(gt_table){

  locations <- list(cells_body())

  if (sum(c(!is.na(gt_table[["_stub_df"]]$row_id),!is.na(gt_table[["_stub_df"]]$group_id)) > 0)){

    locations <- c(locations,
                   list(cells_stub()))
  }

  gt_table %>%
    text_transform(
      locations = locations,
      fn = function(x) {
        # leading and trailing whitespace is nonbreaking unicode whitespace to preserve alignment
        x_trimmed <- str_trim(x)
        space_left <- str_match(x, "^\\s*") %>% nchar()
        space_right <- str_match(x, "\\s*$") %>% nchar()
        space_right[x_trimmed == ""] <- 0

        str_c(
          str_dup("\U00A0", space_left),
          # 2 or more spaces are split into a combination of unicode whitespaces and
          # regular spaces for latex collapsing
          str_replace_all(x_trimmed, pattern = "\\s{2,}", break_duplicate_whitespace),
          str_dup("\U00A0", space_right)
        )


      }
    )
}

# split duplicate space characters with unicode whitespace ones
#' @param x whitespace vector of strings of length >1
#' @importFrom stringr str_sub
#' @noRd
break_duplicate_whitespace <- function(x){

  for(i in 1:length(x)){
    n_spaces <- nchar(x[i])
    if(n_spaces > 1 && !is.na(x[i])){
      #want to swap every even indice for a unicode character
      even_chars <- seq(from = 2, to = n_spaces, by = 2)
      for(j in even_chars){
        stringr::str_sub(x[i], j, j) <- "\u00A0"
      }
    }
  }


  x
}

test_that("convert_ws_unicode works as expected", {
  gt_with_ws <- data.frame(
    group = "    trailing.   and.   leading.  ws.   ",
    value = " ( x  x)"
  ) |>
    gt::gt()

  gt_with_unicode <- convert_ws_unicode(gt_with_ws)

  # test that metadata is present in gt ready to apply transform

  # columns to apply to
  expect_equal(
    gt_with_unicode$`_transforms`[[1]]$resolved$colnames,
    c("group", "value")
  )

  # rows to apply to
  expect_equal(
    gt_with_unicode$`_transforms`[[1]]$resolved$rows,
    1
  )

  # function to apply
  whitespace_function <- gt_with_unicode$`_transforms`[[1]]$fn

  test_strings <- c(
    "nospaces",
    " single spaces ",
    "  multiple   spaces  ",
    # 1 3 1 4 2 1
    "combination of   single and    multiple  spaces ",
    NA,
    "     leading    whitespace",
    'trailing  whitespace     '
  )

  unicode_strings <- c(
    "nospaces",
    "\u00A0single spaces\u00A0",
    "\u00A0\u00A0multiple \u00A0 spaces\u00A0\u00A0",
    "combination of \u00A0 single and \u00A0 \u00A0multiple \u00A0spaces\u00A0",
    NA,
    "\u00A0\u00A0\u00A0\u00A0\u00A0leading \u00A0 \u00A0whitespace",
    'trailing \u00A0whitespace\u00A0\u00A0\u00A0\u00A0\u00A0'
  )

  expect_equal(
    whitespace_function(test_strings),
    unicode_strings
  )
})

test_that("print_to_gt() works", {
  test_data <- tibble::tribble(
    ~group  , ~label  , ~span1   , ~span2  , ~lower     , ~param , ~val ,
    "mygrp" , "mylbl" , "span01" , "span1" , "lower1_a" , "prm"  ,    1 ,
    "mygrp" , "mylbl" , "span01" , "span1" , "lower1_b" , "prm"  ,    1 ,
    "mygrp" , "mylbl" , "span01" , "span2" , "lower2_a" , "prm"  ,    1 ,
    "mygrp" , "mylbl" , "span01" , "span2" , "lower2_b" , "prm"  ,    1 ,
    "mygrp" , "mylbl" , "span02" , "span3" , "lower2_a" , "prm"  ,    1 ,
    "mygrp" , "mylbl" , "span02" , "span3" , "lower2_b" , "prm"  ,    1
  )

  tfrmt_plan <- tfrmt(
    group = "group",
    label = "label",
    param = "param",
    column = c("span1", "span2", "lower"),
    value = "val",
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("x.xx")
      )
    )
  )

  expect_no_error(
    print_to_gt(
      tfrmt_plan,
      .data = test_data
    )
  )

  expect_snapshot(
    print_to_gt(
      tfrmt_plan,
      .data = test_data
    )[["_data"]]
  )
})

test_that("print_to_gt() complains with incorrect inputs", {
  # complains when the first argument is not `tfrmt`
  expect_snapshot(
    error = TRUE,
    print_to_gt(mtcars)
  )

  # complains when the `.data` argument is not a data.frame
  tfrmt_plan <- tfrmt(
    label = label,
    column = column,
    param = param,
    value = value,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{count} {percent}",
          count = frmt("xxx"),
          percent = frmt_when(
            "==100" ~ frmt(""),
            "==0" ~ "",
            "TRUE" ~ frmt("(xx.x%)")
          )
        )
      )
    )
  )

  expect_snapshot(
    error = TRUE,
    print_to_gt(tfrmt_plan, "foo")
  )
})

test_that("print_mock_gt() messages when tfrmt$param is missing", {
  # no message when tfrmt$param is not empty
  tfrmt_plan <- tfrmt(
    label = label,
    column = column,
    param = param,
    value = value,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{count} {percent}",
          count = frmt("xxx"),
          percent = frmt_when(
            "==100" ~ frmt(""),
            "==0" ~ "",
            "TRUE" ~ frmt("(xx.x%)")
          )
        )
      )
    )
  )

  expect_no_message(
    print_mock_gt(
      tfrmt_plan
    )
  )

  # message when tfrmt$param is empty
  tfrmt_plan_no_param <- tfrmt(
    label = label,
    column = column,
    value = value,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{count} {percent}",
          count = frmt("xxx"),
          percent = frmt_when(
            "==100" ~ frmt(""),
            "==0" ~ "",
            "TRUE" ~ frmt("(xx.x%)")
          )
        )
      )
    )
  )

  expect_message(
    print_mock_gt(
      tfrmt_plan_no_param
    ),
    "`tfrmt` will need a `param` value to `print_to_gt` when data is available",
    fixed = TRUE
  )
  expect_snapshot(
    print_mock_gt(
      tfrmt_plan_no_param
    )[["_data"]]
  )
})

test_that("print_mock_gt() messages when tfrmt$column is missing", {
  # no message when tfrmt$column is not missing
  tfrmt_plan <- tfrmt(
    label = label,
    column = column,
    param = param,
    value = value,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{count} {percent}",
          count = frmt("xxx"),
          percent = frmt_when(
            "==100" ~ frmt(""),
            "==0" ~ "",
            "TRUE" ~ frmt("(xx.x%)")
          )
        )
      )
    )
  )

  expect_no_message(
    print_mock_gt(
      tfrmt_plan
    )
  )

  # message when tfrmt$column is missing
  tfrmt_plan_no_column <- tfrmt(
    label = label,
    param = param,
    value = value,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{count} {percent}",
          count = frmt("xxx"),
          percent = frmt_when(
            "==100" ~ frmt(""),
            "==0" ~ "",
            "TRUE" ~ frmt("(xx.x%)")
          )
        )
      )
    )
  )

  expect_message(
    print_mock_gt(
      tfrmt_plan_no_column
    ),
    "`tfrmt` will need `column` value(s) to `print_to_gt` when data is available",
    fixed = TRUE
  )

  expect_snapshot(
    print_mock_gt(
      tfrmt_plan_no_column
    )[["_data"]]
  )
})

test_that("print_mock_gt() messages when tfrmt$value is missing", {
  # no message when tfrmt$value is not missing
  tfrmt_plan <- tfrmt(
    label = label,
    column = column,
    param = param,
    value = value,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{count} {percent}",
          count = frmt("xxx"),
          percent = frmt_when(
            "==100" ~ frmt(""),
            "==0" ~ "",
            "TRUE" ~ frmt("(xx.x%)")
          )
        )
      )
    )
  )

  expect_no_message(
    print_mock_gt(
      tfrmt_plan
    )
  )

  # message when tfrmt$value is missing
  tfrmt_plan_no_value <- tfrmt(
    label = label,
    column = column,
    param = param,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{count} {percent}",
          count = frmt("xxx"),
          percent = frmt_when(
            "==100" ~ frmt(""),
            "==0" ~ "",
            "TRUE" ~ frmt("(xx.x%)")
          )
        )
      )
    )
  )

  expect_message(
    print_mock_gt(
      tfrmt_plan_no_value
    ),
    "`tfrmt` will need `value` value to `print_to_gt` when data is available",
    fixed = TRUE
  )

  expect_snapshot(
    print_mock_gt(
      tfrmt_plan_no_value
    )[["_data"]]
  )
})

test_that("print_mock_gt() with missing body_plan", {
  # when tfrmt$body_plan is missing, one is made up on the fly
  tfrmt_plan_no_body_plan <- tfrmt(
    label = label,
    column = column,
    param = param,
    value = value
  )

  expect_no_message(
    print_mock_gt(
      tfrmt_plan_no_body_plan
    )
  )

  tfrmt_plan_default_body_plan <- tfrmt(
    label = label,
    column = column,
    param = param,
    value = value,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("X.X")
      )
    )
  )

  expect_snapshot(
    print_mock_gt(
      tfrmt_plan_no_body_plan
    )[["_data"]]
  )

  expect_identical(
    print_mock_gt(
      tfrmt_plan_no_body_plan
    )[["_data"]],
    print_mock_gt(
      tfrmt_plan_default_body_plan
    )[["_data"]],
    # .col_plan_vars is a list of quosures and their environments would not match
    ignore_attr = ".col_plan_vars"
  )
})

test_that("print_mock_data() removes `value` when it exists in the input data", {
  data <- tidyr::crossing(
    label = c(
      "Intent-To-Treat (ITT)",
      "Safety",
      "Efficacy",
      "Complete Week 24",
      "Complete Study"
    ),
    column = c(
      "Placebo\n(N=XX)",
      "Xanomeline\nLow Dose\n(N=XX)",
      "Xanomeline\nHigh Dose\n(N=XX)",
      "Total\n(N=XXX)"
    ),
    param = c(
      "n",
      "percent"
    )
  ) |>
    dplyr::mutate(
      value_to_remove = 1
    )

  plan <- tfrmt(
    label = "label",
    column = "column",
    param = "param",
    value = "value_to_remove",
    title = "Summary of Populations",
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{n} ({percent}%)",
          n = frmt("xx"),
          percent = frmt("xxx")
        )
      )
    )
  )

  expect_message(
    print_mock_gt(
      plan,
      data
    ),
    "Removing `value_to_remove` from input data for mocking.",
    fixed = TRUE
  )

  expect_snapshot(
    print_mock_gt(
      plan,
      data
    )["_data"]
  )
})

test_that("cleaned_data_to_gt() works", {
  tfrmt <- tfrmt(
    # specify columns in the data
    group = group,
    label = label,
    column = time,
    param = param,
    value = value,
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(
        "gtdefault"
      )
    )
  )

  test_data <- tibble::tibble(
    label = c("Obs", "Lev", "Lev+5FU"),
    group = c("A", "A", "A"),
    `0` = c("630", "620", "608"),
    `1000` = c("372", "360", "425"),
    `2000` = c("256", "266", "328"),
    `3000` = c("11", "8", "14")
  )

  # when the input is a data.frame the output is a `gt_tbl`
  expect_s3_class(
    cleaned_data_to_gt(
      test_data,
      tfrmt,
      .unicode_ws = TRUE
    ),
    "gt_tbl"
  )

  expect_snapshot(
    cleaned_data_to_gt(
      test_data,
      tfrmt,
      .unicode_ws = TRUE
    )[["_data"]]
  )
})

test_that("cleaned_data_to_gt.list() works", {
  tfrmt <- tfrmt(
    # specify columns in the data
    group = group,
    label = label,
    column = time,
    param = param,
    value = value,
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(
        "gtdefault"
      )
    )
  )

  test_data <- tibble::tibble(
    label = c("Obs", "Lev", "Lev+5FU"),
    group = c("A", "A", "A"),
    `0` = c("630", "620", "608"),
    `1000` = c("372", "360", "425"),
    `2000` = c("256", "266", "328"),
    `3000` = c("11", "8", "14")
  )

  test_data2 <- tibble::tibble(
    label = c("Obs2", "Lev2", "Lev+5FU"),
    group = c("B", "B", "B"),
    `0` = c("631", "621", "609"),
    `1000` = c("373", "361", "426"),
    `2000` = c("257", "267", "329"),
    `3000` = c("12", "9", "15")
  )

  # when the input is a list the output is a `gt_group`
  expect_s3_class(
    cleaned_tables <- cleaned_data_to_gt(
      list(
        test_data,
        test_data2
      ),
      tfrmt,
      .unicode_ws = TRUE
    ),
    "gt_group"
  )

  expect_length(
    cleaned_tables$gt_tbls$gt_tbl,
    2
  )

  expect_snapshot(
    cleaned_tables$gt_tbls$gt_tbl[[1]][["_data"]]
  )

  expect_snapshot(
    cleaned_tables$gt_tbls$gt_tbl[[2]][["_data"]]
  )
})

test_that("cleaned_data_to_gt() with page_plan & note location in subtitle", {
  # Create the original data frame
  original_data <- tibble::tibble(
    Group = rep(c("Age (y)", "Sex"), c(3, 3)),
    Label = rep("n", 6),
    Column = rep(c("Placebo", "Treatment", "Total"), times = 2),
    Param = rep("n", 6),
    Value = c(12, 14, 31, 20, 32, 18)
  ) |>
    dplyr::mutate(
      ord1 = dplyr::if_else(Group == "Age (y)", 1, 2)
    )

  # Duplicate the data and add the `by group` column
  data_101 <- original_data |>
    dplyr::mutate(
      `by group` = "101"
    )
  data_102 <- original_data |>
    dplyr::mutate(
      `by group` = "102"
    )

  # Combine the two data frames
  data <- dplyr::bind_rows(data_101, data_102)

  # Create mock big Ns
  big_ns <- data |>
    dplyr::group_by(
      `by group`,
      Column
    ) |>
    dplyr::summarise(
      Value = sum(Value)
    ) |>
    dplyr::mutate(
      Param = "bigN"
    ) |>
    dplyr::filter(
      `by group` == "101"
    ) |>
    dplyr::ungroup()

  data <- data |>
    dplyr::bind_rows(
      big_ns
    ) |>
    dplyr::arrange(
      dplyr::desc(
        Group
      )
    )

  # Define the tfrmt object with two grouping variables and a page_plan
  # note location in subtitle
  tfrmt_two_groups <- tfrmt(
    group = c(`by group`, Group),
    label = Label,
    column = Column,
    value = Value,
    param = Param,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = "n",
        frmt("xx")
      )
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = ".default",
        element_block(
          post_space = " "
        )
      )
    ),
    page_plan = page_plan(
      page_structure(
        group_val = list(
          `by group` = ".default"
        )
      ),
      note_loc = "subtitle"
    ),
    big_n = big_n_structure(
      param_val = "bigN"
    ),
    col_plan = col_plan(
      -`by group`
    )
  )

  expect_no_error(
    gt_tables <- data |>
      apply_tfrmt(
        tfrmt_two_groups,
        mock = FALSE
      ) |>
      cleaned_data_to_gt(
        tfrmt_two_groups,
        .unicode_ws = TRUE
      )
  )

  # confirm note is located in subtitle
  expect_identical(
    gt_tables$gt_tbls$gt_tbl[[1]]$`_heading`$subtitle,
    "by group: 101"
  )

  expect_identical(
    gt_tables$gt_tbls$gt_tbl[[2]]$`_heading`$subtitle,
    "by group: 102"
  )

  # change note location to preheader
  tfrmt_two_groups$page_plan$note_loc <- "preheader"

  expect_no_error(
    gt_tables <- data |>
      apply_tfrmt(
        tfrmt_two_groups,
        mock = FALSE
      ) |>
      cleaned_data_to_gt(
        tfrmt_two_groups,
        .unicode_ws = TRUE
      )
  )

  # confirm note is located in preheader
  expect_identical(
    gt_tables$gt_tbls$gt_tbl[[1]]$`_heading`$preheader,
    "by group: 101"
  )

  expect_identical(
    gt_tables$gt_tbls$gt_tbl[[2]]$`_heading`$preheader,
    "by group: 102"
  )

  # change note location to source_note
  tfrmt_two_groups$page_plan$note_loc <- "source_note"

  expect_no_error(
    gt_tables <- data |>
      apply_tfrmt(
        tfrmt_two_groups,
        mock = FALSE
      ) |>
      cleaned_data_to_gt(
        tfrmt_two_groups,
        .unicode_ws = TRUE
      )
  )

  # confirm note is located in source_note
  expect_identical(
    gt_tables$gt_tbls$gt_tbl[[1]]$`_source_notes`[[1]],
    "by group: 101"
  )

  expect_identical(
    gt_tables$gt_tbls$gt_tbl[[2]]$`_source_notes`[[1]],
    "by group: 102"
  )
})

test_that("cleaned_data_to_gt() with col_style_plan", {
  test_data <- tibble::tribble(
    ~g1   , ~g2  , ~one       , ~param   , ~column , ~value  ,
    "G1"  , "g3" , "n (%)"    , "n"      , "trt1"  , 12      ,
    "G1"  , "g3" , "n (%)"    , "pct"    , "trt1"  , 34      ,
    "G2_" , "g3" , "mean"     , "mean"   , "trt1"  , 12.3    ,
    "G2_" , "g3" , "sd"       , "sd"     , "trt1"  ,  4.34   ,
    "G2_" , "g3" , "median"   , "median" , "trt1"  , 14      ,
    "G3"  , "g3" , "(q1, q3)" , "q1"     , "trt1"  , 10      ,
    "G3"  , "g3" , "(q1, q3)" , "q3"     , "trt1"  , 20      ,
    "G1"  , "g3" , "n (%)"    , "n"      , "trt2"  , 24      ,
    "G1"  , "g3" , "n (%)"    , "pct"    , "trt2"  , 58      ,
    "G2_" , "g3" , "mean"     , "mean"   , "trt2"  , 15.4    ,
    "G2_" , "g3" , "sd"       , "sd"     , "trt2"  ,  8.25   ,
    "G2_" , "g3" , "median"   , "median" , "trt2"  , 16      ,
    "G3"  , "g3" , "(q1, q3)" , "q1"     , "trt2"  , 22      ,
    "G3"  , "g3" , "(q1, q3)" , "q3"     , "trt2"  , 22      ,
    "G1"  , "g3" , "mean"     , "pval"   , "four"  ,  0.0001
  )

  plan <- tfrmt(
    label = one,
    group = c(g1, g2),
    column = vars(column),
    value = value,
    param = param,
    sorting_cols = c(g1, g2),
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("xx.xx")
      ),
      frmt_structure(
        group_val = ".default",
        label_val = "n (%)",
        frmt_combine(
          "{n} ({pct}%)",
          n = frmt("x"),
          pct = frmt("xx.x")
        )
      ),
      frmt_structure(
        group_val = ".default",
        label_val = "(q1, q3)",
        frmt_combine(
          "({q1}, {q3})",
          q1 = frmt("xx"),
          q3 = frmt("xx")
        )
      ),
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        pval = frmt_when(
          "<.001" ~ "<.001",
          TRUE ~ frmt("x.xxx")
        )
      )
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val=list(g1="G1"), element_block(post_space="----")),
      row_grp_structure(group_val=list(g1="G2_"), element_block(post_space="----")),
      label_loc = element_row_grp_loc(
        location = "spanning"
      )
    ),
    col_style_plan = col_style_plan(
      col_style_structure(
        align = "right",
        col = g1
      ),
      col_style_structure(
        align = "right",
        col = one
      ),
      col_style_structure(
        align = "right",
        width = 4,
        col = vars(
          starts_with(
            "trt"
          )
        )
      ),
      col_style_structure(
        align = "left",
        col = trt1
      ),
      col_style_structure(
        width = 10,
        col = four
      )
    )
  )

  # apply_col_style_plan() handles the alignment -> we snapshot the data and
  # not a gt property
  expect_snapshot(
    test_data |>
      apply_tfrmt(
        plan,
        mock = FALSE
      ) |>
      cleaned_data_to_gt(
        plan,
        .unicode_ws = TRUE
      ) |>
      purrr::pluck(
        "_data"
      )
  )
})

test_that("cleaned_data_to_gt() with row_grp_plan and location = 'column'", {
  test_data <- tibble::tribble(
    ~grp1 , ~grp2 , ~lbl , ~prm , ~column , ~val , ~ord ,
    "d"   , "c"   , "n"  , "n"  ,       1 ,    1 ,    1 ,
    "a"   , "b"   , "m"  , "n"  ,       1 ,    2 ,    2 ,
    "q"   , "v"   , "s"  , "n"  ,       1 ,    3 ,    3 ,
    "b"   , "p"   , "e"  , "n"  ,       1 ,    4 ,    4
  )

  tfrmt_plan <- tfrmt(
    group = c(grp1, grp2),
    label = lbl,
    column = column,
    value = val,
    param = prm,
    sorting_cols = ord,
    col_plan = col_plan(-ord),
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("x")
      )
    ),
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(
        location = "column"
      )
    )
  )

  expect_snapshot(
    test_data |>
      apply_tfrmt(
        tfrmt_plan,
        mock = FALSE
      ) |>
      cleaned_data_to_gt(
        tfrmt_plan,
        .unicode_ws = TRUE
      ) |>
      purrr::pluck(
        "_data"
      )
  )
})

#' Clean Column Names and Remove Internal tfrmt Columns
#'
#' Internal helper to replace the internal tlang delimiter with a user-specified string.
#'
#' @param df A data frame.
#' @param delim Character string to replace the internal "tlang_delim".
#' @param stubhead The _stubhead slot from the gt object.
#' @return A data frame with updated column names.
#' @noRd
clean_data <- function(df, delim, boxhead = NULL, stubhead = NULL) {

  # Update Stub/Group Column Names
  # Boxhead tells us which columns are 'stub' columns
  if (!is.null(boxhead) && !is.null(stubhead)) {

    # Identify the variable names that are marked as 'stub'
    stub_vars <- boxhead %>%
      dplyr::filter(.data$type == "stub") %>%
      dplyr::pull(.data$var)

    # Get the new labels from stubhead
    new_stub_labels <- as.character(unlist(stubhead$label))

    if (length(stub_vars) > 0 && length(stub_vars) == length(new_stub_labels)) {

      lookup <- setNames(stub_vars, new_stub_labels)

      #  Filter out entries where the new name is empty or NA
      valid_names <- names(lookup) != "" & !is.na(names(lookup))
      lookup <- lookup[valid_names]

      if (length(lookup) > 0) {
        df <- df %>% dplyr::rename(dplyr::any_of(lookup))
      }
    }
  }



  df %>%
    # Drop internal tfrmt columns (e.g., ..tfrmt_row_grp_lbl)
    dplyr::select(-dplyr::starts_with("..tfrmt")) %>%
    # Replace the internal tlang_delim pattern in column names
    dplyr::rename_with(
      ~ stringr::str_replace_all(.x, "___tlang_delim___", delim),
      .cols = tidyselect::everything()
    )
}


#' Extract underlying data from tfrmt output
#'
#' Following a call to `print_to_gt`, this function extracts the underlying
#' data frame(s) from the resulting `gt` or `gt_group` object.
#'
#' @param x A `gt_tbl` or `gt_group` object (usually the output of `print_to_gt()`).
#' @param col_delim Character string to replace the internal "tlang_delim"
#'   separator in column names only for tables with spanning headers. Defaults to "_".
#' @return If `gt_tbl`, a single data frame. If `gt_group`, a list of data frames.
#' @importFrom purrr map
#' @export
extract_data <- function(x, col_delim = "_") {

  #Fallback
  if (!inherits(x, c("gt_tbl", "gt_group"))) {
    stop("Input must be a 'gt_tbl' or 'gt_group' object.")
  }

  # Single gt table
  if (inherits(x, "gt_tbl")) {
   return(clean_data(
      df = x[["_data"]],
      delim = col_delim,
      boxhead = x[["_boxhead"]],
      stubhead = x[["_stubhead"]]
    ))
  }

  # Grouped gt object (created when using `page_plan`)
  if (inherits(x, "gt_group")) {

    # Extract the internal list of gt_tbl objects
    tbl_list <- x$gt_tbls$gt_tbl

    # Map over the list to pull the '_data' slot and clean names
    extracted_list <- map(tbl_list, ~ clean_data(.x[["_data"]],
                                                 delim = col_delim,
                                                 boxhead = .x[["_boxhead"]],
                                                 stubhead = .x[["_stubhead"]]
    ))

    return(extracted_list)
  }


}

#' Page Plan
#'
#' Defining the location and/or frequency of page splits with a series of
#' page_structure's and the row_every_n argument, respectively.
#'
#' @param ... a series of [page_structure()] separated by commas
#' @param note_loc Location of the note describing each table's subset value(s).
#'   Useful if the `page_structure` contains only ".default" values (meaning the
#'   table is split by every unique level of a grouping variable), and that
#'   variable is dropped in the col_plan. `preheader` only available for rtf output.
#' @param max_rows Option to set a maximum number of rows per page. Takes a
#'   numeric value.
#' @param transform `r lifecycle::badge("experimental")` optional, a function
#'   or formula to transform the page label.
#'
#'   It should take a character vector as input and return a modified character
#'   vector as output.
#'
#'   A **function** is used as is, e.g. `function(x) {stringr::str_replace(x, "grp", "Group")}`
#'
#'   A **formula**, e.g. `~ stringr::str_remove_all(.x, "_")` is converted to a
#'   function. This allows you to create more compact anonymous functions
#'   (lambdas).
#'
#' @return page_plan object
#' @export
#'
#' @examples
#'  # use of page_struct
#'  page_plan(
#'    page_structure(
#'      group_val = "grp1",
#'      label_val = "lbl1"
#'    )
#'  )
#'
#'  # use of #  rows
#'  page_plan(
#'    max_rows = 5
#'  )
#'
#'  # use of transform with a formula
#'  page_plan(
#'    page_structure(group_val = ".default"),
#'    transform = ~ stringr::str_replace(.x, "grp", "Group")
#'  )
#'
#'  # use of transform with a function
#'  transformation_function <- function(x) {
#'    stringr::str_replace(x, "grp", "Group")
#'  }
#'
#'  page_plan(
#'    page_structure(group_val = ".default"),
#'    transform = transformation_function
#'  )
#'
page_plan <- function(
  ...,
  note_loc = c("noprint", "preheader", "subtitle", "source_note"),
  max_rows = NULL,
  transform = NULL
) {
  page_structure_list <- list(...)
  note_loc <- match.arg(note_loc)

  structure(
    list(
      struct_list = page_structure_list,
      note_loc = note_loc,
      max_rows = max_rows,
      transform = transform
    ),
    class = c(
      "page_plan",
      "plan"
    )
  )
}

#' Page structure
#'
#' @param group_val string or a named list of strings which represent the value of group to split after.
#' Set to ".default" if the split should occur after every unique value of the variable.
#' @param label_val string which represents the value of label to split after.
#'   Set to ".default" if the split should occur after every unique value of
#'   the variable.

#'
#' @return page structure object
#' @export
#'
#' @examples
#' # split page after every unique level of the grouping variable
#'  page_structure(group_val = ".default", label_val = NULL)
#'
#'  # split page after specific levels
#'  page_structure(group_val = "grp1", label_val = "lbl3")
page_structure <- function(group_val = NULL, label_val = NULL){


  if(length(group_val)>1 && is.list(group_val)==FALSE && !is.null(names(group_val))){
    group_val <- as.list(group_val)
  }else if(length(group_val)==1 && !is.null(names(group_val))){
    group_val<-as.list(group_val)
  }


  if(is.list(group_val)){
    group_val_names <- names(group_val)
    if(is.null(group_val_names)){
      stop("when group_val is a list, must be a named list")
    }else if(any(group_val_names == "")){
      stop("when group_val is a list, each entry must be named")
    }
  }

  structure(
    list(
      group_val = group_val,
      label_val = label_val
    ),
    class = c(
      "page_structure",
      "structure"
    )
  )
}

#' Shuffle `cards`
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' This function ingests an ARD object of class `card` and shuffles the information to prepare
#' for analysis. Helpful for streamlining across multiple ARDs.
#'
#' @param x an ARD data frame of class 'card'
#' @param by Grouping variable(s) used in calculations. Defaults to `NULL`. If
#'   available (i.e. if `x` comes from a stacking function), `attributes(x)$by`
#'   will be used instead of `by`.
#' @param trim logical representing whether or not to trim away `fmt_fun`,
#'   `error`, and `warning` columns
#' @param order_rows logical representing whether or not to apply
#'   `cards::tidy_ard_row_order()` to sort the rows
#' @param fill_overall scalar to fill missing grouping or variable levels. If a
#'   character is passed, then it is processed with [glue::glue()] where the
#'   colname element is available to inject into the string,
#'   e.g. `Overall {colname}` may resolve to `"Overall AGE"` for an `AGE`
#'   column. Default is `Overall {colname}`. If `NA` then no fill will occur.
#' @param fill_hierarchical_overall scalar to fill variable levels for overall
#'   hierarchical calculations. If a character is passed, then it is processed
#'   with [glue::glue()] where the colname element is available to inject into
#'   the string, e.g. `Any {colname}` may resolve to `"Any AESOC"` for an
#'   `AESOC` column. Default is `Any {colname}`. If `NA` then no fill will
#'   occur.
#'
#' @return a tibble
#' @export
#'
#' @examples
#' \dontrun{
#' cards::bind_ard(
#'   cards::ard_categorical(cards::ADSL, by = "ARM", variables = "AGEGR1"),
#'   cards::ard_categorical(cards::ADSL, variables = "ARM")
#' ) |>
#'   shuffle_card()
#' }
shuffle_card <- function(x,
                         by = NULL,
                         trim = TRUE,
                         order_rows = TRUE,
                         fill_overall = "Overall {colname}",
                         fill_hierarchical_overall = "Any {colname}") {

  rlang::check_installed(
    pkg = "cards",
    version = "0.8.0",
    compare = ">=",
    reason = "for compatibility with tfrmt v0.4.0"
  )

  check_card(x)

  if (!inherits(trim, "logical")) {
    cli::cli_abort(
      "{.arg trim} argument must be class {.cls logical}}, not \\
      {.obj_type_friendly {trim}}"
    )
  }


  # Check if a 'by' variable is available for bind_ard objs
  by_arg <- get_card_attr_arg(x, "by")
  if (is_bind_ard_card(x) && rlang::is_empty(by)) {

    by_msg <- if (!rlang::is_empty(by_arg)) {
      c("*" = "A {.arg by} value of {.val {by_arg}} was found in the input object's attributes.",
        "*" = "To use it as a grouping variable, pass it explicitly: {.code shuffle_card(by = \"{by_arg}\")}.")
    } else {
      c("*" = "If you want to use a grouping variable, pass it explicitly via the {.arg by} argument.")
    }

    cli::cli_inform(
      c(
        "The {.arg by} argument was not supplied and cannot reliably be inferred from objects of class {.cls bind_ard}.",
        by_msg
      )
    )
  }

  x <- drop_bind_ard_args(x)
  ard_args <- attributes(x)$args
  by <- .process_by(x, by)

  # make sure columns are in order & add index for retaining order
  if (isTRUE(order_rows)){
    x <- x |>
      cards::tidy_ard_row_order()
  }
  dat_cards <- x |>
    cards::tidy_ard_column_order() |>
    dplyr::mutate(
      ..cards_idx.. = dplyr::row_number()
    )

  # split up the data into data/variable info & cards info
  vars_ard <- dat_cards |>
    dplyr::select(
      cards::all_ard_groups(),
      cards::all_ard_variables()
    ) |>
    names()

  # process the data/variable info
  dat_cards_grps_processed <- dat_cards |>
    dplyr::mutate(
      stat_variable = .data$variable
    ) |>
    cards::rename_ard_columns(
      fill='..cards_overall..') |>
    cards::unlist_ard_columns(
      columns = c(cards::all_ard_groups(), cards::all_ard_variables()),
      fct_as_chr = TRUE,
    )|>
    dplyr::relocate(
      "stat_variable",
      .after = "context"
    ) |>
    dplyr::select(
      -tidyselect::any_of(
        c("..ard_total_n..", "..ard_hierarchical_overall..")
      )
    )

  dat_cards_out <- dat_cards_grps_processed |>
    # unlist the list-columns
    cards::unlist_ard_columns() |>
    .fill_overall_grp_values(by, fill_overall, fill_hierarchical_overall) |>
    dplyr::arrange(.data$..cards_idx..) |>
    dplyr::select(-"..cards_idx..")

  output <- dat_cards_out

  if (trim) {
    output <- dat_cards_out |>
      .trim_ard()
  }

  # re-attach the args attribute
  attr(output, "args") <- ard_args

  output
}

#' Trim ARD
#'
#' This function ingests an data.frame and trims columns for
#' downstream use
#'
#' @param x (`data.frame`)\cr
#'   a data frame
#'
#' @return a tibble
#' @noRd
.trim_ard <- function(x) {

  # detect any warning/error messages and notify user
  .detect_msgs(x, "warning", "error")
  # flatten ard table for easier viewing
  x |>
    dplyr::select(
      -tidyselect::any_of(
        c(
          "fmt_fun",
          "fmt_fn",
          "warning",
          "error"
        )
      )
    )
}


#' Detect Columns with Non-Null Contents
#'
#' Function looks for non-null contents in requested columns and notifies user
#' before removal. Specifically used for detecting messages.
#'
#' @param x a data frame
#' @param ... columns to search within
#'
#' @noRd
.detect_msgs <- function(x, ...) {
  dots <- rlang::dots_list(...)

  lapply(dots, function(var) {
    if (any(!map_lgl(x[[var]], is.null))) {
      cli::cli_inform(
        "{.val {var}} column contains messages that will be removed."
      )
    }
  })
}


#' Process `by` variable
#'
#' @param x a data frame
#' @param by Grouping variable(s) used in calculations.
#'
#' @returns character string if `by` variable present
#' @noRd
.process_by <- function(x, by) {
  by_arg <- get_card_attr_arg(x, "by")
  
  # 1. If 'by' is explicitly supplied, it takes absolute priority
  if (!is.null(by)) {
    # Only inform if an attribute actually exists AND it doesn't match
    if (!is.null(by_arg) && !identical(by_arg, by)) {
      cli::cli_inform(
        c(
          "Mismatch between attributes of {.arg x} and supplied value to {.arg by}.",
          "i" = "Supplied value will be used in lieu of attributes.",
          "*" = "Note: As of {.pkg tfrmt} v0.4.0, explicitly supplied {.arg by} \\
                 values take priority over data frame attributes."
        ),
        env = rlang::caller_env()
      )
    }
    return(by)
  }

  # 2. If 'by' was NOT supplied, fall back to the attribute
  by_arg
}

#' Fill Overall Group Variables
#'
#' This function fills the missing values of grouping variables with
#' `"Overall <variable_name>"` or `"Any <variable_name>"`where relevant.
#' Specifically, it will modify grouping values from rows with likely overall
#' calculations present (e.g. non-missing variable/variable_level, missing group
#' variables, and evidence that the `variable` has been computed by group in
#' other rows). `"Overall"` values will be populated only for grouping variables
#' that have been used in other calculations of the same variable and statistics.
#' `"Any"` will be used if it is likely to be a hierarchical calculation.
#'
#' @inheritParams shuffle_card
#'
#' @return data frame
#'
#' @noRd
.fill_overall_grp_values <- function(x,
                                     by,
                                     fill_overall,
                                     fill_hierarchical_overall) {

  grp_vars <- by
  vars_cards_protected <- c(
    "context",
    "stat_variable",
    "stat_name",
    "stat_label",
    "stat",
    "fmt_fun",
    "fmt_fn",
    "warning",
    "error",
    "..cards_idx.."
  )

  # determine grouping and merging variables
  id_vars <- setdiff(names(x), unique(c(vars_cards_protected, grp_vars)))

  if (!is_empty(grp_vars) && !is_empty(id_vars)) {

    # replace NA group values with "..cards_overall.." where it is likely to be
    # an overall calculation
    for (g in grp_vars) {

      # rows with missing group
      x_missing_by <- x |>
        dplyr::filter(is.na(.data[[g]]))

      # rows with non-missing group
      x_nonmissing_by <- x |>
        dplyr::filter(!is.na(.data[[g]]) & !.data[[g]] == "..cards_overall..")

      if (nrow(x_missing_by) > 0 && nrow(x_nonmissing_by) > 0) {
        x_missing_by_replaced <- x_missing_by |>
          dplyr::rows_update(
            x_nonmissing_by |>
              dplyr::mutate(
                !!g := dplyr::if_else(
                  !is.na(.data[[g]]),
                  "..cards_overall..",
                  .data[[g]]
                )
              ) |>
              dplyr::select(
                -tidyselect::any_of(
                  c(
                    setdiff(
                      names(x),
                      c(g, id_vars)
                    )
                  )
                )
              ) |>
              dplyr::distinct(),
            by = id_vars,
            unmatched = "ignore"
          )

        x <- dplyr::rows_update(
          x,
          x_missing_by_replaced,
          by = "..cards_idx.."
        )
      }
    }

    # replace NA variables with "..hierarchical_overall.." when present
    x <- x |>
      dplyr::mutate(
        dplyr::across(
          tidyselect::all_of(
            id_vars
          ),
          ~ ifelse(
            is.na(.x) & .data$stat_variable == "..ard_hierarchical_overall..",
            "..hierarchical_overall..",
            .x
          )
        )
      )
  }

  # replace `"..cards_overall.."` and `"..hierarchical_overall.."` with fill
  # values
  output <- x |>
    dplyr::mutate(
      dplyr::across(
        tidyselect::all_of(
          c(
            grp_vars,
            setdiff(
              id_vars,
              "..cards_idx.."
            )
          )
        ),
        ~ .derive_overall_labels(
          .x,
          colname = dplyr::cur_column(),
          fill_overall,
          fill_hierarchical_overall
        )
      )
    )

  output
}

#' Derive overall labels
#'
#' Transform the `"..cards_overall.."` and `"..hierarchical_overall.."` labels
#' into `"Overall <variable_name>"` and `"Any <variable_name>"` respectively.
#' Also it ensures the labels are unique (in case they already exist) with
#' `make.unique()` which appends a sequence number.
#'
#' @param x (character) content of target (current) column
#' @param colname (character) name of current column
#' @inheritParams shuffle_card
#'
#' @returns a character vector
#'
#' @noRd
.derive_overall_labels <- function(x,
                                   colname = dplyr::cur_column(),
                                   fill_overall,
                                   fill_hierarchical_overall) {

  glue_overall <- ifelse(
    is.na(fill_overall),
    NA,
    glue::glue(fill_overall)
  )
  glue_any <- ifelse(
    is.na(fill_hierarchical_overall),
    NA,
    glue::glue(fill_hierarchical_overall)
  )

  overall_val <- c(unique(x), glue_overall) |>
    make.unique() |>
    dplyr::last()
  any_val <- c(unique(x), glue_any) |>
    make.unique() |>
    dplyr::last()

  if (!is.na(glue_overall) && overall_val != glue_overall) {
    cli::cli_alert_info(
      "{.val {glue_overall}} already exists in the {.code {colname}} column. \\
      Using {.val {overall_val}}."
    )
  }

  if (!is.na(glue_any) && any_val != glue_any) {
    cli::cli_alert_info(
      "{.val {glue_any}} already exists in the {.code {colname}} column. Using\\
       {.val {any_val}}."
    )
  }

  output <- dplyr::case_when(
    x == "..cards_overall.." ~ overall_val,
    x == "..hierarchical_overall.." ~ any_val,
    TRUE ~ as.character(x)
  )

  output
}

test_that("shuffle/trim works", {
  # shuffle without group/var levels
  ard_simple <- cards::ard_continuous(cards::ADSL, by = "ARM",variables = "AGE")

  ard_simple_shuffled <- ard_simple |>
    shuffle_card(by = NULL, trim = FALSE) |>
    as.data.frame()

  expect_snapshot(ard_simple_shuffled)

  # shuffle with 1 by var
  ard_grp <- cards::bind_ard(
    cards::ard_categorical(cards::ADSL, variables = "ARM"),
    cards::ard_categorical(cards::ADSL, by = "ARM", variables = "AGEGR1")
  )
  ard_grp_shuffled <- ard_grp |>
    shuffle_card(by = "ARM", trim = FALSE) |>
    dplyr::filter(!stat_name == "N")
  expect_true(all(!is.na(ard_grp_shuffled$ARM)))

  ard_hier <- cards::ard_hierarchical_count(
    data = cards::ADAE,
    variables = c(AESOC, AEDECOD),
    by = TRTA
  )
  ard_hier_shuff <- ard_hier |>
    shuffle_card(trim = FALSE) |>
    as.data.frame()
  expect_true(all(!is.na(ard_hier_shuff$AESOC)))


  # shuffle many different formats
  ard_test <- cards::bind_ard(
    cards::ard_categorical(cards::ADSL, variables = "ARM"),
    cards::ard_continuous(cards::ADSL, by = "ARM", variables = "AGE", stat_label = ~ list(c("mean", "sd") ~ "Mean(SD)")),
    cards::ard_categorical(cards::ADSL, by = "ARM", variables = "AGEGR1"),
    cards::ard_missing(cards::ADSL, by = "ARM", variables = c("AGEGR1", "AGE"))
  )
  ard_shuffled <- ard_test |>
    shuffle_card(by = "ARM") |>
    as.data.frame()

  expect_snapshot(ard_shuffled[1:5, ])

  # shuffle & trim
  ard_shuff_trim <- ard_test |>
    shuffle_card(by = "ARM") |>
    as.data.frame()
  expect_snapshot(ard_shuff_trim[1:5, ])
  # only numeric stats
  expect_type(ard_shuff_trim$stat, "double")
  # no list columns
  expect_true(!any(map_lgl(ard_shuff_trim, is.list)))
})

# test_that("shuffle_ard handles protected names", {
#   ard_test <- cards::ard_categorical(
#     cards::ADSL |> dplyr::rename(stat = ARM),
#     by = "stat",
#     variables = "AGEGR1"
#   ) |>
#     shuffle_card()
#
#   expect_equal(names(ard_test)[1], "stat.1")
# })

test_that("shuffle_card notifies user about warnings/errors before dropping", {
  expect_snapshot(
    cards::ard_continuous(
      cards::ADSL,
      variables = AGEGR1
    ) |>
      shuffle_card()
  )
})

test_that("shuffle_card message if bind_ard is used without a supplied by argument", {
  # Create a combined ARD
  combined_ard <- cards::bind_ard(
    cards::ard_continuous(cards::ADSL, by = "ARM", variables = "AGE", statistic = ~ cards::continuous_summary_fns("mean")),
    dplyr::tibble(group1 = "ARM", variable = "AGE", stat_name = "p", stat_label = "p", stat = list(0.05))
  )

  # Verify that the function throws the exact expected message
  expect_message(
    shuffle_card(combined_ard),
    regexp = "The `by` argument was not supplied"
  )
})

test_that("shuffle_card correctly handles a combined ARD when by is explicitly supplied", {
  # mix of missing/nonmissing group levels present before shuffle
  expect_snapshot(
    cards::bind_ard(
      cards::ard_continuous(cards::ADSL, by = "ARM", variables = "AGE", statistic = ~ cards::continuous_summary_fns("mean")),
      dplyr::tibble(group1 = "ARM", variable = "AGE", stat_name = "p", stat_label = "p", stat = list(0.05))
    ) |>
      dplyr::filter(dplyr::row_number() <= 5L) |>
      shuffle_card()
  )

  # no group levels present before shuffle
  expect_snapshot(
    cards::bind_ard(
      cards::ard_continuous(cards::ADSL, variables = "AGE", statistic = ~ cards::continuous_summary_fns("mean")),
      dplyr::tibble(group1 = "ARM", variable = "AGE", stat_name = "p", stat_label = "p", stat = list(0.05))
    ) |>
      dplyr::filter(dplyr::row_number() <= 5L) |>
      shuffle_card(by="ARM")
  )

  # mix of group variables - fills overall only if variable has been calculated by group elsewhere
  expect_snapshot(
    cards::bind_ard(
      cards::ard_categorical(cards::ADSL, by = ARM, variables = AGEGR1) |> dplyr::slice(1),
      cards::ard_categorical(cards::ADSL, variables = AGEGR1) |> dplyr::slice(1),
      cards::ard_continuous(cards::ADSL, by = SEX, variables = AGE) |> dplyr::slice(1),
      cards::ard_continuous(cards::ADSL, variables = AGE) |> dplyr::slice(1)
    ) |>
      shuffle_card(by = c("ARM","SEX")) |>
      as.data.frame()
  )
  # custom fill
  expect_snapshot(
    cards::bind_ard(
      cards::ard_categorical(cards::ADSL, by = ARM, variables = AGEGR1) |> dplyr::slice(1),
      cards::ard_categorical(cards::ADSL, variables = AGEGR1) |> dplyr::slice(1),
      cards::ard_continuous(cards::ADSL, by = SEX, variables = AGE) |> dplyr::slice(1),
      cards::ard_continuous(cards::ADSL, variables = AGE) |> dplyr::slice(1)
    ) |>
      shuffle_card(by = c("ARM","SEX"), fill_overall = "{colname}") |>
      as.data.frame()
  )

  # mix of hierarchical group variables - fills overall only if variable has been calculated by group elsewhere
  expect_snapshot(
    cards::bind_ard(
      cards::ard_categorical(cards::ADSL, by = c(ARM, SEX), variables = AGEGR1) |> dplyr::slice(1),
      cards::ard_categorical(cards::ADSL, by = SEX, variables = AGEGR1) |> dplyr::slice(1),
      cards::ard_categorical(cards::ADSL, variables = AGEGR1) |> dplyr::slice(1)
    ) |>
      shuffle_card(by = c("ARM","SEX"))
  )
  # custom fill
  expect_snapshot(
    cards::bind_ard(
      cards::ard_categorical(cards::ADSL, by = c(ARM, SEX), variables = AGEGR1) |> dplyr::slice(1),
      cards::ard_categorical(cards::ADSL, by = SEX, variables = AGEGR1) |> dplyr::slice(1),
      cards::ard_categorical(cards::ADSL, variables = AGEGR1) |> dplyr::slice(1)
    ) |>
      shuffle_card(by = c("ARM","SEX"), fill_overall = "total")
  )
  expect_snapshot(
    cards::bind_ard(
      cards::ard_categorical(cards::ADSL, by = c(ARM, SEX), variables = AGEGR1) |> dplyr::slice(1),
      cards::ard_categorical(cards::ADSL, by = SEX, variables = AGEGR1) |> dplyr::slice(1),
      cards::ard_categorical(cards::ADSL, variables = AGEGR1) |> dplyr::slice(1)
    ) |>
      shuffle_card(by = c("ARM","SEX"), fill_overall = NA)
  )

  # fills with a unique group value if one already exists in the df
  adsl_new <- cards::ADSL |>
    dplyr::mutate(ARM = ifelse(ARM == "Placebo", "Overall ARM", ARM))
  expect_snapshot(
    cards::bind_ard(
      cards::ard_continuous(adsl_new, variables = "AGE", statistic = ~ cards::continuous_summary_fns("mean")),
      cards::ard_continuous(adsl_new, by = "ARM", variables = "AGE", statistic = ~ cards::continuous_summary_fns("mean"))
    ) |>
      shuffle_card(by = "ARM")
  )
})

test_that("shuffle_card doesn't trim off NULL/NA values", {
  # mix of char NA, NULL values
  res <- suppressMessages(
    data.frame(x = rep_len(NA, 10)) |>
      cards::ard_continuous(
        variables = x,
        statistic = ~ cards::continuous_summary_fns(c("median", "p25", "p75"))
      ) |>
      shuffle_card() |>
      dplyr::pull(stat)
  )

  # check that all rows present
  expect_length(res, 3)
})

test_that("shuffle_card coerces all factor groups/variables to character", {
  adsl_ <- cards::ADSL |>
    dplyr::mutate(RACE = factor(RACE))

  res <- cards::ard_categorical(
    data = adsl_,
    by = TRT01A,
    variables = c(RACE, ETHNIC)
  ) |>
    shuffle_card()

  res_classes <- res |>
    dplyr::select(-stat) |>
    sapply(class)

  # all are character
  expect_true(all(res_classes == "character"))

  # correct coersion
  expect_equal(
    sort(unique(res$RACE)),
    sort(unique(as.character(adsl_$RACE)))
  )
  expect_equal(
    sort(unique(res$ETHNIC)),
    sort(unique(as.character(adsl_$ETHNIC)))
  )
})

test_that("shuffle_card fills missing group levels if the group is meaningful for cardx output", {
  # cardx ARD: this is a dput() of a cardx result (see commented out code below) SAVED 2024-08-30
  ard_cardx <- structure(
    list(
      group1 = c("ARM", "ARM", "SEX", "SEX"),
      variable = c(
        "AGEGR1",
        "AGEGR1", "AGEGR1", "AGEGR1"
      ),
      context = c(
        "stats_chisq_test",
        "stats_chisq_test", "stats_chisq_test", "stats_chisq_test"
      ),
      stat_name = c("statistic", "p.value", "statistic", "p.value"),
      stat_label = c(
        "X-squared Statistic", "p-value", "X-squared Statistic",
        "p-value"
      ),
      stat = list(
        statistic = c(`X-squared` = 5.07944166638125),
        p.value = 0.0788884197453486,
        statistic = c(`X-squared` = 1.03944199945198),
        p.value = 0.594686442507218
      ),
      fmt_fun = list(
        statistic = 1L,
        p.value = 1L, statistic = 1L, p.value = 1L
      ),
      warning = list(
        warning = NULL, warning = NULL, warning = NULL, warning = NULL
      ),
      error = list(error = NULL, error = NULL, error = NULL, error = NULL)
    ),
    row.names = c(
      NA,
      -4L
    ),
    class = c("card", "tbl_df", "tbl", "data.frame")
  )

  expect_snapshot(
    ard_cardx |>
      shuffle_card() |>
      as.data.frame()
  )
})

test_that("shuffle_card() fills grouping columns with `Overall <var>` or `Any <var>`", {
  adae <- cards::ADAE |>
    dplyr::filter(
      SAFFL == "Y",
      TRTA %in% c("Placebo", "Xanomeline High Dose"),
      AESOC %in% unique(AESOC)[1:2]
    ) |>
    dplyr::group_by(AESOC) |>
    dplyr::filter(
      AETERM %in% unique(AETERM)[1:2]
    ) |>
    dplyr::ungroup()

  adsl <- cards::ADSL |>
    dplyr::filter(
      SAFFL == "Y",
      TRTA %in% c("Placebo", "Xanomeline High Dose")
    )

  ard_hier <- cards::ard_stack_hierarchical(
    data = adae,
    by = TRTA,
    variables = c(AESOC, AETERM),
    denominator = adsl,
    id = USUBJID,
    overall = TRUE,
    over_variables = TRUE,
    total_n = TRUE
  )

  shuffled_ard_hier <- ard_hier |>
    shuffle_card()

  expect_identical(
    shuffled_ard_hier |>
      dplyr::filter(
        context == "total_n"
      ) |>
      dplyr::pull(TRTA),
    "Overall TRTA"
  )

  expect_identical(
    shuffled_ard_hier |>
      dplyr::filter(stat_variable == "..ard_hierarchical_overall..") |>
      dplyr::pull(AESOC) |>
      unique(),
    "Any AESOC"
  )
  expect_identical(
    shuffled_ard_hier |>
      dplyr::filter(stat_variable == "..ard_hierarchical_overall..") |>
      dplyr::pull(AETERM) |>
      unique(),
    "Any AETERM"
  )

  # custom fill
  shuffled_ard_hier <- ard_hier |>
    shuffle_card(fill_overall = "overall {colname} observed",
                 fill_hierarchical_overall = "any {colname} observed")

  expect_identical(
    shuffled_ard_hier |>
      dplyr::filter(
        context == "total_n"
      ) |>
      dplyr::pull(TRTA),
    "overall TRTA observed"
  )
  expect_identical(
    shuffled_ard_hier |>
      dplyr::filter(stat_variable == "..ard_hierarchical_overall..") |>
      dplyr::pull(AESOC) |>
      unique(),
    "any AESOC observed"
  )
  expect_identical(
    shuffled_ard_hier |>
      dplyr::filter(stat_variable == "..ard_hierarchical_overall..") |>
      dplyr::pull(AETERM) |>
      unique(),
    "any AETERM observed"
  )
})

test_that("shuffle_card() fills with multiple `by` columns", {
  adae <- cards::ADAE |>
    dplyr::filter(
      SAFFL == "Y",
      TRTA %in% c("Placebo", "Xanomeline High Dose"),
      AESOC %in% unique(AESOC)[1:2]
    ) |>
    dplyr::group_by(AESOC) |>
    dplyr::filter(
      AETERM %in% unique(AETERM)[1:2]
    ) |>
    dplyr::ungroup()

  adsl <- cards::ADSL |>
    dplyr::filter(
      SAFFL == "Y",
      TRTA %in% c("Placebo", "Xanomeline High Dose")
    )

  ard <- cards::ard_stack_hierarchical(
    data = adae,
    by = c(TRTA, SEX),
    variables = c(AESOC, AETERM),
    denominator = adsl,
    id = USUBJID,
    overall = TRUE,
    over_variables = TRUE,
    total_n = TRUE
  )

  shuffled_ard <- ard |>
    shuffle_card()

  expect_identical(
    shuffled_ard |>
      dplyr::filter(
        context == "total_n"
      ) |>
      dplyr::select(
        TRTA,
        AESOC,
        SEX
      ),
    data.frame(
      TRTA = "Overall TRTA",
      AESOC = NA_character_,
      SEX = "Overall SEX"
    ),
    # the shuffled_ard preserves the card attributes and returns a tibble. We
    # need to ignore the attributes for the purpose of this comparison
    ignore_attr = TRUE
  )

  # expect_identical(
  #   shuffled_ard |>
  #     dplyr::filter(
  #       variable == "..ard_hierarchical_overall.."
  #     ) |>
  #     dplyr::pull(AESOC) |>
  #     unique(),
  #   "Any AESOC"
  # )


})

test_that("shuffle_card() messages about 'Overall <var>' or 'Any <var>'", {
  test_data <- dplyr::tibble(
    ARM = c("..cards_overall..", "Overall ARM", NA, "BB", NA),
    TRTA = c(NA, NA, "..hierarchical_overall..", "C", "C")
  )

  # messaging actually comes from .derive_overall_labels
  expect_snapshot(
    test_data |>
      dplyr::mutate(
        dplyr::across(
          ARM:TRTA,
          ~ .derive_overall_labels(
            .x,
            fill_overall = "Overall {colname}",
            fill_hierarchical_overall = "Any {colname}"
          )
        )
      )
  )

  adae <- cards::ADAE |>
    dplyr::filter(
      SAFFL == "Y",
      TRTA %in% c("Placebo", "Xanomeline High Dose"),
      AESOC %in% unique(AESOC)[1:2]
    ) |>
    dplyr::group_by(AESOC) |>
    dplyr::filter(
      AETERM %in% unique(AETERM)[1:2]
    ) |>
    dplyr::ungroup() |>
    dplyr::mutate(
      TRTA = dplyr::if_else(
        TRTA == "Xanomeline High Dose",
        "Overall TRTA",
        TRTA
      ),
      AESOC = dplyr::if_else(
        AESOC == "GASTROINTESTINAL DISORDERS",
        "Any AESOC",
        AESOC
      )
    )

  adsl <- cards::ADSL |>
    dplyr::filter(
      SAFFL == "Y",
      TRTA %in% c("Placebo", "Xanomeline High Dose")
    ) |>
    dplyr::mutate(
      TRTA = dplyr::if_else(
        TRTA == "Xanomeline High Dose",
        "Overall TRTA",
        TRTA
      )
    )

  ard <- cards::ard_stack_hierarchical(
    data = adae,
    by = c(TRTA, SEX),
    variables = c(AESOC, AETERM),
    denominator = adsl,
    id = USUBJID,
    overall = TRUE,
    over_variables = TRUE,
    total_n = TRUE
  )

  expect_snapshot(
    shuffled_ard <- ard |>
      shuffle_card()
  )

  expect_identical(
    shuffled_ard |>
      dplyr::filter(
        context == "total_n"
      ) |>
      dplyr::select(
        TRTA,
        AESOC,
        SEX
      ),
    data.frame(
      TRTA = "Overall TRTA.1",
      AESOC = NA_character_,
      SEX = "Overall SEX"
    ),
    # the shuffled_ard preserves the card attributes and returns a tibble. We
    # need to ignore the attributes for the purpose of this comparison
    ignore_attr = TRUE
  )

  # expect_identical(
  #   shuffled_ard |>
  #     dplyr::filter(
  #       variable == "..ard_hierarchical_overall.."
  #     ) |>
  #     dplyr::pull(AESOC) |>
  #     unique(),
  #   "Any AESOC.1"
  # )

})


test_that("shuffle_card() preserves the attributes of a `card` object", {
  adae <- cards::ADAE |>
    dplyr::filter(
      SAFFL == "Y",
      TRTA %in% c("Placebo", "Xanomeline High Dose"),
      AESOC %in% unique(AESOC)[1:2]
    ) |>
    dplyr::group_by(AESOC) |>
    dplyr::filter(
      AETERM %in% unique(AETERM)[1:2]
    ) |>
    dplyr::ungroup()

  adsl <- cards::ADSL |>
    dplyr::filter(
      SAFFL == "Y",
      TRTA %in% c("Placebo", "Xanomeline High Dose")
    )

  ard <- cards::ard_stack_hierarchical(
    data = adae,
    by = TRTA,
    variables = c(AESOC, AETERM),
    denominator = adsl,
    id = USUBJID,
    overall = TRUE,
    over_variables = TRUE,
    total_n = TRUE
  )

  shuffled_ard <- shuffle_card(ard)

  expect_identical(
    attributes(ard)[["args"]],
    attributes(shuffled_ard)[["args"]]
  )

  # ard_stack uses bind_ard under the hood - attributes will be lost
  ard <- cards::ard_stack(
    data = adsl,
    cards::ard_tabulate(variables = "AGEGR1"),
    cards::ard_summary(variables = "AGE"),
    .by = "ARM"
  )

  expect_snapshot(
    shuffled_ard <- shuffle_card(ard)
  )

  # Binding two ARDs together class without a by attribute
  ard_no_by_attr <- cards::bind_ard(
    cards::ard_tabulate(cards::ADSL, variables = "AGEGR1"),
    cards::ard_tabulate(cards::ADSL, variables = "SEX")
  )

  expect_snapshot(
    shuffle_card(ard_no_by_attr)
  )

})


test_that("shuffle_card() sorting option", {
  withr::local_seed(0829)

  ard <- cards::ard_continuous(cards::ADSL, by = "ARM", variables = "AGE")
  # randomly sort data
  ard_mixed <- dplyr::slice(ard, sample.int(nrow(ard)))

  ard_unnest <- ard_mixed |>
    cards::rename_ard_columns(fill = "Overall {colname}") |>
    cards::unlist_ard_columns() |>
    dplyr::select(
      -tidyselect::any_of(
        c(
          "fmt_fn",
          "fmt_fun",
          "warning",
          "error"
        )
      )
    )

  expect_equal(
    ard_unnest,
    shuffle_card(ard_mixed, order_rows = FALSE) |> dplyr::select(-stat_variable),
    ignore_attr = TRUE
  )
})

test_that("shuffle_card() prioritizes supplied `by` and messages on mismatch", {
  # 1. Mismatch: Create an ARD by "SEX", but intentionally spoof the attribute to "ARM"
  ard_mismatch <- cards::ard_continuous(cards::ADSL, by = "SEX", variables = "AGE")
  attr(ard_mismatch, "args")$by <- "ARM"

  # Capture the raw console text sent to the message/stderr stream
  msg_output <- capture.output(
    res_mismatch <- shuffle_card(ard_mismatch, by = "SEX"),
    type = "message"
  )
  msg_string <- paste(msg_output, collapse = "\n")

  # Programmatically verify the exact elements of your message text
  expect_match(msg_string, "Mismatch between attributes of")
  expect_match(msg_string, "Supplied value will be used in lieu of attributes")
  expect_match(msg_string, "As of tfrmt v0.4.0")

  # Verify "SEX" was used as the column name instead of "ARM"
  expect_true("SEX" %in% names(res_mismatch))
  expect_false("ARM" %in% names(res_mismatch))

  # 2. Match: Standard card where supplied matches attribute
  ard_match <- cards::ard_continuous(cards::ADSL, by = "ARM", variables = "AGE")
  expect_silent(
    res_match <- shuffle_card(ard_match, by = "ARM")
  )
  expect_true("ARM" %in% names(res_match))

  # 3. NULL supplied: Should remain silent and fall back to the attribute ("ARM")
  expect_silent(
    res_null <- shuffle_card(ard_match, by = NULL)
  )
  expect_true("ARM" %in% names(res_null))
})

globalVariables(c(".", ".data", ".rename_col", ".original_col","df_names","new_name_in_df",
                  "param_list", "label_quote", "label_collapse"))


## Shared variables
.tlang_delim <- "___tlang_delim___"
.tlang_struct_col_prefix <- "__tlang_span_structure_column__"

#' Apply the formatting to all values in the dataset
#'
#' @param .data data
#' @param table_frmt_plan styling element needed
#' @param group symbolic list of grouping
#' @param label symbolic label
#' @param param symbolic parameter
#' @param value symbolic value
#' @param mock Logical value is this is for a mock or not
#'
#' @noRd
#' @importFrom dplyr tibble mutate group_by arrange slice bind_cols group_split pull select starts_with
#' @importFrom purrr map map_dfr
#' @importFrom tidyr unnest
#' @importFrom rlang !! :=
apply_table_frmt_plan <- function(.data, table_frmt_plan, group, label, param, value, column, mock = FALSE,...){

  ## identify which formatting needs to be applied where
  .data <- .data %>%
    ungroup() %>%
    mutate(TEMP_row = row_number())

  TEMP_appl_row <- table_frmt_plan %>%
    map(fmt_test_data, .data, label, group, param)

  TEMP_fmt_to_apply <- table_frmt_plan %>% map(~.$frmt_to_apply[[1]])

  dat_plus_fmt <- tibble(
    TEMP_appl_row,
    TEMP_fmt_to_apply) %>%
    # TODO? add a warning if a format isn't applied anywhere?
    mutate(TEMP_fmt_rank = row_number()) %>%
    unnest(cols = c(TEMP_appl_row)) %>%
    group_by(TEMP_appl_row) %>%
    #TODO add warning if there are rows not covered
    arrange(TEMP_appl_row, desc(.data$TEMP_fmt_rank)) %>%
    slice(1) %>%
    left_join(.data, ., by= c("TEMP_row" = "TEMP_appl_row")) %>%
    group_by(.data$TEMP_fmt_rank) %>%
    group_split()

  ## apply formatting
  dat_plus_fmt %>%
    map_dfr(function(x){

      cur_fmt <- x %>%
        pull(.data$TEMP_fmt_to_apply) %>%
        .[1] %>%
        .[[1]]

      if(is.null(cur_fmt)){
        if(!mock){
          out <- x %>%
            mutate(!!value := as.character(!!value))
        } else {
          out <- x
        }

        # Add message
        x %>%
          pull(.data$TEMP_row) %>%
          paste0(collapse = ", ") %>%
          paste("The following rows of the given dataset have no format applied to them", .) %>%
          message()
      }else{

        ## apply the formatting based on method of cur_fmt
        out <- apply_frmt(
          frmt_def = cur_fmt,
          .data = x,
          value = value,
          param = param,
          column = column,
          label = label,
          group = group,
          mock = mock
        )
      }

      out
    }) %>%
    arrange(.data$TEMP_row) %>%
    select(
      # drop TEMP_row values
      -tidyselect::starts_with(
        "TEMP_"
      )
    )

}

#' Test of the frmt of the data
#'
#' @param cur_fmt current formatting
#' @param data data to test against NOTE: `TEMP_row` must be in the dataset
#' @param label label symbol should only be one
#' @param group list of the group parameters
#' @param param param symbol should only be one
#'
#' @return vector of the rows which this format could be applied to
#'
#' @importFrom stringr str_remove_all
#' @noRd
fmt_test_data <- function(cur_fmt, .data, label, group, param){

  #get filters for each column type
  grp_expr <- expr_to_filter(group, cur_fmt$group_val)
  lbl_expr <- expr_to_filter(label, cur_fmt$label_val)
  parm_expr <- expr_to_filter(param, cur_fmt$param_val)



  filter_expr <- paste(
      c(lbl_expr,grp_expr,parm_expr),
      collapse = "&"
    ) %>%
    parse_expr()

  out <- .data %>%
    filter(!!filter_expr)

  # Protect against incomplete frmt_combines
  if(is_frmt_combine(cur_fmt$frmt_to_apply[[1]])){
    complet_combo_grps <- out %>%
      select(!!!group, !!label, !!param) %>%
      distinct() %>%
      group_by(!!!group, !!label) %>%
      mutate(test = sum(!!parse_expr(parm_expr))) %>%
      filter(.data$test == length(cur_fmt$frmt_to_apply[[1]]$frmt_ls)) %>%
      ungroup()
    join_by <- c(group, label, param) %>%
      map_chr(as_label) %>%
      keep(~. != "<empty>")

    out <- complet_combo_grps %>%
      left_join(out, by = join_by, multiple = "all")
  }
  out %>%
    pull(.data$TEMP_row)

}


#' @importFrom rlang parse_expr eval_bare
all_missing <- function(cols, .data){
  paste0("is.na(.data$",cols,")", collapse = " & ") %>%
    parse_expr() %>%
    eval_bare(env = environment())
}


#' Replace values
#'
#' based on dplyr replace_with function
#' @param x Current vector
#' @param i vector of TRUE/FALSE if should be replaced
#' @param val New value tos replace with
#'
#' @noRd
replace_val <- function(x, i, val) {
  if (is.null(val)) {
    return(x)
  }

  i[is.na(i)] <- FALSE

  if (length(val) == 1L) {
    x[i] <- val
  } else {
    x[i] <- val[i]
  }

  x
}

#' Format Structure Object
#'
#' Function needed to create a frmt_structure object, which is a building block
#' of [body_plan()]. This specifies the rows the format will be applied to.
#'
#' @seealso [body_plan()] combines the frmt_structures to be applied to the
#'   table body, and [frmt()], [frmt_combine()], and [frmt_when()] define the
#'   format semantics.
#'
#'   \href{https://gsk-biostatistics.github.io/tfrmt/articles/body_plan.html}{Link to related article}
#'
#' @param group_val A string or a named list of strings which represent the
#'   value of group should be when the given frmt is implemented
#' @param label_val A string which represent the value of label should be when
#'   the given frmt is implemented
#' @param ... either a [frmt()], [frmt_combine()], or a [frmt_when()] object.
#'   This can be named to also specify the parameter value
#'
#' @section Images:
#' Here are some example outputs:
#' \if{html}{\out{
#' `r "<img src=\"https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/main/images/tfrmt-frmts.jpg\" alt = \"Example comparing fmt, frmt_combine, and frmt_when\" style=\"width:100\\%;\">"`
#' }}
#'
#' @examples
#'
#' sample_structure <- frmt_structure(
#'           group_val = c("group1"),
#'           label_val = ".default",
#'           frmt("XXX")
#'         )
#' ## multiple group columns
#' sample_structure <- frmt_structure(
#'           group_val = list(grp_col1 = "group1", grp_col2 = "subgroup3"),
#'           label_val = ".default",
#'           frmt("XXX")
#'         )
#'
#' @returns frmt_structure object
#' @export
frmt_structure <- function(group_val = ".default", label_val = ".default", ...){
  param_frmt <- list(...)
  param_val <- names(param_frmt)

  if(length(param_frmt) > 1){
    stop("Can only handle one format per frmt_structure function. Use frmt_combine if a combination is needed")
  }

  if(is_frmt_combine(param_frmt[[1]])){
    param_val <- names(param_frmt[[1]]$frmt_ls)
  } else if(is.null(param_val)){
    param_val <- ".default"
  }

  if(!is_frmt(param_frmt[[1]])){
    stop(paste0("Entry is not an object of class `frmt`"))
  }


  if(is.list(group_val)){
    group_val_names <- names(group_val)
    if(is.null(group_val_names)){
      stop("when group_val is a list, must be a named list")
    }else if(any(group_val_names == "")){
      stop("when group_val is a list, each entry must be named")
    }
  }

  structure(
    list(
      group_val = group_val,
      label_val = label_val,
      param_val = param_val,
      frmt_to_apply = param_frmt),
    class = c("frmt_structure","frmt_table")
  )
}



#' Table Value Formatting
#'
#' @description
#' These functions provide an abstracted way to approach to define formatting of table
#' contents. By defining in this way, the formats can be
#' layered to be more specific and general cell styling can be done first.
#'
#' `frmt()` is the base definition of a format. This defines spacing, rounding,
#' and missing behaviour.
#'
#' `frmt_combine()` is used when two or more rows need to be combined into a
#' single cell in the table. Each of the rows needs to have a defined `frmt()`
#' and need to share a label.
#'
#' `frmt_when()` is used when a rows format behaviour is dependent on the value itself and is written similarly to [dplyr::case_when()].
#'  The left hand side of the equation is a `"TRUE"`for the default case or the right hand side of a boolean expression `">50"`.
#'
#' @seealso [body_plan()] combines the frmt_structures to be applied to the
#'   table body, and [frmt_structure()] defines which rows the formats will be applied
#'   to.
#'
#'   \href{https://gsk-biostatistics.github.io/tfrmt/articles/body_plan.html}{Link to related article}
#'
#' @param expression this is the string representing the intended format. See details: expression for more a detailed description.
#' @param missing when a value is missing that is intended to be formatted, what value to place. See details: missing for more a detailed description.
#' @param scientific a string representing the intended scientific notation to be appended to the expression. Ex. "e^XX" or " x10^XX".
#' @param transform this is what should happen to the value prior to formatting,
#'   It should be a formula or function. Ex. `~.*100`if you want to convert a
#'   percent from a decimal prior to rounding
#' @param ...  See details: `...` for a detailed description.
#'
#'
#' @details
#'
#' ## expression
#'  - `frmt()` All numbers are represented by "x". Any additional character are
#'  printed as-is. If additional X's present to the left of the decimal point
#'  than the value, they will be represented as spaces.
#'  - `frmt_combine()` defines how the parameters will be combined as a
#'  `glue::glue()` statement. Parameters need to be equal to the values in the
#'  param column and defined in the expression as \code{"{param1} {param2}"}.
#'
#' ## missing
#'  - `frmt()` Value to enter when the value is missing. When NULL, the value
#'  is "".
#'  - `frmt_combine()` defines how when all values to be combined are missing.
#'  When NULL the value is "".
#'
#' ## ...
#'    - `frmt()` These dots are for future extensions and must be
#'    empty.
#'    - `frmt_combine()` accepts named arguments defining the `frmt()` to
#'    be applied to which parameters before being combined.
#'    - `frmt_when()`accepts a series of equations separated by commas, similar
#'    to [dplyr::case_when()]. The left hand side of the equation is a `"TRUE"`for the
#'    default case or the right hand side of a boolean expression `">50"`. The
#'    right hand side of the equation is the `frmt()` to apply when the left
#'    side evaluates to `TRUE`.
#'
#' @export
#' @return frmt object
#' @examples
#'
#' frmt("XXX %")
#'
#' frmt("XX.XXX")
#'
#' frmt("xx.xx", scientific = "x10^xx")
#'
#' frmt_combine(
#'  "{param1} {param2}",
#'  param1 = frmt("XXX %"),
#'  param2 = frmt("XX.XXX")
#' )
#'
#' frmt_when(
#'   ">3" ~ frmt("(X.X%)"),
#'   "<=3" ~ frmt("Undetectable")
#'   )
#'
#' frmt_when(
#'   "==100"~ frmt(""),
#'   "==0"~ "",
#'   "TRUE" ~ frmt("(XXX.X%)")
#'   )
#'
#' @rdname frmt
#'
frmt <- function(expression, missing = NULL, scientific = NULL, transform = NULL, ...){
  structure(
    list(expression = expression, missing = missing, scientific = scientific,
         transform = transform),
    class = c("frmt")
  )
}


#' @export
#' @rdname frmt
#' @importFrom stringr str_extract_all str_count str_replace str_detect
frmt_combine <- function(expression, ..., missing = NULL){

  everything_but_curly <- "(?<=\\{)([^}]*)(?=\\})"

  n_vars <- str_count(expression, everything_but_curly)
  vars_to_fmt <- str_extract_all(expression, everything_but_curly, simplify = TRUE)
  vars_to_fmt <- as.vector(vars_to_fmt)
  frmt_ls <- list(...)

  if(n_vars != length(frmt_ls) & length(frmt_ls) > 1){
    stop("The number of formats must be 1 or match the number of parameters", call. = FALSE)
  } else if (n_vars > 1 & length(frmt_ls) == 1){
    frmt_ls <- frmt_ls[rep(1,n_vars)]
  }

  names(frmt_ls) <- vars_to_fmt

  # Adding ` to expression if not there and there is a space/symbol
  replace_val <-case_when(
    str_detect(vars_to_fmt, "^[a-zA-Z0-9_.]*$") ~ vars_to_fmt,
    !str_detect(vars_to_fmt, "^[a-zA-Z0-9_.]*$") & !str_detect(vars_to_fmt, "`") ~ paste0("`", vars_to_fmt, "`"),
    TRUE ~ vars_to_fmt)

  exp_new <- expression
  for(i in seq_along(replace_val)){
    exp_new <- str_replace(exp_new, vars_to_fmt[i], replace_val[i])
  }

  structure(
    list(expression = exp_new, frmt_ls = frmt_ls, missing = missing),
    class = c("frmt_combine","frmt")
  )
}


#' @rdname frmt
#' @export
#' @importFrom rlang list2 f_rhs f_rhs<-
#' @importFrom purrr map
frmt_when <- function(..., missing = NULL){
  frmts <- list2(...)

  frmt_ls <- frmts %>%
    map(function(x){
      f_rhs(x) <- eval(f_rhs(x))
      x
    })

  structure(
    list(frmt_ls = frmt_ls, missing = missing),
    class = c("frmt_when","frmt")
  )
}

test_that("col_style_structure - quo/vars/bare", {

  element_l <- col_style_structure(align = "left", col = n_tot)
  element_r <- col_style_structure(align = "right", col = vars(p))
  element_c <- col_style_structure(align = ".", col = c(trt1, trt2))
  element_ts <- col_style_structure(align = ".", col = starts_with("hello"))

  expect_equal(element_l$cols, vars(n_tot), ignore_attr = TRUE)
  expect_equal(element_r$cols, vars(p), ignore_attr = TRUE)
  expect_equal(element_c$cols, vars(trt1, trt2), ignore_attr = TRUE)
  expect_equal(element_ts$cols, vars(starts_with("hello")), ignore_attr = TRUE)

})

test_that("col_style_structure - char", {

  element_l <- col_style_structure(align = "left", col = "n_tot")
  element_r <- col_style_structure(align = "right", col = "p")
  element_c <- col_style_structure(align = ".", col = c("trt1", "trt2"))

  expect_equal(element_l$cols, vars(n_tot), ignore_attr = TRUE)
  expect_equal(element_r$cols, vars(p), ignore_attr = TRUE)
  expect_equal(element_c$cols, vars(trt1, trt2), ignore_attr = TRUE)

})

test_that("col_style_structure - errors", {

  expect_error(
    col_style_structure(col = "n_tot"),
    "`align` or `width` must be applied to create this col_style_structure",
    fixed = TRUE
  )

  expect_error(
    col_style_structure(col = "n_tot", width = "INVALID"),
    "`width` must be a value that can be converted into a number greater than 0",
    fixed = TRUE
  )

  expect_error(
    col_style_structure(col = "n_tot", width = "-12345"),
    "`width` must be a valid number greater than 0",
    fixed = TRUE
  )

})

test_that("col_style_structure - advanced", {

  element_1 <- col_style_structure(align = "left", col = c(n_tot, p, test))
  element_2 <- col_style_structure(align = "left", col = c(n_tot, p, "test"))
  element_3 <- col_style_structure(align = "right", col = span_structure(col = test, col2 = value))
  element_4 <- col_style_structure(align = ".", col = c(trt1, span_structure(col = test, col2 = value)))

  expect_equal(element_1$cols, list(quo(n_tot),quo(p),quo(test)), ignore_attr = TRUE)
  expect_equal(element_2$cols, list(quo(n_tot),quo(p),quo(test)), ignore_attr = TRUE)
  expect_equal(element_3$cols, list(list(col = vars(test), col1 = vars(value))), ignore_attr = TRUE)
  expect_equal(element_4$cols, list(quo(trt1),list(col = vars(test), col1 = vars(value))), ignore_attr = TRUE)

})

test_that("col_style_plan - basic", {

  csp <- col_style_plan(
    col_style_structure(align = "left", col = "n_tot"),
    col_style_structure(align = "right", col = "p"),
    col_style_structure(align = ".", col = c("trt1", "trt2"))
  )

  expect_equal(length(csp), 3)
  expect_equal(lapply(csp, `[[`, "cols") ,
               list(vars(n_tot), vars(p), vars(trt1, trt2)),
               ignore_attr = TRUE)

})

test_that("col_style_plan - error non-element_col", {

  expect_error({
    col_style_plan(
      element_block(post_space = " "),
      col_style_structure(align = "right", col = "p"),
      col_style_structure(align = ".", col = c("trt1", "trt2"))
    )
  },
  "Entry number 1 is not an object of class `col_style_structure`.",
  fixed = TRUE
  )

})

test_that("left & right align works", {

  vec <- c(" xx.xxx","xx", " x,  x")

  expect_equal(apply_col_alignment(vec, align = "left"),
               c(" xx.xxx","xx     ", " x,  x "))

  expect_equal(apply_col_alignment(vec, align = "right"),
               c(" xx.xxx","     xx", "  x,  x"))

})

test_that("decimal align works", {

  vec <- c(" xx.xx", " x, x", "xxx", " x (x.x)")

  expect_equal(apply_col_alignment(vec, align = c("."," ")),
               c(" xx.xx   ", " x, x    ", "xxx      ", "  x (x.x)"))

  expect_equal(apply_col_alignment(vec, align = c(".", ",", " ")),
               c(" xx.xx   ", "  x, x   ", "xxx      ", "  x (x.x)"))
})

test_that("alignment of multiple columns works", {

  dat <- tibble::tribble(
    ~ one, ~ column, ~ value,
    "n (%)",    "two"    ," 12 (34%)",
    "n (%)",    "three"  ," 24 (58%)",
    "n (%)",    "four"   ,""         ,
    "mean",     "two"    ," 12.3"    ,
    "mean",     "three"  ," 15.4"    ,
    "mean",     "four"   ,"<0.001"   ,
    "sd",       "two"    ,"  4.34"   ,
    "sd",       "three"  ,"  8.25"   ,
    "sd",       "four"   ,""         ,
    "median",   "two"    ," 14"      ,
    "median",   "three"  ," 16"      ,
    "median",   "four"   ,"0.05"     ,
    "(q1, q3)", "two"    ,"(10, 20)" ,
    "(q1, q3)", "three"  ,"(11, 22)" ,
    "(q1, q3)", "four"   ,""   )

  tfrmt_obj <- tfrmt(
    label = one,
    column = vars(column),
    value = value,
    col_style_plan = col_style_plan(
      col_style_structure(align = "left", col = vars(one)),
      col_style_structure(align = "right", col = vars(four)),
      col_style_structure(align = c(".", ",", " "), col = vars(two, three))
    )
  )

  dat_aligned_man <- tibble::tribble(
    ~ one, ~ column, ~ value,
    "n (%)   ","two"    ," 12 (34%)",
    "mean    ","two"    ," 12.3    ",
    "sd      ","two"    ,"  4.34   ",
    "median  ","two"    ," 14      ",
    "(q1, q3)","two"    ,"(10, 20) ",
    "n (%)   ","three"  ," 24 (58%)",
    "mean    ","three"  ," 15.4    ",
    "sd      ","three"  ,"  8.25   ",
    "median  ","three"  ," 16      ",
    "(q1, q3)","three"  ,"(11, 22) ",
    "n (%)   ","four"   ,"      ",
    "mean    ","four"   ,"<0.001",
    "sd      ","four"   ,"      ",
    "median  ","four"   ,"  0.05",
    "(q1, q3)","four"   ,"      "
  )

  dat_aligned <- dat %>%
    pivot_wider(
      names_from = column,
      values_from = value
      ) %>%
    apply_col_style_plan(tfrmt_obj)

  expect_equal(dat_aligned,
               dat_aligned_man %>%
                 pivot_wider(
                   names_from = column,
                   values_from = value
                 ) )


  plan <- tfrmt(
    label = one,
    column = vars(column),
    value = value,
    col_style_plan = col_style_plan(
      col_style_structure(align = "left", col = vars(one)),
      col_style_structure(align = "right", col = vars(two, three, four))
    )
  )

  dat_aligned_man <- tibble::tribble(
    ~ one, ~ column, ~ value,
    "n (%)   "  ,"two"    ," 12 (34%)",
    "mean    "  ,"two"    ,"     12.3",
    "sd      "  ,"two"    ,"     4.34",
    "median  "  ,"two"    ,"       14",
    "(q1, q3)"  ,"two"    ," (10, 20)",
    "n (%)   "  ,"three"  ," 24 (58%)",
    "mean    "  ,"three"  ,"     15.4",
    "sd      "  ,"three"  ,"     8.25",
    "median  "  ,"three"  ,"       16",
    "(q1, q3)"  ,"three"  ," (11, 22)",
    "n (%)   "  ,"four"   ,"      "   ,
    "mean    "  ,"four"   ,"<0.001"   ,
    "sd      "  ,"four"   ,"      "   ,
    "median  "  ,"four"   ,"  0.05"   ,
    "(q1, q3)"  ,"four"   ,"      "
  )

  dat_aligned <- dat  %>%
    pivot_wider(
      names_from = column,
      values_from = value
    ) %>%
    apply_col_style_plan( plan)

  expect_equal(dat_aligned,
               dat_aligned_man %>%
                 pivot_wider(
                   names_from = column,
                   values_from = value
                 ))


})


test_that("tidyselect works", {

  dat <- tibble::tribble(
    ~one      , ~column , ~ value,
    "n (%)"    ,"trt1" ," 12 (34%)",
    "n (%)"    ,"trt2" ," 24 (58%)",
    "n (%)"    ,"four" , ""         ,
    "mean"     ,"trt1" ," 12.3"    ,
    "mean"     ,"trt2" , " 15.4"    ,
    "mean"     ,"four" ,"<0.001"   ,
    "sd"       ,"trt1" , "  4.34"   ,
    "sd"       ,"trt2" ,"  8.25"   ,
    "sd"       ,"four" , ""         ,
    "median"   ,"trt1" ," 14"      ,
    "median"   ,"trt2" , " 16"      ,
    "median"   ,"four" ,"0.05"     ,
    "(q1, q3)" ,"trt1" , "(10, 20)" ,
    "(q1, q3)" ,"trt2" ,"(11, 22)" ,
    "(q1, q3)" ,"four" , ""         )


  starts_with_align_dot_plan <- tfrmt(
    label = one,
    column = vars(column),
    value = value,
    col_style_plan = col_style_plan(
      col_style_structure(align = c(".", ",", " "), col = vars(starts_with("trt"))),
      col_style_structure(align = "right", col = vars(four)))

  )

  dat_aligned_man <- tibble(
    one = c("n (%)", "mean", "sd", "median", "(q1, q3)"),
    trt1 = c(" 12 (34%)", " 12.3    ", "  4.34   ", " 14      ", "(10, 20) "),
    trt2 = c(" 24 (58%)", " 15.4    ", "  8.25   ", " 16      ", "(11, 22) "),
    four = c("      ", "<0.001", "      ", "  0.05", "      ")
    )

  dat_aligned <- dat %>%
    pivot_wider(
      names_from = column,
      values_from = value
      ) %>%
    apply_col_style_plan( starts_with_align_dot_plan)

  expect_equal(dat_aligned, dat_aligned_man)

  vars_starts_with_plan <- tfrmt(
    label = one,
    column = vars(column),
    value = value,
    col_style_plan = col_style_plan(
      col_style_structure(align = "right", col = vars(starts_with("trt")))
    ))

  dat_aligned_man <- tibble(
    one = c("n (%)", "mean", "sd", "median", "(q1, q3)"),
    trt1 = c(" 12 (34%)", "     12.3", "     4.34", "       14", " (10, 20)"),
    trt2 = c(" 24 (58%)", "     15.4", "     8.25", "       16", " (11, 22)"),
    four = c("", "<0.001", "", "0.05", "")
  )

  dat_aligned <-  dat %>%
    pivot_wider(
      names_from = column,
      values_from = value
    ) %>%
    apply_col_style_plan( vars_starts_with_plan)

  expect_equal(dat_aligned, dat_aligned_man)

  starts_with_plan <- tfrmt(
    label = one,
    column = vars(column),
    value = value,
    col_style_plan = col_style_plan(
      col_style_structure(align = "right", col = starts_with("trt"))
    ))

  dat_aligned <-  dat %>%
    pivot_wider(
      names_from = column,
      values_from = value
    ) %>%
    apply_col_style_plan( starts_with_plan)

  expect_equal(dat_aligned, dat_aligned_man)

  plan_everything <- tfrmt(
    label = one,
    column = vars(column),
    value = value,
    col_style_plan = col_style_plan(
      col_style_structure(align = "right", col = everything())
    ))

  dat_aligned_man <- tibble(
    one = c("   n (%)", "    mean", "      sd", "  median", "(q1, q3)"),
    trt1 = c(" 12 (34%)", "     12.3", "     4.34", "       14", " (10, 20)"),
    trt2 = c(" 24 (58%)", "     15.4", "     8.25", "       16", " (11, 22)"),
    four = c("      ", "<0.001", "      ", "  0.05", "      ")
  )

  dat_aligned <-  dat %>%
    pivot_wider(
      names_from = column,
      values_from = value
    ) %>%
    apply_col_style_plan( plan_everything, col_plan_vars = vars(one, trt1, trt2, four))

  expect_equal(dat_aligned, dat_aligned_man)

})

test_that("span_structure works", {

  dat <- tibble::tribble(
    ~one       ,  ~ span_col, ~column ,  ~ value,
    "n (%)"    , "Test Span1",  "trt1" , " 12 (34%)",
    "n (%)"    , "Test Span2",  "trt2" , " 24 (58%)",
    "n (%)"    ,          NA,  "four" , ""         ,
    "mean"     , "Test Span1",  "trt1" , " 12.3"    ,
    "mean"     , "Test Span2",  "trt2" , " 15.4"    ,
    "mean"     ,          NA,  "four" , "<0.001"   ,
    "sd"       , "Test Span1",  "trt1" , "  4.34"   ,
    "sd"       , "Test Span2",  "trt2" , "  8.25"   ,
    "sd"       ,          NA,  "four" , ""         ,
    "median"   , "Test Span1",  "trt1" , " 14"      ,
    "median"   , "Test Span2",  "trt2" , " 16"      ,
    "median"   ,          NA,  "four" , "0.05"     ,
    "(q1, q3)" , "Test Span1",  "trt1" , "(10, 20)" ,
    "(q1, q3)" , "Test Span2",  "trt2" , "(11, 22)" ,
    "(q1, q3)" ,          NA,  "four" , ""         )


  plan <- tfrmt(
    label = one,
    column = vars(span_col, column),
    value = value,
    col_style_plan = col_style_plan(
      col_style_structure(align = c(".", ",", " "), col = span_structure(span_col = "Test Span1")),
      col_style_structure(align = "right", col = vars(four)))
  )

  dat_aligned_man <- tibble(
    one = c("n (%)", "mean", "sd", "median", "(q1, q3)"),
    `Test Span1___tlang_delim___trt1` = c(" 12 (34%)", " 12.3    ", "  4.34   ", " 14      ", "(10, 20) "),
    `Test Span2___tlang_delim___trt2` = c(" 24 (58%)", " 15.4", "  8.25", " 16", "(11, 22)"),
    four = c("      ", "<0.001", "      ", "  0.05", "      ")
  )

  dat_aligned <- dat %>%
    pivot_wider(
      names_from = c(span_col,column),
      names_sep = .tlang_delim,
      values_from = value,
    ) %>%
    clean_spanning_col_names() %>%
    apply_col_style_plan( plan)

  expect_equal(dat_aligned, dat_aligned_man)



})

test_that("span_structure works on a renamed column", {

  dat <- tibble::tribble(
    ~one       ,   ~ span_col,  ~column,  ~ value,    ~param,
    "n (%)"    , "Test Span1",   "trt1", " 12 (34%)", "param",
    "n (%)"    , "Test Span2",   "trt2", " 24 (58%)", "param",
    "n (%)"    ,           NA,   "four", ""         , "param",
    "mean"     , "Test Span1",   "trt1", " 12.3"    , "param",
    "mean"     , "Test Span2",   "trt2", " 15.4"    , "param",
    "mean"     ,           NA,   "four", "<0.001"   , "param",
    "sd"       , "Test Span1",   "trt1", "  4.34"   , "param",
    "sd"       , "Test Span2",   "trt2", "  8.25"   , "param",
    "sd"       ,           NA,   "four", ""         , "param",
    "median"   , "Test Span1",   "trt1", " 14"      , "param",
    "median"   , "Test Span2",   "trt2", " 16"      , "param",
    "median"   ,           NA,   "four", "0.05"     , "param",
    "(q1, q3)" , "Test Span1",   "trt1", "(10, 20)" , "param",
    "(q1, q3)" , "Test Span2",   "trt2", "(11, 22)" , "param",
    "(q1, q3)" ,           NA,   "four", "",  "param"
    )


  plan <- tfrmt(
    label = one,
    column = vars(span_col, column),
    value = value,
    param = param,
    col_plan = col_plan(
      one,
      span_structure(
        span_col = `Test Span1`,
        column = trt1
      ),
      span_structure(
        span_col = c("New Test Span2" = "Test Span2"),
        column = trt2
      ),
      four
    ),
    col_style_plan = col_style_plan(
      col_style_structure(align = c(".", ",", " "), col = span_structure(span_col = "Test Span2")),
      col_style_structure(align = "right", col = vars(four))
    )
  )

  dat_aligned_man <- tibble(
    one = c("n (%)", "mean", "sd", "median", "(q1, q3)"),
    `Test Span1___tlang_delim___trt1` = c(" 12 (34%)", " 12.3", "  4.34", " 14", "(10, 20)"),
    `New Test Span2___tlang_delim___trt2` = c(" 24 (58%)", " 15.4    ", "  8.25   ", " 16      ", "(11, 22) "),
    four = c("      ", "<0.001", "      ", "  0.05", "      ")
  )

  suppressMessages({
  dat_aligned <- apply_tfrmt(dat, plan, mock = FALSE) %>%
    tibble::as_tibble()
  })

  expect_equal(dat_aligned, dat_aligned_man, ignore_attr = TRUE)

})

test_that("Overlapping col_style_structure favors last one",{

  dat <- tibble::tribble(
    ~one      , ~column , ~ value,
    "n (%)"    ,"trt1"  ," 12 (34%)",
    "mean"     ,"trt1"  ," 12.3    ",
    "sd"       ,"trt1"  ,"  4.34   ",
    "median"   ,"trt1"  ," 14      ",
    "(q1, q3)" ,"trt1"  ,"(10, 20) ",
    "n (%)"    ,"trt2"  ," 24 (58%)",
    "mean"     ,"trt2"  ," 15.4    ",
    "sd"       ,"trt2"  ,"  8.25   ",
    "median"   ,"trt2"  ," 16      ",
    "(q1, q3)" ,"trt2"  ,"(11, 22) ",
    "n (%)"    ,"four"  ,""         ,
    "mean"     ,"four"  ,"<0.001"   ,
    "sd"       ,"four"  ,""         ,
    "median"   ,"four"  ,"0.05"   ,
    "(q1, q3)" ,"four"  ,""  )

  plan <- tfrmt(
    label = one,
    column = vars(column),
    value = value,
    col_style_plan =  col_style_plan(
      col_style_structure(align = "right", col = vars(starts_with("trt"))),
      col_style_structure(align = c(".",","," "), col = trt1)))

  dat_aligned_man <- tibble::tribble(~one      , ~column , ~value ,
                             "n (%)" ,"trt1"   ," 12 (34%)",
                             "mean" ,"trt1"   ," 12.3    ",
                             "sd" ,"trt1"   ,"  4.34   ",
                             "median" ,"trt1"   ," 14      ",
                             "(q1, q3)" ,"trt1"   ,"(10, 20) ",
                             "n (%)" ,"trt2"   ," 24 (58%)",
                             "mean" ,"trt2"   ,"     15.4",
                             "sd" ,"trt2"   ,"     8.25",
                             "median" ,"trt2"   ,"       16",
                             "(q1, q3)" ,"trt2"   ," (11, 22)",
                             "n (%)" ,"four"   ,""         ,
                             "mean" ,"four"   ,"<0.001"   ,
                             "sd" ,"four"   ,""         ,
                             "median" ,"four"   ,"0.05"     ,
                             "(q1, q3)" ,"four"   ,""         )

  dat_aligned <- dat %>%
    pivot_wider(
      names_from = column,
      values_from = value
    ) %>%
    apply_col_style_plan(plan)

  expect_equal(dat_aligned, dat_aligned_man%>%
                 pivot_wider(
                   names_from = column,
                   values_from = value
                 ) )
})




test_that("Align strings >1 in length",{

  dat <- tibble::tribble(
    ~one      , ~column , ~ value,
    "n (%)"    ,"trt1"  ," 12 (34%)",
    "mean"     ,"trt1"  ," 12.3    ",
    "sd"       ,"trt1"  ,"  4.34   ",
    "median"   ,"trt1"  ," 14      ",
    "(q1, q3)" ,"trt1"  ,"(10, 20) ",
    "n (%)"    ,"trt2"  ," 24 (58%)",
    "mean"     ,"trt2"  ," 15.4    ",
    "sd"       ,"trt2"  ,"  8.25   ",
    "median"   ,"trt2"  ," 16      ",
    "(q1, q3)" ,"trt2"  ,"(11, 22) ",
    "n (%)"    ,"four"  ,""         ,
    "mean"     ,"four"  ,"<0.001"   ,
    "sd"       ,"four"  ,""         ,
    "median"   ,"four"  ,"0.05"   ,
    "(q1, q3)" ,"four"  ,""  )

  col_style_structure_message <- capture_messages({
   plan <- tfrmt(
    label = one,
    column = vars(column),
    value = value,
    col_style_plan =  col_style_plan(
      col_style_structure(align = "right", col = vars(starts_with("trt"))),
      col_style_structure(align = c("...",",,,,"," "), col = trt1)))
  })

  expect_true(
    !is_empty(col_style_structure_message)
  )

  dat_aligned_man <- tibble::tribble(~one      , ~column , ~value ,
                             "n (%)"    ,"trt1"   ," 12 (34%)",
                             "mean"     ,"trt1"   ," 12.3    ",
                             "sd"       ,"trt1"   ,"  4.34   ",
                             "median"   ,"trt1"   ," 14      ",
                             "(q1, q3)" ,"trt1"   ,"(10, 20) ",
                             "n (%)"    ,"trt2"   ," 24 (58%)",
                             "mean"     ,"trt2"   ,"     15.4",
                             "sd"       ,"trt2"   ,"     8.25",
                             "median"   ,"trt2"   ,"       16",
                             "(q1, q3)" ,"trt2"   ," (11, 22)",
                             "n (%)"    ,"four"   ,""         ,
                             "mean"     ,"four"   ,"<0.001"   ,
                             "sd"       ,"four"   ,""         ,
                             "median"   ,"four"   ,"0.05"     ,
                             "(q1, q3)" ,"four"   ,""         )

  # informs user
  dat_aligned <- dat %>%
    pivot_wider(
      names_from = column,
      values_from = value
    ) %>%
    apply_col_style_plan(plan)

  expect_equal(dat_aligned, dat_aligned_man%>%
                 pivot_wider(
                   names_from = column,
                   values_from = value
                 ))

})



test_that("Alphanumeric align string supplied",{

  dat <- tibble::tribble(
    ~one      , ~column , ~ value,
    "n (%)"    ,"trt1"  ," 12 (34%)",
    "mean"     ,"trt1"  ," 12.3    ",
    "sd"       ,"trt1"  ,"  4.34   ",
    "median"   ,"trt1"  ," 14      ",
    "(q1, q3)" ,"trt1"  ,"(10, 20) ",
    "n (%)"    ,"trt2"  ," 24 (58%)",
    "mean"     ,"trt2"  ," 15.4    ",
    "sd"       ,"trt2"  ,"  8.25   ",
    "median"   ,"trt2"  ," 16      ",
    "(q1, q3)" ,"trt2"  ,"(11, 22) ",
    "n (%)"    ,"four"  ,""         ,
    "mean"     ,"four"  ,"<0.001"   ,
    "sd"       ,"four"  ,""         ,
    "median"   ,"four"  ,"0.05"   ,
    "(q1, q3)" ,"four"  ,""  )

  col_style_structure_message <- capture_messages({

  plan <- tfrmt(
    label = one,
    column = vars(column),
    value = value,
    col_style_plan =  col_style_plan(
      col_style_structure(align = "right", col = vars(starts_with("trt"))),
      col_style_structure(align = c("2","4"), col = trt1)))

  })

  expect_true(
    !is_empty(col_style_structure_message)
  )

  dat_aligned_man <- tibble::tribble(~one      , ~column , ~value ,
                             "n (%)"    ,"trt1"   ,"    12 (34%)",
                             "mean"     ,"trt1"   ,"    12.3    ",
                             "sd"       ,"trt1"   ,"     4.34   ",
                             "median"   ,"trt1"   ,"    14      ",
                             "(q1, q3)" ,"trt1"   ,"(10, 20)    ",
                             "n (%)"    ,"trt2"   ," 24 (58%)",
                             "mean"     ,"trt2"   ,"     15.4",
                             "sd"       ,"trt2"   ,"     8.25",
                             "median"   ,"trt2"   ,"       16",
                             "(q1, q3)" ,"trt2"   ," (11, 22)",
                             "n (%)"    ,"four"   ,""         ,
                             "mean"     ,"four"   ,"<0.001"   ,
                             "sd"       ,"four"   ,""         ,
                             "median"   ,"four"   ,"0.05"     ,
                             "(q1, q3)" ,"four"   ,""         )

  # informs user
  dat_aligned <- dat %>%
    pivot_wider(
      names_from = column,
      values_from = value
    ) %>%
    apply_col_style_plan(plan)

  expect_equal(dat_aligned, dat_aligned_man %>%
                 pivot_wider(
                   names_from = column,
                   values_from = value
                 ))
})


test_that("multi-positional alignment", {

  # 1 position align
  vec <- c(" xx.x (xx.x)", " xx, xx", "xx (xx - xx)", " xx (xx %)")
  vec_aligned <- apply_col_alignment(vec,
                                     align = c(" xx.x |(xx.x)",
                                               " xx, |xx",
                                               "xx |(xx - xx)",
                                               " xx |(xx %)"),
                                     type = "pos")

  expect_equal(vec_aligned,
               c("xx.x (xx.x)   ",
                 " xx, xx       ",
                 "  xx (xx - xx)",
                 "  xx (xx %)   "))

  # 3 position align
  vec <- c(" xx.x", " x, x", "xx (xx - xx)", " x (x.x, x.x)")
  vec_aligned <- apply_col_alignment(vec,
                                     align = c(" xx|.x",
                                               " x|, |x",
                                               "xx| |(xx - |xx)",
                                               " x| |(x.x, |x.x)"),
                                     type = "pos")

  expect_equal(vec_aligned,
               c("xx.x          ",
                 " x, x         ",
                 "xx  (xx - xx) ",
                 " x  (x.x, x.x)"))

  # positional alignment not covering all cases - will treat all not covered as: "xxx|" (last char to be aligned with first position)
  vec <- c(" xx.x", " x, x", "xx (xx - xx)", " x (x.x, x.x)")
  vec_aligned <- suppressMessages({
    apply_col_alignment(vec,
                        align = c(" xx|.x",
                                  " x|, |x",
                                  "xx| |(xx - |xx)"),
                        type = "pos")
  })
  expect_equal(vec_aligned,
               c("          xx.x         ",
                 "           x, x        ",
                 "          xx  (xx - xx)",
                 "x (x.x, x.x)           ")
  )

  # align on 2 positions - full plan
  dat <- tibble::tribble(
    ~ lbl, ~ col, ~ val,
    "18-65"	, "trt", "15 (23.4 %)",
    "18-65" , "placebo", "5 (3.8 %)",
    "66-82" , "trt",	"34 (33.5 %)",
    "66-82" , "placebo",	"48 (82.2%)"
  )

  tfrmt_obj <- tfrmt(
    label = "lbl",
    column = "col",
    value = "val",
    col_style_plan = col_style_plan(
      col_style_structure(align = c("xx| (xx|.x %)",
                                    "xx| (xx|.x%)",
                                    "x| (x|.x %)"),
                          type = "pos",
                          col = vars(trt, placebo))
    )
  )

  dat_aligned <- dat %>%
    pivot_wider(
      names_from = col,
      values_from = val
    ) %>%
    apply_col_style_plan(tfrmt_obj)

  dat_aligned_man <- tibble::tribble(
    ~ lbl    , ~trt    , ~placebo,
    "18-65", "15 (23.4 %)", " 5  (3.8 %)",
    "66-82", "34 (33.5 %)", "48 (82.2%) ",
  )

  expect_equal(dat_aligned, dat_aligned_man)

  # align on 3 positions - full plan

  dat <- tibble::tribble(
    ~ one, ~ column, ~ value,
    "lbl1", "two"    ," 12 (34%)",
    "lbl1", "three"  ," 24 (58%)",
    "lbl2", "two"    ," 12.3 (2.3 - 15.3)",
    "lbl2", "three"  ," 15.4 (3.4 - 17.6)",
    "lbl3", "two"    ,"  4.34"   ,
    "lbl3", "three"  ,"  8.25"   ,
    "lbl4", "two"    ,"(10, 20)" ,
    "lbl4", "three"  ,"(11, 22)" )

  tfrmt_obj <- tfrmt(
    label = one,
    column = vars(column),
    value = value,
    col_style_plan = col_style_plan(
      col_style_structure(align = c(" xx| |(xx%)",
                                    " xx|.x |(x.x - |xx.x)",
                                    "||(xx, |xx)",
                                    "x|.xx"),
                          type = "pos",
                          col = vars(two, three))
    )
  )

  dat_aligned <- dat %>%
    pivot_wider(
      names_from = column,
      values_from = value
    ) %>%
    apply_col_style_plan(tfrmt_obj)

  dat_aligned_man <- tibble::tribble(
    ~one    , ~two               ,~three           ,
    "lbl1",  "12   (34%)       ", "24   (58%)       ",
    "lbl2",  "12.3 (2.3 - 15.3)", "15.4 (3.4 - 17.6)",
    "lbl3",  " 4.34            ", " 8.25            ",
    "lbl4",  "     (10,   20)  ", "     (11,   22)  "
  )

  expect_equal(dat_aligned, dat_aligned_man)


})

test_that("multi-positional alignment detects inadequate inputs", {

  dat <- tibble::tribble(
    ~ lbl, ~ col, ~ val,
    "lbl1", "two"    ," 12 (34%)",
    "lbl1", "three"  ," 24 (58%)",
    "lbl2", "two"    ," 12.3 (2.3)",
    "lbl2", "three"  ," 15.4 (5.4)")

  tfrmt_obj <- tfrmt(
    label = lbl,
    column = vars(col),
    value = val,
    col_style_plan = col_style_plan(
      col_style_structure(align = c(" xx |(xx)"),
                          type = "pos",
                          col = vars(two, three))
    )
  )
  dat_wide <- dat %>%
    pivot_wider(
      names_from = col,
      values_from = val
    )

  msgs <- capture_messages(apply_col_style_plan(dat_wide, tfrmt_obj))

  expect_equal(msgs, c("`align` input for `type`=\"pos\" in col_style_structure does not cover all possible values. Some cells may not be aligned.\n",
                       "`align` input for `type`=\"pos\" in col_style_structure does not cover all possible values. Some cells may not be aligned.\n"))


  dat <- tibble::tribble(
    ~ lbl, ~ col, ~ val,
    "lbl1", "two"    ," 12(34%)",
    "lbl1", "three"  ," 24(58%)",
    "lbl2", "two"    ," 12.3(12.3%)",
    "lbl2", "three"  ," 15.4(15.4%)")

  tfrmt_obj <- tfrmt(
    label = lbl,
    column = vars(col),
    value = val,
    col_style_plan = col_style_plan(
      col_style_structure(align = c(" xx|(xx|%)",
                                    " xx|.x(xx|.x%)"),
                          type = "pos",
                          col = vars(two, three))
    )
  )
  dat_wide <- dat %>%
    pivot_wider(
      names_from = col,
      values_from = val
    )

  msgs <- capture_messages(apply_col_style_plan(dat_wide, tfrmt_obj))

  expect_equal(msgs, c("Unable to complete positional alignment in col_style_structure due to lack of whitespace available formatted value\n",
                       "Unable to complete positional alignment in col_style_structure due to lack of whitespace available formatted value\n"))
})


test_that("helper for constructing positional alignment works",{

  dat <- tibble::tribble(
    ~ lbl, ~ col, ~param, ~ val,
    "lbl1", "two"    ,"n",  12,
    "lbl1", "two"    ,"pct", 33.9999,
    "lbl1", "three"  ,"n",  24,
    "lbl1", "three"  ,"pct", 58.222,
    "lbl2", "two"    ,"n",  12,
    "lbl2", "two"    ,"pct", 12.4353,
    "lbl2", "three"  ,"n",  15,
    "lbl2", "three"  ,"pct", 15.354)

  tfrmt_obj <- tfrmt(
    label = lbl,
    column = vars(col),
    param = param,
    value = val,
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = "lbl1",
                     frmt_combine("{n} ({pct}%)",
                                  n = frmt("x"),
                                  pct = frmt("xx"))),
      frmt_structure(group_val = ".default", label_val = "lbl2",
                     frmt_combine("{n} ({pct}%)",
                                  n = frmt("xxx"),
                                  pct = frmt("xx.x")))
    )
  )

  expect_equal(
    display_val_frmts(tfrmt_obj, .data = dat, col = vars(everything())),
    "c(\"xx (xx%)\",
  \" xx (xx.x%)\")"
  )
})

test_that("applying footnote plan",{

  # set up data
  es_data <-tibble::tibble(rowlbl1 =c(rep("Completion Status",12),rep("Primary reason for withdrawal",28)),
                           rowlbl2 =c(rep("Completed",4),rep("Adverse Event",4),rep("Unknown",4),rep("Adverse Event",4),rep("Lost to follow-up",4),rep("Protocol violation",4),rep("Subject decided to withdraw",4),rep("Protocol Violation",4),rep("Pre-Operative Dose[1]",4),rep("Other",4)),
                           param=c(rep(c("n","n","pct","pct"),10)),
                           trt=c(rep(c("Placebo","Treatment"),20)),
                           value=c(24,19,2400/48,1900/38,5,1,500/48,100/38,19,18,1900/48,1800/38,1,1,100/48,100/38,0,0,0,0,0,0,0,0,1,1,100/48,100/38,1,4,100/48,400/38,1,0,100/48,0,2,3,200/48,300/38)

  )


  gt_start <-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = trt,
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "spanning")),
  ) %>%
    print_to_gt(es_data)


  # no footnote plan ###########################################################

  tfrmt1<- tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = trt,
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")),
  )

  gt1<- apply_footnote_plan(gt_start,tfrmt1,list())

  expect_equal(nrow(gt1$`_footnotes`),0)

  # source footnote #################################################################

  tfrmt2<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = trt,
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Source Note"),
      marks="letters"

    )
  )

  gt2<- apply_footnote_plan(gt_start,tfrmt2,list(list(col=NULL,spanning=FALSE,note="Source Note")))

  expect_equal(gt2$`_footnotes`$footnotes,list("Source Note"))

  # column footnote ##################################################################

  tfrmt3<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = trt,
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    # row_grp_plan = row_grp_plan(
    #   row_grp_structure(group_val = ".default", element_block(post_space = " ")),
    #   label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 2",column_val ="Placebo"),
      marks="letters"

    )
  )

  gt3<- apply_footnote_plan(gt_start,tfrmt3,list(list(col="Placebo",spanning=FALSE,note="Test foontote 2")))
  expect_equal(nrow(gt3$`_footnotes`),1)


  # spanner footnote ##################################################################

  es_data2<- es_data %>%
    mutate(col2 = "Treatment column")

  gt_start2 <- tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column =c(col2,trt),
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")),
  ) %>%
    print_to_gt(es_data2)

  tfrmt4<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = c(col2,trt),
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 3",column_val=list(col2 = "Treatment column")),
      marks="numbers"

    )
  )

  gt4<- apply_footnote_plan(gt_start2,tfrmt4,list(list(col="Treatment column",spanning=TRUE,note="Test foontote 3")))
  expect_equal(nrow(gt4$`_footnotes`),1)

  # body footnote ##################################################################


  tfrmt5<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = trt,
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    # row_grp_plan = row_grp_plan(
    #   row_grp_structure(group_val = ".default", element_block(post_space = " ")),
    #   label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 2",
                         column_val ="Placebo",
                         label_val = "Adverse Event",
                         group_val = "Completion Status"),
      marks="letters"

    )
  )

  gt5<- apply_footnote_plan(gt_start,tfrmt5,list(list(col="Placebo",spanning=TRUE,row=1,note="Test foontote 2")))
  expect_equal(nrow(gt5$`_footnotes`),1)


  # stub footnote ##################################################################
  tfrmt6<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = c(trt),
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote",label_val ="Adverse Event"),
      marks="letters"

    )
  )

  gt6<- apply_footnote_plan(gt_start,tfrmt6,list(list(col="rowlbl2",spanning=FALSE,row=c(1,2),note="Test foontote")))
  expect_equal(nrow(gt6$`_footnotes`),2)


  # group footnote ##################################################################

  tfrmt7<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = c(trt),
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "spanning")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote",group_val = "Completion Status"),
      marks="letters"

    )
  )

  gt7<- apply_footnote_plan(gt_start,tfrmt7,list(list(col="rowlbl1",spanning=FALSE,row=1,note="Test foontote")))
  expect_equal(nrow(gt7$`_footnotes`),1)
})



test_that("Check footnote order option works as expected", {

  #default marks_first
  tfrmt_ord <- tfrmt(
    label = label,
    column = column,
    param = param,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{count} {percent}",
          count = frmt("xxx"),
          percent = frmt_when(
            "==100" ~ frmt(""),
            "==0" ~ "",
            "TRUE" ~ frmt("(xx.x%)")
          )
        )
      )
    ),
    footnote_plan = footnote_plan(
      footnote_structure("BMI: Body Mass Index"),
      footnote_structure("Placebo drug + standard of care", column_val = "column2"),
      footnote_structure("Source Note"),
      footnote_structure("label_1 footnote",  label_val="label_1"),
      marks = "numbers"
    )
  )

  gt_out <- print_mock_gt(tfrmt_ord)

  current_order_opt <- gt_out[["_options"]][["value"]][[1]]
  expect_equal(current_order_opt, "marks_first")

  #check the footnotes are all printed
  actual_fns <- gt_out$`_footnotes`$footnotes

  expected_fns <- c(
    "BMI: Body Mass Index",
    "Placebo drug + standard of care",
    "Source Note",
    "label_1 footnote"
  )

  expect_equal(as.character(actual_fns), expected_fns)

  #check preserve_order
  tfrmt_ord <- tfrmt_ord |>
    tfrmt(
    footnote_plan = footnote_plan(
      footnote_structure("BMI: Body Mass Index"),
      footnote_structure("Placebo drug + standard of care", column_val = "column2"),
      footnote_structure("Source Note"),
      footnote_structure("label_1 footnote",  label_val="label_1"),
      marks = "numbers",
      order = "preserve_order"
    )
  )

  gt_out <- print_mock_gt(tfrmt_ord)
  current_order_opt <- gt_out[["_options"]][["value"]][[1]]
  expect_equal(current_order_opt, "preserve_order")

  # check marks last
  tfrmt_ord <- tfrmt_ord |>
    tfrmt(
      footnote_plan = footnote_plan(
        footnote_structure("BMI: Body Mass Index"),
        footnote_structure("Placebo drug + standard of care", column_val = "column2"),
        footnote_structure("Source Note"),
        footnote_structure("label_1 footnote",  label_val="label_1"),
        marks = "numbers",
        order = "marks_last"
      )
    )

  gt_out <- print_mock_gt(tfrmt_ord)
  current_order_opt <- gt_out[["_options"]][["value"]][[1]]
  expect_equal(current_order_opt, "marks_last")

})

test_that("Page plan with defined split", {

  df <- tibble::tribble(
    ~ grp, ~ lbl, ~ prm, ~ trt,
    "A"  ,  "a" , "n"  , 22,
    "A"  ,  "b" , "n"  , 11,
    "B"  ,  "a" , "n"  , 24,
    "B"  ,  "b" , "n"  , 55,
    "C"  ,  "a" , "n"  , 12,
    "C"  ,  "b" , "n"  , 19,
  )
  my_page_plan <- page_plan(
    page_structure(group_val = "A")
  )

  auto_split <- apply_page_plan(df, my_page_plan, vars(grp), quo(lbl))
  man_split <- list(
    tibble::tribble(
      ~ grp, ~ lbl, ~ prm, ~ trt,
      "A"  ,  "a" , "n"  , 22,
      "A"  ,  "b" , "n"  , 11
    ),
    tibble::tribble(
      ~ grp, ~ lbl, ~ prm, ~ trt,
      "B"  ,  "a" , "n"  , 24,
      "B"  ,  "b" , "n"  , 55,
      "C"  ,  "a" , "n"  , 12,
      "C"  ,  "b" , "n"  , 19,
    )
  )

  expect_equal(auto_split, man_split)
})



test_that("Page plan with grouped split", {

  # single grouping var
  df <- tibble::tribble(
    ~ grp, ~ lbl, ~ prm, ~ trt,
    "A"  ,  "a" , "n"  , 22,
    "A"  ,  "b" , "n"  , 11,
    "B"  ,  "a" , "n"  , 24,
    "B"  ,  "b" , "n"  , 55,
    "C"  ,  "a" , "n"  , 12,
    "C"  ,  "b" , "n"  , 19,
  )
  my_page_plan <- page_plan(
    page_structure(group_val = ".default")
  )

  auto_split <- apply_page_plan(df, my_page_plan, vars(grp), quo(lbl))
  man_split <- list(
    tibble::tribble(
      ~ grp, ~ lbl, ~ prm, ~ trt,
      "A"  ,  "a" , "n"  , 22,
      "A"  ,  "b" , "n"  , 11
    ),
    tibble::tribble(
      ~ grp, ~ lbl, ~ prm, ~ trt,
      "B"  ,  "a" , "n"  , 24,
      "B"  ,  "b" , "n"  , 55
    ),
    tibble::tribble(
      ~ grp, ~ lbl, ~ prm, ~ trt,
      "C"  ,  "a" , "n"  , 12,
      "C"  ,  "b" , "n"  , 19,
    )
  )

  expect_equal(auto_split, man_split, ignore_attr = c(".page_note", ".page_grp_vars"))

  expect_equal(
    map_chr(auto_split, ~ attr(.x, ".page_note")),
    c("grp: A","grp: B", "grp: C")
  )
  expect_equal(
    attr(auto_split, ".page_grp_vars"),
    "grp"
  )

  # multi grouping vars - 1 selected
  df <- tibble::tribble(
    ~ grp1, ~ grp2, ~ lbl, ~ prm, ~ trt,
    "A"  ,  "a" ,   "n"  , "n"  , 22,
    "A"  ,  "b" ,   "n"  , "n"  , 11,
    "B"  ,  "a" ,   "n"  , "n"  , 24,
    "B"  ,  "b" ,   "n"  , "n"  , 55,
    "C"  ,  "a" ,   "n"  , "n"  , 12,
    "C"  ,  "b" ,   "n"  , "n"  , 19,
  )
  my_page_plan <- page_plan(
    page_structure(group_val = list(grp2 = ".default"))
  )

  auto_split <- apply_page_plan(df, my_page_plan, vars(grp1, grp2), quo(lbl))
  man_split <- list(
    tibble::tribble(
      ~ grp1, ~ grp2, ~ lbl, ~ prm, ~ trt,
      "A"  ,  "a" ,   "n"  , "n"  , 22,
      "B"  ,  "a" ,   "n"  , "n"  , 24,
      "C"  ,  "a" ,   "n"  , "n"  , 12
    ),
    tibble::tribble(
      ~ grp1, ~ grp2, ~ lbl, ~ prm, ~ trt,
      "A"  ,  "b" ,   "n"  , "n"  , 11,
      "B"  ,  "b" ,   "n"  , "n"  , 55,
      "C"  ,  "b" ,   "n"  , "n"  , 19,
    )
  )

  expect_equal(auto_split, man_split, ignore_attr = c(".page_note", ".page_grp_vars"))

  expect_equal(
    map_chr(auto_split, ~ attr(.x, ".page_note")),
    c("grp2: a","grp2: b")
  )
  expect_equal(
    attr(auto_split, ".page_grp_vars"),
    "grp2"
  )

  # multi grouping vars - 2 selected
  df <- tibble::tribble(
    ~ grp1, ~ grp2, ~ lbl, ~ prm, ~ trt,
    "A"  ,  "a" ,   "n"  , "n"  , 22,
    "A"  ,  "b" ,   "n"  , "n"  , 11,
    "B"  ,  "a" ,   "n"  , "n"  , 24,
    "B"  ,  "b" ,   "n"  , "n"  , 55
  )
  my_page_plan <- page_plan(
    page_structure(group_val = ".default")
  )

  auto_split <- apply_page_plan(df, my_page_plan, vars(grp1, grp2), quo(lbl))

  man_split <- list(
    tibble::tribble(
      ~ grp1, ~ grp2, ~ lbl, ~ prm, ~ trt,
      "A"  ,  "a" ,   "n"  , "n"  , 22
    ),
    tibble::tribble(
      ~ grp1, ~ grp2, ~ lbl, ~ prm, ~ trt,
      "A"  ,  "b" ,   "n"  , "n"  , 11
    ),
    tibble::tribble(
      ~ grp1, ~ grp2, ~ lbl, ~ prm, ~ trt,
      "B"  ,  "a" ,   "n"  , "n"  , 24
    ),
    tibble::tribble(
      ~ grp1, ~ grp2, ~ lbl, ~ prm, ~ trt,
      "B"  ,  "b" ,   "n"  , "n"  , 55
    )
  )
  expect_equal(auto_split, man_split, ignore_attr = c(".page_note", ".page_grp_vars"))

  expect_equal(
    map_chr(auto_split, ~ attr(.x, ".page_note")),
    c("grp1: A, grp2: a", "grp1: A, grp2: b", "grp1: B, grp2: a", "grp1: B, grp2: b")
  )
  expect_equal(
    attr(auto_split, ".page_grp_vars"),
    c("grp1","grp2")
  )

  # multi grouping vars w/ lbl
  df <- tibble::tribble(
    ~ grp1, ~ grp2, ~ lbl, ~ prm, ~ trt,
    "A"  ,  "a" ,   "n"  , "n"  , 22,
    "A"  ,  "a" ,   "pct", "pct"  , 11,
    "A"  ,  "b" ,   "n"  , "n"  , 24,
    "A"  ,  "b" ,   "pct", "pct"  , 55
  )
  my_page_plan <- page_plan(
    page_structure(label_val = ".default")
  )

  auto_split <- apply_page_plan(df, my_page_plan, vars(grp1, grp2), quo(lbl))
  man_split <- list(
    tibble::tribble(
      ~ grp1, ~ grp2, ~ lbl, ~ prm, ~ trt,
      "A"  ,  "a" ,   "n"  , "n"  , 22,
      "A"  ,  "b" ,   "n"  , "n"  , 24
    ) ,
    tibble::tribble(
      ~ grp1, ~ grp2, ~ lbl, ~ prm, ~ trt,
      "A"  ,  "a" ,   "pct", "pct"  , 11,
      "A"  ,  "b" ,   "pct", "pct"  , 55
    )
  )

  expect_equal(auto_split, man_split, ignore_attr = c(".page_note",".page_grp_vars"))

  expect_equal(
    map_chr(auto_split, ~ attr(.x, ".page_note")),
    c("lbl: n", "lbl: pct")
  )
  expect_equal(
    attr(auto_split, ".page_grp_vars"),
    "lbl"
  )

  # specific lbl value
  df <- tibble::tribble(
    ~ grp1, ~ grp2, ~ lbl, ~ prm, ~ trt,
    "A"  ,  "a" ,   "lbl1"  , "n"  , 22,
    "A"  ,  "b" ,   "lbl1", "pct"  , 11,
    "A"  ,  "a" ,   "lbl2"  , "n"  , 34,
    "A"  ,  "b" ,   "lbl2", "pct"  , 65,
    "A"  ,  "c" ,   "lbl1"  , "n"  , 97,
    "A"  ,  "d" ,   "lbl1", "pct"  , 23,
    "A"  ,  "c" ,   "lbl2"  , "n"  , 34,
    "A"  ,  "d" ,   "lbl2", "pct"  , 42
  )
  my_page_plan <- page_plan(
    page_structure(label_val = "lbl1")
  )
  auto_split <- apply_page_plan(df, my_page_plan, vars(grp1, grp2), quo(lbl))

  man_split <- list(
    tibble::tribble(
      ~ grp1, ~ grp2, ~ lbl, ~ prm, ~ trt,
      "A"  ,  "a" ,   "lbl1"  , "n"  , 22,
      "A"  ,  "b" ,   "lbl1", "pct"  , 11
    ),
    tibble::tribble(
      ~ grp1, ~ grp2, ~ lbl, ~ prm, ~ trt,
      "A"  ,  "a" ,   "lbl2"  , "n"  , 34,
      "A"  ,  "b" ,   "lbl2", "pct"  , 65,
      "A"  ,  "c" ,   "lbl1"  , "n"  , 97,
      "A"  ,  "d" ,   "lbl1", "pct"  , 23
    ),
    tibble::tribble(
      ~ grp1, ~ grp2, ~ lbl, ~ prm, ~ trt,
      "A"  ,  "c" ,   "lbl2"  , "n"  , 34,
      "A"  ,  "d" ,   "lbl2", "pct"  , 42
    )
  )

  expect_equal(auto_split, man_split, ignore_attr = c(".page_note",".page_grp_vars"))

})


test_that("page plan with mix of defined & group splits",{

  df <- tibble::tribble(
    ~ grp1, ~ grp2, ~lbl, ~prm, ~trt,
    "A"   , "a"   , "n" , "n" , 22,
    "A"   , "a"   , "pct" , "pct" , 11,
    "A"   , "b"   , "n" , "n" , 44,
    "A"   , "b"   , "pct" , "pct" , 22,
    "B"   , "a"   , "n" , "n" , 54,
    "B"   , "a"   , "pct" , "pct" , 67,
    "B"   , "b"   , "n" , "n" , 72,
    "B"   , "b"   , "pct" , "pct" , 12
  )
  my_page_plan <- page_plan(
    page_structure(group_val = list(grp1 = ".default", grp2 = "a"))
  )

  auto_split <- apply_page_plan(df, my_page_plan, vars(grp1, grp2), quo(lbl))

  ## CURRENT BEHAVIOR:
  # 1. split every level of grp1
  # 2. within 1, split after each consecutive set of grp2 ="a"

  man_split <- list(
    tibble::tribble(
      ~ grp1, ~ grp2, ~lbl, ~prm, ~trt,
      "A"   , "a"   , "n" , "n" , 22,
      "A"   , "a"   , "pct" , "pct" , 11
    ),
    tibble::tribble(
      ~ grp1, ~ grp2, ~lbl, ~prm, ~trt,
      "A"   , "b"   , "n" , "n" , 44,
      "A"   , "b"   , "pct" , "pct" , 22
    ),
    tibble::tribble(
      ~ grp1, ~ grp2, ~lbl, ~prm, ~trt,
      "B"   , "a"   , "n" , "n" , 54,
      "B"   , "a"   , "pct" , "pct" , 67
    ),
    tibble::tribble(
      ~ grp1, ~ grp2, ~lbl, ~prm, ~trt,
      "B"   , "b"   , "n" , "n" , 72,
      "B"   , "b"   , "pct" , "pct" , 12
    )
  )

  expect_equal(auto_split, man_split, ignore_attr = c(".page_note",".page_grp_vars"))

  expect_equal(
    map_chr(auto_split, ~ attr(.x, ".page_note")),
    c("grp1: A", "grp1: A", "grp1: B", "grp1: B")
  )
  expect_equal(
    attr(auto_split, ".page_grp_vars"),
    "grp1"
  )
})

test_that("page plan with multiple structures", {

  df <- tibble::tribble(
    ~ grp1, ~ grp2, ~lbl, ~prm, ~trt,
    "A"   , "a"   , "n" , "n" , 22,
    "A"   , "b"   , "n" , "n" , 11,
    "A"   , "c"   , "n" , "n" , 44,
    "B"   , "a"   , "n" , "n" , 54,
    "B"   , "b"   , "n" , "n" , 67,
    "B"   , "c"   , "n" , "n" , 72
  )

  # valid
  my_page_plan <- page_plan(
    page_structure(label_val = ".default"),
    page_structure(group_val = list(grp1 = ".default", grp2 = "a")),
    page_structure(group_val = list(grp2 = "b"))
  )

  expect_message(
    auto_split <- apply_page_plan(df, my_page_plan, vars(grp1, grp2), quo(lbl)),
    paste0(
      c("`page_plan` contains multiple `page_structures` with values set to \".default\". ",
      "Only the last one specified will be used."),
      collapse = "\n"
  )
  )



  man_split <- list(
    tibble::tribble(
      ~ grp1, ~ grp2, ~lbl, ~prm, ~trt,
      "A"   , "a"   , "n" , "n" , 22
    ),
    tibble::tribble(
      ~ grp1, ~ grp2, ~lbl, ~prm, ~trt,
      "A"   , "b"   , "n" , "n" , 11
    ),
    tibble::tribble(
      ~ grp1, ~ grp2, ~lbl, ~prm, ~trt,
      "A"   , "c"   , "n" , "n" , 44
    ),
    tibble::tribble(
      ~ grp1, ~ grp2, ~lbl, ~prm, ~trt,
      "B"   , "a"   , "n" , "n" , 54
    ),
    tibble::tribble(
      ~ grp1, ~ grp2, ~lbl, ~prm, ~trt,
      "B"   , "b"   , "n" , "n" , 67
    ),
    tibble::tribble(
      ~ grp1, ~ grp2, ~lbl, ~prm, ~trt,
      "B"   , "c"   , "n" , "n" , 72
    )
  )

  expect_equal(auto_split, man_split, ignore_attr = c(".page_note", ".page_grp_vars"))

  expect_equal(
    map_chr(auto_split, ~ attr(.x, ".page_note")),
    c("grp1: A", "grp1: A", "grp1: A", "grp1: B", "grp1: B", "grp1: B")
  )
  expect_equal(
    attr(auto_split, ".page_grp_vars"),
    "grp1"
  )
})




test_that("Page plan with max_rows", {

  # one group - row_grp_plan label_loc = "indented"
  df <- tibble::tribble(
    ~ grp, ~ lbl, ~ prm, ~ trt,
    "A"  ,  "a" , "n"  , 22,
    "A"  ,  "b" , "n"  , 11,
    "B"  ,  "a" , "n"  , 24,
    "B"  ,  "b" , "n"  , 55,
    "C"  ,  "a" , "n"  , 12,
    "C"  ,  "b" , "n"  , 19,
  ) %>%
    pivot_longer(trt, names_to = "column", values_to = "value")
  mytfrmt <- tfrmt(
    group = "grp",
    label = "lbl",
    param = "prm",
    column = "column",
    value = "value",
    body_plan = body_plan(frmt_structure(group_val=".default", label_val=".default", frmt("xx"))),
    page_plan = page_plan(max_rows = 3)
  )

  auto_split <- apply_tfrmt(df, mytfrmt)
  man_split <- list(
    tibble::tribble(
      ~ lbl, ~ trt, ~`..tfrmt_row_grp_lbl`,
      "A"   , NA, TRUE,
      "  a" , '22', FALSE,
      "  b" , '11', FALSE
    ),
    tibble::tribble(
      ~ lbl, ~ trt, ~`..tfrmt_row_grp_lbl`,
      "B"   , NA, TRUE,
      "  a" , '24', FALSE,
      "  b" , '55', FALSE
    ),
    tibble::tribble(
      ~ lbl, ~ trt, ~`..tfrmt_row_grp_lbl`,
      "C"   , NA, TRUE,
      "  a" , '12', FALSE,
      "  b" , '19', FALSE
    )
  )

  expect_equal(auto_split, man_split, ignore_attr = TRUE)


  # one group - row_grp_plan label_loc = "column"
  mytfrmt <- tfrmt(
    group = "grp",
    label = "lbl",
    param = "prm",
    column = "column",
    value = "value",
    body_plan = body_plan(frmt_structure(group_val=".default", label_val=".default", frmt("xx"))),
    row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc(location = "column")),
    page_plan = page_plan(max_rows = 3)
  )
  auto_split <- apply_tfrmt(df, mytfrmt)
  man_split <- list(
      tibble::tribble(
        ~grp, ~ lbl, ~ trt,
        "A" ,  "a" , '22',
        "A" ,  "b" , '11',
        "B" ,  "a" , '24',
      ),
      tibble::tribble(
        ~grp, ~ lbl, ~ trt,
        "B" ,  "b" , '55',
        "C" ,  "a" , '12',
        "C" ,  "b" , '19',
      )
    )
  expect_equal(auto_split, man_split, ignore_attr = TRUE)


  # two groups - row_grp_plan label_loc = "column"
  df <- tibble::tribble(
    ~ grp1, ~grp2, ~ lbl, ~ prm, ~ trt,
    "AA"  ,  "A", "a" , "n"  , 22,
    "AA"  ,  "A", "b" , "n"  , 11,
    "AA"  ,  "B", "a" , "n"  , 24,
    "BB"  ,  "B", "b" , "n"  , 55,
    "BB"  ,  "C", "a" , "n"  , 12,
    "BB"  ,  "C", "b" , "n"  , 19,
  ) %>%
    pivot_longer(trt, names_to = "column", values_to = "value")
  mytfrmt <- tfrmt(
    group = c("grp1","grp2"),
    label = "lbl",
    param = "prm",
    column = "column",
    value = "value",
    body_plan = body_plan(frmt_structure(group_val=".default", label_val=".default", frmt("xx"))),
    row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc(location = "column")),
    page_plan = page_plan(max_rows = 3)
  )
  auto_split <- apply_tfrmt(df, mytfrmt)
  man_split <- list(
    tibble::tribble(
      ~grp1, ~grp2, ~ lbl, ~ trt,
      "AA" ,  "A" ,  "a" , '22',
      "AA" ,  "A" ,  "b" , '11',
      "AA" ,  "B" ,  "a" , '24',
    ),
    tibble::tribble(
      ~grp1, ~grp2, ~ lbl, ~ trt,
      "BB" ,  "B" ,  "b" , '55',
      "BB" ,  "C" ,  "a" , '12',
      "BB" ,  "C" ,  "b" , '19',
    )
  )
  expect_equal(auto_split, man_split, ignore_attr = TRUE)

})

test_that("Page plan with max_rows & group-level summary rows",{

  dat_summ <- tibble::tribble(
    ~grp1  ,~grp2        ,~my_label     ,   ~prm , ~column, ~val,
    "cat_1" ,"cat_1"      , "cat_1"       , "pct" , "trt" , 34,
    "cat_2" ,"cat_2"      , "cat_2"       , "pct" , "trt" , 43,
    "cat_2" ,"sub_cat_2"  , "sub_cat_2"   , "pct" , "trt" , 12,
    "cat_2" ,"sub_cat_2"  , "sub_cat_3"   , "pct" , "trt" , 76,
    "cat_3" ,"cat_3"      , "cat_3"       , "pct" , "trt" , 56,
    "cat_3" ,"sub_cat_3a" , "sub_cat_3a"  , "pct" , "trt" , 98,
    "cat_3" ,"sub_cat_3b" , "sub_cat_3b_1", "pct" , "trt" , 11,
    "cat_3" ,"sub_cat_3b" , "sub_cat_3b_3", "pct" , "trt" , 5,
    "cat_1" ,"cat_1"      , "cat_1"       , "pct" , "pla" , 23,
    "cat_2" ,"cat_2"      , "cat_2"       , "pct" , "pla" , 15,
    "cat_2" ,"sub_cat_2"  , "sub_cat_2"   , "pct" , "pla" , 73,
    "cat_2" ,"sub_cat_2"  , "sub_cat_3"   , "pct" , "pla" , 10,
    "cat_3" ,"cat_3"      , "cat_3"       , "pct" , "pla" , 65,
    "cat_3" ,"sub_cat_3a" , "sub_cat_3a"  , "pct" , "pla" , 78,
    "cat_3" ,"sub_cat_3b" , "sub_cat_3b_1", "pct" , "pla" , 35,
    "cat_3" ,"sub_cat_3b" , "sub_cat_3b_3", "pct" , "pla" , 8,
  )
  mytfrmt <- tfrmt(
    group = c("grp1","grp2"),
    label = "my_label",
    param = "prm",
    column = "column",
    value = "val",
    body_plan = body_plan(frmt_structure(group_val=".default", label_val=".default", frmt("xx"))),
    row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc(location = "indented")),
    page_plan = page_plan(max_rows = 3)
  )
  auto_split <- apply_tfrmt(dat_summ, mytfrmt)

  man_split <- list(
       tibble::tribble(
         ~my_label     ,~trt  ,~pla  ,~`..tfrmt_row_grp_lbl`,
         "cat_1"       ,"34"  ,"23"   ,FALSE  ,
         "cat_2"       ,"43"  ,"15"   ,FALSE   ,
         "  sub_cat_2" ,"12"  ,"73"   ,FALSE
         ),
       tibble::tribble(
         ~my_label     ,~trt  ,~pla  ,~`..tfrmt_row_grp_lbl`,
         "cat_2"       ,"43"  ,"15"   ,FALSE   ,
         "  sub_cat_2" ,"12"  ,"73"   ,FALSE    ,
         "    sub_cat_3","76"  ,"10"  ,FALSE
       ),
       tibble::tribble(
         ~my_label     ,~trt  ,~pla  ,~`..tfrmt_row_grp_lbl`,
         "cat_3"            ,"56"  ,"65"  ,FALSE  ,
         "  sub_cat_3a"     ,"98"  ,"78"  ,FALSE
       ),
       tibble::tribble(
         ~my_label     ,~trt  ,~pla  ,~`..tfrmt_row_grp_lbl` ,
         "cat_3"            ,"56"  ,"65"  ,FALSE  ,
         "  sub_cat_3b"     ,NA    ,NA    ,TRUE   ,
         "    sub_cat_3b_1"  ,"11"  ,"35" ,FALSE
       ),
       tibble::tribble(
         ~my_label     ,~trt  ,~pla  ,~`..tfrmt_row_grp_lbl` ,
         "cat_3"            ,"56"  ,"65"  ,FALSE  ,
         "  sub_cat_3b"     ,NA    ,NA    ,TRUE   ,
         "    sub_cat_3b_3"  ," 5"  ," 8" ,FALSE
       )
  )
  expect_equal(auto_split, man_split, ignore_attr = TRUE)


  # increase the max_rows
  mytfrmt <- tfrmt(
    group = c("grp1","grp2"),
    label = "my_label",
    param = "prm",
    column = "column",
    value = "val",
    body_plan = body_plan(frmt_structure(group_val=".default", label_val=".default", frmt("xx"))),
    row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc(location = "indented")),
    page_plan = page_plan(max_rows = 8)
  )
  auto_split <- apply_tfrmt(dat_summ, mytfrmt)

  man_split <- list(
    tibble::tribble(
      ~my_label         ,~trt  ,~pla  ,~`..tfrmt_row_grp_lbl`,
      "cat_1"           ,"34"  ,"23"   ,FALSE  ,
      "cat_2"           ,"43"  ,"15"   ,FALSE   ,
      "  sub_cat_2"     ,"12"  ,"73"   ,FALSE    ,
      "    sub_cat_3"   ,"76"  ,"10"  ,FALSE    ,
      "cat_3"           ,"56"  ,"65"  ,FALSE  ,
      "  sub_cat_3a"    ,"98"  ,"78"  ,FALSE,
      "  sub_cat_3b"    ,NA    ,NA    ,TRUE     ,
      "    sub_cat_3b_1","11" ,"35"   , FALSE
    ),
    tibble::tribble(
      ~my_label     ,~trt  ,~pla  ,~`..tfrmt_row_grp_lbl`,
      "cat_3"            ,"56"    ,"65"    ,FALSE   ,
      "  sub_cat_3b"     ,NA    ,NA    ,TRUE   ,
      "    sub_cat_3b_3"  ," 5"  ," 8" ,FALSE
    )
  )
  expect_equal(auto_split, man_split, ignore_attr = TRUE)
})

test_that("page plan with both page_structure and max_rows",{

  dat_summ <- tibble::tribble(
    ~grp1  ,~grp2        ,~my_label     ,   ~prm , ~column, ~val,
    "cat_1" ,"cat_1"      , "cat_1"       , "pct" , "trt" , 34,
    "cat_2" ,"cat_2"      , "cat_2"       , "pct" , "trt" , 43,
    "cat_2" ,"sub_cat_2"  , "sub_cat_2"   , "pct" , "trt" , 12,
    "cat_2" ,"sub_cat_2"  , "sub_cat_3"   , "pct" , "trt" , 76,
    "cat_3" ,"cat_3"      , "cat_3"       , "pct" , "trt" , 56,
    "cat_3" ,"sub_cat_3a" , "sub_cat_3a"  , "pct" , "trt" , 98,
    "cat_3" ,"sub_cat_3b" , "sub_cat_3b_1", "pct" , "trt" , 11,
    "cat_3" ,"sub_cat_3b" , "sub_cat_3b_3", "pct" , "trt" , 5
  )

  mypp <- page_plan(
    page_structure(group_val = list(grp1 = ".default")),
    max_rows = 3)

  mytfrmt <- tfrmt(
    group = c("grp1","grp2"),
    label = "my_label",
    param = "prm",
    column = "column",
    value = "val",
    body_plan = body_plan(frmt_structure(group_val=".default", label_val=".default", frmt("xx"))),
    row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc(location = "indented")),
    page_plan = mypp)
  auto_split <- apply_tfrmt(dat_summ, mytfrmt)

  man_split <- list(
      tibble::tribble(
        ~my_label         ,~trt  ,~`..tfrmt_row_grp_lbl`,
        "cat_1"           ,"34"  ,FALSE  ),
      tibble::tribble(
        ~my_label         ,~trt  ,~`..tfrmt_row_grp_lbl`,
        "cat_2"           ,"43"  ,FALSE   ,
        "  sub_cat_2"     ,"12"  ,FALSE    ,
        "    sub_cat_3"   ,"76"  ,FALSE    ),
      tibble::tribble(
        ~my_label         ,~trt  ,~`..tfrmt_row_grp_lbl`,
        "cat_3"           ,"56"  ,FALSE  ,
        "  sub_cat_3a"    ,"98"  ,FALSE ),
      tibble::tribble(
        ~my_label         ,~trt  ,~`..tfrmt_row_grp_lbl`,
        "cat_3"           ,"56"  ,FALSE  ,
        "  sub_cat_3b"    ,NA    ,TRUE     ,
        "    sub_cat_3b_1","11" ,FALSE
      ),
      tibble::tribble(
        ~my_label     ,~trt  ,~`..tfrmt_row_grp_lbl`,
        "cat_3"            ,"56"  ,FALSE   ,
        "  sub_cat_3b"     ,NA    ,TRUE   ,
        "    sub_cat_3b_3" ," 5"  ,FALSE
      )
    )
  expect_equal(auto_split, man_split, ignore_attr = TRUE)

  expect_equal(
    purrr::map_chr(auto_split, ~ attr(.x, ".page_note")),
    c("grp1: cat_1", "grp1: cat_2", "grp1: cat_3", "grp1: cat_3", "grp1: cat_3")
  )


})

test_that("page plan with page_structure, single level variable",{

  dat_summ <- tibble::tribble(
    ~grp1  ,~grp2        ,~my_label     ,   ~prm , ~column, ~val,
    "cat_1" ,"cat_1"      , "cat_1"       , "pct" , "trt" , 34,
    "cat_1" ,"cat_2"      , "cat_2"       , "pct" , "trt" , 43,
    "cat_1" ,"sub_cat_2"  , "sub_cat_2"   , "pct" , "trt" , 12,
    "cat_1" ,"sub_cat_2"  , "sub_cat_3"   , "pct" , "trt" , 76,
    "cat_1" ,"cat_3"      , "cat_3"       , "pct" , "trt" , 56,
    "cat_1" ,"sub_cat_3a" , "sub_cat_3a"  , "pct" , "trt" , 98,
    "cat_1" ,"sub_cat_3b" , "sub_cat_3b_1", "pct" , "trt" , 11,
    "cat_1" ,"sub_cat_3b" , "sub_cat_3b_3", "pct" , "trt" , 5
  )

  mypp <- page_plan(
    page_structure(group_val = list(grp1 = ".default")))

  mytfrmt <- tfrmt(
    group = c("grp1","grp2"),
    label = "my_label",
    param = "prm",
    column = "column",
    value = "val",
    body_plan = body_plan(frmt_structure(group_val=".default", label_val=".default", frmt("xx"))),
    row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc(location = "indented")),
    page_plan = mypp)
  auto_split <- apply_tfrmt(dat_summ, mytfrmt)

  man_split <- list(
    tibble::tribble(
      ~my_label         ,~trt  ,~`..tfrmt_row_grp_lbl`,
      "cat_1"           ,"34"  ,FALSE,
      "  cat_2"         ,"43"  ,FALSE,
      "  sub_cat_2"     ,"12"  ,FALSE,
      "    sub_cat_3"   ,"76"  ,FALSE,
      "  cat_3"         ,"56"  ,FALSE,
      "  sub_cat_3a"    ,"98"  ,FALSE,
      "  sub_cat_3b"    ,NA    ,TRUE,
      "    sub_cat_3b_1"   ,"11"  ,FALSE,
      "    sub_cat_3b_3"   ," 5"  ,FALSE))

  expect_equal(auto_split, man_split, ignore_attr = TRUE)

  expect_equal(
    purrr::map_chr(auto_split, ~ attr(.x, ".page_note")),
    c("grp1: cat_1")
  )
})

test_that("page_plan() with transform", {
  # transform as formula
  test_data <- tibble::tribble(
    ~grp1   , ~grp2        , ~my_label      , ~prm  , ~column , ~val ,
    "cat_1" , "cat_1"      , "cat_1"        , "pct" , "trt"   ,   34 ,
    "cat_1" , "cat_2"      , "cat_2"        , "pct" , "trt"   ,   43 ,
    "cat_1" , "sub_cat_2"  , "sub_cat_2"    , "pct" , "trt"   ,   12 ,
    "cat_1" , "sub_cat_2"  , "sub_cat_3"    , "pct" , "trt"   ,   76 ,
    "cat_1" , "cat_3"      , "cat_3"        , "pct" , "trt"   ,   56 ,
    "cat_1" , "sub_cat_3a" , "sub_cat_3a"   , "pct" , "trt"   ,   98 ,
    "cat_1" , "sub_cat_3b" , "sub_cat_3b_1" , "pct" , "trt"   ,   11 ,
    "cat_1" , "sub_cat_3b" , "sub_cat_3b_3" , "pct" , "trt"   ,    5
  )

  tfrmt_plan <- tfrmt(
    group = c("grp1", "grp2"),
    label = "my_label",
    param = "prm",
    column = "column",
    value = "val",
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("xx")
      )
    ),
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(
        location = "indented"
      )
    ),
    page_plan = page_plan(
      page_structure(
        group_val = list(
          grp1 = ".default"
        )
      ),
      transform = ~ stringr::str_replace(.x, "grp1", "group 1")
    )
  )

  auto_split <- apply_tfrmt(test_data, tfrmt_plan)

  expect_equal(
    purrr::map_chr(auto_split, ~ attr(.x, ".page_note")),
    c("group 1: cat_1")
  )

  # transform as function
  tfrmt_plan$page_plan$transform <- function(x) {
    x |>
      stringr::str_replace("grp1", "group 1") |>
      stringr::str_replace("cat_", "category ")
  }

  auto_split <- apply_tfrmt(test_data, tfrmt_plan)

  expect_equal(
    purrr::map_chr(auto_split, ~ attr(.x, ".page_note")),
    c("group 1: category 1")
  )
})

test_that("page_plan() with transform and multiple 'page by' variables", {
  test_data <- tibble::tribble(
    ~grp1   , ~grp2        , ~my_label      , ~prm  , ~column , ~val ,
    "cat_1" , "cat_1"      , "cat_1"        , "pct" , "trt"   ,   34 ,
    "cat_1" , "cat_2"      , "cat_2"        , "pct" , "trt"   ,   43 ,
    "cat_1" , "sub_cat_2"  , "sub_cat_2"    , "pct" , "trt"   ,   12 ,
    "cat_1" , "sub_cat_2"  , "sub_cat_3"    , "pct" , "trt"   ,   76 ,
    "cat_1" , "cat_3"      , "cat_3"        , "pct" , "trt"   ,   56 ,
    "cat_1" , "sub_cat_3a" , "sub_cat_3a"   , "pct" , "trt"   ,   98 ,
    "cat_1" , "sub_cat_3b" , "sub_cat_3b_1" , "pct" , "trt"   ,   11 ,
    "cat_1" , "sub_cat_3b" , "sub_cat_3b_3" , "pct" , "trt"   ,    5
  )

  tfrmt_plan <- tfrmt(
    group = c("grp1", "grp2"),
    label = "my_label",
    param = "prm",
    column = "column",
    value = "val",
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("xx")
      )
    ),
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(
        location = "indented"
      )
    ),
    page_plan = page_plan(
      page_structure(
        group_val = ".default"
      ),
      transform = NULL
    )
  )

  # without page label transformation
  auto_split <- apply_tfrmt(test_data, tfrmt_plan)

  expect_equal(
    purrr::map_chr(auto_split, ~ attr(.x, ".page_note")),
    c(
      "grp1: cat_1, grp2: cat_1",
      "grp1: cat_1, grp2: cat_2",
      "grp1: cat_1, grp2: sub_cat_2",
      "grp1: cat_1, grp2: cat_3",
      "grp1: cat_1, grp2: sub_cat_3a",
      "grp1: cat_1, grp2: sub_cat_3b"
    )
  )

  # with page label transformation
  # we want to go from `grp1: cat_1,\ngrp2: cat_1` to `Group 1 (cat_1),\nGroup 2 (cat_1)`
  tfrmt_plan$page_plan$transform <- function(x) {

    interim <- x |>
      stringr::str_replace_all("grp", "Group ") |>
      stringr::str_replace_all(": ", " (") |>
      stringr::str_replace_all(",", "),")

    output <- stringr::str_pad(
      interim,
      width = nchar(interim)+1,
      side = "right",
      pad = ")"
    )

    output
  }

  auto_split <- apply_tfrmt(test_data, tfrmt_plan)

  expect_equal(
    purrr::map_chr(auto_split, ~ attr(.x, ".page_note")),
    c(
      "Group 1 (cat_1), Group 2 (cat_1)",
      "Group 1 (cat_1), Group 2 (cat_2)",
      "Group 1 (cat_1), Group 2 (sub_cat_2)",
      "Group 1 (cat_1), Group 2 (cat_3)",
      "Group 1 (cat_1), Group 2 (sub_cat_3a)",
      "Group 1 (cat_1), Group 2 (sub_cat_3b)"
    )
  )
})

test_that("apply_page_plan() with label transformation in a complex table", {
  # include only subjects marked as part of the safety population
  adsl <- pharmaverseadam::adsl |>
    dplyr::filter(SAFFL == "Y")

  # Load adverse event data
  adae <- pharmaverseadam::adae |>
    dplyr::filter(SAFFL == "Y" & TRTEMFL == "Y")

  # limit printed rows
  adae <- adae |>
    dplyr::filter(AESOC %in% unique(AESOC)[1:3]) |>
    dplyr::group_by(AESOC) |>
    dplyr::filter(AEDECOD %in% unique(AEDECOD)[1:3]) |>
    ungroup()

  # Create an ARD that stacks hierarchical data of adverse events
  # Grouping by treatment, severity, system organ class, and preferred term
  ard_ae <- cards::ard_stack_hierarchical(
    data = adae,
    # by variables must be present in the denominator dataset
    by = c(TRT01A, AESEV),
    variables = c(AESOC, AETERM),
    denominator = adsl,
    statistic = ~ c("n", "p"),
    id = USUBJID,
    over_variables = TRUE,
    overall = TRUE
  )

  # create an ARD for where AESEV is "TOTAL"
  ard_ae2 <- cards::ard_stack_hierarchical(
    data = adae,
    # Note: by variables must be present in the denominator dataset
    by = TRT01A,
    variables = c(AESOC, AETERM),
    denominator = adsl,
    statistic = ~ c("n", "p"),
    id = USUBJID,
    over_variables = TRUE,
    overall = TRUE
  )

  # Keep required AESEV "TOTAL" rows
  ard_ae2 <- ard_ae2 |>
    dplyr::filter(
      variable != "TRT01A" & group1 != "AESOC" & group1 != "AESEV"
    )

  ard_ae4 <- ard_ae |>
    # combine the ards and convert them to the format required for tfrmt
    cards::bind_ard(ard_ae2) |>
    # reshape the data
    shuffle_card(
      by = c("TRT01A", "AESEV"),
      fill_overall = "Total",
      fill_hierarchical_overall = "ANY EVENT"
    ) |>
    # transform group-level freqs/pcts into a singular "bigN" row
    prep_big_n(
      vars = "TRT01A"
    ) |>
    # for nested variables, fill any missing values with "ANY EVENT"
    prep_hierarchical_fill(
      vars = c("AESOC", "AETERM"),
      fill = "ANY EVENT"
    )

  # create ordering variables, sort by AESOC using the total AESEV variable
  ordering_aesoc <- ard_ae4 |>
    dplyr::filter(
      stat_name == "n",
      AETERM == "ANY EVENT",
      TRT01A == "Total",
      AESEV == "Total"
    ) |>
    dplyr::arrange(
      dplyr::desc(
        stat
      )
    ) |>
    dplyr::mutate(
      ord1 = dplyr::row_number()
    ) |>
    dplyr::select(
      AESOC,
      ord1
    )

  # Sort by AETERM after AESOC
  ordering_aeterm <- ard_ae4 |>
    dplyr::filter(
      TRT01A == "Total",
      stat_name == "n",
      AESEV == "Total"
    ) |>
    dplyr::group_by(
      AESOC
    ) |>
    dplyr::arrange(
      dplyr::desc(
        stat
      )
    ) |>
    dplyr::mutate(
      ord2 = dplyr::row_number()
    ) |>
    dplyr::select(
      AESOC,
      AETERM,
      ord2
    )

  # Join on ordering columns
  ard_ae5 <- ard_ae4 |>
    dplyr::full_join(
      ordering_aesoc,
      by = c("AESOC")
    ) |>
    dplyr::full_join(
      ordering_aeterm,
      by = c("AESOC", "AETERM")
    )

  # Extract N values for each treatment where stat_name == "bigN" and use them
  # to put in names of treatment for formatting output
  treatment_N <- ard_ae5 |>
    dplyr::filter(
      stat_name == "bigN"
    ) |>
    dplyr::select(
      TRT01A,
      stat
    ) |>
    dplyr::rename(
      stat_N = stat
    )

  ard_ae6 <- ard_ae5 |>
    dplyr::filter(stat_name != "bigN") |>
    dplyr::left_join(treatment_N, by = "TRT01A", suffix = c("", "_N")) |>
    dplyr::rename(Treatment = TRT01A) |>
    dplyr::mutate(
      Treatment = dplyr::case_when(
        Treatment == "Xanomeline Low Dose" ~ paste0(
          "Xanomeline Low Dose (N=",
          stat_N,
          ")"
        ),
        Treatment == "Xanomeline High Dose" ~ paste0(
          "Xanomeline High Dose (N=",
          stat_N,
          ")"
        ),
        Treatment == "Placebo" ~ paste0("Placebo (N=", stat_N, ")"),
        TRUE ~ Treatment
      )
    ) |>
    # Remove extra stat column
    dplyr::select(
      -stat_N
    ) |>
    # remove total treatment column
    dplyr::filter(
      Treatment != "Total"
    ) |>
    dplyr::select(
      Treatment,
      AESOC,
      AETERM,
      stat,
      stat_name,
      AESEV,
      ord1,
      ord2
    )

  # test with transform as formula
  test_tfrmt <- tfrmt_n_pct(
    n = "n",
    pct = "p",
    pct_frmt_when = frmt_when(
      "==1" ~ frmt("(100%)"),
      ">=0.995" ~ frmt("(>99%)"),
      "==0" ~ frmt(""),
      "<=0.01" ~ frmt("(<1%)"),
      "TRUE" ~ frmt("(xx.x%)", transform = ~ . * 100)
    )
  ) |>
    tfrmt(
      group = c(Treatment, AESOC),
      label = AETERM,
      param = stat_name,
      value = stat,
      column = AESEV,
      sorting_cols = c(Treatment, ord1, ord2),
      col_plan = col_plan(
        "System Organ Class
        Preferred Term" = AESOC,
        -Treatment,
        -ord1,
        -ord2
      ),
      page_plan = page_plan(
        page_structure(
          group_val = list(
            Treatment = ".default"
          )
        ),
        note_loc = "subtitle",
        transform = ~ stringr::str_replace(.x, "Treatment", "Group")
      ),
      row_grp_plan = row_grp_plan(
        row_grp_structure(
          group_val = ".default",
          element_block(post_space = " ")
        )
      )
    )

  auto_split <- apply_tfrmt(ard_ae6, test_tfrmt)

  expect_identical(
    purrr::map_chr(
      auto_split,
      ~ attr(.x, ".page_note")
    ),
    c(
      "Group: Placebo (N=86)",
      "Group: Xanomeline High Dose (N=72)",
      "Group: Xanomeline Low Dose (N=96)"
    )
  )

  # test with transform as function
  test_tfrmt$page_plan$transform <- function(x) {
    stringr::str_replace(x, "Treatment", "Group")
  }

  auto_split <- apply_tfrmt(ard_ae6, test_tfrmt)

    expect_identical(
    purrr::map_chr(
      auto_split,
      ~ attr(.x, ".page_note")
    ),
    c(
      "Group: Placebo (N=86)",
      "Group: Xanomeline High Dose (N=72)",
      "Group: Xanomeline Low Dose (N=96)"
    )
  )

  # test with transform NULL -> we get the original behaviour (page labels using
  # the variable name - Treatment)
  test_tfrmt$page_plan$transform <- NULL

  auto_split <- apply_tfrmt(ard_ae6, test_tfrmt)

  expect_identical(
    purrr::map_chr(
      auto_split,
      ~ attr(.x, ".page_note")
    ),
    c(
      "Treatment: Placebo (N=86)",
      "Treatment: Xanomeline High Dose (N=72)",
      "Treatment: Xanomeline Low Dose (N=96)"
    )
  )
})

test_that("page_plan handles empty string groups without Index 1 error", {

  df <- tibble::tribble(
    ~ grp, ~ lbl, ~ prm, ~ trt,
    ""   , "summ", "n"  , 22,
    "A"  , "a"   , "n"  , 22,
    "A"  , "b"   , "n"  , 11,
    "B"  , "a"   , "n"  , 24,
    "B"  , "b"   , "n"  , 55
  ) %>%
    pivot_longer(trt, names_to = "column", values_to = "value")

  mytfrmt <- tfrmt(
    group = "grp",
    label = "lbl",
    param = "prm",
    column = "column",
    value = "value",
    body_plan = body_plan(
      frmt_structure(group_val=".default", label_val=".default", frmt("xx"))
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " "))
    ),
    page_plan = page_plan(max_rows = 4)
  )

  # confirm "Unable to apply apply_page_plan" message doesnt appears
  expect_no_error({
    expect_no_warning({
      expect_silent({
        result <- print_to_gt(mytfrmt, df)
      })
    })
  })

  expect_s3_class(result, "gt_group")
})

test_that("page_plan handles empty string groups in factor columns, with no row group plan", {

  # Create data where 'grp' is explicitly a factor
  df <- tibble::tribble(
    ~ grp, ~ lbl, ~ prm, ~ trt,
    ""   , "summ", "n"  , 22,
    "A"  , "a"   , "n"  , 22,
    "A"  , "b"   , "n"  , 11,
    "B"  , "a"   , "n"  , 24,
    "B"  , "b"   , "n"  , 55
  ) %>%
    mutate(grp = factor(grp, levels = c( "A", "B", ""))) %>% # Explicitly a factor
    tidyr::pivot_longer(trt, names_to = "column", values_to = "value")

  mytfrmt <- tfrmt(
    group = "grp",
    label = "lbl",
    param = "prm",
    column = "column",
    value = "value",
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("xx"))
    ),
    page_plan = page_plan(max_rows = 4)
  )

  expect_no_error({
    result <- print_to_gt(mytfrmt, df)
  })

  expect_s3_class(result, "gt_group")

})

test_that("applying frmt", {

  sample_df <- data.frame(
    x = c(1234.5678, 345.6789, 56.7891, 4567.8910, 8.9101)
  )

  sample_frmt_no_dec <- frmt("xxx")
  sample_frmt_single_dec <- frmt("xxx.x")
  sample_frmt_double_dec <- frmt("xxx.xx")

  sample_df_no_dec_frmted <- apply_frmt.frmt(
    frmt_def = sample_frmt_no_dec,
    .data = sample_df,
    value = quo(x)
  )

  sample_df_single_dec_frmted <- apply_frmt.frmt(
    frmt_def = sample_frmt_single_dec,
    .data = sample_df,
    value = quo(x)
  )

  sample_df_double_dec_frmted <- apply_frmt.frmt(
    frmt_def = sample_frmt_double_dec,
    .data = sample_df,
    value = quo(x)
  )

  expect_equal(
    sample_df_no_dec_frmted$x,
    c("1235", "346", " 57", "4568", "  9")
  )

  expect_equal(
    sample_df_single_dec_frmted$x,
    c("1234.6", "345.7", " 56.8", "4567.9", "  8.9")
  )

  expect_equal(
    sample_df_double_dec_frmted$x,
    c("1234.57", "345.68", " 56.79", "4567.89", "  8.91")
  )

})

test_that("applying frmt - scientific", {

  sample_df <- data.frame(
    x = c(1234.5678, 345.6789, 56.7891, 4567.8910, 8.9101, 0.0678)
  )

  sample_frmt_10x <- frmt(expression = "xxx.x", scientific = " x10^x")
  sample_frmt_10xx <- frmt(expression = "xxx.x", scientific = " x10^xx")
  sample_frmt_ex <- frmt(expression = "xxx.x", scientific = "e^x")
  sample_frmt_exxxx <- frmt(expression = "xxx.x", scientific = "e^xxxx")

  sample_df_frmted_10x <- apply_frmt.frmt(
    .data = sample_df,
    value = sym("x"),
    frmt_def = sample_frmt_10x
  )

  sample_df_frmted_10xx <- apply_frmt.frmt(
    .data = sample_df,
    value = sym("x"),
    frmt_def = sample_frmt_10xx
  )

  sample_df_frmted_ex <- apply_frmt.frmt(
    .data = sample_df,
    value = sym("x"),
    frmt_def = sample_frmt_ex
  )

  sample_df_frmted_exxxx <- apply_frmt.frmt(
    .data = sample_df,
    value = sym("x"),
    frmt_def = sample_frmt_exxxx
  )

  expect_equal(
    sample_df_frmted_10x$x,
    c("  1.2 x10^3",
      "  3.5 x10^2",
      "  5.7 x10^1",
      "  4.6 x10^3",
      "  8.9 x10^0",
      "  6.8 x10^-2")
  )

  expect_equal(
    sample_df_frmted_10xx$x,
    c("  1.2 x10^ 3",
      "  3.5 x10^ 2",
      "  5.7 x10^ 1",
      "  4.6 x10^ 3",
      "  8.9 x10^ 0",
      "  6.8 x10^-2")
  )

  expect_equal(
    sample_df_frmted_ex$x,
    c("  1.2e^3",
      "  3.5e^2",
      "  5.7e^1",
      "  4.6e^3",
      "  8.9e^0",
      "  6.8e^-2")
  )
  expect_equal(
    sample_df_frmted_exxxx$x,
    c("  1.2e^   3",
      "  3.5e^   2",
      "  5.7e^   1",
      "  4.6e^   3",
      "  8.9e^   0",
      "  6.8e^  -2")
  )
})


test_that("applying frmt - transform", {

  sample_df <- data.frame(
    x = c(1234.5678, 345.6789, 56.7891, 4567.8910, 8.9101)
  )

  formula_transform <- frmt("xxx", transform = ~.*100)
  fx_transform <- frmt("xxx.x", transform = function(x){x^2})

  formula_result<- apply_frmt.frmt(
    frmt_def = formula_transform,
    .data = sample_df,
    value = quo(x)
  ) %>%
    pull(x)

  expect_equal(formula_result,
               c("123457", "34568",  "5679" ,  "456789", "891"))

  fx_result <- apply_frmt.frmt(
    frmt_def = fx_transform,
    .data = sample_df,
    value = quo(x)
  ) %>%
    pull(x)

  expect_equal(fx_result,
               c("1524157.7", "119493.9",   "3225.0",     "20865628.2", " 79.4"  ))



  })


test_that("applying frmt - preserves decimal places after rounding", {


  sample_df <- data.frame(
    x = c(10, 12.3647, 3, 100, 167.299)
  )

  sample_frmt_1dec <- frmt(expression = "xxx.x")
  sample_frmt_2dec <- frmt(expression = "xxx.xx")
  sample_frmt_10x <- frmt(expression = "xxx.x", scientific = " x10^x")

  sample_df_frmted_1dec <- apply_frmt.frmt(
    .data = sample_df,
    value = sym("x"),
    frmt_def = sample_frmt_1dec
  )

  sample_df_frmted_2dec <- apply_frmt.frmt(
    .data = sample_df,
    value = sym("x"),
    frmt_def = sample_frmt_2dec
  )

  sample_df_frmted_10x <- apply_frmt.frmt(
    .data = sample_df,
    value = sym("x"),
    frmt_def = sample_frmt_10x
  )


  expect_equal(
    sample_df_frmted_1dec$x,
    c(" 10.0",
      " 12.4",
      "  3.0",
      "100.0",
      "167.3")
  )

  expect_equal(
    sample_df_frmted_2dec$x,
    c(" 10.00",
      " 12.36",
      "  3.00",
      "100.00",
      "167.30")
  )

  expect_equal(
    sample_df_frmted_10x$x,
    c("  1.0 x10^1",
      "  1.2 x10^1",
      "  3.0 x10^0",
      "  1.0 x10^2",
      "  1.7 x10^2")
  )


})

test_that("applying frmt_combine - 2x", {

  sample_df <- tibble(
    group = "group",
    lab = rep(paste("lab",1:5),2),
    col = "col",
    y = rep(c("A","B"),each = 5),
    x = c(1234.5678, 2345.6789, 3456.7891, 4567.8910, 5678.9101,
          1.2345678, 2.3456789, 3.4567891, 4.5678910, 5.6789101)
  )

  sample_frmt <- frmt_combine(
    "{A} {B}",
    A = frmt("xxx.x"),
    B = frmt("(X.X%)")
  )

  sample_df_frmted <- apply_frmt.frmt_combine(
    frmt_def = sample_frmt,
    .data = sample_df,
    value = quo(x),
    param = quo(y),
    column = vars(col),
    label = quo(lab),
    group = vars(group)
  )

  expect_equal(
    sample_df_frmted,
    tibble(
      group = "group",
      lab = paste("lab",1:5),
      col = "col",
      y = "A",
      x = c("1234.6 (1.2%)", "2345.7 (2.3%)", "3456.8 (3.5%)", "4567.9 (4.6%)", "5678.9 (5.7%)")
    )
  )

})

test_that("applying frmt_combine missing",{
  #Both missing
  sample_df <- tibble(
    group = "group",
    lab = rep(paste("lab",1:5),2),
    col = "col",
    y = rep(c("A","B"),each = 5),
    x = c(1234.5678, 2345.6789, 3456.7891, 4567.8910, NA,
          1.2345678, 2.3456789, 3.4567891, 4.5678910, NA)
  )

  sample_frmt <- frmt_combine(
    "{A} {B}",
    A = frmt("xxx.x"),
    B = frmt("(X.X%)"),
    missing = "Missing"
  )

  sample_df_frmted <- apply_frmt.frmt_combine(
    frmt_def = sample_frmt,
    .data = sample_df,
    value = quo(x),
    param = quo(y),
    column = vars(col),
    label = quo(lab),
    group = vars(group)
  )

  expect_equal(
    sample_df_frmted,
    tibble(
      group = "group",
      lab = paste("lab",1:5),
      col = "col",
      y = "A",
      x = c("1234.6 (1.2%)", "2345.7 (2.3%)", "3456.8 (3.5%)", "4567.9 (4.6%)", "Missing")
    )
  )
  #One Missing
  sample_df <- tibble(
    group = "group",
    lab = rep(paste("lab",1:5),2),
    col = "col",
    y = rep(c("A","B"),each = 5),
    x = c(1234.5678, 2345.6789, 3456.7891, 4567.8910, NA,
          1.2345678, 2.3456789, 3.4567891, 4.5678910, 5.6789101)
  )

  sample_frmt <- frmt_combine(
    "{A} {B}",
    A = frmt("xxx.x"),
    B = frmt("(X.X%)"),
    missing = "Missing"
  )

  sample_df_frmted <- apply_frmt.frmt_combine(
    frmt_def = sample_frmt,
    .data = sample_df,
    value = quo(x),
    param = quo(y),
    column = vars(col),
    label = quo(lab),
    group = vars(group)
  )

  expect_equal(
    sample_df_frmted,
    tibble(
      group = "group",
      lab = paste("lab",1:5),
      col = "col",
      y = "A",
      x = c("1234.6 (1.2%)", "2345.7 (2.3%)", "3456.8 (3.5%)", "4567.9 (4.6%)", "NA (5.7%)")
    )
  )

})

test_that("applying frmt_combine - 3x", {

  sample_df <- tibble(
    group = "group",
    lab = rep(paste("lab",1:5),3),
    col = "col",
    y = rep(c("A","B","C"),each = 5),
    x = c(1234.5678, 2345.6789, 3456.7891, 4567.8910, 5678.9101,
          1.2345678, 2.3456789, 3.4567891, 4.5678910, 5.6789101,
          10,111,1112,13,114)
  )

  sample_frmt <- frmt_combine(
    "{A} {B} - {C}",
    A = frmt("xxx.x"),
    B = frmt("(X.X%)"),
    C = frmt("*XXXX.X")
  )

  sample_df_frmted <- apply_frmt.frmt_combine(
    frmt_def = sample_frmt,
    .data = sample_df,
    value = quo(x),
    param = quo(y),
    column = vars(col),
    label = quo(lab),
    group = vars(group)
  )

  expect_equal(
    sample_df_frmted,
    tibble(
      group = "group",
      lab = paste("lab",1:5),
      col = "col",
      y = "A",
      x = c("1234.6 (1.2%) - *  10.0", "2345.7 (2.3%) - * 111.0", "3456.8 (3.5%) - *1112.0", "4567.9 (4.6%) - *  13.0", "5678.9 (5.7%) - * 114.0")
    )
  )

})

test_that("applying frmt_combine - no unique labels, so unable to frmt_combine", {

  sample_df <- tibble(
    group = "group",
    lab = paste("lab",1:15),
    col = "col",
    y = rep(c("A","B","C"),each = 5),
    x = c(1234.5678, 2345.6789, 3456.7891, 4567.8910, 5678.9101,
          1.2345678, 2.3456789, 3.4567891, 4.5678910, 5.6789101,
          10,111,1112,13,114)
  )

  sample_frmt <- frmt_combine(
    "{A} {B} - {C}",
    A = frmt("xxx.x"),
    B = frmt("(X.X%)"),
    C = frmt("*XXXX.X")
  )

  expect_warning(
    sample_df_frmted <- apply_frmt.frmt_combine(
      frmt_def = sample_frmt,
      .data = sample_df,
      value = quo(x),
      param = quo(y),
      column = vars(col),
      label = quo(lab),
      group = vars(group)
    ),
    "Unable to apply `frmt_combine` due to uniqueness of column/row identifiers. Params that are to be combined need to have matching values across: "
  )

  expect_equal(
    sample_df_frmted,
    tibble(
      group = "group",
      lab = c("lab 1", "lab 10",
              "lab 11", "lab 12", "lab 13", "lab 14", "lab 15", "lab 2", "lab 3",
              "lab 4", "lab 5", "lab 6", "lab 7", "lab 8", "lab 9"),
      col = "col",
      y = c("A", "B", "C", "C",
           "C", "C", "C", "A", "A", "A", "A", "B", "B", "B", "B"),
      x = c("1234.6 NA - NA", "NA (5.7%) - NA", "NA NA - *  10.0", "NA NA - * 111.0",
            "NA NA - *1112.0", "NA NA - *  13.0", "NA NA - * 114.0", "2345.7 NA - NA",
            "3456.8 NA - NA", "4567.9 NA - NA", "5678.9 NA - NA", "NA (1.2%) - NA",
            "NA (2.3%) - NA", "NA (3.5%) - NA", "NA (4.6%) - NA")
    )
  )
})


test_that("applying frmt_when", {
  #Test frmt_when alone
  sample_df <- tibble(
    group = "group",
    lab = rep(paste("lab",1:5), 2),
    col = "col",
    y = rep(c("A","B"),each = 5),
    x = c(1234.5678, 2345.6789, 3456.7891, 4567.8910, 5678.9101,
          1.2345678, 2.3456789, 3.4567891, 4.5678910, 5.6789101)
  )

  sample_frmt <- frmt_when(
    ">1000" ~ frmt("XXX.X"),
    "TRUE" ~ "Undectable"
  )

  sample_df_frmted <- apply_frmt(
    frmt_def = sample_frmt,
    .data = sample_df,
    value = quo(x),
    param = "y",
    column = vars(col),
    label = quo(lab),
    group = vars(group)
  )

  man_df <- tibble::tribble(
    ~group, ~lab,        ~col, ~y,    ~x,
    "group", "lab 1", "col",   "A",     "1234.6",
    "group", "lab 2", "col",   "A",     "2345.7",
    "group", "lab 3", "col",   "A",     "3456.8",
    "group", "lab 4", "col",   "A",     "4567.9",
    "group", "lab 5", "col",   "A",     "5678.9",
    "group", "lab 1", "col",   "B",     "Undectable",
    "group", "lab 2", "col",   "B",     "Undectable",
    "group", "lab 3", "col",   "B",     "Undectable",
    "group", "lab 4", "col",   "B",     "Undectable",
    "group", "lab 5", "col",   "B",     "Undectable",
  )

  expect_equal(sample_df_frmted, man_df)


  #Test in combination
  sample_frmt_combo <- frmt_combine(
    "{A} {B}",
    A = frmt("xxx.x"),
    B = frmt_when(">3" ~ frmt("(X.X%)"),
                  "<=3" ~ frmt("Undetectable")
    )
  )

  sample_df_frmted <- apply_frmt.frmt_combine(
    frmt_def = sample_frmt_combo,
    .data = sample_df,
    value = quo(x),
    param = quo(y),
    column = vars(col),
    label = quo(lab),
    group = vars(group)
  )

  man_df_combo <- tibble::tribble(
    ~group, ~lab,   ~col,   ~y,     ~x,
    "group", "lab 1", "col",  "A",     "1234.6 Undetectable",
    "group", "lab 2", "col",  "A",     "2345.7 Undetectable",
    "group", "lab 3", "col",  "A",     "3456.8 (3.5%)",
    "group", "lab 4", "col",  "A",     "4567.9 (4.6%)",
    "group", "lab 5", "col",  "A",     "5678.9 (5.7%)",
  )
  expect_equal(sample_df_frmted, man_df_combo)


})


test_that("mocks return correctly", {

  #frmt
  frmt_mock <- apply_frmt.frmt(
    frmt_def = frmt("xxx.x"),
    .data = iris,
    value = quo(mock),
    mock = TRUE
  )%>%
    pull(mock)
  expect_equal(frmt_mock, rep("xxx.x", nrow(iris)))

  # frmt_when
  frmt_when_true <- apply_frmt.frmt_when(frmt_when("==100"~ frmt(""),
                                                   "==0"~ "",
                                                   "TRUE" ~ frmt("(XXX.X%)")),
                                         .data = iris, sym("value"),mock = TRUE) %>%
    pull(value)
  expect_equal(frmt_when_true, rep("(XXX.X%)", nrow(iris)))


  frmt_when_no_true <-apply_frmt.frmt_when(frmt_when("==100"~ frmt("Hello"),
                                                     "==0"~ ""),
                                           .data = iris, sym("value"),mock = TRUE) %>%
    pull(value)
  expect_equal(frmt_when_no_true, rep("Hello", nrow(iris)))

  #frmt_combine
  sample_df <- tibble(
    group = "group",
    lab = rep(paste("lab",1:5),2),
    col = "col",
    y = rep(c("A","B"),each = 5)
  )

  sample_frmt <- frmt_combine(
    "{A} {B}",
    A = frmt("xxx.x"),
    B = frmt("(X.X%)"),
    missing = "Missing"
  )

  sample_df_frmted <- apply_frmt.frmt_combine(
    frmt_def = sample_frmt,
    .data = sample_df,
    value = quo(x),
    param = quo(y),
    column = vars(col),
    label = quo(lab),
    group = vars(group),
    mock = TRUE
  ) %>%
    pull(x)

  expect_equal(sample_df_frmted, rep("xxx.x (X.X%)", 5))

})



test_that("Space in Param", {
  no_ten <- frmt_combine("{LM mean} ({LM stderr})",
                         `LM mean` = frmt("xx.x"),
                         `LM stderr` = frmt("xx.xx")
  )


  expect_equal(no_ten$expression, "{`LM mean`} ({`LM stderr`})")

  mixed <- frmt_combine("{mean} ({CV %})",
                        mean = frmt("xx.x"),
                        `CV %` = frmt("xx.xx")
  )

  expect_equal(mixed$expression, "{mean} ({`CV %`})")


  data <- tibble::tribble(
    ~group,	~type,	~label,	~column,	~param,	~value,
    "baseline",	"description",	"Week 12 analysis",	"Placebo",	"LM mean",	79.0,
    "baseline",	"description",	"Week 12 analysis",	"Placebo",	"LM stderr",	5.0,
    "Primary analysis",	"trt comparison",	"Week 12 analysis",	"TRT - PBO",	"LM mean",	-0.3,
    "Primary analysis",	"trt comparison",	"Week 12 analysis",	"TRT - PBO",	"LM stderr",	0.4
  )


  space_combo <- frmt_combine("{`LM mean`} ({`LM stderr`})",
                              `LM mean` = frmt("xx.x"),
                              `LM stderr` = frmt("xx.xx")

  )

  expect_equal(space_combo$expression, "{`LM mean`} ({`LM stderr`})")

  sample_df_frmted <- apply_frmt.frmt_combine(
    frmt_def = space_combo,
    .data = data,
    value = quo(value),
    param = quo(param),
    column = vars(column),
    label = quo(label),
    group = vars(group, type),
    mock = FALSE
  ) %>%
    pull(value)


  expect_equal(sample_df_frmted, c("79.0 ( 5.00)", "-0.3 ( 0.40)"))


})


test_that("frmt_combine only applies when all parameters are in the data", {
  data <- tibble(Group = rep(c("Age (y)", "Sex", "Age (y)", "Sex"), c(3, 3, 6,12)),
                 Label = rep(c("n", "Mean (SD)", "Male","Female"), c(6, 6,6,6)),
                 Column = rep(c("Placebo", "Treatment", "Total"), times = 8),
                 Param = rep(c("n", "mean", "sd", "n", "pct", "n", "pct"),  c(6, 3, 3, 3,3,3,3)),
                 Value = c(15,13,28,14,13,27,73.56, 74.231,71.84,9.347,7.234,8.293,8,7,15,8/14,7/13,15/27,6,6,12,6/14,6/13,12/27
                 )
  ) %>%
    # Note because tfrmt only does rounding we will need to have the percents multiplied by 100
    mutate(Value = case_when(Param == "pct" ~ Value * 100,
                             TRUE ~ Value),
           ord1 = if_else(Group == "Age (y)", 1, 2),
           ord2 = if_else(Label == "n", 1, 2),
           TEMP_row = row_number())


  test_combo <- frmt_structure(group_val = ".default", label_val = ".default",
                               frmt_combine("{n} ({pct}%)",
                                            n = frmt("XX"),
                                            pct = frmt("x.x"))
  )

  rows_to_use <- fmt_test_data(test_combo, data, group= vars(Group),
                label = quo(Label), param = quo(Param) )
  expected <- data %>%
    filter(Label %in% c("Male","Female")) %>%
    pull(TEMP_row)

  expect_equal(rows_to_use, expected)
})

test_that("frmt_combine fills with partially missing values where a column is missing the value", {


  data <- tibble(
      Group = rep(c("Age (y)"), c(6)),
      Label = rep(c("Mean (SD)"), c(6)),
      Column = rep(c("Placebo", "Treatment", "Total"), each = c(2)),
      Param = rep(c("mean", "sd"),  times = c(3)),
      Value = c(1, 2, 3, 4, 5, 6)
    ) %>%
    .[-1,] # remove first row - where a "mean" is, but is otherwise complete


  test_combo <- frmt_combine(
      "{mean} {sd}",
      mean = frmt("XX", missing = " -"),
      sd = frmt("(xx)")
    )

  sample_df_frmted <- apply_frmt.frmt_combine(
    frmt_def = test_combo,
    .data = data,
    value = quo(Value),
    param = quo(Param),
    column = vars(Column),
    label = quo(Label),
    group = vars(Group),
    mock = FALSE
  )

  expect_equal(sample_df_frmted,
               tibble(
                 Group = rep(c("Age (y)"), c(3)),
                 Label = rep(c("Mean (SD)"), c(3)),
                 Column = c("Placebo", "Total", "Treatment"),
                 Param = c("sd","mean","mean"),
                 Value = c(" - ( 2)", " 5 ( 6)", " 3 ( 4)")
               ))
  # Test the NA still comes through when missing isn't provided
  test_combo_na <- frmt_combine(
    "{mean} {sd}",
    mean = frmt("XX"),
    sd = frmt("(xx)")
  )

  sample_df_frmted <- apply_frmt.frmt_combine(
    frmt_def = test_combo_na,
    .data = data,
    value = quo(Value),
    param = quo(Param),
    column = vars(Column),
    label = quo(Label),
    group = vars(Group),
    mock = FALSE
  )

  expect_equal(sample_df_frmted,
               tibble(
                 Group = rep(c("Age (y)"), c(3)),
                 Label = rep(c("Mean (SD)"), c(3)),
                 Column = c("Placebo", "Total", "Treatment"),
                 Param = c("sd","mean","mean"),
                 Value = c("NA ( 2)", " 5 ( 6)", " 3 ( 4)")
               ))


})

test_that("apply_tfrmt drops ..tfrmt_post_space_row and inserts post space rows", {

  dat <- tibble::tribble(
    ~grp,   ~lbl,  ~param, ~column, ~val,
    "A",    "n",   "n",    "trt1",  1,
    "A",    "n",   "n",    "trt2",  2,
    "B",    "n",   "n",    "trt1",  3,
    "B",    "n",   "n",    "trt2",  4
  )

  tfrmt_spec <- tfrmt(
    group = grp,
    label = lbl,
    column = column,
    param = param,
    value = val,
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("x"))
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "indented")
    ),
    col_plan = col_plan(grp, lbl, trt1, trt2)
  )

  result <- apply_tfrmt(dat, tfrmt_spec, mock = FALSE)

  # ..tfrmt_post_space_row should not appear in final output
  expect_false("..tfrmt_post_space_row" %in% names(result))

  # post-space rows should be present (more rows than input groups x labels)
  expect_gt(nrow(result), 2L)
})

test_that("pivot_wider_tfrmt gives message when frmt_combine may be missing",{

  dat1 <- tibble::tribble(
    ~grp2, ~lbl, ~prm, ~column, ~val, ~ord,
    "c",   "n", "n",   1,   1,  1,
    "c",   "n", "n_2", 1,   1.1,  1,
    "b",   "m", "n",   1,   2,  2,
    "b",   "m", "n_2", 1,   2.1,  2,
    "v",   "s", "n",   1,   3,  3,
    "v",   "s", "n_3",   1,   3.3,  3,
    "p",   "e", "n",   1,   4,  4
  )

  dat2 <- tibble::tribble(
    ~grp1, ~grp2, ~lbl, ~prm, ~column, ~val, ~ord,
    "d",   "c",   "n", "n",   1,   1,  1,
    "d",   "c",   "n", "n_2", 1,   1.1,  1,
    "a",   "b",   "m", "n",   1,   2,  2,
    "a",   "b",   "m", "n_2",   1,   2.1,  2,
    "q",   "v",   "s", "n",   1,   3,  3,
    "b",   "p",   "e", "n",   1,   4,  4
  )


  dat3 <- tibble::tribble(
    ~grp1, ~grp2, ~lbl, ~prm, ~column, ~val, ~ord,
    "d",   "c",   "n", "n",   1,   1,  1,
    "d",   "c",   "n", "n_2", 1,   1.1,  1,
    "a",   "b",   "m", "n",   1,   2,  2,
    "a",   "b",   "m", "n_2", 1,   2.1,  2,
    "q",   "v",   "s", "n",   1,   3,  3,
    "q",   "v",   "s", "n_3",   1,   3.3,  3,
    "b",   "p",   "e", "n",   1,   4,  4
  )




  tfrmt_temp <- tfrmt(
    group = grp2,
    label = lbl,
    column = column,
    value = val,
    param = prm,
    sorting_cols = ord,
    col_plan = col_plan(-ord),
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("x.x")
        ),
      frmt_structure(
        group_val = ".default",
        label_val = "m",
        frmt_combine("{n}/{n_2}",
                     n = frmt("x"),
                     n_2 = frmt("x.x")
                     )
        )
      )
    )

  expect_message(
    processed_dat <- apply_tfrmt(dat1, tfrmt_temp, mock = FALSE),
    paste0(
      c(
        "Multiple param listed for the same group/label values.",
        "The following frmt_structures may be missing from the body_plan",
        "or the order may need to be changed:",
        "- `frmt_structure(group_val = \"c\", label_val = \"n\", frmt_combine(\"{n}, {n_2}\",n = frmt(\"xx\"), n_2 = frmt(\"xx\")))`",
        "- `frmt_structure(group_val = \"v\", label_val = \"s\", frmt_combine(\"{n}, {n_3}\",n = frmt(\"xx\"), n_3 = frmt(\"xx\")))`"
      ),
      collapse = "\n"
    ),
    fixed = TRUE
  )

  expect_message(
    processed_dat <- apply_tfrmt(dat2, tfrmt_temp %>% tfrmt(group = c(grp1,grp2)), mock = FALSE),
    paste0(
      c(
        "Multiple param listed for the same group/label values.",
        "The following frmt_structures may be missing from the body_plan",
        "or the order may need to be changed:",
        "- `frmt_structure(group_val = list(grp1 = \"d\", grp2 = \"c\"), label_val = \"n\", frmt_combine(\"{n}, {n_2}\",n = frmt(\"xx\"), n_2 = frmt(\"xx\")))`"
      ),
      collapse = "\n"
    ),
    fixed = TRUE
  )

  expect_message(
    processed_dat <- apply_tfrmt(dat3, tfrmt_temp %>% tfrmt(group = c(grp1,grp2)), mock = FALSE),
    paste0(
      c(
        "Multiple param listed for the same group/label values.",
        "The following frmt_structures may be missing from the body_plan",
        "or the order may need to be changed:",
        "- `frmt_structure(group_val = list(grp1 = \"d\", grp2 = \"c\"), label_val = \"n\", frmt_combine(\"{n}, {n_2}\",n = frmt(\"xx\"), n_2 = frmt(\"xx\")))`",
        "- `frmt_structure(group_val = list(grp1 = \"q\", grp2 = \"v\"), label_val = \"s\", frmt_combine(\"{n}, {n_3}\",n = frmt(\"xx\"), n_3 = frmt(\"xx\")))`"
      ),
      collapse = "\n"
    ),
    fixed = TRUE
  )


  tfrmt_temp2<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1,grp),
    label = rowlbl2,
    column = column,
    param = param,
    value = value,
    sorting_cols = c(ord1, ord2))

  # use quietly to grab messages from apply_tfrmt
  safe_apply_tfrmt<-purrr::quietly(apply_tfrmt)

  expect_equal(
    safe_apply_tfrmt(data_demog %>% filter(rowlbl1=="Age (y)"), tfrmt_temp2 , mock = FALSE)$messages,
    c(
    "The following rows of the given dataset have no format applied to them 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57\n"
    ,"Multiple param listed for the same group/label values.\nThe following frmt_structures may be missing from the body_plan\nor the order may need to be changed:\n- `frmt_structure(group_val = list(rowlbl1 = \"Age (y)\", grp = \"cat\"), label_val = c(\"65-80 yrs\",\"<65 yrs\",\">80 yrs\"), frmt_combine(\"{n}, {pct}\",n = frmt(\"xx\"), pct = frmt(\"xx\")))`"      )
    )


})

test_that("test tentative_process",{

  passing_func <- function(x,y = "value"){
    paste0(x,y)
  }

  failing_func <- function(x,y = "value"){
    stop("this function failed")
    paste0(x,y)
  }

  rlang_abort_func <- function(x,y = "value"){
    rlang::abort("this function failed2")
    paste0(x,y)
  }

  ## function passing in tentative process
  expect_equal(
    tentative_process("x",passing_func),
    "xvalue"
  )

  passing_func_messages <- capture_messages({
    tentative_process("x",passing_func)
  })

  expect_true(is_empty(passing_func_messages))
  expect_equal(
    passing_func_messages,
    character()
  )

  ## function failing in tentative process
  expect_equal(
    suppressMessages(tentative_process("x",failing_func)),
    "x"
  )

  failing_func_messages <- capture_messages({
    tentative_process("x",failing_func)
  })

  expect_true(!is_empty(failing_func_messages))
  expect_equal(
    failing_func_messages,
    "Unable to to apply failing_func.\nReason: this function failed\n"
  )

  ## function failing in tentative process
  expect_equal(
    suppressMessages(tentative_process("x",rlang_abort_func)),
    "x"
  )

  rlang_abort_func_messages <- capture_messages({
    tentative_process("x",rlang_abort_func)
  })

  expect_true(!is_empty(rlang_abort_func_messages))
  expect_equal(
    rlang_abort_func_messages,
    "Unable to to apply rlang_abort_func.\nReason: this function failed2\n"
  )

})

test_that("check_order_vars() messages when order variables cause mismatching rows", {
  test_data <- tibble::tibble(
    param = c("N", "N", "n", "n", "pct", "pct"),
    value = c(111, 222, 11, 22, 11, 22),
    order1 = c(NA, NA, 1, 1, 1, 1),
    order2 = c(NA, NA, 1, 2, 1, 2),
    order3 = c(1, 2, NA, NA, NA, NA),
    label = c(
      "Training set",
      "Training set",
      "Sex, n(%)",
      "Sex, n(%)",
      "Sex, n(%)",
      "Sex, n(%)"
    ),
    column = c(
      "Test",
      "Train",
      "Test",
      "Train",
      "Test",
      "Train"
    )
  )

  tfrmt_plan <- tfrmt(
    label = label,
    column = column,
    param = param,
    value = value,
    sorting_cols = c(
      order1,
      order2,
      order3
    ),
    col_plan = col_plan(
      -order1,
      -order2,
      -order3
    )
  )

  expect_snapshot(
    check_order_vars(
      test_data,
      tfrmt_plan
    )
  )

  test_data2 <- test_data |>
    dplyr::bind_rows(test_data) |>
    dplyr::mutate(
      group = c(
        rep("group1", 6),
        rep("group2", 6)
      )
    )

  tfrmt_plan2 <- tfrmt(
    group = group,
    label = label,
    column = column,
    param = param,
    value = value,
    sorting_cols = c(
      order1,
      order2,
      order3
    ),
    col_plan = col_plan(
      -order1,
      -order2,
      -order3
    )
  )

  expect_snapshot(
    check_order_vars(
      test_data2,
      tfrmt_plan2
    )
  )

  test_data3 <- tibble::tibble(
    param = c("N", "N", "n", "n", "pct", "pct"),
    value = c(111, 222, 11, 22, 11, 22),
    order1 = c(NA, NA, 1, 1, 1, 1),
    order2 = c(NA, NA, 1, 1, 1, 1),
    order3 = c(1, 1, NA, NA, NA, NA),
    label = c(
      "Training set",
      "Training set",
      "Sex, n(%)",
      "Sex, n(%)",
      "Sex, n(%)",
      "Sex, n(%)"
    ),
    column = c(
      "Test",
      "Train",
      "Test",
      "Train",
      "Test",
      "Train"
    )
  )

  expect_no_message(
    check_order_vars(
      test_data3,
      tfrmt_plan
    )
  )
})

test_that("json basic tfrmt",{
  #Empty tfrmt
  #Writing
  tfrmt() %>%
    as_json() %>%
    expect_snapshot()
  #Reading
  tfrmt() %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(tfrmt())

  #Complete tfrmt
  basic_filled_in <- tfrmt(
    group = group,
    label = label,
    param = parm,
    value = val,
    column = c(span2, span1, my_col))
  # Writing
  basic_filled_in %>%
    as_json() %>%
    expect_snapshot()
  # Reading
  basic_filled_in %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(basic_filled_in, ignore_attr = TRUE)

})


test_that("json Titles and subtitle", {
  # Titles and subtitle
  titles <- tfrmt(
    title = "Test Title",
    subtitle = "Also a test"
  )
  titles %>%
    as_json() %>%
    expect_snapshot()

  titles %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(titles, ignore_attr = TRUE)
})

test_that("json row group plans",{
  rgp <- tfrmt(
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = c("A","C"), element_block(post_space = "---")),
      row_grp_structure(group_val = c("B"), element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")
    )
  )

  rgp %>%
    as_json() %>%
    expect_snapshot()

  rgp %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(rgp, ignore_attr = TRUE )

  rgp_named <- tfrmt(
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = list(grp1 = "A", grp2 = "b"), element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "spanning")
    )
  )
  rgp_named %>%
    as_json() %>%
    expect_snapshot()

  rgp_named %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(rgp_named, ignore_attr = TRUE )
})

test_that("json body plan", {
  frmt1 <- tfrmt(
    body_plan = body_plan(
      frmt_structure(
        group_val = "group1",
        label_val = ".default",
        frmt("XXX")
      )
    )
  )

  frmt1 %>%
    as_json() %>%
    expect_snapshot()
  frmt1 %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(frmt1, ignore_attr = TRUE)



  frmt2 <- tfrmt(
    body_plan = body_plan(
      frmt_structure(
        group_val = list(grp_col1 = "group1", grp_col2 = "subgroup"),
        label_val = ".default",
        test = frmt("XXX")
      )
    )
  )

  frmt2 %>%
    as_json() %>%
    expect_snapshot()

  frmt2 %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(frmt2, ignore_attr = TRUE)

  frmt3 <- tfrmt(
    body_plan = body_plan(
      frmt_structure(
        group_val = "group1",
        label_val = ".default",
        frmt("XXX", transform = ~.*100)
      )
    )
  )

  frmt3 %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(frmt3, ignore_attr = TRUE)

  frmt4 <- tfrmt(
    body_plan = body_plan(
      frmt_structure(
        group_val = "group1",
        label_val = ".default",
        frmt("XXX", transform = function(x){x*4})
      )
    )
  )
  new_frmt <- frmt4 %>%
    as_json() %>%
    json_to_tfrmt(json = .)
  # Get out the function to see if it runs the same
  fx1 <- frmt4[[6]][[1]][[4]][[1]][[4]]
  fx2 <- new_frmt[[6]][[1]][[4]][[1]][[4]]
  expect_equal(fx1(1:5), fx2(1:5))

  #Format when test
  frmt_when_simp <- tfrmt(
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_when(
          ">3" ~ frmt("(X.X%)"),
          "<=3" ~ frmt("Undetectable")
        )
      )
    )
  )
  frmt_when_simp %>%
    as_json() %>%
    expect_snapshot()

  frmt_when_simp %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(frmt_when_simp, ignore_attr = TRUE)


  # format when with scalars
  frmt_when_scalar <- tfrmt(
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_when(
          "<1"~ "<1",
          '==100' ~ '',
          '==0' ~'',
          TRUE ~ frmt("(xx)")
        )
      )
    )
  )
  frmt_when_scalar %>%
    as_json() |>
    expect_snapshot()

  frmt_when_scalar %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(frmt_when_scalar, ignore_attr = TRUE)

  #Format combine test
  frmt_comb_simp <- tfrmt(
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{param1} {param2}",
          param1 = frmt("XXX %"),
          param2 = frmt("XX.XXX")
        )
      )
    )
  )
  frmt_comb_simp %>%
    as_json() %>%
    expect_snapshot()
  frmt_comb_simp %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(frmt_comb_simp, ignore_attr = TRUE)
  #Scientific test
  sci <- tfrmt(
    body_plan = body_plan(
      frmt_structure(
        group_val = "group1",
        label_val = ".default",
        frmt("xx.xx", scientific = "x10^xx")
      )
    )
  )

  sci %>%
    as_json() %>%
    expect_snapshot()

  sci %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(sci, ignore_attr = TRUE)

  #Everything test
  complex_frmt <- tfrmt(
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{param1} {param2}",
          param1 = frmt("XXX %"),
          param2 = frmt_when(
            ">3" ~ frmt("(X.X%)"),
            "<=3" ~ frmt("Undetectable")
          ),
          missing = ""
        )
      ),
      frmt_structure(
        group_val = "test1",
        label_val = ".default",
        foo = frmt("xx.x", missing = "--")
      ),
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_when(
          ">0.4" ~ frmt("(X.X%)", missing = ""),
          "<=0.4" ~ frmt_combine("[{par2m1}-{param2}]",
                                 param1 = frmt("XXX"),
                                 param2 = frmt("XXX")),
          missing = NULL
        )
      )
    )
  )

  complex_frmt %>%
    as_json() %>%
    expect_snapshot()

  complex_frmt %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(complex_frmt, ignore_attr = TRUE)
})


# big n's
test_that("json big n", {
  big_n <- tfrmt(
    big_n = big_n_structure(param_val = "bigN", n_frmt = frmt("\nN = xx"))
  )
  big_n %>%
    as_json() %>%
    expect_snapshot()

  big_n %>%
    as_json() %>%
    json_to_tfrmt(json = . ) %>%
    expect_equal(big_n, ignore_attr = TRUE)

})

#Footnote plans
test_that("json footnote plan", {
  fn_simp <- tfrmt(
    footnote_plan = footnote_plan(
      footnote_structure(footnote_text = "Source Note"),
      footnote_structure(footnote_text = "Placebo", column_val = "PL"),
      marks = "standard")
  )
  fn_simp %>%
    as_json() %>%
    expect_snapshot()

  fn_simp%>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(fn_simp, ignore_attr = TRUE)

  # multiple columns
  fn_cols <- tfrmt(
    footnote_plan = footnote_plan(
      footnote_structure(footnote_text = "All Treatments",
                         column_val = list(column = c("T1","T2", "T1&T2"))),
      marks = "numbers")
  )
  fn_cols %>%
    as_json() %>%
    expect_snapshot()

  fn_cols%>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(fn_cols, ignore_attr = TRUE)

  # group and labels
  gl_fn <- tfrmt(
    footnote_plan = footnote_plan(
      footnote_structure(footnote_text = "Footnote goes here",
                         group_val = "group 1", label_val = "label 1"))
  )
  gl_fn %>%
    as_json() %>%
    expect_snapshot()
  gl_fn %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(gl_fn)

  # Nest columns
  nested_fn <- tfrmt(
    footnote_plan = footnote_plan(
      footnote_structure(footnote_text = "Footnote goes here",
                         column_val = list(span = "Treatment", column = "T1&T2"))
    )
  )
  nested_fn %>%
    as_json() %>%
    expect_snapshot()
  nested_fn %>%
    as_json() %>%
    json_to_tfrmt(json = . ) %>%
    expect_equal(nested_fn, ignore_attr = TRUE)

})

test_that("json col_plan", {
  #Basic test
  cp <- tfrmt(col_plan = col_plan(col1, col2, col3))
  cp %>%
    as_json()%>%
    expect_snapshot()

  cp %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(cp, ignore_attr = TRUE)

  #Basic renaming
  rename <- tfrmt(col_plan = col_plan("foo"=col1, col2, col3))

  rename %>%
    as_json() %>%
    expect_snapshot()

  rename %>%
    as_json() %>%
    json_to_tfrmt(json = .)%>%
    expect_equal(rename, ignore_attr = TRUE)

  #Basic tidyselect
  base_ts <- tfrmt(col_plan = col_plan(starts_with('col')))
  base_ts %>%
    as_json() %>%
    expect_snapshot()

  base_ts %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(base_ts, ignore_attr = c(".Environment"))

  #Basic span structure
  span <- tfrmt(
    column = c(span1, col),
    col_plan = col_plan(span_structure(span1 = c("col 4"))))

  span %>%
    as_json() %>%
    expect_snapshot()

  span %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(span, ignore_attr = TRUE)


  # Span structure test
  span_tfrmt <- tfrmt(
    column = c(span2, span1, my_col),
    col_plan = col_plan(
      group,
      label,
      span_structure(span1 = c("col 4")),
      span_structure(span1 = c("cols 1,2"), my_col = c("col2", "col1")),
      span_structure(
        span2 = "Top Label Level 1",
        span1 = "Second Label Level 1.1",
        my_col = c(col_3, col_4)
      ),
      span_structure(
        span2 = "Top Label Level 1",
        span1 = c("col2", "col1"),
        my_col = starts_with("B")
      ),
      span_structure(
        span2 = "Top Label Level 1",
        my_col = col_5
      ),
      span_structure(
        span2 = "Top Label Level 2",
        my_col = c(col_6, col_7)
      ),
      everything(),
      new_col_3 = mycol3,
      -mycol5
    )
  )
  span_tfrmt %>%
    as_json() %>%
    expect_snapshot()

  span_tfrmt %>%
    as_json() %>%
    json_to_tfrmt(json = . )%>%
    expect_equal(span_tfrmt, ignore_attr =TRUE)


})

#col_style_plan
test_that("json col_style_plan",{
  csp <- tfrmt(
    col_style_plan= col_style_plan(
      col_style_structure(align = "left", width = 100, col = "my_var"),
      col_style_structure(align = "right", col = vars(four)),
      col_style_structure(align = c(".", ",", " "), col = vars(two, three)),
      col_style_structure(width = 25, col = span_structure(span = value, col = val2))
    )
  )
  csp %>%
    as_json() %>%
    expect_snapshot()
  csp %>%
    as_json() %>%
    json_to_tfrmt(json = . ) %>%
    expect_equal(csp, ignore_attr = TRUE)


  csp_spaces <- tfrmt(
    col_style_plan= col_style_plan(
      col_style_structure(align = "left", width = 100, col = "my var")
    )
  )
  csp_spaces %>%
    as_json() %>%
    expect_snapshot()
  csp_spaces %>%
    as_json() %>%
    json_to_tfrmt(json = . ) %>%
    expect_equal(csp_spaces, ignore_attr = TRUE)
})

test_that("json page plan",{
  pp <- tfrmt(
    page_plan = page_plan(
      page_structure(group_val = ".default", label_val = NULL),
      page_structure(label_val = "A"),
      note_loc = "source_note"
    )
  )

  expect_snapshot(
    as_json(pp)
  )

  pp %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(pp, ignore_attr = TRUE )

  pp_max_rows <- tfrmt(
    page_plan = page_plan(
      max_rows = 5
    )
  )
  pp_max_rows %>%
    as_json() %>%
    expect_snapshot()

  pp_max_rows %>%
    as_json() %>%
    json_to_tfrmt(json = .) %>%
    expect_equal(pp_max_rows, ignore_attr = TRUE )
})

test_that("page_plan() roundtrip to JSON with transform", {
  # transform as formula / lambda function
  page_plan_lambda <- tfrmt(
    page_plan = page_plan(
      page_structure(
        group_val = ".default",
        label_val = NULL
      ),
      page_structure(label_val = "A"),
      note_loc = "source_note",
      transform = ~ stringr::str_replace(.x, "foo", "bar")
    )
  )

  expect_snapshot(
    as_json(page_plan_lambda)
  )

  page_plan_lambda_from_json <- page_plan_lambda |>
    as_json() |>
    json_to_tfrmt(json = _)

  expect_identical(
    page_plan_lambda_from_json,
    page_plan_lambda,
    # the formula environments will not be the same
    ignore_formula_env = TRUE
  )

  func_pp_lambda <- rlang::as_function(page_plan_lambda$page_plan$transform)
  func_pp_lambda_from_json <- rlang::as_function(page_plan_lambda_from_json$page_plan$transform)

  expect_identical(
    func_pp_lambda("foo: baz"),
    "bar: baz"
  )

  expect_identical(
    func_pp_lambda("foo: baz"),
    func_pp_lambda_from_json("foo: baz")
  )

  # transform as function
  transform_function <- function(x) {
    stringr::str_replace(x, "foo", "bar")
  }

  page_plan_function <- tfrmt(
    page_plan = page_plan(
      page_structure(
        group_val = ".default",
        label_val = NULL
      ),
      page_structure(label_val = "A"),
      note_loc = "source_note",
      transform = transform_function
    )
  )

  expect_snapshot(
    as_json(page_plan_function)
  )

  page_plan_function_from_json <- page_plan_function |>
    as_json() |>
    json_to_tfrmt(json = _)

  expect_equal(
    page_plan_function_from_json,
    page_plan_function,
    # the function environments will not be the same
    ignore_function_env = TRUE
  )

  func_pp_function <- rlang::as_function(page_plan_function$page_plan$transform)
  func_pp_function_from_json <- rlang::as_function(page_plan_function_from_json$page_plan$transform)

  expect_identical(
    func_pp_function("foo: baz"),
    "bar: baz"
  )

  expect_identical(
    func_pp_function("foo: baz"),
    func_pp_function_from_json("foo: baz")
  )
})

test_that("json read/write", {
  test_loc <- "test.json"
  test_tfrmt <- tfrmt(
    # specify columns in the data
    group = c(rowlbl1,grp),
    label = rowlbl2,
    column = column,
    param = param,
    value = value,
    sorting_cols = c(ord1, ord2),
    # specify value formatting
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)")))),
      frmt_structure(group_val = ".default", label_val = "n", frmt("xxx")),
      frmt_structure(group_val = ".default", label_val = c("Mean", "Median", "Min","Max"), frmt("xxx.x")),
      frmt_structure(group_val = ".default", label_val = "SD", frmt("xxx.xx")),
      frmt_structure(group_val = ".default", label_val = ".default", p = frmt("")),
      frmt_structure(group_val = ".default", label_val = c("n","<65 yrs","<12 months","<25"), p = frmt_when(">0.99" ~ ">0.99",
                                                                                                            "<0.001" ~ "<0.001",
                                                                                                            TRUE ~ frmt("x.xxx", missing = "")))
    ),
    # remove extra cols
    col_plan = col_plan(-grp,
                        -starts_with("ord") ),
    # Specify column styling plan
    col_style_plan = col_style_plan(
      col_style_structure(align = c(".",","," "), col = vars(everything()))
    ),

    # Specify row group plan
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")
    ),

    # specify page plan
    page_plan = page_plan(
      max_rows = 8
    )

  )

  #Write out to json file
  tfrmt_to_json(test_tfrmt, path = test_loc)

  # Reading in
  read_tfrmt <- json_to_tfrmt(path = test_loc)

  expect_equal(read_tfrmt, test_tfrmt,
               ignore_attr = TRUE)

  # remove json
  file.remove(test_loc)

})

test_that("json col_plan span_structure roundtrip with parentheses in column names", {
  span_parenthesis <- tfrmt(
    group = group1,
    label = label,
    param = statistic,
    value = value,
    column = c(span, column),
    col_plan = col_plan(
      group1,
      label,
      span_structure(span = "Placebo",              column = c("n (%)", "evt")),
      span_structure(span = "Xanomeline Low Dose",  column = c("n (%)", "evt")),
      span_structure(span = "Xanomeline High Dose", column = c("n (%)", "evt")),
      span_structure(span = "Total",                column = c("n (%)", "evt")),
      .drop = TRUE
    )
  )

  span_parenthesis |>
    tfrmt_to_json()|>
    json_to_tfrmt(json = _) |>
    expect_equal(span_parenthesis, ignore_attr = TRUE)
})

test_that("insert post space - single grouping variable",{

  df <- tibble(
    grp1 = c("A","B","C","D"),
    label = as.character(1:4),
    trtA = rep("xx (xx%)", 4),
    trtB = rep("xx (xx%)", 4),
    trtC = rep("xx (xx%)", 4),
  )

  sample_grp_plan <- row_grp_plan(
    row_grp_structure(group_val = ".default", element_block(post_space = " ")),
    label_loc = element_row_grp_loc(location = "spanning")
  )

  expect_equal(
    apply_row_grp_struct(df, sample_grp_plan$struct_list, vars(grp1), sym("label")) %>%
      select(-..tfrmt_post_space_row),
    tibble::tribble(
      ~grp1,~label, ~trtA,      ~trtB,      ~trtC,
      "A",  "1",  "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "A",  " ",  " ", " ", " ",
      "B",  "2",  "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "B",  " ",  " ", " ", " ",
      "C",  "3",  "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "C",  " ",  " ", " ", " ",
      "D",  "4",  "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "D", " ", " ", " ", " ",
    ))


  sample_grp_plan <- row_grp_plan(
    row_grp_structure(group_val = ".default", element_block(post_space = " "))
  )

  expect_equal(
    apply_row_grp_struct(df, sample_grp_plan$struct_list, vars(grp1), sym("label")) %>%
      select(-..tfrmt_post_space_row),
    tibble::tribble(
      ~grp1, ~label, ~trtA,      ~trtB,       ~trtC,
      "A", "1", "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "A", " ", " ", " ", " ",
      "B", "2", "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "B", " ", " ", " ", " ",
      "C", "3", "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "C", " ", " ", " ", " ",
      "D", "4", "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "D", " ", " ", " ", " ",
    ))

})

test_that("insert post space - two grouping variables",{

  df <- tibble(
    crossing(grp1 = c("A","B","C"),
             grp2 = c("a","b"),
             label = as.character(1:2)),
    trtA = rep("xx (xx%)", 12),
    trtB = rep("xx (xx%)", 12),
    trtC = rep("xx (xx%)", 12),
  )

  sample_grp_plan <- row_grp_plan(
    row_grp_structure(group_val = list(grp1 = ".default", grp2 = "b"), element_block(post_space = " ")),
    label_loc = element_row_grp_loc(location = "spanning")
  )

  expect_equal(
    apply_row_grp_struct(df, sample_grp_plan$struct_list, vars(grp1, grp2), label = sym("label")) %>%
      select(-..tfrmt_post_space_row),
    tibble::tribble(
      ~grp1, ~grp2, ~label,   ~trtA,       ~trtB,     ~trtC,
      "A",  "a",   "1", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
      "A",  "a",   "2", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
      "A",  "b",   "1", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
      "A",  "b",   "2", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
      "A",  "b",   " ", " " ," ", " ",
      "B",  "a",   "1", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
      "B",  "a",   "2", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
      "B",  "b",   "1", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
      "B",  "b",   "2", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
      "B",  "b",   " ", " " ," ", " ",
      "C",  "a",   "1", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
      "C",  "a",   "2", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
      "C",  "b",   "1", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
      "C",  "b",   "2", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
      "C",  "b",   " ", " " ," ", " ",
    ))

  # grouping var not referenced - assumed NULL
  sample_grp_plan <- row_grp_plan(
    row_grp_structure(group_val = list(grp1 = "A"), element_block(post_space = " ")),
    label_loc = element_row_grp_loc(location = "spanning")
  )

   expect_equal(
     apply_row_grp_struct(df, sample_grp_plan$struct_list, vars(grp1, grp2), label = sym("label")) %>%
       select(-..tfrmt_post_space_row),
     tibble::tribble(
       ~grp1, ~grp2, ~label,   ~trtA,       ~trtB,     ~trtC,
       "A",  "a",   "1", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
       "A",  "a",   "2", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
       "A",  "b",   "1", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
       "A",  "b",   "2", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
       "A",  "b",   " ", " " ," ", " ",
       "B",  "a",   "1", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
       "B",  "a",   "2", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
       "B",  "b",   "1", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
       "B",  "b",   "2", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
       "C",  "a",   "1", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
       "C",  "a",   "2", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
       "C",  "b",   "1", "xx (xx%)" ,"xx (xx%)", "xx (xx%)",
       "C",  "b",   "2", "xx (xx%)" ,"xx (xx%)", "xx (xx%)"
     ))
})


test_that("insert mix - single grouping variable",{

  df <- tibble(
    grp1 = c("A","B","C","D"),
    trtA = rep("xx (xx%)", 4),
    trtB = rep("xx (xx%)", 4),
    trtC = rep("xx (xx%)", 4),
  )

  sample_grp_plan <- row_grp_plan(
    row_grp_structure(group_val = c("A","C"), element_block(post_space = "---")),
    row_grp_structure(group_val = c("B"), element_block(post_space = " ")),
    label_loc = element_row_grp_loc(location = "spanning")
  )

  expect_equal(
    apply_row_grp_struct(df, sample_grp_plan$struct_list, vars(grp1)) %>%
      select(-..tfrmt_post_space_row),
    tibble::tribble(
      ~grp1, ~trtA,      ~trtB,      ~trtC,
      "A",  "xx (xx%)", "xx (xx%)",  "xx (xx%)",
      "A",  "--------", "--------",  "--------",
      "B",  "xx (xx%)", "xx (xx%)",  "xx (xx%)",
      "B",  " ", " ",  " ",
      "C",  "xx (xx%)", "xx (xx%)",  "xx (xx%)",
      "C",  "--------", "--------",  "--------",
      "D",  "xx (xx%)", "xx (xx%)",  "xx (xx%)"
    )
    )
})



test_that("insert post space after specific value",{

  df <- tibble(
    crossing(grp1 = c("A","B","C"),
             grp2 = c("a","b")),
    label = as.character(1),
    trtA = rep("xx (xx%)", 6),
    trtB = rep("xx (xx%)", 6),
    trtC = rep("xx (xx%)", 6),
  )

  sample_grp_plan <- row_grp_plan(
    row_grp_structure(group_val = list(grp1 = "A", grp2 = "b"), element_block(post_space = " ")),
    label_loc = element_row_grp_loc(location = "spanning")
  )

  expect_equal(
    apply_row_grp_struct(df, sample_grp_plan$struct_list, vars(grp1, grp2), label = sym("label")) %>%
      select(-..tfrmt_post_space_row),
    tibble::tribble(
      ~grp1,  ~grp2, ~label,   ~trtA,       ~trtB,     ~trtC,
      "A",     "a", "1",   "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "A",     "b", "1",   "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "A",     "b", " ",   " ", " ", " ",
      "B",     "a", "1",   "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "B",     "b", "1",   "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "C",     "a", "1",   "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "C",     "b", "1",   "xx (xx%)", "xx (xx%)", "xx (xx%)"
    )
    )

})




test_that("overlapping row_grp_structures - prefers latest",{

  df <- tibble(
    crossing(grp1 = c("A","B","C"),
             grp2 = c("a","b")),
    label = as.character(1),
    trtA = rep("xx (xx%)", 6),
    trtB = rep("xx (xx%)", 6),
    trtC = rep("xx (xx%)", 6),
  )

  sample_grp_plan <- row_grp_plan(
    row_grp_structure(group_val = ".default", element_block(post_space = " ")),
    row_grp_structure(group_val = list(grp1 = "A", grp2 = "b"), element_block(post_space = "***")),
    label_loc = element_row_grp_loc(location = "column")
  )

  expect_equal(
    apply_row_grp_struct(df, sample_grp_plan$struct_list, vars(grp1, grp2), label = sym("label")) %>%
      select(-..tfrmt_post_space_row),
    tibble::tribble(
      ~grp1,  ~grp2, ~label,   ~trtA,       ~trtB,     ~trtC,
      "A",     "a", "1",   "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "A",     "a", " ",   " ", " ", " ",
      "A",     "b", "1",   "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "A",     "b", "*",   "********", "********", "********",
      "B",     "a", "1",   "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "B",     "a", " ",   " ", " ", " ",
      "B",     "b", "1",   "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "B",     "b", " ",   " ", " ", " ",
      "C",     "a", "1",   "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "C",     "a", " ",   " ", " ", " ",
      "C",     "b", "1",   "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "C",     "b", " ",   " ", " ", " "
    ) )

})


test_that("no post space added if NULL",{

  df <- tibble(
    grp1 = c("A","B","C","D"),
    trtA = rep("xx (xx%)", 4),
    trtB = rep("xx (xx%)", 4),
    trtC = rep("xx (xx%)", 4),
  )

  sample_grp_plan <- row_grp_plan(
    row_grp_structure(group_val = ".default", element_block(post_space = NULL)),
    label_loc = element_row_grp_loc(location = "spanning")
  )

  expect_equal(
    apply_row_grp_struct(df, sample_grp_plan$struct_list, vars(grp1)),
    tibble::tribble(
      ~grp1, ~trtA,      ~trtB,      ~trtC,
      "A",  "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "B",  "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "C",  "xx (xx%)", "xx (xx%)", "xx (xx%)",
      "D",  "xx (xx%)", "xx (xx%)", "xx (xx%)",
    ))
})



test_that("post space is truncated to data width",{

  df <- tibble(
    grp1 = c("A","B","C","D"),
    trtA = rep("xx (xx%)", 4),
    trtB = rep("xx (xx%)", 4),
    trtC = rep("xx (xx%)", 4),
  )

  sample_grp_plan <- row_grp_plan(
    row_grp_structure(group_val = ".default", element_block(post_space ="----------------------")),
    label_loc =element_row_grp_loc(location = "spanning")
  )

  expect_equal(
    apply_row_grp_struct(df, sample_grp_plan$struct_list, vars(grp1)) %>%
      select(-..tfrmt_post_space_row),
    tibble::tribble(
      ~grp1, ~trtA,      ~trtB,      ~trtC,
      "A",  "xx (xx%)", "xx (xx%)",  "xx (xx%)",
      "A",  "--------", "--------",  "--------",
      "B",  "xx (xx%)", "xx (xx%)",  "xx (xx%)",
      "B",  "--------", "--------",  "--------",
      "C",  "xx (xx%)", "xx (xx%)",  "xx (xx%)",
      "C",  "--------", "--------",  "--------",
      "D",  "xx (xx%)", "xx (xx%)",  "xx (xx%)",
      "D",  "--------", "--------",  "--------"))
})


test_that("do not recycle the post space for full width",{

  df <- tibble(
    grp1 = c("A","B","C","D"),
    trtA = rep("xx (xx%)", 4),
    trtB = rep("xx (xx%)", 4),
    trtC = rep("xx (xx%)", 4),
  )

  sample_grp_plan <- row_grp_plan(
    row_grp_structure(group_val = ".default", element_block(post_space ="--", fill = FALSE)),
    label_loc =element_row_grp_loc(location = "spanning")
  )

  expect_equal(
    apply_row_grp_struct(df, sample_grp_plan$struct_list, vars(grp1)) %>%
      select(-..tfrmt_post_space_row),
    tibble::tribble(
      ~grp1, ~trtA,      ~trtB,      ~trtC,
      "A",  "xx (xx%)", "xx (xx%)",  "xx (xx%)",
      "A",  "--"      , "--"      ,  "--"      ,
      "B",  "xx (xx%)", "xx (xx%)",  "xx (xx%)",
      "B",  "--"      , "--"      ,  "--"      ,
      "C",  "xx (xx%)", "xx (xx%)",  "xx (xx%)",
      "C",  "--"      , "--"      ,  "--"      ,
      "D",  "xx (xx%)", "xx (xx%)",  "xx (xx%)",
      "D",  "--"      , "--"      ,  "--"      ))
})

test_that("post space works when data contains NAs",{

  df <- tibble(
    grp1 = c("A","B","C","D"),
    trtA = rep("xx (xx%)", 4),
    trtB = rep("xx (xx%)", 4),
    trtC = rep("xx (xx%)", 4),
    other = c("a","a","b", NA)
  )

  sample_grp_plan <- row_grp_plan(
    row_grp_structure(group_val = ".default", element_block(post_space ="----------------------")),
    label_loc =element_row_grp_loc(location = "spanning")
  )

  expect_equal(
    apply_row_grp_struct(df, sample_grp_plan$struct_list, vars(grp1)) %>%
      select(-..tfrmt_post_space_row),
    tibble::tribble(
      ~grp1, ~trtA,      ~trtB,      ~trtC,      ~other,
      "A",  "xx (xx%)", "xx (xx%)",  "xx (xx%)", "a",
      "A",  "--------", "--------",  "--------", "-",
      "B",  "xx (xx%)", "xx (xx%)",  "xx (xx%)", "a",
      "B",  "--------", "--------",  "--------", "-",
      "C",  "xx (xx%)", "xx (xx%)",  "xx (xx%)", "b",
      "C",  "--------", "--------",  "--------", "-",
      "D",  "xx (xx%)", "xx (xx%)",  "xx (xx%)", NA,
      "D",  "--------", "--------",  "--------", "-"))
})

test_that("Check combine_group_cols with a single group", {

  mock_single_grp <- tibble(
    crossing(grp1 = c("A","B","C"),
             lab = c("a","b")),
    trtA = rep("xx (xx%)", 6),
    trtB = rep("xx (xx%)", 6),
    trtC = rep("xx (xx%)", 6),
  )

  auto_test_no_span <- combine_group_cols(mock_single_grp,
                                          group = vars(grp1), label = sym("lab"))
  man_test_no_span <- tibble::tribble(
    ~grp1,  ~lab,   ~trtA,     ~trtB,   ~trtC,       ~..tfrmt_row_grp_lbl,
    "A",  "A"  , NA,         NA,       NA,           TRUE,
    "A",  "  a", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
    "A",  "  b", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
    "B",  "B"  , NA,         NA,       NA,           TRUE,
    "B",  "  a", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
    "B",  "  b", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
    "C",  "C"  , NA,         NA,       NA,           TRUE,
    "C",  "  a", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
    "C",  "  b", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE)


  expect_equal(auto_test_no_span, man_test_no_span)
  #With spanning (so no change to the data)
  expect_equal(combine_group_cols(mock_single_grp,
                                  group = vars(grp1), label = sym("lab"),
                                  element_row_grp_loc(location = "spanning")) %>%
                 select(-..tfrmt_row_grp_lbl),
               mock_single_grp
  )
})


test_that("Check combine_group_cols with a multi groups", {
  mock_multi_grp <- tibble::tribble(
    ~grp1,    ~grp2,     ~ my_label,
    "grp1_1", "grp2_1", "my_label_1",
    "grp1_1", "grp2_1", "my_label_2",
    "grp1_1", "grp2_2", "my_label_1",
    "grp1_1", "grp2_2", "my_label_2",
    "grp1_2", "grp2_1", "my_label_1",
    "grp1_2", "grp2_1", "my_label_2",
    "grp1_2", "grp2_2", "my_label_1",
    "grp1_2", "grp2_2", "my_label_2",
  ) %>%
    mutate(
      trtA = rep("xx (xx%)", 8),
      trtB = rep("xx (xx%)", 8),
      trtC = rep("xx (xx%)", 8),
    )

  auto_test_no_span <- combine_group_cols(mock_multi_grp,
                                          group = vars(grp1, grp2), label = sym("my_label"),
                                          element_row_grp_loc(location = "indented"))

  man_test_no_span <- tibble::tribble(
    ~grp1,     ~my_label,      ~grp2,   ~trtA,     ~trtB,     ~trtC,         ~..tfrmt_row_grp_lbl,
    "grp1_1", "grp1_1"        ,NA     , NA      , NA      , NA      ,        TRUE,
    "grp1_1", "  grp2_1"      ,"grp2_1", NA     , NA      , NA      ,        TRUE,
    "grp1_1", "    my_label_1","grp2_1", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
    "grp1_1", "    my_label_2","grp2_1", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
    "grp1_1", "  grp2_2"      ,"grp2_2", NA      , NA      , NA      ,        TRUE,
    "grp1_1", "    my_label_1","grp2_2", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
    "grp1_1", "    my_label_2","grp2_2", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
    "grp1_2", "grp1_2"        , NA     , NA      , NA      , NA      ,        TRUE,
    "grp1_2", "  grp2_1"      ,"grp2_1", NA      , NA      , NA      ,        TRUE,
    "grp1_2", "    my_label_1","grp2_1", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
    "grp1_2", "    my_label_2","grp2_1", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
    "grp1_2", "  grp2_2"      ,"grp2_2", NA      , NA      , NA      ,        TRUE,
    "grp1_2", "    my_label_1","grp2_2", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
    "grp1_2", "    my_label_2","grp2_2", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE
  )

  expect_equal(auto_test_no_span, man_test_no_span)

  auto_test_with_span <- combine_group_cols(mock_multi_grp,
                                            group = vars(grp1, grp2), label = sym("my_label"),
                                            element_row_grp_loc(location = "spanning"))

  #Should be the same as removing a group
  man_test_with_span <- mock_multi_grp %>%
    group_by(grp1) %>%
    group_split() %>%
    map_dfr(combine_group_cols,group = vars(grp2), label = sym("my_label")) %>%
    select(grp1, grp2, everything()) %>%
    mutate(grp1 = ifelse(grp1=="", NA, grp1)) %>%
    fill(grp1, .direction = "up")
  expect_equal(auto_test_with_span, man_test_with_span)

})


test_that("Check apply_row_grp_* w/ list-columns (in case of incomplete body_plan)", {
  mock_multi_grp <- tibble::tribble(
    ~grp1,    ~grp2,     ~ my_label,
    "grp1_1", "grp2_1", "my_label_1",
    "grp1_1", "grp2_1", "my_label_2",
    "grp1_1", "grp2_2", "my_label_1",
    "grp1_1", "grp2_2", "my_label_2",
    "grp1_2", "grp2_1", "my_label_1",
    "grp1_2", "grp2_1", "my_label_2",
    "grp1_2", "grp2_2", "my_label_1",
    "grp1_2", "grp2_2", "my_label_2",
  ) %>%
    mutate(
      trtA = rep("xx (xx%)", 8) %>% as.list(),
      trtB = rep("xx (xx%)", 8) %>% as.list(),
      trtC = rep("xx (xx%)", 8) %>% as.list(),
    )

  sample_grp_plan <- row_grp_plan(
    row_grp_structure(group_val = ".default", element_block(post_space =" ")),
    label_loc = element_row_grp_loc(location = "indented")
  )

  auto_test_listcols <- apply_row_grp_lbl(mock_multi_grp, sample_grp_plan$label_loc,group = vars(grp1, grp2), label = sym("my_label")) %>%
    remove_grp_cols(sample_grp_plan$label_loc,group = vars(grp1, grp2),label = sym("my_label"))

  man_test_listcols <- tibble::tribble(
    ~my_label,      ~trtA,     ~trtB,     ~trtC,         ~..tfrmt_row_grp_lbl,
   "grp1_1"        , NA     , NA       , NA      ,       TRUE,
   "  grp2_1"      , NA       , NA       , NA       ,      TRUE,
   "    my_label_1", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
   "    my_label_2", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
   "  grp2_2"      , NA       , NA       , NA       ,      TRUE,
   "    my_label_1", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
   "    my_label_2", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
   "grp1_2"        , NA       , NA       , NA       ,      TRUE,
   "  grp2_1"      , NA       , NA       , NA       ,      TRUE,
   "    my_label_1", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
   "    my_label_2", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
   "  grp2_2"      , NA      , NA       , NA       ,      TRUE,
   "    my_label_1", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE,
   "    my_label_2", "xx (xx%)", "xx (xx%)", "xx (xx%)", FALSE
  ) %>%
    mutate(across(trtA:trtC, ~as.list(.x)))

  expect_equal(auto_test_listcols, man_test_listcols)


  auto_test_listcols <- apply_row_grp_struct(mock_multi_grp, sample_grp_plan$struct_list,group = vars(grp1, grp2), label = sym("my_label")) %>%
    select(-..tfrmt_post_space_row)

  man_test_listcols <- tibble::tribble(
    ~grp1,    ~grp2,   ~my_label,      ~trtA,     ~trtB,     ~trtC,
    "grp1_1", "grp2_1", "my_label_1", "xx (xx%)", "xx (xx%)", "xx (xx%)",
    "grp1_1", "grp2_1", "my_label_2", "xx (xx%)", "xx (xx%)", "xx (xx%)",
    "grp1_1", "grp2_1", " ", " ", " ", " ",
    "grp1_1", "grp2_2", "my_label_1", "xx (xx%)", "xx (xx%)", "xx (xx%)",
    "grp1_1", "grp2_2", "my_label_2", "xx (xx%)", "xx (xx%)", "xx (xx%)",
    "grp1_1", "grp2_2", " ", " ", " ", " ",
    "grp1_2", "grp2_1", "my_label_1", "xx (xx%)", "xx (xx%)", "xx (xx%)",
    "grp1_2", "grp2_1", "my_label_2", "xx (xx%)", "xx (xx%)", "xx (xx%)",
    "grp1_2", "grp2_1", " ", " ", " ", " ",
    "grp1_2", "grp2_2", "my_label_1", "xx (xx%)", "xx (xx%)", "xx (xx%)",
    "grp1_2", "grp2_2", "my_label_2", "xx (xx%)", "xx (xx%)", "xx (xx%)",
    "grp1_2", "grp2_2", " ", " ", " ", " "
  ) %>%
    mutate(across(trtA:trtC, ~as.list(.x)))

  expect_equal(auto_test_listcols, man_test_listcols)



})



test_that("> 2 groups with and without spanner_label", {
  mock_multi_grp <- tibble::tribble(
    ~grp1,    ~grp2,     ~grp3, ~ my_label,
    "grp1_1", "grp2_1", "grp3_1", "my_label_1",
    "grp1_1", "grp2_1", "grp3_1", "my_label_2",
    "grp1_1", "grp2_1", "grp3_2", "my_label_1",
    "grp1_1", "grp2_1", "grp3_2", "my_label_2",
    "grp1_1", "grp2_2", "grp3_1", "my_label_1",
    "grp1_1", "grp2_2", "grp3_1", "my_label_2",
    "grp1_1", "grp2_2", "grp3_2", "my_label_1",
    "grp1_1", "grp2_2", "grp3_2", "my_label_2",
  ) %>%
    mutate(
      trtA = rep("xx (xx%)", 8),
      trtB = rep("xx (xx%)", 8),
      trtC = rep("xx (xx%)", 8),
    )

  plan_no_span <- row_grp_plan(
  )

  expect_equal(
    apply_row_grp_lbl(mock_multi_grp, plan_no_span$label_loc, vars(grp1, grp2, grp3), sym("my_label")) %>%
      remove_grp_cols(plan_no_span$label_loc,vars(grp1, grp2, grp3)),
    tibble::tribble(
      ~my_label        , ~trtA     , ~trtB     , ~trtC   ,     ~..tfrmt_row_grp_lbl,
      "grp1_1"           ,NA          ,NA         ,NA        , TRUE,
      "  grp2_1"         ,NA         ,NA         ,NA        , TRUE,
      "    grp3_1"       ,NA         ,NA         ,NA        , TRUE,
      "      my_label_1" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
      "      my_label_2" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
      "    grp3_2"       ,NA         ,NA         ,NA,         TRUE,
      "      my_label_1" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
      "      my_label_2" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
      "  grp2_2"         ,NA         ,NA         ,NA        , TRUE,
      "    grp3_1"       ,NA         ,NA         ,NA        , TRUE,
      "      my_label_1" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
      "      my_label_2" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
      "    grp3_2"       ,NA         ,NA        ,NA        ,  TRUE,
      "      my_label_1" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
      "      my_label_2" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)" ,FALSE
    )
  )

  plan_with_span <- row_grp_plan(label_loc= element_row_grp_loc(location = "spanning"))

  expect_equal(
    apply_row_grp_lbl(mock_multi_grp, plan_with_span$label_loc, vars(grp1, grp2, grp3), sym("my_label")) %>%
      remove_grp_cols(plan_with_span$label_loc, vars(grp1, grp2, grp3)),
    tibble::tribble(
     ~grp1,   ~my_label        , ~trtA     , ~trtB     , ~trtC   ,    ~..tfrmt_row_grp_lbl,
     "grp1_1", "grp2_1"         ,NA         ,NA         ,NA,          TRUE,
     "grp1_1", "  grp3_1"       ,NA         ,NA         ,NA,         TRUE,
     "grp1_1", "    my_label_1" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
     "grp1_1", "    my_label_2" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
     "grp1_1", "  grp3_2"       ,NA         ,NA         ,NA,         TRUE,
     "grp1_1", "    my_label_1" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
     "grp1_1", "    my_label_2" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
     "grp1_1", "grp2_2"         ,NA         ,NA         ,NA,         TRUE,
     "grp1_1", "  grp3_1"       ,NA         ,NA         ,NA,         TRUE,
     "grp1_1", "    my_label_1" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
     "grp1_1", "    my_label_2" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
     "grp1_1", "  grp3_2"       ,NA         ,NA         ,NA,         TRUE,
     "grp1_1", "    my_label_1" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
     "grp1_1", "    my_label_2" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE
    ) %>% group_by(grp1)
  )

})


test_that("Summary rows are not indented", {
  mock_multi_grp <- tibble::tribble(
    ~grp1,    ~grp2, ~my_label,
    "cat_1", "cat_1", "cat_1",
    "cat_2", "cat_2", "cat_2",
    "cat_2", "sub_cat_2", "sub_cat_2",
    "cat_2", "sub_cat_2", "sub_cat_3",
    "cat_3", "cat_3", "cat_3",
    "cat_3", "sub_cat_3a", "sub_cat_3a",
    "cat_3", "sub_cat_3b", "sub_cat_3b_1",
    "cat_3", "sub_cat_3b", "sub_cat_3b_3",
  ) %>%    mutate(
      trtA = rep("xx (xx%)", 8),
      trtB = rep("xx (xx%)", 8),
      trtC = rep("xx (xx%)", 8),
    )

  plan_no_span <- row_grp_plan(
  )

  expect_equal(
    apply_row_grp_lbl(mock_multi_grp, plan_no_span$label_loc, vars(grp1, grp2), sym("my_label")) %>%
      remove_grp_cols( plan_no_span$label_loc, vars(grp1, grp2)),
    tibble::tribble(
      ~my_label ,        ~trtA       , ~trtB       , ~trtC,   ~..tfrmt_row_grp_lbl,
      "cat_1"            ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
      "cat_2"            ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
      "  sub_cat_2"      ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
      "    sub_cat_3"    ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
      "cat_3"            ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
      "  sub_cat_3a"     ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
      "  sub_cat_3b"     ,NA         ,NA         ,NA        ,TRUE,
      "    sub_cat_3b_1" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE,
      "    sub_cat_3b_3" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)", FALSE
    )
  )

  plan_with_span <- row_grp_plan(label_loc= element_row_grp_loc(location = "spanning"))

  expect_equal(
    apply_row_grp_lbl(mock_multi_grp, plan_with_span$label_loc, vars(grp1, grp2), sym("my_label")) %>%
      remove_grp_cols(plan_with_span$label_loc, vars(grp1, grp2)),
    tibble::tribble(
      ~grp1,   ~my_label ,        ~trtA       , ~trtB       , ~trtC,  ~..tfrmt_row_grp_lbl,
       "cat_1", "cat_1"          ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)",FALSE,
       "cat_2", "cat_2"          ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)",FALSE,
       "cat_2", "sub_cat_2"      ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)",FALSE,
       "cat_2", "  sub_cat_3"    ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)",FALSE,
       "cat_3", "cat_3"          ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)",FALSE,
       "cat_3", "sub_cat_3a"     ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)",FALSE,
       "cat_3", "sub_cat_3b"     ,NA         ,NA         ,NA        ,TRUE,
       "cat_3", "  sub_cat_3b_1" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)",FALSE,
       "cat_3", "  sub_cat_3b_3" ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)",FALSE
    ) %>% group_by(grp1)
  )


  plan_with_column <- row_grp_plan(label_loc= element_row_grp_loc(location = "column"))

  expect_equal(
    apply_row_grp_lbl(mock_multi_grp, plan_with_column$label_loc, vars(grp1, grp2), sym("my_label")) %>%
      remove_grp_cols(plan_with_column$label_loc, vars(grp1, grp2)),
    tibble::tribble(
      ~grp1,   ~grp2,        ~my_label ,     ~trtA       , ~trtB       , ~trtC,
      "cat_1", "cat_1",      "cat_1"          ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)",
      "cat_2", "cat_2",      "cat_2"         ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)",
      "cat_2", "sub_cat_2",  "sub_cat_2"     ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)",
      "cat_2", "sub_cat_2",  "sub_cat_3"     ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)",
      "cat_3", "cat_3",      "cat_3"         ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)",
      "cat_3", "sub_cat_3a", "sub_cat_3a"    ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)",
      "cat_3", "sub_cat_3b", "sub_cat_3b_1"  ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)",
      "cat_3", "sub_cat_3b", "sub_cat_3b_3"  ,"xx (xx%)" ,"xx (xx%)" ,"xx (xx%)",
    )
  )

})

test_that("row order is retained for all selections",{

  dat <- tibble::tribble(
    ~grp1, ~grp2, ~lbl, ~prm, ~column, ~val, ~ord,
    "d",   "c",   "n", "n",   1,   1,  1,
    "a",   "b",   "m", "n",   1,   2,  2,
    "q",   "v",   "s", "n",   1,   3,  3,
    "b",   "p",   "e", "n",   1,   4,  4
  )

  tfrmt_temp <- tfrmt(
    group = c(grp1, grp2),
    label = lbl,
    column = column,
    value = val,
    param = prm,
    sorting_cols = ord,
    col_plan = col_plan(-ord),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("x"))
    )
  )

  gt_indented <-  tfrmt_temp %>%
    tfrmt(
      row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc(location = "indented"))
    ) %>%
    print_to_gt(dat)


  gt_indented_dat <- gt_indented$`_data`
  gt_indented_man <- tibble::tribble(
    ~lbl,       ~`1`, ~..tfrmt_row_grp_lbl,
    "d"       ,NA ,  TRUE,
    "  c"    ,NA,  TRUE,
    "    n"  ,"1", FALSE,
    "a"      ,NA , TRUE,
    "  b"    ,NA , TRUE,
    "    m"  ,"2", FALSE,
    "q"      ,NA , TRUE,
    "  v"    ,NA , TRUE,
    "    s"  ,"3", FALSE,
    "b"      ,NA , TRUE,
    "  p"    ,NA , TRUE,
    "    e"  ,"4", FALSE )

  expect_equal(gt_indented_dat, gt_indented_man, ignore_attr = c(".col_plan_vars",".footnote_locs", ".stub_header"))


  gt_spanning <-  tfrmt_temp %>%
    tfrmt(
      row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc(location = "spanning"))
    ) %>%
    print_to_gt(dat)

  gt_spanning_dat <- gt_spanning$`_data`
  gt_spanning_man <- tibble::tribble(
    ~ grp1,  ~lbl,       ~`1`, ~..tfrmt_row_grp_lbl,
      "d",     "c"   , NA , TRUE,
      "d",     "  n" , "1", FALSE,
      "a",     "b"   , NA , TRUE,
      "a",     "  m" , "2", FALSE,
      "q",     "v"   , NA , TRUE,
      "q",     "  s" , "3", FALSE,
      "b",     "p"   , NA , TRUE,
      "b",     "  e" , "4", FALSE)
  expect_equal(gt_spanning_dat, gt_spanning_man)

  gt_column <-  tfrmt_temp %>%
    tfrmt(
      row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc(location = "column"))
    ) %>%
    print_to_gt(dat)

  gt_column_dat <- gt_column$`_data`
  gt_column_man <- tibble::tribble(
    ~ grp1,  ~grp2,  ~lbl, ~`1`, ~..tfrmt_row_grp_lbl,
    "d",   "c",     "n",     "1",     FALSE,
    "a",   "b",     "m",     "2",     FALSE,
    "q",   "v",     "s",     "3",     FALSE,
    "b",   "p",     "e",     "4",     FALSE  )
  expect_equal(gt_column_dat, gt_column_man, ignore_attr = TRUE)

  # original order also respected if no order variable supplied
  tfrmt(
    group = c(grp1, grp2),
    label = lbl,
    column = column,
    value = val,
    param = prm,
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("x"))
    ),
    row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc(location = "indented"))
    ) %>%
    print_to_gt(dat %>% select(-ord))


  gt_indented_dat <- gt_indented$`_data`
  gt_indented_man <- tibble::tribble(
    ~lbl,       ~`1`, ~..tfrmt_row_grp_lbl,
    "d"       ,NA ,  TRUE,
    "  c"    , NA, TRUE,
    "    n"  ,"1", FALSE,
    "a"      ,NA , TRUE,
    "  b"    ,NA , TRUE,
    "    m"  ,"2", FALSE,
    "q"      ,NA , TRUE,
    "  v"    ,NA , TRUE,
    "    s"  ,"3", FALSE,
    "b"      ,NA , TRUE,
    "  p"    ,NA , TRUE,
    "    e"  ,"4",  FALSE)
  expect_equal(gt_indented_dat, gt_indented_man, ignore_attr = c(".col_plan_vars",".footnote_locs", ".stub_header"))
})


test_that("Row group plans with col style plan",{


  raw_dat <- tibble::tribble(
      ~g1,  ~g2,       ~one,   ~param, ~column, ~ value,
     "G1", "g3",    "n (%)",      "n",  "trt1",      12,
     "G1", "g3",    "n (%)",    "pct",  "trt1",      34,
     "G2_", "g3",     "mean",   "mean",  "trt1",    12.3,
     "G2_", "g3",       "sd",     "sd",  "trt1",    4.34,
     "G2_", "g3",   "median", "median",  "trt1",      14,
     "G3", "g3", "(q1, q3)",     "q1",  "trt1",      10,
     "G3", "g3", "(q1, q3)",     "q3",  "trt1",      20,
     "G1", "g3",    "n (%)",      "n",  "trt2",      24,
     "G1", "g3",    "n (%)",    "pct",  "trt2",      58,
     "G2_", "g3",     "mean",   "mean",  "trt2",    15.4,
     "G2_", "g3",       "sd",     "sd",  "trt2",    8.25,
     "G2_", "g3",   "median", "median",  "trt2",      16,
     "G3", "g3", "(q1, q3)",     "q1",  "trt2",      22,
     "G3", "g3", "(q1, q3)",     "q3",  "trt2",      22,
     "G1", "g3",     "mean",   "pval",  "four",   0.0001
  )

  plan <- tfrmt(
    label = one,
    group = c(g1,g2),
    sorting_cols = c(g1, g2),
    column = vars(column),
    value = value,
    param = param,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",label_val = ".default",
        frmt("xx.xx")
      ),
      frmt_structure(
        group_val = ".default",label_val = "n (%)",
        frmt_combine("{n} ({pct}%)",
                     n = frmt("x"),
                     pct = frmt("xx.x"))
      ),
      frmt_structure(
        group_val = ".default",label_val = "(q1, q3)",
        frmt_combine("({q1}, {q3})",
                     q1 = frmt("xx"),
                     q3 = frmt("xx"))
      ),
      frmt_structure(
        group_val = ".default",label_val = ".default",
        pval = frmt_when(
          "<.001" ~ "<.001",
          TRUE ~ frmt("x.xxx")
        )
      )
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val=list(g1="G1"), element_block(post_space="----")),
      row_grp_structure(group_val=list(g1="G2_"), element_block(post_space="----")),
      label_loc = element_row_grp_loc(location = "spanning")
    ),
    col_style_plan =  col_style_plan(
      col_style_structure(align = "right", col = g1), # col must be the top lebel group
      col_style_structure(align = "right", col = one), # always bueno
      col_style_structure(align = "right", width = 4, col = vars(starts_with("trt"))),
      col_style_structure(align = "left", col = trt1),
      col_style_structure(width = 10, col = four)
    )
  )

  ## suppressing warning from alignment using multiple values. Not pertinent to this test
  suppressWarnings({
    tfrmt_gt <- print_to_gt(plan, raw_dat)
  })

  expect_equal(
    tfrmt_gt$`_data` %>%
      select(-`..tfrmt_row_grp_lbl`) %>%
      as.list(),
    list(
      g1 = c(" G1", " G1", " G1", " G1", "G2_", "G2_", "G2_", "G2_", "G2_", " G3", " G3"),
      one = c( "        g3", "     n (%)", "      mean", "  --------","        g3", "      mean", "        sd", "    median", "  --------", "        g3", "  (q1, q3)"),
      trt1 = c(NA, "12 (34.0%)", "          ", "----------", NA, "12.30     ", " 4.34     ", "14.00     ", "----------",  NA, "(10, 20)  "),
      trt2 = c(NA, "24\n(58.0%)", "          ", "----------", NA, "     15.40", "      8.25", "     16.00", "----------", NA, "  (22,\n  22)"),
      four = c( NA, "", "<.001", "-----",NA, "", "", "", "-----", NA, "")
      )
  )

  # label_loc = "column"
  raw_dat <- tibble::tribble(
    ~g1,  ~g2,       ~one,   ~param, ~column, ~ value,
    "G1", "g3_long", "n (%)",  "n",    "trt1",      12,
    "G1", "g3"     , "n (%)",  "pct",  "trt1",      34,
    "G2_", "g3_long","mean",   "mean", "trt1",    12.3,
    "G2_", "g3"     ,"sd",     "sd",   "trt1",    4.34,
    "G1", "g_long",  "n (%)",  "n",    "trt2",      24,
    "G1", "g3"  ,    "n (%)",  "pct",  "trt2",      58,
    "G2_", "g3_long","mean",   "mean", "trt2",    15.4,
    "G2_", "g3",     "sd",     "sd",   "trt2",    8.25
  )
  plan <- tfrmt(
    label = one,
    group = c(g1,g2),
    column = vars(column),
    value = value,
    param = param,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",label_val = ".default",
        frmt("xx.xx")
      ),
      frmt_structure(
        group_val = ".default",label_val = "n (%)",
        frmt_combine("{n} ({pct}%)",
                     n = frmt("x"),
                     pct = frmt("xx.x"))
      ),
      frmt_structure(
        group_val = ".default",label_val = "(q1, q3)",
        frmt_combine("({q1}, {q3})",
                     q1 = frmt("xx"),
                     q3 = frmt("xx"))
      ),
      frmt_structure(
        group_val = ".default",label_val = ".default",
        pval = frmt_when(
          "<.001" ~ "<.001",
          TRUE ~ frmt("x.xxx")
        )
      )
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = list(g1 = c("G1","G2_"), g2 = ".default"),
        element_block = element_block(post_space = "----")
      ),
      label_loc = element_row_grp_loc(location = "column")
    ),
    col_style_plan =  col_style_plan(
      col_style_structure(align = "right", col = g1), # col must be the top label group
      col_style_structure(align = "right", col = g2), # col must be the top label group
      col_style_structure(align = "right", col = one), # always bueno
      col_style_structure(align = "right", width = 4, col = vars(starts_with("trt"))),
      col_style_structure(align = "left", col = trt1)
    )
  )
  tfrmt_gt <- print_to_gt(plan, raw_dat)

  expect_equal(
    tfrmt_gt$`_data` %>%
      select(-`..tfrmt_row_grp_lbl`) %>%
      as.list(),
    list(
      g1 = c(" G1", " G1", " G1", " G1", "G2_", "G2_", "G2_", "G2_", " G1"),
      g2 = c("g3_long", "g3_long", "     g3", "     g3", "g3_long", "g3_long", "     g3", "     g3", " g_long"),
      one = c( "n (%)", "-----", "n (%)", "-----", " mean", "-----", "   sd", "-----", "n (%)"),
      trt1 = c("12.00", "-----", "34.00", "-----", "12.30", "-----", " 4.34", "-----", "     "),
      trt2 = c("     " ,"-----", "58.00", "-----", "15.40", "-----", " 8.25", "-----", "24.00")
    ),
    ignore_attr = TRUE
  )

})


test_that("Suppress printing of groups", {

  mock_multi_grp <- tibble::tribble(
    ~grp1,    ~grp2,     ~ my_label,
    "grp1_1", "grp2_1", "my_label_1",
    "grp1_1", "grp2_1", "my_label_2",
    "grp1_1", "grp2_2", "my_label_1",
    "grp1_1", "grp2_2", "my_label_2",
    "grp1_2", "grp2_1", "my_label_1",
    "grp1_2", "grp2_1", "my_label_2",
    "grp1_2", "grp2_2", "my_label_1",
    "grp1_2", "grp2_2", "my_label_2",
  ) %>%
    mutate(
      trtA = rep("xx (xx%)", 8),
      trtB = rep("xx (xx%)", 8),
      trtC = rep("xx (xx%)", 8),
    )


  my_plan <- row_grp_plan(label_loc = element_row_grp_loc(location = "noprint"))

  df_no_grp <- tibble::tribble(
    ~my_label,   ~trtA,     ~trtB,     ~trtC  ,
    "my_label_1", "xx (xx%)", "xx (xx%)", "xx (xx%)",
    "my_label_2", "xx (xx%)", "xx (xx%)", "xx (xx%)",
    "my_label_1", "xx (xx%)", "xx (xx%)", "xx (xx%)",
    "my_label_2", "xx (xx%)", "xx (xx%)", "xx (xx%)",
    "my_label_1", "xx (xx%)", "xx (xx%)", "xx (xx%)",
    "my_label_2", "xx (xx%)", "xx (xx%)", "xx (xx%)",
    "my_label_1", "xx (xx%)", "xx (xx%)", "xx (xx%)",
    "my_label_2", "xx (xx%)", "xx (xx%)", "xx (xx%)",
  )

  expect_equal(apply_row_grp_lbl(mock_multi_grp, my_plan$label_loc, vars(grp1, grp2), sym("my_label")) %>%
                 remove_grp_cols(my_plan$label_loc, vars(grp1, grp2)),
               df_no_grp)
})


test_that("Row group plan indenting handles factor variables", {

  dat <- tibble::tribble(
    ~grp_span, ~grp,      ~rowlbl,    ~column, ~param, ~value,
    "span", "topgrp1", "topgrp1"  ,  "A",  "mean",   1,
    "span", "topgrp1", "topgrp1"  ,  "B",  "mean",   1,
    "span", "topgrp1", "lowergrp1",  "A",  "mean",   1,
    "span", "topgrp1", "lowergrp1",  "B",  "mean",   2,
    "span", "topgrp2", "topgrp2"  ,  "A",  "mean",   3,
    "span", "topgrp2", "topgrp2"  ,  "B",  "mean",   4,
    "span", "topgrp2", "lowergrp1",  "A",  "mean",   2,
    "span", "topgrp2", "lowergrp1",  "B",  "mean",   1,
  )

  grp_plan <- row_grp_plan(
  )

  expected <-  tibble::tribble(
    ~ rowlbl,  ~ column,  ~param,  ~value, ~..tfrmt_row_grp_lbl,
    "span"          , NA ,    NA    ,   NA  , TRUE   ,
    "  topgrp1"     ,"A",   "mean",      1 , FALSE,
    "  topgrp1"     ,"B",   "mean",      1 , FALSE,
    "    lowergrp1" ,"A",   "mean",      1 , FALSE,
    "    lowergrp1" ,"B",   "mean",      2 , FALSE,
    "  topgrp2"     ,"A",   "mean",      3 , FALSE,
    "  topgrp2"     ,"B",   "mean",      4 , FALSE,
    "    lowergrp1" ,"A",   "mean",      2 , FALSE,
    "    lowergrp1" ,"B",   "mean",      1 , FALSE)

  expect_equal(
    apply_row_grp_lbl(dat %>% mutate(across(grp_span:rowlbl, as.factor)),
                      grp_plan$label_loc, vars(grp_span, grp), sym("rowlbl")) %>%
      remove_grp_cols(grp_plan$label_loc, vars(grp_span, grp)),
    expected)

  expect_equal(
    apply_row_grp_lbl(dat %>% mutate(across(rowlbl, as.factor)),
                      grp_plan$label_loc, vars(grp_span, grp), sym("rowlbl")) %>%
      remove_grp_cols(grp_plan$label_loc, vars(grp_span, grp)),
    expected)

  expect_equal(
    apply_row_grp_lbl(dat %>% mutate(across(grp, as.factor)),
                      grp_plan$label_loc, vars(grp_span, grp), sym("rowlbl")) %>%
      remove_grp_cols(grp_plan$label_loc, vars(grp_span, grp)),
    expected)

  expect_equal(
    apply_row_grp_lbl(dat %>% mutate(across(grp_span, as.factor)),
                      grp_plan$label_loc, vars(grp_span, grp), sym("rowlbl")) %>%
      remove_grp_cols(grp_plan$label_loc, vars(grp_span, grp)),
    expected)
})


test_that("Check row group plan in tfrmt - expect error when NA in label column",{

  # create data
  data_ae2 <- data_ae %>%
    group_by(AEBODSYS, AETERM) %>%
    mutate(pct_high = value[col2=="Xanomeline High Dose" & param=="pct"]) %>%
    ungroup %>%
    filter(pct_high >10) %>%
    select(-pct_high)

  data_ae2$AETERM <- ifelse(data_ae2$AETERM == "ANY BODY SYSTEM", NA, data_ae2$AETERM)


  # expect error message
  expect_message({
    tfrmt(
      group = AEBODSYS,
      label = AETERM,
      column = c(col2, col1),
      param = param,
      value = value,
      sorting_cols = c(ord1, ord2),
      body_plan = body_plan(
        frmt_structure(group_val = ".default", label_val = ".default",
                       frmt_combine("{n} {pct}",
                                    n = frmt("XXX"),
                                    pct = frmt_when(
                                      "==100" ~ "",
                                      "==0" ~ "",
                                      TRUE ~ frmt("(xx.x %)")))),
        frmt_structure(group_val = ".default", label_val = ".default",
                       AEs = frmt("[XXX]")),
        frmt_structure(group_val = ".default", label_val = ".default",
                       pval = frmt_when(">0.99" ~ ">0.99",
                                        "<0.001" ~ "<0.001",
                                        "<0.05" ~ frmt("x.xxx*"),
                                        TRUE ~ frmt("x.xxx", missing ="--")))
      ),
      col_plan = col_plan(
        -starts_with("ord")
      )
    ) %>%
      print_to_gt(data_ae2) %>%
      tab_options(
        container.width = 1000
      )

  },
  paste("Unable to to apply apply_row_grp_lbl.",
        "Reason: `label` column AETERM contains NA values. For group-level summary data, `label` and the relevant `group` values should match.",
  sep = "\n"),
  )
})


test_that("Check apply_row_grp_lbl - expect error when NA in label column", {

  # create mock data
  mock_multi_grp <- tibble::tribble(
    ~grp1,    ~grp2,     ~ my_label,
    "grp1_1", "grp2_1", "my_label_1",
    "grp1_1", "grp2_1", "my_label_2",
    "grp1_1", "grp2_2", "my_label_1",
    "grp1_1", "grp2_2", "my_label_2",
    "grp1_2", "grp2_1", "my_label_1",
    "grp1_2", "grp2_1", "my_label_2",
    "grp1_2", "grp2_2", "my_label_1",
    "grp1_2", "grp2_2", "my_label_2",
  ) %>%
    mutate(
      trtA = rep("xx (xx%)", 8),
      trtB = rep("xx (xx%)", 8),
      trtC = rep("xx (xx%)", 8),
    )

  mock_multi_grp$my_label <- ifelse(mock_multi_grp$my_label == "my_label_1", NA, mock_multi_grp$my_label)

  sample_grp_plan <- row_grp_plan(
    row_grp_structure(group_val = ".default", element_block(post_space =" ")),
    label_loc = element_row_grp_loc(location = "indented")
  )

  # expect error message
  expect_error({
    auto_test_listcols <- apply_row_grp_lbl(mock_multi_grp, sample_grp_plan$label_loc,group = vars(grp1, grp2), label = sym("my_label"))
  },
  paste("`label` column my_label contains NA values. For group-level summary data, `label` and the relevant `group` values should match.")
  )
})

test_that("footnote structure",{

  # force list if entered as vector

  tfrmt<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = c(col2,trt),
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 2",column_val=c(trt="Placebo")),
      marks="letters"

    )
  )

  expect_equal(is.list(tfrmt$footnote_plan$struct_list[[1]]$column_val),TRUE)



  tfrmt2<-tfrmt(
    # specify columns in the data
    group = c(rowlbl0,rowlbl1),
    label = rowlbl2,
    column = c(col2,trt),
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 2",group_val=c(rowlbl1="Completion Status")),
      marks="letters"

    )
  )

  expect_equal(is.list(tfrmt2$footnote_plan$struct_list[[1]]$group_val),TRUE)

  expect_error(tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = c(col2,trt),
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 2",column_val=list("Treatment column","Placebo")),
      marks="letters"

    )
  ) ,  "when column_val is a list, must be a named list")

  expect_error(tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = c(col2,trt),
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 2",column_val=list("Treatment column",trt="Placebo")),
      marks="letters"

    )
  ) , "when column_val is a list, each entry must be named")


  expect_error(tfrmt(
    # specify columns in the data
    group = c(rowlbl0,rowlbl1),
    label = rowlbl2,
    column = c(col2,trt),
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 2",group_val=c("Group","Completion Status")),
      marks="letters"

    )
  ),  "when tfrmt contains multiple groups, group_val must be a named list")


  expect_error(tfrmt(
    # specify columns in the data
    group = c(rowlbl0,rowlbl1),
    label = rowlbl2,
    column = c(col2,trt),
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 2",group_val=c("Group",rowlbl1="Completion Status")),
      marks="letters"

    )
  ),  "when group_val is a list, each entry must be named")

})

test_that("Testing error messages for missing argument col_style_structure",{

  expect_error(
    tfrmt(
    # specify columns in the data
    group = c(rowlbl1,grp),
    label = rowlbl2,
    column = column,
    param = param,
    value = value,
    sorting_cols = c(ord1, ord2),
    # specify value formatting
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)")))),
      frmt_structure(group_val = ".default", label_val = "n", frmt("xxx")),
      frmt_structure(group_val = ".default", label_val = c("Mean", "Median", "Min","Max"), frmt("xxx.x")),
      frmt_structure(group_val = ".default", label_val = "SD", frmt("xxx.xx")),
      frmt_structure(group_val = ".default", label_val = ".default", p = frmt("")),
      frmt_structure(group_val = ".default", label_val = c("n","<65 yrs","<12 months","<25"), p = frmt_when(">0.99" ~ ">0.99",
                                                                                                            "<0.001" ~ "<0.001",
                                                                                                            TRUE ~ frmt("x.xxx", missing = "")))
    ),
    # remove extra cols
    col_plan = col_plan(-grp,
                        -starts_with("ord") ),
    # Specify column styling plan
    col_style_plan = col_style_plan(
      col_style_structure(align = c(".",","," "))
    ),

    # Specify row group plan
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")
    )
    ),
    "Column element is missing from col_style_structure[.] Note: col here refers to the values within the column variable in your data, rather than the variable name itself")
})


test_that("Testing error message for invalid input to plan parameters, specifically for col_style_plan",{
  expect_error(
    # create tfrmt object
    tfrmt_object <- tfrmt(
      label = label,
      column = column,
      param = param,
      value = value,
      sorting_cols = c(ord),
      col_plan = col_plan(-ord),
      body_plan = body_plan(
        frmt_structure(group_val = ".default", label_val = ".default",
                       frmt("xx", missing = " ")),
        frmt_structure(group_val = ".default", label_val = ".default",
                       frmt_combine("{mean} ({sd})",
                                    mean = frmt("xx.x"),
                                    sd = frmt("xx.xx"),
                                    missing=" ")),
        frmt_structure(group_val = ".default", label_val = ".default",
                       pval = frmt_when(">0.99" ~ ">0.99",
                                        "<0.001" ~ "<0.001",
                                        "<0.05" ~ frmt("x.xxx*"),
                                        TRUE ~ frmt("x.xxx", missing = "")))
      ),
      # col_style_structure() supplied to col_style_plan param instead of col_style_plan() funciton
      col_style_plan = col_style_structure(col = `p-value`, align = c("."), type = "char")
    ),
    "Invalid input supplied to the `col_style_plan` parameter. Please supply a `col_style_plan()`.",
    fixed=TRUE
  )
})


test_that("Testing error message for invalid input to big_n parameter",{
  expect_error(
    # create tfrmt object
    tfrmt_object <- tfrmt(
      group = Group,
      label = Label,
      column = Column,
      value = Value,
      param = Param,
      sorting_cols = c(ord1, ord2),
      body_plan = body_plan(
        frmt_structure(group_val = ".default",
                       label_val = ".default",
                       frmt_combine("{n} {pct}",
                                    n = frmt("X"),
                                    pct = frmt("(xx.x%)", missing = " ")
                       )
        ),
        frmt_structure(group_val = "Age (y)", label_val = "Mean (SD)",
                       frmt_combine("{mean} ({sd})",
                                    mean = frmt("XX.X"),
                                    sd = frmt("x.xx")
                       )
        ),
        frmt_structure(group_val = ".default", label_val = "n", frmt("xx"))
      ),
      col_plan = col_plan(
        everything(),
        -starts_with("ord"),
        "Total"
      ),
      row_grp_plan = row_grp_plan(
        row_grp_structure(group_val = ".default", element_block(post_space = " "))
      ),
      # col_plan() supplied to big_n parameter instead of big_n_structure() function
      big_n = col_plan()
    ),
    "Invalid input supplied to the `big_n` parameter. Please supply a `big_n_structure()`.",
    fixed=TRUE
  )
})

test_that("applying footnote meta column val",{

  # set up data

  es_data <-tibble::tibble(rowlbl1 =c(rep("Completion Status",12),rep("Primary reason for withdrawal",28)),
                           rowlbl2 =c(rep("Completed",4),rep("Adverse Event",4),rep("Unknown",4),rep("Adverse Event",4),rep("Lost to follow-up",4),rep("Protocol violation",4),rep("Subject decided to withdraw",4),rep("Protocol Violation",4),rep("Pre-Operative Dose[1]",4),rep("Other",4)),
                           param=c(rep(c("n","n","pct","pct"),10)),
                           trt=c(rep(c("Placebo","Treatment"),20)),
                           value=c(24,19,2400/48,1900/38,5,1,500/48,100/38,19,18,1900/48,1800/38,1,1,100/48,100/38,0,0,0,0,0,0,0,0,1,1,100/48,100/38,1,4,100/48,400/38,1,0,100/48,0,2,3,200/48,300/38)

  )

  # tfrmt with column val
  tfrmt<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = trt,
    param = param,
    value = value,
    # set formatting for value
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    # row_grp_plan = row_grp_plan(
    #   row_grp_structure(group_val = ".default", element_block(post_space = " ")),
    #   label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 1",column_val ="Placebo"),
      marks="letters"

    )
  )

  expect_equal(
    attr(apply_tfrmt(es_data,tfrmt),".footnote_locs"),
    list(list("col"="Placebo","spanning"=FALSE,"note"="Test footnote 1"))

  )
  # named column vals
  tfrmt2<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = trt,
    param = param,
    value = value,
    # set formatting for value
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 1",column_val =list(trt="Placebo")),
      marks="letters"

    )
  )

  expect_equal(
    attr(apply_tfrmt(es_data,tfrmt2),".footnote_locs"),
    list(list("col"="Placebo","spanning"=FALSE,"note"="Test footnote 1"))

  )

  # spanning
  es_data2<- es_data %>%
    mutate(col2 = "Treatment column")

  tfrmt3<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = c(col2,trt),
    param = param,
    value = value,
    # set formatting for value
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 3",column_val=list(col2 = "Treatment column")),
      marks="numbers"

    )
  )

  expect_equal(
    attr(apply_tfrmt(es_data2,tfrmt3),".footnote_locs"),
    list(list("col"="Treatment column","spanning"=TRUE,"note"="Test footnote 3"))

  )

  # spanned
  es_data3<-es_data2 %>%
    mutate(col2="Treatment column 2") %>%
    rbind(es_data2)

  tfrmt3<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = c(col2,trt),
    param = param,
    value = value,
    # set formatting for value
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 2",column_val=list(trt="Placebo")),
      marks="letters"

    )
  )

  expect_equal(
    attr(apply_tfrmt(es_data2,tfrmt3),".footnote_locs"),
    list(list("col"="Treatment column___tlang_delim___Placebo","spanning"=FALSE,"note"="Test footnote 2"))

  )


})


test_that("applying footnote meta group val",{

  # set up data

  es_data <-tibble::tibble(rowlbl1 =c(rep("Completion Status",12),rep("Primary reason for withdrawal",28)),
                           rowlbl2 =c(rep("Completed",4),rep("Adverse Event",4),rep("Unknown",4),rep("Adverse Event",4),rep("Lost to follow-up",4),rep("Protocol violation",4),rep("Subject decided to withdraw",4),rep("Protocol Violation",4),rep("Pre-Operative Dose[1]",4),rep("Other",4)),
                           param=c(rep(c("n","n","pct","pct"),10)),
                           trt=c(rep(c("Placebo","Treatment"),20)),
                           value=c(24,19,2400/48,1900/38,5,1,500/48,100/38,19,18,1900/48,1800/38,1,1,100/48,100/38,0,0,0,0,0,0,0,0,1,1,100/48,100/38,1,4,100/48,400/38,1,0,100/48,0,2,3,200/48,300/38)

  )


  # spanning
  es_data2<- es_data %>%
    mutate(col2 = "Treatment column")


  # test warnings

  tfrmt<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = c(col2,trt),
    param = param,
    value = value,
    # set formatting for value
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 3",column_val=list(col2 = "Treatment column"),group_val = "Completion Status"),
      marks="numbers"

    )
  )

  expect_warning(apply_tfrmt(es_data2,tfrmt),
                 "Cannot apply footnotes to rows when you have only specified a spanning column")

  tfrmt2<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = c(trt),
    param = param,
    value = value,
    # set formatting for value
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "noprint")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 3",group_val = "Completion Status"),
      marks="numbers"

    )
  )

  expect_warning(apply_tfrmt(es_data2,tfrmt2),
                 "Can not apply footnotes to group columns when 'noprint' is set"
                 )


  # footnote in group column

  tfrmt3<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = c(trt),
    param = param,
    value = value,
    # set formatting for value
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote",group_val = "Completion Status"),
      marks="letters"

    )
  )


  expect_equal(
    attr(apply_tfrmt(es_data2,tfrmt3),".footnote_locs"),
    list(list("col"="rowlbl1","spanning"=FALSE,"row" =1,"note"="Test footnote"))

  )

  # footnote in label column
  tfrmt4<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = c(trt),
    param = param,
    value = value,
    # set formatting for value
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    #Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "indented")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote",group_val = list(rowlbl1="Completion Status")),
      marks="letters"

    )
  )

  expect_equal(
    attr(apply_tfrmt(es_data2,tfrmt4),".footnote_locs"),
    list(list("col"="rowlbl2","spanning"=FALSE,"row" =1,"note"="Test footnote"))

  )

  # no row group plan

  es_data3<- es_data %>%
    mutate(rowlbl0="Test group")


  tfrmt5<-tfrmt(
    # specify columns in the data
    group = c(rowlbl0,rowlbl1),
    label = rowlbl2,
    column = trt,
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "gtdefault")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote",group_val=list(rowlbl0="Test group",rowlbl1="Completion Status"),
                         # label_val=list(rowlbl2="Adverse Event")
      ),
      marks="letters"
    )
  )


  expect_equal(
    attr(apply_tfrmt(es_data3,tfrmt5),".footnote_locs"),
    list(list("col"="rowlbl1","spanning"=FALSE,"row" =1,"note"="Test footnote"))
  )

  # 2 levels of grouping and row_grp_plan w/ spanning/column
  tfrmt6<-tfrmt(
    # specify columns in the data
    group = c(rowlbl0,rowlbl1),
    label = rowlbl2,
    column = trt,
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc(location = "spanning")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote")
    )
  )
  expect_equal(
    attr(apply_tfrmt(es_data3,tfrmt6),".footnote_locs"),
    list(list("col"=NULL,"spanning"=FALSE,"note"="Test footnote"))
  )
})


test_that("If 1 group/column var, can pass an unnamed vector",{


  # set up data
  dd <- tibble::tibble(
      rowlbl1 = "Completion Status",
      rowlbl2 = c("Completed", "Ongoing", "Unknown")) %>%
    bind_rows(
      tibble(
        rowlbl1 = "Primary reason for withdrawal",
        rowlbl2 = c("Other", "Lost to follow-up"))
      ) %>%
    crossing(
    param = c("n","pct"),
    trt = c("Placebo", "Trt1","Trt2","Trt3")
  ) %>%
    bind_cols(
       value=c(24,19,2400/48,1900/38,5,1,500/48,100/38,19,18,1900/48,1800/38,1,1,100/48,100/38,0,0,0,0,0,0,0,0,1,1,100/48,100/38,1,4,100/48,400/38,1,0,100/48,0,2,3,200/48,300/38)
  )

  # column_val locs

  tfrmt<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = trt,
    param = param,
    value = value,
    # set formatting for value
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 1",column_val = c("Trt1", "Trt2", "Trt3")),
      marks="letters"

    )
  )

  expect_equal(
    attr(apply_tfrmt(dd,tfrmt),".footnote_locs"),
    list(list("col"=c("Trt1","Trt2","Trt3"),"spanning"=FALSE,"note"="Test footnote 1"))

  )


  # group_val locs

  tfrmt<-tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = trt,
    param = param,
    value = value,
    # set formatting for value
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 1",group_val = c("Completion Status", "Primary reason for withdrawal")),
      marks="letters"

    ),
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(location = "spanning"))
  )

  expect_equal(
    attr(apply_tfrmt(dd,tfrmt),".footnote_locs"),
    list(list("col"="rowlbl1","spanning"=FALSE,"row"=c(1,2), note="Test footnote 1"))

  )


})

test_that("Display row formats for tfrmt with <frmt>", {

  df <- crossing(
    label = c("label 1", "label 2", "label 3"),
    column = c("PL", "T1", "T2"),
    param = c("count")) %>%
    dplyr::arrange_all() %>%
    mutate(value = seq(1:nrow(.)))

  my_tfrmt <- tfrmt(
    label = label,
    column = column,
    param = param,
    value = value,
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default",
                        element_block(post_space = "   ")) ),

    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default",
                     frmt("xx.x"))
    ))

  row_frmts_table_true <- display_row_frmts(tfrmt = my_tfrmt,
                                            .data = df,
                                            convert_to_txt = TRUE)
  row_frmts_table_false <- display_row_frmts(tfrmt = my_tfrmt,
                                             .data = df,
                                             convert_to_txt = FALSE)

  # expect error when convert_to_txt not Boolean
  expect_error(display_row_frmts(tfrmt = my_tfrmt, .data = df, convert_to_txt = "hello"),
               "Please pass a boolean value into the `convert_to_txt` parameter")

  # expect dataframe and tfrmt input
  expect_s3_class(my_tfrmt, "tfrmt")
  expect_equal(class(df), c("tbl_df", "tbl", "data.frame"))

  # expect dataframe output
  expect_equal(class(row_frmts_table_true), c("tbl_df", "tbl", "data.frame"))
  expect_equal(class(row_frmts_table_false), c("tbl_df", "tbl", "data.frame"))

})

test_that("Display row formats for tfrmt with <frmt> <frmt_combine>", {

  df <- bind_rows(
    crossing(
      label = c("label 1"),
      column = c("PL", "T1", "T2"),
      param = c("count")),
    crossing(
      label = c("label 2", "label 3"),
      column = c("PL", "T1", "T2"),
      param = c("count", "percent"))
  ) %>%
    dplyr::arrange_all() %>%
    mutate(value = seq(1:nrow(.)))

  my_tfrmt <- tfrmt(
    label = label,
    column = column,
    param = param,
    value = value,
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default",
                        element_block(post_space = "   ")) ),

    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = "label 1",
                     frmt("xxx")),
      frmt_structure(group_val = ".default", label_val = c("label 2", "label 3"),
                     frmt_combine("{count} ({percent})",
                                  count = frmt("xx"),
                                  percent = frmt("xx.x")))
    ))

  row_frmts_table_true <- display_row_frmts(tfrmt = my_tfrmt,
                                            .data = df,
                                            convert_to_txt = TRUE)
  row_frmts_table_false <- display_row_frmts(tfrmt = my_tfrmt,
                                             .data = df,
                                             convert_to_txt = FALSE)

  # expect error when convert_to_txt not Boolean
  expect_error(display_row_frmts(tfrmt = my_tfrmt, .data = df, convert_to_txt = "hello"),
               "Please pass a boolean value into the `convert_to_txt` parameter")

  # expect dataframe and tfrmt input
  expect_s3_class(my_tfrmt, "tfrmt")
  expect_equal(class(df), c("tbl_df", "tbl", "data.frame"))

  # expect dataframe output
  expect_equal(class(row_frmts_table_true), c("tbl_df", "tbl", "data.frame"))
  expect_equal(class(row_frmts_table_false), c("tbl_df", "tbl", "data.frame"))

})

test_that("Display row formats for tfrmt with <frmt> <frmt_combine> <frmt_when>", {

  df <- bind_rows(
    crossing(
      label = c("label 1"),
      column = c("PL", "T1", "T2"),
      param = c("n")),
    crossing(
      label = c("label 2"),
      column = c("PL", "T1", "T2"),
      param = c("median", "sd")),
    crossing(
      label = c("label 3"),
      column = c("PL", "T1", "T2"),
      param = c("pval"))
  ) %>%
    dplyr::arrange_all() %>%
    mutate(value = seq(1:nrow(.)))

  my_tfrmt <- tfrmt(
    label = label,
    column = column,
    param = param,
    value = value,

    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = "label 1",
                     frmt("xx")
      ),
      frmt_structure(group_val = ".default", label_val = "label 2",
                     frmt_combine("{median} ({sd})",
                                  median = frmt("xx"),
                                  sd = frmt("xx.x"))
      ),
      frmt_structure(group_val = ".default", label_val = "label 3",
                     frmt_when(">0.99" ~ ">0.99",
                               "<0.001" ~ "<0.001",
                               TRUE ~ frmt("x.xxx", missing = ""))
      )
    ))

  row_frmts_table_true <- display_row_frmts(tfrmt = my_tfrmt,
                                            .data = df,
                                            convert_to_txt = TRUE)
  row_frmts_table_false <- display_row_frmts(tfrmt = my_tfrmt,
                                             .data = df,
                                             convert_to_txt = FALSE)

  # expect error when convert_to_txt not Boolean
  expect_error(display_row_frmts(tfrmt = my_tfrmt, .data = df, convert_to_txt = "hello"),
               "Please pass a boolean value into the `convert_to_txt` parameter")

  # expect dataframe and tfrmt input
  expect_s3_class(my_tfrmt, "tfrmt")
  expect_equal(class(df), c("tbl_df", "tbl", "data.frame"))

  # expect dataframe output
  expect_equal(class(row_frmts_table_true), c("tbl_df", "tbl", "data.frame"))
  expect_equal(class(row_frmts_table_false), c("tbl_df", "tbl", "data.frame"))

})

test_that("Defining the big Ns", {
  bn1 <- big_n_structure(
    param_val = "bigN",
    n_frmt = frmt("Hello World")
  )
  bn2 <- big_n_structure(
    param_val = c("bigN", "N")
  )

  expect_s3_class(bn1, "big_n_structure")
  expect_s3_class(bn2, "big_n_structure")

  expect_equal(
    bn1[["param_val"]],
    c("bigN"),
    ignore_attr = ".Environment"
  )

  expect_equal(
    bn1[["n_frmt"]],
    frmt("Hello World"),
    ignore_attr = ".Environment"
  )

  expect_equal(
    bn2[["param_val"]],
    c("bigN", "N"),
    ignore_attr = ".Environment"
  )

  expect_equal(
    bn2[["n_frmt"]],
    frmt("\nN = xx"),
    ignore_attr = ".Environment"
  )

  expect_error(
    big_n_structure(
      param_val = "bigN",
      n_frmt = "Hello World"
    )
  )

  expect_error(
    big_n_structure(
      param_val = "bigN",
      n_frmt = frmt_when(
        ">3" ~ frmt("(X.X%)"),
        "<=3" ~ frmt("Undetectable")
      )
    )
  )
})

test_that("Simple Case big_n", {

  data <- tibble::tibble(
    Group = rep(
      c("Age (y)", "Sex", "Age (y)", "Sex"),
      c(3, 3, 6, 12)
    ),
    Label = rep(
      c("n", "Mean (SD)", "Male", "Female"),
      c(6, 6, 6, 6)
    ),
    Column = rep(
      c("Placebo", "Treatment", "Total"),
      times = 8
    ),
    Param = rep(
      c("n", "mean", "sd", "n", "pct", "n", "pct"),
      c(6, 3, 3, 3, 3, 3, 3)
    ),
    Value = c(
      15, 13, 28, 14, 13, 27, 73.56, 74.231, 71.84, 9.347, 7.234, 8.293, 8, 7,
      15, 8 / 14, 7 / 13, 15 / 27, 6, 6, 12, 6 / 14, 6 / 13, 12 / 27
    )
  ) |>
    # Note because tfrmt only does rounding we will need to have the percents
    # multiplied by 100
    dplyr::mutate(
      Value = dplyr::case_when(
        Param == "pct" ~ Value * 100,
        TRUE ~ Value
      ),
      ord1 = dplyr::if_else(
        Group == "Age (y)",
        1,
        2
      ),
      ord2 = dplyr::if_else(
        Label == "n",
        1,
        2
      )
    )

  big_ns <- tibble::tibble(
    Column = c("Placebo", "Treatment", "Total"),
    Param = "bigN",
    Value = c(30, 30, 60)
  )


  data <- dplyr::bind_rows(data, big_ns)

  tfrmt_sans_colplan <- tfrmt(
    group = Group,
    label = Label,
    column = Column,
    value = Value,
    param = Param,
    sorting_cols = c(ord1, ord2),
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{n} {pct}",
          n = frmt("X"),
          pct = frmt(
            "(xx.x%)",
            missing = " "
          )
        )
      ),
      frmt_structure(
        group_val = "Age (y)",
        label_val = "Mean (SD)",
        frmt_combine(
          "{mean} ({sd})",
          mean = frmt("XX.X"),
          sd = frmt("x.xx")
        )
      ),
      frmt_structure(
        group_val = ".default",
        label_val = "n",
        frmt("xx")
      )
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = ".default",
        element_block(
          post_space = " "
        )
      )
    ),
    big_n = big_n_structure(
      param_val = "bigN"
    )
  )

  tfrmt_wit_colplan <- tfrmt_sans_colplan |>
    tfrmt(
      col_plan = col_plan(
        everything(),
        -starts_with("ord"),
        "Total"
      )
    )

  auto_sans_colplan <- tfrmt_sans_colplan |>
    apply_tfrmt(
      .data = data,
      tfrmt = _,
      mock = FALSE
    ) |>
    names()

  expect_identical(
    auto_sans_colplan,
    c(
      "Label",
      "ord1",
      "ord2",
      "Placebo\nN = 30",
      "Treatment\nN = 30",
      "Total\nN = 60",
      "..tfrmt_row_grp_lbl"
    )
  )

  auto_wit_colplan <- tfrmt_wit_colplan |>
    apply_tfrmt(
      .data = data,
      tfrmt = _,
      mock = FALSE
    ) |>
    names()

  expect_identical(
    auto_wit_colplan,
    c(
      "Label",
      "Placebo\nN = 30",
      "Treatment\nN = 30",
      "Total\nN = 60",
      "..tfrmt_row_grp_lbl"
    )
  )

  auto_mock <- apply_tfrmt(
    .data = select(data, -Value),
    tfrmt = tfrmt_wit_colplan,
    mock = TRUE
  ) |>
    names()

  expect_identical(
    auto_mock,
    c(
      "Label",
      "Placebo\nN = xx",
      "Treatment\nN = xx",
      "Total\nN = xx",
      "..tfrmt_row_grp_lbl"
    )
  )
})

test_that("Test with spanning headers", {

  dat <- tibble::tribble(
    ~group,     ~label,        ~span2,  ~span1,     ~my_col,    ~parm,  ~val,
    "g1", "rowlabel1",  "column cols", "cols 1,2",   "col1",  "value",    1,
    "g1", "rowlabel1",  "column cols", "cols 1,2",   "col2",  "value",    1,
    "g1", "rowlabel1",             NA,         NA, "mycol3",  "value",    1,
    "g1", "rowlabel1",  "column cols",    "col 4",   "col4",  "value",    1,
    "g1", "rowlabel1",             NA,         NA, "mycol5",  "value",    1,
    "g1", "rowlabel2",  "column cols", "cols 1,2",   "col1",  "value",    2,
    "g1", "rowlabel2",  "column cols", "cols 1,2",   "col2",  "value",    2,
    "g1", "rowlabel2",             NA,         NA, "mycol3",  "value",    2,
    "g1", "rowlabel2",  "column cols",    "col 4",   "col4",  "value",    2,
    "g1", "rowlabel2",             NA,         NA, "mycol5",  "value",    2,
    "g2", "rowlabel3",  "column cols", "cols 1,2",   "col1",  "value",    3,
    "g2", "rowlabel3",  "column cols", "cols 1,2",   "col2",  "value",    3,
    "g2", "rowlabel3",             NA,         NA, "mycol3",  "value",    3,
    "g2", "rowlabel3",  "column cols",    "col 4",   "col4",  "value",    3,
    "g2", "rowlabel3",             NA,         NA, "mycol5",  "value",    3,
    # big n's
    NA, NA,  "column cols",         NA,       NA,  "bigN",    18,
    NA, NA,  "column cols", "cols 1,2",       NA,  "bigN",    12,
    NA, NA,  "column cols",    "col 4",   "col4",  "bigN",     6,
    NA, NA,             NA,         NA, "mycol3",  "bigN",     6,
  )

  auto_tfrmt <- tfrmt(
    group = group,
    label = label,
    param = parm,
    value = val,
    column = c(span2, span1, my_col),
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("x")
      )
    ),
    col_plan = col_plan(
      group,
      label,
      starts_with("col"),
      new_col_3 = mycol3,
      -mycol5
    ),
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(
        location = "spanning"
      )
    ),
    big_n = big_n_structure(
      param_val = "bigN"
    )
  )

  auto <- apply_tfrmt(
    .data = dat,
    tfrmt = auto_tfrmt
  ) |>
    names()

  man <- c(
    "group",
    "label",
    "column cols\nN = 18___tlang_delim___cols 1,2\nN = 12___tlang_delim___col1",
    "column cols\nN = 18___tlang_delim___cols 1,2\nN = 12___tlang_delim___col2",
    "column cols\nN = 18___tlang_delim___col 4___tlang_delim___col4\nN =  6",
    "new_col_3\nN =  6",
    "..tfrmt_row_grp_lbl"
  )

  expect_identical(auto, man)

  # try with empty strings rather than NA
  dat_blank <- dat |>
    dplyr::mutate(
      dplyr::across(
        tidyselect::where(is.character),
        ~ tidyr::replace_na(.x, "")
      )
    )

  auto_blank <- apply_tfrmt(
    .data = dat_blank,
    tfrmt = auto_tfrmt
  ) |>
    names()

  expect_identical(auto_blank, man)
})

test_that("Multiple big N params", {

  data <- tibble::tibble(
    Group = rep(
      c("Age (y)", "Sex", "Age (y)", "Sex"),
      c(3, 3, 6, 12)
    ),
    Label = rep(
      c("n", "Mean (SD)", "Male", "Female"),
      c(6, 6, 6, 6)
    ),
    Column = rep(
      c("Placebo", "Treatment", "Total"),
      times = 8
    ),
    Param = rep(
      c("n", "mean", "sd", "n", "pct", "n", "pct"),
      c(6, 3, 3, 3, 3, 3, 3)
    ),
    Value = c(
      15, 13, 28, 14, 13, 27, 73.56, 74.231, 71.84, 9.347, 7.234, 8.293, 8, 7,
      15, 8 / 14, 7 / 13, 15 / 27, 6, 6, 12, 6 / 14, 6 / 13, 12 / 27
    )
  ) |>
    # Note because tfrmt only does rounding we will need to have the percents
    # multiplied by 100
    dplyr::mutate(
      Value = dplyr::case_when(
        Param == "pct" ~ Value * 100,
        TRUE ~ Value
      ),
      ord1 = dplyr::if_else(
        Group == "Age (y)",
        1,
        2
      ),
      ord2 = dplyr::if_else(
        Label == "n",
        1,
        2
      )
    )

  big_ns <- tibble::tibble(
    Column = c("Placebo", "Treatment", "Total"),
    Param = c("bigN", "big_n", "big_n"),
    Value = c(30, 30, 60)
  )


  data <- dplyr::bind_rows(data, big_ns)

  auto <- tfrmt(
    group = Group,
    label = Label,
    column = Column,
    value = Value,
    param = Param,
    sorting_cols = c(ord1, ord2),
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{n} {pct}",
          n = frmt("X"),
          pct = frmt(
            "(xx.x%)",
            missing = " "
          )
        )
      ),
      frmt_structure(
        group_val = "Age (y)",
        label_val = "Mean (SD)",
        frmt_combine(
          "{mean} ({sd})",
          mean = frmt("XX.X"),
          sd = frmt("x.xx")
        )
      ),
      frmt_structure(
        group_val = ".default",
        label_val = "n",
        frmt("xx")
      )
    ),
    col_plan = col_plan(
      everything(),
      -starts_with("ord"),
      "Total"
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = ".default",
        element_block(
          post_space = " "
        )
      )
    ),
    big_n = big_n_structure(
      param_val = c("bigN", "big_n")
    )
  ) |>
    apply_tfrmt(
      .data = data,
      tfrmt = _,
      mock = FALSE
    ) |>
    names()

  expect_identical(
    auto,
    c(
      "Label",
      "Placebo\nN = 30",
      "Treatment\nN = 30",
      "Total\nN = 60",
      "..tfrmt_row_grp_lbl"
    )
  )
})

test_that("Overlapping Big N's", {

  data <- tibble::tibble(
    Group = rep(
      c("Age (y)", "Sex", "Age (y)", "Sex"),
      c(3, 3, 6, 12)
    ),
    Label = rep(
      c("n", "Mean (SD)", "Male", "Female"),
      c(6, 6, 6, 6)
    ),
    Column = rep(
      c("Placebo", "Treatment", "Total"),
      times = 8
    ),
    Param = rep(
      c("n", "mean", "sd", "n", "pct", "n", "pct"),
      c(6, 3, 3, 3, 3, 3, 3)
    ),
    Value = c(
      15, 13, 28, 14, 13, 27, 73.56, 74.231, 71.84, 9.347, 7.234, 8.293, 8, 7,
      15, 8 / 14, 7 / 13, 15 / 27, 6, 6, 12, 6 / 14, 6 / 13, 12 / 27
    )
  ) |>
    # Note because tfrmt only does rounding we will need to have the percents
    # multiplied by 100
    dplyr::mutate(
      Value = dplyr::case_when(
        Param == "pct" ~ Value * 100,
        TRUE ~ Value
      ),
      ord1 = dplyr::if_else(
        Group == "Age (y)",
        1,
        2
      ),
      ord2 = dplyr::if_else(
        Label == "n",
        1,
        2
      )
    )

  big_ns <- tibble::tibble(
    Column = c("Placebo", "Treatment", "Total", "Total"),
    Param = "bigN",
    Value = c(30, 30, 60, 65)
  )

  data <- dplyr::bind_rows(data, big_ns)

  tfrmt_test <- tfrmt(
    group = Group,
    label = Label,
    column = Column,
    value = Value,
    param = Param,
    sorting_cols = c(ord1, ord2),
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{n} {pct}",
          n = frmt("X"),
          pct = frmt(
            "(xx.x%)",
            missing = " "
          )
        )
      ),
      frmt_structure(
        group_val = "Age (y)",
        label_val = "Mean (SD)",
        frmt_combine(
          "{mean} ({sd})",
          mean = frmt("XX.X"),
          sd = frmt("x.xx")
        )
      ),
      frmt_structure(
        group_val = ".default",
        label_val = "n",
        frmt("xx")
      )
    ),
    col_plan = col_plan(
      everything(),
      -starts_with("ord"),
      "Total"
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = ".default",
        element_block(post_space = " ")
      )
    ),
    big_n = big_n_structure(
      param_val = "bigN"
    )
  )

  expect_warning(
    apply_tfrmt(
      .data = data,
      tfrmt = tfrmt_test,
      mock = FALSE
    ) |>
      names()
  )
})

test_that("Missing Big N in dataset", {
  dat <- tibble::tribble(
    ~group,     ~label,        ~span2,  ~span1,     ~my_col,    ~parm,   ~val,
    "g1", "rowlabel1",  "column cols", "cols 1,2",   "col1",  "value",    1,
    "g1", "rowlabel1",  "column cols", "cols 1,2",   "col2",  "value",    1,
    "g1", "rowlabel1",             NA,         NA, "mycol3",  "value",    1,
    "g1", "rowlabel1",  "column cols",    "col 4",   "col4",  "value",    1,
    "g1", "rowlabel1",             NA,         NA, "mycol5",  "value",    1,
    "g1", "rowlabel2",  "column cols", "cols 1,2",   "col1",  "value",    2,
    "g1", "rowlabel2",  "column cols", "cols 1,2",   "col2",  "value",    2,
    "g1", "rowlabel2",             NA,         NA, "mycol3",  "value",    2,
    "g1", "rowlabel2",  "column cols",    "col 4",   "col4",  "value",    2,
    "g1", "rowlabel2",             NA,         NA, "mycol5",  "value",    2,
    "g2", "rowlabel3",  "column cols", "cols 1,2",   "col1",  "value",    3,
    "g2", "rowlabel3",  "column cols", "cols 1,2",   "col2",  "value",    3,
    "g2", "rowlabel3",             NA,         NA, "mycol3",  "value",    3,
    "g2", "rowlabel3",  "column cols",    "col 4",   "col4",  "value",    3,
    "g2", "rowlabel3",             NA,         NA, "mycol5",  "value",    3,
  )

  tfrmt_test <- tfrmt(
    group = group,
    label = label,
    param = parm,
    value = val,
    column = c(span2, span1, my_col),
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("x")
      )
    ),
    col_plan = col_plan(
      group,
      label,
      starts_with("col")
    ),
    big_n = big_n_structure(
      param_val = "bigN"
    )
  )

  expect_warning(
    apply_tfrmt(
      .data = dat,
      tfrmt = tfrmt_test,
      mock = FALSE
    )
  )
})

test_that("using 'value' for values column where there may be conflict in big_n", {

  dat <- tibble::tribble(
    ~group,     ~label,     ~span2,      ~span1,    ~my_col,  ~parm,   ~value,
    "g1", "rowlabel1",  "column cols", "cols 1,2",   "col1",  "value",    1,
    "g1", "rowlabel1",  "column cols", "cols 1,2",   "col2",  "value",    1,
    "g1", "rowlabel1",             NA,         NA, "mycol3",  "value",    1,
    "g1", "rowlabel1",  "column cols",    "col 4",   "col4",  "value",    1,
    "g1", "rowlabel1",             NA,         NA, "mycol5",  "value",    1,
    "g1", "rowlabel2",  "column cols", "cols 1,2",   "col1",  "value",    2,
    "g1", "rowlabel2",  "column cols", "cols 1,2",   "col2",  "value",    2,
    "g1", "rowlabel2",             NA,         NA, "mycol3",  "value",    2,
    "g1", "rowlabel2",  "column cols",    "col 4",   "col4",  "value",    2,
    "g1", "rowlabel2",             NA,         NA, "mycol5",  "value",    2,
    "g2", "rowlabel3",  "column cols", "cols 1,2",   "col1",  "value",    3,
    "g2", "rowlabel3",  "column cols", "cols 1,2",   "col2",  "value",    3,
    "g2", "rowlabel3",             NA,         NA, "mycol3",  "value",    3,
    "g2", "rowlabel3",  "column cols",    "col 4",   "col4",  "value",    3,
    "g2", "rowlabel3",             NA,         NA, "mycol5",  "value",    3,
    #big n's
    NA, NA,  "column cols",         NA,       NA, "bigN", 18,
    NA, NA,  "column cols", "cols 1,2",       NA, "bigN", 12,
    NA, NA,  "column cols",    "col 4",   "col4", "bigN",  6,
    NA, NA,             NA,         NA, "mycol3", "bigN",  6,
  )

  auto <- tfrmt(
    group = group,
    label = label,
    param = parm,
    value = value,
    column = c(span2, span1, my_col),
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("x")
      )
    ),
    col_plan = col_plan(
      group,
      label,
      starts_with("col"),
      new_col_3 = mycol3,
      -mycol5
    ),
    big_n = big_n_structure(
      param_val = "bigN"
    ),
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(
        location = "spanning"
      )
    )
  ) |>
    apply_tfrmt(
      .data = dat,
      tfrmt = _
    ) |>
    names()

  man <- c(
    "group",
    "label",
    "column cols\nN = 18___tlang_delim___cols 1,2\nN = 12___tlang_delim___col1",
    "column cols\nN = 18___tlang_delim___cols 1,2\nN = 12___tlang_delim___col2",
    "column cols\nN = 18___tlang_delim___col 4___tlang_delim___col4\nN =  6",
    "new_col_3\nN =  6",
    "..tfrmt_row_grp_lbl"
  )

  expect_identical(auto, man)
})

test_that("Test big n with footnotes", {

  # Create mock data
  df <- tidyr::crossing(
    group = c("group 1", "group 2"),
    label = c("label 1", "label 2"),
    column = c("PL", "T1", "T2", "T1&T2"),
    param = c("count", "percent")
  )

  # This one is used for examples 5 and 6
  span_df <- df |>
    dplyr::mutate(
      span = dplyr::case_when(
        column == "PL" ~ "Placebo",
        column %in% c("T1", "T2", "T1&T2") == TRUE ~ "Treatment"
      )
    )

  span_df_big_n <- dplyr::bind_rows(
    span_df,
    tibble::tibble(
      group = NA,
      label = NA,
      column = c("T1", "T2", "T1&T2"),
      param = "big_n",
      span = NA
    )
  )

  # Add specification
  big_n_footnote_plan_gt <- tfrmt(
    value = fake_value,
    group = group,
    label = label,
    column = c("span", "column"),
    param = param,
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = ".default",
        element_block(post_space = "   ")
      )
    ),
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{count} ({percent})",
          count = frmt("xx"),
          percent = frmt("xx.x")
        )
      )
    ),
    col_plan = col_plan(
      group,
      label,
      span_structure(
        span = c("Placebo"),
        column = c("PL")
      ),
      span_structure(
        span = c("Treatment"),
        column = c("T1", "T2", "T1&T2")
      )
    ),

    # Add footnote here
    footnote_plan = footnote_plan(
      footnote_structure(
        footnote_text = "Footnote goes here",
        column_val = list(
          span = "Treatment",
          column = "T1&T2"
        )
      ),
      footnote_structure(
        footnote_text = "Footnote goes here 2",
        group_val = list(group = "group 1"),
        label_val = list(label = "label 1"),
        column_val = list(
          span = "Treatment",
          column = "T1"
        )
      ),
      footnote_structure(
        footnote_text = "Footnote goes here 3",
        label_val = list(label = "label 1"),
        column_val = list(
          span = "Treatment",
          column = "T1"
        )
      ),
      footnote_structure(
        footnote_text = "Footnote goes here 4",
        label_val = list(label = "label 1"),
      ),
      footnote_structure(
        footnote_text = "Footnote goes here 5",
        group_val = list(group = "group 1")
      ),
      footnote_structure(
        footnote_text = "Footnote goes here 6",
        label_val = list(label = "label 1"),
        column_val = list(
          span = "Treatment",
          column = c("T2")
        )
      )
    ),
    big_n = big_n_structure(
      param_val = "big_n"
    )
  ) |>
    print_mock_gt(span_df_big_n)


  ## ensure big_n got applied
  expect_identical(
    names(big_n_footnote_plan_gt$`_data`),
    c(
      "label",
      "Placebo___tlang_delim___PL",
      "Treatment___tlang_delim___T1\nN = xx",
      "Treatment___tlang_delim___T2\nN = xx",
      "Treatment___tlang_delim___T1&T2\nN = xx",
      "..tfrmt_row_grp_lbl"
    )
  )

  ## confirm location of footnotes gets recorded correctly
  expect_identical(
    big_n_footnote_plan_gt$`_footnotes` |>
      select(
        locname,
        colname,
        locnum,
        rownum,
        footnotes
      ) |>
      as_tibble(),
    tibble::tibble(
      locname = c(
        "columns_columns",
        "data",
        "data",
        "data",
        "stub",
        "stub",
        "stub",
        "data",
        "data"
      ),
      colname = c(
        "Treatment___tlang_delim___T1&T2\nN = xx",
        "Treatment___tlang_delim___T1\nN = xx",
        "Treatment___tlang_delim___T1\nN = xx",
        "Treatment___tlang_delim___T1\nN = xx",
        "label",
        "label",
        "label",
        "Treatment___tlang_delim___T2\nN = xx",
        "Treatment___tlang_delim___T2\nN = xx"
      ),
      locnum = c(4, 5, 5, 5, 5, 5, 5, 5, 5),
      rownum = c(NA, 2L, 2L, 6L, 2L, 6L, 1L, 2L, 6L),
      footnotes = list(
        "Footnote goes here",
        "Footnote goes here 2",
        "Footnote goes here 3",
        "Footnote goes here 3",
        "Footnote goes here 4",
        "Footnote goes here 4",
        "Footnote goes here 5",
        "Footnote goes here 6",
        "Footnote goes here 6"
      )
    )
  )
})

test_that("big Ns vary by page", {

  data <- tibble::tibble(
    Group = rep(c("Age (y)", "Sex"), c(3, 3)),
    Label = rep("n", 6),
    Column = rep(c("Placebo", "Treatment", "Total"), times = 2),
    Param = rep("n", 6),
    Value = c(12, 14, 31, 20, 32, 18)
  ) |>
    dplyr::mutate(
      ord1 = dplyr::if_else(
        Group == "Age (y)",
        1,
        2
      )
    )

  big_ns <- data |>
    dplyr::summarise(
      .by = c(Group, Column),
      Value = sum(Value)
    ) |>
    dplyr::mutate(
      Param = "big_N"
    )

  data <- bind_rows(data, big_ns)

  mytfrmt <- tfrmt(
    group = Group,
    label = Label,
    column = Column,
    value = Value,
    param = Param,
    sorting_cols = ord1,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("xx")
      )
    ),
    col_plan = col_plan(
      everything(),
      -starts_with("ord"),
      "Total"
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = ".default",
        element_block(post_space = " ")
      )
    ),
    page_plan = page_plan(
      page_structure(
        group_val = ".default"
      )
    ),
    big_n = big_n_structure(
      param_val = c("big_N"),
      by_page = TRUE
    )
  )

  auto <- mytfrmt |>
    apply_tfrmt(
      .data = data,
      tfrmt = _,
      mock = FALSE
    )

  auto_names <- map(auto, names)
  man_names <- list(
    c(
      "Label",
      "Placebo\nN = 12",
      "Treatment\nN = 14",
      "Total\nN = 31",
      "..tfrmt_row_grp_lbl"
    ),
    c(
      "Label",
      "Placebo\nN = 20",
      "Treatment\nN = 32",
      "Total\nN = 18",
      "..tfrmt_row_grp_lbl"
    )
  )
  expect_identical(auto_names, man_names)
})

test_that("big Ns constant by page", {

  # big Ns are constant by page
  data <- tibble::tibble(
    Group = rep(c("Age (y)", "Sex"), c(3, 3)),
    Label = rep("n", 6),
    Column = rep(c("Placebo", "Treatment", "Total"), times = 2),
    Param = rep("n", 6),
    Value = c(12, 14, 31, 20, 32, 18)
  ) |>
    dplyr::mutate(
      ord1 = dplyr::if_else(
        Group == "Age (y)",
        1,
        2
      )
    )

  big_ns <- data |>
    dplyr::summarise(
      .by = c(Column),
      Value = sum(Value)
    ) |>
    dplyr::mutate(
      Param = "big_N"
    )

  data <- bind_rows(data, big_ns)

  mytfrmt <- tfrmt(
    group = Group,
    label = Label,
    column = Column,
    value = Value,
    param = Param,
    sorting_cols = ord1,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("xx")
      )
    ),
    col_plan = col_plan(
      everything(),
      -starts_with("ord"),
      "Total"
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = ".default",
        element_block(post_space = " ")
      )
    ),
    page_plan = page_plan(
      page_structure(
        group_val = ".default"
      )
    ),
    big_n = big_n_structure(
      param_val = c("big_N"),
      by_page = FALSE
    )
  )

  auto <- mytfrmt |>
    apply_tfrmt(
      .data = data,
      tfrmt = _,
      mock = FALSE
    )

  expect_identical(
    map(auto, names),
    list(
      c(
        "Label",
        "Placebo\nN = 32",
        "Treatment\nN = 46",
        "Total\nN = 49",
        "..tfrmt_row_grp_lbl"
      ),
      c(
        "Label",
        "Placebo\nN = 32",
        "Treatment\nN = 46",
        "Total\nN = 49",
        "..tfrmt_row_grp_lbl"
      )
    )
  )

  # big Ns constant by page but by_page=TRUE in the big_n_structure gives
  # message
  mytfrmt <- tfrmt(
    group = Group,
    label = Label,
    column = Column,
    value = Value,
    param = Param,
    sorting_cols = ord1,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("xx")
      )
    ),
    col_plan = col_plan(
      everything(),
      -starts_with("ord"),
      "Total"
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = ".default",
        element_block(post_space = " ")
      )
    ),
    page_plan = page_plan(
      page_structure(
        group_val = ".default"
      )
    ),
    big_n = big_n_structure(
      param_val = c("big_N"),
      by_page = TRUE
    )
  )

  expect_snapshot(
    auto <- mytfrmt |>
      apply_tfrmt(
        .data = data,
        tfrmt = _,
        mock = FALSE
      )
  )

  expect_identical(
    map(auto, names),
    list(
      c(
        "Label",
        "Placebo\nN = 32",
        "Treatment\nN = 46",
        "Total\nN = 49",
        "..tfrmt_row_grp_lbl"
      ),
      c(
        "Label",
        "Placebo\nN = 32",
        "Treatment\nN = 46",
        "Total\nN = 49",
        "..tfrmt_row_grp_lbl"
      )
    )
  )

  # too many big Ns provided with by_page = FALSE
  data <- tibble::tibble(
    Group = rep(c("Age (y)", "Sex"), c(3, 3)),
    Label = rep("n", 6),
    Column = rep(c("Placebo", "Treatment", "Total"), times = 2),
    Param = rep("n", 6),
    Value = c(12, 14, 31, 20, 32, 18)
  ) |>
    dplyr::mutate(
      ord1 = dplyr::if_else(
        Group == "Age (y)",
        1,
        2
      )
    )

  big_ns <- data |>
    dplyr::summarise(
      .by = c(Group, Column),
      Value = sum(Value)
    ) |>
    dplyr::mutate(
      Param = "big_N"
    )

  data <- bind_rows(data, big_ns)

  mytfrmt <- tfrmt(
    group = Group,
    label = Label,
    column = Column,
    value = Value,
    param = Param,
    sorting_cols = ord1,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("xx")
      )
    ),
    col_plan = col_plan(
      everything(),
      -starts_with("ord"),
      "Total"
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = ".default",
        element_block(post_space = " ")
      )
    ),
    page_plan = page_plan(
      page_structure(
        group_val = ".default"
      )
    ),
    big_n = big_n_structure(
      param_val = c("big_N"),
      by_page = FALSE
    )
  )

  expect_snapshot_warning(
    mytfrmt |>
      apply_tfrmt(
        .data = data,
        tfrmt = _,
        mock = FALSE
      )
  )
})


test_that("not enough big Ns by page", {

  # big Ns are constant by page
  data <- tibble::tibble(
    Group = rep(c("Age (y)", "Sex"), c(3, 3)),
    Label = rep("n", 6),
    Column = rep(c("Placebo", "Treatment", "Total"), times = 2),
    Param = rep("n", 6),
    Value = c(12, 14, 31, 20, 32, 18)
  ) |>
    dplyr::mutate(
      ord1 = dplyr::if_else(
        Group == "Age (y)",
        1,
        2
      )
    )

  #remove sex bigNs so we have a mismatch of big ns
  big_ns <- data |>
    dplyr::filter(
      Group == "Age (y)"
    ) |>
    dplyr::summarise(
      .by = c(Column, Group),
      Value = sum(Value)
    ) |>
    dplyr::mutate(
      Param = "big_N"
    )

  data <- bind_rows(data, big_ns)

  mytfrmt <- tfrmt(
    group = Group,
    label = Label,
    column = Column,
    value = Value,
    param = Param,
    sorting_cols = ord1,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("xx")
      )
    ),
    col_plan = col_plan(
      everything(),
      -starts_with("ord"),
      "Total"
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = ".default",
        element_block(post_space = " ")
      )
    ),
    page_plan = page_plan(
      page_structure(
        group_val = ".default"
      )
    ),
    big_n = big_n_structure(
      param_val = c("big_N"),
      by_page = TRUE
    )
  )

  expect_snapshot(
    mytfrmt |>
      apply_tfrmt(
        .data = data,
        tfrmt = _,
        mock = FALSE
      )
  )
})


test_that("Paging (group) variable is sorted non-alphabetically", {

  # same sorting of data and big Ns
  data <- tibble::tibble(
    Group = rep(c("Age (y)", "Sex"), c(3, 3)),
    Label = rep("n", 6),
    Column = rep(c("Placebo", "Treatment", "Total"), times = 2),
    Param = rep("n", 6),
    Value = c(12, 14, 31, 20, 32, 18)
  ) |>
    dplyr::mutate(
      ord1 = dplyr::if_else(
        Group == "Age (y)",
        1,
        2
      )
    )

  big_ns <- data |>
    dplyr::summarise(
      .by = c(Column, Group),
      Value = sum(Value)
    ) |>
    dplyr::mutate(
      Param = "big_N"
    )

  data <- bind_rows(data, big_ns) |>
    dplyr::arrange(
      dplyr::desc(Group)
    )

  mytfrmt <- tfrmt(
    group = Group,
    label = Label,
    column = Column,
    value = Value,
    param = Param,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("xx")
      )
    ),
    col_plan = col_plan(
      everything(),
      -starts_with("ord"),
      "Total"
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = ".default",
        element_block(post_space = " ")
      )
    ),
    page_plan = page_plan(
      page_structure(
        group_val = list(
          Group = ".default"
        )
      )
    ),
    big_n = big_n_structure(
      param_val = "big_N",
      by_page = TRUE
    )
  )

  auto <- mytfrmt |>
    apply_tfrmt(
      .data = data,
      tfrmt = _,
      mock = FALSE
    )

  # check that the labels have printed in the changed order
  expect_identical(
    # This extracts the first value of the "Label" column from each element
    map(auto, ~ .x[["Label"]][1]),
    list(
      "Sex",
      "Age (y)"
    )
  )

  # check big Ns have been correctly applied
  expect_identical(
    map(auto, names),
    list(
      c(
        "Label",
        "Placebo\nN = 20",
        "Treatment\nN = 32",
        "Total\nN = 18",
        "..tfrmt_row_grp_lbl"
      ),
      c(
        "Label",
        "Placebo\nN = 12",
        "Treatment\nN = 14",
        "Total\nN = 31",
        "..tfrmt_row_grp_lbl"
      )
    )
  )

  # different sorting of data and big Ns
  data <- tibble::tibble(
    Group = rep(c("Age (y)", "Sex"), c(3, 3)),
    Label = rep("n", 6),
    Column = rep(c("Placebo", "Treatment", "Total"), times = 2),
    Param = rep("n", 6),
    Value = c(12, 14, 31, 20, 32, 18)
  ) |>
    dplyr::mutate(
      ord1 = dplyr::if_else(
        Group == "Age (y)",
        1,
        2
      )
    ) |>
    dplyr::arrange(
      dplyr::desc(Group)
    )

  big_ns <- data |>
    dplyr::summarise(
      .by = c(Column, Group),
      Value = sum(Value)
    ) |>
    dplyr::mutate(
      Param = "big_N"
    ) |>
    dplyr::arrange(
      Group
    )

  data <- bind_rows(data, big_ns)

  mytfrmt <- tfrmt(
    group = Group,
    label = Label,
    column = Column,
    value = Value,
    param = Param,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("xx")
      )
    ),
    col_plan = col_plan(
      everything(),
      -starts_with("ord"),
      "Total"
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = ".default",
        element_block(post_space = " ")
      )
    ),
    page_plan = page_plan(
      page_structure(
        group_val = list(
          Group = ".default"
        )
      )
    ),
    big_n = big_n_structure(
      param_val = "big_N",
      by_page = TRUE
    )
  )

  expect_snapshot(
    mytfrmt |>
      apply_tfrmt(
        .data = data,
        tfrmt = _,
        mock = FALSE
      )
  )
})


test_that("Two grouping variables with a page_plan work as expected (renamed variables with a space)", {

  # Create the original data frame
  original_data <- tibble::tibble(
    Group = rep(c("Age (y)", "Sex"), c(3, 3)),
    Label = rep("n", 6),
    Column = rep(c("Placebo", "Treatment", "Total"), times = 2),
    Param = rep("n", 6),
    Value = c(12, 14, 31, 20, 32, 18)
  ) |>
    dplyr::mutate(
      ord1 = dplyr::if_else(
        Group == "Age (y)",
        1,
        2
      )
    )

  # Duplicate the data and add the `by group` column
  data_101 <- original_data |> dplyr::mutate(`by group` = "101")
  data_102 <- original_data |> dplyr::mutate(`by group` = "102")

  # Combine the two data frames
  data <- bind_rows(data_101, data_102)

  # Create mock big Ns
  big_ns <- data |>
    dplyr::group_by(
      `by group`,
      Column
    ) |>
    dplyr::summarise(
      Value = sum(Value)
    ) |>
    dplyr::mutate(
      Param = "bigN"
    ) |>
    dplyr::filter(
      `by group` == "101"
    ) |>
    dplyr::ungroup()

  data <- dplyr::bind_rows(data, big_ns) |>
    dplyr::arrange(
      dplyr::desc(Group)
    )

  # Define the tfrmt object with two grouping variables and a page_plan
  tfrmt_two_groups <- tfrmt(
    group = c(`by group`, Group),
    label = Label,
    column = Column,
    value = Value,
    param = Param,
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = "n",
        frmt("xx")
      )
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = ".default",
        element_block(
          post_space = " "
        )
      )
    ),
    page_plan = page_plan(
      page_structure(
        group_val = list(
          `by group` = ".default"
        )
      ),
      note_loc = "subtitle"
    ),
    big_n = big_n_structure(
      param_val = "bigN"
    ),
    col_plan = col_plan(
      -`by group`
    )
  )

  # Apply the tfrmt and check the output for each page
  output_list <- tfrmt_two_groups |>
    apply_tfrmt(
      .data = data,
      tfrmt = _,
      mock = FALSE
    )

  # Expect two elements in the output list, one for each `By group` value
  expect_length(output_list, 2)

  # Check the order of page notes
  expect_identical(
    purrr::map_chr(
      output_list,
      ~ attr(.x, ".page_note")
    ),
    c(
      "by group: 101",
      "by group: 102"
    )
  )

  # Check the first page (`by group` = "101")
  page_1_names <- names(output_list[[1]])
  expect_identical(
    page_1_names,
    c(
      "Label",
      "ord1",
      "Placebo\nN = 32",
      "Treatment\nN = 46",
      "Total\nN = 49",
      "..tfrmt_row_grp_lbl"
    )
  )

  # Check the second page (`by group` = "102")
  page_2_names <- names(output_list[[2]])
  expect_identical(
    page_2_names,
    c(
      "Label",
      "ord1",
      "Placebo\nN = 32",
      "Treatment\nN = 46",
      "Total\nN = 49",
      "..tfrmt_row_grp_lbl"
    )
  )
})

test_that("basic tfrmt - title", {

  t_frmt <- tfrmt(
    title = "Table Title"
  )

  expect_s3_class(t_frmt, "tfrmt")

  expect_equal(t_frmt$title, "Table Title")
  expect_equal(t_frmt$group,vars())
  expect_equal(t_frmt$label,quo())
  expect_equal(t_frmt$param,quo())
  expect_equal(t_frmt$value,quo())
  expect_equal(t_frmt$column,vars())

})

test_that("basic tfrmt - selecting group/label/param/value/column - quo", {

  t_frmt <- tfrmt(
    title = "Table Title",
    group = vars(row_label1),
    label = quo(row_label2),
    param = quo(param),
    value = quo(value),
    column = vars(column)
  )

  expect_s3_class(t_frmt,"tfrmt")

  expect_equal( t_frmt$title, "Table Title")
  expect_equal( t_frmt$group, vars(row_label1), ignore_attr = TRUE )
  expect_equal( t_frmt$label, quo(row_label2), ignore_attr = TRUE )
  expect_equal( t_frmt$param, quo(param), ignore_attr = TRUE )
  expect_equal( t_frmt$value, quo(value), ignore_attr = TRUE )
  expect_equal( t_frmt$column, vars(column), ignore_attr = TRUE )
})

test_that("basic tfrmt - selecting group/label/param/value/column - quo into var entries", {

  t_frmt <- tfrmt(
    title = "Table Title",
    group = quo(row_label1),
    label = quo(row_label2),
    param = quo(param),
    value = quo(value),
    column = quo(column)
  )

  expect_s3_class(t_frmt,"tfrmt")

  expect_equal( t_frmt$title, "Table Title")
  expect_equal( t_frmt$group, vars(row_label1), ignore_attr = TRUE )
  expect_equal( t_frmt$label, quo(row_label2), ignore_attr = TRUE )
  expect_equal( t_frmt$param, quo(param), ignore_attr = TRUE )
  expect_equal( t_frmt$value, quo(value), ignore_attr = TRUE )
  expect_equal( t_frmt$column, vars(column), ignore_attr = TRUE )
})

test_that("basic tfrmt - selecting group/label/param/value/column - char", {

  t_frmt <- tfrmt(
    title = "Table Title",
    group = c("row_label1"),
    label = c("row_label2"),
    param = c("param"),
    value = c("value"),
    column = c("column")
  )

  expect_s3_class(t_frmt,"tfrmt")

  expect_equal( t_frmt$title, "Table Title")
  expect_equal( t_frmt$group, vars(row_label1), ignore_attr = TRUE )
  expect_equal( t_frmt$label, quo(row_label2), ignore_attr = TRUE)
  expect_equal( t_frmt$param, quo(param), ignore_attr = TRUE)
  expect_equal( t_frmt$value, quo(value), ignore_attr = TRUE)
  expect_equal( t_frmt$column, vars(column), ignore_attr = TRUE)
})

test_that("basic tfrmt - selecting group/label/param/value/column - bare", {

  t_frmt <- tfrmt(
    title = "Table Title",
    group = c(row_label1, row_label4),
    label = row_label2,
    param = param,
    value = value,
    column = column
  )

  expect_s3_class(t_frmt,"tfrmt")
  expect_equal( t_frmt$title, "Table Title")
  expect_equal( t_frmt$group, vars(row_label1, row_label4), ignore_attr = TRUE)
  expect_equal( t_frmt$label, quo(row_label2), ignore_attr = TRUE)
  expect_equal( t_frmt$param, quo(param), ignore_attr = TRUE)
  expect_equal( t_frmt$value, quo(value), ignore_attr = TRUE)
  expect_equal( t_frmt$column, vars(column), ignore_attr = TRUE)
})

test_that("basic tfrmt - length one quo warning", {

  single_warning <- capture_warnings({
    tfrmt(
      title = "Table Title",
      group = row_label1,
      label = vars(row_label2,row_label3),
      param = param,
      value = value,
      column = column
    )
  })

  expect_equal(
    single_warning,
    paste0(
      "Passed more than one quosure to the argument `",
      "label",
      "`. Selecting the first entry."
    )
  )

  multi_warning <- capture_warnings({
    tfrmt(
      title = "Table Title",
      group = row_label1,
      label = vars(row_label2,row_label3),
      param = vars(param, param2),
      value = vars(value, value2),
      column = vars(column, column2)
    )
  })

  expect_equal(
    multi_warning,
    c(
      "Passed more than one quosure to the argument `label`. Selecting the first entry.",
      "Passed more than one quosure to the argument `param`. Selecting the first entry.",
      "Passed more than one quosure to the argument `value`. Selecting the first entry."
    )
  )

})

test_that("basic tfrmt - bare/char mix error", {

  expect_error(
    tfrmt(
      title = "Table Title",
      group = c(row_label1, "row_label4"),
      label = row_label2,
      param = param,
      value = value,
      column = column
    ),
    paste0(
      "Entries for `",
      "group",
      "` argument must be vars(), a character vector, or unquoted column name.\n",
      "  Consider updating the argument input to `",
      "group",
      "` to:\n\t",
      "vars(row_label1,row_label4)"
    ),
    fixed = TRUE
  )

})

test_that("layering tfrmt - default table elements - func/tfrmt",{

  t_frmt_title <- tfrmt(
    title = "Table Title"
  )

  t_frmt_layered <- t_frmt_title %>%
    layer_tfrmt(
      tfrmt(
        subtitle = "Table Subtitle"
    ))

  expect_s3_class(
    t_frmt_layered,
    "tfrmt"
  )

  expect_equal(
    t_frmt_layered$title,
    "Table Title"
  )

  expect_equal(
    t_frmt_layered$subtitle,
    "Table Subtitle"
  )

  expect_equal(t_frmt_layered$group,vars())
  expect_equal(t_frmt_layered$label,quo())
  expect_equal(t_frmt_layered$param,quo())
  expect_equal(t_frmt_layered$value,quo())
  expect_equal(t_frmt_layered$column,vars())

})

test_that("layering tfrmt - select latest table elements",{

  t_frmt_title <- tfrmt(
    title = "Table Title"
  )

  t_frmt_layered <- t_frmt_title %>%
    layer_tfrmt(
      tfrmt(
        title = "Table Title 2",
        subtitle = "Table Subtitle"
    ))


  expect_s3_class( t_frmt_layered,"tfrmt"  )
  expect_equal( t_frmt_layered$title, "Table Title 2")
  expect_equal( t_frmt_layered$subtitle, "Table Subtitle")

})

test_that("layering tfrmt - body style elements",{

  t_frmt_title <- tfrmt(
    title = "Table Title",
    body_plan = body_plan(
      frmt_structure(
        group_val = c("group1"),
        label_val = ".default",
        frmt("XXX")
        )
      )
    )


  t_frmt_layered <- t_frmt_title %>%
    layer_tfrmt(
      tfrmt(
        body_plan = body_plan(
          frmt_structure(
            group_val = c("group2"),
            label_val = ".default",
            frmt("xx.x")
          )
        )
      )
    )

  expect_s3_class(t_frmt_layered, "tfrmt")

  expect_equal( t_frmt_layered$title, "Table Title")
  expect_equal(t_frmt_layered$body_plan,
               body_plan(frmt_structure(group_val = "group1",
                                              label_val = ".default",
                                              frmt("XXX")),
                               frmt_structure(group_val = "group2",
                                              label_val = ".default",
                                              frmt("xx.x"))))

})

test_that("layering tfrmt - body style elements - multiple",{

  t_frmt_title <- tfrmt(
    title = "Table Title",
    body_plan = body_plan(
      frmt_structure(
        group_val = "group1",
        label_val = ".default",
        frmt("XXX")
      )
    )
  )

  t_frmt_layered <- t_frmt_title %>%
    layer_tfrmt(
      tfrmt(subtitle = "Table Subtitle",
            body_plan = body_plan(
             frmt_structure(group_val = "group2",label_val = ".default", frmt("xx.x")),
             frmt_structure(group_val = "group3",label_val = ".default", frmt("xx.xx"))
             )
           )
    )

  expect_s3_class(t_frmt_layered,"tfrmt")


  expect_equal(t_frmt_layered$title, "Table Title")
  expect_equal(t_frmt_layered$subtitle, "Table Subtitle")

  expect_equal(
    t_frmt_layered$body_plan,
    body_plan(
      frmt_structure(group_val = "group1",label_val = ".default",frmt("XXX")),
      frmt_structure(group_val = "group2",label_val = ".default",frmt("xx.x")),
      frmt_structure(group_val = "group3",label_val = ".default",frmt("xx.xx"))
    )
  )

})

test_that("layering tfrmt - body style elements - join_body_style FALSE",{

  t_frmt_title <- tfrmt(
    title = "Table Title",
    body_plan = body_plan(
      frmt_structure(
        group_val = "group1",
        label_val = ".default",
        frmt("XXX")
      )
    )
  )

  t_frmt_layered <- t_frmt_title %>%
    layer_tfrmt(
      tfrmt(subtitle = "Table Subtitle",
            body_plan = body_plan(
             frmt_structure(group_val = "group2",label_val = ".default",frmt("xx.x")),
             frmt_structure(group_val = "group3",label_val = ".default",frmt("xx.xx"))
           )
      ),
      join_body_plans = FALSE
    )


  expect_s3_class(t_frmt_layered,"tfrmt")


  expect_equal(t_frmt_layered$title, "Table Title")
  expect_equal(t_frmt_layered$subtitle, "Table Subtitle")

  expect_equal(
    t_frmt_layered$body_plan,
    body_plan(
      frmt_structure(group_val = "group2",label_val = ".default",frmt("xx.x")),
      frmt_structure(group_val = "group3",label_val = ".default",frmt("xx.xx"))
    )
  )

})

test_that("layering tfrmt - keeping original var/quo",{

  t_frmt_o <- tfrmt(
    title = "Table Title",
    group = c(Group1, Group2),
    label = label1
  )

  t_frmt_layered <- t_frmt_o %>%
    layer_tfrmt(
      tfrmt(
        title = "Table Title 2",
        subtitle = "Table Subtitle"
      ))

  expect_s3_class( t_frmt_layered,"tfrmt"  )
  expect_equal( t_frmt_layered$title, "Table Title 2")
  expect_equal( t_frmt_layered$subtitle, "Table Subtitle")
  expect_equal( t_frmt_layered$group, vars(Group1, Group2), ignore_attr = TRUE )
  expect_equal( t_frmt_layered$label, quo(label1), ignore_attr = TRUE )

})

test_that("layering tfrmt - Mixing var/quo",{

  t_frmt_o <- tfrmt(
    title = "Table Title",
    group = c(Group1, Group2),
    label = label1
  )

  t_frmt_layered <- t_frmt_o %>%
    layer_tfrmt(
      tfrmt(
        title = "Table Title 2",
        subtitle = "Table Subtitle",
        label = label3

      ))

  expect_s3_class( t_frmt_layered,"tfrmt"  )
  expect_equal( t_frmt_layered$title, "Table Title 2")
  expect_equal( t_frmt_layered$subtitle, "Table Subtitle")
  expect_equal( t_frmt_layered$group, vars(Group1, Group2), ignore_attr = TRUE )
  expect_equal( t_frmt_layered$label, quo(label3), ignore_attr = TRUE )

})

test_that("basic tfrmt - ... args",{

  ## arg is not to actual arg
  message_res <- capture_messages(
    tfrmt(
      totally_fake_arg = "my_col"
    )
  )
  expect_equal(
    message_res,
    "Argument 'totally_fake_arg' passed to tfrmt is not a recognized argument."
  )

  ## arg is spelled close to actual arg
  message_res <- capture_messages(
    tfrmt(
      colmn = "my_col",
    )
  )
  expect_equal(
    message_res,
   "Argument 'colmn' passed to tfrmt is not a recognized argument.\nDid you intend to use the argument `column`?"
  )

  message_res <- capture_messages(
    tfrmt(
      colmn = "my_col",
      lalbl = "label"
    )
  )
  expect_equal(
    message_res,
    c("Argument 'colmn' passed to tfrmt is not a recognized argument.\nDid you intend to use the argument `column`?",
      "Argument 'lalbl' passed to tfrmt is not a recognized argument.\nDid you intend to use the argument `label`?")
  )

})

test_that("basic tfrmt - erroring args", {

  expect_error(
    tfrmt(
        body_plan = body_plan(
          frmt_structure(group_val = ".default", label_val = ".default", frmt("XX")),
        )
      ),
    paste0("Error in evaluating argument `body_plan`:\n ",
           "Error in body_plan(frmt_structure(group_val = \".default\",",
           " label_val = \".default\", : argument is missing, with no default"
           ),
    fixed = TRUE
  )

})

test_that("basic tfrmt - func calls into quo and var args", {

  t_frmt <- tfrmt(
    title = "Table Title",
    group = c(col, df),
    label = runif,
    param = abs,
    value = acos,
    column = adist
  )

  expect_s3_class(t_frmt,"tfrmt")
  expect_equal( t_frmt$title, "Table Title")
  expect_equal( t_frmt$group, vars(col, df), ignore_attr = TRUE)
  expect_equal( t_frmt$label, quo(runif), ignore_attr = TRUE)
  expect_equal( t_frmt$param, quo(abs), ignore_attr = TRUE)
  expect_equal( t_frmt$value, quo(acos), ignore_attr = TRUE)
  expect_equal( t_frmt$column, vars(adist), ignore_attr = TRUE)
})

test_that("advanced tfrmt - tfrmt maker", {


  tfrmt_maker <- function(title, group, label, param_val){

    tfrmt(
      title = title,
      group = group,
      label = label,
      param = param_val
    )

  }

  tfrmt_maker_2 <- function(title, group, label, param_val){

    tfrmt_inputs <- quo_get(c("group","label","param_val"), as_var_args = "group", as_quo_args = c("label","param_val"))

    tfrmt(
      title = title,
      group = tfrmt_inputs$group,
      label = tfrmt_inputs$label,
      param = tfrmt_inputs$param_val
    )

  }

  tfrmt_maker_3 <- function(title, group, label, param_val, ...){

    tfrmt_inputs <- quo_get(c("group","label","param_val"), as_var_args = "group", as_quo_args = c("label","param_val"))

    bp <- body_plan(
      ...
    )

    tfrmt(
      title = title,
      group = tfrmt_inputs$group,
      label = tfrmt_inputs$label,
      param = tfrmt_inputs$param_val,
      body_plan = bp
    )

  }


  new_tfrmt <- tfrmt_maker("Table Title", vars(value1, value2), quo(labs), "parameter")
  new_tfrmt_char <- tfrmt_maker("Table Title", c("value1", "value2"), "labs", "parameter")
  new_tfrmt_2 <- tfrmt_maker_2("Table Title", vars(value1, value2), quo(labs), "parameter")
  new_tfrmt_2_char <- tfrmt_maker_2("Table Title", c("value1", "value2"), "labs", "parameter")

  expect_s3_class(new_tfrmt,"tfrmt")
  expect_equal( new_tfrmt$title, "Table Title")
  expect_equal( new_tfrmt$group, vars(value1, value2), ignore_attr = TRUE)
  expect_equal( new_tfrmt$label, quo(labs), ignore_attr = TRUE)
  expect_equal( new_tfrmt$param, quo(parameter), ignore_attr = TRUE)

  expect_s3_class(new_tfrmt_char,"tfrmt")
  expect_equal( new_tfrmt_char$title, "Table Title")
  expect_equal( new_tfrmt_char$group, vars(value1, value2), ignore_attr = TRUE)
  expect_equal( new_tfrmt_char$label, quo(labs), ignore_attr = TRUE)
  expect_equal( new_tfrmt_char$param, quo(parameter), ignore_attr = TRUE)

  expect_s3_class(new_tfrmt_2,"tfrmt")
  expect_equal( new_tfrmt_2$title, "Table Title")
  expect_equal( new_tfrmt_2$group, vars(value1, value2), ignore_attr = TRUE)
  expect_equal( new_tfrmt_2$label, quo(labs), ignore_attr = TRUE)
  expect_equal( new_tfrmt_2$param, quo(parameter), ignore_attr = TRUE)

  expect_s3_class(new_tfrmt_2_char,"tfrmt")
  expect_equal( new_tfrmt_2_char$title, "Table Title")
  expect_equal( new_tfrmt_2_char$group, vars(value1, value2), ignore_attr = TRUE)
  expect_equal( new_tfrmt_2_char$label, quo(labs), ignore_attr = TRUE)
  expect_equal( new_tfrmt_2_char$param, quo(parameter), ignore_attr = TRUE)

  new_tfrmt_with_bp <- tfrmt_maker_3(
    "Table Title",
    vars(value1, value2),
    quo(labs),
    "parameter",
    frmt_structure(group_val = ".default", label_val = ".default", frmt("xx.xx") ),
    frmt_structure(group_val = "group1", label_val = ".default", frmt("xxx.xx") )
  )

  expect_s3_class(new_tfrmt_with_bp,"tfrmt")
  expect_equal( new_tfrmt_with_bp$title, "Table Title")
  expect_equal( new_tfrmt_with_bp$group, vars(value1, value2), ignore_attr = TRUE)
  expect_equal( new_tfrmt_with_bp$label, quo(labs), ignore_attr = TRUE)
  expect_equal( new_tfrmt_with_bp$param, quo(parameter), ignore_attr = TRUE)
  expect_equal( new_tfrmt_with_bp$body_plan, body_plan(
    frmt_structure(group_val = ".default", label_val = ".default", frmt("xx.xx") ),
    frmt_structure(group_val = "group1", label_val = ".default", frmt("xxx.xx") )
  ), ignore_attr = TRUE)


  bp <- body_plan(
    frmt_structure(group_val = ".default", label_val = ".default", frmt("xx.xx") )
  )

  new_tfrmt_with_bp_2 <- tfrmt_maker_3(
    "Table Title",
    vars(value1, value2),
    quo(labs),
    "parameter",
    frmt_structure(group_val = ".default", label_val = ".default", frmt("xx.xx") ),
    frmt_structure(group_val = "group1", label_val = ".default", frmt("xxx.xx") )
  )

  expect_s3_class(new_tfrmt_with_bp_2,"tfrmt")
  expect_equal( new_tfrmt_with_bp_2$title, "Table Title")
  expect_equal( new_tfrmt_with_bp_2$group, vars(value1, value2), ignore_attr = TRUE)
  expect_equal( new_tfrmt_with_bp_2$label, quo(labs), ignore_attr = TRUE)
  expect_equal( new_tfrmt_with_bp_2$param, quo(parameter), ignore_attr = TRUE)
  ## make sure it selects the right bp (not the global env one first)
  expect_equal( new_tfrmt_with_bp_2$body_plan, body_plan(
    frmt_structure(group_val = ".default", label_val = ".default", frmt("xx.xx") ),
    frmt_structure(group_val = "group1", label_val = ".default", frmt("xxx.xx") )
  ), ignore_attr = TRUE)


})

test_that("layering tfrmt - valid layering should be silent - even when a quo arg is equal to itself",{

  expect_silent({
    tfrmt(
      label = label
    ) %>%
      tfrmt(
        # Specify title, subtitle, footer
        title = "Table Name",
        subtitle = "Study ID: GSK12345"
      )
  })

  expect_silent({
    tfrmt(
      label = label
    ) %>%
      tfrmt(
        label = label
    )
  })

})

test_that("basic tfrmt - error when body_plan groups does not match group arg",{

  expect_silent(
    tfrmt(
      group = vars(group1, group2),
      body_plan = body_plan(
        frmt_structure(
          group_val = c("value"),
          label_val = ".default",
          frmt("XXX")
       ),
       frmt_structure(
         group_val = list(group1 = "value"),
         label_val = ".default",
         frmt("XXX")
       ),
       frmt_structure(
         group_val = list(group2 = "value"),
         label_val = ".default",
         frmt("XXX")
       ),
       frmt_structure(
         group_val = list(group1 = "value", group2 = "value"),
         label_val = ".default",
         frmt("XXX")
       )
      )
    )
  )

  expect_error(
    tfrmt(
      group = vars(group1, group2),
      body_plan = body_plan(
        frmt_structure(
          group_val = list(invalid = "value", invalid2 = "value"),
          label_val = ".default",
          frmt("XXX")
        )
      )
     ),
    paste0(
     "Inconsistencies between group and body_plan\n",
     "Invalid Format Structure in body_plan at position `1`:\n",
     "  Malformed Group: invalid, invalid2"
    ),
    fixed = TRUE
  )

})

test_that("basic tfrmt - error when row_grp_plan groups does not match group arg",{

  expect_silent(
    tfrmt(
      group = vars(group1, group2),
      row_grp_plan = row_grp_plan(
        row_grp_structure(
          group_val = "value",
          element_block = element_block(" ")
        ),
        row_grp_structure(
          group_val = list(group2 = "value"),
          element_block = element_block(post_space = "-")
        ),
        row_grp_structure(
          group_val = list(group1 = "value", group2 = "value"),
          element_block = element_block(post_space = NULL)
        )
      )
    )
  )

  expect_error(
    tfrmt(
      group = vars(group1, group2),
      row_grp_plan = row_grp_plan(
        row_grp_structure(
          group_val = list(invalid = "value", invalid2 = "value"),
          element_block = element_block(post_space = NULL)
        )
      )
    ),
    paste0(
      "Inconsistencies between group and row_grp_plan row_grp_structures\n",
      "Invalid Format Structure in row_grp_plan for row_grp_structure `1`:\n",
      "  Malformed Group: invalid, invalid2"
    ),
    fixed = TRUE
  )

})

test_that("basic tfrmt - error when footnote_plan groups does not match group arg",{

  expect_silent(
    tfrmt(
      group = vars(group1, group2),
      footnote_plan = footnote_plan(
        footnote_structure(
          footnote_text = "footnote 2",
          group_val = list(group2 = "value"),
          column_val = "column value",
          label_val = "label value"
        ),
        footnote_structure(
          footnote_text = "footnote 3",
          group_val = list(group1 = "value", group2 = "value"),
          column_val = "column value",
          label_val = "label value"
        ),
        marks = "standard"
      )
    )
  )

  expect_error(
    tfrmt(
      group = vars(group1, group2),
      footnote_plan = footnote_plan(
        footnote_structure(
          footnote_text = "footnote 2",
          group_val = list(group2 = "value"),
          column_val = "column value",
          label_val = "label value"
        ),
        footnote_structure(
          footnote_text = "footnote 3",
          group_val = list(invalid = "value", invalid2 = "value"),
          column_val = "column value",
          label_val = "label value"
        ),
        marks = "standard"
      )
    ),
    paste0(
      "Inconsistencies between group and footnote_plan footnote_structures\n",
      "Invalid Format Structure in footnote_plan for footnote_structure `2`:\n",
      "  Malformed Group: invalid, invalid2"
    ),
    fixed = TRUE
  )

})

test_that("basic tfrmt - error when body_plan, row_grp_plan, or footnote_plan groups does not match group arg",{

  expect_silent(
    tfrmt(
      group = vars(group1, group2),
      body_plan = body_plan(
        frmt_structure(
          group_val = list(group1 = "value", group2 = "value"),
          label_val = ".default",
          frmt("XXX")
        )
      ),
      row_grp_plan = row_grp_plan(
        row_grp_structure(
          group_val = list(group1 = "value", group2 = "value"),
          element_block = element_block(post_space = NULL)
        )
      ),
      footnote_plan = footnote_plan(
        footnote_structure(
          footnote_text = "footnote 3",
          group_val = list(group1 = "value", group2 = "value"),
          column_val = "column value",
          label_val = "label value"
        ),
        marks = "standard"
      )
    )
  )

  expect_error(
    tfrmt(
      group = vars(group1, group2),
      body_plan = body_plan(
        frmt_structure(
          group_val = list(invalid = "value", invalid2 = "value"),
          label_val = ".default",
          frmt("XXX")
        )
      ),
      row_grp_plan = row_grp_plan(
        row_grp_structure(
          group_val = list(invalid = "value", invalid2 = "value"),
          element_block = element_block(post_space = NULL)
        )
      ),
      footnote_plan = footnote_plan(
        footnote_structure(
          footnote_text = "footnote 3",
          group_val = list(invalid = "value", invalid2 = "value"),
          column_val = "column value",
          label_val = "label value"
        ),
        marks = "standard"
      )
    ),
    paste0(
      "Inconsistencies between group and body_plan\n",
      "Invalid Format Structure in body_plan at position `1`:\n",
      "  Malformed Group: invalid, invalid2\n\n",
      "Inconsistencies between group and row_grp_plan row_grp_structures\n",
      "Invalid Format Structure in row_grp_plan for row_grp_structure `1`:\n",
      "  Malformed Group: invalid, invalid2\n\n",
      "Inconsistencies between group and footnote_plan footnote_structures\n",
      "Invalid Format Structure in footnote_plan for footnote_structure `1`:\n",
      "  Malformed Group: invalid, invalid2"
    ),
    fixed = TRUE
  )

})

test_that("layering tfrmt - error when body_plan groups no longer match group arg",{

  basic_tfrmt <- tfrmt(
    group = vars(group1, group2),
    body_plan = body_plan(
      frmt_structure(
        group_val = c("value"),
        label_val = ".default",
        frmt("XXX")
      ),
      frmt_structure(
        group_val = list(group1 = "value"),
        label_val = ".default",
        frmt("XXX")
      ),
      frmt_structure(
        group_val = list(group2 = "value"),
        label_val = ".default",
        frmt("XXX")
      ),
      frmt_structure(
        group_val = list(group1 = "value", group2 = "value"),
        label_val = ".default",
        frmt("XXX")
      )
    )
  )


  expect_silent(
    basic_tfrmt %>%
      tfrmt(
        group = vars(group2, group1)
      )
  )

  expect_error(
    basic_tfrmt %>%
      tfrmt(
        group = vars(new_group1, new_group2)
      ),
    "You might need to update group names using \"update_group(`new_group1` = `group1`,`new_group2` = `group2`)\"",
    fixed = TRUE
  )

})

test_that("layering tfrmt - error when plan groups no longer match group arg",{

  basic_tfrmt <- tfrmt(
    group = vars(group1, group2),
    body_plan = body_plan(
      frmt_structure(
        group_val = list(group1 = "value", group2 = "value"),
        label_val = ".default",
        frmt("XXX")
      )
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = list(group1 = "value", group2 = "value"),
        element_block = element_block(post_space = NULL)
      )
    ),
    footnote_plan = footnote_plan(
      footnote_structure(
        footnote_text = "footnote 3",
        group_val = list(group1 = "value", group2 = "value"),
        column_val = "column value",
        label_val = "label value"
      ),
      marks = "standard"
    )
  )


  expect_silent(
    basic_tfrmt %>%
      tfrmt(
        group = vars(group2, group1)
      )
  )

  expect_error(
    basic_tfrmt %>%
      tfrmt(
        group = vars(new_group1, new_group2)
      ),
    "You might need to update group names using \"update_group(`new_group1` = `group1`,`new_group2` = `group2`)\"",
    fixed = TRUE
  )

})

test_that("updating tfrmt - updating group vars - body_plan",{

  tfrmt1 <- tfrmt(
      group = c(group1, group2),
      body_plan  = body_plan(
        frmt_structure(
          group_val = "value",
          label_val = ".default",
          frmt("XXX")
        ),
        frmt_structure(
           group_val = list(group2 = "value"),
           label_val = ".default",
           frmt("XXXX")
           ),
        frmt_structure(
           group_val = list(group1 = "value", group2 = "value"),
           label_val = ".default",
           frmt("XXXXX")
         )
      )
    )

  tfrmt2 <- tfrmt1 %>%
    update_group(New_Group = group1)

  expect_equal(
    tfrmt2,
    tfrmt(
      group = c(New_Group, group2),
      body_plan  = body_plan(
        frmt_structure(
          group_val = "value",
          label_val = ".default",
          frmt("XXX")
        ),
        frmt_structure(
          group_val = list(group2 = "value"),
          label_val = ".default",
          frmt("XXXX")
        ),
        frmt_structure(
          group_val = list(New_Group = "value", group2 = "value"),
          label_val = ".default",
          frmt("XXXXX")
        )
      )
    ),
    ignore_function_env = TRUE,
    ignore_formula_env = TRUE
  )

  tfrmt3 <- tfrmt1 %>%
    update_group(
      New_Group = group1,
      `best group` = group2
      )

  expect_equal(
    tfrmt3,
    tfrmt(
      group = c(New_Group, `best group`),
      body_plan  = body_plan(
        frmt_structure(
          group_val = "value",
          label_val = ".default",
          frmt("XXX")
        ),
        frmt_structure(
          group_val = list(`best group` = "value"),
          label_val = ".default",
          frmt("XXXX")
        ),
        frmt_structure(
          group_val = list(New_Group = "value", `best group` = "value"),
          label_val = ".default",
          frmt("XXXXX")
        )
      )
    ),
    ignore_function_env = TRUE,
    ignore_formula_env = TRUE
  )

})

test_that("updating tfrmt - updating group vars - row_grp_plan",{

  tfrmt1 <- tfrmt(
    group = c(group1, group2),
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = "value",
        element_block = element_block(" ")
      ),
      row_grp_structure(
        group_val = list(group2 = "value"),
        element_block = element_block(post_space = "-")
      ),
      row_grp_structure(
        group_val = list(group1 = "value", group2 = "value"),
        element_block = element_block(post_space = NULL)
      )
    )
  )

  tfrmt2 <- tfrmt1 %>%
    update_group(New_Group = group1)

  expect_equal(
    tfrmt2,
    tfrmt(
      group = c(New_Group, group2),
      row_grp_plan = row_grp_plan(
        row_grp_structure(
          group_val = "value",
          element_block = element_block(" ")
        ),
        row_grp_structure(
          group_val = list(group2 = "value"),
          element_block = element_block(post_space = "-")
        ),
        row_grp_structure(
          group_val = list(New_Group = "value", group2 = "value"),
          element_block = element_block(post_space = NULL)
        )
      )
    ),
    ignore_function_env = TRUE,
    ignore_formula_env = TRUE
  )

  tfrmt3 <- tfrmt1 %>%
    update_group(
      New_Group = group1,
      `best group` = group2
    )

  expect_equal(
    tfrmt3,
    tfrmt(
      group = c("New_Group", "best group"),
      row_grp_plan = row_grp_plan(
        row_grp_structure(
          group_val = "value",
          element_block = element_block(" ")
        ),
        row_grp_structure(
          group_val = list(`best group` = "value"),
          element_block = element_block(post_space = "-")
        ),
        row_grp_structure(
          group_val = list(New_Group = "value", `best group` = "value"),
          element_block = element_block(post_space = NULL)
        )
      )
    ),
    ignore_function_env = TRUE,
    ignore_formula_env = TRUE
  )

})

test_that("updating tfrmt - updating group vars - footnote_plan",{

  tfrmt1 <- tfrmt(
    group = c(group1, group2),
    footnote_plan = footnote_plan(
      footnote_structure(
        footnote_text = "footnote 2",
        group_val = list(group2 = "value"),
        column_val = "column value",
        label_val = "label value"
      ),
      footnote_structure(
        footnote_text = "footnote 3",
        group_val = list(group1 = "value", group2 = "value"),
        column_val = "column value",
        label_val = "label value"
      ),
      marks = "standard"
    )
  )

  tfrmt2 <- tfrmt1 %>%
    update_group(New_Group = group1)

  expect_equal(
    tfrmt2,
    tfrmt(
      group = c(New_Group, group2),
      footnote_plan = footnote_plan(
        footnote_structure(
          footnote_text = "footnote 2",
          group_val = list(group2 = "value"),
          column_val = "column value",
          label_val = "label value"
        ),
        footnote_structure(
          footnote_text = "footnote 3",
          group_val = list(New_Group = "value", group2 = "value"),
          column_val = "column value",
          label_val = "label value"
        ),
        marks = "standard"
      )
    ),
    ignore_function_env = TRUE,
    ignore_formula_env = TRUE
  )

  tfrmt3 <- tfrmt1 %>%
    update_group(
      New_Group = group1,
      `best group` = group2
    )

  expect_equal(
    tfrmt3,
    tfrmt(
      group = c("New_Group", "best group"),
      footnote_plan = footnote_plan(
        footnote_structure(
          footnote_text = "footnote 2",
          group_val = list(`best group` = "value"),
          column_val = "column value",
          label_val = "label value"
        ),
        footnote_structure(
          footnote_text = "footnote 3",
          group_val = list(New_Group = "value", `best group` = "value"),
          column_val = "column value",
          label_val = "label value"
        ),
        marks = "standard"
      )
    ),
    ignore_function_env = TRUE,
    ignore_formula_env = TRUE
  )

})

test_that("Defining the spanning structure", {

  s1 <- span_structure(
    c1 = "Test Label",
    c2 = vars(A,B)
  )

  s2 <- span_structure(
    c1 = "Test Label",
    c2 = "Test Sub Label",
    c3 = vars(A,B)
  )

  expect_s3_class(s1,"span_structure")
  expect_s3_class(s2,"span_structure")

  expect_equal(s1[["c1"]], list(quo(`Test Label`)), ignore_attr = c(".Environment"))
  expect_equal(s1[["c2"]], list(quo(`A`),quo(`B`)), ignore_attr = c(".Environment"))
  expect_equal(s1[["c1"]], list(quo(`Test Label`)), ignore_attr = c(".Environment"))
  expect_equal(s2[["c2"]], list(quo(`Test Sub Label`)), ignore_attr = c(".Environment"))
  expect_equal(s2[["c3"]], list(quo(`A`),quo(`B`)), ignore_attr = c(".Environment"))

})

test_that("Defining the col plan",{

  s1 <- col_plan(
    span_structure(
      c1 = "Test Label",
      c2 = vars(A,B)
    ))

  expect_s3_class(s1,"col_plan")
  expect_s3_class(s1,"plan")

})

test_that("Defining the col plan in tfrmt", {

  #Go into object
  tfrmt_test <- tfrmt(
    column = vars(col1, col2),
    col_plan = col_plan(
      span_structure(col1 = "test label",
                     col2 = vars(col3)
                     )
      )
    )

  expect_true(!is.null(tfrmt_test$col_plan))

  # expect error when writing invalid col_plan
    expect_error(
      tfrmt(
        col_plan = col_plan(
          group,label,
          span_structure(col1 = "test label", col2 = col3)
        )
      ),
      "No columns defined in the `column` argument of tfrmt but provided a span_structure() in `col_plan`.",
      fixed = TRUE
    )

    expect_error(
      tfrmt(
        column = col2,
        col_plan = col_plan(
          group,label,
          span_structure(col1 = "test label", col2 = col3)
        )
      ),
      "A single column defined in `column` argument of tfrmt but provided a span_structure() in `col_plan`.",
      fixed = TRUE
    )

    expect_error(
      tfrmt(
        column = c(col1, col2),
        col_plan = col_plan(
          group,label,
          span_structure(col11 = "test label", col2 = col3)
        )
      ),
      "Columns defined in `span_structure` are not defined columns in the tfrmt\nColumn Values: `col1`, `col2`\nInvalid Column Names in Span Structure: `col11`",
      fixed = TRUE
    )

})

test_that("Test applying a col_plan - simple",{

  col_vars <- vars(col1, col2)

  cp_keep <- col_plan(
    first_col,
    span_structure(col1 = c("test val"), col2 = c("val2","val1"))
  )

  cp_drop <- col_plan(
    first_col,
    span_structure(col1 = "test val", col2 = c("val2","val1")),
    .drop = TRUE
  )

  cp_subtraction <- col_plan(
    -first_col,
    span_structure(col1 = c("test val"), col2 = c("val2","val1"))
  )

  cp_subtraction_span <- col_plan(
    -first_col,
    span_structure(col1 = c("test val"), col2 = c(-val1))
  )

  name_col <- c(
    "first_col",
    "test val___tlang_delim___val1",
    "test val___tlang_delim___val2",
    "extra_col"
  )

  expect_equal(
    create_col_order(cp = cp_keep, columns = col_vars, data_names = name_col),
    vars(
      first_col,
      `test val___tlang_delim___val2`,
      `test val___tlang_delim___val1`,
      extra_col
    ),
    ignore_attr = c(".Environment")
  )

  expect_equal(
    create_col_order(cp = cp_drop, columns = col_vars, data_names = name_col),
    vars(
      first_col,
      `test val___tlang_delim___val2`,
      `test val___tlang_delim___val1`,
    ),
    ignore_attr = c(".Environment")
  )

  expect_equal(
    create_col_order(cp = cp_subtraction, columns = col_vars, data_names = name_col),
    vars(
      -first_col,
      `test val___tlang_delim___val2`,
      `test val___tlang_delim___val1`,
      extra_col
    ),
    ignore_attr = c(".Environment")
  )

  expect_equal(
    create_col_order(cp = cp_subtraction_span, columns = col_vars, data_names = name_col),
    vars(
      -first_col,
      -`test val___tlang_delim___val1`,
      `test val___tlang_delim___val2`,
      extra_col
    ),
    ignore_attr = c(".Environment")
  )

  name_col_mixed_order <- c(
    "first_col",
    "extra_col",
    "test val___tlang_delim___val1",
    "test val___tlang_delim___val2"
  )

  expect_equal(
    create_col_order(cp = cp_keep, columns = col_vars, data_names = name_col_mixed_order),
    vars(
      first_col,
      `test val___tlang_delim___val2`,
      `test val___tlang_delim___val1`,
      extra_col
    ),
    ignore_attr = c(".Environment")
  )

  expect_equal(
    create_col_order(cp = cp_drop, columns = col_vars, data_names = name_col_mixed_order),
    vars(
      first_col,
      `test val___tlang_delim___val2`,
      `test val___tlang_delim___val1`
    ),
    ignore_attr = c(".Environment")
  )

  expect_equal(
    create_col_order(cp = cp_subtraction, columns = col_vars, data_names = name_col_mixed_order),
    vars(
      -first_col,
      `test val___tlang_delim___val2`,
      `test val___tlang_delim___val1`,
      extra_col
    ),
    ignore_attr = c(".Environment")
  )

  expect_equal(
    create_col_order(cp = cp_subtraction_span, columns = col_vars, data_names = name_col_mixed_order),
    vars(
      -first_col,
      -`test val___tlang_delim___val1`,
      `test val___tlang_delim___val2`,
      extra_col
    ),
    ignore_attr = c(".Environment")
  )

})

test_that("Test applying a col_plan - tidyselect",{

  col_vars <- vars(col1, col2)

  cp_keep <- col_plan(
    span_structure(col1 = starts_with("test"), col2 = c(val2, everything())),
    everything(),
    first_col
  )

  cp_drop <- col_plan(
    span_structure(
      col1 = starts_with("test"),
      col2 = c(val2, everything())
    ),
    everything(),
    first_col,
    .drop = TRUE
  )

  name_col <- c(
    "first_col",
    "test val___tlang_delim___val1",
    "test val___tlang_delim___val2",
    "extra_col"
  )

  cp_vars_keep <- create_col_order(cp = cp_keep, columns = col_vars, data_names = name_col)

  expect_equal(
    cp_vars_keep,
    vars(
      `test val___tlang_delim___val2`,
      `test val___tlang_delim___val1`,
      extra_col,
      first_col
    ),
    ignore_attr = c(".Environment")
  )

  cp_vars_drop <- create_col_order(cp = cp_drop, columns = col_vars, data_names = name_col)

  expect_equal(
    cp_vars_drop,
    vars(
      `test val___tlang_delim___val2`,
      `test val___tlang_delim___val1`,
      extra_col,
      first_col
    ),
    ignore_attr = c(".Environment")
  )

  name_col_mixed_order <- c(
    "first_col",
    "extra_col",
    "test val___tlang_delim___val1",
    "test val___tlang_delim___val2"
  )

  cp_vars_keep_mixed_order <- create_col_order(cp = cp_keep, columns = col_vars, data_names = name_col_mixed_order)

  expect_equal(
    cp_vars_keep_mixed_order,
    vars(
      `test val___tlang_delim___val2`,
      `test val___tlang_delim___val1`,
      extra_col,
      first_col
    ),
    ignore_attr = c(".Environment")
  )

})

test_that("Test applying a col_plan - renaming",{

  col_vars <- vars(col1, col2)

  cp_keep <- col_plan(
    new_first_col = first_col,
    span_structure(
      col1 = c(`new header col` = "test val"),
      col2 = c(best_value = "val2", "val1")
    ),
    preserved_col
  )

  cp_drop <- col_plan(
    new_first_col = first_col,
    span_structure(
      col1 = c(`new header col` = "test val"),
      col2 = c(best_value = "val2", "val1")
    ),
    preserved_col,
    .drop = TRUE
  )

  cp_keep_rename_spanned_col <- col_plan(
    new_first_col = first_col,
    span_structure(
      col1 = c(`new header col` = "test val")
    ),
    best_value = "val2",
    val1,
    preserved_col
  )

  cp_drop_rename_spanned_col <- col_plan(
    new_first_col = first_col,
    span_structure(
      col1 = c(`new header col` = "test val")
    ),
    best_value = "val2",
    val1,
    preserved_col,
    .drop = TRUE
  )

  name_col <- c(
    "first_col",
    "test val___tlang_delim___val1",
    "test val___tlang_delim___val2",
    "preserved_col",
    "extra_col"
  )

  name_col_mixed_order <- c(
    "first_col",
    "preserved_col",
    "extra_col",
    "test val___tlang_delim___val1",
    "test val___tlang_delim___val2"
  )

  expect_equal(
    create_col_order(cp = cp_keep, columns = col_vars, data_names = name_col),
    vars(
      new_first_col = first_col,
      `new header col___tlang_delim___best_value` = `test val___tlang_delim___val2`,
      `new header col___tlang_delim___val1` = `test val___tlang_delim___val1`,
      preserved_col,
      extra_col
    ),
    ignore_attr = c(".Environment")
  )

  expect_equal(
    create_col_order(cp = cp_drop, columns = col_vars, data_names = name_col),
    vars(
      new_first_col = first_col,
      `new header col___tlang_delim___best_value` = `test val___tlang_delim___val2`,
      `new header col___tlang_delim___val1` = `test val___tlang_delim___val1`,
      preserved_col
    ),
    ignore_attr = c(".Environment")
  )

  expect_equal(
    create_col_order(cp = cp_keep_rename_spanned_col, columns = col_vars, data_names = name_col),
    vars(
      new_first_col = first_col,
      `new header col___tlang_delim___best_value` = `test val___tlang_delim___val2`,
      `new header col___tlang_delim___val1` = `test val___tlang_delim___val1`,
      preserved_col,
      extra_col
    ),
    ignore_attr = c(".Environment")
  )

  expect_equal(
    create_col_order(cp = cp_drop_rename_spanned_col, columns = col_vars, data_names = name_col),
    vars(
      new_first_col = first_col,
      `new header col___tlang_delim___best_value` = `test val___tlang_delim___val2`,
      `new header col___tlang_delim___val1` = `test val___tlang_delim___val1`,
      preserved_col
    ),
    ignore_attr = c(".Environment")
  )

  expect_equal(
    create_col_order(cp = cp_keep, columns = col_vars, data_names = name_col_mixed_order),
    vars(
      new_first_col = first_col,
      `new header col___tlang_delim___best_value` = `test val___tlang_delim___val2`,
      `new header col___tlang_delim___val1` = `test val___tlang_delim___val1`,
      preserved_col,
      extra_col
    ),
    ignore_attr = c(".Environment")
  )

  expect_equal(
    create_col_order(cp = cp_drop, columns = col_vars, data_names = name_col_mixed_order),
    vars(
      new_first_col = first_col,
      `new header col___tlang_delim___best_value` = `test val___tlang_delim___val2`,
      `new header col___tlang_delim___val1` = `test val___tlang_delim___val1`,
      preserved_col
    ),
    ignore_attr = c(".Environment")
  )

  expect_equal(
    create_col_order(cp = cp_keep_rename_spanned_col, columns = col_vars, data_names = name_col_mixed_order),
    vars(
      new_first_col = first_col,
      `new header col___tlang_delim___best_value` = `test val___tlang_delim___val2`,
      `new header col___tlang_delim___val1` = `test val___tlang_delim___val1`,
      preserved_col,
      extra_col
    ),
    ignore_attr = c(".Environment")
  )

  expect_equal(
    create_col_order(cp = cp_drop_rename_spanned_col, columns = col_vars, data_names = name_col_mixed_order),
    vars(
      new_first_col = first_col,
      `new header col___tlang_delim___best_value` = `test val___tlang_delim___val2`,
      `new header col___tlang_delim___val1` = `test val___tlang_delim___val1`,
      preserved_col
    ),
    ignore_attr = c(".Environment")
  )

})

test_that("Test applying a col_plan - renaming twice",{

  col_vars <- vars(col1, col2)

  cp_keep <- col_plan(
    new_first_col = first_col,
    span_structure(
      col1 = c(`new header col` = "test val"),
      col2 = c(best_value = "val2", "val1")
    ),
    new_best_value = val2,
    val1,
    preserved_col
  )

  cp_drop <- col_plan(
    new_first_col = first_col,
    span_structure(
      col1 = c(`new header col` = "test val"),
      col2 = c(best_value = "val2", "val1")
    ),
    new_best_value = val2,
    val1,
    preserved_col,
    .drop = TRUE
  )

  name_col <- c(
    "first_col",
    "test val___tlang_delim___val1",
    "test val___tlang_delim___val2",
    "preserved_col",
    "extra_col"
  )

  expect_equal(
    create_col_order(cp = cp_keep, columns = col_vars, data_names = name_col),
    vars(
      new_first_col = first_col,
      `new header col___tlang_delim___new_best_value` = `test val___tlang_delim___val2`,
      `new header col___tlang_delim___val1` = `test val___tlang_delim___val1`,
      preserved_col,
      extra_col
    ),
    ignore_attr = c(".Environment")
  )

  expect_equal(
    create_col_order(cp = cp_drop, columns = col_vars, data_names = name_col),
    vars(
      new_first_col = first_col,
      `new header col___tlang_delim___new_best_value` = `test val___tlang_delim___val2`,
      `new header col___tlang_delim___val1` = `test val___tlang_delim___val1`,
      preserved_col
    ),
    ignore_attr = c(".Environment")
  )

  name_col_mixed_order <- c(
    "first_col",
    "preserved_col",
    "extra_col",
    "test val___tlang_delim___val1",
    "test val___tlang_delim___val2"
  )

  expect_equal(
    create_col_order(cp = cp_keep, columns = col_vars, data_names = name_col_mixed_order),
    vars(
      new_first_col = first_col,
      `new header col___tlang_delim___new_best_value` = `test val___tlang_delim___val2`,
      `new header col___tlang_delim___val1` = `test val___tlang_delim___val1`,
      preserved_col,
      extra_col
    ),
    ignore_attr = c(".Environment")
  )

  expect_equal(
    create_col_order(cp = cp_drop, columns = col_vars, data_names = name_col_mixed_order),
    vars(
      new_first_col = first_col,
      `new header col___tlang_delim___new_best_value` = `test val___tlang_delim___val2`,
      `new header col___tlang_delim___val1` = `test val___tlang_delim___val1`,
      preserved_col
    ),
    ignore_attr = c(".Environment")
  )

})

test_that("Test applying a col_plan - tidyselect with renaming",{

  col_vars <- vars(col1, col2)

  cp_keep <- col_plan(
    span_structure(col1 = c(new_col_name = starts_with("test")), col2 = c(val2, everything())),
    everything(),
    first_col,
    starts_with("col")
  )

  cp_drop <- col_plan(
    span_structure(col1 = c(new_col_name = starts_with("test")), col2 = c(val2, everything())),
    everything(),
    first_col,
    starts_with("col"),
    .drop = TRUE
  )

  name_col <- c(
    "first_col",
    "test one___tlang_delim___val1",
    "test two___tlang_delim___val2",
    "extra_col",
    "col1",
    "col2",
    "col3"
  )

  cp_vars_keep <- create_col_order(cp = cp_keep, columns = col_vars, data_names = name_col)

  expect_equal(
    cp_vars_keep,
    vars(
      new_col_name1___tlang_delim___val1 = `test one___tlang_delim___val1`,
      new_col_name2___tlang_delim___val2 = `test two___tlang_delim___val2`,
      extra_col,
      first_col,
      col1,
      col2,
      col3
    ),
    ignore_attr = c(".Environment")
  )

  cp_vars_drop <- create_col_order(cp = cp_drop, columns = col_vars, data_names = name_col)

  expect_equal(
    cp_vars_drop,
    vars(
      new_col_name1___tlang_delim___val1 = `test one___tlang_delim___val1`,
      new_col_name2___tlang_delim___val2 = `test two___tlang_delim___val2`,
      extra_col,
      first_col,
      col1,
      col2,
      col3
    ),
    ignore_attr = c(".Environment")
  )

  name_col_mixed_order <- c(
    "col2",
    "first_col",
    "test two___tlang_delim___val2",
    "test one___tlang_delim___val1",
    "extra_col",
    "col1",
    "col3"
  )

  cp_vars_keep_mixed_order <- create_col_order(cp = cp_keep, columns = col_vars, data_names = name_col_mixed_order)

  expect_equal(
    cp_vars_keep_mixed_order,
    vars(
      new_col_name1___tlang_delim___val2 = `test two___tlang_delim___val2`,
      new_col_name2___tlang_delim___val1 = `test one___tlang_delim___val1`,
      extra_col,
      first_col,
      col2,
      col1,
      col3
    ),
    ignore_attr = c(".Environment")
  )

})

test_that("Test applying a col_plan - ordering on multiple columns",{

  col_vars <- vars(c1, c2)

  cp <- col_plan(
    span_structure(c1 = c(`test val2`, `test val1`, `another val`), c2 = c(val2, val1, everything())),
    first_col,
    extra_col
  )

  name_col <- c(
    "first_col",
    "test val1___tlang_delim___val1",
    "test val1___tlang_delim___val2",
    "test val2___tlang_delim___val1",
    "test val2___tlang_delim___val2",
    "another val___tlang_delim___val1",
    "another val___tlang_delim___val2",
    "another val___tlang_delim___val3",
    "preserved_col",
    "extra_col"
  )

  expect_equal(
    create_col_order(cp = cp, columns = col_vars, data_names = name_col),
    vars(
      `test val2___tlang_delim___val2`,
      `test val2___tlang_delim___val1`,
      `test val1___tlang_delim___val2`,
      `test val1___tlang_delim___val1`,
      `another val___tlang_delim___val2`,
      `another val___tlang_delim___val1`,
      `another val___tlang_delim___val3`,
      first_col,
      extra_col,
      preserved_col
    ),
    ignore_attr = c(".Environment")
  )

  cp <- col_plan(
    span_structure(c1 = c(`test val2`, `test val1`, `another val`), c2 = c(val2, val1)),
    preserved_col,
    span_structure(c1 = c(`another val`), c2 = c(val3)),
    first_col,
    extra_col
  )

  cp_vars <- create_col_order(cp = cp, columns = col_vars, data_names = name_col)

  expect_equal(
    cp_vars,
    vars(
      `test val2___tlang_delim___val2`,
      `test val2___tlang_delim___val1`,
      `test val1___tlang_delim___val2`,
      `test val1___tlang_delim___val1`,
      `another val___tlang_delim___val2`,
      `another val___tlang_delim___val1`,
      preserved_col,
      `another val___tlang_delim___val3`,
      first_col,
      extra_col
    ),
    ignore_attr = c(".Environment")
  )

})

test_that("Unorthodox col_plans",{

  col_vars <- vars(c1, c2)

  cp <- col_plan(
    val4,
    span_structure(c1 = "test value", c2 = c(val1, val3)),
    val2, val6, val3,
    everything(),
    val5
  )

  name_col <- c(
    "test value___tlang_delim___val1",
    "test value___tlang_delim___val2",
    "test value___tlang_delim___val3",
    "val4",
    "val5",
    "val6",
    "val7"
  )

  expect_equal(
    create_col_order(cp = cp, columns = col_vars, data_names = name_col),
    vars(
      val4,
      `test value___tlang_delim___val1`,
      `test value___tlang_delim___val2`,
      val6,
      `test value___tlang_delim___val3`,
      val7,
      val5
    ),
    ignore_attr = c(".Environment")
  )

  cp2 <- col_plan(
    val4,
    span_structure(c2 = c(val1, val3, everything())),
    val5,
    span_structure(c2 = c(val1)),
    -val6,
    -val7,
    .drop = TRUE
  )

  expect_equal(
    create_col_order(cp = cp2, columns = col_vars, data_names = name_col),
    vars(
      val4,
      `test value___tlang_delim___val3`,
      `test value___tlang_delim___val2`,
      val5,
      `test value___tlang_delim___val1`,
      -val6,
      -val7
    ),
    ignore_attr = c(".Environment")
  )

})

test_that("Order is kept for multi-col columns",{

  test <- tibble(col_1 = "test",
                 col_2 = c("this", "other"),
                 col_3 = c("delm", "delm"),
                 label = "label",
                 param = "pam",
                 value = c(1.08089, 9.23948))


  tfrmt <- tfrmt(
    label = label,
    param = param,
    value = value,
    column = vars(col_1, col_2, col_3),
    body_plan = body_plan(
      frmt_structure(pam = frmt("x.xx"))
    ),
    col_style_plan = col_style_plan(
      col_style_structure(align = ".", col = delm),
      col_style_structure(align = "right", col = everything())
      )
  )


  new_name_ord <- apply_tfrmt(test, tfrmt) %>%
    select(-label) %>%
    names()

  new_name_ord_in_dat <- test %>%
    select(
      tidyselect::starts_with("col")
    ) %>%
    unite("new", sep=.tlang_delim) %>%
    pull(new)

  expect_equal(new_name_ord, new_name_ord_in_dat)

})

test_that("Build simple tfrmt with multiple columns and apply to basic data and compare against spanning_structure",{

  basic_multi_column_template <- tfrmt(
    group = group,
    label = quo(label),
    param = parm,
    value = val,
    column = c(test1,test2),
    col_plan = col_plan(
      group,
      label,
      col4,
      span_structure(test1 = `span 1`, test2 = c(col1, col2)),
      col3,
      -col5
    ),
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(location = "spanning"))
  )

  basic_example_dataset <- tibble::tribble(
    ~group,     ~label,    ~test1, ~test2,    ~parm, ~val,
    "g1", "rowlabel1",  "span 1", "col1",  "value",    1,
    "g1", "rowlabel1",  "span 1", "col2",  "value",    1,
    "g1", "rowlabel1",        NA, "col3",  "value",    1,
    "g1", "rowlabel1",        NA, "col4",  "value",    1,
    "g1", "rowlabel1",        NA, "col5",  "value",    1,
    "g1", "rowlabel2",  "span 1", "col1",  "value",    2,
    "g1", "rowlabel2",  "span 1", "col2",  "value",    2,
    "g1", "rowlabel2",        NA, "col3",  "value",    2,
    "g1", "rowlabel2",        NA, "col4",  "value",    2,
    "g1", "rowlabel2",        NA, "col5",  "value",    2,
    "g2", "rowlabel3",  "span 1", "col1",  "value",    3,
    "g2", "rowlabel3",  "span 1", "col2",  "value",    3,
    "g2", "rowlabel3",        NA, "col3",  "value",    3,
    "g2", "rowlabel3",        NA, "col4",  "value",    3,
    "g2", "rowlabel3",        NA, "col5",  "value",    3,
  )

  suppressMessages({
    processed_gt <- print_to_gt(tfrmt = basic_multi_column_template, .data = basic_example_dataset)
  })

  expect_equal(
    processed_gt[["_boxhead"]]$column_label %>% map_chr(as.character),
    c("group", "label", "col4", "col1", "col2", "col3", "..tfrmt_row_grp_lbl"
    )
  )

  expect_equal(
    processed_gt[["_spanners"]]$spanner_label %>% map_chr(as.character),
    c("span 1")
  )

  expect_equal(
    processed_gt[["_spanners"]]$vars,
    list(
      c("span 1___tlang_delim___col1", "span 1___tlang_delim___col2")
    )
  )

})

test_that("Build simple tfrmt with multiple columns and apply to basic data and compare against spanning_structure - with renaming",{

  basic_multi_column_template <- tfrmt(
    group = group,
    label = quo(label),
    param = parm,
    value = val,
    column = c(test1,test2),
    col_plan = col_plan(
      new_col_4 = col4, new_col_1 = col1, col2, col3, -col5
    ),
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(location = "spanning"))
  )


  basic_example_dataset <- tibble::tribble(
    ~group,     ~label,    ~test1, ~test2,    ~parm, ~val,
    "g1", "rowlabel1",  "span 1", "col1",  "value",    1,
    "g1", "rowlabel1",  "span 1", "col2",  "value",    1,
    "g1", "rowlabel1",        NA, "col3",  "value",    1,
    "g1", "rowlabel1",        NA, "col4",  "value",    1,
    "g1", "rowlabel1",        NA, "col5",  "value",    1,
    "g1", "rowlabel2",  "span 1", "col1",  "value",    2,
    "g1", "rowlabel2",  "span 1", "col2",  "value",    2,
    "g1", "rowlabel2",        NA, "col3",  "value",    2,
    "g1", "rowlabel2",        NA, "col4",  "value",    2,
    "g1", "rowlabel2",        NA, "col5",  "value",    2,
    "g2", "rowlabel3",  "span 1", "col1",  "value",    3,
    "g2", "rowlabel3",  "span 1", "col2",  "value",    3,
    "g2", "rowlabel3",        NA, "col3",  "value",    3,
    "g2", "rowlabel3",        NA, "col4",  "value",    3,
    "g2", "rowlabel3",        NA, "col5",  "value",    3,
  )

  suppressMessages({
      processed_gt <- print_to_gt(tfrmt = basic_multi_column_template, .data = basic_example_dataset)
  })


  expect_equal(
    processed_gt[["_boxhead"]]$column_label %>% map_chr(as.character),
    c( "new_col_4", "new_col_1", "col2", "col3", "group", "label", "..tfrmt_row_grp_lbl" )
  )

  expect_equal(
    processed_gt[["_spanners"]]$spanner_label %>% map_chr(as.character),
    c("span 1")
  )

  expect_equal(
    processed_gt[["_spanners"]]$vars,
    list(
      c("span 1___tlang_delim___new_col_1", "span 1___tlang_delim___col2")
    )
  )

})

test_that("Build simple tfrmt with multiple columns and apply to basic data and compare against spanning_structure - with renaming multiple levels",{

  basic_multi_column_template <- tfrmt(
    group = group,
    label = quo(label),
    param = parm,
    value = val,
    column = c(test1,test2),
    col_plan = col_plan(
      new_col_4 = col4,
      span_structure(
        test1 = c(`new span name` = "span 1"),
        test2 = c(new_col_1 = col1, col2)
      ),
      col2, col3, -col5
    ),
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(location = "spanning"))
  )


  basic_example_dataset <- tibble::tribble(
    ~group,     ~label,    ~test1, ~test2,    ~parm, ~val,
    "g1", "rowlabel1",  "span 1", "col1",  "value",    1,
    "g1", "rowlabel1",  "span 1", "col2",  "value",    1,
    "g1", "rowlabel1",        NA, "col3",  "value",    1,
    "g1", "rowlabel1",        NA, "col4",  "value",    1,
    "g1", "rowlabel1",        NA, "col5",  "value",    1,
    "g1", "rowlabel2",  "span 1", "col1",  "value",    2,
    "g1", "rowlabel2",  "span 1", "col2",  "value",    2,
    "g1", "rowlabel2",        NA, "col3",  "value",    2,
    "g1", "rowlabel2",        NA, "col4",  "value",    2,
    "g1", "rowlabel2",        NA, "col5",  "value",    2,
    "g2", "rowlabel3",  "span 1", "col1",  "value",    3,
    "g2", "rowlabel3",  "span 1", "col2",  "value",    3,
    "g2", "rowlabel3",        NA, "col3",  "value",    3,
    "g2", "rowlabel3",        NA, "col4",  "value",    3,
    "g2", "rowlabel3",        NA, "col5",  "value",    3,
  )

  suppressMessages({
    processed_gt <- print_to_gt(tfrmt = basic_multi_column_template, .data = basic_example_dataset)
  })

  expect_equal(
    processed_gt[["_boxhead"]]$column_label %>% map_chr(as.character),
    c( "new_col_4", "new_col_1", "col2", "col3", "group","label","..tfrmt_row_grp_lbl")
  )

  expect_equal(
    processed_gt[["_spanners"]]$spanner_label %>% map_chr(as.character),
    c("new span name")
  )

  expect_equal(
    processed_gt[["_spanners"]]$vars,
    list(
      c("new span name___tlang_delim___new_col_1", "new span name___tlang_delim___col2")
    )
  )

})

test_that("Build simple tfrmt with multiple columns and with renaming duplicated colnames across spans",{

  multi_col_df <- tibble::tribble(
    ~label, ~col0, ~col1, ~col2, ~param, ~value,
    "A",    "A_",  "A",    "AA",  "p1", 123,
    "A",    "A_",  "A",    "AB",  "p1", 12,
    "A",    "A_",  "B",    "AA",  "p1", 1,
    "A",    "A_",  "B",    "AB",  "p1", 123,
    "A",    "B_",  "C",    "AA",  "p1", 12,
    "A",    "B_",  "C",    "AB",  "p1", 1,
    "A",    "B_",  "D",    "BB",  "p1", 123,
    "A",     NA,   NA,    "BB",  "p1", 12,
    "B",    "A_",  "A",    "AA",  "p1", 123,
    "B",    "A_",  "A",    "AB",  "p1", 12,
    "B",    "A_",  "B",    "AA",  "p1", 1,
    "B",    "A_",  "B",    "AB",  "p1", 123,
    "B",    "B_",  "C",    "AA",  "p1", 12,
    "B",    "B_",  "C",    "AB",  "p1", 1,
    "B",    "B_",  "D",    "BB",  "p1", 123,
    "B",     NA,   NA,    "BB",  "p1", 12,
    "B",     NA,   NA,    "CC",  "p1", 12
  )

  multi_column_template <- tfrmt(
    label = label,
    column = c(col0, col1, col2),
    param = param,
    value = value,
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("XXXX"))
    ),
    col_plan = col_plan(
      label,
      span_structure(
        col0 = c(starts_with("A_"), starts_with("B_")),
        col2 = c(renamed_A = AA, AB, renamed_BB = BB)
      ),
      renamed_BB = BB,
      renamed_CC = CC
      ),
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(location = "spanning"))
    )

  suppressMessages({
      processed_gt <- print_to_gt(tfrmt = multi_column_template, .data = multi_col_df)
  })


  expect_equal(
    processed_gt[["_boxhead"]]$column_label %>% map_chr(as.character),
    c("label", "renamed_A", "AB", "renamed_A", "AB", "renamed_A",
      "AB", "renamed_BB", "renamed_BB", "renamed_CC", "..tfrmt_row_grp_lbl"
    )
  )

  expect_equal(
    processed_gt[["_spanners"]]$spanner_label %>% map_chr(as.character),
    c("A",  "B",  "C" , "D" , "A_" ,"B_")
  )

  expect_equal(
    processed_gt[["_spanners"]]$spanner_level,
    c(1,1,1,1,2,2)
  )

  expect_equal(
    processed_gt[["_spanners"]]$vars,
    list(
      c(
        "A____tlang_delim___A___tlang_delim___renamed_A",
        "A____tlang_delim___A___tlang_delim___AB"
      ),
      c(
        "A____tlang_delim___B___tlang_delim___renamed_A",
        "A____tlang_delim___B___tlang_delim___AB"
      ),
      c(
        "B____tlang_delim___C___tlang_delim___renamed_A",
        "B____tlang_delim___C___tlang_delim___AB"
      ),
      "B____tlang_delim___D___tlang_delim___renamed_BB",
      c(
        "A____tlang_delim___A___tlang_delim___renamed_A",
        "A____tlang_delim___A___tlang_delim___AB",
        "A____tlang_delim___B___tlang_delim___renamed_A",
        "A____tlang_delim___B___tlang_delim___AB"
      ),
      c(
        "B____tlang_delim___C___tlang_delim___renamed_A",
        "B____tlang_delim___C___tlang_delim___AB",
        "B____tlang_delim___D___tlang_delim___renamed_BB"
      )
    )
  )

})

test_that("Build simple tfrmt with spans with child spans that are and are not spanned",{

  dat <- tibble::tribble(
    ~group,      ~label,     ~top_span, ~child_span,  ~my_col,    ~parm, ~val,
    "g1", "rowlabel1", "column cols",  "cols 1,2",   "col1",  "value",    1,
    "g1", "rowlabel1", "column cols",  "cols 1,2",   "col2",  "value",    1,
    "g1", "rowlabel1",     "my cols",          NA, "mycol3",  "value",    1,
    "g1", "rowlabel1", "column cols",          NA,   "col4",  "value",    1,
    "g1", "rowlabel1",     "my cols",          NA, "mycol5",  "value",    1,
    "g1", "rowlabel2", "column cols",  "cols 1,2",   "col1",  "value",    2,
    "g1", "rowlabel2", "column cols",  "cols 1,2",   "col2",  "value",    2,
    "g1", "rowlabel2",     "my cols",          NA, "mycol3",  "value",    2,
    "g1", "rowlabel2", "column cols",          NA,   "col4",  "value",    2,
    "g1", "rowlabel2",     "my cols",          NA, "mycol5",  "value",    2,
    "g2", "rowlabel3", "column cols",  "cols 1,2",   "col1",  "value",    3,
    "g2", "rowlabel3", "column cols",  "cols 1,2",   "col2",  "value",    3,
    "g2", "rowlabel3",     "my cols",          NA, "mycol3",  "value",    3,
    "g2", "rowlabel3", "column cols",          NA,   "col4",  "value",    3,
    "g2", "rowlabel3",     "my cols",          NA, "mycol5",  "value",    3
  )

  tfrmt_with_parial_child_span <- tfrmt(
    group = group,
    label = label,
    param = parm,
    value = val,
    column = c(top_span, child_span, my_col),
    col_plan = col_plan(
      group,
      label,
      span_structure(
        top_span = "column cols",
        child_span = "cols 1,2",
        my_col = c("col1", "col2")
      ),
      span_structure(
        top_span = "column cols",
        my_col = "col4"
      ),
      span_structure(
        top_span = "my cols",
        my_col = c(new_col_3 = mycol3, mycol5)
      )
    ),
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(location = "spanning"))
  )


  suppressMessages({
      processed_gt <- print_to_gt(tfrmt = tfrmt_with_parial_child_span, .data = dat)
  })

  expect_equal(
    processed_gt[["_boxhead"]]$column_label %>% map_chr(as.character),
    c("group", "label", "col1", "col2", "col4", "new_col_3", "mycol5",
      "..tfrmt_row_grp_lbl")
  )

  expect_equal(
    processed_gt[["_spanners"]]$spanner_label %>% map_chr(as.character),
    c("cols 1,2","column cols", "my cols" )
  )

  expect_equal(
    processed_gt[["_spanners"]]$spanner_level,
    c(1,2,1)
  )

  expect_equal(
    processed_gt[["_spanners"]]$vars,
    list(
      c(
        "column cols___tlang_delim___cols 1,2___tlang_delim___col1",
        "column cols___tlang_delim___cols 1,2___tlang_delim___col2"
      ),
      c(
        "column cols___tlang_delim___cols 1,2___tlang_delim___col1",
        "column cols___tlang_delim___cols 1,2___tlang_delim___col2",
        "column cols___tlang_delim___NA___tlang_delim___col4"
      ),
      c(
        "my cols___tlang_delim___NA___tlang_delim___new_col_3",
        "my cols___tlang_delim___NA___tlang_delim___mycol5"
      )
    )
  )

})

test_that("Build simple tfrmt with spans with child spans that are and are not spanned - removal",{

  dat <- tibble::tribble(
    ~group,      ~label,     ~top_span, ~child_span,  ~my_col,    ~parm, ~val,
    "g1", "rowlabel1", "column cols",  "cols 1,2",   "col1",  "value",    1,
    "g1", "rowlabel1", "column cols",  "cols 1,2",   "col2",  "value",    1,
    "g1", "rowlabel1",     "my cols",          NA, "mycol3",  "value",    1,
    "g1", "rowlabel1", "column cols",          NA,   "col4",  "value",    1,
    "g1", "rowlabel1",     "my cols",          NA, "mycol5",  "value",    1,
    "g1", "rowlabel2", "column cols",  "cols 1,2",   "col1",  "value",    2,
    "g1", "rowlabel2", "column cols",  "cols 1,2",   "col2",  "value",    2,
    "g1", "rowlabel2",     "my cols",          NA, "mycol3",  "value",    2,
    "g1", "rowlabel2", "column cols",          NA,   "col4",  "value",    2,
    "g1", "rowlabel2",     "my cols",          NA, "mycol5",  "value",    2,
    "g2", "rowlabel3", "column cols",  "cols 1,2",   "col1",  "value",    3,
    "g2", "rowlabel3", "column cols",  "cols 1,2",   "col2",  "value",    3,
    "g2", "rowlabel3",     "my cols",          NA, "mycol3",  "value",    3,
    "g2", "rowlabel3", "column cols",          NA,   "col4",  "value",    3,
    "g2", "rowlabel3",     "my cols",          NA, "mycol5",  "value",    3
  )


  tfrmt_with_parial_child_span <- tfrmt(
    group = group,
    label = label,
    param = parm,
    value = val,
    column = c(top_span, child_span, my_col),
    col_plan = col_plan(
      group,
      label,
      span_structure(
        top_span = "column cols",
        child_span = "cols 1,2",
        my_col = c("col1", "col2")
      ),
      span_structure(
        top_span = "column cols",
        my_col = "col4"
      ),
      span_structure(
        top_span = "my cols",
        my_col = c(new_col_3 = mycol3, -mycol5)
      )
    ),
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(location = "spanning"))
  )


  suppressMessages({
    processed_gt <- print_to_gt(tfrmt = tfrmt_with_parial_child_span, .data = dat)
  })

  expect_equal(
    processed_gt[["_boxhead"]]$column_label %>% map_chr(as.character),
    c("group", "label", "col1", "col2", "col4", "new_col_3",
      "..tfrmt_row_grp_lbl")
  )

  expect_equal(
    processed_gt[["_spanners"]]$spanner_label %>% map_chr(as.character),
    c("cols 1,2","column cols", "my cols" )
  )

  expect_equal(
    processed_gt[["_spanners"]]$spanner_level,
    c(1,2,1)
  )

  expect_equal(
    processed_gt[["_spanners"]]$vars,
    list(
      c(
        "column cols___tlang_delim___cols 1,2___tlang_delim___col1",
        "column cols___tlang_delim___cols 1,2___tlang_delim___col2"
      ),
      c(
        "column cols___tlang_delim___cols 1,2___tlang_delim___col1",
        "column cols___tlang_delim___cols 1,2___tlang_delim___col2",
        "column cols___tlang_delim___NA___tlang_delim___col4"
      ),
      c(
        "my cols___tlang_delim___NA___tlang_delim___new_col_3"
      )
    )
  )

})

test_that("col_plan returns correct errors",{

  expect_error(
    col_plan(
      func("invalid entry")
    ),
    paste0(
      "Invalid entry: `func(\"invalid entry\")`\n",
      "Only span_structures (`span_structure()`),",
      " selection helpers (See <https://tidyselect.r-lib.org/reference>),",
      "  or unquoted expressions representing variable names  can be entered",
      " as contents. Changing the names of individual variables using",
      " new_name = old_name syntax is allowable"
    ),
    fixed = TRUE
  )

})

test_that("span_structure misc, including errors",{

  ## unnamed values
  expect_error(
    span_structure(`blah blah blah`),
    "Entries of a span_stucture must be named:\n span_structure(`blah blah blah`)",
    fixed = TRUE
  )

  ## weird values
  expect_error(
    span_structure(
      name_col = 1i
    ),
    "Unexpected entry type in span_structure()",
    fixed = TRUE
  )

  ## other functions values
  expect_error(
    span_structure(
      name_col = vars(matrix())
    ),
    "Invalid entry: `matrix()`\nOnly selection helpers (See <https://tidyselect.r-lib.org/reference>),",
    fixed = TRUE
  )

})


test_that("Tidyselect subtraction with span_structure",{

  df <- crossing(label = c("label 1", "label 2", "label 3"),
                 column = c("trt1", "trt2", "pl", "trt1&trt2"),
                 param = c("count", "percent")) %>%
    mutate(ord1 = rep(seq(1:length(unique(.$label))), each = nrow(.)/length(unique(.$label)) )) %>%
    mutate(t_or_p = case_when(
      column %in% c("trt1", "trt2", "trt1&trt2") ~ "Treatment",
      column %in% c("pl") ~ "Placebo",
    ))

  df_fake_values <- df %>% mutate(value = runif(nrow(df)))

  tfrmt_minus_selection <-  tfrmt(
      # Specify columns in the data
      label = label,
      column = c(t_or_p, column),
      param = param,
      value = value,
      sorting_cols = c(ord1),
      # Specify body plan
      body_plan = body_plan(
        frmt_structure(
          group_val = ".default",
          label_val = ".default",
          frmt_combine(
            "{count} {percent}",
            count = frmt("XXX"),
            percent = frmt_when(
              "==100"~ frmt(""),
              "==0"~ "",
              "TRUE" ~ frmt("(XX.X%)")
            )
          )
        )
      ),
      # Remove extra cols and create spans
      col_plan = col_plan(
        -starts_with("ord"),
        span_structure(
          t_or_p = "Treatment",
          column = c(T1 = trt1,T2 = trt2,`T1&T2` = `trt1&trt2`)),
        span_structure(
          t_or_p = "Placebo",
          column = c(PL = pl))
      )
    )


  mock_gt <- print_mock_gt(tfrmt_minus_selection, df)

  # keeps the spanners & original cols other than ones that start with "ord".
  # renaming occurs as needed
  expect_equal(
    names(mock_gt$`_data`),
    c("label", "Placebo___tlang_delim___PL", "Treatment___tlang_delim___T1",
      "Treatment___tlang_delim___T1&T2", "Treatment___tlang_delim___T2",
      "..tfrmt_row_grp_lbl"
    )
  )

  real_gt <- print_to_gt(tfrmt_minus_selection, df_fake_values)

  # keeps the spanners & original cols other than ones that start with "ord".
  # renaming occurs as needed
  expect_equal(
    names(real_gt$`_data`),
    c("label", "Placebo___tlang_delim___PL", "Treatment___tlang_delim___T1",
      "Treatment___tlang_delim___T1&T2", "Treatment___tlang_delim___T2",
      "..tfrmt_row_grp_lbl"
    )
  )


  tfrmt_minus_selection_2 <- tfrmt_minus_selection %>%
    tfrmt(
      col_plan = col_plan(
        span_structure(
          t_or_p = "Treatment",
          column = c(T1 = trt1,T2 = trt2,`T1&T2` = `trt1&trt2`)),
        span_structure(
          t_or_p = "Placebo",
          column = c(PL = pl)),
        -starts_with("ord"),
        .drop = TRUE
      )
    )

  mock_gt2 <- print_mock_gt(tfrmt_minus_selection_2, df)


  ## keeps only the spanners, label is dropped
  expect_equal(
    names(mock_gt2$`_data`),
    c("Treatment___tlang_delim___T1",
      "Treatment___tlang_delim___T2",
      "Treatment___tlang_delim___T1&T2",
      "Placebo___tlang_delim___PL",
      "..tfrmt_row_grp_lbl"
    )
  )

  real_gt2 <- print_to_gt(tfrmt_minus_selection_2, df_fake_values)

  ## keeps the spanners & original cols other than ones that start with "ord". renaming occurs as needed
  expect_equal(
    names(real_gt$`_data`),
    c("label", "Placebo___tlang_delim___PL", "Treatment___tlang_delim___T1",
      "Treatment___tlang_delim___T1&T2", "Treatment___tlang_delim___T2",
      "..tfrmt_row_grp_lbl"
    )
  )

})




test_that("Build simple tfrmt with stub header",{

  basic_multi_column_template <- tfrmt(
    group = group,
    label = quo(label),
    param = parm,
    value = val,
    column = c(test1,test2),
    col_plan = col_plan(
      grp = group,
      label,
      tst = col4,
      col3,
      col1,
      -col5,
      -col2
    ),
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(location = "indented"))
  )

  basic_example_dataset <- tibble::tribble(
    ~group,     ~label,    ~test1, ~test2,    ~parm, ~val,
    "g1", "rowlabel1",  "span 1", "col1",  "value",    1,
    "g1", "rowlabel1",  "span 1", "col2",  "value",    1,
    "g1", "rowlabel1",        NA, "col3",  "value",    1,
    "g1", "rowlabel1",        NA, "col4",  "value",    1,
    "g1", "rowlabel1",        NA, "col5",  "value",    1,
    "g1", "rowlabel2",  "span 1", "col1",  "value",    2,
    "g1", "rowlabel2",  "span 1", "col2",  "value",    2,
    "g1", "rowlabel2",        NA, "col3",  "value",    2,
    "g1", "rowlabel2",        NA, "col4",  "value",    2,
    "g1", "rowlabel2",        NA, "col5",  "value",    2,
    "g2", "rowlabel3",  "span 1", "col1",  "value",    3,
    "g2", "rowlabel3",  "span 1", "col2",  "value",    3,
    "g2", "rowlabel3",        NA, "col3",  "value",    3,
    "g2", "rowlabel3",        NA, "col4",  "value",    3,
    "g2", "rowlabel3",        NA, "col5",  "value",    3,
  )

  suppressMessages({
    processed_gt <- print_to_gt(
      tfrmt = basic_multi_column_template,
      .data = basic_example_dataset
    )
  })

  expect_equal(
    processed_gt[["_boxhead"]]$column_label %>% map_chr(as.character),
    c("label", "tst", "col3", "col1", "..tfrmt_row_grp_lbl"
    )
  )
  expect_equal(
    processed_gt[["_stubhead"]]$label,
    md("grp")
  )


  # no stubhead if no group column
  basic_multi_column_template2 <- tfrmt(
    label = quo(label),
    param = parm,
    value = val,
    column = c(test1,test2),
    col_plan = col_plan(
      label,
      tst = col4,
      col3,
      col1,
      -col5,
      -col2
    )
  )

  basic_example_dataset2 <- tibble::tribble(
    ~label,    ~test1, ~test2,    ~parm, ~val,
    "rowlabel1",  "span 1", "col1",  "value",    1,
    "rowlabel1",  "span 1", "col2",  "value",    1,
    "rowlabel1",        NA, "col3",  "value",    1,
    "rowlabel1",        NA, "col4",  "value",    1,
    "rowlabel1",        NA, "col5",  "value",    1,
    "rowlabel2",  "span 1", "col1",  "value",    2,
    "rowlabel2",  "span 1", "col2",  "value",    2,
    "rowlabel2",        NA, "col3",  "value",    2,
    "rowlabel2",        NA, "col4",  "value",    2,
    "rowlabel2",        NA, "col5",  "value",    2,
    "rowlabel3",  "span 1", "col1",  "value",    3,
    "rowlabel3",  "span 1", "col2",  "value",    3,
    "rowlabel3",        NA, "col3",  "value",    3,
    "rowlabel3",        NA, "col4",  "value",    3,
    "rowlabel3",        NA, "col5",  "value",    3,
  )

  suppressMessages({
    processed_gt <- print_to_gt(tfrmt = basic_multi_column_template2, .data = basic_example_dataset2)
  })

  expect_equal(
    processed_gt[["_boxhead"]]$column_label %>% map_chr(as.character),
    c("label", "tst", "col3", "col1", "..tfrmt_row_grp_lbl"
    )
  )
  expect_equal(
    processed_gt[["_stubhead"]]$label,
    md("")
  )


  # multi group stub header
  basic_multi_column_template <- tfrmt(
    group = c(grp1, grp2),
    label = quo(label),
    param = parm,
    value = val,
    column = c(test1,test2),
    col_plan = col_plan(
      `Group 1` = grp1,
      `Group 2` = grp2,
      `Row label` =label,
      tst = col4,
      col3,
      col1,
      -col5,
      -col2
    ),
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(location = "column"))
  )

  basic_example_dataset <- tibble::tribble(
    ~grp1, ~grp2,     ~label,    ~test1, ~test2,    ~parm, ~val,
    "G1", "g1", "rowlabel1",  "span 1", "col1",  "value",    1,
    "G1", "g1", "rowlabel1",  "span 1", "col2",  "value",    1,
    "G1", "g1", "rowlabel1",        NA, "col3",  "value",    1,
    "G1", "g1", "rowlabel1",        NA, "col4",  "value",    1,
    "G1", "g1", "rowlabel1",        NA, "col5",  "value",    1,
    "G1", "g1", "rowlabel2",  "span 1", "col1",  "value",    2,
    "G1", "g1", "rowlabel2",  "span 1", "col2",  "value",    2,
    "G1", "g1", "rowlabel2",        NA, "col3",  "value",    2,
    "G1", "g1", "rowlabel2",        NA, "col4",  "value",    2,
    "G1", "g1", "rowlabel2",        NA, "col5",  "value",    2,
    "G1", "g2", "rowlabel3",  "span 1", "col1",  "value",    3,
    "G1", "g2", "rowlabel3",  "span 1", "col2",  "value",    3,
    "G1", "g2", "rowlabel3",        NA, "col3",  "value",    3,
    "G1", "g2", "rowlabel3",        NA, "col4",  "value",    3,
    "G1", "g2", "rowlabel3",        NA, "col5",  "value",    3,
  )

  suppressMessages({
    processed_gt <- print_to_gt(tfrmt = basic_multi_column_template, .data = basic_example_dataset)
  })

  expect_equal(
    processed_gt[["_boxhead"]]$column_label %>% map_chr(as.character),
    c("grp1", "grp2", "label", "tst", "col3", "col1", "..tfrmt_row_grp_lbl"
    )
  )
  expect_equal(
    processed_gt[["_stubhead"]]$label,
    md(c("Group 1", "Group 2", "Row label"))
  )

})

test_that("Mock data column names are correct", {

  plan  <- tfrmt(
    group = "my_group",
    label = "my_label",
    param = "param2",
    value = "val2",
    column = "col",
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("xx.x"))
    )
  )
  mock_dat <- make_mock_data(plan)


  expect_equal(c("my_group", "my_label", "param2", "col"),
               names(mock_dat))
})

test_that("Mock data contains all levels", {

  # handle 2 group vars
  plan  <- tfrmt(
    group = vars(grp1, grp2),
    label = "my_label",
    param = "param2",
    value = "val2",
    column = "col",
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("xx.x"))
    )
  )
  mock_dat <- make_mock_data(plan, .default = 1:2, n_cols = 1)

  expect_equal(mock_dat,
               tibble::tribble(
                 ~grp1,    ~grp2,     ~ my_label,  ~param2,  ~col ,
                 "grp1_1", "grp2_1", "my_label_1", "param2_1", "col1" ,
                 "grp1_1", "grp2_1", "my_label_2", "param2_1", "col1" ,
                 "grp1_1", "grp2_2", "my_label_1", "param2_1", "col1",
                 "grp1_1", "grp2_2", "my_label_2", "param2_1", "col1",
                 "grp1_2", "grp2_1", "my_label_1", "param2_1", "col1",
                 "grp1_2", "grp2_1", "my_label_2", "param2_1", "col1",
                 "grp1_2", "grp2_2", "my_label_1", "param2_1", "col1",
                 "grp1_2", "grp2_2", "my_label_2", "param2_1", "col1"
               ))

  # handle many group vars
  plan  <- tfrmt(
    group = vars(grp1, grp2, grp3, grp4),
    label = "my_label",
    param = "param2",
    value = "val2",
    column = "col",
    body_plan = body_plan(
      frmt_structure(group_val = list(grp1 = "A", grp2 = c("a","b")), label_val = ".default", frmt("xx.x")),
      frmt_structure(group_val = list(grp1 = "B", grp2 = c("a","b")), label_val = ".default", frmt("xx.x")),
      frmt_structure(group_val = list(grp3 = "C", grp4 = c("a","b")), label_val = ".default", frmt("xx.x")),
      frmt_structure(group_val = list(grp3 = "D", grp4 = c("a","b")), label_val = ".default", frmt("xx.x"))
    )
  )
  mock_dat <- make_mock_data(plan, .default = 1, n_cols = 1)

  expect_equal(mock_dat,
               tibble::tribble(
                 ~grp1,    ~grp2  ,  ~grp3   , ~grp4   ,  ~my_label  , ~param2   ,~col,
                 "A"     , "a"      ,"grp3_1" ,"grp4_1" ,"my_label_1" ,"param2_1" ,"col1" ,
                 "A"     , "b"      ,"grp3_1" ,"grp4_1" ,"my_label_1" ,"param2_1" ,"col1" ,
                 "B"     , "a"      ,"grp3_1" ,"grp4_1" ,"my_label_1" ,"param2_1" ,"col1" ,
                 "B"     , "b"      ,"grp3_1" ,"grp4_1" ,"my_label_1" ,"param2_1" ,"col1" ,
                 "grp1_1", "grp2_1" ,"C"      ,"a"      ,"my_label_1" ,"param2_1" ,"col1" ,
                 "grp1_1", "grp2_1" ,"C"      ,"b"      ,"my_label_1" ,"param2_1" ,"col1" ,
                 "grp1_1", "grp2_1" ,"D"      ,"a"      ,"my_label_1" ,"param2_1" ,"col1" ,
                 "grp1_1" ,"grp2_1" ,"D"      ,"b"      ,"my_label_1" ,"param2_1" ,"col1",
               ))




  # group & label values specified
  plan  <- tfrmt(
    group = vars(grp1, grp2),
    label = "my_label",
    param = "param2",
    value = "val2",
    column = "col",
    body_plan = body_plan(
      frmt_structure(group_val = list(grp1 = ".default", grp2 = c("c","d")), label_val = c("e","f"), mean = frmt("xx.x"))
    )
  )
  mock_dat <- make_mock_data(plan, .default = 1, n_cols = 1)

  expect_equal(mock_dat,
               tibble::tribble(
                 ~ grp1  , ~ grp2, ~ my_label, ~ param2, ~ col,
                 "grp1_1", "c"   , "e"       , "mean"  , "col1",
                 "grp1_1", "c"   , "f"       , "mean"  , "col1",
                 "grp1_1", "d"   , "e"       , "mean"  , "col1",
                 "grp1_1", "d"   , "f"       , "mean"  , "col1"
               ))


  # group & label specified + multiple levels/columns
  mock_dat <- make_mock_data(plan, .default = 1:2, n_cols = 2)

  expect_equal(mock_dat,
               tibble::tribble(
                 ~ grp1  , ~ grp2, ~ my_label, ~ param2, ~ col,
                 "grp1_1", "c"   , "e"       , "mean"  , "col1",
                 "grp1_1", "c"   , "e"       , "mean"  , "col2",
                 "grp1_1", "c"   , "f"       , "mean"  , "col1",
                 "grp1_1", "c"   , "f"       , "mean"  , "col2",
                 "grp1_1", "d"   , "e"       , "mean"  , "col1",
                 "grp1_1", "d"   , "e"       , "mean"  , "col2",
                 "grp1_1", "d"   , "f"       , "mean"  , "col1",
                 "grp1_1", "d"   , "f"       , "mean"  , "col2",
                 "grp1_2", "c"   , "e"       , "mean"  , "col1",
                 "grp1_2", "c"   , "e"       , "mean"  , "col2",
                 "grp1_2", "c"   , "f"       , "mean"  , "col1",
                 "grp1_2", "c"   , "f"       , "mean"  , "col2",
                 "grp1_2", "d"   , "e"       , "mean"  , "col1",
                 "grp1_2", "d"   , "e"       , "mean"  , "col2",
                 "grp1_2", "d"   , "f"       , "mean"  , "col1",
                 "grp1_2", "d"   , "f"       , "mean"  , "col2"
               ))

  # multiple frmt_structure
  plan  <- tfrmt(
    group = vars(grp1, grp2),
    label = "my_label",
    param = "param2",
    value = "val2",
    column = "col",
    body_plan = body_plan(
      frmt_structure(group_val = list(grp1 = ".default", grp2 = c("c","d")), label_val = c("e","f"), mean = frmt("xx.x")),
      frmt_structure(group_val = ".default", label_val = ".default", N = frmt("xx"))
    )
  )
  mock_dat <- make_mock_data(plan, .default = 1:2, n_cols = 1)

  expect_equal(mock_dat,
               tibble::tribble(
                 ~ grp1  , ~ grp2  , ~ my_label    , ~ param2, ~ col,
                 "grp1_1", "c"     , "e"           , "mean"  , "col1",
                 "grp1_1", "c"     , "f"           , "mean"  , "col1",
                 "grp1_1", "d"     , "e"           , "mean"  , "col1",
                 "grp1_1", "d"     , "f"           , "mean"  , "col1",
                 "grp1_2", "c"     , "e"           , "mean"  , "col1",
                 "grp1_2", "c"     , "f"           , "mean"  , "col1",
                 "grp1_2", "d"     , "e"           , "mean"  , "col1",
                 "grp1_2", "d"     , "f"           , "mean"  , "col1",
                 "grp1_1", "grp2_1", "my_label_1"  , "N"     , "col1",
                 "grp1_1", "grp2_1", "my_label_2"  , "N"     , "col1",
                 "grp1_1", "grp2_2", "my_label_1"  , "N"     , "col1",
                 "grp1_1", "grp2_2", "my_label_2"  , "N"     , "col1",
                 "grp1_2", "grp2_1", "my_label_1"  , "N"     , "col1",
                 "grp1_2", "grp2_1", "my_label_2"  , "N"     , "col1",
                 "grp1_2", "grp2_2", "my_label_1"  , "N"     , "col1",
                 "grp1_2", "grp2_2", "my_label_2"  , "N"     , "col1"
               ))

  # no duplicate levels introduced if body plan references a dummy level
  plan1  <- tfrmt(
    group = "grp",
    label = "my_label",
    param = "param2",
    value = "val2",
    column = "col",
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("xx.x")),
      frmt_structure(group_val = list(grp = "grp_1"), label_val = ".default", frmt("xx.xx")
    )
    )
  )
  mock_dat1 <- make_mock_data(plan1, .default = 1:2, n_cols = 1)

  plan2  <- tfrmt(
    group = "grp",
    label = "my_label",
    param = "param2",
    value = "val2",
    column = "col",
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("xx.x"))
      )
    )
  mock_dat2 <- make_mock_data(plan2, .default = 1:2, n_cols = 1)

  expect_equal(mock_dat1, mock_dat2)
})


test_that("Check mock when value is missing", {
  data <- crossing(
    label = c("Intent-To-Treat (ITT)",
              "Safety",
              "Efficacy",
              "Complete Week 24",
              "Complete Study"
    ),
    column = c("Placebo\n(N=XX)",
               "Xanomeline\nLow Dose\n(N=XX)",
               "Xanomeline\nHigh Dose\n(N=XX)",
               "Total\n(N=XXX)"
    ),
    param = c("n", "percent")
  )

  plan <- tfrmt(
    label = "label",
    column = "column",
    param = "param",
    title = "Summary of Populations",
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default", label_val = ".default",
        frmt_combine("{n} ({percent}%)",
                     n = frmt("xx"),
                     percent = frmt("xxx"))
      )
    )
  )

  #Make mock
  expect_snapshot(
    out <- print_mock_gt(plan, data)
  )
})


test_that("Test when no body_style or values is present", {

  tfrmt_obj_one_span <- tfrmt(
    label = label,
    group = group,
    param = param,
    column = vars(spanner, columns),
    col_plan = col_plan(
      group,label,
      col1, col2,
      span_structure(spanner = "test label", columns= col3),
      vars(col4:col10)
      ),
    row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc("gtdefault"))
    )

  input_data <- tibble(
    group = "groupvar",
    label = "labels",
    param = "params",
    spanner = "test label",
    columns = paste0("col",1:10)
  )


  expect_message(
    gt_out <- print_mock_gt(tfrmt_obj_one_span, input_data),
    "Message: `tfrmt` will need `value` value to `print_to_gt` when data is available",
    fixed = TRUE
  )

  expect_equal(gt_out$`_data`,
               input_data %>%
                 mutate(val = "X.X") %>%
                 pivot_wider(
                   names_from = c(spanner, columns),
                   names_sep =  .tlang_delim,
                   values_from = val
                   ) %>%
                 select(-param) %>%
                 mutate(..tfrmt_row_grp_lbl=FALSE)
               )


})

test_that("Mock data contains sorting_cols when available", {

  # handle single sorting cols
  plan  <- tfrmt(
    group = vars(grp1, grp2),
    label = "my_label",
    param = "param2",
    value = "val2",
    column = "col",
    sorting_cols = c(ord1),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("xx.x"))
    )
  )

  mock_dat <- make_mock_data(plan, .default = 1:2, n_cols = 1)

  expect_equal(mock_dat,
               tibble::tribble(
                 ~grp1,    ~grp2,     ~ my_label,  ~param2,    ~ord1,   ~col,
                 "grp1_1", "grp2_1", "my_label_1", "param2_1",     1, "col1",
                 "grp1_1", "grp2_1", "my_label_2", "param2_1",     1, "col1",
                 "grp1_1", "grp2_2", "my_label_1", "param2_1",     1, "col1",
                 "grp1_1", "grp2_2", "my_label_2", "param2_1",     1, "col1",
                 "grp1_2", "grp2_1", "my_label_1", "param2_1",     1, "col1",
                 "grp1_2", "grp2_1", "my_label_2", "param2_1",     1, "col1",
                 "grp1_2", "grp2_2", "my_label_1", "param2_1",     1, "col1",
                 "grp1_2", "grp2_2", "my_label_2", "param2_1",     1, "col1"
               ))

  # handle 2 sorting cols
  plan  <- tfrmt(
    group = vars(grp1, grp2),
    label = "my_label",
    param = "param2",
    value = "val2",
    column = "col",
    sorting_cols = c(ord1, ord2),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("xx.x"))
    )
  )

  mock_dat <- make_mock_data(plan, .default = 1:2, n_cols = 1)

  expect_equal(mock_dat,
               tibble::tribble(
                 ~grp1,    ~grp2,     ~ my_label,  ~param2,    ~ord1, ~ord2,  ~col,
                 "grp1_1", "grp2_1", "my_label_1", "param2_1",     1,     1,"col1",
                 "grp1_1", "grp2_1", "my_label_2", "param2_1",     1,     1,"col1",
                 "grp1_1", "grp2_2", "my_label_1", "param2_1",     1,     1,"col1",
                 "grp1_1", "grp2_2", "my_label_2", "param2_1",     1,     1,"col1",
                 "grp1_2", "grp2_1", "my_label_1", "param2_1",     1,     1,"col1",
                 "grp1_2", "grp2_1", "my_label_2", "param2_1",     1,     1,"col1",
                 "grp1_2", "grp2_2", "my_label_1", "param2_1",     1,     1,"col1",
                 "grp1_2", "grp2_2", "my_label_2", "param2_1",     1,     1,"col1"
               ))

})

test_that("Mock data includes all columns identified in tfrmt", {

  # handle one column
  plan  <- tfrmt(
    group = grp,
    label = "my_label",
    param = "param2",
    value = "val2",
    column = col,
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("xx.x"))
    )
  )

  mock_dat <- make_mock_data(plan, .default = 1:2, n_cols = 2)

  expect_equal(mock_dat,
               tibble::tribble(
                 ~grp,   ~ my_label,    ~param2,     ~col,
                 "grp_1", "my_label_1", "param2_1", "col1",
                 "grp_1", "my_label_1", "param2_1", "col2",
                 "grp_1", "my_label_2", "param2_1", "col1",
                 "grp_1", "my_label_2", "param2_1", "col2",
                 "grp_2", "my_label_1", "param2_1", "col1",
                 "grp_2", "my_label_1", "param2_1", "col2",
                 "grp_2", "my_label_2", "param2_1", "col1",
                 "grp_2", "my_label_2", "param2_1", "col2"
               ))

  # handle two columns
  plan  <- tfrmt(
    group = grp,
    label = "my_label",
    param = "param2",
    value = "val2",
    column = c(col1,col2),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("xx.x"))
    )
  )

  mock_dat <- make_mock_data(plan, .default = 1:2, n_cols = 2)

  expect_equal(mock_dat,
               tibble::tribble(
                    ~grp,   ~ my_label,    ~param2,       ~col1,  ~col2,
                 "grp_1", "my_label_1", "param2_1", "span_col1", "col21",
                 "grp_1", "my_label_1", "param2_1", "span_col1", "col22",
                 "grp_1", "my_label_2", "param2_1", "span_col1", "col21",
                 "grp_1", "my_label_2", "param2_1", "span_col1", "col22",
                 "grp_2", "my_label_1", "param2_1", "span_col1", "col21",
                 "grp_2", "my_label_1", "param2_1", "span_col1", "col22",
                 "grp_2", "my_label_2", "param2_1", "span_col1", "col21",
                 "grp_2", "my_label_2", "param2_1", "span_col1", "col22"
               ))

  # handle three columns
  plan  <- tfrmt(
    group = grp,
    label = "my_label",
    param = "param2",
    value = "val2",
    column = c(col1,col2,col3),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("xx.x"))
    )
  )

  mock_dat <- make_mock_data(plan, .default = 1:2, n_cols = 2)

  expect_equal(mock_dat,
               tibble::tribble(
                    ~grp,   ~ my_label,    ~param2,       ~col1,       ~col2,  ~col3,
                 "grp_1", "my_label_1", "param2_1", "span_col1", "span_col2", "col31",
                 "grp_1", "my_label_1", "param2_1", "span_col1", "span_col2", "col32",
                 "grp_1", "my_label_2", "param2_1", "span_col1", "span_col2", "col31",
                 "grp_1", "my_label_2", "param2_1", "span_col1", "span_col2", "col32",
                 "grp_2", "my_label_1", "param2_1", "span_col1", "span_col2", "col31",
                 "grp_2", "my_label_1", "param2_1", "span_col1", "span_col2", "col32",
                 "grp_2", "my_label_2", "param2_1", "span_col1", "span_col2", "col31",
                 "grp_2", "my_label_2", "param2_1", "span_col1", "span_col2", "col32"
               ))

})

test_that("Mock data can be made and printed without label",{


  plan <- tfrmt(
    column = "column",
    param = "param",
    value = "value_to_remove",
    title = "Summary of Populations",
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default", label_val = ".default",
        frmt_combine("{n} ({percent}%)",
                     n = frmt("xx"),
                     percent = frmt("xxx"))
      )
    )
  )

  dat <- make_mock_data(plan)

  expect_equal(
    dat,
    tibble(
      param = c("n","n","n","percent","percent","percent"),
      column = paste0("column", rep(1:3, times = 2))
    )
  )

  #Make mock
  expect_silent(
    print_mock_gt(plan, .data = dat)
  )

})


test_that("Mock data can be printed from a tfrmt without a body plan",{


  plan <- tfrmt(
    column = "column",
    param = "param",
    value = "value_to_remove",
    title = "Summary of Populations"
  )

  mock_gt <- print_mock_gt(plan)

  #Make mock
  expect_equal(
    mock_gt,
    print_mock_gt(plan, .data = tibble::tibble(column = c("column1","column2","column3"), param = "n")),
    ignore_function_env = TRUE,
    ignore_attr = TRUE
  )


})

test_that("Using col_plan to get column names", {
  # Without any spanning
  basic_cols <- tfrmt(
    group = "group",
    label = "label",
    column = "column",
    param = "param",
    value = "value",
    sorting_cols = c(ord1, ord2),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("X.X"))
    ),
    col_plan = col_plan(
      group, label,
      "Placebo",
      new_name = "Low",
      contains("High")
    )
  )

  col_names <- make_mock_data(basic_cols) %>%
    pull(column) %>%
    unique()
  expect_equal(col_names, c("Placebo", "Low", "High"))

  #With spanning
  auto_col_df <- tfrmt(
    group = group,
    label = quo(label),
    param = parm,
    column = c(test1,test2),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("X.X"))
    ),
    col_plan = col_plan(
      group,
      label,
      col4,
      span_structure(test1 = `span 1`, test2 = c(col1, contains("col2"))),
      span_structure(test1 = `span 2`, test2 = c(col7, col8)),
      col3,
      -col5
    )
  ) %>%
    make_mock_data() %>%
    select(test1, test2) %>%
    distinct(test1, test2)

  man_col_df <- tibble(test1 = c(rep(NA, 3), rep(c("span 1", "span 2"), each = 2)),
                       test2 = c("col4","col3", "col5", "col1", "col2", "col7","col8"))
  expect_equal(auto_col_df, man_col_df)


# When you do crossing in the span structure
  auto_col_crossing <- tfrmt(
    group = group,
    label = quo(label),
    param = parm,
    column = c(visit,trt),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("X.X"))
    ),
  col_plan = col_plan(
    model_results_category,
    measure,
    span_structure(
      visit = c(`Week 4`,`Week 8`, `Week 12`),
      trt = c(`Placebo`,`GSK123456 100 mg`)
    ),
    -starts_with("ord")
  )) %>%
    make_mock_data() %>%
    distinct(trt, visit)

  man_col_crossing <- tibble::tribble(
    ~trt ,                   ~visit ,
    "model_results_category", NA_character_,
    "measure",                NA_character_,
    "ord",                    NA_character_,
    "GSK123456 100 mg",       "Week 12",
    "Placebo",                "Week 12",
    "GSK123456 100 mg",       "Week 4",
    "Placebo",                "Week 4",
    "GSK123456 100 mg",       "Week 8",
    "Placebo",                "Week 8",
  )

  expect_equal(auto_col_crossing, man_col_crossing)



})

test_that("Using col_style_plan to get names",{

  basic_cols <- tfrmt(
    group = "group",
    label = "label",
    column = "column",
    param = "param",
    value = "value",
    sorting_cols = c(ord1, ord2),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("X.X"))
    ),
    col_style_plan = col_style_plan(
      col_style_structure(align = ".", col = starts_with("Active")),
      col_style_structure(align = " ", col = c("Placebo","Total"))
    )
  )

  col_names <- make_mock_data(basic_cols) %>%
    pull(column) %>%
    unique()
  expect_equal(col_names, c("Active","Placebo","Total"))


  # combination col_style_plan/col_plan
  auto_col_df <- tfrmt(
    group = group,
    label = quo(label),
    param = parm,
    column = c(test1,test2),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("X.X"))
    ),
    col_plan = col_plan(
      group,
      label,
      col4,
      span_structure(test1 = `span 1`, test2 = c(col1, contains("col2"))),
      span_structure(test1 = `span 2`, test2 = c(col7, col8)),
      col3,
      -col5
    ),
    col_style_plan = col_style_plan(
      col_style_structure(align = ".", col = starts_with("Active")),
      col_style_structure(align = " ", col = c("Placebo","Total"))
    )
  ) %>%
    make_mock_data() %>%
    select(test1, test2) %>%
    distinct(test1, test2)

  man_col_df <- tibble(test1 = c(rep(NA, 3), rep(c("span 1", "span 2"), each = 2), rep(NA, 3)),
                       test2 = c("col4","col3", "col5", "col1", "col2", "col7","col8", "Active","Placebo","Total"))
  expect_equal(auto_col_df, man_col_df)
})

test_that("Will add big N avaliable", {
  pop_tbl_tfrmt <- tfrmt(
    column = TRT01A,
    label = name,
    param = param,
    value = value,
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default",
                     frmt_combine("{n} ({pct}%)",
                                  n = frmt("xx"),
                                  pct = frmt("xx")))
    ),
    big_n = big_n_structure(param_val = "big_n"),
    col_plan = col_plan(
      starts_with("Xanomeline"),
      "Placebo",
      "Total"
    )
  )

  auto_big_n_df <- make_mock_data(pop_tbl_tfrmt) %>%
    filter(param == "big_n")

  man_big_n_df <- tibble::tribble(
    ~name,  ~param,  ~TRT01A,
    NA_character_,    "big_n", "Xanomeline",
    NA_character_,    "big_n", "Placebo"   ,
    NA_character_,    "big_n", "Total"
  )

  expect_equal(auto_big_n_df, man_big_n_df)

})

test_that("Mock data for col_plan with only drops", {

  drop_tfrmt <- tfrmt(
    # specify columns in the data
    group = c(rowlbl1,grp),
    label = rowlbl2,
    column = column,
    param = "param",
    value = value,
    sorting_cols = c(ord1, ord2),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default",
                     frmt_combine("{n} ({pct}%)",
                                  n = frmt("xx"),
                                  pct = frmt("xx")))
    ),
    # remove extra cols
    col_plan = col_plan(-grp,
                        -starts_with("ord") )
  )

  make_mock_data(drop_tfrmt) %>%
    pull(column) %>%
    unique() %>%
    expect_equal(c("column1", "column2", "column3"))
})

test_that("Mock data for col_plan does not add group, label, or sorting_cols names to `column` variable", {


  tf_cols <- tfrmt(
    group="grp",
    label="lbl",
    param="prm",
    column="col",
    value="val",
    sorting_cols = c("ord1","ord2"),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("xx.x"))
    ),
    col_plan = col_plan(
      grp,
      lbl,
      col1,
      col2,
      -ord1,
      -ord2
    )
  )

  make_mock_data(tf_cols) %>%
    pull(col) %>%
    unique() %>%
    expect_equal(c("col1","col2"))

})

test_that("Mock data ignores col_plan with everything()",{

  tf_everything <- tfrmt(
    group = "group",
    label = "label",
    column = "column",
    param = "param",
    value = "value",
    sorting_cols = c(ord1, ord2),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("X.X"))
    ),
    col_plan = col_plan(
      -Total,
      everything()
    ))

  col_names <- make_mock_data(tf_everything) %>%
    pull(column) %>%
    unique()
  expect_equal(col_names, c("column1","column2","column3"))
})

test_that("tfrmt_n_pct", {

  # format to avoid issues with the environment
  expect_equal(format(tfrmt_n_pct()$body_plan),
               body_plan(
                 frmt_structure(
                   group_val = ".default", label_val = ".default",
                   frmt_combine("{n} {pct}", n = frmt("xxx"),
                                pct = frmt_when("==100"~ frmt(""),
                                                ">99"~ frmt("(>99%)"),
                                                "==0"~ "",
                                                "<1" ~ frmt("(<1%)"),
                                                "TRUE" ~ frmt("(xx.x%)")))
                 )
               ) %>% format()
  )
  # See that it can change when n is changed
  expect_equal(format(tfrmt_n_pct("n_distinct")$body_plan),
               body_plan(
                 frmt_structure(
                   group_val = ".default", label_val = ".default",
                   frmt_combine("{n_distinct} {pct}", n_distinct = frmt("xxx"),
                                pct = frmt_when("==100"~ frmt(""),
                                                ">99"~ frmt("(>99%)"),
                                                "==0"~ "",
                                                "<1" ~ frmt("(<1%)"),
                                                "TRUE" ~ frmt("(xx.x%)")))
                 )
               ) %>% format()
  )
  # Change the frmt_when
  expect_equal(format(tfrmt_n_pct(pct_frmt_when =
                                   frmt_when(
                                     "==100" ~ "",
                                     "==0" ~ "",
                                     TRUE ~ frmt("(xx.x %)")
                                   ))$body_plan),
               body_plan(
                 frmt_structure(
                   group_val = ".default", label_val = ".default",
                   frmt_combine("{n} {pct}", n = frmt("xxx"),
                                pct = frmt_when(
                                  "==100" ~ "",
                                  "==0" ~ "",
                                  TRUE ~ frmt("(xx.x %)")
                                ))
                 )
               ) %>% format()
  )

  # layer with existing tfrmt
  test <- tfrmt(
    column = column
  )
  expect_equal(tfrmt_n_pct(tfrmt_obj = test)$column[[1]] %>%
    as_label(),
    "column")

  expect_error(tfrmt_n_pct(n = ""))
  expect_error(tfrmt_n_pct(pct = NULL))

})

test_that("extract_data works for a single gt_tbl object", {
  data_demog_test <- data_demog |>
    filter(
      rowlbl1 %in% c("Age (y)", "Sex"),
      column != "p-value"
    )

  tfrmt_dem <- tfrmt(
    # specify columns in the data
    group = c(rowlbl1, grp),
    label = rowlbl2,
    column = column,
    param = param,
    value = value,
    sorting_cols = c(ord1, ord2),
    # specify value formatting
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
        n = frmt("xxx"),
        pct = frmt_when(
          "==100" ~ "",
          "==0" ~ "",
          TRUE ~ frmt("(xx.x %)")
        )
      )),
      frmt_structure(group_val = ".default", label_val = "n", frmt("xxx")),
      frmt_structure(group_val = ".default", label_val = c("Mean", "Median", "Min", "Max"), frmt("xxx.x")),
      frmt_structure(group_val = ".default", label_val = "SD", frmt("xxx.xx"))
    ),
    # remove extra cols
    col_plan = col_plan(
      -grp,
      -starts_with("ord")
    )
  ) |>
    print_to_gt(data_demog_test)

  tfrmt_data_extracted <- extract_data(tfrmt_dem)
  tfrmt_data_manual <- tfrmt_dem[["_data"]] |>
    select(-`..tfrmt_row_grp_lbl`)

  expect_s3_class(tfrmt_data_extracted, "data.frame")
  expect_equal(tfrmt_data_extracted, tfrmt_data_manual)
  expect_true(nrow(tfrmt_data_extracted) > 0)
})

test_that("extract_data extracts updated names changed in the col_plan including group/label vars", {
  data_demog_test <- data_demog |>
    filter(
      rowlbl1 %in% c("Age (y)", "Sex"),
      column != "p-value"
    )

  tfrmt_dem <- tfrmt(
    # specify columns in the data
    group = c(rowlbl1, grp),
    label = rowlbl2,
    column = column,
    param = param,
    value = value,
    sorting_cols = c(ord1, ord2),
    # specify value formatting
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
        n = frmt("xxx"),
        pct = frmt_when(
          "==100" ~ "",
          "==0" ~ "",
          TRUE ~ frmt("(xx.x %)")
        )
      )),
      frmt_structure(group_val = ".default", label_val = "n", frmt("xxx")),
      frmt_structure(group_val = ".default", label_val = c("Mean", "Median", "Min", "Max"), frmt("xxx.x")),
      frmt_structure(group_val = ".default", label_val = "SD", frmt("xxx.xx"))
    ),
    row_grp_plan = row_grp_plan(row_grp_structure(group_val = ".default", element_block(post_space = "   ")),
      label_loc = element_row_grp_loc(location = "column")
    ),
    # remove extra cols
    col_plan = col_plan(
      -grp,
      "rowlbl1_new" = rowlbl1,
      "rowlbl2_new" = rowlbl2,
      -starts_with("ord"),
      PLACEBO = Placebo,
      TOTAL = Total,
      `XANOMELINE LOW DOSE` = `Xanomeline Low Dose`
    )
  ) |>
    print_to_gt(data_demog_test)

  tfrmt_data_extracted <- extract_data(tfrmt_dem)

  tfrmt_data_manual <- tfrmt_dem[["_data"]] |>
    rename(
      "rowlbl1_new" = "rowlbl1",
      "rowlbl2_new" = "rowlbl2"
    ) |>
    select(-`..tfrmt_row_grp_lbl`)

  expect_s3_class(tfrmt_data_extracted, "data.frame")
  expect_equal(tfrmt_data_extracted, tfrmt_data_manual)
  expect_true(nrow(tfrmt_data_extracted) > 0)
})

test_that("extract_data extracts updated names changed in the col_plan including 1 group/label vars", {
  data_demog_test <- data_demog |>
    filter(
      rowlbl1 %in% c("Age (y)", "Sex"),
      column != "p-value"
    )

  tfrmt_dem <- tfrmt(
    # specify columns in the data
    group = c(rowlbl1, grp),
    label = rowlbl2,
    column = column,
    param = param,
    value = value,
    sorting_cols = c(ord1, ord2),
    # specify value formatting
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
        n = frmt("xxx"),
        pct = frmt_when(
          "==100" ~ "",
          "==0" ~ "",
          TRUE ~ frmt("(xx.x %)")
        )
      )),
      frmt_structure(group_val = ".default", label_val = "n", frmt("xxx")),
      frmt_structure(group_val = ".default", label_val = c("Mean", "Median", "Min", "Max"), frmt("xxx.x")),
      frmt_structure(group_val = ".default", label_val = "SD", frmt("xxx.xx"))
    ),
    row_grp_plan = row_grp_plan(row_grp_structure(group_val = ".default", element_block(post_space = "   ")),
      label_loc = element_row_grp_loc(location = "column")
    ),
    # remove extra cols
    col_plan = col_plan(
      -grp,
      "rowlbl2_new" = rowlbl2,
      -starts_with("ord"),
      PLACEBO = Placebo,
      TOTAL = Total,
      `XANOMELINE LOW DOSE` = `Xanomeline Low Dose`
    )
  ) |>
    print_to_gt(data_demog_test)

  tfrmt_data_extracted <- extract_data(tfrmt_dem)

  tfrmt_data_manual <- tfrmt_dem[["_data"]] |>
    rename("rowlbl2_new" = "rowlbl2") |>
    select(-`..tfrmt_row_grp_lbl`)

  expect_s3_class(tfrmt_data_extracted, "data.frame")
  expect_equal(tfrmt_data_extracted, tfrmt_data_manual)
  expect_true(nrow(tfrmt_data_extracted) > 0)
})

test_that("extract_data extracts updated names changed in the col_plan", {
  data_demog_test <- data_demog |>
    filter(
      rowlbl1 %in% c("Age (y)", "Sex"),
      column != "p-value"
    )

  tfrmt_dem <- tfrmt(
    # specify columns in the data
    group = c(rowlbl1, grp),
    label = rowlbl2,
    column = column,
    param = param,
    value = value,
    sorting_cols = c(ord1, ord2),
    # specify value formatting
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
        n = frmt("xxx"),
        pct = frmt_when(
          "==100" ~ "",
          "==0" ~ "",
          TRUE ~ frmt("(xx.x %)")
        )
      )),
      frmt_structure(group_val = ".default", label_val = "n", frmt("xxx")),
      frmt_structure(group_val = ".default", label_val = c("Mean", "Median", "Min", "Max"), frmt("xxx.x")),
      frmt_structure(group_val = ".default", label_val = "SD", frmt("xxx.xx"))
    ),
    row_grp_plan = row_grp_plan(row_grp_structure(group_val = ".default", element_block(post_space = "   ")),
      label_loc = element_row_grp_loc(location = "column")
    ),
    # remove extra cols
    col_plan = col_plan(
      -grp,
      -starts_with("ord"),
      PLACEBO = Placebo,
      TOTAL = Total,
      `XANOMELINE LOW DOSE` = `Xanomeline Low Dose`
    )
  ) |>
    print_to_gt(data_demog_test)

  tfrmt_data_extracted <- extract_data(tfrmt_dem)

  tfrmt_data_manual <- tfrmt_dem[["_data"]] |>
    select(-`..tfrmt_row_grp_lbl`)

  expect_s3_class(tfrmt_data_extracted, "data.frame")
  expect_equal(tfrmt_data_extracted, tfrmt_data_manual)
  expect_true(nrow(tfrmt_data_extracted) > 0)
})

test_that("extract_data works for a gt_group object (paged tables)", {
  data_demog_test <- data_demog |>
    filter(
      rowlbl1 %in% c("Age (y)", "Sex"),
      column != "p-value"
    )

  tfrmt_paged <- tfrmt(
    # specify columns in the data
    group = c(rowlbl1, grp),
    label = rowlbl2,
    column = column,
    param = param,
    value = value,
    sorting_cols = c(ord1, ord2),
    # specify value formatting
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{n} {pct}",
          n = frmt("xxx"),
          pct = frmt_when(
            "==100" ~ "",
            "==0" ~ "",
            TRUE ~ frmt("(xx.x %)")
          )
        )
      ),
      frmt_structure(
        group_val = ".default",
        label_val = "n",
        frmt("xxx")
      ),
      frmt_structure(
        group_val = ".default",
        label_val = c("Mean", "Median", "Min", "Max"),
        frmt("xxx.x")
      ),
      frmt_structure(
        group_val = ".default",
        label_val = "SD",
        frmt("xxx.xx")
      )
    ),
    # remove extra cols
    col_plan = col_plan(
      -grp,
      -starts_with("ord")
    ),
    page_plan = page_plan(
      page_structure(
        group_val = list(
          rowlbl1 = ".default"
        )
      ),
      note_loc = "source_note"
    )
  ) |>
    print_to_gt(data_demog_test)

  #  Extract using function vs manual (using purrr::map)
  gt_group_data_extracted <- extract_data(tfrmt_paged)
  gt_group_manual <- purrr::map(tfrmt_paged$gt_tbls$gt_tbl, ~ .x[["_data"]] %>%
    select(-starts_with("..tfrmt")))

  #  Assertions
  expect_type(gt_group_data_extracted, "list")
  expect_length(gt_group_data_extracted, length(gt_group_manual))
  expect_equal(gt_group_data_extracted, gt_group_manual)

  # Check that each element in the list is a data frame
  expect_s3_class(gt_group_data_extracted[[1]], "data.frame")
})

test_that("extract_data throws an error for invalid inputs", {
  expect_error(extract_data(iris), "Input must be a 'gt_tbl' or 'gt_group' object")
  expect_error(extract_data(NULL), "Input must be a 'gt_tbl' or 'gt_group' object")
})

test_that("extract_data works for a table with bigN values", {
  data <- tibble::tibble(
    Group = c("N", "N", "N", rep(c("Age (y)", "Sex", "Age (y)", "Sex"), c(3, 3, 6, 12))),
    Label = c("N", "N", "N", rep(c("n", "Mean (SD)", "Male", "Female"), c(6, 6, 6, 6))),
    Column = c("Placebo", "Treatment", "Total", rep(c("Placebo", "Treatment", "Total"), times = 8)),
    Param = c("bigN", "bigN", "bigN", rep(c("n", "mean", "sd", "n", "pct", "n", "pct"), c(6, 3, 3, 3, 3, 3, 3))),
    Value = c(
      30, 40, 60, 15, 13, 28, 14, 13, 27, 73.56, 74.231, 71.84, 9.347, 7.234, 8.293,
      8, 7, 15, 8 / 14, 7 / 13, 15 / 27, 6, 6, 12, 6 / 14, 6 / 13, 12 / 27
    )
  ) |>
    dplyr::mutate(
      Value = dplyr::case_when(
        Param == "pct" ~ Value * 100,
        TRUE ~ Value
      ),
      ord1 = dplyr::if_else(Param == "bigN", 0, 1),
      ord2 = dplyr::if_else(Param == "bigN", 0, 1)
    )


  bign <- tfrmt(
    group = Group,
    label = Label,
    column = Column,
    value = Value,
    param = Param,
    sorting_cols = c(ord1, ord2),
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine("{n} {pct}",
          n = frmt("X"),
          pct = frmt("(xx.x%)", missing = " ")
        )
      ),
      frmt_structure(
        group_val = "Age (y)", label_val = "Mean (SD)",
        frmt_combine("{mean} ({sd})",
          mean = frmt("XX.X"),
          sd = frmt("x.xx")
        )
      ),
      frmt_structure(group_val = ".default", label_val = "n", frmt("xx"))
    ),
    col_plan = col_plan(everything(), -starts_with("ord"), "Total"),
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " "))
    ),
    big_n = big_n_structure(param_val = "bigN", n_frmt = frmt("\nN = xx"))
  ) |>
    print_to_gt(data)


  extracted <- extract_data(bign)
  manual <- bign[["_data"]] |>
    select(-`..tfrmt_row_grp_lbl`)

  # check expected data is equal to manual extraction
  expect_s3_class(extracted, "data.frame")
  expect_equal(extracted, manual)
  expect_true(nrow(extracted) > 0)

  # check that the bigns are present in the column names of extracted data
  column_labels <- colnames(extracted)

  # Define big N values
  expected_patterns <- c("N = 30", "N = 40", "N = 60")

  # Assert that the labels contain your expected strings
  expect_true(any(grepl("N = 30", column_labels)))
  expect_true(any(grepl("N = 40", column_labels)))
  expect_true(any(grepl("N = 60", column_labels)))
})

test_that("extract_data handles various spanning header depths", {
  data <- tibble::tribble(
    ~group, ~label, ~span2, ~span1, ~my_col, ~parm, ~val,
    "g1", "rowlabel1", "column cols", "cols 1,2", "col1", "value", 1,
    "g1", "rowlabel1", "column cols", "cols 1,2", "col2", "value", 1,
    "g1", "rowlabel1", NA, NA, "mycol3", "value", 1,
    "g1", "rowlabel1", "column cols", "col 4", "col4", "value", 1,
    "g1", "rowlabel1", NA, NA, "mycol5", "value", 1,
    "g1", "rowlabel2", "column cols", "cols 1,2", "col1", "value", 2,
    "g1", "rowlabel2", "column cols", "cols 1,2", "col2", "value", 2,
    "g1", "rowlabel2", NA, NA, "mycol3", "value", 2,
    "g1", "rowlabel2", "column cols", "col 4", "col4", "value", 2,
    "g1", "rowlabel2", NA, NA, "mycol5", "value", 2,
    "g2", "rowlabel3", "column cols", "cols 1,2", "col1", "value", 3,
    "g2", "rowlabel3", "column cols", "cols 1,2", "col2", "value", 3,
    "g2", "rowlabel3", NA, NA, "mycol3", "value", 3,
    "g2", "rowlabel3", "column cols", "col 4", "col4", "value", 3,
    "g2", "rowlabel3", NA, NA, "mycol5", "value", 3,
  )

  # 2 layers of spanning headers
  spanning_tfrmt <- tfrmt(
    group = group,
    label = label,
    param = parm,
    value = val,
    column = c(span2, span1, my_col),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("x"))
    ),
    col_plan = col_plan(
      group,
      label,
      starts_with("col")
    )
  ) |> print_to_gt(data)


  res_layer <- extract_data(spanning_tfrmt)

  expect_contains(colnames(res_layer), "column cols_cols 1,2_col1")
  expect_contains(colnames(res_layer), "column cols_col 4_col4")

  res_layer2 <- extract_data(spanning_tfrmt, col_delim = "/")

  expect_contains(colnames(res_layer2), "column cols/cols 1,2/col1")
  expect_contains(colnames(res_layer2), "column cols/col 4/col4")

  # 1 layer of spanning headers
  data2 <- data |>
    select(-span2)

  spanning_tfrmt2 <- tfrmt(
    group = group,
    label = label,
    param = parm,
    value = val,
    column = c(span1, my_col),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("x"))
    ),
    col_plan = col_plan(
      group,
      label,
      starts_with("col")
    )
  ) |> print_to_gt(data2)

  res_layer2 <- extract_data(spanning_tfrmt2, col_delim = "_")

  expect_contains(colnames(res_layer2), "cols 1,2_col1")
  expect_contains(colnames(res_layer2), "col 4_col4")

  # 1 layer with page plan

  spanning_tfrmt_page <- tfrmt(
    group = group,
    label = label,
    param = parm,
    value = val,
    column = c(span1, my_col),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("x"))
    ),
    col_plan = col_plan(
      group,
      label,
      starts_with("col")
    ),
    page_plan = page_plan(
      max_rows = 3
    )
  ) |> print_to_gt(data2)

  res_layer_page <- extract_data(spanning_tfrmt_page, col_delim = "_")

  expect_contains(colnames(res_layer_page[[1]]), "cols 1,2_col1")
  expect_contains(colnames(res_layer_page[[1]]), "col 4_col4")
})

test_that("markdown column labels - no spanning",{

  mock_data <- tibble(rowlbl1 =c(rep("Completion Status",12),rep("Primary reason for withdrawal",28)),
                      rowlbl2 =c(rep("Completed",4),rep("Prematurely Withdrawn",4),rep("Unknown",4),rep("Adverse Event",4),rep("Lost to follow-up",4),rep("Protocol violation",4),rep("Subject decided to withdraw",4),rep("Protocol Violation",4),rep("Pre-Operative Dose[1]",4),rep("Other",4)),
                      param=c(rep(c("n","n","pct","pct"),10)),
                      column=c(rep(c("Placebo<br/>(N=20)","Treatment"),20)),
                      value=c(24,19,2400/48,1900/38,5,1,500/48,100/38,19,18,1900/48,1800/38,1,1,100/48,100/38,0,0,0,0,0,0,0,0,1,1,100/48,100/38,1,4,100/48,400/38,1,0,100/48,0,2,3,200/48,300/38)

  )


  test_tfrmt <- tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = column,
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{n} {pct}",
          n = frmt("xxx"),
          pct = frmt_when("==100" ~ "",
                          "==0" ~ "",
                          TRUE ~ frmt("(xx.x %)"))
        )
      )
    ),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "indented")
    )
  ) %>%
    print_to_gt(mock_data) %>%
    tab_options(container.width = 1000)

  # test that format of column headers is markdown
  expect_equal(lapply(test_tfrmt$`_boxhead`$column_label,attr,"class"),
               as.list(rep("from_markdown",length(test_tfrmt$`_boxhead`$column_label)))
  )

})

test_that("markdown column labels - spanning", {

  # set up data for tfrmt
  mock_data <- tibble::tribble(
    ~`Pooled Id`,  ~`Site Id`,
    "701",  "701",
    "703",  "703",
    "704",  "704",
    "705",  "705",
    "708",  "708",
    "709",  "709",
    "710",  "710",
    "713",  "713",
    "716",  "716",
    "718",  "718",
    "900",  "702",
    "900",  "706",
    "900",  "707",
    "900",  "711",
    "900",  "714",
    "900",  "715",
    "900",  "717",
    "Total", " ") %>%
    crossing(col1 = c("Placebo (N=86)",
                      "Xanomeline Low Dose <br /> (N=84)",
                      "Xanomeline High Dose (N=84)",
                      "Total (N=254)"),
             col2 = factor(c("ITT </br> (N=10)", "Eff", "Com"), levels = c("ITT </br> (N=10)", "Eff", "Com"))) %>%
    mutate(val = rpois(216, 15),
           param = "val")

  # create output with spanning headers
  test_tfrmt<-tfrmt(
    param = "param",
    value = "val",
    column = vars(col1, col2),
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("XX"))
    ),
    row_grp_plan = row_grp_plan(label_loc =element_row_grp_loc("column")),
    col_plan = col_plan(
      `Pooled Id`,  `Site Id`,
      contains("Placebo"),
      contains("High Dose"),
      contains("Low Dose"),
      everything()
    )
  ) %>%
    print_to_gt(mock_data)

  # test that format of both column headers and spannning headers is markdown
  expect_equal(lapply(test_tfrmt$`_boxhead`$column_label,attr,"class"),
               as.list(rep("from_markdown",length(test_tfrmt$`_boxhead`$column_label)))
  )
  expect_equal(lapply(test_tfrmt$`_spanners`$spanner_label,attr,"class"),
               as.list(rep("from_markdown",length(test_tfrmt$`_spanners`$spanner_label)))
  )
})


test_that("markdown column labels - renamed", {

  mock_data <- tibble::tribble(
    ~group,     ~label,  ~my_col,    ~parm, ~val,
    "g1", "rowlabel1",  "col1"  ,  "value",    1,
    "g1", "rowlabel1",  "col2"  ,  "value",    1,
    "g1", "rowlabel1",  "mycol3",  "value",    1,
    "g1", "rowlabel1",  "col4"  ,  "value",    1,
    "g1", "rowlabel1",  "mycol5",  "value",    1,
    "g1", "rowlabel2",  "col1"  ,  "value",    2,
    "g1", "rowlabel2",  "col2"  ,  "value",    2,
    "g1", "rowlabel2",  "mycol3",  "value",    2,
    "g1", "rowlabel2",  "col4"  ,  "value",    2,
    "g1", "rowlabel2",  "mycol5",  "value",    2,
    "g2", "rowlabel3",  "col1"  ,  "value",    3,
    "g2", "rowlabel3",  "col2"  ,  "value",    3,
    "g2", "rowlabel3",  "mycol3",  "value",    3,
    "g2", "rowlabel3",  "col4"  ,  "value",    3,
    "g2", "rowlabel3",  "mycol5",  "value",    3)

  test_tfrmt <- tfrmt(
    group = group,
    label = label,
    param = parm,
    value = val,
    column = my_col,
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("x"))
    ),
    col_plan = col_plan(
      group,
      label,
      starts_with("col"),
      "test <br/> newline" = mycol3,
      -mycol5
    )
  ) %>%
    print_to_gt(mock_data)

  expect_equal(lapply(test_tfrmt$`_boxhead`$column_label,attr,"class"),
               as.list(rep("from_markdown",length(test_tfrmt$`_boxhead`$column_label)))
  )
})

test_that("column spanners and labels are appropriately aligned", {
  dat <- tibble::tribble(
    ~ group, ~label, ~span1,   ~span2,  ~lower, ~param,     ~val,
    "mygrp", "mylbl", "span01",  "span1", "lower1_a", "prm", 1,
    "mygrp", "mylbl", "span01",  "span1", "lower1_b", "prm",1,
    "mygrp", "mylbl", "span01",  "span2", "lower2_a", "prm",1,
    "mygrp", "mylbl", "span01",  "span2", "lower2_b", "prm",1,
    "mygrp", "mylbl", "span02",  "span3", "lower2_a", "prm",1,
    "mygrp", "mylbl", "span02",  "span3", "lower2_b", "prm",1
  )

  tfrmt_spec <- tfrmt(
    group = "group",
    label = "label",
    param = "param",
    column =c("span1", "span2","lower"),
    value = "val",
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("x.xx"))
    )
  )

 gt_out <- tfrmt_spec %>%
      print_to_gt(.data = dat)

  # reconstruct the original columns from the gt object

 # get spanner labels
  spans <-  gt_out$`_spanners` %>%
    select(var = vars, spanner_label, spanner_level ) %>%
    unnest(
      tidyselect::everything()
    )
  # get lower labels
  lower <- gt_out$`_boxhead` %>%
    select(var, column_label) %>%
    unnest(
      tidyselect::everything()
    )

  # get tfrmt cols from spec
  chr_cols <- map_chr(tfrmt_spec$column, as_name) %>% rev

  # combine spanner & lower labels and rename as per tfrmt spec
  gt_cols <- dplyr::full_join(lower, spans, by = "var", multiple = "all") %>%
    unique %>%
    dplyr::filter(!is.na(spanner_label)) %>%
    tidyr::pivot_wider(names_from = spanner_level,
                values_from = spanner_label) %>%
    dplyr::select(-var) %>%
    setNames(., chr_cols)

  # original data - keep tfrmt spec cols
  orig_cols <- dat %>%
    dplyr::select(
      tidyselect::all_of(
        chr_cols
      )
    ) %>%
    unique

  expect_equal(gt_cols,
               orig_cols,
               ignore_attr = TRUE)

})

set.seed(1234)

raw_data_cat <- crossing(group = "A",label = c("w", "x", "y", "z"),
                         col = paste("Var", 1:4), param2 = c("count", "pct")) %>%
  rowwise() %>%
  mutate(ord1 = 1,
         ord2 = 26-which(label==letters),
         val2 = case_when(
           label == "w" & param2 == "pct" ~100.0,
           param2=="count" ~ as.double(rpois(n=1, lambda = 150)),
           param2 == "pct" ~runif(n=1,max = 100)
         )
  )

raw_data_cont <- crossing(group = "B",label = c("w", "i", "j", "k"),
                          col = paste("Var", 1:4), param2 = c("val")) %>%
  rowwise() %>%
  mutate(ord1 = 2,
         ord2 = which(label==letters),
         val2 = case_when(
           label == "w" ~ as.double(rpois(n=1, lambda = 150)),
           label == "i" ~ rnorm(n=1, mean = 75, 13),
           label == "j" ~ rnorm(n=1, mean = 10, 3),
           label == "k" ~ rnorm(n=1, mean = 72, 7)
         )
  )
raw_dat <- bind_rows(raw_data_cat, raw_data_cont)

plan  <- tfrmt(
  #These are the columns that control the general structure of the data
  group = vars(group),
  label = "label",
  param = "param2",
  value = "val2",
  column = "col",
  #This controls how the rows are sorted
  sorting_cols = vars(ord1, ord2),
  col_style_plan = col_style_plan(
    col_style_structure(align = c(" ", ",", "."), col = vars(starts_with("Var")))),
  body_plan = body_plan(
    frmt_structure(group_val = ".default", label_val = ".default", frmt("XXX.XX")),
    frmt_structure(group_val = ".default", label_val = ".default",
                   frmt_combine(
                     "{count} {pct}",
                     count = frmt("XXX"),
                     percent = frmt_when("==100"~ frmt(""),
                                         "==0"~ "",
                                         "TRUE" ~ frmt("(XXX.X%)"))
                   )),
    frmt_structure(group_val = c("B"), label_val = "w", # Value(s) in the label column where you would want to apply this fmt
                   frmt("XXX")),
    frmt_structure(group_val = "B", label_val = c("i", "k"), frmt("xx.x")),
    frmt_structure(group_val = "B", label_val = "j", frmt("xx.xx"))
  ),
  # These are the variables to keep
  col_plan = col_plan(
    everything(),
    -starts_with("ord")
  ),
  row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc("spanning"))
)

test_that("Check apply_tfrmt", {


  man_df <-  tibble::tribble(
    ~group, ~label, ~`Var 1`,        ~`Var 2`,        ~`Var 3`,        ~`Var 4`,
    "A",     "w",     "135         ", "141         ", "143         ", "137         ",
    "A",     "x",     "129 ( 76.0%)", "139 ( 31.2%)", "153 ( 24.4%)", "158 ( 15.3%)",
    "A",     "y",     "150 (  4.2%)", "144 ( 56.5%)", "165 ( 66.8%)", "167 ( 89.9%)",
    "A",     "z",     "146 ( 13.2%)", "134 ( 56.5%)", "142 (  3.9%)", "156 ( 94.6%)",
    "B",     "i",     " 83.5       ", " 68.9       ", " 78.2       ", " 79.2       ",
    "B",     "j",     " 10.77      ", " 11.05      ", "  8.79      ", "  5.70      ",
    "B",     "k",     " 80.3       ", " 72.5       ", " 87.3       ", " 71.6       ",
    "B",     "w",     "147         ", "149         ", "143         ", "159         "
  ) %>%
    mutate(..tfrmt_row_grp_lbl = FALSE)

  expect_equal(
    apply_tfrmt(raw_dat, plan) %>% ungroup() %>% arrange(group, label) ,
    man_df %>% arrange(group, label),
    ignore_attr = c("class",".col_plan_vars",".footnote_locs")
  )

  plan$sorting_cols <- NULL

  man_df_ord <- man_df %>%
    arrange(group, label)

  expect_equal(apply_tfrmt(raw_dat, plan) %>% ungroup(),
               man_df_ord,
               ignore_attr = c("class",".col_plan_vars",".footnote_locs"))

  expect_error(
    apply_tfrmt(raw_dat, tfrmt(
      label = "label2")),
    "Variable Specified in 'label' doesn't exist in the supplied dataset. Please check the tfrmt and try again."
  )

  expect_error(
    apply_tfrmt(raw_dat, tfrmt(
      group = "label2")),
    "Variable Specified in 'group' doesn't exist in the supplied dataset. Please check the tfrmt and try again."
  )

})

test_that("Check apply_tfrmt for mock data",{

  # mock for example data above

  mock_dat <- raw_dat %>% select(-val2)

  mock_man_df <-  tibble::tribble(
    ~group, ~label, ~`Var 1`,        ~`Var 2`,        ~`Var 3`,        ~`Var 4`,
    "A",     "z",     "XXX (XXX.X%)", "XXX (XXX.X%)" ,"XXX (XXX.X%)" ,"XXX (XXX.X%)",
    "A",     "y",     "XXX (XXX.X%)", "XXX (XXX.X%)" ,"XXX (XXX.X%)" ,"XXX (XXX.X%)",
    "A",     "x",     "XXX (XXX.X%)", "XXX (XXX.X%)" ,"XXX (XXX.X%)" ,"XXX (XXX.X%)",
    "A",     "w",     "XXX (XXX.X%)", "XXX (XXX.X%)" ,"XXX (XXX.X%)" ,"XXX (XXX.X%)",
    "B",     "i",     " xx.x       ", " xx.x       " ," xx.x       " ," xx.x       ",
    "B",     "j",     " xx.xx      ", " xx.xx      " ," xx.xx      " ," xx.xx      ",
    "B",     "k",     " xx.x       ", " xx.x       " ," xx.x       " ," xx.x       ",
    "B",     "w",     "XXX         ", "XXX         " ,"XXX         " ,"XXX         ",
  ) %>%
    mutate("..tfrmt_row_grp_lbl" = FALSE) %>%
    arrange(group, label)

  expect_equal(apply_tfrmt(mock_dat, plan, mock = TRUE) %>% ungroup() %>% arrange(group, label) ,
               mock_man_df,
               ignore_attr = c("class",".col_plan_vars",".footnote_locs"))


  # mock for plan alone
  plan  <- tfrmt(
    #These are the columns that control the general structure of the data
    group = vars(group),
    label = "label",
    param = "param2",
    value = "val2",
    column = "col",
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default",
                     frmt_combine(
                       "{count} {pct}",
                       count = frmt("XXX"),
                       percent = frmt_when("==100"~ frmt(""),
                                           "==0"~ "",
                                           "TRUE" ~ frmt("(XXX.X%)"))
                     )),
      frmt_structure(group_val = c("B"), label_val = "w", # Value(s) in the label column where you would want to apply this fmt
                     frmt("XXX")),
      frmt_structure(group_val = "B", label_val = c("i", "k"), frmt("xx.x")),
      frmt_structure(group_val = "B", label_val = "j", frmt("xx.xx"))
    ),
    row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc("spanning")),
    # These are the variables to keep
    col_plan = col_plan(
      everything(),
      -starts_with("ord")
    )
  )


  mock_dat <- make_mock_data(plan, .default = 1:2, n_cols = 4)
  mock_man_df <-  tibble::tribble(
    ~group,  ~label,   ~ col1,         ~col2,        ~ col3,        ~ col4,
    "group_1", "label_1", "XXX (XXX.X%)", "XXX (XXX.X%)" ,"XXX (XXX.X%)" ,"XXX (XXX.X%)",
    "group_2", "label_1", "XXX (XXX.X%)", "XXX (XXX.X%)" ,"XXX (XXX.X%)" ,"XXX (XXX.X%)",
    "group_1", "label_2", "XXX (XXX.X%)", "XXX (XXX.X%)" ,"XXX (XXX.X%)" ,"XXX (XXX.X%)",
    "group_2", "label_2", "XXX (XXX.X%)", "XXX (XXX.X%)" ,"XXX (XXX.X%)" ,"XXX (XXX.X%)",
    "B"      , "w",     "XXX"          , "XXX"          ,"XXX"          ,"XXX",
    "B"      , "i",     "xx.x"         , "xx.x"         ,"xx.x"         ,"xx.x",
    "B"      , "k",     "xx.x"         , "xx.x"         ,"xx.x"         ,"xx.x",
    "B"      , "j",     "xx.xx"        , "xx.xx"        ,"xx.xx"        ,"xx.xx"
  ) %>%
    mutate("..tfrmt_row_grp_lbl" = FALSE) %>%
    arrange(group, label)

  expect_equal(
    apply_tfrmt(mock_dat, plan, mock = TRUE) %>% ungroup() %>% arrange(group, label) ,
    mock_man_df,
    ignore_attr = c("class",".col_plan_vars",".footnote_locs"))



  # plan with multiple group variables
  plan  <- tfrmt(
    group = vars(grp1, grp2, grp3, grp4),
    label = "my_label",
    param = "param2",
    value = "val2",
    column = "col",
    body_plan = body_plan(
      frmt_structure(group_val = list(grp1 = "A", grp2 = c("a","b"), grp3 = ".default", grp4 = ".default"), label_val = ".default", frmt("xx.x")),
      frmt_structure(group_val = list(grp1 = "B", grp2 = c("a","b"), grp3 = ".default", grp4 = ".default"), label_val = ".default", frmt("xx.x")),
      frmt_structure(group_val = list(grp1 = ".default", grp2 = ".default", grp3 = "C", grp4 = c("a","b")), label_val = ".default", frmt("xx.x")),
      frmt_structure(group_val = list(grp1 = ".default", grp2 = ".default", grp3 = "D", grp4 = c("a","b")), label_val = ".default", frmt("xx.x"))
    ),
    row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc("gtdefault"))
  )
  mock_dat <- make_mock_data(plan, .default = 1, n_cols = 1) %>%
    apply_tfrmt(plan, mock =TRUE)

  expected_dat <-  tibble::tribble(
    ~grp1,   ~grp2,     ~grp3,     ~grp4,   ~my_label,   ~col1,
    "A"      ,"a"      ,"grp3_1" ,"grp4_1" ,"my_label_1" ,"xx.x" ,
    "A"      ,"b"      ,"grp3_1" ,"grp4_1" ,"my_label_1" ,"xx.x" ,
    "B"      ,"a"      ,"grp3_1" ,"grp4_1" ,"my_label_1" ,"xx.x" ,
    "B"      ,"b"      ,"grp3_1" ,"grp4_1" ,"my_label_1" ,"xx.x" ,
    "grp1_1" ,"grp2_1" ,"C"      ,"a"      ,"my_label_1" ,"xx.x" ,
    "grp1_1" ,"grp2_1" ,"C"      ,"b"      ,"my_label_1" ,"xx.x" ,
    "grp1_1" ,"grp2_1" ,"D"      ,"a"      ,"my_label_1" ,"xx.x" ,
    "grp1_1" ,"grp2_1" ,"D"      ,"b"      ,"my_label_1" ,"xx.x"
  )

  expect_equal(
    mock_dat,
    expected_dat,
    ignore_attr = c("class",".col_plan_vars",".footnote_locs", ".stub_header")
  )

  # duplicate params for a single group/label combo
  plan  <- tfrmt(
    group = "grp1",
    label = "my_label",
    param = "param2",
    value = "val2",
    column = "col",
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", N = frmt("xxx")),
      frmt_structure(group_val = ".default", label_val = ".default", mean = frmt("xx.x"))
    ),
    row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc("gtdefault"))
  )
  mock_dat <- make_mock_data(plan, .default = 1:2, n_cols = 2)

  make_mock_dat_message <- mock_dat %>%
    apply_tfrmt(plan, mock =TRUE) %>%
    capture_messages()

  ## capturing second message
  expect_equal(
    make_mock_dat_message,
    "Mock data contains more than 1 param per unique label value. Param values will appear in separate rows.\n"
  )

  test_dat <- mock_dat %>%
    quietly(apply_tfrmt)(plan, mock =TRUE) %>%
    .[["result"]]

  expect_equal(test_dat,
               tibble::tribble(
                 ~grp1,   ~my_label,   ~col1,  ~col2,
                 "grp1_1", "my_label_1", "xxx" ,  "xxx"  ,
                 "grp1_1", "my_label_1", "xx.x",  "xx.x" ,
                 "grp1_1", "my_label_2", "xxx" ,  "xxx"  ,
                 "grp1_1", "my_label_2", "xx.x",  "xx.x" ,
                 "grp1_2", "my_label_1", "xxx" ,  "xxx"  ,
                 "grp1_2", "my_label_1", "xx.x",  "xx.x" ,
                 "grp1_2", "my_label_2", "xxx" ,  "xxx"  ,
                 "grp1_2", "my_label_2", "xx.x",  "xx.x" ,
               ),
               ignore_attr = c("class",".col_plan_vars",".footnote_locs", ".stub_header"))

})

test_that("Test body_plan missing", {
  input_data <- tibble(
    group = "groupvar",
    label = paste0("label", 1:10),
    param = "params",
    column = "col1",
    val = 1:10
  )

  expect_message(
    empty_body_plan <- tfrmt(
      group = group,
      label = label,
      param = param,
      column = column,
      value = val,
      row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc("gtdefault"))
    ) %>%
      apply_tfrmt(input_data, .),
    "The following rows of the given dataset have no format applied to them 1, 2, 3, 4, 5, 6, 7, 8, 9, 10"
  )

  expect_equal(empty_body_plan,
               input_data %>%
                 select(-param) %>%
                 mutate(val = as.character(val)) %>%
                 pivot_wider(names_from = column, values_from = val),
               ignore_attr = c("class",".col_plan_vars",".footnote_locs", ".stub_header"))
})


test_that("incomplete body_plan where params share label",{

  dd <- tibble::tribble(
    ~rowlbl1, ~grp, ~rowlbl2, ~column, ~param, ~value,
    "topgrp", "lowergrp1", "n pct", "A",  "n",   1,
    "topgrp", "lowergrp1", "n pct", "A",  "pct",   50,
    "topgrp", "lowergrp1", "mean", "A",  "mean",   2,
    "topgrp", "lowergrp2", "n pct", "A",  "n",   2,
    "topgrp", "lowergrp2", "n pct", "A",  "pct",   40,
    "topgrp", "lowergrp2", "mean", "A",  "mean",   5
  )

  tfrmt_spec <- tfrmt(
    group = c(rowlbl1,grp),
    label = rowlbl2,
    column = column,
    param = param,
    value = value,
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = "mean", frmt("xx.x"))
    ),
    row_grp_plan = row_grp_plan(
      label_loc = element_row_grp_loc(location = "column")
    )
  )


    expect_message(
      auto_tfrmt <- apply_tfrmt(dd, tfrmt_spec),
      "The following rows of the given dataset have no format applied to them 1, 2, 4, 5"
    ) %>%
      expect_message(
        "Multiple param listed for the same group/label values"
      )

  man_tfrmt <- tibble::tribble(
    ~rowlbl1, ~grp,  ~rowlbl2,    ~ A   ,
    "topgrp", "lowergrp1", "n pct"  , c("1","50"),
    "topgrp", "lowergrp1", "mean"   , " 2.0",
    "topgrp", "lowergrp2", "n pct"  , c("2","40"),
    "topgrp", "lowergrp2", "mean"   , " 5.0"
  )

  expect_equal(auto_tfrmt, man_tfrmt,
               ignore_attr = c("class",".col_plan_vars",".footnote_locs", ".stub_header"))
})



test_that("incorrect footnote plan formats",{
  expect_error( tfrmt(
    # specify columns in the data
    group = c(rowlbl0,rowlbl1),
    label = rowlbl2,
    column = trt,
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    # row_grp_plan = row_grp_plan(
    #   row_grp_structure(group_val = ".default", element_block(post_space = " ")),
    #   label_loc = element_row_grp_loc(location = "indented")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote",group_val="Test group"),
      marks="letters"
    )
  ),
  "when tfrmt contains multiple groups, group_val must be a named list")


  expect_error(tfrmt(
    # specify columns in the data
    group = c(rowlbl1),
    label = rowlbl2,
    column = c(col2,trt),
    param = param,
    value = value,
    # set formatting for values
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} {pct}",
                                                                                  n = frmt("xxx"),
                                                                                  pct = frmt_when("==100" ~ "",
                                                                                                  "==0" ~ "",
                                                                                                  TRUE ~ frmt("(xx.x %)"))))),

    # Specify row group plan
    # Indent the rowlbl2
    row_grp_plan = row_grp_plan(
      row_grp_structure(group_val = ".default", element_block(post_space = " ")),
      label_loc = element_row_grp_loc(location = "column")),
    footnote_plan = footnote_plan(
      footnote_structure("Test footnote 2",column_val="Treatment column"),
      marks="letters"

    )
  ),
  "when tfrmt contains multiple columns, column_val must be a named list")

})




test_that("struct utils quote escaping",{

  dd <- tibble::tribble(
   ~grp, ~rowlbl2, ~column, ~param, ~value,
   "lowergrp1's", "n", "A",  "n",   1,
   "lowergrp1's", "mean", "A",  "mean",   2,
   "\"lowergrp2\"", "n", "A",  "n",   2,
   '"lowergrp2"', "mean", "A",  "mean",   5
  )

  tfrmt_spec <- tfrmt(
    group = grp,
    label = rowlbl2,
    column = column,
    param = param,
    value = value,
    body_plan = body_plan(
      frmt_structure(group_val = ".default", label_val = ".default", frmt("x")),
      frmt_structure(group_val = "lowergrp's", label_val = "n", frmt("xx")),
      frmt_structure(group_val = 'lowergrp\'s', label_val = "mean", frmt("xx.x")),
      frmt_structure(group_val = "\"lowergrp2\"", label_val = "n", frmt("xxxx")),
      frmt_structure(group_val = '"lowergrp2"', label_val = "mean", frmt("xx.xx"))
    )
  )


  man_tfrmt <- tibble::tribble(
    ~ rowlbl2         , ~A      ,
    "lowergrp1's"   , NA  ,
    "  n"           ,"1"    ,
    "  mean"        ,"2"    ,
    "\"lowergrp2\"" , NA    ,
    "  n"           ,"   2" ,
    "  mean"        ," 5.00",
  )
  expect_equal(
    auto_tfrmt <- apply_tfrmt(dd, tfrmt_spec) |> dplyr::select(rowlbl2:A),
    man_tfrmt,
    ignore_attr = c("class",".col_plan_vars",".footnote_locs", ".stub_header")
  )

})

#' Layer tfrmt objects together
#'
#' Provide utility for layering tfrmt objects together. If both tfrmt's have
#' values, it will preferentially choose the second tfrmt by default. This is an
#' alternative to piping together tfrmt's
#'
#' @param x,y tfrmt objects that need to be combined
#' @param ... arguments passed to layer_tfrmt_arg functions for combining different tfrmt elements
#' @param join_body_plans should the `body_plans` be combined, or just keep styling in y. See details: join_body_plans for more details.
#'
#' @details
#'
#' ## join_body_plan
#'
#' When combining two body_plans, the body plans will stack together, first the
#' body plan from x tfrmt then y tfrmt. This means that frmt_structures in y
#' will take priority over those in x.
#'
#' Combining two tfrmt with large body_plans can lead to slow table evaluation.
#' Consider setting `join_body_plan` to `FALSE`. Only the y `body_plan` will be
#' preserved.
#'
#' @returns tfrmt object
#' @export
#' @examples
#'
#' tfrmt_1 <- tfrmt(title = "title1")
#'
#' tfrmt_2 <- tfrmt(title = "title2",subtitle = "subtitle2")
#'
#' layered_table_format <- layer_tfrmt(tfrmt_1, tfrmt_2)
#'
layer_tfrmt <- function(x, y, ..., join_body_plans = TRUE){

  if(missing(x)){
    stopifnot(is_tfrmt(y))
    return(y)
  }else if(missing(y)){
    stopifnot(is_tfrmt(x))
    return(x)
  }

  stopifnot(is_tfrmt(y))
  stopifnot(is_tfrmt(x))

  args <- union(names(x), names(y))

  arg_list <- lapply(args, function(argname, x, y, ..., join_body_plans){
    func <- get_layer_tfrmt_arg_method(argname)
    func(x, y, argname, ..., join_body_plans = join_body_plans)
  },x=x, y = y, ..., join_body_plans = join_body_plans)

  names(arg_list) <- args

  ## remove null values that may have made it through
  arg_list <- arg_list[!sapply(arg_list, is.null)]

  tfrmt_call <- as.call(c(as.name("tfrmt"), arg_list))

  tryCatch(
    eval(tfrmt_call),
    error = function(e){
      if(inherits(e, "_tfrmt_mismatched_group_vals")){
        e <- append_update_group_message(e, x, y)
      }
      abort(
        e$message,
        call = e$call,
        trace = e$trace
      )
    }
  )

}

get_layer_tfrmt_arg_method <- function(argname){
  tryCatch(
    get(paste0("layer_tfrmt_arg.",argname),envir = asNamespace("tfrmt"), inherits = FALSE),
    error = function(e){ layer_tfrmt_arg.default}
  )
}

layer_tfrmt_arg.default<- function(x, y, arg_name, ...){
  x_arg_val <- x[[arg_name]]
  y_arg_val <- y[[arg_name]]

  if(is.null(y_arg_val)){
    x_arg_val
  }else{
    y_arg_val
  }
}

## if group is an empty vars, keep the original value
layer_tfrmt_arg_vars<- function(x, y, arg_name, ...){
  x_arg_val <- x[[arg_name]]
  y_arg_val <- y[[arg_name]]

  if(is.null(y_arg_val) | identical(y_arg_val, vars())){
    x_arg_val
  }else{
    y_arg_val
  }
}

## if label/param/value/column is an empty quo, keep the original value
layer_tfrmt_arg_quo<- function(x, y, arg_name, ...){
  x_arg_val <- x[[arg_name]]
  y_arg_val <- y[[arg_name]]

  if(is.null(y_arg_val) | identical(y_arg_val, quo())){
    x_arg_val
  }else{
    y_arg_val
  }
}

layer_tfrmt_arg.group <- layer_tfrmt_arg_vars
layer_tfrmt_arg.label <- layer_tfrmt_arg_quo
layer_tfrmt_arg.param <- layer_tfrmt_arg_quo
layer_tfrmt_arg.value <- layer_tfrmt_arg_quo
layer_tfrmt_arg.column <- layer_tfrmt_arg_vars
layer_tfrmt_arg.sorting_cols <- layer_tfrmt_arg_vars


layer_tfrmt_arg.body_plan <- function(x, y, ...,  join_body_plans = TRUE){
  x_body_plan <- x[["body_plan"]]
  y_body_plan <- y[["body_plan"]]

  if(join_body_plans){
    body_plan_el <- unique(c(x_body_plan, y_body_plan))
  }else{
    body_plan_el <- y_body_plan
  }

  do.call(body_plan,body_plan_el)
}



#' Remap group values in a tfrmt
#'
#' @param tfrmt a `tfrmt`
#' @param ... Use new_name = old_name to rename selected variables
#'
#' @return
#' A `tfrmt` with the `group` variables updated in all places
#'
#' @importFrom rlang as_label is_empty
#'
#' @returns tfrmt object with updated groups#'
#' @export
#' @examples
#'
#' tfrmt_spec <- tfrmt(
#'     group = c(group1, group2),
#'     body_plan  = body_plan(
#'       frmt_structure(
#'          group_val = list(group2 = "value"),
#'          label_val = ".default",
#'          frmt("XXX")
#'          ),
#'      frmt_structure(
#'          group_val = list(group1 = "value", group2 = "value"),
#'          label_val = ".default",
#'          frmt("XXX")
#'        )
#'     ))
#'
#' tfrmt_spec %>%
#'   update_group(New_Group = group1)
#'
update_group <- function(tfrmt, ...){

  dots <- as.list(substitute(substitute(...)))[-1]

  old_groups <- do.call(vars, unname(dots))
  new_group_map <- setNames(names(dots), map_chr(old_groups, as_label))


  if(!is_empty(tfrmt$group)){

    var_list <- sapply(tfrmt$group, function(x){
      x_lab <- as_label(x)
      if(x_lab %in% names(new_group_map)){
        new_group_map[[x_lab]]
      }else{
        x_lab
      }
    })

    tfrmt$group <- as_vars(var_list)

  }else{
    stop("No group values defined in input tfrmt.")
  }

  ## Update body_plan
  if(!is.null(tfrmt$body_plan)){
    tfrmt$body_plan <- update_groups_body_plan(tfrmt$body_plan, new_group_map)
  }

  ## Update row_grp_plan
  if(!is.null(tfrmt$row_grp_plan)){
    tfrmt$row_grp_plan <- update_groups_row_grp_plan(tfrmt$row_grp_plan, new_group_map)
  }

  ## update footnote_plan
  if(!is.null(tfrmt$footnote_plan)){
    tfrmt$footnote_plan <- update_groups_footnote_plan(tfrmt$footnote_plan, new_group_map)
  }

  check_group_var_consistency(tfrmt)

  tfrmt

}

update_groups_body_plan <- function(tfrmt_body_plan, new_group_map){
    bp_list <- lapply(tfrmt_body_plan, function(struct){
      if(is.list(struct$group_val)){
        struct_groups <- names(struct$group_val)
        for(struct_group_idx in seq_along(struct_groups)){
          if(struct_groups[struct_group_idx] %in% names(new_group_map)){
            names(struct$group_val)[struct_group_idx] <- new_group_map[struct_groups[struct_group_idx]]
          }
        }
      }
      struct
    })
    do.call("body_plan", bp_list)
}

update_groups_row_grp_plan <- function(tfrmt_row_grp_plan, new_group_map){
    row_grp_structs <- lapply(tfrmt_row_grp_plan$struct_list, function(struct){
      if(is.list(struct$group_val)){
        struct_groups <- names(struct$group_val)
        for(struct_group_idx in seq_along(struct_groups)){
          if(struct_groups[struct_group_idx] %in% names(new_group_map)){
            names(struct$group_val)[struct_group_idx] <- new_group_map[struct_groups[struct_group_idx]]
          }
        }
      }
      struct
    })
    do.call("row_grp_plan", c(row_grp_structs, tfrmt_row_grp_plan["label_loc"]))
}

update_groups_footnote_plan <- function(tfrmt_footnote_plan, new_group_map){
  footnote_structs <- lapply(tfrmt_footnote_plan$struct_list, function(struct){
    if(is.list(struct$group_val)){
      struct_groups <- names(struct$group_val)
      for(struct_group_idx in seq_along(struct_groups)){
        if(struct_groups[struct_group_idx] %in% names(new_group_map)){
          names(struct$group_val)[struct_group_idx] <- new_group_map[struct_groups[struct_group_idx]]
        }
      }
    }
    struct
  })
  do.call("footnote_plan", c(footnote_structs, tfrmt_footnote_plan["marks"]))
}

append_update_group_message <- function(e, x, y){

  x_grp <- map_chr(x$group, as_label)
  y_grp <- map_chr(y$group, as_label)

  if(!is_empty(y_grp) && !is_empty(x_grp)){
    update_grp_message <- c(i = paste0(
      "You might need to update group names using ",
      "\"update_group(",
      paste0("`",y_grp,"` = `", x_grp,"`", collapse = ","),
      ")\""))

    e$message <- c(e$message, "", update_grp_message)
  }

  e
}

#' Reset or Remove a Component from a tfrmt Object
#'
#' This utility function allows users to remove a specific component (e.g., `body_plan`,
#' `row_grp_plan`, `col_plan`) from a `tfrmt` object by setting it to `NULL`.
#'
#' @param tfrmt_obj A `tfrmt` object to be modified.
#' @param component_name A character string specifying the name of the component
#'   to reset/remove (e.g., "body_plan").
#'
#' @return A modified `tfrmt` object with the specified component removed.
#'
#' @export
#'
#' @examples
#' \dontrun{
#'   my_tfrmt <- tfrmt(column = "col1", label = "label1")
#'   # Remove the column component
#'   reset_tfrmt <- reset_component(my_tfrmt, "column")
#' }
reset_component <- function(tfrmt_obj, component_name) {
  # Validate inputs
  # check for class attribute
  if (!inherits(tfrmt_obj, "tfrmt")) {
    stop("The input object must be of class 'tfrmt'.")
  }
  if (!is.character(component_name) || length(component_name) != 1) {
    stop("The component name must be a single string.")
  }

  # Check if the specified component exists in the list
  if (!component_name %in% names(tfrmt_obj)) {
    stop(paste0("component '", component_name, "' does not exist in the tfrmt object."))
  }

  # Reset the specified component by setting it to NULL
  tfrmt_obj[[component_name]] <- NULL

  # Return the modified tfrmt object
  return(tfrmt_obj)
}

test_that("Basic functionality: Resets a component and other components remain unchanged", {
  t_frmt <- tfrmt(
    title = "Table Title",
    group = c("row_label1"),
    label = c("row_label2"),
    param = c("param"),
    value = c("value"),
    column = c("column")
  )


  modified_tfrmt <- reset_component(t_frmt, "title")

  expect_null(modified_tfrmt$title)
  expect_equal(modified_tfrmt$group, t_frmt$group, ignore_attr = TRUE )
  expect_equal(modified_tfrmt$label, t_frmt$label, ignore_attr = TRUE)
  expect_equal(modified_tfrmt$param, t_frmt$param, ignore_attr = TRUE)
  expect_equal(modified_tfrmt$value, t_frmt$value, ignore_attr = TRUE)
  expect_equal(modified_tfrmt$column, t_frmt$column, ignore_attr = TRUE)
})

test_that("Error for non-'tfrmt' object", {
  non_tfrmt_obj <- list(title = "My Title", subtitle = "My Subtitle")

  expect_error(
    reset_component(non_tfrmt_obj, "subtitle"),
    "The input object must be of class 'tfrmt'."
  )
})

test_that("Error for invalid component_name type", {
  t_frmt <- tfrmt(
    title = "Table Title",
    group = c("row_label1"),
    label = c("row_label2"),
    param = c("param"),
    value = c("value"),
    column = c("column")
  )


  expect_error(
    reset_component(t_frmt, c("title", "data")),
    "The component name must be a single string."
  )

  expect_error(
    reset_component(t_frmt, 123),
    "The component name must be a single string."
  )
})

test_that("Error for nonexistent component", {
  t_frmt <- tfrmt(
    title = "Table Title",
    group = c("row_label1"),
    label = c("row_label2"),
    param = c("param"),
    value = c("value"),
    column = c("column")
  )


  expect_error(
    reset_component(t_frmt, "nonexistent"),
    "component 'nonexistent' does not exist in the tfrmt object."
  )
})

#' Print to ggplot
#'
#' @param tfrmt tfrmt object that will dictate the structure of the ggplot object
#' @param .data Data to style in order to make the ggplot object
#' @param ... Inputs to geom_text to modify the style of the table body
#'
#' @return a stylized ggplot object
#' @export
#'
#' @section Examples:
#'
#' ```r
#'
#' # Create data
#' risk<-tibble(time=c(rep(c(0,1000,2000,3000),3)),
#'             label=c(rep("Obs",4),rep("Lev",4),rep("Lev+5FU",4)),
#'             value=c(630,372,256,11,620,360,266,8,608,425,328,14),
#'             param=rep("n",12))
#'
#'table<-tfrmt(
#'  label = label ,
#'  column = time,
#'  param = param,
#'  value = value) |>
#'   print_to_ggplot(risk)
#'
#'table
#'
#' ```
#' \if{html}{\out{
#' `r "<img src=\"https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/master/images/example_print_to_ggplot.png\" alt=\"Simple table to stack with a KM-plot\" style=\"width:100\\%;\">"`
#' }}
#'
#' @importFrom rlang quo_is_missing as_label
#' @importFrom dplyr select pull
#' @importFrom magrittr %>%

print_to_ggplot <- function(tfrmt, .data, ...){
  if(!is_tfrmt(tfrmt)){
    stop("Requires a tfrmt object")
  }

  if(!is.data.frame(.data)){
    stop("Requires data")
  }

  # stop if label location is not indented
  if(is.null(tfrmt$row_grp_plan)==FALSE && tfrmt$row_grp_plan$label_loc[1] != "indented"){
    stop("print_to_ggplot must have label location 'indented' if row_group_plan is present")
  }

  # stop if span structures are present
  if(is.null(tfrmt$col_plan$span_structures)==FALSE){
    stop("print_to_ggplot does not support spanning headers")
  }

  # stop if more than one column variable
  if(length(tfrmt$column)>1){
    stop("print_to_ggplot does not support multiple column variables")
  }

  # stop if column style plan added
  if(is.null(tfrmt$col_style_plan)==FALSE){
    stop("print_to_ggplot does not support col_style_plan elements")
  }

  # stop if param, column values not provided
  if (quo_is_missing(tfrmt$param)){
    stop("param variable required for print_to_ggplot")
  }
  if (is_empty(tfrmt$column)){
    stop("column variable required for print_to_ggplot")
  }
  if (quo_is_missing(tfrmt$label)){
    stop("label variable required for print_to_ggplot")
  }

  if(quo_is_missing(tfrmt$value)){
    stop("value variable required for print_to_ggplot")
  }


  # Keeping the original data of column to preserve data type later on
  column_name <- as_label(tfrmt$column[[1]])
  column_data <-.data %>%
    pull(!!column_name)

  apply_tfrmt(.data, tfrmt, mock = FALSE) %>%
    cleaned_data_to_ggplot(tfrmt,column_data, ...)

}

#' Do all the formatting for the ggplot
#'
#' @param .data data to clean
#' @param tfrmt tfrmt object for formatting
#' @param column_data original column data to preserve type
#' @param ... Inputs to geom_text to modify the style of the table body
#'
#' @return ggplot object
#' @noRd
#'
#' @importFrom ggplot2 ggplot .data ylab xlab theme_void theme scale_y_discrete scale_x_continuous scale_x_discrete element_text aes geom_text margin unit element_blank
#' @importFrom tidyr pivot_longer
#' @importFrom magrittr %>%
cleaned_data_to_ggplot <- function(.data,tfrmt,column_data, ...){

  # apply grouping if any
  # create y variable to preserve ordering and levels
  .data<-apply_grp_ggplot(.data,tfrmt) %>%
    mutate(y=n():1)

  # handle cases for "..tfrmt_row_grp_lbl pivoting
  if("..tfrmt_row_grp_lbl" %in% names(.data)){
    # reshape data for ggplot
    long_data <- .data %>%
      pivot_longer(-c(as_label(tfrmt$label),"y","..tfrmt_row_grp_lbl"),
                   names_to = "column",values_to="value") %>%
      mutate(value=if_else(.data$`..tfrmt_row_grp_lbl`==TRUE,"",.data$value))
  }else{
    long_data <- .data %>%
      pivot_longer(-c(as_label(tfrmt$label),"y"),
                   names_to = "column",values_to="value")

  }

  # preserve data types from original data - changed by pivoting
  column_type <- class(column_data)
  if(column_type=="factor"){
    column_levels<-levels(column_data)
    long_data$column<-factor(long_data$column,levels=column_levels)
  }else if(column_type == "character"){
    # preserves character column order
    column_levels <- unique(long_data$column)
    long_data$column <- factor(long_data$column, levels = column_levels)
  }

  if(column_type=="numeric"){
    long_data$column <- as.numeric(long_data$column)
    plot<- ggplot(long_data, aes(x=as.numeric(.data$column),
                                 y=as.factor(.data$y), label = .data$value)) +
      scale_x_continuous(position = "top",
                         breaks = unique(long_data$column),
                         labels = as.character(unique(long_data$column))
                         )
  }else{
    plot <- ggplot(long_data, aes(x=.data$column, y=as.factor(.data$y),
                                  label = .data$value)) +
      scale_x_discrete(position = "top")
  }

  plot +
    geom_text(...) +
    xlab("") +
    ylab("") +
    theme_void() +
    theme(
      axis.text.y = element_text(
        size = 10,
        margin = margin(r = 0),
        hjust = 0
      ),
      panel.spacing = unit(0, "mm"),
      strip.text = element_blank(),
      axis.text.x = element_text(size = 10)
    ) + # replace y values with labels
    scale_y_discrete(labels = pull(long_data, !!tfrmt$label),
                     breaks = long_data$y)

}

#' Apply grouping for ggplot
#'
#' @param .data data with group variable
#' @param tfrmt tfrmt for formatting
#'
#' @return dataset wth grouping columns combined
#' @noRd
#'
#' @importFrom rlang quo_name is_empty
#' @importFrom magrittr %>%
#' @importFrom dplyr all_of select
apply_grp_ggplot<-function(.data,tfrmt){

  if(!is.null(tfrmt$row_grp_plan) && is_empty(tfrmt$group)==FALSE &&
     tfrmt$row_grp_plan$label_loc$location == "gtdefault"){
    group_name <- quo_name(tfrmt$group[[1]])

    element<-element_row_grp_loc(location="indented", indent = "    ")

    combine_group_cols(.data,tfrmt$group,tfrmt$label,element) %>%
      select(
        -tidyselect::all_of(
          group_name
        )
      )


  }else{
    .data
  }
}

#' Make mock data for display shells
#'
#' @param tfrmt tfrmt object
#' @param .default Number of unique levels to create for group/label values set
#'   to ".default"
#' @param n_cols Number of columns in the output table (not including
#'   group/label variables). If not supplied it will default to using the
#'   `col_plan` from the `tfrmt`. If neither are available it will use 3.
#'
#' @return tibble containing mock data
#'
#' @importFrom tidyr crossing unnest expand
#' @importFrom dplyr rowwise mutate pull rename ungroup coalesce group_by tibble
#'   across cur_column
#' @importFrom purrr map map_dfr map_chr map_dfc reduce
#' @importFrom rlang as_name is_empty
#'
#' @export
#' @examples
#'
#' tfrmt_spec <- tfrmt(
#'   label = label,
#'   column = column,
#'   param = param,
#'   value=value,
#'   body_plan = body_plan(
#'     frmt_structure(group_val=".default", label_val=".default", frmt("xx.x"))
#'     )
#'   )
#'
#' make_mock_data(tfrmt_spec)
#'
make_mock_data <- function(tfrmt, .default = 1:3, n_cols = NULL){

  body_plan <- tfrmt$body_plan
  grp_vars <- tfrmt$group %>% map_chr(as_name)


  # create tibble of all frmt_structure grp/label/param: 1 row per group_val per frmt_structure
  all_frmt_spec <- body_plan %>%
    map_dfr(
      function(x){
        crossing(
          # if group_val is a named list, return as a tibble with list names as colnames
          # otherwise (group_val = ".default") convert to tibble with colname "grp"
          if(is.list(x$group_val)) as_tibble(x$group_val) else tibble(..grp = x$group_val[[1]]),
          !!tfrmt$label := x$label_val,
          !!tfrmt$param := x$param_val
        )
      },
      .id = "frmt_num")


  # find out which group variables are not present
  cols_to_add <- setdiff(c("..grp", grp_vars), names(all_frmt_spec))

  # add the missing group variables & fill using .grp
  # & replace .default's
  all_frmt_vals <- bind_cols(all_frmt_spec,
                             map(cols_to_add, function(x) tibble(!!x := NA_character_))) %>%
    mutate(
      ..grp = replace_na(.data$..grp, ".default"),
      across(
        tidyselect::all_of(grp_vars),
        ~ coalesce(.x, .data$..grp)
      )
    ) %>%
    select(-"..grp")  %>%
    rowwise()  %>%
    mutate(
      across(
        !!tfrmt$param,
        ~ process_for_mock(.x, cur_column(), 1)
      ),
      across(
        tidyselect::all_of(grp_vars),
        ~ process_for_mock(.x, cur_column(), .default)
      )
    )

  expand_cols <- c(tfrmt$group)

  if(!quo_is_missing(tfrmt$label)){
    all_frmt_vals <- all_frmt_vals %>%
      mutate(
        across(!!tfrmt$label, ~ process_for_mock(.x, cur_column(), .default))
      )
    expand_cols <- c(expand_cols, tfrmt$label )
  }

  expand_cols <- c(expand_cols, tfrmt$param)

  output_dat <- all_frmt_vals %>%
    unnest(
      tidyselect::everything()
    ) %>%
    group_by(.data$frmt_num) %>%
    expand(!!!expand_cols) %>%
    ungroup() %>%
    add_sorting_cols(tfrmt$sorting_cols)

  ## add `column` columns
  col_def <- make_col_df(column = tfrmt$column, group = tfrmt$group,
                         label = tfrmt$label,
                         sorting_cols = tfrmt$sorting_cols,
                         col_plan = tfrmt$col_plan,
                         col_style_plan = tfrmt$col_style_plan,
                         n_cols = n_cols)

  output_dat <- output_dat %>%
    mutate(
      `__tfrmt__mock__columns` = list(col_def)
    ) %>%
    unnest("__tfrmt__mock__columns") %>%
  #Add big N's
    add_mock_big_ns(column = tfrmt$column,
                    param = tfrmt$param,
                    big_n_struct = tfrmt$big_n)



  # remove the frmt_num field
  output_dat %>%
    select(-"frmt_num") %>%
    unique()

}


#' Given the input value, generate values to be used for mock data
#'
#' @param x input character value
#' @param column type of column (group, label, param)
#' @param .default Number of numbered values to create
#'
#' @return list of character values
#' @noRd
process_for_mock <-function(x, column, .default = 1:3){

  if(x == ".default"){
    str_c(column, "_", .default) %>% list
  } else {
    list(x)
  }
}


#' Regex to remove things like contains, and - etc.
#'
#' @param names string vec
#' @param dont_inc values to remove
#'
#' @return cleaned up string vec
#' @noRd
clean_col_names <- function(names, dont_inc){
  names %>%
    map_chr(as_label) %>%
    str_remove_all('^.*\\(\\"') %>%
    str_remove_all('^-') %>%
    str_remove_all('\\"\\)') %>%
    setdiff(dont_inc)
}

# Adds the sorting columns if relevant otherwise just returns data
add_sorting_cols <- function(data, sorting_cols){
  if(!is.null(sorting_cols)){

    sorting_cols_vars <- sorting_cols %>% map_chr(as_name)
    n_sorting_cols <- length(sorting_cols_vars)

    sorting_cols_def <- map_dfc(seq_len(n_sorting_cols), function(x){
      tibble(!!sorting_cols_vars[x] := 1)
    })

    data <- data %>%
      mutate(
        `__tfrmt__mock__sorting_col` = list(sorting_cols_def)
      ) %>%
      unnest("__tfrmt__mock__sorting_col")
  }
  data
}

make_col_df <- function(column, group, label, sorting_cols, col_plan, col_style_plan, n_cols){

  column_vars <- column %>% map_chr(as_label)
  grp_lb_vars <- c(group %>% map_chr(as_name), as_label(label), sorting_cols %>% map_chr(as_name))
  if(identical(column_vars, "__tfrmt__column")){
    column_vars <- "col"
  }

  n_spans <- length(column_vars)

  # test if col_plan/col_style_plan have names to use
  col_plan_test_res <- col_plan_test(col_plan)
  col_style_plan_test_res <- col_style_plan_test(col_style_plan)
  col_def <- tibble()

  # Use provided column names if there is no spanning
  if (col_plan_test_res || col_style_plan_test_res){
    if(col_plan_test_res & n_spans == 1 & is.null(n_cols)){
      cols_to_use <- col_plan$dots %>%
        clean_col_names(dont_inc = grp_lb_vars)
      col_def <- tibble(!!column_vars[n_spans] := cols_to_use)
    } else if(col_plan_test_res & is.null(n_cols)){
      # Gets the lowest level columns only
      low_lvl_vars <- col_plan$dots %>%
        discard(is.list) %>%
        clean_col_names(dont_inc = grp_lb_vars)

      low_lvl_def <- tibble(!!column_vars[max(n_spans)] := low_lvl_vars)

      # creates a df for each span structure
      span_df <- col_plan$dots %>%
        keep(is.list) %>%
        map_dfr(function(x) {
          span_df <- x %>%
            map(~clean_col_names(., c())) %>%
            reduce(crossing) %>%
            unnest(
              cols = tidyselect::everything()
            )
          names(span_df) <- names(x)
          span_df
        })

      col_def <- bind_rows(low_lvl_def, span_df)

    }

    # get col_style_plan referenced cols
    if (col_style_plan_test_res){
      cols_from_sp <- map(col_style_plan, ~.x$cols)  |>
        list_flatten() |>
        clean_col_names(dont_inc = grp_lb_vars)  %>%
        tibble(.)
      names(cols_from_sp) <- last(column_vars)

      col_def <- bind_rows(col_def, cols_from_sp) |> unique()
    }

  }
  else {
    n_cols <- ifelse(is.null(n_cols), 3, n_cols)
    col_def <- tibble(!!column_vars[n_spans] := paste0(column_vars[[n_spans]], seq(1:n_cols)))
    if(n_spans > 1){
      col_spans_df <- map_dfc(seq_len(n_spans-1), function(x){
        tibble(!!column_vars[x] := rep(paste0("span_", column_vars[x]), n_cols))
      })
      col_def <- bind_cols(col_spans_df, col_def)
    }
  }


  col_def
}

add_mock_big_ns <- function(data, column, param, big_n_struct){
  if(!is.null(big_n_struct)){
    col <- column %>% last()
    col_vals <- data %>%
      pull(!!col) %>%
      unique()

    data <- tibble(!!col := col_vals,
           !!param := big_n_struct$param_val) %>%
      bind_rows(data, .)
  }
  data
}

# Check the col plan contain positive information, isn't null, and doesn't contain `everything()`
col_plan_test <- function(col_plan){
  if(is.null(col_plan)){
    out <- FALSE
  } else {
    all_names <- col_plan$dots %>%
      map_chr(as_label)
    first_chr <- all_names %>%
      str_sub(end = 1)
    out <- (!all(first_chr == "-")) && (!"everything()" %in% all_names)
  }
  out
}


# check the col_style_plan contains something besides `everything()`
col_style_plan_test <- function(col_style_plan){
  if(is.null(col_style_plan)){
    out <- FALSE
  } else {
    all_names <- map(col_style_plan, ~.x$cols)  |>
      list_flatten() %>%
      map_chr(as_label)
    out <- !all("everything()" %in% all_names)
  }
  out
}

#' Print to JSON
#'
#' @param tfrmt tfrmt to print
#' @param path file path to save JSON to. If not provided the JSON will just print to the console
#'
#' @return JSON
#' @export
#' @examples
#' tfrmt(
#'   label = label,
#'   column = column,
#'   param = param,
#'   value=value) |>
#'   tfrmt_to_json()
#'
#' @importFrom jsonlite toJSON validate
tfrmt_to_json <- function(tfrmt, path = NULL){
  if(!is_tfrmt(tfrmt)){
    stop("Needs tfrmt")
  }
  output <- as_json(tfrmt)

  if(!is.null(path)){
    message(paste0("Writing json file out to:\n", path))
    write(output, path)
  } else {
    return(output)
  }

}


#' Convert tfrmt elements
#'
#' This collection of functions is used to create a json or json ready objects.
#' While primarily intended for internal use these functions can be used externally
#'
#' @param x tfrmt or tfrmt element
#'
#' @return as_json.tfrmt() will return a json object all other methods will
#'   return a list that is cleaned so it can be directly converted to a json
#'   object
#'
#' @keywords internal
#' @export
as_json <- function(x){
  UseMethod("as_json", x)
}

#' @export
#' @importFrom jsonlite toJSON validate
#' @importFrom stringr str_replace_all
as_json.tfrmt <- function(x){
  # Prepare each element to get converted to JSON
  tfrmt_nm <- names(x)

  out <- vector("list", length = length(x))
  for(i in 1:length(x)){
    out[[i]] <- x[[i]] %>%
      as_json()
  }
  names(out) <- tfrmt_nm

  # Converts list to a json object
  output_json <- out %>%
    toJSON(pretty = TRUE, force = TRUE)

  # Removing names added in by jsonlite
  # only change spaces at the beginning of the row
  json_split <- output_json %>%
    str_split("\\\n") %>%
    unlist()
  # Needs updating
  to_replace <- str_which(json_split, '^\\s+\"\\s(\\.\\d+)?\"')
  json_split[to_replace] <- json_split[to_replace] %>%
    str_replace_all('\"\\s(\\.\\d+)?\"',  '\"\"')
  json_clean <- str_c(json_split, collapse = "\n")
  if(validate(json_clean)){
    class(json_clean) <- "json"
  } else {
    stop("Error when creating the json object")
  }
  json_clean
}

#' @export
as_json.default <- function(x){
  x
}

#' @export
as_json.quosures <- function(x){
  x %>%
    map_chr(.,as_label)
}


#' @export
as_json.quosure <- function(x){
  out <- x %>%
    as_label()
  if(out != "<empty>") {
    return(out)
  }
}

#' @export
as_json.body_plan <- function(x){
  x %>%
    map(as_json)
}

#' @export
as_json.frmt_structure <- function(x){
  x$frmt_to_apply <- x$frmt_to_apply %>%
    map(as_json)

  list(group_val = x$group_val,
       label_val = x$label_val,
       param_val = x$param_val) %>%
    c(x$frmt_to_apply[[1]][1])
}

#' @export
as_json.frmt <- function(x){
  if(!is.null(x$transform)){
    x$transform <- deparse(x$transform) %>%
      str_c(collapse = "")
  }
  list(frmt = x)
}

#' @export
as_json.frmt_when <- function(x){
  lhs <- map_chr(x$frmt_ls, f_lhs_as_char)
  rhs <- map(x$frmt_ls, f_rhs) %>% map(as_json)
  names(rhs) <- lhs
  list(frmt_when = list(frmt_ls = rhs, missing = x$missing))
}

#' @export
as_json.frmt_combine <- function(x){
  frmts <- x$frmt_ls %>%
    map(as_json)
  list(frmt_combine = list(expression = x$expression,
                           frmt_ls = frmts, missing = x$missing))
}

#' @export
as_json.col_plan <- function(x){
  if(is.null(x)){
    c()
  } else {
    dot_ls <- x$dots %>%
      map(as_json)

    if(!is.null(names(dot_ls))){
      names(dot_ls) <- names(x$dots) %>% ifelse(.=="", " ", .)
    }
    list(col_plan = list(dots = dot_ls, .drop = x$.drop))
  }
}

#' @export
as_json.span_structure <- function(x){
  x %>%
    map(function(foo){
      foo %>%
        map_chr(as_json)
    }) %>%
    list(span_structure = .)
}


#' @export
as_json.col_style_plan <- function(x){
  x %>%
    map(as_json)
}

#' @export
as_json.col_style_structure <- function(x){
  x$cols <- x$cols %>%
    map(as_json)
  x
}

#' @export
as_json.page_plan <- function(x) {
  if (!is.null(x$transform)) {
    x$transform <- rlang::expr_text(x$transform)
  }

  x
}

#' json to tfrmt
#'
#' Reader to read JSON files/objects into tfrmt objects
#' @param path location of the json file to read in
#'
#' @param json json object to read in. By default this is null. This function
#'   will read in json object preferentially. So if both a path and a json
#'   object are supplied the json object will be read in.
#'
#' @importFrom jsonlite read_json parse_json
#' @export
json_to_tfrmt <- function(path = NULL, json = NULL){
  if(!is.null(json)){
    dirty_list <- parse_json(json)
  } else if(!is.null(path)){
    dirty_list <- read_json(path)
  } else {
    stop("Path or json object needed")
  }

  json_nm <- names(dirty_list)
  simple_vars <- c("group", "label", "param", "value", "column",
                   "title", "subtitle", "sorting_cols")

  simple_params <- dirty_list[intersect(simple_vars, json_nm)] %>%
    map(unlist)

  rgp <- ls_to_row_grp_plan(dirty_list$row_grp_plan)
  bp <- ls_to_body_plan(dirty_list$body_plan)
  csp <- ls_to_col_style_plan(dirty_list$col_style_plan)
  cp <- ls_to_col_plan(dirty_list$col_plan)
  bn <- ls_to_big_n(dirty_list$big_n)
  fnp <- ls_to_footnote_plan(dirty_list$footnote_plan)
  pp <- ls_to_page_plan(dirty_list$page_plan)

  all_params <- c(simple_params,
                  list(row_grp_plan = rgp),
                  list(body_plan = bp),
                  list(big_n = bn),
                  list(footnote_plan = fnp),
                  list(col_plan = cp),
                  list(col_style_plan = csp),
                  list(page_plan = pp)) %>%
    discard(is.null)
  do.call(tfrmt, all_params)
}

ls_to_row_grp_plan <- function(ls){
  if(!is.null(ls)){
    struct_ls <- ls$struct_list %>%
      map(function(struct){
        el_block <- do.call(element_block, struct$block_to_apply %>% map(unlist))
        if(!is.null(names(struct$group_val))){
          group_val = map(struct$group_val, unlist)
        } else {
          group_val = unlist(struct$group_val)
        }
        do.call(row_grp_structure, list(group_val = group_val, element_block = el_block))
      })

    label_loc <- do.call(element_row_grp_loc, ls$label_loc %>% map(unlist))
    ls <- do.call(row_grp_plan, c(struct_ls, list(label_loc = label_loc)))
  }
  ls
}

#' @importFrom stringr str_which
ls_to_body_plan <- function(ls){
  if(!is.null(ls)){
    frmts_ls <- ls %>%
      map(function(struct){
        frmt_loc <- str_which(names(struct), "frmt*")
        type <- names(struct)[frmt_loc]
        frmt_val <- do.call(paste0("ls_to_", type), list(x = struct[[type]])) %>%
          list()
        group_val = simplify_group_val(struct$group_val)
        arg_list <- c(list(group_val = group_val),
                      list(label_val= unlist(struct$label_val)),
                      frmt_val)
        out_struct <- do.call(frmt_structure, arg_list)
        out_struct$param_val <- unlist(struct$param_val)
        out_struct
      })

    do.call(body_plan, frmts_ls)
  }
}

ls_to_frmt <- function(x){
  x <- x %>%
    map(unlist)

  if(!is.null(x$transform)){
    x$transform <- str2lang(x$transform)
  }
  do.call(frmt, list(expression = x$expression, missing = unlist(x$missing),
                     scientific = x$scientific,
                     transform = x$transform))
}

ls_to_frmt_combine <- function(x){
  fmts <- x$frmt_ls %>%
    map(function(a_frmt){
      do.call(paste0("ls_to_", names(a_frmt)), list(x = a_frmt[[1]]))
    })
  x <- x %>%
    map(unlist)
  do.call(frmt_combine, c(list(expression = x$expression),
                          fmts, list(missing = x$missing)))
}

#' @importFrom stats as.formula
ls_to_frmt_when <- function(x){
  fmts <- x$frmt_ls %>%
    map(function(a_frmt){
      if(!is.null(names(a_frmt)))
      {
        out <- do.call(paste0("ls_to_", names(a_frmt)), list(x = a_frmt[[1]])) %>%
          as.character()
      } else {
        out <- str_c("'", a_frmt[[1]], "'")
      }
      out
    })

  lhs <- if_else(names(fmts) == "TRUE", names(fmts), str_c("'", names(fmts), "'"))
  formula_ls <- str_c(lhs, " ~ ", fmts) %>%
    map(as.formula)

  do.call(frmt_when, c(formula_ls, list(missing = unlist(x$missing))))
}

ls_to_big_n <- function(ls){
  if(!is.null(ls)){
    n_frmt = do.call(ls_to_frmt, list(ls$n_frmt))
    do.call(big_n_structure, list(param_val = unlist(ls$param_val),
                                  n_frmt = n_frmt,
                                  by_page = unlist(ls$by_page)))
  }
}

ls_to_footnote_plan <- function(ls){
  if(!is.null(ls)){
    struct_ls <- ls$struct_list %>%
      map(function(struct){
        group_val = simplify_group_val(struct$group_val)
        column_val = simplify_group_val(struct$column_val)
        unlisted <- struct%>%
          map(unlist)
        do.call(footnote_structure, list(footnote_text = unlisted$footnote_text,
                                         group_val = group_val,
                                         label_val = unlisted$label_val,
                                         column_val = column_val))

      })

    do.call(footnote_plan, c(struct_ls, list(
      marks = unlist(ls$marks),
      order = unlist(ls$order)
      )))
  }
}

#' @importFrom rlang parse_expr quo_get_expr
ls_to_col_plan <- function(ls){
  if(!is.null(ls)){
    dots <- ls$col_plan$dots %>%
      map(function(el){
        if(!is.null(names(el)) && names(el) == "span_structure"){
          el$span_structure %>%
            ls_to_span_structure() %>%
            as.character() %>%
            parse_expr()
        } else{

          el[[1]] %>%
            str_replace_all("\\\"", "'") %>%
            char_as_quo() %>%
            quo_get_expr()

        }
      })

    do.call(col_plan, c(dots, list(.drop = unlist(ls$col_plan$.drop))))
  }

}

ls_to_span_structure <- function(ls){
  span_ls <- ls %>%
    map(~unlist(.) %>%
          str_c("'", . ,"'") %>%
          str_c(collapse = ", ") %>%
          str_c("c(", ., ")") %>%
          parse_expr(.))

  do.call(span_structure, span_ls)
}

#' @importFrom rlang expr_text
ls_to_col_style_plan <- function(ls){
  if(!is.null(ls)){
    struct_ls <- ls %>% map(function(struct){
      stuct_in <- struct %>% map(unlist)
      names(stuct_in) <- names(stuct_in) %>% str_replace("cols", "col")
      cols_val <- struct[["cols"]][[1]]
      if(!is.null(names(cols_val)) && names(cols_val) == "span_structure"){
        stuct_in[["col"]] <-  cols_val[[1]] %>%
          ls_to_span_structure() %>%
          as.character() %>%
          parse_expr()
      } else {
        stuct_in[["col"]] <- map_chr(stuct_in[["col"]], ~ char_as_quo(.x) %>%
                                       quo_get_expr() %>%
                                     expr_text()) %>%
            str_c(., collapse = ", ") %>%
            paste0("vars(",., ")") %>%
          parse_expr()
      }
      do.call(col_style_structure, stuct_in)
    })

    do.call(col_style_plan, struct_ls)
  }
}

ls_to_page_plan <- function(ls) {
  if (!is.null(ls)) {
    struct_ls <- ls$struct_list |>
      purrr::map(
        function(struct) {
          group_val <- simplify_group_val(struct$group_val)

          unlisted <- purrr::map(struct, unlist)

          do.call(
            page_structure,
            list(
              group_val = group_val,
              label_val = unlisted$label_val
            )
          )
        }
      )

    transform <- unlist(ls$transform)

    if (!is.null(transform)) {
      transform <- rlang::parse_expr(transform)
    }

    do.call(
      page_plan,
      c(
        struct_ls,
        list(
          note_loc = unlist(ls$note_loc),
          max_rows = unlist(ls$max_rows),
          transform = transform
        )
      )
    )
  }
}

simplify_group_val <- function(group_ls){
  if(length(group_ls) == 0){
    group_val = NULL
  } else if(!is.null(names(group_ls))){
    group_val = map(group_ls, unlist)
  } else {
    group_val = unlist(group_ls)
  }
  group_val
}

#' Apply formatting
#'
#' @param .data data, but only what is getting changed
#' @param frmt_def formatting to be applied
#' @param value value symbol should only be one
#' @param mock Logical value is this is for a mock or not. By default `FALSE`
#' @param ... additional arguments for methods
#' @param param param column as a quosure
#' @param column column columns as a list of quosures
#' @param label label column as a quosure
#' @param group group column as a list of quosures
#'
#' @return formatted dataset
#'
#' @importFrom stringr str_count str_trim str_dup str_c str_remove
#' @importFrom dplyr if_else case_when tibble
#' @export
#' @examples
#'
#' library(tibble)
#' library(dplyr)
#' # Set up data
#' df <- tibble(x = c(20.12,34.54,12.34))
#'
#' apply_frmt(
#'  frmt_def = frmt("XX.X"),
#'  .data=df,
#'  value=quo(x))
#'
#' @rdname apply_frmt
apply_frmt <- function(frmt_def, .data, value, mock = FALSE, ...){
  UseMethod("apply_frmt", frmt_def)
}


#' @importFrom stringr str_count str_trim str_dup str_c str_remove str_extract str_detect
#' @importFrom dplyr case_when tibble pull mutate
#' @importFrom rlang := as_function
#' @export
#'
#' @rdname apply_frmt
apply_frmt.frmt <- function( frmt_def, .data, value, mock = FALSE, ...){
  if(mock){
    out <- .data %>%
      mutate(!!value := frmt_def$expression)
  } else {
    vals <- .data %>%
      pull(!!value)

    if(length(vals) == 0){
      return(.data)
    } else if(!is.null(frmt_def$transform)){
      vals <- as_function(frmt_def$transform)(vals)
    }

    if(str_detect(frmt_def$expression, "[x|X]")){

      # digits following period in expression
      dig <- frmt_def$expression %>%
        str_extract("(?<=\\.)[X|x]+") %>%
        str_count("[X|x]")

      ## There were no x's after a `.` to extract, so assume none
      if(is.na(dig)){
        dig <- 0
      }

      ## convert to scientific if scientific
      if(!is.null(frmt_def$scientific)){

        vals_sci <- format(vals, scientific = TRUE)

        vals <- vals_sci %>%
          str_extract("[^e]+") %>%
          as.numeric()

        ## remove x's from end of scientific
        multiply <- str_remove(frmt_def$scientific, "[xX]+(?<=$)")
        sci_width <- str_extract(frmt_def$scientific, "[xX]+(?<=$)") %>%
          str_count("[X|x]")

        vals_sci_post <- vals_sci %>%
          str_extract("[^e]+$") %>%
          as.numeric() %>%
          format(trim = TRUE,width = sci_width) %>%
          paste0(multiply,.)

      }else{
        vals_sci_post <- ""
      }

      # digits preceding period in expression
      pre_dec_expr <- frmt_def$expression %>%
        str_remove("\\..*$") %>%
        str_count("[X|x]")

      # vals rounded and trimmed
      rounded_vals <- format(
        round(vals, dig),
        decimal.mark = ".",
        nsmall = dig
        ) %>%
        str_trim()

      fmt_options <- tibble(
        rounded = rounded_vals,
        # digits preceding period in vals
        act_pre_dec  = rounded_vals %>%
          str_remove("\\..*$") %>%
          str_count(".")
        ) %>%
        mutate(
          # keep from being negative
          space_to_add = pmax(pre_dec_expr - .data$act_pre_dec, 0),
        )


      # when scientific is null paste rounded value, if not then append scientific expression
      fmt_vals <- str_c(str_dup(" ", fmt_options$space_to_add), fmt_options$rounded, vals_sci_post)

      expr_start <- frmt_def$expression %>%
          str_extract("^[^X|^x]*(?=[X|x])")

      expr_end <- frmt_def$expression %>%
          str_extract("(?<=[X|x])[^X|^x]*$")

      if(!is.null(frmt_def$missing)){
        miss_val <- frmt_def$missing
      } else {
        miss_val <- NA_character_
      }

      # Combining the additional formatting
      fmt_val_output <- case_when(
        fmt_options$rounded == "NA" ~ miss_val,
        TRUE ~ str_c(expr_start, fmt_vals, expr_end)
      )

    } else {
      fmt_val_output <- frmt_def$expression
    }

    out <- .data %>%
      mutate(
        !!value := fmt_val_output
      )
  }

  out

}


#' @importFrom stringr str_extract_all str_count str_trim str_dup str_c str_remove str_glue
#' @importFrom dplyr case_when tibble filter pull left_join
#' @importFrom tidyr pivot_wider replace_na
#' @importFrom purrr map_dfr map_chr discard
#' @importFrom rlang :=
#' @export
#'
#' @rdname apply_frmt
apply_frmt.frmt_combine <- function(frmt_def, .data, value, mock = FALSE, param, column, label, group, ...){

  fmt_param_vals <- frmt_def$expression %>%
    str_extract_all("(?<=\\{)[^\\}]+(?=\\})") %>%
    unlist()

  # Adding the unquoted version to match while long
  fmt_param_vals_uq <- str_remove_all(fmt_param_vals, "`")

  # Check if unspecified param values are in the dataset

  if(!setequal(names(frmt_def$frmt_ls), fmt_param_vals)){
    stop("The values in the expression don't match the names of the given formats ")
  }

  ## format params as needed
  .tmp_data <- map_dfr(fmt_param_vals, function(`__var`){
    fmt_to_apply <- frmt_def$frmt_ls[[`__var`]]
    .data %>%
      filter(!!param == str_remove_all(`__var`, "`")) %>%
      apply_frmt(
        frmt_def = fmt_to_apply,
        .data = .,
        value = value,
        column = column,
        param = param,
        label = label,
        group = group,
        mock=mock,
        ...
      )
  })

  #Test if common information exists
  miss_param_from_data <- .tmp_data %>%
    pull(!!param) %>%
    unique() %>%
    setdiff(fmt_param_vals_uq, .)

  if(length(miss_param_from_data) > 0 ){
    stop(paste0("Unable to create formatting combination because the following parameters are missing from the data:\n ",
                paste0(miss_param_from_data, collapse = " \n")))
  }

  .tmp_data_wide <- .tmp_data %>%
    select(!!value, !!param, !!!column, !!label, !!!group) %>%
    pivot_wider(
      values_from = !!value,
      names_from = !!param
    ) %>%
    mutate(
      .is_all_missing =  all_missing(fmt_param_vals,.)
    )

  missing_param_replacements <-
    map(fmt_param_vals, ~ frmt_def$frmt_ls[[.x]]$missing) %>%
    setNames(fmt_param_vals) %>%
    discard(is.null)

  if(length(missing_param_replacements)>0){
    ## after .is_all_missing so that can be tabulated first
    .tmp_data_wide <- .tmp_data_wide %>%
      replace_na(missing_param_replacements)
  }

  # check that pivot_wider resulted in a reduction of rows, which indicates that at least
  #  1 row will successfully have a frmt_combine in it
  if (nrow(.tmp_data_wide)==nrow(.tmp_data)){
    id_cols <- .tmp_data %>% select(!!!column, !!label, !!!group, !!param)
    warning(paste0("Unable to apply `frmt_combine` due to uniqueness of column/row identifiers. Params that are to be combined need to have matching values across: ",
                   paste(names(id_cols %>% select(-!!param)), collapse = ", "),
                   ". Current values:\n",
                   paste(capture.output(id_cols %>% as.data.frame), collapse = "\n")))
  }


  if(is.null(frmt_def$missing)){
    frmt_def$missing <- ""
  }

  ## if both params are missing, then drop in frmt definition missing value
  ## otherwise concat the params
  .tmp_data_fmted <- .tmp_data_wide %>%
    mutate(
      !!value := case_when(
        .data$.is_all_missing ~ frmt_def$missing,
        TRUE ~ str_glue(!!frmt_def$expression) %>% as.character()
      )
    ) %>%
    select(
      -tidyselect::all_of(
        fmt_param_vals_uq
      ),
      -".is_all_missing"
    )

  ## if not mock remove
  if(!mock){
    .data <- .data %>%
      select(-!!value)
  }

  merge_group <- map(
    c(column, label, group),
    function(x){
      if(!quo_is_missing(x)){x}
    }) %>%
    discard(is.null) %>%
    do.call("vars", .)

  # merge on new values, and remove cases other than first occurance of group/label/column pairing
  .data %>%
    left_join(
      .tmp_data_fmted,
      by = map_chr(merge_group, as_label)
    ) %>%
    group_by(!!!merge_group) %>%
    slice(1) %>%
    ungroup()

}

#' @export
#' @importFrom rlang as_label f_rhs f_lhs parse_exprs eval_tidy
#' @importFrom dplyr pull if_else mutate
#' @importFrom purrr map map_chr keep
#' @importFrom rlang :=
#' @importFrom tidyr replace_na
#'
#' @rdname apply_frmt
apply_frmt.frmt_when <- function(frmt_def, .data, value, mock = FALSE, ...){

  if(mock){
    frmt_to_prt <- frmt_def$frmt_ls %>%
      keep(~f_lhs(.) == "TRUE")
    if(length(frmt_to_prt) < 1){
      frmt_to_prt <- frmt_def$frmt_ls
    }
    str_to_prnt <- f_rhs(frmt_to_prt[[1]])$expression
    out <- .data %>%
      mutate(!!value := str_to_prnt)

  } else {
    values_str <- as_label(value)
    n <- length(frmt_def$frmt_ls)

    val_len <- length(pull(.data, !!value))
    right <- frmt_def$frmt_ls %>%
      map(f_rhs) %>%
      map(function(x) {
        if(is_frmt(x)){
          out <- apply_frmt(x, .data, value, ...) %>% pull(!!value)
        } else {
          out <- rep(x, val_len)
        }
        out
      })

    left <- frmt_def$frmt_ls %>%
      map_chr(f_lhs_as_char) %>%
      if_else(. == "TRUE", ., paste0(values_str, .)) %>%
      parse_exprs() %>%
      map(eval_tidy, .data)

    out <- rep(NA_character_, val_len)
    replaced <- rep(FALSE, val_len)

    for(i in seq_len(n)){
      out <- replace_val(out, left[[i]] & !replaced, right[[i]])
      replaced <- replaced | (left[[i]] & !is.na(left[[i]]))
    }

    if (is.null(frmt_def$missing)){
      out <- out
    } else if (!is.null(frmt_def$missing)){
      out <- out %>% replace_na(replace = frmt_def$missing)
    }

    out <- .data %>%
      mutate(
        !!value := out
      )
  }
  out

}

#' Define the Column Plan & Span Structures
#'
#' Using <[`tidy-select`][dplyr::dplyr_tidy_select]> expressions and a series
#' span_structures, define the order of the columns. The selection follows "last
#' selected" principals, meaning columns are moved to the _last_ selection as
#' opposed to preserving the first location.
#'
#'
#' @details
#'
#' ## Column Selection
#'
#' When col_plan gets applied and is used to create the output table, the
#' underlying logic sorts out which column specifically is being selected. If a column
#' is selected twice, the _last_ instance in which the column is selected will be
#' the location it gets rendered.
#'
#' Avoid beginning the \code{col_plan()} column selection with a deselection (i.e.
#' \code{col_plan(-col1)}, \code{col_plan(-starts_with("value")))}. This will
#' result in the table preserving all columns not "de-selected" in the
#' statement, and the order of the columns not changed. It is preferred when
#' creating the \code{col_plan()} to identify all the columns planned on
#' preserving in the order they are wished to appear, or if
#' <[`tidy-select`][dplyr::dplyr_tidy_select]> arguments - such as
#' \code{\link[dplyr]{everything}}- are used, identify the de-selection after
#' the positive-selection.
#'
#' Alternatively, once the gt table is produced, use the
#' \code{\link[gt]{cols_hide}} function to remove un-wanted columns.
#'
#'
#' @rdname col_plan
#'
#' @param ... For a col_plan and span_structure,
#'   <[`tidy-select`][dplyr::dplyr_tidy_select]> arguments, unquoted expressions
#'   separated by commas, and span_structures. span_structures must have the
#'   arguments named to match the name the column in the input data has to identify the correct columns. See the examples
#' @param .drop Boolean. Should un-listed columns be dropped from the data.
#'   Defaults to FALSE.
#'
#' @seealso \href{https://gsk-biostatistics.github.io/tfrmt/articles/col_plan.html}{Link to related article}
#'
#' @return col_plan object
#' @export
#' @examples
#'
#' library(dplyr)
#'
#' ## select col_1 as the first column, remove col_last, then create spanning
#' ## structures that have multiple levels
#' ##
#' ## examples also assume the tfrmt has the column argument set to c(c1, c2, c3)
#' ##
#' spanning_col_plan_ex <- col_plan(
#'  col_1,
#'  -col_last,
#'  span_structure(
#'    c1 = "Top Label Level 1",
#'    c2 = "Second Label Level 1.1",
#'    c3 = c(col_3, col_4)
#'  ),
#'  span_structure(
#'    c1 = "Top Label Level 1",
#'    c2 = "Second Label Level 1.2",
#'    c3 = starts_with("B")
#'    ),
#'  span_structure(
#'    c1 = "Top Label Level 1",
#'    c3 = col_5
#'  ),
#'  span_structure(
#'    c2 = "Top Label Level 2",
#'    c3 = c(col_6, col_7)
#'  )
#' )
#'
#' ## select my_col_1 as the first column, then
#' ## rename col_2 to new_col_1 and put as the
#' ## second column, then select the rest of the columns
#' renaming_col_plan_ex <- col_plan(
#'    my_col_1,
#'    new_col_1 = col_2,
#'    everything()
#'  )
#'
#' renaming_col_plan_ex2 <- col_plan(
#'    my_col_1,
#'    new_col_1 = col_2,
#'    span_structure(
#'     c1 = c(`My Favorite span name` = "Top Label Level 1"),
#'     c3 = c(`the results column` = col_5)
#'    )
#'  )
#'
#' ## To one or more stub headers rename the corresponding group/label variables in the column plan.
#' ## If multiple group variables exist, any of them can be renamed.
#' ## Note: Multiple stub headers are only possible if the `row_grp_plan` label
#' ## location is set to "column". Otherwise, if more than one group/label column is renamed,
#' ## {tfrmt} will use the highest level group name available.
#'
#'  renaming_group <- col_plan(
#'     my_grp = group, # rename group
#'     label,
#'     starts_with("col")
#'   )
#'
##' @section Images:
#' Here are some example outputs:
#'
#' \if{html}{\out{
#' `r "<img src=\"https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/main/images/tfrmt-span_structure-cropped.jpg\" alt =\"Example of a dataset being turned into a table with spanning columns\" style=\"width:100\\%;\">"`
#' }}
#'
col_plan <- function(..., .drop = FALSE) {
  ## selectively evaluate dots (only if is a span_structure)
  ## confirm contents otherwise
  dots <- as.list(substitute(substitute(...)))[-1]
  dots <- check_col_plan_dots(dots)

  structure(
    list(
      dots = dots,
      .drop = .drop
    ),
    class = c("col_plan", "plan")
  )
}

#' @rdname col_plan
#'
#' @return span_structure object
#'
#' @export
span_structure <- function(...) {
  span_cols <- as.list(substitute(substitute(...)))[-1]
  span_cols <- check_span_structure_dots(span_cols)

  structure(
    span_cols,
    class = c("span_structure")
  )
}

is_span_structure <- function(x) {
  inherits(x, "span_structure")
}

#' @importFrom rlang eval_tidy
#' @noRd
check_span_structure_dots <- function(x) {
  x_names <- names(x)

  if (is.null(x_names) | any(x_names == "")) {
    abort(
      paste0(
        "Entries of a span_stucture must be named:\n ",
        format(caller_call())
      ),
      call = caller_call()
    )
  }

  x_dots <- x %>%
    map(
      ~ lapply(trim_vars_quo_c(.x), function(x) {
        if (is.name(x)) {
          if (identical(as_label(x), "<empty>")) {
            return(NULL)
          } else {
            return(quo(!!x))
          }
        } else if (is.call(x)) {
          if (is_valid_tidyselect_call(x)) {
            quo(!!x)
          } else if (is_valid_quo_call(x)) {
            return(eval_tidy(x))
          } else {
            abort(
              message = paste0(
                "Invalid entry: `",
                format(x),
                "`\n",
                "Only selection helpers (See <https://tidyselect.r-lib.org/reference>), ",
                " or unquoted expressions representing variable names ",
                " can be entered as contents.",
                " Changing the names of individual variables using new_name = old_name syntax is allowable"
              ),
              call = caller_call()
            )
          }
        } else if (is.character(x)) {
          return(as_length_one_quo.character(x))
        } else {
          abort(
            "Unexpected entry type in span_structure()",
            call = caller_call()
          )
        }
      })
    )

  x_dots[!sapply(x_dots, is.null)]
}

is_valid_span_structure_call <- function(x) {
  as.character(as.list(x)[[1]]) %in% c("span_structure")
}

is_valid_tidyselect_call <- function(x) {
  ## drop - from determining if
  if (as.character(as.list(x)[[1]]) == "-") {
    x <- x[[-1]]
    if (is.name(x)) {
      return(TRUE)
    }
  }
  as.character(as.list(x)[[1]]) %in% c(names(tidyselect::vars_select_helpers))
}

is_valid_quo_call <- function(x) {
  ## drop - from determining if
  if (as.character(as.list(x)[[1]]) == "-") {
    x <- x[[-1]]
    if (is.name(x)) {
      return(TRUE)
    }
  }
  as.character(as.list(x)[[1]]) %in% c("vars", "quo")
}

check_col_plan_dots <- function(x) {
  lapply(x, function(x) {
    if (is.name(x)) {
      if (identical(as_label(x), "<empty>")) {
        return(NULL)
      } else {
        return(quo(!!x))
      }
    } else if (is.call(x)) {
      if (is_valid_tidyselect_call(x)) {
        quo(!!x)
      } else if (is_valid_quo_call(x) | is_valid_span_structure_call(x)) {
        return(eval_tidy(x))
      } else {
        stop(
          "Invalid entry: `",
          format(x),
          "`\n",
          "Only span_structures (`span_structure()`), ",
          "selection helpers (See <https://tidyselect.r-lib.org/reference>), ",
          " or unquoted expressions representing variable names ",
          " can be entered as contents.",
          " Changing the names of individual variables using new_name = old_name syntax is allowable",
          call. = FALSE
        )
      }
    } else if (is.character(x)) {
      return(as_length_one_quo.character(x))
    } else {
      stop("Unexpected entry type in span_structure()")
    }
  })
}

#' Footnote Plan
#'
#' Defining the location and content of footnotes with a series of footnote
#' structures. Each structure is a footnote and can be applied in multiple locations.
#'
#' @param ...  a series of [footnote_structure()] separated by commas
#' @param marks type of marks required for footnotes, properties inherited from
#'   tab_footnote in 'gt'. Available options are "numbers", "letters",
#'   "standard" and "extended" (standard for a traditional set of 4 symbols,
#'   extended for 6 symbols). The default option is set to "numbers".
#' @param order how to order footnotes in the footer. Options are "marks_first" (anchored notes appear first, default),
#'   "preserve_order" or "marks_last" (general notes appear first).
#'
#'
#' @return footnote plan object
#' @export
#'
#' @examples
#'
#' # Adds a footnote indicated by letters rather than numbers to Group 1
#' footnote_plan <- footnote_plan(
#'     footnote_structure(footnote_text = "footnote", group_val = "Group 1"),
#'     marks = "letters")
#'
#' # Adds a footnote to the 'Placebo' column
#' footnote_plan <- footnote_plan(
#'     footnote_structure(footnote_text = "footnote", column_val = "Placebo"),
#'     marks = "numbers")
#'
#' # Preserve order of footnotes
#' footnote_plan <- footnote_plan(
#'     footnote_structure(footnote_text = "footnote 1", group_val = "Group 1"),
#'     footnote_structure(footnote_text = "footnote 2", column_val = "Placebo"),
#'     order = "marks_first")
#'
footnote_plan <- function(...,
                          marks = c("numbers","letters","standard","extended"),
                          order = c("marks_first", "preserve_order", "marks_last")){
  footnote_structure_list <- list(...)
  marks = match.arg(marks)
  order = match.arg(order)

  structure(
    list(
      struct_list = footnote_structure_list,
      marks = marks,
      order = order),
    class = c("footnote_plan", "plan")
  )
}

#' Footnote Structure
#'
#' @param footnote_text string with text for footnote
#' @param column_val string or a named list of strings which represent the column to apply the footnote to
#' @param group_val string or a named list of strings which represent the value of group to apply the footnote to
#' @param label_val string which represents the value of label to apply the footnote to
#'
#' @return footnote structure object
#' @export
#'
#' @examples
#'
#' # Adds a source note aka a footnote without a symbol in the table
#' footnote_structure <- footnote_structure(footnote_text = "Source Note")
#'
#' # Adds a footnote to the 'Placebo' column
#' footnote_structure <- footnote_structure(footnote_text = "Text",
#'                                column_val = "Placebo")
#'
#' # Adds a footnote to either 'Placebo' or 'Treatment groups' depending on which
#' # which is last to appear in the column vector
#' footnote_structure <- footnote_structure(footnote_text = "Text",
#'       column_val = list(col1 = "Placebo", col2= "Treatment groups"))
#'
#' # Adds a footnote to the 'Adverse Event' label
#' footnote_structure <- footnote_structure("Text", label_val = "Adverse Event")
footnote_structure <- function(footnote_text, column_val = NULL, group_val = NULL, label_val = NULL){

  # force column_val and group_val into a list if a named vector
  if(length(column_val)>1 && is.list(column_val)==FALSE && !is.null(names(column_val))){
    column_val <- as.list(column_val)
  }else if(length(column_val)==1 && !is.null(names(column_val))){
    column_val<-as.list(column_val)
  }

  if(length(group_val)>1 && is.list(group_val)==FALSE && !is.null(names(group_val))){
    group_val <- as.list(group_val)
  }else if(length(group_val)==1 && !is.null(names(group_val))){
    group_val<-as.list(group_val)
  }

  # warnings if elements arent named

  if(is.list(column_val)){
    column_val_names <- names(column_val)
    if(is.null(column_val_names)){
      stop("when column_val is a list, must be a named list")
    }else if(any(column_val_names == "")){
      stop("when column_val is a list, each entry must be named")
    }
  }

  if(is.list(group_val)){
    group_val_names <- names(group_val)
    if(is.null(group_val_names)){
      stop("when group_val is a list, must be a named list")
    }else if(any(group_val_names == "")){
      stop("when group_val is a list, each entry must be named")
    }
  }

   structure(
    list(
      column_val = column_val,
      group_val = group_val,
      label_val = label_val,
      footnote_text = footnote_text),
    class = c("footnote_structure","structure")
  )


}

#' Row Group Plan
#'
#' Define the look of the table groups on the output. This function allows you to
#' add spaces after blocks and allows you to control how the groups are viewed
#' whether they span the entire table or are nested as a column.
#'
#' @seealso [row_grp_structure()] for more details on how to specify row group
#'   structures, [element_block()] for more details on how to specify spacing
#'   between each group, [element_row_grp_loc()] for more details on how to
#'   specify whether row group titles span the entire table or collapse.
#'
#'   \href{https://gsk-biostatistics.github.io/tfrmt/articles/row_grp_plan.html}{Link to related article}
#'
#' @param ...  Row group structure objects separated by commas
#' @param label_loc [element_row_grp_loc()] object specifying location
#'
#'
#' @return row_grp_plan object
#'
#' @examples
#'
#'
#'   ## single grouping variable example
#'   sample_grp_plan <- row_grp_plan(
#'     row_grp_structure(group_val = c("A","C"), element_block(post_space = "---")),
#'     row_grp_structure(group_val = c("B"), element_block(post_space = " ")),
#'     label_loc = element_row_grp_loc(location = "column")
#'   )
#'
#'   ## example with multiple grouping variables
#'   sample_grp_plan <- row_grp_plan(
#'      row_grp_structure(group_val = list(grp1 = "A", grp2 = "b"), element_block(post_space = " ")),
#'      label_loc = element_row_grp_loc(location = "spanning")
#'      )
#'
#' @export
#'
row_grp_plan <- function(..., label_loc = element_row_grp_loc(location = "indented")){

  row_grp_structure_list <- list(...)

  for(struct_idx in seq_along(row_grp_structure_list)){
    if(!is_row_grp_structure(row_grp_structure_list[[struct_idx]])){
      stop(paste0("Entry number ",struct_idx," is not an object of class `row_grp_structure`.
                  If you want specify `spanning_label` please enter 'spanning_label ='"))
    }
  }

  structure(
    list(struct_list = row_grp_structure_list, label_loc = label_loc),
    class = c("row_grp_plan", "frmt_table")
  )
}

#' Row Group Structure Object
#'
#' Function needed to create a row_grp_structure object, which is a building block
#' of [row_grp_plan()]
#'
#' @seealso [row_grp_plan()] for more details on how to group row group
#'   structures, [element_block()] for more details on how to specify spacing
#'   between each group.
#'
#'   \href{https://gsk-biostatistics.github.io/tfrmt/articles/row_grp_plan.html}{Link to related article}
#'
#' @param group_val A string or a named list of strings which represent the
#'   value of group should be when the given frmt is implemented
#' @param element_block element_block() object to define the block styling
#'
#' @returns row_grp_structure object
#' @export
#' @examples
#'
#' ## single grouping variable example
#' row_grp_structure(group_val = c("A","C"), element_block(post_space = "---"))
#'
#' ## example with multiple grouping variables
#' row_grp_structure(group_val = list(grp1 = "A", grp2 = "b"), element_block(post_space = " "))
#'
row_grp_structure <- function(group_val = ".default", element_block){

  if(!is_element_block(element_block)){
    stop("element_block, must be an element_block type")
  }

  if(is.list(group_val)){
    group_val_names <- names(group_val)
    if(is.null(group_val_names)){
      stop("when group_val is a list, must be a named list")
    }else if(any(group_val_names == "")){
      stop("when group_val is a list, each entry must be named")
    }
  }

  structure(
    list(
      group_val = group_val,
      block_to_apply = element_block),
    class = c("row_grp_structure","frmt_table")
  )
}

#' Apply row group structure formatting to data
#'
#' @param .data data
#' @param row_grp_struct_list list of row group structure objects
#' @param group symbolic list of grouping
#' @param label symbolic label column
#'
#' @noRd
#' @importFrom dplyr tibble mutate group_by arrange slice group_map case_when left_join row_number select summarise across
#' @importFrom purrr map map2_dfr
#' @importFrom tidyr unnest nest unnest_longer
#' @importFrom rlang !!!
#' @importFrom stringr str_split
apply_row_grp_struct <- function(.data, row_grp_struct_list, group, label = NULL, ...){
  # Locate which groups need which formatting
  # determine which rows each block applies to
  .data <- .data %>%
    mutate(TEMP_row = row_number())

  # for each structure object, (1) split the data on any default values, (2) split the data on specific data values
  # get nested list object:
  #  length = number of structures, each element contains list of data splits (row indices)
  TEMP_appl_row <- row_grp_struct_list %>%
    map(function(struct){
      grping <- expr_to_grouping(struct, group)

      split_dat <- .data %>%
        group_by(
          across(
            tidyselect::all_of(
              grping
            )
          )
        ) %>%
        group_split()
      map(split_dat, function(dat) struct_val_idx(struct, dat, group, label)) %>% list_flatten()
    })

  TEMP_block_to_apply <- row_grp_struct_list %>% map(~.$block_to_apply)

  # similar to frmts, only allow 1 element_block for a given row
  #   - within block-specific data, split data further by grouping vars
  dat_plus_block <- tibble(
    TEMP_appl_row,
    TEMP_block_to_apply)%>%
    mutate(TEMP_block_rank = row_number()) %>%
    # unnest to 1 rec per data chunk
    unnest_longer(TEMP_appl_row, indices_to = "TEMP_chunk_num", transform = unlist) %>%
    # unnest to 1 rec per data row, to handle where chunk >1 row
    unnest(TEMP_appl_row) %>%
    group_by(TEMP_appl_row) %>%
    arrange(TEMP_appl_row, desc(.data$TEMP_block_rank)) %>%
    slice(1) %>%
    left_join(.data, ., by= c("TEMP_row" = "TEMP_appl_row")) %>%
    group_by(.data$TEMP_block_rank, .data$TEMP_chunk_num, .data$TEMP_block_to_apply ) %>%
    nest()

  # get max character width for each column in the full data
  dat_max_widths <- .data %>%
    summarise(
      across(
        tidyselect::everything(),
        function(x) {
          if (is.character(x)) {
            str_split(x, "\\n") %>%
              unlist() %>%
              nchar() %>%
              max(na.rm = TRUE)
          } else{
            max(nchar(x), na.rm = TRUE)
          }
        }
      )
    )

  # apply group block function to data subsets
  add_ln_df <- map2_dfr(dat_plus_block$data,
                        dat_plus_block$TEMP_block_to_apply,
                        function(x,y) {
                          if(is.null(y)){
                            x
                          } else {
                            apply_grp_block(.data = x,
                                            group = group,
                                            element_block = y,
                                            widths = dat_max_widths)
                          }
                        }) %>%
    arrange(.data$TEMP_row) %>%
    select(-"TEMP_row")

  add_ln_df

}



#' Apply row group block labelled to data
#'
#' @param .data data
#' @param element_row_grp_loc element object specifying row group label location
#' @param group symbolic list of grouping
#' @param label symbolic label column
#'
#' @noRd
#' @importFrom dplyr select group_by
#' @importFrom rlang !!! eval_tidy
apply_row_grp_lbl <- function(.data, element_row_grp_loc, group, label = NULL, ...){

  # store values of label column
  lbl_col <- eval_tidy(label, .data)

  # check if lbl_col contains NA
  if (any(is.na(lbl_col))) {
    stop(paste0("`label` column ", quo_name(label), " contains NA values. For group-level summary data, `label` and the relevant `group` values should match."))
  }


  # check which group/label columns are available

  grps_avail <- eval_tidyselect_on_colvec(group, names(.data))

  if(length(grps_avail)==0 || is_empty(label) || element_row_grp_loc$location %in% c("gtdefault", "noprint", "column")){
    add_ln_df <- .data
  } else{
  #  combine any grouping columns that need combining into label
  add_ln_df <- .data %>% combine_group_cols(as_vars(grps_avail),
                                              label,
                                              element_row_grp_loc)

  }
  add_ln_df
}




#' Test of the grp rows in the data
#'
#' @param cur_block current row group block
#' @param .data data to test against NOTE: `TEMP_row` must be in the dataset
#' @param group list of the group parameters
#'
#' @return vector of the rows which this format could be applied to
#' @noRd
#'
#' @importFrom dplyr filter pull
#' @importFrom rlang parse_expr
grp_row_test_data <- function(cur_block, .data, group){

  filter_expr <- expr_to_filter.quosures(group, cur_block$group_val) %>%
    parse_expr()

  .data %>%
    mutate(across(c(!!!group), ~str_trim(.x))) %>%
    filter(!!filter_expr) %>%
    pull(.data$TEMP_row)
}


#' Apply row group element blocks
#'
#' @param .data data, but only what is getting changed
#' @param group list of the group parameters
#' @param element_block element_block to be applied
#' @param widths tibble of widths for each column
#'
#' @return dataset with element block applied
#'
#' @importFrom dplyr slice n mutate across bind_rows cur_column
#' @importFrom tidyr fill
#' @importFrom purrr map_chr
#' @importFrom rlang !!!
#'
#' @noRd
apply_grp_block <- function(.data, group, element_block, widths){

  if (!is.null(element_block$post_space)){

    # create add-on row
    # utilize TEMP_row to retain the ordering
    grp_row_add <- .data %>%
      slice(n()) %>%
      mutate(
        across(
          c(
            - map_chr(group, as_name),
            - tidyselect::where(is.numeric)),
          ~ replace(
            .x,
            values = fill_post_space(
              post_space = element_block$post_space,
              fill = element_block$fill,
              width = widths[[cur_column()]]
            )
          )
        ),
        TEMP_row = .data$TEMP_row + 0.1
      )


    # combine with original data
    bind_rows(.data, grp_row_add) %>%
      fill(!!!group) %>%
      mutate(..tfrmt_post_space_row = .data$TEMP_row %% 1 != 0)

  } else {
    .data
  }

}


#' Fill the cell value with post space character
#'
#' @param post_space Character value for post space
#' @param fill Whether to recycle value in `post_space` to match data width
#' @param width width to make the post_space value in order to fill the cell
#'
#' @return character value containing post space value modified to fill cell
#' @noRd
#'
#' @importFrom stringr str_sub
fill_post_space <- function(post_space, fill, width){

  ## if only white space, no need to make wider for visuals
  if(grepl("^\\s*$", post_space)){
    return(" ")
  }

  length_post_space <- nchar(post_space)

  if (fill) {
    reps <- ceiling(width/length_post_space)
    fill_val <- strrep(post_space, reps) %>% str_sub(1, width)
  } else {
    fill_val <- str_sub(post_space, 1, width) # truncate to data width if needed
  }

  return(fill_val)

}


#' Combine group cols into the Labels
#'
#' @param .data Pre-processed data that just needs columns combining
#' @param group list of the group parameters
#' @param label label symbol should only be one
#' @param element_row_grp_loc row group location element. If null then will just indent
#'
#' @return dataset with the group columns combines
#' @noRd
#' @importFrom dplyr group_by group_split mutate select distinct bind_rows across last any_of slice
#' @importFrom tidyr replace_na
#' @importFrom stringr str_trim
#' @importFrom purrr map_dfr map_chr
#' @importFrom forcats fct_inorder
#' @importFrom tibble add_row
combine_group_cols <- function(.data, group, label, element_row_grp_loc = NULL){

  orig_group_names <- map_chr(group, as_name)
  top_grouping <- group #used for spliting in case of spanning label

  .data <- .data %>%
    mutate(..tfrmt_row_grp_lbl = FALSE)

  # ensure label is character
  .data <- .data %>%
    mutate(across(!!label, ~as.character(.x)))

  if(is.null(element_row_grp_loc)){
    indent = "  "
  } else if(element_row_grp_loc$location=="spanning" & length(group) > 0){
    group = group[-1]
    indent = element_row_grp_loc$indent
  } else {
    indent = element_row_grp_loc$indent
  }

  while(length(group) > 0 & !is.null(label)){

    split_dat <- .data %>%
      group_by(run_id = dplyr::consecutive_id(!!!top_grouping)) %>%
      group_split() %>%
      map(~select(.x, -run_id))

    .data<- split_dat %>%
      map_dfr(function(lone_dat) {

        lone_dat_summ <- lone_dat %>%
          mutate(..tfrmt_summary_row = str_trim(!!label, side = "left") == str_trim(!!last(group), side = "left"))

        if (any(lone_dat_summ$..tfrmt_summary_row)==FALSE){

          # if the set of rows contains NO group-level summary data, create an
          # extra row to be added

          # first containing grouping/label values
          new_row <- lone_dat %>%
            select(!!!top_grouping, !!label) %>%
            mutate(!!label := !!last(group)) %>%
            distinct()

          # next all of the other variables (as missing)
          new_row <- lone_dat %>%
            select(
              -tidyselect::any_of(
                names(new_row)
              )
            ) %>%
            slice(0) %>%
            add_row()  %>%
            mutate(
              across(
                #convert NULL to NA in list-cols
                tidyselect::where(is.list),
                ~map(.x, ~if (is.null(.)) NA_character_ else .)
              )
            )  %>%
            bind_cols(new_row, .)%>%
            mutate(..tfrmt_row_grp_lbl = TRUE)

        } else {
          new_row <- tibble()
        }

        lone_dat_summ  %>%
          # only indent if not a summary row
          mutate(!!label := ifelse(.data$..tfrmt_summary_row==TRUE,
                                   !!label,
                                   str_c(indent, !!label))) %>%
          select(-"..tfrmt_summary_row") %>%
          bind_rows(new_row, .)
      })
    group = group[-length(group)]
    top_grouping = top_grouping[-length(top_grouping)]
  }

  .data%>%
    mutate(
      across(
        tidyselect::any_of(
          orig_group_names
        ),
        ~as.character(.x)
      )
    )


}

#' Remove row groups based on element_row_grp_loc and grouping
#'
#' @param .data data
#' @param element_row_grp_loc element object specifying row group label location
#' @param group symbolic list of grouping
#' @param label symbolic label column
#'
#' @noRd
remove_grp_cols <- function(.data, element_row_grp_loc, group, label = NULL){

  # check which group/label columns are available
  grps_avail <- eval_tidyselect_on_colvec(group, names(.data))

  if(length(grps_avail)==0 || element_row_grp_loc$location %in% c("gtdefault", "column")){
    add_ln_df <- .data
  } else{

    group <- as_vars(grps_avail)

    # Either drop group columns ("no print"), or format them w/ label
    if (element_row_grp_loc$location=="noprint"){

      add_ln_df <- .data %>% select(-c(!!!group))

    } else if(element_row_grp_loc$location == "indented"){
      add_ln_df <- .data %>%
        select(-c(!!!group))
    } else if(length(group) == 1){ #Using the grouping in gt + a single grouping
      add_ln_df <- .data %>%
        group_by(!!group[[1]])
    } else { # Using the grouping in gt, but needs to drop all groups in label
      add_ln_df <- .data %>%
        select(-c(!!!group[-1])) %>%
        group_by(!!group[[1]])
    }
  }
  add_ln_df
}



#' Remove trailing post-space rows and helper column
#' @param .data processed wide tbl
#' @noRd
apply_post_space_trim <- function(.data) {

  target_col <- "..tfrmt_post_space_row"

  if (target_col %in% names(.data)) {
    # If the very last row was tagged as a spacer, drop it
    if (isTRUE(last(.data[[target_col]]))) {
      .data <- .data %>%
        dplyr::slice(-dplyr::n())
    }
    # Always drop the helper column before returning
    .data <- .data %>%
      dplyr::select(-dplyr::all_of(target_col))
  }

  .data
}

#' Check if an object is a card
#'
#' Validates that the input is a valid `card` object. If the check fails,
#' it throws an informative error message using [rlang::stop_input_type()].
#'
#' @param card An object to check.
#' @param arg Character string specifying the argument name to display in error messages.
#' @param call The execution environment for error backtraces.
#' @param allow_null Logical indicating whether `NULL` should be accepted as a valid input.
#'
#' @return Invisible `NULL` if validation passes, otherwise throws an error.
#' @noRd
check_card <- function(
    card,
    arg = rlang::caller_arg(card),
    call = rlang::caller_env(),
    allow_null = FALSE
) {
  if (!missing(card)) {
    if (is_card(card)) {
      return(invisible(NULL))
    }

    if (allow_null && is.null(card)) {
      return(invisible(NULL))
    }
  }

  rlang::stop_input_type(
    card,
    "a card object",
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

#' Test if an object is a card
#'
#' Checks whether an object inherits from the `"card"` class.
#'
#' @param x An object to test.
#'
#' @return Logical `TRUE` if the object is a card, `FALSE` otherwise.
#' @noRd
is_card <- function(x) {
  inherits(x, "card")
}

#' Check if an object inherits from 'bind_ard'
#'
#' @param x An object to check.
#' @return `TRUE` if the object inherits from 'bind_ard', `FALSE` otherwise.
#' @noRd
is_bind_ard_card <- function(x) {
  inherits(x, "bind_ard")
}

#' Extract an argument from card attributes
#'
#' @param x a card object (data frame)
#' @param arg character string of the argument to extract. Defaults to `"by"`.
#'
#' @returns the value of the argument, or `NULL` if not found
#' @noRd
get_card_attr_arg <- function(x, arg = "by") {
  attr(x, "args")[[arg]]
}

#' Set a specific parameter inside an object's 'args' attribute
#'
#' @param x An object.
#' @param name A string naming the element to modify inside the 'args' attribute list.
#' @param value The value to assign to that element (e.g., NULL to drop it).
#' @return The modified object.
#' @noRd
set_card_args <- function(x, name, value) {
  attr(x, "args")[[name]] <- value
  x
}

#' Drop stale attributes from a combined ARD object
#'
#' @param x An object to check and modify.
#' @return The modified object with stale attributes removed if it inherits
#'   from 'bind_ard' and contains an 'args' attribute.
#' @noRd
drop_bind_ard_args <- function(x) {
  if (is_bind_ard_card(x) && !is.null(attr(x, "args"))) {
    x <- set_card_args(x, "by", NULL)
    x <- set_card_args(x, "variable", NULL)
    x <- set_card_args(x, "strata", NULL)
  }

  x
}

test_that("is_card works correctly", {
  # Setup mock objects
  good_card <- structure(list(), class = "card")
  bad_card <- list()

  expect_true(is_card(good_card))
  expect_false(is_card(bad_card))
})

test_that("is_bind_ard_card works correctly", {
  good_bind <- structure(list(), class = "bind_ard")
  bad_bind <- structure(list(), class = "card")

  expect_true(is_bind_ard_card(good_bind))
  expect_false(is_bind_ard_card(bad_bind))
})

test_that("check_card passes valid inputs and handles allow_null", {
  valid_card <- structure(list(), class = "card")

  # Valid card should pass invisibly
  expect_invisible(check_card(valid_card))
  expect_null(check_card(valid_card))

  # NULL should fail by default
  expect_error(check_card(NULL), regexp = "`NULL` must be a card object, not `NULL`")

  # NULL should pass when allow_null = TRUE
  expect_invisible(check_card(NULL, allow_null = TRUE))
  expect_null(check_card(NULL, allow_null = TRUE))
})

test_that("check_card throws correct rlang errors for invalid inputs", {
  invalid_input <- "not a card"

  # Test standard error throwing and message formatting
  expect_error(
    check_card(invalid_input, arg = "my_arg"),
    regexp = "`my_arg` must be a card object"
  )

  # Missing argument should also fall through to stop_input_type
  expect_error(
    check_card(),
    regexp ='argument "expr" is missing'
  )
})

test_that("get_card_attr_arg extracts attributes properly", {
  mock_card <- structure(
    data.frame(),
    class = "card",
    args = list(by = "group_var", strata = "age_strata")
  )

  # Default "by" argument
  expect_equal(get_card_attr_arg(mock_card), "group_var")

  # Custom argument extraction
  expect_equal(get_card_attr_arg(mock_card, arg = "strata"), "age_strata")

  # Non-existent argument returns NULL
  expect_null(get_card_attr_arg(mock_card, arg = "missing_arg"))
})

test_that("set_card_args modifies parameters properly", {
  mock_card <- structure(data.frame(), class = "card", args = list(by = "old"))

  # Modify an existing attribute element
  modified_card <- set_card_args(mock_card, "by", "new")
  expect_equal(attr(modified_card, "args")$by, "new")

  # Add a new attribute element
  modified_card <- set_card_args(modified_card, "variable", "var_name")
  expect_equal(attr(modified_card, "args")$variable, "var_name")
})

test_that("drop_bind_ard_args clears stale args on bind_ard objects", {
  # Case 1: Object is a bind_ard and has args attribute -> should drop targeted keys
  mock_bind_ard <- structure(
    data.frame(),
    class = "bind_ard",
    args = list(by = "sex", variable = "bmi", strata = "site", leave_me = "keep")
  )

  cleaned <- drop_bind_ard_args(mock_bind_ard)
  expect_null(attr(cleaned, "args")$by)
  expect_null(attr(cleaned, "args")$variable)
  expect_null(attr(cleaned, "args")$strata)
  expect_equal(attr(cleaned, "args")$leave_me, "keep")

  # Case 2: Object is a bind_ard but has NO args attribute -> should do nothing without crashing
  mock_no_args <- structure(data.frame(), class = "bind_ard")
  expect_equal(drop_bind_ard_args(mock_no_args), mock_no_args)

  # Case 3: Object is NOT a bind_ard -> should leave args untouched
  mock_regular_card <- structure(
    data.frame(),
    class = "card",
    args = list(by = "sex", variable = "bmi")
  )
  expect_equal(drop_bind_ard_args(mock_regular_card), mock_regular_card)
})

#' check in tfrmt that the column and col_plan are compatable
#' @noRd
#' @param x tfrmt to be checked
#' @importFrom rlang caller_call abort

#'
check_column_and_col_plan <- function(x){

  multi_column_defined <- length(x$column) > 1
  if(!is.null(x$col_plan)){
    span_strucs_idx <- sapply(x$col_plan$dots, is_span_structure)
    span_structures_defined <- any(span_strucs_idx)
    span_structs <- x$col_plan$dots[span_strucs_idx]
  }else{
    span_structures_defined <-  FALSE
    span_structs <- NULL
  }

  if(!multi_column_defined & span_structures_defined){

    if(length(x$column) == 1){
      n_col_desc <- "A single column defined in `column` argument of tfrmt "
    }else{
      n_col_desc <- "No columns defined in the `column` argument of tfrmt "
    }

    abort(
      paste0(
        n_col_desc,
        "but provided a span_structure() in `col_plan`.\n",
        "span_structure()'s are only valid when multiple `column` values are provided.\n",
        "Add values to `column` or remove span_structures from `col_plan()`"
      ),
      class = "_tfrmt_invalid_col_plan"
    )
  }

  if(span_structures_defined){
    column_strings <- map_chr(x$column, as_label)

    for(struct in span_structs){
      if(!all(vals <- names(struct) %in% column_strings)){
        abort(
          "Columns defined in `span_structure` are not defined columns in the tfrmt",
          body = paste0(
            "Column Values: ", paste0("`",column_strings,"`", collapse = ", " ),
            "\n",
            "Invalid Column Names in Span Structure: ",  paste0("`",names(struct)[!vals],"`", collapse = ", " )
          ),
          class = "_tfrmt_invalid_span_structure_col"
        )
      }
    }
  }

}

#' check that in tfrmt that the group var and body_plan is consistent
#' @noRd
#' @param x tfrmt to be checked
#'
#' @importFrom rlang caller_call is_empty as_label abort
#' @importFrom purrr map_chr
#'
check_group_var_consistency <- function(x){
  if(!is_empty(x$group)){

    group_var_consistency_message <- trimws(
      paste0(c(
        check_group_var_consistency_body_plan(x),
        check_group_var_consistency_row_grp_plan(x),
        check_group_var_consistency_footnote_plan(x)
      ),
      collapse = "\n"
      )
    )

    if(!identical(group_var_consistency_message,"")){
      abort(
        group_var_consistency_message ,
        class = "_tfrmt_mismatched_group_vals"
      )
    }

  }

}

check_group_var_consistency_body_plan <- function(x){
  if(!is.null(x$body_plan)){

    is_invalid_body_plan <- FALSE
    is_invalid_body_plan_message <- "Inconsistencies between group and body_plan"
    group_as_char <- map_chr(x$group, as_label)

    for(idx in seq_along(x$body_plan)){
      struct <- x$body_plan[[idx]]
      ## if the group_vars is a list, check that the names are matching group vars
      if(is.list(struct$group_val)){

        struct_groups <- names(struct$group_val)
        invalid_group_idx <- !struct_groups %in% group_as_char

        if(any(invalid_group_idx)){
          is_invalid_body_plan <- TRUE
          invalid_groups <- struct_groups[invalid_group_idx]
          is_invalid_body_plan_message <- c(
            is_invalid_body_plan_message,
            paste0(
              paste0("Invalid Format Structure in body_plan at position `",idx,"`:\n"),
              paste0("  Malformed Group: ",paste0(invalid_groups, collapse = ", "),"\n")
            )
          )
        }
      }
    }


    if(is_invalid_body_plan){
      is_invalid_body_plan_message
    }
  }
}

check_group_var_consistency_row_grp_plan <- function(x){

  if(!is.null(x$row_grp_plan)){

    is_invalid_row_grp_plan <- FALSE
    is_invalid_row_grp_plan_message <- "Inconsistencies between group and row_grp_plan row_grp_structures"
    group_as_char <- map_chr(x$group, as_label)

    for(idx in seq_along(x$row_grp_plan$struct_list)){
      struct <- x$row_grp_plan$struct_list[[idx]]
      ## if the group_vars is a list, check that the names are matching group vars
      if(is.list(struct$group_val)){

        struct_groups <- names(struct$group_val)
        invalid_group_idx <- !struct_groups %in% group_as_char

        if(any(invalid_group_idx)){
          is_invalid_row_grp_plan <- TRUE
          invalid_groups <- struct_groups[invalid_group_idx]
          is_invalid_row_grp_plan_message <- c(
            is_invalid_row_grp_plan_message,
            paste0(
              paste0("Invalid Format Structure in row_grp_plan for row_grp_structure `",idx,"`:\n"),
              paste0("  Malformed Group: ",paste0(invalid_groups, collapse = ", "),"\n")
            )
          )
        }
      }
    }


    if(is_invalid_row_grp_plan){
        is_invalid_row_grp_plan_message
    }
  }
}

check_group_var_consistency_footnote_plan <- function(x){

  if(!is.null(x$footnote_plan)){

    is_invalid_footnote_plan <- FALSE
    is_invalid_footnote_plan_message <- "Inconsistencies between group and footnote_plan footnote_structures"
    group_as_char <- map_chr(x$group, as_label)

    for(idx in seq_along(x$footnote_plan$struct_list)){
      struct <- x$footnote_plan$struct_list[[idx]]
      ## if the group_vars is a list, check that the names are matching group vars
      if(is.list(struct$group_val)){

        struct_groups <- names(struct$group_val)
        invalid_group_idx <- !struct_groups %in% group_as_char

        if(any(invalid_group_idx)){
          is_invalid_footnote_plan <- TRUE
          invalid_groups <- struct_groups[invalid_group_idx]
          is_invalid_footnote_plan_message <- c(
            is_invalid_footnote_plan_message,
            paste0(
              paste0("Invalid Format Structure in footnote_plan for footnote_structure `",idx,"`:\n"),
              paste0("  Malformed Group: ",paste0(invalid_groups, collapse = ", "),"\n")
            )
          )
        }
      }
    }


    if(is_invalid_footnote_plan){
      is_invalid_footnote_plan_message
    }
  }
}

#' check that in tfrmt that only the top level group gets styling if
#' row_grp_plan has and body_plan is consistent
#' @noRd
#' @param x tfrmt to be checked
#'
#' @importFrom rlang caller_call is_empty as_label abort
#' @importFrom purrr map_chr
#'
check_col_style_row_grp_consistency <- function(x){

  if(!is_empty(x$group) & !is.null(x$col_style_plan) & !is_empty(x$row_grp_plan)){


    is_invalid_plan <- FALSE
    is_invalid_plan_message <- "Invalid Entries based on col_align_plan and row_grp_plan"
    group_as_char <- map_chr(x$group, as_label)

    r_grp_plan_col_loc <- x$row_grp_plan$label_loc$location

    col_align_plan_as_char <- x$col_style_plan %>%
      lapply(function(x) x$cols %>% map_chr(as_label))
    for(cap_vars_idx in seq_along(col_align_plan_as_char)){
      grp_in_cap <- group_as_char %in% col_align_plan_as_char[[cap_vars_idx]]
      if(length(x$col_style_plan[[cap_vars_idx]]$cols) == 0){
        stop("Column element is missing from col_style_structure. Note: col here refers to the values within the column variable in your data, rather than the variable name itself")
      }

      if(any(grp_in_cap[-1]) && ! r_grp_plan_col_loc == "column" ){
        is_invalid_plan <- TRUE
        invalid_groups <- group_as_char[grp_in_cap]
        is_invalid_plan_message <- c(
          is_invalid_plan_message,
          paste0(
            paste0("Invalid col_style_structure in row_grp_plan at position `",cap_vars_idx,"`:\n"),
            paste0("  `col` value",ifelse(length(invalid_groups) > 1,"s",""),": ",paste0(invalid_groups, collapse = ", "),"\n")
          )
        )
      }
    }

    if(is_invalid_plan){
      abort(
        is_invalid_plan_message ,
        class = "_tfrmt_invalid_row_grp_col_style_plan"
      )
    }

  }
}





check_footnote_plan <- function(x){




  if(!is_empty(x$footnote_plan)){
  for(i in 1:length(x$footnote_plan$struct_list)){
    # if multiple columns then column_val must be a named list
    if(length(x$column)>1 && is.list(x$footnote_plan$struct_list[[i]]$column_val)==FALSE && is.null(x$footnote_plan$struct_list[[i]]$column_val)==FALSE){
      stop("when tfrmt contains multiple columns, column_val must be a named list")
    }

    # if multiple groups then group_val must be a named list
    if(length(x$group)>1 && is.list(x$footnote_plan$struct_list[[i]]$group_val)==FALSE && is.null(x$footnote_plan$struct_list[[i]]$group_val)==FALSE){
      stop("when tfrmt contains multiple groups, group_val must be a named list")
    }
  }
    }


}


#' Check if the plan parameter is supplied the correct plan function in tfrmt object
#' @noRd
#' @param tfrmt_object tfrmt object to be checked
#' @param plan name of plan, e.g., "col_style_plan"
#' @param parent_env parent environment
#' @importFrom rlang abort
#'
#'
check_plan <- function(tfrmt_object, plan, parent_env = caller_env()) {
  # extract the plan element from the tfrmt_object
  plan_element <- tfrmt_object[[plan]]

  # check if the user supplied a value to the plan parameter
  if (!is.null(plan_element)) {
    # check if the plan element inherits the plan attribute
    if (!inherits(plan_element, plan)) {
      # display error message
      abort(paste0("Invalid input supplied to the `", plan, "` parameter. Please supply a `", plan, "()`."), call = parent_env)
    }
  }
}


check_big_n <- function(tfrmt_object, parent_env = caller_env()) {
  # extract the big_n element from the tfrmt_object
  big_n_element <- tfrmt_object[["big_n"]]

  # check if the user supplied a value to the big_n parameter
  if (!is.null(big_n_element)) {
    # check if the big_n element inherits the big_n_structure
    if (!inherits(big_n_element, "big_n_structure")) {
      # display error message
      abort(paste0("Invalid input supplied to the `big_n` parameter. Please supply a `big_n_structure()`."), call = parent_env)
    }
  }
}

#' Element Row Group Location
#'
#'
#' @param location Location of the row group labels. Specifying 'indented'
#'  combines all group and label variables into a single column with each
#'  sub-group indented under its parent. 'spanning' and 'column' retain the
#'  highest level group variable in its own column and combine all remaining
#'  group and label variables into a single column with sub-groups indented. The
#'  highest level group column will either be printed as a spanning header or in
#'  its own column in the gt. The 'noprint' option allows the user to suppress
#'  group values from being printed. Finally, the 'gtdefault' option allows
#'  users to use the 'gt' defaults for styling multiple group columns.
#' @param indent A string of the number of spaces you want to indent
#'
#' @seealso [row_grp_plan()] for more details on how to group row group
#'  structures, [row_grp_structure()] for more details on how to specify row
#'  group structures, [element_block()] for more details on how to specify
#'  spacing between each group.
#'
#'  \href{https://gsk-biostatistics.github.io/tfrmt/articles/row_grp_plan.html}{Link
#'  to related article}
#'
#' @returns element_row_grp_loc object
#' @export
#' @examples
#'
#' tfrmt_spec <- tfrmt(
#'   group = c(grp1, grp2),
#'   label = label,
#'   param = param,
#'   value = value,
#'   column = column,
#'   row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc(location = "noprint")),
#'   body_plan = body_plan(
#'     frmt_structure(group_val = ".default", label_val = ".default", frmt("xx"))
#'   )
#' )
#'
#' @section Images: Here are some example outputs:
#'
#'  \if{html}{\out{ `r "<img src=\"https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/main/images/tfrmt-row_group_plan-cropped.jpg\" alt=\"Examples showing the difference between the row group locations\" style=\"width:100\\%;\">"` }}
#'
element_row_grp_loc <- function(location = c("indented", "spanning", "column", "noprint", "gtdefault"),
                                indent = "  "){
  location = match.arg(location)
  structure(
    list(location = location, indent = indent),
    class = c("element_row_grp_loc", "element")
  )
}

is_element_row_grp_loc <- function(x){
  inherits(x, "element_row_grp_loc")
}


#' Element block
#'
#' @param post_space Values to show in a new line created after the group block
#' @param fill Whether to recycle the value of `post_space` to match width of the data. Defaults to `TRUE`
#'
#' @return element block object
#'
#' @seealso [row_grp_plan()] for more details on how to group row group
#'   structures, [row_grp_structure()] for more details on how to specify row group
#'   structures, [element_row_grp_loc()] for more details on how to
#'   specify whether row group titles span the entire table or collapse.
#'
#' @export
#' @examples
#'
#' tfrmt_spec <- tfrmt(
#'   group = grp1,
#'   label = label,
#'   param = param,
#'   value = value,
#'   column = column,
#'   row_grp_plan = row_grp_plan(
#'     row_grp_structure(group_val = ".default", element_block(post_space = "   "))
#'   ),
#'   body_plan = body_plan(
#'     frmt_structure(group_val = ".default", label_val = ".default", frmt("xx"))
#'   )
#' )
element_block <- function(post_space = c(NULL, " ", "-"),
                          fill = TRUE){
  structure(
    list(post_space = post_space, fill = fill),
    class = c("element_block", "element")
  )

}

is_element_block <- function(x){
  inherits(x, "element_block")
}



element_stub <- function(collapse_ord = vars(), collapse_into = vars(), remove_dups = NULL){
  structure(
    list(collapse_ord = collapse_ord, collapse_into = collapse_into, remove_dups = remove_dups),
    class = c("element_stub", "element")
  )
}

#col_labels = element_label(newCol = "Hello World", wrap_txt = 30)
element_label <- function(..., wrap_txt = 30){
  structure(
    list(...) %>%
      c(wrap_txt = wrap_txt),
    class = c("element_label", "element")
  )
}

is_element_label <- function(x){
  inherits(x, "element_label")
}

# Utilities ---------------------------------------------------------------

expr_to_filter <- function(cols, val){
  UseMethod("expr_to_filter", cols)
}

#' @importFrom stringr str_detect str_sub
#' @importFrom rlang as_label
expr_to_filter.quosure <- function(cols, val){
  ## If is missing a quosure, nothing to filter
  if(quo_is_missing(cols)){
    return("TRUE")
  }

  # This is all so it works when there is a list
  if(all(val == ".default")){
    out <- "TRUE"
  } else {
    val <- ifelse(str_detect(val, "^`.*`$"), str_sub(val, 2, -2), val)
    out <- as_label(cols) %>%
      paste0("`", ., "`") %>%
      paste0(" %in% c(",
             paste0(shQuote(val, type = "cmd"), collapse = ", "),
             ")")
  }
  out
}



#' @importFrom purrr map2_chr map_chr
expr_to_filter.quosures <- function(cols, val){

  if (is.null(val)){
    out <- "TRUE"
  } else if(!is.list(val) & length(cols) == 1){
    cols <- cols[[1]]
    out <- expr_to_filter(cols,val)
  } else if(!is.list(val) && all(val == ".default")){
    out <- "TRUE"
  }else if(!is.list(val)){
    stop("If multiple cols are provided, val must be a named list")
  }else{
    if(!all(names(val) %in% map_chr(cols, as_label))){
      stop("Names of val entries do not all match col values")
    }
    out <- map2_chr(cols, val[map_chr(cols, as_label)], ~ expr_to_filter(.x, .y)) %>%
      paste0(collapse = " & ")
  }
  out
}

#' Given a *_structure with specific group/label value(s) (i.e. non-default),
#' return the row indices corresponding to each unique value
#'
#' @param cur_struct current structure object
#' @param .data data to test against
#' @param group list of the group parameters
#' @param label label symbol should only be one
#'
#' @return list of row indices
#' @noRd
#'
#' @importFrom dplyr filter pull select mutate group_by group_split
#' @importFrom rlang parse_expr
#' @importFrom purrr map_lgl map
struct_val_idx <- function(cur_struct, .data, group, label){

  grp_expr <- "TRUE"
  lbl_expr <- "TRUE"
  keep_vars <- NULL

  # only do this if cur_struct contains a non-default value
  if (detect_non_default(cur_struct$group_val)){
    grp_expr <- expr_to_filter(group, cur_struct$group_val)

    if (!is.list(cur_struct$group_val)){
      keep_vars <- group
    } else {
      keep_vars <- group[map_lgl(cur_struct$group_val, ~!all(.x==".default"))]
    }
  }

  if (detect_non_default(cur_struct$label_val)){
    lbl_expr <- expr_to_filter(label, cur_struct$label_val)
    keep_vars <- c(keep_vars, label)
  }

  if (!is.null(keep_vars)){
    filter_expr <- paste(
      c(lbl_expr,grp_expr),
      collapse = "&"
    ) %>%
      parse_expr()

    .data %>%
      filter(!!filter_expr) %>%
      select(
        tidyselect::any_of(
          c(
            map_chr(keep_vars, as_label),
            "TEMP_row"
          )
        )
      ) %>%
      # split only after non-consecutive sequence
      mutate(breaks = .data$TEMP_row==lag(.data$TEMP_row, default = 0)+1,
             breaks = cumsum(!.data$breaks)) %>%
      group_by(.data$breaks) %>%
      group_split() %>%
      map(function(x) pull(x, .data$TEMP_row))

  } else {
    .data %>%
      pull(.data$TEMP_row) %>%
      list()
  }

}

# detect use of .default in a *_structure object
#' @importFrom purrr map_lgl
#' @noRd
detect_default <- function(struct){

  map_lgl(struct, ~ any(!is.null(.x) && any(.x==".default"))) %>% any()
}

# detect use of non-default in a  *_structure object entry
detect_non_default <- function(struct_val){

  any(!is.null(struct_val) && any(!struct_val==".default"))

}

#' Create the group_by expression for the data
#'
#' @param cur_struct current structure object
#' @param group list of the group parameters
#' @param label label symbol should only be one
#'
#' @return character vector of variable names to group by
#' @noRd
#'
#' @importFrom rlang as_label
#' @importFrom purrr map_lgl map_chr
expr_to_grouping <- function(cur_struct, group, label){

  grouping <- NULL

  if (!is.null(cur_struct$group_val)){
    if(!is.list(cur_struct$group_val) && all(cur_struct$group_val==".default")){
      grp_to_add <- map_chr(group, as_label)
      grouping <- c(grouping, grp_to_add)
    } else if (is.list(cur_struct$group_val) && any(cur_struct$group_val==".default")){
      grp_to_add <- names(cur_struct$group_val)[map_lgl(cur_struct$group_val, ~all(.x==".default"))]
      grouping <- c(grouping, grp_to_add)
    }
  }
  if (!is.null(cur_struct$label_val) && cur_struct$label_val==".default"){
    grouping <- c(grouping, as_label(label))
  }

  grouping %>% unname()
}

test_that("expr_to_filter - quosure", {

  var <- "value1"
  default <- ".default"
  quo_val <- quo(col)

  filter_var <- expr_to_filter.quosure(cols = quo_val, val = var)
  filter_default <- expr_to_filter.quosure(cols = quo_val, val = default)

  expect_equal(filter_var, "`col` %in% c(\"value1\")")
  expect_equal(filter_default, "TRUE")

})


test_that("expr_to_filter - quosures", {

  var <- "value1"
  default <- ".default"
  var_list_named <- list(col1 = "value1", col2 = "value2")
  var_list_default <- list(col1 = ".default", col2 = "value2")
  var_list_misnamed <- list(col1 = ".default", col12 = "value2")
  var_list_missing <- list(col2 = "value2")
  quos_val_1 <- vars(col1)
  quos_val_2 <- vars(col1,col2)

  ## length 1 vars = length one var
  filter_var <- expr_to_filter.quosures(cols = quos_val_1, val = var)
  filter_default <- expr_to_filter.quosures(cols = quos_val_1, val = default)

  expect_equal(filter_var, "`col1` %in% c(\"value1\")")
  expect_equal(filter_default, "TRUE")

  ## length 2 vars & length one var

  ### Must be a list
  expect_error(
    expr_to_filter.quosures(cols = quos_val_2, val = var),
    "If multiple cols are provided, val must be a named list",
    fixed = TRUE
  )

  ### .default is allowed
  expect_equal(
    expr_to_filter.quosures(cols = quos_val_2, val = default),
    "TRUE"
    )

  ## length 2 vars & length two var
  filter_var_list_named <- expr_to_filter.quosures(cols = quos_val_2, val = var_list_named)
  filter_var_list_default <- expr_to_filter.quosures(cols = quos_val_2, val = var_list_default)

  expect_equal(filter_var_list_named, "`col1` %in% c(\"value1\") & `col2` %in% c(\"value2\")")
  expect_equal(filter_var_list_default, "TRUE & `col2` %in% c(\"value2\")")

  ## length 2 vars & length two var incorrect names
  expect_error(
    expr_to_filter.quosures(cols = quos_val_2, val = var_list_misnamed),
    "Names of val entries do not all match col values",
    fixed = TRUE
  )



})

test_that("expr_to_filter - quosure - with quotes", {

  quo_val <- quo(col)

  var <- "value1's"
  filter_var <- expr_to_filter.quosure(cols = quo_val, val = var)
  expect_equal(filter_var, "`col` %in% c(\"value1's\")")

  var <- 'value1\'s'
  filter_var <- expr_to_filter.quosure(cols = quo_val, val = var)
  expect_equal(filter_var, "`col` %in% c(\"value1's\")")

  var <- '"a value with quotes"'
  filter_var <- expr_to_filter.quosure(cols = quo_val, val = var)
  expect_equal(filter_var, "`col` %in% c(\"\\\"a value with quotes\\\"\")")

  var <- "\"a value with quotes\""
  filter_var <- expr_to_filter.quosure(cols = quo_val, val = var)
  expect_equal(filter_var, "`col` %in% c(\"\\\"a value with quotes\\\"\")")
})

#' Apply column style plan - alignment - non-value cols
#'
#' @param .data data
#' @param tfrmt_obj tfrmt object
#' @param col_plan_vars the planned renaming of columns
#' @importFrom dplyr mutate across select tibble group_by slice n filter cur_column pull ungroup
#' @importFrom tidyr unnest
#' @importFrom purrr map map_dfr discard
#' @importFrom rlang as_name
#' @importFrom tibble as_tibble_row
#' @importFrom forcats fct_inorder
#'
#' @noRd
apply_col_style_plan <- function(.data, tfrmt_obj, col_plan_vars = as_vars.character(colnames(.data))){

  style_plan <- tfrmt_obj$col_style_plan

  if(is.null(style_plan) || length(style_plan) == 0){
    return(.data)
  }

  if(is_empty(tfrmt_obj$column)){
    # create placeholder
    column_names <- "col"
  }else{
    column_names <- map_chr(tfrmt_obj$column, as_label)
  }

  total_col_style_selection <- list()

  for(el_idx in seq_along(style_plan)){

    style_el <- style_plan[[el_idx]]

    col_selections <- list()

    for( sel_id in seq_along(style_el$cols)){

      selection <- style_el$cols[sel_id]

      col_selection <- col_style_selections(selection, column_names, col_plan_vars)

      col_selection <- setdiff(col_selection, "..tfrmt_row_grp_lbl")

      col_selections <- c(col_selections, col_selection)
    }

    if(length(col_selections) > 0){
      col_style_selection <- style_el[setdiff(names(style_el),"cols")] %>%
        map(list) %>%
        as_tibble() %>%
        bind_cols(
          tibble( col = unlist(col_selections) )
        )
      total_col_style_selection <- c(
        total_col_style_selection,
        list(col_style_selection)
      )
    }
  }

  if(length(total_col_style_selection) > 0){

    total_col_styles <- bind_rows(total_col_style_selection) %>%
      group_by(col) %>%
      slice(n()) %>%
      ungroup()

    for(col_style_idx in seq_len(nrow(total_col_styles))){

      col_style_to_apply <- total_col_styles %>%
        slice(col_style_idx) %>%
        as.list()

      col_to_modify <- col_style_to_apply$col %>% char_as_quo()

      .data <-  .data %>%
        mutate(!!col_to_modify := apply_style_to_col(!!col_to_modify, col_style_to_apply[setdiff(names(col_style_to_apply),"col")]))
    }

    force(.data)
  }

  force(.data)

  return(.data)

}

# function to get all columns the col_style_structure applies to
col_style_selections <- function(selection, column_names, col_plan_vars){
  if(!is_span_structure(selection[[1]])){
    col_selection <- col_plan_quo_to_vars(
      x = selection,
      column_names = column_names,
      data_names = c(),
      preselected_cols = col_plan_vars %>% map_chr(as_label),
      return_only_selected = TRUE,
      default_everything_behavior = TRUE
    )
  }else{
    col_selection <- col_plan_span_structure_to_vars(
      x = selection,
      column_names = column_names,
      data_names = c(),
      preselected_cols = col_plan_vars %>% map_chr(as_label),
      return_only_selected = TRUE
    )
  }

  col_selection <- col_selection[!grepl("^-",col_selection)]

  ## use names if they exist, else use content
  if(!is.null(names(col_selection))){
    col_sel_names <- names(col_selection)
    if(any(col_sel_names == "")){
      col_sel_names[col_sel_names == ""] <- col_selection[col_sel_names == ""]
    }
    col_selection <- col_sel_names
  }

  col_selection
}

#' Column level styling to vector defined in styling list
#'
#' @param x vector to be styled
#' @param styling_list list of styles to apply
#'
#' @noRd

apply_style_to_col <- function(x, styling_list){

  x %>%
    tentative_process(
      apply_col_alignment,
      styling_list$align[[1]],
      styling_list$type
    ) %>%
    tentative_process(
      apply_col_width,
      styling_list$width[[1]]
    )

}


#' Apply alignment to a column
#'
#' @param col Character vector of data values to align
#' @param align Alignment to be applied to column. Defaults to `left` alignment.
#' @param type Type of alignment: "char" or "pos", for character alignment (default), and positional-alignment, respectively. Positional alignment allows for aligning over multiple positions in the column.
#' @param column_name name alignment being applied to. Defaults to NULL
#'
#' @return Character vector containing aligned data values
#'
#' @noRd
apply_col_alignment <- function(col, align, type = "char"){

  col_na_idx <- which(is.na(col))
  if(length(col_na_idx) > 0){
    col[col_na_idx] <- ""
  }

  if (type=="char"){
    out <- apply_col_alignment_char(col, align)
  } else {
    out <- apply_col_alignment_pos(col, align)
  }

  if(length(col_na_idx) > 0){
    out[col_na_idx] <- NA
  }

  out

}

#' Apply alignment to a column - character alignment
#'
#' @param col Character vector of data values to align
#' @param align Alignment to be applied to column.
#'
#' @importFrom tidyr separate replace_na
#' @importFrom dplyr mutate across tibble bind_cols
#' @importFrom stringr str_dup str_c str_trim str_detect
#' @importFrom rlang warn
#'
#' @noRd
apply_col_alignment_char <- function(col, align){

  if (!all(align %in% c("left","right"))){
    align <- ifelse(str_detect(align, "[[:alnum:]]"), paste0("\"", align, "\""), paste0("\\", align))
    align <- paste(align, collapse = "|")
    align <- paste0("(?=[", align, "])")
    tbl_dat <-  tibble(col = trimws(col)) %>%
      separate(col, c("add_left","add_right"), sep = align, extra = "merge", fill = "right", remove = FALSE) %>%
      mutate(across(c("add_left", "add_right"), function(x) {
        replace_na(x, "") %>%
          nchar() %>%
          {max(.)-.} %>%
          {str_dup(" ", .)}
      }))
  } else {

    tbl_dat <-  tibble(col = str_trim(col, side = "right")) %>%
      mutate(string_col = nchar(.data$col),
             string_tot = max(.data$string_col),
             space_to_add = str_dup(" ", .data$string_tot-.data$string_col))
    if(align == "left"){
      tbl_dat <- tibble(add_left = "",
                        add_right = tbl_dat$space_to_add) %>%
        bind_cols(tbl_dat, .)
    } else {
      tbl_dat <- tibble(add_left = tbl_dat$space_to_add,
                        add_right = "") %>%
        bind_cols(tbl_dat, .)
    }

  }

  str_c(
    tbl_dat$add_left,
    tbl_dat$col,
    tbl_dat$add_right)
}

#' Apply alignment to a column - positional alignment
#'
#' @param col Character vector of data values to align
#' @param align Alignment to be applied to column.
#'
#' @importFrom dplyr lag left_join tibble mutate group_by summarise arrange row_number select filter
#' @importFrom tidyr separate pivot_longer replace_na
#' @importFrom stringr str_replace_all str_count str_dup str_extract str_detect str_c
#'
#' @noRd
apply_col_alignment_pos <- function(col, align){

  # merge the alignment vec in with the column
  col_with_align <- tibble(col = trimws(col),
                           col_as_x = str_replace_all(col, "[0-9]", "x")) %>% # convert column values to x's
    left_join(
      tibble(align = trimws(align),
             col_as_x = str_replace_all(align, "\\|", "")),
      by = "col_as_x"
    )

  if (any(is.na(col_with_align$align))){
    message("`align` input for `type`=\"pos\" in col_style_structure does not cover all possible values. Some cells may not be aligned.")
  }

  # if nothing to align on, just return the original vec
  if (all(is.na(col_with_align$align))){
    return(col)
  }

  # account for any with missing align strings
  col_with_align <- col_with_align %>%
    mutate(align = ifelse(is.na(.data$align), .data$col_as_x, .data$align)) %>%
    select(-"col_as_x") %>%
    mutate(n_split_levs = str_count(.data$align, "(?<!\\\\)[\\|]")+1)

  # get the maximum number of splits to make
  # ignore any |'s that have been escaped by user
  n_split_levs_max <- max(col_with_align$n_split_levs, na.rm = TRUE)

  # create splits on the align vec at the |'s and count the number of chars for each
  # this will be used to help us split the col by position
  col_with_pos <- col_with_align %>%
    mutate(col_idx= row_number()) %>%
    separate("align", into = paste0("col_split_",1:n_split_levs_max), sep = "(?<!\\\\)[\\|]", remove = FALSE, fill = "right") %>%
    pivot_longer(
      tidyselect::starts_with(
        "col_split_"
      ),
      names_to = "col_split_lev",
      values_to = "col_split_val"
    ) %>%
    arrange(.data$col_idx, .data$col_split_lev) %>%
    group_by(.data$col_idx) %>%
    mutate(col_split_end = nchar(.data$col_split_val) %>% cumsum(),
           col_split_start = case_when(
             is.na(.data$col_split_val) ~ NA ,
             TRUE ~ lag(.data$col_split_end, default = 0)+1),
           col_split_lev = gsub("col_split_", "", .data$col_split_lev) %>% as.numeric())

  # 1) split the original col using the positions identified above
  #     * except on the final one, do not split as this will be right-padded
  #
  # 2) further split into 2 pieces, so padding can be added consistently:
  #  - if the first substring, padding will be added to the left of entire string
  #  - if > first substring, split will occur on 1st identified space (if available).
  #       padding will be added to the left of 2nd piece
  col_with_splits <- col_with_pos %>%
    mutate(col_sub = substr(.data$col, .data$col_split_start, .data$col_split_end)) %>%
    mutate(col_sub_1 = case_when(
      .data$col_split_lev == 1 ~ NA_character_, # first substring so do not split - will go to  col_sub_2
      .data$col_split_lev == .data$n_split_levs ~ col_sub, # last substring so do not split - will go to col_sub_1
      !str_detect(.data$col_sub, " ") & ! .data$col_split_lev == 1 ~ col_sub, # no space found - cannot split or pad
      TRUE ~ str_extract(.data$col_sub, "^.+?(?= )")), # extract string prior to first space
      col_sub_2 = case_when(
        .data$col_split_lev == 1 ~ .data$col_sub,   # first substring so put the whole thing here
        .data$col_split_lev == .data$n_split_levs ~ NA_character_, # last substring, nothing to left-pad
        TRUE ~ str_extract(.data$col_sub, "(?= ).*")))  # extract string following & including first space

  # within each split level, find the # of chars it needs to take up, then left pad
  col_left_padded00 <- col_with_splits %>%
    group_by(.data$col_split_lev) %>%
    mutate(to_add_left = nchar(.data$col_sub) %>%
             {max(., na.rm=TRUE)-.} %>%
             {str_dup(" ", .)})

  # notify user if left padding was intended but no space found
  col_left_padded01 <- col_left_padded00 %>%
    mutate(no_space = ifelse((.data$col_split_lev >1 & .data$col_split_lev < .data$n_split_levs) & # not the first or final level
                               (is.na(.data$col_sub_2))& # unable to split on a space
                               (! .data$col_sub=="") &    # there is actually a value there
                               (nchar(.data$to_add_left)>0),# there is postive padding
                             TRUE, FALSE),
           to_add_left = ifelse(.data$no_space, "", .data$to_add_left),
           across(c("col_sub_1", "to_add_left", "col_sub_2"), ~replace_na(., "")),
           col_sub_out = str_c(.data$col_sub_1, .data$to_add_left, .data$col_sub_2))

  if (nrow(filter(col_left_padded01, .data$no_space))>0){
    message("Unable to complete positional alignment in col_style_structure due to lack of whitespace available formatted value")
  }

  # collapse back to 1 rec per formatted string
  # & pad the right hand side
  col_left_padded_sum <- col_left_padded01 %>%
    group_by(.data$col_idx) %>%
    summarise(col = paste0(.data$col_sub_out, collapse = "")) %>%
    mutate(to_add_right = .data$col %>%
             nchar() %>%
             {max(.)-.} %>%
             {str_dup(" ", .)})

  str_c(col_left_padded_sum$col,
        col_left_padded_sum$to_add_right)

}


#' Set column widths to a specified number of characters
#'
#' @param col Character vector of data values to align
#' @param width Width of column, as number of characters
#'
#' @return Character vector containing padded aligned data values
#'
#' @importFrom stringr str_dup str_pad
#' @importFrom purrr pmap_chr
#' @noRd
apply_col_width <- function(col, width){

  col_na_idx <- which(is.na(col))
  col_empty_strings_idx <- which(grepl("^\\s+$",col))

  if(length(col_na_idx) > 0){
    col[col_na_idx] <- ""
  }

  pad_left <- str_dup(" ", nchar(col) - nchar(trimws(col, "left")))
  pad_right <- str_dup(" ", nchar(col) - nchar(trimws(col, "right")))
  out <- pmap_chr(list(trimws(col), width, pad_left, pad_right), wrap_string)

  if(length(col_na_idx) > 0){
    out[col_na_idx] <- NA
  }
  if(length(col_empty_strings_idx) > 0){
    out[col_empty_strings_idx] <- str_pad(col[col_empty_strings_idx], min(c(nchar(col[col_empty_strings_idx]), width)))
  }

  out
}

#' @importFrom stringi stri_wrap
wrap_string <- function(x, width, pad_left, pad_right){
  word_list <- stri_wrap(x, width = width, normalize = FALSE)
  paste0(pad_left, word_list, pad_right,collapse = "\n")
}

#' @importFrom tidyr unite
#' @importFrom dplyr as_tibble relocate
#' @importFrom stringr str_remove str_detect
#' @importFrom purrr pmap_chr map2
#' @importFrom utils capture.output
#' @importFrom rlang quo
apply_col_plan <- function(data, col_selection, grp_lbl){

  if(is.character(col_selection)){
    quo_col_selections <- map(col_selection, ~char_as_quo(.x))
    col_selection <- do.call(vars, quo_col_selections)
  }

  #exclude group and label variables from renaming
  col_selection_grp_lbl <- col_selection %in% grp_lbl
  if (any(col_selection_grp_lbl)){
    names(col_selection)[col_selection_grp_lbl] <- ""
  }

  select(data, !!!col_selection)

}


#' create the stub header for table
#' @importFrom purrr map_chr
#' @noRd
create_stub_head <- function(col_plan_vars, group, label, row_grp_plan_label_loc){

  # all group/label vars
  grps <- map_chr(c(group, label), as_label)
  # all column labels
  col_plan_vars_chr <- map_chr(col_plan_vars, as_label)

  stub <- ""
  # subset the column labels to just group/label vars
  if (length(grps)>0 && length(col_plan_vars_chr)>0){

    # grab just the groups from col_plan_vars_chr
    col_plan_grps <- col_plan_vars_chr[which(col_plan_vars_chr %in% grps)]
    nms_grps <- col_plan_grps[match(grps, col_plan_grps, nomatch = 0)] %>%
      names()

    if (length(nms_grps)>0){
      stub <- nms_grps

      # only row_grp_plan "column" option gets >1 stub label
      if (!row_grp_plan_label_loc=="column"){
        stub_no_empty <- setdiff(stub, "")
        stub <- c(stub_no_empty, "")[1]
      }

    }
  }
  stub
}

#' Creates a named vector explicitly calling all the columns
#'
#' @importFrom rlang is_empty
#' @importFrom purrr map map_chr
#'
#' @noRd
create_col_order <- function(data_names, columns, cp){

  if(is.null(cp)){
    col_selections <- data_names
  }else{
    if(is_empty(columns)){
      # create placeholder
      column_names <- "col"
    }else{
      column_names <- map_chr(columns, as_label)
    }

    col_selections <- c()

    for(cp_el_idx in seq_along(cp$dots)){
      cp_el <- cp$dots[cp_el_idx]
      if(!is_span_structure(cp_el[[1]])){
        col_selections <- col_plan_quo_to_vars(
          x = cp_el,
          column_names = column_names,
          data_names = data_names,
          preselected_cols = col_selections
        )
      }else{
        col_selections <- col_plan_span_structure_to_vars(
          x = cp_el,
          column_names = column_names,
          data_names = data_names,
          preselected_cols = col_selections
          )
      }

      col_selections <- col_selections[!duplicated(gsub("^-","",col_selections), fromLast = TRUE)]

    }

    ## add back in the non-specified columns
    if(cp$.drop == FALSE){
      missing_cols <- setdiff(data_names, gsub("^-","",col_selections))
      col_selections <- c(col_selections,missing_cols)
    }
  }

  quo_col_selections <- map(col_selections, ~char_as_quo(.x))

  do.call(vars, quo_col_selections)

}

col_plan_quo_to_vars <- function(x, column_names, data_names, preselected_cols,
                                 return_only_selected = FALSE, default_everything_behavior = FALSE){

  ## ensure data_names order matches preselected_cols
  split_data_names <- split_data_names_to_df(data_names, preselected_cols, column_names)

  col_id <- column_names[length(column_names)]
  col_quo <- quo(!!sym(col_id))
  col_name_quo <- quo(!!sym(paste0("__tfrmt_new_name__", col_id)))

  ## only apply tidyselect to _bottom_ column
  data_names_tmp <- split_data_names[[col_id]]

  selected <- eval_col_plan_quo(x[[1]], data_names_tmp, preselected_cols, default_everything_behavior = default_everything_behavior)

  ## if is subtraction, inverse selection to get subtracted columns and prepend with -
  if(grepl("^-",as_label(x[[1]]))){
    split_data_names <- split_data_names %>%
      filter(!(!!col_quo %in% selected)) %>%
      mutate(
        subtraction_status = TRUE
      )
  }else{
    split_data_names <- split_data_names %>%
      filter(!!col_quo %in% selected)

    if(!is.null(names(x)) && names(x) != ""){

      rename_val <- names(x)

      if(is_valid_tidyselect_call(quo_get_expr(x[[1]])) && length(selected) > 1){
        rename_val <- paste0(rename_val, seq_len(length(selected)))
      }

      rows_to_rename <- split_data_names[[as_label(col_quo)]] %in% selected
      split_data_names[rows_to_rename, as_label(col_name_quo)] <- rename_val

    }

  }

  unite_df_to_data_names(split_data_names, preselected_cols, column_names,
                         return_only_selected  = return_only_selected )
}

#' @importFrom rlang quo_get_expr quo
#' @importFrom tidyr separate unite
#' @importFrom dplyr filter pull mutate arrange left_join select
col_plan_span_structure_to_vars <- function(x, column_names, data_names, preselected_cols,
                                            return_only_selected = FALSE){

  split_data_names <- split_data_names_to_df(data_names, preselected_cols, column_names)

  col_selections <- list()

  ## evaluate selections to identify columns
  for(col_id_idx in seq_along(column_names)){

    col_id <- column_names[col_id_idx]
    col_quo <- quo(!!sym(col_id))
    col_name_quo <- quo(!!sym(paste0("__tfrmt_new_name__", col_id)))

    if(col_id %in% names(x[[1]])){

      selections <- x[[1]][[col_id]]
      split_data_selections <- list()

      for(sel_id_idx in seq_along(selections)){

        sel_id <- selections[[sel_id_idx]]
        sel_id_col_selections <- unique(eval_tidyselect_on_colvec(sel_id, split_data_names[[col_id]]))

        is_subtraction_selection <- grepl("^-",as_label(sel_id))

        if(!is_subtraction_selection){
          if(!is.null(names(selections)) && names(selections)[[sel_id_idx]] != ""){

            rename_val <- names(selections)[[sel_id_idx]]

            if(is_valid_tidyselect_call(quo_get_expr(sel_id)) && length(sel_id_col_selections) > 1){
              rename_val <- paste0(rename_val, seq_len(length(sel_id_col_selections)))
            }

            rows_to_rename <- split_data_names[[as_label(col_quo)]] %in% sel_id_col_selections
            split_data_names[rows_to_rename, as_label(col_name_quo)] <- rename_val

          }

          split_data_selections[[sel_id_idx]] <- split_data_names %>%
            filter(!!col_quo %in% sel_id_col_selections)

        }else{

          split_data_selections[[sel_id_idx]] <- split_data_names %>%
            mutate(
              subtraction_status = case_when(
                !(!!col_quo %in% sel_id_col_selections) ~ TRUE,
                TRUE ~ FALSE
              )
            )
        }
      }

      split_data_names <- bind_rows(split_data_selections) %>%
        unique()

      col_selections[[col_id]] <- split_data_names %>%
        pull(!!col_quo) %>%
        unique

    }else{

      split_data_names <- split_data_names %>%
        filter(!is.na(!!col_quo))

      col_selections[[col_id]] <- split_data_names %>%
        pull(!!col_quo) %>%
        unique
    }

  }

  ## create order df
  ords <- do.call("crossing",col_selections) %>%
    mutate(
      across(
        tidyselect::everything(),
        ~factor(
          .x,
          levels = col_selections[[cur_column()]]
        )
      )
    ) %>%
    arrange(
      across(
        tidyselect::everything()
      )
    ) %>%
    mutate(
      ord_col = seq_len(n())
    )

  split_data_names %>%
    left_join(ords, by = names(col_selections)) %>%
    arrange(.data$ord_col) %>%
    select(-"ord_col") %>%
    unite_df_to_data_names(preselected_cols, column_names,
                           return_only_selected = return_only_selected)

}

## given a string - x - see how to convert to a quosure.
##  if negative is TRUE, it will mark it as a `-`.
char_as_quo <- function(x) {

  is_negative <- grepl("^-", x)
  x <- gsub("^-", "", x) # Removes the leading '-'
  x <- gsub("^`|`$", "", x) # Removes leading/trailing backticks for col names with spaces in

  ## if x is a valid tidyselect call, leave it as is,
  ## otherwise wrap it in "`". This is so we can pass
  ## colnames with spaces (which are common in spanned columns)
  ## to quo. IE `spanned col header__tfrmt_delim__col1`
  x_text <- tryCatch({
    x_lang <- parse(text = x)[[1]]
    if (is_valid_tidyselect_call(x_lang)) {
      x
    } else{
      paste0("`", x, "`")
    }},
    error = function(e) {
      paste0("`", x, "`")
    }
  )

  if (is_negative) {
    expr_to_eval <- paste0("quo(-", x_text, ")")
  } else{
    expr_to_eval <- paste0("quo(", x_text, ")")
  }

  eval(parse(text = expr_to_eval)[[1]])
}


#' Evaluate a `col_plan` quosure
#'
#' @param x a (single) quosure to evaluate
#' @importFrom rlang quo_get_expr as_label is_empty
#'
#' @noRd
eval_col_plan_quo <- function(x, data_names, preselected_vals, default_everything_behavior = FALSE){

  if(identical(as_label(x), "everything()") & !default_everything_behavior){
    # dump any pre-selected columns from everything() call. we are _not_ using
    # the default behavior of everything().

    if(!is_empty(preselected_vals)){
      data_names <- data_names[-seq_along(preselected_vals)]
    }
  }

  eval_tidyselect_on_colvec(x, data_names)
}


#' Creates a split out data.frame of the potential columns and their spans.
#'
#' Combines the original column names with the potential renamed ones from
#' preselected_cols into a single vector, removing duplicates then splits out
#' the contents into a data.frame.
#'
#' @param data_names original names of the dataset
#' @param preselected_cols named character vector of selected columns from the
#'   dataset. names of the vector reflect the new name to apply
#' @param column_names the original ARD column names as a character vector
#'
#' @noRd
#'
#' @importFrom dplyr mutate
#' @importFrom tidyr separate
#' @importFrom tibble tibble
split_data_names_to_df <- function(data_names, preselected_cols, column_names){

  data_names <- c(preselected_cols, setdiff(data_names, preselected_cols))

  if(is.null(names(data_names))){
    names(data_names) <- data_names
  }else{
    if(any(is_preserved_name <- names(data_names) == "")){
      names(data_names)[is_preserved_name] <- data_names[is_preserved_name]
    }
  }

  tibble(
    original = unname(data_names),
    new_name = names(data_names)
   ) %>%
    mutate(
      subtraction_status = str_detect(.data$original ,"^-"),
      original = str_remove(.data$original,"^-")
    ) %>%
    separate(
      .data$original,
      into = column_names,
      sep = .tlang_delim,
      fill = "left"
    ) %>%
    separate(
      .data$new_name,
      into = paste0("__tfrmt_new_name__", column_names),
      sep = .tlang_delim,
      fill = "left"
    )

}

#' Combines the split out data.frame of the potential columns and their spans into a named vector.
#'
#'
#' @param split_data_names tibble containing two columns for every
#'   "column_name" value - one that is the column name, the second is the column
#'   name prepended with "__tfrmt_new_name__". Finally, there is a "subtraction_status" column,
#'   indicating if the column should be subtracted or not.
#' @param preselected_cols named character vector of selected columns from the
#'   dataset. names of the vector reflect the new name to apply
#' @param column_names the original ARD column names as a character vector
#'
#' @noRd
#'
#' @importFrom dplyr case_when mutate pull
#' @importFrom tidyr unite
#' @importFrom tibble tibble
unite_df_to_data_names <- function(split_data_names, preselected_cols, column_names,
                                   return_only_selected = FALSE){

  new_preselected_cols_full <- split_data_names %>%
    unite(
      "original",
      -c(
        tidyselect::starts_with(
          "__tfrmt_new_name__"
        ),
        "subtraction_status"
      ),
      sep = .tlang_delim
    ) %>%
    unite(
      "new_name",
      tidyselect::starts_with("__tfrmt_new_name__"),
      sep = .tlang_delim
    ) %>%
    mutate(
      across(
        c("original", "new_name"),
        ~remove_empty_layers(
          .x,
          length(column_names) - 1
        )
      )
    )

  selected <- new_preselected_cols_full %>%
    mutate(
      original = case_when(
        .data$subtraction_status == TRUE ~ paste0("-",.data$original),
        TRUE ~ .data$original
      )
    ) %>%
    pull(.data$original)

  new_names <- ifelse(new_preselected_cols_full$original != new_preselected_cols_full$new_name,
                      new_preselected_cols_full$new_name, "")

  names(selected) <- new_names

  if(return_only_selected){
    selected
  }else{
    c(preselected_cols,selected)
  }
}

test_that("create_stub_head() works", {
  # no labels
  expect_identical(
    create_stub_head(
      col_plan_vars = rlang::quos(-ord, grp2, lbl, `1`),
      group = rlang::quos(grp2),
      label = rlang::quos(lbl),
      row_grp_plan_label_loc = "indented"
    ),
    ""
  )

  expect_identical(
    create_stub_head(
      col_plan_vars = rlang::quos(-ord, grp2, lbl, `1`),
      group = rlang::quos(grp2),
      label = rlang::quos(lbl),
      row_grp_plan_label_loc = "column"
    ),
    c("", "")
  )

  # multiple labels

  expect_identical(
    create_stub_head(
      col_plan_vars = rlang::quos(
        -ord,
        Group2 = grp2,
        Group1 = grp1,
        Label = lbl,
        `1`
      ),
      group = rlang::quos(grp2, grp1),
      label = rlang::quos(lbl),
      row_grp_plan_label_loc = "indented"
    ),
    "Group2"
  )

  expect_identical(
    create_stub_head(
      col_plan_vars = rlang::quos(
        -ord,
        Group2 = grp2,
        Group1 = grp1,
        Label = lbl,
        `1`
      ),
      group = rlang::quos(grp2, grp1),
      label = rlang::quos(lbl),
      row_grp_plan_label_loc = "column"
    ),
    c("Group2", "Group1", "Label")
  )

  # mix of labeled/unlabeled

  expect_identical(
    create_stub_head(
      col_plan_vars = rlang::quos(-ord, grp2, Group1 = grp1, Label = lbl, `1`),
      group = rlang::quos(grp2, grp1),
      label = rlang::quos(lbl),
      row_grp_plan_label_loc = "indented"
    ),
    "Group1"
  )
  expect_identical(
    create_stub_head(
      col_plan_vars = rlang::quos(-ord, grp2, Group1 = grp1, Label = lbl, `1`),
      group = rlang::quos(grp2, grp1),
      label = rlang::quos(lbl),
      row_grp_plan_label_loc = "column"
    ),
    c("", "Group1", "Label")
  )

  expect_identical(
    create_stub_head(
      col_plan_vars = rlang::quos(-ord, grp2, Group1 = grp1, Label = lbl, `1`),
      group = rlang::quos(grp2, grp1),
      label = rlang::quos(lbl),
      row_grp_plan_label_loc = "column"
    ),
    c("", "Group1", "Label")
  )

  # col plan out of order (should retain group/label original order)

  expect_identical(
    create_stub_head(
      col_plan_vars = rlang::quos(
        Label = lbl,
        grp1,
        Group2 = grp2,
        everything()
      ),
      group = rlang::quos(grp2, grp1),
      label = rlang::quos(lbl),
      row_grp_plan_label_loc = "indented"
    ),
    "Group2"
  )

  expect_identical(
    create_stub_head(
      col_plan_vars = rlang::quos(
        Label = lbl,
        grp1,
        Group2 = grp2,
        everything()
      ),
      group = rlang::quos(grp2, grp1),
      label = rlang::quos(lbl),
      row_grp_plan_label_loc = "column"
    ),
    c("Group2", "", "Label")
  )

  # col plan with subtraction

  expect_identical(
    create_stub_head(
      col_plan_vars = rlang::quos(
        -grp1,
        Label = lbl,
        Group2 = grp2,
        everything()
      ),
      group = rlang::quos(grp2, grp1),
      label = rlang::quos(lbl),
      row_grp_plan_label_loc = "indented"
    ),
    "Group2"
  )

  expect_identical(
    create_stub_head(
      col_plan_vars = rlang::quos(
        -grp1,
        Label = lbl,
        Group2 = grp2,
        everything()
      ),
      group = rlang::quos(grp2, grp1),
      label = rlang::quos(lbl),
      row_grp_plan_label_loc = "column"
    ),
    c("Group2", "Label")
  )

  expect_snapshot(
    create_stub_head(
      col_plan_vars = rlang::quos(-ord, grp2, lbl, `1`),
      group = rlang::quos(grp2),
      label = rlang::quos(lbl),
      row_grp_plan_label_loc = "indented"
    )
  )

  expect_snapshot(
    create_stub_head(
      col_plan_vars = rlang::quos(-ord, grp2, lbl, `1`),
      group = rlang::quos(grp2),
      label = rlang::quos(lbl),
      row_grp_plan_label_loc = "column"
    )
  )
})

test_that("apply_col_plan() works", {
  df1 <- tibble::tribble(
    ~grp2 , ~lbl , ~prm  , ~column , ~val , ~ord ,
    "c"   , "n"  , "n"   ,       1 , 1    ,    1 ,
    "c"   , "n"  , "n_2" ,       1 , 1.1  ,    1 ,
    "b"   , "m"  , "n"   ,       1 , 2    ,    2 ,
    "b"   , "m"  , "n_2" ,       1 , 2.1  ,    2 ,
    "v"   , "s"  , "n"   ,       1 , 3    ,    3 ,
    "v"   , "s"  , "n_3" ,       1 , 3.3  ,    3 ,
    "p"   , "e"  , "n"   ,       1 , 4    ,    4
  )

  expect_snapshot(
    apply_col_plan(
      df1,
      col_selection = "-ord",
      grp_lbl = rlang::quos(grp2, lbl)
    )
  )

  processed_df1 <- apply_col_plan(
    df1,
    col_selection = "-ord",
    grp_lbl = rlang::quos(grp2, lbl)
  )

  expect_identical(
    processed_df1,
    dplyr::select(df1, -ord)
  )

  # col_selection can be either character or a list of quosures
  expect_identical(
    apply_col_plan(
      df1,
      col_selection = "-ord",
      grp_lbl = rlang::quos(grp2, lbl)
    ),
    apply_col_plan(
      df1,
      col_selection = rlang::quos(-ord),
      grp_lbl = rlang::quos(grp2, lbl)
    )
  )
})

test_that("apply_col_plan() group and label vars are excluded from renaming", {
  df1 <- tibble::tribble(
    ~grp2 , ~lbl , ~prm  , ~column , ~val , ~ord ,
    "c"   , "n"  , "n"   ,       1 , 1    ,    1 ,
    "c"   , "n"  , "n_2" ,       1 , 1.1  ,    1 ,
    "b"   , "m"  , "n"   ,       1 , 2    ,    2 ,
    "b"   , "m"  , "n_2" ,       1 , 2.1  ,    2 ,
    "v"   , "s"  , "n"   ,       1 , 3    ,    3 ,
    "v"   , "s"  , "n_3" ,       1 , 3.3  ,    3 ,
    "p"   , "e"  , "n"   ,       1 , 4    ,    4
  )

  expect_snapshot(
    apply_col_plan(
      df1,
      col_selection = c("-ord", "lbl"),
      grp_lbl = rlang::quos(grp2, lbl)
    )
  )

  processed_df1 <- apply_col_plan(
    df1,
    col_selection = c("-ord", "lbl"),
    grp_lbl = rlang::quos(grp2, lbl)
  )

  expect_identical(
    processed_df1,
    dplyr::select(df1, -ord)
  )
})

test_that("create_col_order() works", {
  expect_s3_class(
    create_col_order(
      data_names = c("grp2", "lbl", "ord", "1"),
      columns = rlang::quos(column),
      cp = col_plan(-ord)
    ),
    c("quosures", "list")
  )

  expect_snapshot(
    create_col_order(
      data_names = c("grp2", "lbl", "ord", "1"),
      columns = rlang::quos(column),
      cp = col_plan(-ord)
    ),
    # we strip out the environment memory address (aka the label)
    transform = strip_env_label
  )

  expect_equal(
    create_col_order(
      data_names = c("grp2", "lbl", "ord", "1"),
      columns = rlang::quos(column),
      cp = col_plan(-ord)
    ),
    rlang::quos(-ord, grp2, lbl, `1`),
    ignore_formula_env = TRUE
  )
})

test_that("create_col_order() with NULL col plan", {
  # the output is a list of the data_names as quosures
  expect_s3_class(
    create_col_order(
      data_names = c("grp2", "lbl", "ord", "1"),
      columns = rlang::quos(column),
      cp = NULL
    ),
    c("quosures", "list")
  )

  expect_snapshot(
    create_col_order(
      data_names = c("grp2", "lbl", "ord", "1"),
      columns = rlang::quos(column),
      cp = NULL
    ),
    # we strip out the environment memory address (aka the label)
    transform = strip_env_label
  )

  expect_equal(
    create_col_order(
      data_names = c("grp2", "lbl", "ord", "1"),
      columns = rlang::quos(column),
      cp = NULL
    ),
    rlang::quos(
      grp2,
      lbl,
      ord,
      `1`
    ),
    ignore_formula_env = TRUE
  )
})

test_that("create_col_order() with empty columns arg & cp not NULL", {
  expect_s3_class(
    create_col_order(
      data_names = c("grp2", "lbl", "ord", "1"),
      columns = rlang::quos(),
      cp = col_plan(-ord)
    ),
    c("quosures", "list")
  )

  expect_snapshot(
    create_col_order(
      data_names = c("grp2", "lbl", "ord", "1"),
      columns = rlang::quos(),
      cp = col_plan(-ord)
    ),
    # we strip out the environment memory address (aka the label)
    transform = strip_env_label
  )

  expect_identical(
    create_col_order(
      data_names = c("grp2", "lbl", "ord", "1"),
      columns = rlang::quos(),
      cp = col_plan(-ord)
    ),
    rlang::quos(-ord, grp2, lbl, `1`),
    ignore_formula_env = TRUE
  )
})

test_that("create_col_order() with span_structure()", {
  expect_snapshot(
    create_col_order(
      data_names = c(
        "group",
        "label",
        "cols 1,2___tlang_delim___col1",
        "cols 1,2___tlang_delim___col2",
        "mycol3",
        "col 4___tlang_delim___col4",
        "mycol5"
      ),
      columns = rlang::quos(span1, my_col),
      cp = col_plan(
        # rename column spanner in same level
        span_structure(
          span1 = c(
            "first cols" = "cols 1,2"
          )
        ),
        group,
        label,
        starts_with("col"),
        new_col_3 = mycol3,
        -mycol5
      )
    ),
    transform = strip_env_label
  )

  expect_identical(
    create_col_order(
      data_names = c(
        "group",
        "label",
        "cols 1,2___tlang_delim___col1",
        "cols 1,2___tlang_delim___col2",
        "mycol3",
        "col 4___tlang_delim___col4",
        "mycol5"
      ),
      columns = rlang::quos(span1, my_col),
      cp = col_plan(
        # rename column spanner in same level
        span_structure(
          span1 = c(
            "first cols" = "cols 1,2"
          )
        ),
        group,
        label,
        starts_with("col"),
        new_col_3 = mycol3,
        -mycol5
      )
    ),
    rlang::quos(
      group,
      label,
      `first cols___tlang_delim___col1` = `cols 1,2___tlang_delim___col1`,
      `first cols___tlang_delim___col2` = `cols 1,2___tlang_delim___col2`,
      `col 4___tlang_delim___col4`,
      new_col_3 = mycol3,
      -mycol5
    ),
    ignore_formula_env = TRUE
  )
})

test_that("col_plan_quo_to_vars() works", {
  # base case
  expect_identical(
    col_plan_quo_to_vars(
      x = rlang::quos(ord),
      column_names = "column",
      data_names = c("grp2", "lbl", "ord", "1"),
      preselected_cols = NULL
    ),
    # the output is a named vector with empty names
    rlang::set_names("ord", "")
  )

  # negative selection
  expect_identical(
    col_plan_quo_to_vars(
      x = rlang::quos(-ord),
      column_names = "column",
      data_names = c("grp2", "lbl", "ord", "1"),
      preselected_cols = NULL
    ),
    # the output is a named vector with empty names
    rlang::set_names("-ord", "")
  )

  # names are propagated if x contains named quosures
  expect_identical(
    col_plan_quo_to_vars(
      x = rlang::quos(new_order = ord),
      column_names = "column",
      data_names = c("grp2", "lbl", "ord", "1"),
      preselected_cols = NULL
    ),
    c(new_order = "ord")
  )

  # when x is a named list of quosures the output is a named vector with unique
  # names derived from the tidyselect expression
  expect_identical(
    col_plan_quo_to_vars(
      x = rlang::quos(starts_with("ord")) |> rlang::quos_auto_name(),
      column_names = "column",
      data_names = c("grp2", "lbl", "ord", "order_new", "1"),
      preselected_cols = NULL
    ),
    c(
      `starts_with("ord")1` = "ord",
      `starts_with("ord")2` = "order_new"
    )
  )

  expect_identical(
    col_plan_quo_to_vars(
      x = rlang::quos(ord),
      column_names = "column",
      data_names = c("grp2", "lbl", "ord", "1"),
      preselected_cols = "lbl"
    ),
    rlang::set_names(
      c("lbl", "ord"),
      ""
    )
  )

  # impact of return_only_selected == TRUE
  expect_identical(
    col_plan_quo_to_vars(
      x = rlang::quos(ord),
      column_names = "column",
      data_names = c("grp2", "lbl", "ord", "1"),
      preselected_cols = "lbl",
      return_only_selected = TRUE
    ),
    rlang::set_names("ord", "")
  )

  # when return_only_selected is FALSE then both preselected and selected cols
  # are returned
  expect_identical(
    col_plan_quo_to_vars(
      x = rlang::quos(ord),
      column_names = "column",
      data_names = c("grp2", "lbl", "ord", "1"),
      preselected_cols = "lbl",
      return_only_selected = FALSE
    ),
    rlang::set_names(
      c("lbl", "ord"),
      ""
    )
  )

  # default_everything_behavior = FALSE results in a reordering of the output
  # (preselected_cols come first)
  expect_identical(
    col_plan_quo_to_vars(
      x = rlang::quos(everything()),
      column_names = "column",
      data_names = c("grp2", "lbl", "ord", "1"),
      preselected_cols = c("lbl", "1"),
      default_everything_behavior = FALSE
    ),
    rlang::set_names(
      c("lbl", "1", "grp2", "ord"),
      ""
    )
  )
})

test_that("char_as_quo() works", {
  expect_s3_class(
    char_as_quo("x"),
    "quosure"
  )

  expect_identical(
    char_as_quo("x"),
    rlang::quo(x),
    ignore_formula_env = TRUE
  )

  expect_s3_class(
    char_as_quo(
      "starts_with('foo')"
    ),
    "quosure"
  )

  expect_identical(
    char_as_quo(
      "starts_with('foo')"
    ),
    rlang::quo(
      starts_with("foo")
    ),
    ignore_formula_env = TRUE
  )

  # when x is a valid(ish) tidyselect expression the output is a call
  expect_true(
    char_as_quo("starts_with('foo')") |>
      rlang::quo_is_call()
  )

  expect_false(
    char_as_quo("starts_with('foo')") |>
      rlang::quo_is_symbol()
  )

  # when x is not a valid tidyselect call the output is a symbol (enclosed
  # in backticks) and not a call
  expect_identical(
    char_as_quo("like('foo')"),
    rlang::quo(`like('foo')`),
    ignore_formula_env = TRUE
  )

  expect_true(
    char_as_quo("like('foo')") |>
      rlang::quo_is_symbol()
  )

  expect_false(
    char_as_quo("like('foo')") |>
      rlang::quo_is_call()
  )

  # parse errors are converted to symbols
  expect_true(
    char_as_quo(
      "foo12-3bar"
    ) |>
      rlang::quo_is_symbol()
  )

  expect_identical(
    char_as_quo("foo12-3bar"),
    rlang::quo(`foo12-3bar`),
    ignore_formula_env = TRUE
  )
})

test_that("eval_col_plan_quo() works", {
  expect_identical(
    eval_col_plan_quo(
      x = rlang::quo(ord),
      data_names = c("grp2", "lbl", "ord", "1"),
      preselected_vals = NULL,
      default_everything_behavior = FALSE
    ),
    "ord"
  )

  expect_identical(
    eval_col_plan_quo(
      x = rlang::quo(everything()),
      data_names = c("grp2", "lbl", "ord", "1"),
      preselected_vals = NULL,
      default_everything_behavior = TRUE
    ),
    c("grp2", "lbl", "ord", "1")
  )
})

test_that("col_plan_span_structure_to_vars() works", {
  expect_identical(
    col_plan_span_structure_to_vars(
      x = span_structure(
        span1 = c("first cols" = "cols 1,2")
      ),
      column_names = c("span1", "my_col"),
      data_names = c(
        "group",
        "label",
        "cols 1,2___tlang_delim___col1",
        "cols 1,2___tlang_delim___col2",
        "mycol3",
        "col 4___tlang_delim___col4",
        "mycol5"
      ),
      preselected_cols = NULL
    ),
    rlang::set_names(
      c(
        "cols 1,2___tlang_delim___col1",
        "cols 1,2___tlang_delim___col2",
        "col 4___tlang_delim___col4"
      ),
      ""
    )
  )
})

test_that("split_data_names_to_df() works", {
  # simple case
  expect_snapshot(
    split_data_names_to_df(
      data_names = c("grp2", "lbl", "ord", "1"),
      preselected_cols = NULL,
      column_names = "column"
    )
  )

  expect_identical(
    split_data_names_to_df(
      data_names = c("grp2", "lbl", "ord", "1"),
      preselected_cols = NULL,
      column_names = "column"
    ),
    tibble::tibble(
      column = c("grp2", "lbl", "ord", "1"),
      `__tfrmt_new_name__column` = c("grp2", "lbl", "ord", "1"),
      subtraction_status = c(FALSE, FALSE, FALSE, FALSE)
    )
  )

  # preselected_cols renames
  expect_snapshot(
    split_data_names_to_df(
      data_names = c("grp2", "lbl", "ord", "1"),
      preselected_cols = c(order = "ord"),
      column_names = "column"
    )
  )

  # column splitting by column_names
  expect_snapshot(
    split_data_names_to_df(
      data_names = c("grp2", "lbl", "ord", "1"),
      preselected_cols = c(order = "ord"),
      column_names = c("foo", "bar")
    )
  )
})

test_that("unite_df_to_data_names() works", {
  expect_identical(
    split_data_names_to_df(
      data_names = c("grp2", "lbl", "ord", "1"),
      preselected_cols = NULL,
      column_names = "column"
    ) |>
      unite_df_to_data_names(
        preselected_cols = NULL,
        column_names = "column",
        return_only_selected = FALSE
      ),
    rlang::set_names(
      c("grp2", "lbl", "ord", "1"),
      ""
    )
  )

  expect_identical(
    split_data_names_to_df(
      data_names = c("grp2", "lbl", "ord", "1"),
      preselected_cols = "ord",
      column_names = "column"
    ) |>
      unite_df_to_data_names(
        preselected_cols = "ord",
        column_names = "column",
        return_only_selected = TRUE
      ),
    rlang::set_names(
      c("ord", "grp2", "lbl", "1"),
      ""
    )
  )
})

#' Create frmt character string from significant digits spec
#'
#' @param sigdig Number of significant digits to add to default setting for that param
#' @param single_glue_to_frmt optional glue syntax for stand-alone (non-combined) parameters. NA if not applicable
#'
#' @return formatted spec as character string
#' @noRd
#'
#' @importFrom stringr str_dup str_replace
sigdig_frmt_string <- function(sigdig = 2, single_glue_to_frmt) {

  if (is.na(sigdig)){
    frmted_string <- "x"
  } else {
    frmted_dec <- str_dup("x", sigdig)
    if (!frmted_dec==""){
      frmted_dec <- paste0(".", frmted_dec)
    }
    frmted_string <- paste0("x", frmted_dec)
  }

  if (is.na(single_glue_to_frmt)){
     frmted_string
  } else {
    str_replace(single_glue_to_frmt, "\\{.*\\}", frmted_string)
  }
}


#' Set custom parameter-level significant digits rounding
#'
#' @param ... Series of name-value pairs, optionally formatted using
#'   `glue::glue()` syntax (note `glue` syntax is required for combined
#'   parameters).The name represents the parameter and the value represents the number of places to round the parameter to.
#'   For combined parameters (e.g., \code{"{min}, {max}"}), value should
#'   be a vector of the same length (e.g., c(1,1)).
#'
#' @details Type `param_set()` in console to view package defaults. Use of the
#'   function will add to the defaults and/or override included defaults of the
#'   same name. For values that are integers, use `NA` so no decimal places will
#'   be added.
#'
#' @examples
#' # View included defaults
#' param_set()
#'
#' # Update the defaults
#' param_set("{mean} ({sd})" = c(2,3), "pct" = 1)
#'
#' # Separate mean and SD to different lines
#' param_set("mean" = 2, "sd" = 3)
#'
#' # Add formatting using the glue syntax
#' param_set("{pct} %" = 1)
#'
#' @return list of default parameter-level significant digits rounding
#' @export
#' @importFrom purrr map_lgl map2_lgl
param_set <- function(...){
  args <-  list(...)

  if (length(args)>0){
    all_numeric_args <- map_lgl(args, ~is.numeric(.) || is.na(.))  %>% all()
    all_named_args <- names(args) %>% nchar() %>% all(.>0)
    if (!all_numeric_args || !all_named_args){
      stop("`param_set` entry must be named numeric vector.")
    }
  }

  # make list of all params: default + user specified
  param_list <- list(
    "min" = 1,
    "max" = 1,
    "median" = 1,
    "{mean} ({sd})" = c(1,2),
    "n" = NA
  )

  # determine if any existing params need to be overwritten

  args_params <- c(names(args), str_extract_all(names(args), "(?<=\\{)[^\\}]+(?=\\})") %>% unlist)

  idx_drop <- seq_along(param_list) %>%
    map_dfr(~tibble(param_display = names(param_list)[.x],
                    params = str_extract_all(.data$param_display, "(?<=\\{)[^\\}]+(?=\\})"))) %>%
    mutate(idx = row_number()) %>%
    unnest("params", keep_empty = TRUE) %>%
    mutate(drop = map2_lgl(.data$param_display, .data$params, ~  (.x %in% args_params || .y %in% args_params))) %>%
    filter(drop == TRUE) %>%
    pull(.data$idx) %>%
    unique()

  if(length(idx_drop)>0){
    param_list <- param_list[-idx_drop]
  }
  c(param_list,
    args)

}



#' Create tfrmt object from significant digits spec
#'
#' This function creates a tfrmt based on significant digits specifications for
#' group/label values. The input data spec provided to `sigdig_df` will contain
#' group/label value specifications. `tfrmt_sigdig` assumes that these columns
#' are group columns unless otherwise specified. The user may optionally choose
#' to pass the names of the group and/or label columns as arguments to the
#' function.
#'
#'
#' @param sigdig_df data frame containing significant digits formatting spec.
#'   Has 1 record per group/label value, and columns for relevant group and/or
#'   label variables, as well as a numeric column `sigdig` containing the
#'   significant digits rounding to be applied in addition to the default. If
#'   unique group/label values are represented in multiple rows, this will
#'   result in only one of the `sigdig` values being carried through in
#'   implementation.
#' @param group what are the grouping vars of the input dataset
#' @param label what is the label column of the input dataset
#' @param param_defaults Option to override or add to default parameters.
#' @param missing missing option to be included in all `frmt`s
#' @param tfrmt_obj an optional tfrmt object to layer
#' @param ... These dots are for future extensions and must be empty.
#'
#' @return `tfrmt` object with a `body_plan` constructed based on the
#'   significant digits data spec and param-level significant digits defaults.
#'
#' @details
#'
#' ## Formats covered
#'
#' Currently covers specifications for `frmt` and
#' `frmt_combine`. `frmt_when` not supported and must be supplied in additional
#' `tfrmt` that is layered on.
#'
#' ## Group/label variables
#'
#' If the group/label variables are not provided to the arguments, the body_plan
#' will be constructed from the input data with the following behaviour:
#' - If no group or label are supplied, it will be assumed that all columns in the input
#' data are group columns.
#' - If a label variable is provided, but nothing is
#' specified for group, any leftover columns (i.e. not matching `sigdig` or the
#' supplied label variable name) in the input data will be assumed to be group
#' columns.
#' - If any group variable is provided, any leftover columns (i.e. not
#' matching `sigdig` or the supplied group/label variable) will be disregarded.
#'
#' @section Examples:
#'
#' ```r
#' sig_input <- tibble::tribble(
#'   ~group1,   ~group2, ~sigdig,
#'   "CHEMISTRY",   ".default", 3,
#'   "CHEMISTRY",   "ALBUMIN",  1,
#'   "CHEMISTRY",   "CALCIUM",   1,
#'   ".default",    ".default",  2
#' )
#'
#' # Subset data for the example
#' data <- dplyr::filter(data_labs, group2 == "BASOPHILS", col1 %in% c("Placebo", "Xanomeline Low Dose"))
#' tfrmt_sigdig(sigdig_df = sig_input,
#'              group = vars(group1, group2),
#'              label = rowlbl,
#'              param_defaults = param_set("[{n}]" = NA)) |>
#'   tfrmt(column = vars(col1, col2),
#'         param = param,
#'         value = value,
#'         sorting_cols = vars(ord1, ord2, ord3),
#'         col_plan = col_plan(-starts_with("ord"))) |>
#'   print_to_gt(.data = data)
#' ```
#' \if{html}{\out{
#' `r "<img src=\"https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/master/images/example_sigdig.png\" alt=\"Table of Hematology, which are rounded for visits baseline to week 26\" style=\"width:100\\%;\">"`
#' }}
#'
#'
#' @export
#'
#' @importFrom dplyr rowwise group_split desc vars all_of
#' @importFrom tidyr unite
#' @importFrom purrr map
#' @importFrom rlang quo_is_missing syms as_name as_label
tfrmt_sigdig <- function(sigdig_df,
                         group=vars(),
                         label=quo(),
                         param_defaults = param_set(),
                         missing = NULL,
                         tfrmt_obj = NULL,
                         ...){

  tfrmt_inputs <-  quo_get(c("group","label"), as_var_args = "group", as_quo_args = "label")

  # if a tfrmt_obj is supplied and no group or label parameters are passed, use the one from the tfrmt_obj
  if(!is.null(tfrmt_obj)){
    if(is_empty(tfrmt_inputs$group) && !is_empty(tfrmt_obj$group)){
        tfrmt_inputs$group <- tfrmt_obj$group
    }
    if(quo_is_missing(tfrmt_inputs$label) && !quo_is_missing(tfrmt_obj$label)){
      tfrmt_inputs$label <- tfrmt_obj$label
    }
  }

  # error if no sigdig column
  if (!"sigdig" %in% names(sigdig_df)){
    stop("`sigdig_df` input must contain `sigdig` column.")
  }

  # error if no group/label columns available
  data_names <- sigdig_df %>% select(-"sigdig") %>% names()
  if (length(data_names)==0){
    stop("`sigdig_df` input must contain group and/or label value columns.")
  }

  group_names <- map_chr(tfrmt_inputs$group, as_label)
  label_name <- if (quo_is_missing(tfrmt_inputs$label)) character(0) else as_label(tfrmt_inputs$label)

  # if group param is provided, figure out which group/label variables are present in data and only keep those
  if (length(group_names)>0){
    sigdig_df <- sigdig_df %>%
      select(
        tidyselect::any_of(
          c(
            group_names,
            label_name,
            "sigdig"
          )
        )
      )

    # error if mismatch between provided group (and label, if it exists) & data columns
    data_names <- sigdig_df %>% select(-"sigdig") %>% names()
    if (length(data_names)==0){
      group_msg <- if(length(group_names)>0) paste0("group: ", paste(group_names, collapse = ", "), "\n") else ""
      label_msg <- if(length(label_name)>0) paste0("label: ", paste(label_name, collapse = ", ")) else ""
      stop("`sigdig_df` input does not contain any of the specified group/label params:\n",
           group_msg,
           label_msg)
    }
  }

  # if group param is NOT provided, any columns not covered by label param will be set to group param
  if (length(group_names)==0){
    groups_to_add <- setdiff(data_names, label_name)
    tfrmt_inputs$group <- c(tfrmt_inputs$group, vars(!!!syms(groups_to_add)))
  }


  # warning if provided group params are not present in the data
  new_group_names <- map_chr(tfrmt_inputs$group, as_label)

  if (!all(new_group_names %in% names(sigdig_df))){
    grp <- setdiff(new_group_names, names(sigdig_df))
    warning("`sigdig_df` input does not contain the following group params: ", paste0(grp, collapse = ", "))
  }


  # if input data contains grouping variables, establish ordering based on
  # whether any of the grouping values are set to .default
  groups_in_data <- intersect(data_names, new_group_names)

  if (length(groups_in_data)>0){
    data_ord <- sigdig_df %>%
      unite(
        "def_ord",
        tidyselect::all_of(groups_in_data),
        remove = FALSE
      ) %>%
      mutate(def_ord = str_count(.data$def_ord, ".default"))
  } else {
    data_ord <- sigdig_df %>%
      mutate(def_ord = 0)
  }

  # Create body plan
  frmt_structure_list <- data_ord %>%
    group_by(def_ord = desc(.data$def_ord), .data$sigdig) %>%
    group_split() %>%
    map(select, -"def_ord") %>%
    map(body_plan_builder, tfrmt_inputs$group, tfrmt_inputs$label, param_defaults, missing = missing)

  bp <- frmt_structure_list %>%
    do.call("c",.)   %>%
    do.call("body_plan", .)


  # output tfrmt (layer with previous, if applicable)
  tfrmt_out <- tfrmt(
    group = tfrmt_inputs$group,
    label = tfrmt_inputs$label,
    body_plan = bp)

  if(!is.null(tfrmt_obj)){
    layer_tfrmt(x = tfrmt_obj,
                y = tfrmt_out)
  } else {
     tfrmt_out
  }

}

#' Table Body Plan
#'
#' Define the formatting of the body contents of the table through a series of
#' frmt_structures. Structures get applied in order from bottom up, so the last
#' added structure is the first applied.
#'
#' @seealso [frmt_structure()] defines which rows the formats will be applied
#'   to, and [frmt()], [frmt_combine()], and [frmt_when()] define the format
#'   semantics.
#'
#'   \href{https://gsk-biostatistics.github.io/tfrmt/articles/body_plan.html}{Link to related article}
#'
#' @param ... list of frmt_structures defining the body formatting
#'
#' @return body_plan object
#'
#' @examples
#'
#'   tfrmt_spec<- tfrmt(
#'       title = "Table Title",
#'       body_plan = body_plan(
#'         frmt_structure(
#'           group_val = c("group1"),
#'           label_val = ".default",
#'           frmt("XXX")
#'         )
#'       )
#'      )
#'
#' @export
#'
body_plan <- function(...){

  frmt_structure_list <- list(...)

  for(struct_idx in seq_along(frmt_structure_list)){
    if(!is_frmt_structure(frmt_structure_list[[struct_idx]])){
      stop(paste0("Entry number ",struct_idx," is not an object of class `frmt_structure`."))
    }
  }

  structure(
    frmt_structure_list,
    class = c("body_plan", "frmt_table")
  )
}


#' Build contents of body (group/label value-specific) plan based on significant digits specifications
#'
#' @param data significant digits data for a given set of group/label values
#' @param tfrmt tfrmt object
#' @param param_defaults parameter-level significant digits specifications
#' @param missing missing option to be included in all `frmt`s
#'
#' @return list of `frmt_structure` objects
#' @noRd
#' @importFrom stringr str_detect str_extract_all
#' @importFrom purrr map_dfr map map_chr quietly pmap_chr
#' @importFrom dplyr mutate group_by filter group_split select across
#' @importFrom tidyr unnest
#' @importFrom rlang as_name quo_is_missing
body_plan_builder <- function(data, group, label, param_defaults, missing = NULL){

  # prep params for frmt functions
  param_tbl <- seq_along(param_defaults) %>%
    map_dfr(~tibble(param_display = names(param_defaults)[.x],
                    sigdig = list(param_defaults[[.x]] + data$sigdig[[1]]),
                    pos = .x)) %>%
    mutate(contains_glue = str_detect(.data$param_display, "\\{.*\\}"),  # is this to be a frmt_combine
           param = map2(.data$param_display, .data$contains_glue, ~ if(.y==TRUE){
             str_extract_all(.x, "(?<=\\{)[^\\}]+(?=\\})") %>% unlist
           } else {.x}),
           single_glue_to_frmt = pmap_chr(list(.data$contains_glue, .data$param, .data$param_display), function(a,b,c){
             if(a==TRUE & length(b) == 1) c else NA_character_
           } )) %>%
    unnest(
      tidyselect::everything()
    ) %>%
    mutate(frmt_string = map2_chr(.data$sigdig, .data$single_glue_to_frmt, sigdig_frmt_string))

  frmt_vec <- param_tbl %>%
    group_by(.data$pos) %>%
    group_split() %>%
    map(function(x){
      if(sum(x$contains_glue)>1){
        frmt_combine_builder(x$param_display[[1]], x$param, x$frmt_string, missing)
      } else{
        frmt_builder(x$param, x$frmt_string, missing)
      }
    })

  frmt_vec <-do.call(c, frmt_vec)

  # group/label names from tfrmt
  grp_names <- if (length(group)==0) character(0) else group %>% map_chr(as_name)
  lbl_names <- if(quo_is_missing(label)) character(0) else as_name(label)

  # sigdig value
  sigdig <- data$sigdig[[1]]

  which_grp <- grp_names[grp_names %in% names(data)]
  which_lbl <- lbl_names[lbl_names %in% names(data)]

  if(length(which_grp)>0){
    group_val <- data[,which_grp] %>%
      as.list() %>%
      map(unique)

    if (length(grp_names)>length(group_val)){
      group_val_to_add <- grp_names[!grp_names %in% names(group_val)]
      group_list_to_add <- rep(".default", length(group_val_to_add)) %>%
        as.list() %>%
        setNames(group_val_to_add)
      group_val <- c(group_val, group_list_to_add)[grp_names]
    }
  } else {
    group_val <- ".default"
  }

  if(length(which_lbl)>0){
    label_val <- data[,which_lbl, drop = TRUE] %>% unique()
    label_val <- if(any(label_val==".default")){".default"} else {label_val}
  } else {
    label_val <- ".default"
  }

  frmt_structure_builder(group_val, label_val, frmt_vec)

}

prep_info_return <- "Unable to apply {.fn {prep_func}}."

#' Combine variables
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' A wrapper around `tidyr::unite()` which pastes several columns into one.
#' In addition it checks the output is identical to `dplyr::coalesce()`. If not
#' identical, the input data.frame is returned unchanged. Useful for uniting
#' sparsely populated columns, for example when processing an ard that was
#' created with [cards::ard_stack()] then shuffled with `[shuffle_card()]`.
#'
#' If the data is the result of a hierarchical ard stack (with
#' [cards::ard_stack_hierarchical()] or
#' [cards::ard_stack_hierarchical_count()]), the input is returned unchanged.
#' This is assessed from the information in the `context` column which needs to
#' be present. If the input data does not have a `context` column, the input
#' will be returned unmodified.
#'
#' @param df (data.frame)
#' @param vars (character) a vector of variables to unite. If a single variable
#'   is supplied, the input is returned unchanged.
#' @inheritParams tidyr::unite
#'
#' @returns a data.frame with an additional column, called `variable_level` or
#'   the input unchanged.
#' @export
#'
#' @examples
#' df <- data.frame(
#'   a = 1:6,
#'   context = rep("categorical", 6),
#'   b = c("a", rep(NA, 5)),
#'   c = c(NA, "b", rep(NA, 4)),
#'   d = c(NA, NA, "c", rep(NA, 3)),
#'   e = c(NA, NA, NA, "d", rep(NA, 2)),
#'   f = c(NA, NA, NA, NA, "e", NA),
#'   g = c(rep(NA, 5), "f")
#' )
#'
#' prep_combine_vars(
#'   df,
#'   vars = c("b", "c", "d", "e", "f", "g")
#' )
prep_combine_vars <- function(df, vars, remove = TRUE) {

  if (!rlang::is_character(vars)) {
    cli::cli_abort(
      "{.arg vars} must be a character vector. You have supplied \\
      {.obj_type_friendly {vars}}."
    )
  }

  prep_func <- rlang::frame_call() |>
    rlang::call_name()

  required_cols <- "context"
  missing_cols <- setdiff(required_cols, names(df))

  if (!rlang::is_empty(missing_cols)) {
    cli::cli_inform(
      c(
        "i" = "Required column{?s} ({.code {missing_cols}}) not present in \\
        the input data.",
        "*" = prep_info_return
      )
    )
    return(df)
  }

  if ("hierarchical" %in% unique(df$context)) {
    cli::cli_inform(
      c(
        "i" = "The {.code context} column indicates data comes from a \\
        hierarchical {.code ard} stack.",
        "*" = prep_info_return
      )
    )
    return(df)
  }

  # we do cannot unite a single variable
  if (length(vars) == 1) {
    cli::cli_inform(
      c(
        "i" = "You supplied a single column in {.code vars}.",
        "*" = prep_info_return
      )
    )
    return(df)
  }

  interim <- df |>
    dplyr::mutate(
      var_level_coalesced = coalesce(
        !!!rlang::syms(vars)
      )
    ) |>
    tidyr::unite(
      col = "var_level_untd",
      tidyselect::all_of(vars),
      na.rm = TRUE,
      remove = remove
    ) |>
    dplyr::mutate(
      var_level_untd = dplyr::if_else(
        .data$var_level_untd == "",
        NA_character_,
        .data$var_level_untd
      )
    )

  if (!identical(interim$var_level_untd, interim$var_level_coalesced)) {
    cli::cli_inform(
      c(
        "i" = "Combining the columns listed in {.code vars} would result in \\
        a loss of information.",
        "*" = prep_info_return
      )
    )
    return(df)
  }

  output <- interim |>
    dplyr::select(
      -"var_level_coalesced"
    ) |>
    dplyr::rename(
      "variable_level" = "var_level_untd",
    )

  output
}

#' Prepare `bigN` stat variables
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' `prep_big_n()`:
#'   * recodes the `"n"` `stat_name` into `bigN` for the desired variables,
#'   and
#'   * drops all other `stat_names` for the same variables.
#'
#' If your `tfrmt` contains a [big_n_structure()] you pass the tfrmt `column` to
#' `prep_big_n()` via `vars`.
#'
#' @param df (data.frame)
#' @param vars (character) a vector of variables to prepare `bigN` for.
#'
#' @returns a data.frame with the same columns as the input. The `stat_name`
#'   column is modified.
#' @export
#'
#' @examples
#' df <- data.frame(
#'   stat_name = c("n", "max", "min", rep(c("n", "N", "p"), times = 2)),
#'   context = rep(c("continuous", "hierarchical", "categorical"), each = 3),
#'   stat_variable = rep(c("a", "b", "c"), each = 3)
#' ) |>
#'   dplyr::bind_rows(
#'     data.frame(
#'       stat_name = "n",
#'       context = "total_n",
#'       stat_variable = "d"
#'     )
#'   )
#'
#' prep_big_n(
#'   df,
#'   vars = c("b", "c")
#' )
prep_big_n <- function(df, vars) {

  if (!rlang::is_character(vars)) {
    cli::cli_abort(
      "{.arg vars} must be a character vector. You have supplied \\
      {.obj_type_friendly {vars}}."
    )
  }

  prep_func <- rlang::frame_call() |>
    rlang::call_name()

  required_cols <- c("context", "stat_variable", "stat_name")
  missing_cols <- setdiff(required_cols, names(df))

  if (!rlang::is_empty(missing_cols)) {
    cli::cli_inform(
      c(
        "i" = "Required column{?s} ({.code {missing_cols}}) not present in \\
        the input data.",
        "*" = prep_info_return
      )
    )
    return(df)
  }

  output <- df |>
    dplyr::mutate(
      stat_name = dplyr::case_when(
        .data$context == "total_n" ~ "bigN",
        # we only want to keep the subgroup totals, which get recoded to bigN
        .data$stat_variable %in% vars & .data$stat_name == "n" ~ "bigN",
        # we only want the bigN for overall -> we remove "out"
        .data$stat_variable %in% vars & .data$stat_name != "n" ~ "out",
        TRUE ~ .data$stat_name
      )
    ) |>
    dplyr::filter(
      .data$stat_name != "out"
    )

  output
}

#' Prepare label
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Adds a `label` column which is a combination of `stat_label` (for continuous
#' variables) and `variable_level` (for categorical ones) if these 2 columns are
#' present in the input data frame.
#'
#' @param df (data.frame)
#'
#' @returns a data.frame with a `label` column (if the input has the required
#'   columns) or the input unchanged.
#' @export
#'
#' @examples
#' df <- data.frame(
#'   variable_level = c("d", "e", "f"),
#'   stat_label = c("a", "b", "c"),
#'   stat_name = c("n", "N", "n"),
#'   context = c("categorical", "continuous", "hierarchical")
#' )
#'
#' prep_label(df)
prep_label <- function(df) {

  prep_func <- rlang::frame_call() |>
    rlang::call_name()

  required_cols <- c("context", "variable_level", "stat_label", "stat_name")
  missing_cols <- setdiff(required_cols, names(df))

  if (!rlang::is_empty(missing_cols)) {
    cli::cli_inform(
      c(
        "i" = "Required column{?s} ({.code {missing_cols}}) not present in \\
        the input data.",
        "*" = prep_info_return
      )
    )
    return(df)
  }

  output <- df |>
    dplyr::mutate(
      label = .data$stat_label,
      label = dplyr::if_else(
        !.data$context %in% c("continuous", "summary") &
          .data$stat_name %in% c("n", "N", "p"),
        .data$variable_level,
        .data$label
      )
    )

  output
}

#' Fill missing values in hierarchical variables
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Replace `NA` values in one column conditional on the same row having a
#' non-NA value in a different column.
#'
#' The user supplies a vector of columns from which the pairs will be extracted
#' with a rolling window. For example `vars <- c("A", "B", "C")` will generate
#' 2 pairs `("A", "B")` and `("B", "C")`. Therefore the order of the variables
#' matters.
#'
#' In each pair the second column `B` will be filled if `A` is not missing. One
#' can choose the value to fill with:
#'   * `"Any {colname}"`, in this case evaluating to `"Any B"` is the default.
#'   * Any other value. For example `"Any event"` for an adverse effects table.
#'   * the value of pair's first column. In this case, the value of `A`.
#'
#' @param df (data.frame)
#' @param vars (character) a vector of variables to generate pairs from.
#' @param fill (character) value to replace with. Defaults to `"Any {colname}"`,
#'   in which case `colname` will be replaced with the name of the column.
#' @param fill_from_left (logical) indicating whether to fill from the left
#'   (first) column in the pair. Defaults to `FALSE`. If `TRUE` it takes
#'   precedence over `fill`.
#'
#' @returns a data.frame with the same columns as the input, but in which some
#'   the desired columns have been filled pairwise.
#' @export
#'
#' @examples
#' df <- data.frame(
#'   x = c(1, 2, NA),
#'   y = c("a", NA, "b"),
#'   z = rep(NA, 3)
#' )
#'
#' prep_hierarchical_fill(
#'   df,
#'   vars = c("x", "y")
#' )
#'
#' prep_hierarchical_fill(
#'   df,
#'   vars = c("x", "y"),
#'   fill = "foo"
#' )
#'
#' prep_hierarchical_fill(
#'   df,
#'   vars = c("x", "y", "z"),
#'   fill_from_left = TRUE
#' )
prep_hierarchical_fill <- function(df,
                                   vars,
                                   fill = "Any {colname}",
                                   fill_from_left = FALSE) {

  if (!rlang::is_character(vars)) {
    cli::cli_abort(
      "{.arg vars} must be a character vector. You have supplied \\
      {.obj_type_friendly {vars}}."
    )
  }

  prep_func <- rlang::frame_call() |>
    rlang::call_name()

  if (length(vars) < 2) {
    cli::cli_inform(
      c(
        "i" = "At least 2 columns must be supplied to {.code vars}.",
        "*" = prep_info_return
      )
    )
    return(df)
  }

  pair_list <- generate_pairs(vars)

  output <- df

  for (i in seq_along(pair_list)) {
    output <- replace_na_pairwise(
      output,
      pair = pair_list[[i]],
      fill = fill,
      fill_from_left = fill_from_left
    )
  }

  output
}

#' Generate pairs for pairwise filling
#'
#' [prep_hierarchical_fill()] does pairwise conditional replacement of `NA`s.
#' `generate_pairs()` builds those pairs.
#'
#' @param x (character) a vector of 2 or more column names
#' @inheritParams cli::cli_abort
#'
#' @returns a list of length 2 character vectors (pairs of column names)
#' @noRd
#'
#' @examples
#' tfrmt:::generate_pairs(c("foo", "bar", "baz"))
generate_pairs <- function(x, call = rlang::caller_env()) {

  if (!rlang::is_character(x)) {
    cli::cli_abort(
      "{.arg x} must be a character vector. You have supplied \\
      {.obj_type_friendly {x}}.",
      call = call
    )
  }

  if (length(x) < 2) {
    cli::cli_abort(
      "{.arg x} must contain at least 2 column names. It contains {length(x)}.",
      call = call
    )
  }

  output <- tibble::tibble(x = x) |>
    dplyr::mutate(
      x_lead = dplyr::lead(x)
    ) |>
    tidyr::drop_na() |>
    purrr::pmap(c, use.names = FALSE)

  output
}

#' Replace `NA`s pairwise conditionally
#'
#' Replace missing values in one variable if a another variable is not `NA`.
#' This is the function used by [prep_hierarchical_fill()] to iterate over the
#' pairs of columns.
#'
#' @param x (data.frame) a shuffled card.
#' @param pair (character) a vector of exactly 2 column names.
#' @inheritParams prep_hierarchical_fill
#' @inheritParams cli::cli_abort
#'
#' @returns a list of length 2 character vectors (pairs of column names)
#' @noRd
#'
#' @examples
#' tfrmt:::replace_na_pairwise(
#'   data.frame(
#'   x = c(1, 2, NA),
#'   y = c("a", NA, "b"),
#'   z = rep(NA, 3)
#'   ),
#'   pair = c("y", "z")
#' )
replace_na_pairwise <- function(x,
                                pair,
                                fill = "Any {colname}",
                                fill_from_left = FALSE,
                                call = rlang::caller_env()) {

  if (!rlang::is_character(pair)) {
    cli::cli_abort(
      "{.arg pair} must be a character vector. You have supplied \\
      {.obj_type_friendly {pair}}.",
      call = call
    )
  }

  if (length(pair) != 2) {
    cli::cli_abort(
      "{.arg pair} must contain exactly 2 elements. The one you supplied has \\
      {length(pair)}.",
      call = call
    )
  }

  if (!rlang::is_scalar_character(fill)) {
    cli::cli_abort(
      "{.arg fill} must be a character vector of length 1.",
      call = call
    )
  }

  variables_syms <- rlang::syms(pair)

  if (fill == "Any {colname}") {
    fill <- glue::glue("Any {variables_syms[[2]]}") |>
      as.character()
  }

  if (fill_from_left) {
    fill <- rlang::quo(as.character(!!variables_syms[[1]]))
  }

  output <- x |>
    dplyr::mutate(
      !!variables_syms[[2]] := dplyr::if_else(
        is.na(!!variables_syms[[2]]) & !is.na(!!variables_syms[[1]]),
        !!fill,
        !!variables_syms[[2]]
      )
    )

  output
}

test_that("prep_...() pipe with demographic data", {

  # data prep -------------------------------------------------------------
  adsl <- pharmaverseadam::adsl |>
    dplyr::filter(SAFFL == "Y") |>
    dplyr::select(USUBJID, ARM, SEX, AGE, AGEGR1, ETHNIC, RACE) |>
    dplyr::mutate(
      SEX = factor(
        SEX,
        levels = c("F", "M")
      ),
      AGEGR1 = factor(
        AGEGR1,
        levels = c("<18", "18-64", ">64"),
        labels = c("<=18 years", "Between 18 and 65 years", ">=65 years")
      ),
      ETHNIC = factor(
        ETHNIC,
        levels = c(
          "HISPANIC OR LATINO",
          "NOT HISPANIC OR LATINO",
          "NOT REPORTED"
        )
      ),
      RACE = factor(
        RACE,
        levels = c(
          "AMERICAN INDIAN OR ALASKA NATIVE",
          "ASIAN",
          "BLACK OR AFRICAN AMERICAN",
          "NATIVE HAWAIIAN OR OTHER PACIFIC ISLANDER",
          "WHITE"
        )
      )
    )

  # create card -----------------------------------------------------------
  ard <- cards::ard_stack(
    data = adsl,
    .by = ARM,
    cards::ard_continuous(
      variables = AGE,
      statistic = ~ cards::continuous_summary_fns(
        c("N", "mean", "sd", "min", "max")
      )
    ),
    cards::ard_categorical(
      variables = c(AGEGR1, SEX, ETHNIC, RACE)
    ),
    .overall = TRUE,
    .total_n = TRUE
  )


  # manual tidy -------------------------------------------------------------
  ard_tbl <- ard |>
    # rename cols, coalesce variable levels/stat_labels, unnest
    cards::rename_ard_columns(columns = c("group1")) |>
    cards::unlist_ard_columns() |>
    dplyr::mutate(
      label = purrr::map_chr(
        variable_level,
        ~ {
          # Assign NA if the element is empty
          if (length(.x) == 0) {
            return(NA_character_)
          }

          as.character(.x[[1]]) # Convert to character
        }
      )
    ) |>
    # set missing labels to stat_lavel
    dplyr::mutate(
      label = dplyr::if_else(is.na(label), stat_label, label),
      # Add big N labels
      ARM = dplyr::if_else(
        variable == "ARM",
        label,
        ARM
      ),
      # ID total trt
      ARM = dplyr::if_else(
        is.na(ARM) | variable == "..ard_total_n..",
        "Overall ARM",
        ARM
      ),
      # unique stat names for big N's
      stat_name = dplyr::if_else(
        (variable == "ARM" & stat_name == "n") | variable == "..ard_total_n..",
        "bigN",
        stat_name
      ),
      # relabel the label for n's
      label = dplyr::if_else(
        stat_name == "N",
        "n",
        label
      )
    ) |>
    # remove unneeded stats
    dplyr::filter(
      !(variable == "ARM" & stat_name != "bigN")
    ) |>
    # sorting (mostly handled by factors, above)
    dplyr::mutate(
      ord1 = forcats::fct_inorder(variable) |>
        forcats::fct_relevel("SEX", after = 0) |>
        as.numeric(),
      ord2 = dplyr::if_else(label == "n", 1, 2)
    ) |>
    # drop variables not needed
    dplyr::select(ARM, variable, label, stat_name, stat, ord1, ord2) |>
    # remove dups (extra denoms per variable level)
    unique()

  # the tfrmt table -------------------------------------------------------
  dm_t01 <- tfrmt(
    title = "Summary of Demographic Characteristics",
    group = variable,
    label = label,
    param = stat_name,
    value = stat,
    column = ARM,
    sorting_cols = c(ord1, ord2),
    body_plan = body_plan(
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt("xxx")
      ),
      frmt_structure(
        group_val = ".default",
        label_val = ".default",
        frmt_combine(
          "{n} ({p}%)",
          n = frmt("xxx"),
          p = frmt("xx", transform = ~ . * 100)
        )
      )
    ),
    big_n = big_n_structure(
      param_val = "bigN",
      n_frmt = frmt("\n(N=xx)")
    ),
    col_plan = col_plan(
      -starts_with("ord")
    ),
    col_style_plan = col_style_plan(
      col_style_structure(
        col = everything(),
        align = "left"
      )
    ),
    row_grp_plan = row_grp_plan(
      row_grp_structure(
        group_val = ".default",
        element_block(
          post_space = " "
        )
      )
    )
  )

  a <- print_to_gt(dm_t01, ard_tbl)
  a_html <- a |>
    gt::as_raw_html() |>
    # the id is randomly generated
    strip_id()


  # tidy with `prep_...()` functions ----------------------------------------
  prepped_ard <- ard |>
    shuffle_card(
      by = "ARM"
    ) |>
    prep_combine_vars(
      c("AGE", "AGEGR1", "SEX", "ETHNIC", "RACE")
    ) |>
    prep_big_n(
      vars = "ARM"
    ) |>
    prep_label() |>
    dplyr::mutate(
      label = dplyr::if_else(
        .data$stat_name == "N",
        "n",
        .data$label
      ),
      variable_level = dplyr::if_else(
        .data$stat_name == "N",
        NA,
        .data$variable_level
      )
    ) |>
    unique() |>
    dplyr::mutate(
      # we need these additional steps for compatibility with the manual
      # processing
      label = dplyr::if_else(
        .data$stat_name == "bigN" &
          .data$context %in% c("categorical", "tabulate"),
        ARM,
        .data$label
      ),
      ord1 = forcats::fct_inorder(stat_variable) |>
        forcats::fct_relevel("SEX", after = 0) |>
        as.numeric(),
      ord2 = dplyr::if_else(label == "n", 1, 2)
    ) |>
    dplyr::select(
      ARM,
      variable = stat_variable,
      label,
      stat_name,
      stat,
      ord1,
      ord2
    )

  expect_identical(
    dplyr::arrange(ard_tbl, ord1, ord2),
    dplyr::arrange(prepped_ard, ord1, ord2),
    ignore_attr = "class"
  )

  expect_no_error(
    b <- print_to_gt(dm_t01, prepped_ard)
  )

  b_html <- b |>
    gt::as_raw_html() |>
    # the id is randomly generated so we strip it
    strip_id()

  expect_identical(
    a_html,
    b_html
  )

  expect_identical(
    gt::extract_body(a),
    gt::extract_body(b)
  )

  expect_snapshot(
    gt::as_raw_html(b),
    transform = strip_id
  )
})

test_that("prep_...() pipe with adverse effects data", {

  # data prep -------------------------------------------------------------
  # Filter to include only subjects marked as part of the safety population
  adsl <- pharmaverseadam::adsl |>
    dplyr::filter(SAFFL == "Y")

  #subset data to limit printed rows for demo purposes
  adae <- pharmaverseadam::adae |>
    dplyr::filter(AESOC %in% unique(AESOC)[1:3]) |>
    dplyr::group_by(AESOC) |>
    dplyr::filter(AEDECOD %in% unique(AEDECOD)[1:3]) |>
    dplyr::ungroup()

  # create card -----------------------------------------------------------
  suppressMessages(
    ae_ard <- cards::ard_stack_hierarchical(
      data = adae,
      by = c(TRT01A, AESEV),
      variables = c(AEBODSYS, AETERM),
      statistic = ~ c("n", "p"), # Calculate count and percentage
      denominator = adsl,
      id = USUBJID,
      over_variables = TRUE,
      total_n = TRUE
    )
  )


  # manual tidy -------------------------------------------------------------
  ae2_ard_tbl <- ae_ard |>
    cards::rename_ard_columns() |>
    cards::unlist_ard_columns() |>
    dplyr::mutate(
      # label any event rows for top level any event and each aesoc any event
      # rows
      AETERM = dplyr::if_else(
        !is.na(..ard_hierarchical_overall..) &
          ..ard_hierarchical_overall.. == TRUE |
          is.na(AETERM) & !is.na(AEBODSYS),
        "ANY EVENT",
        AETERM
      ),
      # update na rows in aebodsys to be ANY EVENT so labels appear correctly
      # in tfrmt
      AEBODSYS = dplyr::if_else(
        AETERM == "ANY EVENT" &
          is.na(AEBODSYS),
        "ANY EVENT",
        AEBODSYS
      ),
      #create bigN values
      stat_name = dplyr::if_else(
        is.na(AETERM) & stat_name == "n" | context == "total_n",
        "bigN",
        stat_name
      )
    ) |>
    #filter to just the needed stats
    dplyr::filter(
      !(is.na(AETERM) & stat_name %in% c("N", "p"))
    ) |>
    dplyr::select(
      TRT01A,
      AESEV,
      AEBODSYS,
      AETERM,
      stat,
      stat_name
    )

  # the tfrmt table -------------------------------------------------------
  ae_t02 <- tfrmt_n_pct(
    n = "n",
    pct = "p",
    pct_frmt_when = frmt_when(
      "==1" ~ frmt("100"),
      ">.99" ~ frmt("(>99%)"),
      "==0" ~ "",
      "<.01" ~ frmt("(<1%)"),
      "TRUE" ~ frmt("(xx%)", transform = ~ . * 100)
    )
  ) |>
    tfrmt(
      group = AEBODSYS,
      label = AETERM,
      param = stat_name,
      value = stat,
      column = c(TRT01A, AESEV),
      sorting_cols = AEBODSYS,
      row_grp_plan = row_grp_plan(
        row_grp_structure(
          group_val = ".default",
          element_block(
            post_space = " "
          )
        )
      ),
      big_n = big_n_structure(
        param_val = "bigN",
        n_frmt = frmt(" (N=xx)")
      )
    )

  a <- print_to_gt(ae_t02, ae2_ard_tbl)
  a

  # tidy with `prep_...()` functions ----------------------------------------
  prepped_ard <- ae_ard |>
    shuffle_card(
      by = c("TRT01A", "AESEV"),
      fill_overall = NA,
      fill_hierarchical_overall = "ANY EVENT"
    ) |>
    prep_big_n(
      vars = c("TRT01A", "AESEV")
    ) |>
    prep_hierarchical_fill(
      vars = c("AEBODSYS", "AETERM"),
      fill = "ANY EVENT"
    ) |>
    dplyr::select(-context, -stat_variable, -stat_label) |>
    dplyr::relocate(stat_name, .after = stat)

  expect_equal(
    arrange(ae2_ard_tbl, TRT01A, AESEV, AEBODSYS, AETERM),
    arrange(prepped_ard, TRT01A, AESEV, AEBODSYS, AETERM),
    ignore_attr = TRUE
  )

  expect_no_error(
    b <- print_to_gt(ae_t02, prepped_ard)
  )

  expect_identical(
    gt::extract_body(a),
    gt::extract_body(b),
    ignore_attr = ".col_plan_vars"
  )

  # check messaging around combining vars in a hierarchical ard stack df
  expect_message(
    ae_ard |>
      shuffle_card(
        by = c("TRT01A", "AESEV"),
        fill_overall = NA,
        fill_hierarchical_overall = "ANY EVENT"
      ) |>
      prep_combine_vars(
        vars = c("AEBODSYS", "AETERM")
      ),
    regexp = "The `context` column indicates data comes from a hierarchical `ard` stack.",
    fixed = TRUE
  )

  # check messaging around prep_label() needing `variable_level`
  expect_message(
    ae_ard |>
      shuffle_card(
        by = c("TRT01A", "AESEV"),
        fill_overall = NA,
        fill_hierarchical_overall = "ANY EVENT"
      ) |>
      prep_big_n(
        vars = c("TRT01A", "AESEV")
      ) |>
      prep_label(),
    regexp = "Required column (`variable_level`) not present in the input data.",
    fixed = TRUE
  )
})

test_that("prep_combine_vars() works", {
  df <- tibble::tibble(
    a = 1:6,
    context = rep("categorical", 6),
    b = c("a", rep(NA, 5)),
    c = c(NA, "b", rep(NA, 4)),
    d = c(NA, NA, "c", rep(NA, 3)),
    e = c(NA, NA, NA, "d", rep(NA, 2)),
    f = c(NA, NA, NA, NA, "e", NA),
    g = c(rep(NA, 5), "f")
  )

  expect_identical(
    prep_combine_vars(
      df,
      vars = c("b", "c", "d", "e", "f", "g")
    ),
    tibble::tibble(
      a = 1:6,
      context = rep("categorical", 6),
      variable_level = c("a", "b", "c", "d", "e", "f")
    )
  )

  expect_snapshot(
    prep_combine_vars(
      df,
      vars = c("b", "c", "d", "e", "f", "g")
    )
  )

  expect_snapshot(
    prep_combine_vars(
      df,
      vars = c("b", "c", "d", "e", "f"),
      remove = FALSE
    )
  )
})

test_that("prep_combine_vars() returns the input when context hierarchical", {

  df <- tibble::tibble(
    a = 1:6,
    context = rep("hierarchical", 6),
    b = c("a", rep(NA, 5)),
    c = c(NA, "b", rep(NA, 4)),
    d = c(NA, NA, "c", rep(NA, 3)),
    e = c(NA, NA, NA, "d", rep(NA, 2)),
    f = c(NA, NA, NA, NA, "e", NA),
    g = c(rep(NA, 5), "f")
  )

  expect_identical(
    suppressMessages(
      prep_combine_vars(
        df,
        vars = c("b", "c", "d", "e", "f", "g")
      )
    ),
    df
  )

  expect_snapshot(
    prep_combine_vars(
      df,
      vars = c("b", "c", "d", "e", "f", "g")
    )
  )
})

test_that("prep_combine_vars() return input unchanged when length(vars)=1", {

  df <- tibble::tibble(
    a = 1:6,
    context = rep("categorical", 6),
    b = c("a", rep(NA, 5)),
    c = c(NA, "b", rep(NA, 4)),
    d = c(NA, NA, "c", rep(NA, 3)),
    e = c(NA, NA, NA, "d", rep(NA, 2)),
    f = c(NA, NA, NA, NA, "e", NA),
    g = c(rep(NA, 5), "f")
  )

  expect_identical(
    suppressMessages(
      prep_combine_vars(
        df,
        vars = "b"
      )
    ),
    df
  )

  expect_snapshot(
    prep_combine_vars(
      df,
      vars = "b"
    )
  )
})

test_that("prep_combine_vars() does not over unite", {

  # c, d and e are identical, the are not pasted together in the output
  # the input is returned unchanged
  df <- tibble::tibble(
    a = 1:6,
    context = rep("categorical", 6),
    b = c("a", rep(NA, 5)),
    c = c(NA, "b", rep(NA, 4)),
    d = c(NA, "b", rep(NA, 4)),
    e = c(NA, "b", rep(NA, 4)),
    f = c(NA, NA, NA, NA, "e", NA),
    g = c(rep(NA, 5), "f")
  )

  expect_identical(
    suppressMessages(
      prep_combine_vars(
        df,
        vars = c("b", "c", "d", "e", "f", "g")
      )
    ),
    df
  )

  expect_snapshot(
    prep_combine_vars(
      df,
      vars = c("b", "c", "d", "e", "f", "g")
    )
  )
})

test_that("prep_combine_vars() informs when the context col is missing", {
  df <- tibble::tibble(
    a = 1:6,
    b = c("a", rep(NA, 5)),
    c = c(NA, "b", rep(NA, 4)),
    d = c(NA, "b", rep(NA, 4)),
    e = c(NA, "b", rep(NA, 4)),
    f = c(NA, NA, NA, NA, "e", NA),
    g = c(rep(NA, 5), "f")
  )

  expect_snapshot(
    prep_combine_vars(
      df,
      vars = c("b", "c", "d", "e", "f", "g")
    )
  )
})

test_that("prep_combine_vars() errors when `vars` is not character", {
  df <- tibble::tibble(
    a = 1:6,
    b = c("a", rep(NA, 5)),
    c = c(NA, "b", rep(NA, 4)),
    d = c(NA, "b", rep(NA, 4)),
    e = c(NA, "b", rep(NA, 4)),
    f = c(NA, NA, NA, NA, "e", NA),
    g = c(rep(NA, 5), "f")
  )

  expect_snapshot(
    error = TRUE,
    prep_combine_vars(
      df,
      vars = 1:3
    )
  )

  expect_snapshot(
    error = TRUE,
    prep_combine_vars(
      df,
      vars = c(TRUE, FALSE)
    )
  )
})

test_that("prep_big_n() works", {
  df <- tibble::tibble(
    stat_name = c("n", "max", "min", rep(c("n", "N", "p"), times = 2)),
    context = rep(c("continuous", "hierarchical", "categorical"), each = 3),
    stat_variable = rep(c("a", "b", "c"), each = 3)
  ) |>
    bind_rows(
      tibble::tibble(
        stat_name = "n",
        context = "total_n",
        stat_variable = "d"
      )
    )

  expect_identical(
    prep_big_n(
      df,
      vars = c("b", "c")
    ),
    tibble::tibble(
      stat_name = c("n", "max", "min", rep("bigN", 3)),
      context = c(
        rep("continuous", 3),
        "hierarchical",
        "categorical",
        "total_n"
      ),
      stat_variable = c(rep("a", 3), "b", "c", "d")
    )
  )

  expect_snapshot(
    prep_big_n(
      df,
      vars = c("b", "c")
    )
  )

  expect_snapshot(
    prep_big_n(
      df,
      vars = "b"
    )
  )
})

test_that("prep_big_n() informs when required columns are missing", {
  df <- tibble::tibble(
    a = c("n", "max", "min", rep(c("n", "N", "p"), times = 2)),
    context = rep(c("continuous", "hierarchical", "categorical"), each = 3),
    stat_variable = rep(c("a", "b", "c"), each = 3)
  )

  expect_identical(
    suppressMessages(
      prep_big_n(
        df,
        vars = "a"
      )
    ),
    df
  )

  expect_snapshot(
    prep_big_n(
      df,
      vars = "a"
    )
  )
})

test_that("prep_big_n() errors when `vars` is not character", {
  df <- tibble::tibble(
    a = c("n", "max", "min", rep(c("n", "N", "p"), times = 2)),
    context = rep(c("continuous", "hierarchical", "categorical"), each = 3),
    stat_variable = rep(c("a", "b", "c"), each = 3)
  )

  expect_snapshot(
    error = TRUE,
    prep_big_n(
      df,
      vars = 1
    )
  )

  expect_snapshot(
    error = TRUE,
    prep_big_n(
      df,
      vars = TRUE
    )
  )
})

test_that("prep_label() works", {
  df <- tibble::tibble(
    variable_level = rep(c("a", "b", "c"), each = 3),
    stat_label = c("n", "N", "%", "N", "Mean", "SD", "n", "N", "%"),
    stat_name = c("n", "N", "p", "N", "mean", "sd", "n", "N", "p"),
    context = rep(c("categorical", "continuous", "hierarchical"), each = 3)
  )

  expect_identical(
    prep_label(df),
    tibble::tibble(
      variable_level = rep(c("a", "b", "c"), each = 3),
      stat_label = c("n", "N", "%", "N", "Mean", "SD", "n", "N", "%"),
      stat_name = c("n", "N", "p", "N", "mean", "sd", "n", "N", "p"),
      context = rep(c("categorical", "continuous", "hierarchical"), each = 3),
      label = c("a", "a", "a", "N", "Mean", "SD", "c", "c", "c")
    )
  )

  expect_snapshot(
    prep_label(df)
  )
})

test_that("prep_label() returns the input when the required cols are missing", {
  # `variable_level` col is not present
  df <- tibble::tibble(
    x = c("d", "e", "f"),
    stat_label = c("a", "b", "c"),
    context = c("categorical", "continuous", "hierarchical")
  )

  expect_identical(
    suppressMessages(prep_label(df)),
    df
  )

  expect_snapshot(
    prep_label(df)
  )

  # `stat_label` col is not present
  df2 <- tibble::tibble(
    variable_level = c("d", "e", "f"),
    y = c("a", "b", "c"),
    context = c("categorical", "continuous", "hierarchical")
  )

  expect_snapshot(
    prep_label(df2)
  )

  expect_identical(
    suppressMessages(prep_label(df2)),
    df2
  )
})

test_that("prep_hierarchical_fill() returns input when `length(vars) < 2`", {
  df <- tibble::tibble(
    x = c(1, 2, NA),
    y = c("a", NA, "b"),
    z = rep(NA, 3)
  )

  expect_identical(
    suppressMessages(
      prep_hierarchical_fill(
        df,
        vars = "y"
      )
    ),
    df
  )

  expect_snapshot(
    prep_hierarchical_fill(
      df,
      vars = "y"
    ),
  )
})

test_that("prep_hierarchical_fill() fills pairwise conditionally", {

  df <- tibble::tibble(
    x = c(1, 2, NA),
    y = c("a", NA, "b"),
    z = rep(NA, 3)
  )

  # z is not filled - still 3 NAs
  expect_identical(
    prep_hierarchical_fill(df, vars = c("x", "y")),
    tibble::tibble(
      x = c(1, 2, NA),
      y = c("a", "Any y", "b"),
      z = rep(NA, 3)
    )
  )

  expect_identical(
    prep_hierarchical_fill(df, vars = c("x", "y", "z")),
    tibble::tibble(
      x = c(1, 2, NA),
      y = c("a", "Any y", "b"),
      z = rep("Any z", 3)
    )
  )
})

test_that("prep_hierarchical_fill() with `fill`", {

  df <- tibble::tibble(
    x = c(1, 2, NA),
    y = c("a", NA, "b"),
    z = rep(NA, "3")
  )

  # the second value of y is replaced with "2
  expect_identical(
    prep_hierarchical_fill(
      df,
      vars = c("x", "y"),
      fill = "foo"
    ),
    tibble::tibble(
      x = c(1, 2, NA),
      y = c("a", "foo", "b"),
      z = rep(NA, 3)
    )
  )

  expect_identical(
    prep_hierarchical_fill(
      df,
      vars = c("x", "y", "z"),
      fill = "bar"
    ),
    tibble::tibble(
      x = c(1, 2, NA),
      y = c("a", "bar", "b"),
      z = c("bar", "bar", "bar")
    )
  )
})

test_that("prep_hierarchical_fill() with `fill` 'Any {colname}'", {

  df <- tibble::tibble(
    x = c(1, 2, NA),
    y = c("a", NA, "b"),
    z = rep(NA, "3")
  )

  expect_identical(
    prep_hierarchical_fill(
      df,
      vars = c("x", "y"),
      fill = "Any {colname}"
    ),
    tibble::tibble(
      x = c(1, 2, NA),
      y = c("a", "Any y", "b"),
      z = rep(NA, 3)
    )
  )

  expect_identical(
    prep_hierarchical_fill(
      df,
      vars = c("x", "y", "z"),
      fill = "Any {colname}"
    ),
    tibble::tibble(
      x = c(1, 2, NA),
      y = c("a", "Any y", "b"),
      z = c("Any z", "Any z", "Any z")
    )
  )
})

test_that("prep_hierarchical_fill() with `fill_from_left = TRUE' works", {

  df <- tibble::tibble(
    x = c(1, 2, NA),
    y = c("a", NA, "b"),
    z = rep(NA, "3")
  )

  expect_identical(
    prep_hierarchical_fill(
      df,
      vars = c("x", "y"),
      fill_from_left = TRUE
    ),
    tibble::tibble(
      x = c(1, 2, NA),
      y = c("a", "2", "b"),
      z = rep(NA, 3)
    )
  )

  expect_identical(
    prep_hierarchical_fill(
      df,
      vars = c("x", "y", "z"),
      fill_from_left = TRUE
    ),
    tibble::tibble(
      x = c(1, 2, NA),
      y = c("a", "2", "b"),
      z = c("a", "2", "b")
    )
  )
})

test_that("prep_hierarchical_fill() errors when `vars` is not character", {

  df <- tibble::tibble(
    x = c(1, 2, NA),
    y = c("a", NA, "b"),
    z = rep(NA, "3")
  )

  expect_snapshot(
    error = TRUE,
    prep_hierarchical_fill(
      df,
      vars = 1:3
    )
  )

  expect_snapshot(
    error = TRUE,
    prep_hierarchical_fill(
      df,
      vars = c(TRUE, FALSE)
    )
  )
})

test_that("prep_hierarchical_fill() errors when `fill` is not character", {

  df <- tibble::tibble(
    x = c(1, 2, NA),
    y = c("a", NA, "b"),
    z = rep(NA, "3")
  )

  expect_snapshot(
    error = TRUE,
    prep_hierarchical_fill(
      df,
      vars = c("x", "y", "z"),
      fill = 2
    )
  )
})

test_that("generate_pairs() works", {

  expect_identical(
    generate_pairs(
      c("foo", "bar", "baz")
    ),
    list(
      c("foo", "bar"),
      c("bar", "baz")
    )
  )
})

test_that("generate_pairs() complains", {

  # with non-character input
  expect_snapshot(
    error = TRUE,
    generate_pairs(1:3)
  )

  # with an input having less than 2 elements
  expect_snapshot(
    error = TRUE,
    generate_pairs("foo")
  )

})

test_that("replace_na_pairwise() works", {

  expect_identical(
    replace_na_pairwise(
      tibble::tibble(
        x = c(1, 2, NA),
        y = c("a", NA, "b"),
        z = rep(NA, 3)
      ),
      pair = c("y", "z")
    ),
    # all NAs in z (when y is not NA) are replaced with `"Any z"`
    tibble::tibble(
      x = c(1, 2, NA),
      y = c("a", NA, "b"),
      z = c("Any z", NA, "Any z")
    )
  )

  expect_identical(
    replace_na_pairwise(
      tibble::tibble(
        x = c(1, 2, NA),
        y = c("a", NA, "b"),
        z = rep(NA, 3)
      ),
      pair = c("y", "z"),
      fill = "foo"
    ),
    # all NAs in z (when y is not NA) are replaced with `"Any z"`
    tibble::tibble(
      x = c(1, 2, NA),
      y = c("a", NA, "b"),
      z = c("foo", NA, "foo")
    )
  )

  expect_identical(
    replace_na_pairwise(
      tibble::tibble(
        x = c(1, 2, NA),
        y = c("a", NA, "b"),
        z = rep(NA, 3)
      ),
      pair = c("y", "z"),
      fill_from_left = TRUE
    ),
    # all NAs in z (when y is not NA) are replaced with `"Any z"`
    tibble::tibble(
      x = c(1, 2, NA),
      y = c("a", NA, "b"),
      z = c("a", NA, "b")
    )
  )
})

test_that("replace_na_pairwise() complains", {

  expect_snapshot(
    error = TRUE,
    replace_na_pairwise(
      tibble::tibble(
        x = c(1, 2, NA),
        y = c("a", NA, "b"),
        z = rep(NA, 3)
      ),
      pair = 1:2
    )
  )

  expect_snapshot(
    error = TRUE,
    replace_na_pairwise(
      tibble::tibble(
        x = c(1, 2, NA),
        y = c("a", NA, "b"),
        z = rep(NA, 3)
      ),
      pair = c("x", "y", "z")
    )
  )

  expect_snapshot(
    error = TRUE,
    replace_na_pairwise(
      tibble::tibble(
        x = c(1, 2, NA),
        y = c("a", NA, "b"),
        z = rep(NA, "3")
      ),
      pair = c("y", "z"),
      fill = 2
    )
  )

})

#' Demography Analysis Results Data
#'
#' A dataset containing the results needed for a demography table. Using the
#' CDISC pilot data.
#'
#' @format A data frame with 386 rows and 7 variables:
#'  \describe{
#'   \item{rowlbl1}{highest level row labels}
#'   \item{rowlbl2}{more specific row labels}
#'   \item{param}{parameter to explain each value}
#'   \item{grp}{grouping column used to distinguish continuous and categorical}
#'   \item{ord1}{controls ordering}
#'   \item{ord2}{more ordering controls}
#'   \item{column}{column names}
#'   \item{value}{values to put in a table}
#'   }
"data_demog"


#' Adverse Events Analysis Results Data
#'
#' A dataset containing the results needed for an AE table. Using the
#' CDISC pilot data.
#'
#' @format A data frame with 2,794 rows and 8 variables:
#'  \describe{
#'   \item{AEBODSYS}{highest level row labels: System Organ Class}
#'   \item{AETERM}{more specific row labels: Preferred Term}
#'   \item{col2}{higher level column names (spanners)}
#'   \item{col1}{lower level column names}
#'   \item{param}{parameter to explain each value}
#'   \item{value}{values to put in a table}
#'   \item{ord1}{controls ordering}
#'   \item{ord2}{more ordering controls}
#'   }
"data_ae"

#' Efficacy Analysis Results Data
#'
#' A dataset containing the results needed for an Efficacy table. Using the
#' CDISC pilot data for ADAS-Cog(11).
#'
#' @format A data frame with 70 rows and 7 variables:
#'  \describe{
#'   \item{group}{highest level row labels}
#'   \item{label}{more specific row labels}
#'   \item{column}{column names}
#'   \item{param}{parameter to explain each value}
#'   \item{value}{values to put in a table}
#'   \item{ord1}{controls ordering}
#'   \item{ord2}{more ordering controls}
#'   }
"data_efficacy"


#' Labs Analysis Results Data
#'
#' A dataset containing the results needed for an labs results table. Using the
#' CDISC pilot data.
#'
#' @format A data frame with 4,950 rows and 7 variables:
#'  \describe{
#'   \item{group1}{highest level row labels: Lab value class}
#'   \item{group2}{more specific row labels: Lab parameter}
#'   \item{rowlbl}{most specific row labels: Study visit}
#'   \item{col1}{higher level column names (spanners)}
#'   \item{col2}{lower level column names}
#'   \item{param}{parameter to explain each value}
#'   \item{value}{values to put in a table}
#'   \item{ord1}{controls ordering}
#'   \item{ord2}{more ordering controls}
#'   \item{ord3}{more ordering controls}
#'   }
"data_labs"

# helper function to be used with the `expect_snapshot()` `transform` argument
# it takes a string and replaces the memory address of an environment (which is
# {rlang} uses as the environment label) with `"<env-address>"`. The memory
# address usually starts with "0x" or "0X" (as an indication that what follows
# is in hexadecimal) followed by 9, 12 or 16 characters (depending on the OS)
strip_env_label <- function(x) {
  stringr::str_replace_all(
    x,
    # memory addresses have 9 (macOS), 12(linux) or 16(windows) characters
    pattern = "(0[xX]){1}[A-Fa-f0-9]{9,16}",
    replacement = "<env-address>"
  )
}

test_that("strip_env_label() helper works", {
  expect_identical(
    strip_env_label("env:  0x72db165b8"),
    "env:  <env-address>"
  )

  expect_snapshot(
    strip_env_label(
      "<list_of<quosure>>\n\n[[1]]\n<quosure>\nexpr: ^grp2\nenv:  0x72db165b8\n\n[[2]]\n<quosure>\nexpr: ^lbl\nenv:  0x72db31118"
    ) |>
      cat()
  )
})

#' N Percent Template
#'
#' This function creates an tfrmt for an n % table, so count based table. The
#' parameter values for n and percent can be provided (by default it will assume
#' `n` and `pct`). Additionally the `frmt_when` for formatting the percent can
#' be specified. By default 100% and 0% will not appear and everything between
#' 99% and 100% and 0% and 1% will be rounded using greater than (>) and less
#' than (<) signs respectively.
#' @param n name of count (n) value in the parameter column
#' @param pct  name of percent (pct) value in the parameter column
#' @param pct_frmt_when formatting to be used on the the percent values
#' @param tfrmt_obj an optional tfrmt object to layer
#'
#' @returns tfrmt object
#' @export
#' @section Examples:
#'
#' ```r
#' print_mock_gt(tfrmt_n_pct())
#' ```
#'
#' \if{html}{\out{
#' `r "<img src=\"https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/master/images/example_n_percent.png\" alt=\"3 by 3 table\" style=\"width:50\\%;\">"`
#' }}
#'
#' @importFrom rlang parse_expr
tfrmt_n_pct <- function(n = "n",
                       pct = "pct",
                       pct_frmt_when = frmt_when("==100"~ frmt(""),
                                             ">99"~ frmt("(>99%)"),
                                             "==0"~ "",
                                             "<1" ~ frmt("(<1%)"),
                                             "TRUE" ~ frmt("(xx.x%)")),
                       tfrmt_obj = NULL){
  if(is.null(n)|is.na(n)|n==""){
    stop("`n` value must be provided")
  }
  if(is.null(pct)|is.na(pct)|pct==""){
    stop("`pct` value must be provided")
  }

  combo <- paste0(
    "frmt_combine('{",
    n,
    "} {", pct, "}',",
    n, "=frmt('x'),",
    pct, "=pct_frmt_when)"
  ) %>%
    parse_expr() %>% eval()

  if(!is.null(tfrmt_obj)){
    ae_tbl <- tfrmt(
      body_plan = body_plan(
        frmt_structure(
          group_val = ".default", label_val = ".default",
          combo
        )
      )
    )
    ae_tbl <- layer_tfrmt(x = tfrmt_obj, y = ae_tbl)
  } else {
    ae_tbl <- tfrmt(
      param = "param",
      label = "row_label1",
      column = "col1",
      value = "value",
      body_plan = body_plan(
        frmt_structure(
          group_val = ".default", label_val = ".default",
          combo
        )
      )
    )
  }
  ae_tbl
}

#' Apply formatting to dataset
#'
#' @param .data Data to apply the tfrmt to
#' @param tfrmt tfrmt object to apply to the data
#' @param mock Logical value is this is for a mock or not. By default `FALSE`
#'
#' @return formatted tibble
#' @noRd
apply_tfrmt <- function(.data, tfrmt, mock = FALSE){

  validate_cols_match(.data, tfrmt, mock)

  if(!is_tfrmt(tfrmt)){
    stop("Requires a tfrmt object")
  }

  tbl_dat <- .data %>%
    remove_big_ns(param = tfrmt$param,
                  big_n_structure = tfrmt$big_n) %>%
    apply_table_frmt_plan(
      .data = .,
      table_frmt_plan = tfrmt$body_plan,
      group = tfrmt$group,
      label = tfrmt$label,
      param = tfrmt$param,
      value = tfrmt$value,
      column = tfrmt$column,
      mock = mock
    )

  # check if order vars are causing rows to print over 2 lines
  check_order_vars(tbl_dat,tfrmt)

  big_n_df <- get_big_ns(.data, param = tfrmt$param,
                         value = tfrmt$value,
                         columns =tfrmt$column,
                         big_n_structure = tfrmt$big_n,
                         mock = mock)

  tbl_dat_wide <- tbl_dat %>%
    pivot_wider_tfrmt(tfrmt, mock) %>%
    # arrange if sorting cols are applied
    tentative_process(arrange_enquo, tfrmt$sorting_cols, fail_desc = "Unable to arrange dataset")


  if(is.null(tfrmt$row_grp_plan$label_loc)){
    tfrmt$row_grp_plan$label_loc <-  element_row_grp_loc(location = "indented")
  }

  tbl_dat_wide_processed <- tbl_dat_wide %>%
    #Apply row group structures defined in row_grp_plan
    tentative_process(
      apply_row_grp_struct,
      tfrmt$row_grp_plan$struct_list,
      tfrmt$group,
      tfrmt$label,
      fail_desc = "Unable to apply row group structure"
    ) %>%
    tentative_process(
      apply_page_plan,
      tfrmt$page_plan,
      tfrmt$group,
      tfrmt$label,
      tfrmt$row_grp_plan$label_loc$location)

  # if big_n is to be by page, check that the big N's match the # of tables,
  # with the right groups
  check_big_n_page(big_n_df, tbl_dat_wide_processed, tfrmt)

  # Results of col plan
  col_plan_vars <- tentative_process(
    names(tbl_dat_wide),
    create_col_order,
    cp = tfrmt$col_plan,
    columns = tfrmt$column,
    fail_desc = "Unable to create dataset subset vars"
  ) %>%
    as_vars()

  # just the stub
  stub_header <- create_stub_head(col_plan_vars,
                                  tfrmt$group,
                                  tfrmt$label,
                                  tfrmt$row_grp_plan$label_loc$location)

  # apply remaining processing to single table or map over list of tables (paginated)
  apply_tfrmt_subtable_mapper(tfrmt, tbl_dat_wide_processed, col_plan_vars, stub_header, big_n_df)

}


#' Process the remainder of table or list of tables
#'
#' @param tfrmt tfrmt object
#' @param .data processed wide tbl or list of processed wide tbls
#' @param col_plan_vars col plan converted to vars() to be applied
#' @param stub_header stub header (NULL if none)
#' @param big_n_df tbl of big Ns to apply (if 1 set of overall Ns) or list of tbl of bigNs (if grouping by page)
#' to apply
#'
#' @importFrom purrr map map2
#' @noRd
apply_tfrmt_subtable_mapper <- function(tfrmt, .data, col_plan_vars, stub_header, big_n_df){

  # there is a list of >1 tables
  if (inherits(.data, "list")){

    # there is a list of >1 big N tibbles
    if (inherits(big_n_df, "list") && length(big_n_df)>1){
      map2(.data, big_n_df, ~ apply_tfrmt_subtable(tfrmt, .x, col_plan_vars, stub_header, .y))
    } else {

      # there is only 1 set of (overall) big Ns
      map(.data, ~ apply_tfrmt_subtable(tfrmt, .x, col_plan_vars, stub_header, big_n_df))
    }
  } else {

    # there is a single table
    apply_tfrmt_subtable(tfrmt, .data, col_plan_vars, stub_header, big_n_df)
  }
}

#' Processing to apply to each table
#'
#' @param tfrmt tfrmt object
#' @param .data processed wide tbl
#' @param col_plan_vars col plan converted to vars() to be applied
#' @param stub_header stub header (NULL if none)
#' @param big_n_df big Ns to apply
#' @noRd
apply_tfrmt_subtable <- function(tfrmt, .data, col_plan_vars, stub_header, big_n_df){

  if (inherits(big_n_df, "list")) {
    big_n_df <- big_n_df[[1]]
  }

  col_plan_vars <- big_n_df %>%
    tentative_process(
      apply_big_n_df,
      col_plan_vars,
      columns = tfrmt$column,
      value = tfrmt$value,
      fail_desc = "Unable to add big N's"
    )

  tbl_dat_wide_processed <- .data  %>%
    #Select before grouping to not have to deal with if it indents or not
    tentative_process(apply_col_plan,
                      append(col_plan_vars, rlang::quos(tidyselect::any_of("..tfrmt_post_space_row"))),
                      c(tfrmt$group,tfrmt$label),
                      fail_desc = "Unable to subset dataset columns") %>%
    tentative_process(
                  apply_row_grp_lbl,
                  tfrmt$row_grp_plan$label_loc,
                  tfrmt$group,
                  tfrmt$label) %>%
    #Not in a tentative process cause some of the inputs might be null but still valid
    apply_footnote_meta(
                  footnote_plan = tfrmt$footnote_plan,
                  col_plan_vars = col_plan_vars,
                  element_row_grp_loc = tfrmt$row_grp_plan$label_loc,
                  tfrmt$group,
                  tfrmt$label,
                  columns = tfrmt$column
    ) %>%
    tentative_process(
                  apply_col_style_plan,
                  tfrmt,
                  col_plan_vars = col_plan_vars
    ) %>%
    tentative_process(
      remove_grp_cols,
      tfrmt$row_grp_plan$label_loc,
      tfrmt$group,
      tfrmt$label
    ) %>%
    tentative_process(
      apply_post_space_trim,
      fail_desc = "Unable to trim trailing spacer rows"
    )

    structure(
      tbl_dat_wide_processed,
      .col_plan_vars = col_plan_vars,
      .page_note = attr(.data, ".page_note"),
      .stub_header = stub_header,
      class = class(tbl_dat_wide_processed)
    )

}



#' Tentatively apply an element
#'
#' Will only apply the element to the data if the param isn't null
#'
#' @param .data data to process
#' @param fx processing function
#' @param ... inputs supplied to function arguments
#'
#' @return processed data
#' @importFrom purrr map_lgl
#' @importFrom rlang is_empty
#' @noRd
tentative_process <- function(.data, fx, ..., fail_desc = NULL){

  args <- list(...)

  if(any(map_lgl(args, is_empty))){
    out <- .data
  } else{
    out <- .data %>%
      safely(fx)(...)
    if(!is.null(out[["error"]])){
      if(is.null(fail_desc)){
       fx_char <- as.character(substitute(fx))
       fail_desc <- paste0("Unable to to apply ",fx_char,".")
      }
      error_message <- out[["error"]]$message

      if(error_message == ""){
        error_message <- format(out[["error"]],backtrace = FALSE)
      }

      fail_desc <-paste0(
        fail_desc,"\n",
        "Reason: ",
        error_message
      )
      message(fail_desc)

      out <- .data
    }else{
      out <- out$result
    }
  }
  out
}


#' Checks required columns exsist
#'
#' @param .data Data to apply the tfrmt to
#' @param tfrmt tfrmt object to apply to the data
#' @param mock Logical value is this is for a mock or not. By default `FALSE`
#'
#' @return formatted tibble
#' @noRd
#' @importFrom purrr map safely
#' @importFrom rlang !! !!!
#' @importFrom dplyr select
validate_cols_match <- function(.data, tfrmt, mock){

  #Required variables
  if(mock){
    req_quo <- c("label", "param")
  } else {
    req_quo <- c("label", "param", "value")
  }
  req_var <- c("group","column")

  .data <- .data %>% ungroup

  req_quo %>%
    map(function(x){
      var_test <- tfrmt[[x]]
      check <- safely(select)(.data, !!var_test)
      if(!is.null(check$error)){
        stop(paste0("Variable Specified in '", x, "' doesn't exist in the supplied dataset. Please check the tfrmt and try again."),
             call. = FALSE)
      }
    }
    )

  req_var %>%
    map(function(x){
      var_test <- tfrmt[[x]]
      check <- safely(select)(.data, !!!var_test)
      if(!is.null(check$error)){
        stop(paste0("Variable Specified in '", x, "' doesn't exist in the supplied dataset. Please check the tfrmt and try again."),
             call. = FALSE)
      }
    }
    )

}

#' Arrange data based on quosures
#'
#' @param dat data to arrange
#' @param param list of quosures to arrange on
#'
#' @noRd
#' @importFrom dplyr arrange
arrange_enquo <- function(dat, param){
  arrange(dat, !!!param)
}

#' Clean Spanning column names
#'
#' This function removes the prefix on columns that aren't nested
#' @param data data to rename
#'
#' @return dataset with renaming in needed
#' @noRd
#' @importFrom stringr str_count str_remove
#' @importFrom dplyr rename_with
clean_spanning_col_names <- function(data){
  # Get number of layers
  lyrs <- count_spanning_layers(names(data))
  # remove the layering for unnested columns
  if(lyrs > 0){
    data <- data %>%
      rename_with(~remove_empty_layers(.x, nlayers = lyrs))
  }
  data
}

count_spanning_layers <- function(x){
  x %>%
    str_count(.tlang_delim) %>%
    max()
}


## also used in select_col_plan to process column names the same way
remove_empty_layers <- function(x, nlayers = 1){
  empty_str <- paste0("^",strrep(paste0("NA", .tlang_delim), nlayers))
  str_remove(x, empty_str)
}

#' Pivot formatted values into a wide dataset
#'
#' @param data
#'
#' @return data pivoted wider
#' @noRd
#'
#' @importFrom purrr quietly
#' @importFrom dplyr group_by across summarise n tally pull na_if all_of reframe
#' @importFrom stringr str_detect
#' @importFrom tidyr unnest
pivot_wider_tfrmt <- function(data, tfrmt, mock){

  # check if data can be transformed wide w/o list columns
  num_rec_by_row <- data %>%
    group_by(across(c(-!!tfrmt$value, -!!tfrmt$param))) %>%
    summarise(
      param_list = list(!!tfrmt$param),
      n = n()
    )

  if (any(num_rec_by_row$n>1) ){
    val_fill <- list("")
    if(!mock){
      suggested_frmt_structs <- num_rec_by_row %>%
        ungroup %>%
        filter(n > 1) %>%
        select(-c(!!!tfrmt$column)) %>%
        unique() %>%
        group_by(!!!tfrmt$group,param_list) %>%
        mutate(label_quote=paste0('"',!!tfrmt$label,'"')) %>%
        reframe(label_collapse=as.character(paste(label_quote,collapse=',')),!!!tfrmt$group,n) %>%
        unique() %>%
        rowwise() %>%
        mutate(
          suggested_frmt_struct = frmt_struct_string(
            grp = list(!!!tfrmt$group),
            lbl = label_collapse,
            param_vals = .data$param_list
            )
        ) %>%
        pull(.data$suggested_frmt_struct) %>%
        paste0("- `",.,"`",collapse = "\n")


     inform(
       paste0(
       "Multiple param listed for the same group/label values.\n",
       "The following frmt_structures may be missing from the body_plan\n",
       "or the order may need to be changed:"
       ),
       body = suggested_frmt_structs,
       class = "_tlang_missing_frmt_structs"
     )
    }
  } else {
    val_fill <- ""
  }

  column_cols <- tfrmt$column %>%
    map_chr(as_name)
  tbl_dat_wide <- data %>%
    select(-!!tfrmt$param) %>%
    mutate(
      across(
        tidyselect::all_of(column_cols),
        ~as.character(.x)
      )
    ) %>%
    mutate(
      across(
        tidyselect::all_of(column_cols),
        ~na_if(.x, "")
      )
    ) %>%
    quietly(pivot_wider)(
      names_from = c(
        tidyselect::starts_with(
          .tlang_struct_col_prefix
        ),
        !!!tfrmt$column
      ),
      names_sep = .tlang_delim,
      values_from = !!tfrmt$value,
      values_fill = val_fill
    )

  if (mock == TRUE && length(tbl_dat_wide$warnings)>0 &&
      any(str_detect(tbl_dat_wide$warnings, paste0("Values from `", as_label(tfrmt$value), "` are not uniquely identified")))){
    message("Mock data contains more than 1 param per unique label value. Param values will appear in separate rows.")
    tbl_dat_wide <- tbl_dat_wide$result %>%
      unnest(
        cols = tidyselect::everything()
      ) %>%
      clean_spanning_col_names()
  } else {
    tbl_dat_wide <- tbl_dat_wide$result %>%
      clean_spanning_col_names()
  }

  tbl_dat_wide

}


frmt_struct_string <- function(grp, lbl, param_vals){
  length_lbl <- str_count(lbl,",")+1

  group_names <- substitute(grp) %>%
    as.list() %>%
    map_chr(as_label) %>% .[-1]
  if(length(group_names) > 1){
    group_val_char <- capture.output(dput(setNames(grp, group_names)))
  }else if(length(group_names) == 1){
    group_val_char <-  paste(capture.output(dput(grp[[1]])), collapse = "")
  }else{
    group_val_char <-  "\".default\""
  }

  if(length_lbl>1){
    lbl_names=paste0( "c(",lbl,")")
  }else{
    lbl_names = lbl
  }

  #label_val_char <- capture.output(dput(lbl))

  param_expr_char <- paste0("\"",paste0("{",param_vals,"}", collapse = ", "),"\"")
  param_frmt_char <- paste0(param_vals," = frmt(\"xx\")", collapse = ", ")

  paste0(
    "frmt_structure(",
    "group_val = ",group_val_char,
    ", label_val = ",lbl_names,
    ", frmt_combine(",
    param_expr_char,",",
    param_frmt_char,
    "))"
  )
}



#' Check Order Vars
#'
#' @param .data data into apply_tfrmt
#' @param tfrmt tfrmt object to apply to the data
#'
#' @return warning if order variables are causing formatting issues
#'
#' @importFrom dplyr n_distinct
#' @noRd
check_order_vars <- function(.data,tfrmt){

  if(is_empty(tfrmt$sorting_cols)==FALSE){
    # check for values printing on different lines due to incorrect order variables
    if(is_empty(tfrmt$group)==FALSE){
      order_check <- .data %>%
        group_by(!!!tfrmt$group,!!(tfrmt$label)) %>%
        mutate(n1=n_distinct(!!(tfrmt$label),!!!tfrmt$sorting_cols),
               n2=n_distinct(!!(tfrmt$label)))
    }else{
      order_check <- .data %>%
        group_by(!!tfrmt$label) %>%
        mutate(n1=n_distinct(!!tfrmt$label,!!!tfrmt$sorting_cols),
               n2=n_distinct(!!tfrmt$label))
    }

    # print warning if the number of lines printed over is greater than 1
    if(sum(order_check$n1)>nrow(order_check) & all(order_check$n1 == order_check$n2)==FALSE){
      message("Note: Some row labels have values printed over more than 1 line.\n This could be due to incorrect sorting variables. Each row in your output table should have only one sorting var combination assigned to it.")

    }}
}

#' check that the # of big N's correspond to the # of pages
#' @importFrom purrr map_dfr
#' @importFrom dplyr select all_of distinct
#' @noRd
check_big_n_page <- function(big_n_df, data_wide, tfrmt){

  if (!is.null(big_n_df) && tfrmt$big_n$by_page) {
      expected_pops <- length(data_wide)
      expected_grp_vars <- attr(data_wide, ".page_grp_vars")
      expected_grp_levs <- map_dfr(
        data_wide,
        ~ select(
          .x,
          tidyselect::all_of(
            expected_grp_vars
          )
        ) %>%
        distinct()
      )
      actual_pops <- length(big_n_df)
      actual_grp_levs <- map_dfr(
        big_n_df,
        ~ select(
          .x,
          tidyselect::any_of(
            expected_grp_vars
          )
        ) %>%
        distinct()
      )

      if (!identical(expected_pops,actual_pops) |
          (!is_empty(actual_grp_levs) &&
           !isTRUE(all.equal(expected_grp_levs, actual_grp_levs, check.attributes = FALSE))
           )){
        message("Mismatch between big Ns and page_plan. For varying big N's by page (`by_page` = TRUE in `big_n_structure`), data must contain 1 big N value per unique grouping variable/value set to \".default\" in `page_plan` and bigNs must be in order of the data")
      }
  }
}

## ---------------------------------------
## determine which columns to span across
## ---------------------------------------
eval_tidyselect_on_colvec <- function(x, column_vec){
  UseMethod("eval_tidyselect_on_colvec", x)
}

#' @importFrom rlang !!! as_label
#' @importFrom dplyr expr
#' @importFrom purrr keep
eval_tidyselect_on_colvec.quosures <- function(x, column_vec){
  names(column_vec) <- column_vec
  avli_x <- x %>%
    keep(~as_label(.) %in% column_vec)

  names(
    tidyselect::eval_select(
      expr(c(!!!avli_x)),
      data = column_vec
    )
  )
}

#' @importFrom rlang !!
#' @importFrom dplyr expr
eval_tidyselect_on_colvec.quosure <- function(x, column_vec){

  names(column_vec) <- column_vec


  names(
    tidyselect::eval_select(
      expr(c(!!x)),
      data = column_vec
    )
  )
}

#' Apply the footnote metadata to data
#'
#' Gets the location of each footnote in the footnote plan and adds it to the dataset as an attribute
#' @param .data formatted data
#' @param footnote_plan footnote plan
#' @param col_plan_vars named vector of the columns with new and old names
#' @param element_row_grp_loc element_row_grp_loc to help with where the footnote should go
#' @param group symbolic group(s)
#' @param label symbolic label
#' @param columns symbolic column(s)
#'
#' @return a processed tfrmt tbl object, which has a .footnote_locs attribute
#' @noRd
apply_footnote_meta <- function(.data, footnote_plan, col_plan_vars, element_row_grp_loc,
                                group, label, columns){

  footnote_locs <- footnote_plan$struct_list %>%
    map(locate_fn, .data = .data,
        col_plan_vars = col_plan_vars, element_row_grp_loc = element_row_grp_loc,
        group = group, label = label, columns = columns)

  structure(
    .data,
    .footnote_locs = footnote_locs,
    class = c("processed_tfrmt_tbl",class(.data))
  )
}


#' Locates the footnote for a given  footnote structure
#'
#' @param footnote_structure footnote structure
#' @param .data data set
#' @param col_plan_vars col plan variables
#' @param element_row_grp_loc row group location element
#' @param group group quosures
#' @param label label quosure
#' @param columns column quosure
#'
#' @return list with the row, column and footnote
#' @noRd
#' @importFrom dplyr inner_join first last cur_group_id
locate_fn <- function(footnote_structure, .data, col_plan_vars, element_row_grp_loc,
                            group, label, columns){

  col_info <- get_col_loc(footnote_structure, .data, col_plan_vars, columns)

  loc_info <- get_row_loc(footnote_structure, .data, element_row_grp_loc,
                          group, label, col_info)
  loc_info$note <- footnote_structure$footnote_text
  loc_info
}




#' Get the column location of the footnote
#'
#' @param footnote_structure a single footnote structure
#' @param .data formatted data
#' @param col_plan_vars named vector of the columns with new and old names
#' @param columns symbolic column(s)
#'
#' @return list with column locations (col) and if they are spanning or not (spanning)
#' @noRd
get_col_loc <- function(footnote_structure, .data, col_plan_vars, columns){

  loc_info <- footnote_structure %>%
    discard(is.null) %>%
    .[names(.) != "footnote_text"]

  # Get column information
  if( "column_val" %in% names(loc_info)){
    col_str <- columns %>% map_chr(as_label)

    if(is_empty(names(loc_info$column_val))){
      col_val_nm <- col_str
      loc_col_df <- tibble(!!col_str := loc_info$column_val)
    } else {
      loc_col_df <- loc_info$column_val %>%
        expand.grid(stringsAsFactors = FALSE)
      col_val_nm <- names(loc_info$column_val)
    }

    col_loc_df <- split_data_names_to_df(NULL, col_plan_vars%>%
                                           map_chr(as_label), col_str) %>%
      inner_join(loc_col_df, by = col_val_nm)

    if(nrow(col_loc_df) == 0){

      message_text <- c(
       paste0(
        "The provided column location does not exist in the provided data for the footnote",
        "\"",
        footnote_structure$footnote_text,
        "\""
        ),
        "Provided column location:"
        )

      for(col_var in col_val_nm){
        message_text <- c(
          message_text,
          paste0(col_var,": ",paste0("`",loc_info$column_val[[col_var]],"`", collapse = ","))
        )
      }

      message(paste0(message_text, collapse = "\n"))

      out <- list(col = NULL, spanning = FALSE)

    }else{
      # if not a column return the spanning column name
      span_lvl <- col_str %in% col_val_nm %>%
        which() %>%
        max() %>%
        col_str[.]

      if(last(col_str) == span_lvl){
        col_loc <- unite_df_to_data_names(col_loc_df, preselected_cols = c(), column_names = col_str)
        if(!is.null(names(col_loc))){
          col_loc <- if_else(names(col_loc) != "", names(col_loc), unname(col_loc))
        }
        out <- list(col = col_loc, spanning = FALSE)
      } else {
        col_loc <- col_loc_df %>%
          pull(paste0("__tfrmt_new_name__", span_lvl)) %>%
          unique()
        out <- list(col = col_loc, spanning = TRUE)
      }
    }
  } else {
    out <- list(col = NULL, spanning = FALSE)
  }
  out
}

#' Get the row location of the footnote
#'
#' @param footnote_structure a single footnote structure
#' @param .data formatted dataset
#' @param element_row_grp_loc row group location element
#' @param group group quosures
#' @param label label quosure
#' @param col_info list of column information from `get_col_loc()`
#'
#' @return a list with all column information, row, col, and spanning
#' @noRd
get_row_loc <- function(footnote_structure, .data, element_row_grp_loc,
                        group, label, col_info){


  loc_info <- footnote_structure %>%
    discard(is.null) %>%
    .[names(.) != "footnote_text"]

  row_grp <- ifelse(is.null(element_row_grp_loc$location), "",
                     element_row_grp_loc$location)
   if(any(names(loc_info) %in% c("group_val", "label_val"))){
     # Things that don't have rows
     if(!is.null(col_info$spanning) && col_info$spanning){
       warning("Cannot apply footnotes to rows when you have only specified a spanning column")
       col_info$row <- NULL
     } else if(row_grp == "noprint" & !is_empty(loc_info$group_val)){
       warning("Can not apply footnotes to group columns when 'noprint' is set")
       col_info$row <- NULL
     } else {
       group_str <- group %>% map_chr(as_label)
       # Test if there are more than the first group
       highest_grp <- setdiff(names(loc_info$group_val), first(group_str)) %>%
         length() == 0

       #Will the footnote be in the label column
       lb_col_test <- !is.null(loc_info$label_val) | #label level so in label col
         row_grp == "indented" | #indented so in label column
         (!highest_grp & !row_grp %in% c("", "gtdefault", "column")) #there is a row_grp_plan and this isn't the highest group so indented

       if(lb_col_test){
         grp_expr <- expr_to_filter(group, loc_info$group_val)
         label_vals <- ifelse(is.null(loc_info$label_val), loc_info$group_val,
                               loc_info$label_val) %>% unlist()
         lbl_expr <- expr_to_filter(label, label_vals)

         filter_expr <- paste(c(lbl_expr,grp_expr),collapse = "&") %>%
           parse_expr()

         col_info$row <- .data %>%
           ungroup() %>%
           mutate(
             across(c(!!!group, !!label),
                    ~str_remove(.x, paste0("^", element_row_grp_loc$indent, "+"))),
             `___tfrmt_test` = !!filter_expr,
             `___tfrmt_TEMP_rows` = row_number()) %>%
           filter(.data$`___tfrmt_test`) %>%
           pull(.data$`___tfrmt_TEMP_rows`)

         col_info$col <- ifelse(is.null(col_info$col), as_label(label),
                                     col_info$col)

       } else if(highest_grp){
         filter_expr <- expr_to_filter(group, loc_info$group_val) %>%
           parse_expr()
         col_info$row<-.data %>%
           group_by(!!first(group)) %>%
           mutate(`___tfrmt_grp_n` = cur_group_id(),
                  `___tfrmt_test` = !!filter_expr) %>%
           filter(.data$`___tfrmt_test`) %>%
           pull(.data$`___tfrmt_grp_n`) %>%
           unique()
         col_info$col <- ifelse(is.null(col_info$col), first(group_str),
                                     col_info$col)
       } else if(row_grp %in% c("", "gtdefault")){

         filter_expr <- expr_to_filter(group, loc_info$group_val) %>%
           parse_expr()
         col_info$row<-.data %>%
           group_by(!!first(group)) %>%
           mutate(`___tfrmt_grp_n` = cur_group_id(),
                  `___tfrmt_test` = !!filter_expr) %>%
           filter(.data$`___tfrmt_test`) %>%
           pull(.data$`___tfrmt_grp_n`) %>%
           unique()

         lowest_grp <- group_str %in% names(loc_info$group_val) %>%
           which() %>%
           max() %>%
           group_str[.]

         col_info$col <- ifelse(is.null(col_info$col), lowest_grp,
                                col_info$col)

       } else if(row_grp=="column"){

         filter_expr <- expr_to_filter(group, loc_info$group_val) %>%
           parse_expr()
         col_info$row<-.data %>%
           # group_by(!!first(group)) %>%
           mutate(`___tfrmt_grp_n` = row_number(), #cur_group_id(),
                  `___tfrmt_test` = !!filter_expr) %>%
           filter(.data$`___tfrmt_test`) %>%
           pull(.data$`___tfrmt_grp_n`) %>%
           unique()

         lowest_grp <- group_str %in% names(loc_info$group_val) %>%
           which() %>%
           max() %>%
           group_str[.]

         col_info$col <- ifelse(is.null(col_info$col), lowest_grp,
                                col_info$col)

       }

     }
  } else{
    col_info$row <- NULL
  }
  col_info

}

#' Big N Structure
#'
#' Big N structure allows you to specify which values should become the subject
#' totals ("big N" values) and how they should be formatted in the table's
#' column labels. Values are specified by providing the value(s) of the `param`
#' column for which the values are big N's. This will remove these from the
#' body of the table and place them into columns matching the values in the
#' column column(s). The default formatting is `N = xx`, on its own line, but
#' that can be changed by providing a different `frmt()` to `n_frmt`.
#'
#' @param param_val row value(s) of the parameter column for which the values
#'   are big N's
#' @param n_frmt [frmt()] to control the formatting of the big N's
#' @param by_page Option to include different big N's for each group-defined set
#'   of pages (defined by any variables set to `".default"` in the `page_plan`).
#'   Default is `FALSE`, meaning only the overall Ns are applied
#'
#' @seealso \href{https://gsk-biostatistics.github.io/tfrmt/articles/big_ns.html}{Link to related article}
#'
#' @return big_n_structure object
#' @export
#'
big_n_structure <- function(param_val,
                            n_frmt = frmt("\nN = xx"),
                            by_page = FALSE) {

  if (!is_frmt(n_frmt) || is_frmt_combine(n_frmt) || is_frmt_when(n_frmt)) {
    stop("`n_frmt` must be given a frmt object")
  }

  structure(
    list(
      param_val = param_val,
      n_frmt = n_frmt,
      by_page = by_page
    ),
    class = c("big_n_structure", "structure")
  )
}


#' Applies the big N df to the column names
#'
#' @param col_plan_vars named string vector of columns
#' @param columns column quosure list
#' @param value value column quosure
#' @param big_n_df big_n_df
#'
#' @return named string vector
#' @noRd
apply_big_n_df <- function(big_n_df, col_plan_vars, columns, value) {

  if (!is.null(big_n_df) && nrow(big_n_df) > 0) {
    col_lab <- columns |>
      purrr::map_chr(as_label)

    data_names <- col_plan_vars |>
      purrr::map_chr(as_label) |>
      split_data_names_to_df(
        data_names = c(),
        preselected_cols = _,
        column_names = col_lab
      )

    for (i in seq_len(nrow(big_n_df))){
      big_n_i <- big_n_df |>
        dplyr::slice(i)
      data_names <- data_names |>
        dplyr::mutate(
          !!big_n_i$`__tfrmt_big_n_names__` := dplyr::if_else(
            !!parse_expr(big_n_i$exp),
            paste0(
              !!rlang::sym(big_n_i$`__tfrmt_big_n_names__`),
              dplyr::pull(big_n_i, !!value)
            ),
            !!rlang::sym(big_n_i$`__tfrmt_big_n_names__`)
          )
        )
    }

    out <- unite_df_to_data_names(
      data_names,
      preselected_cols = c(),
      column_names = col_lab
    ) |>
      purrr::map(~ char_as_quo(.x)) |>
      do.call("vars", args = _)

  } else {
    out <- col_plan_vars
  }
  out
}

#' Remove big N's from data
#'
#' @param .data ARD
#' @param param param quosure
#' @param big_n_structure big_n_structure object
#'
#' @return ARD with big n's in
#' @noRd
remove_big_ns <- function(.data, param, big_n_structure) {
  if (!is.null(big_n_structure)) {
    .data <- .data |>
      dplyr::filter(
        !(!!param) %in% big_n_structure$param_val
      )
  }
  .data
}


#' Makes the big n data from the raw ARD
#'
#' @param .data raw ARD
#' @param param param quosures
#' @param value value quosures
#' @param columns list of column quosures
#' @param big_n_structure big_n_stcuture object
#' @param mock boolean if it is T/F
#' @return tibble of the formatted big n's and expressions for where each goes
#'
#' @noRd
get_big_ns <-  function(.data, param, value, columns, big_n_structure, mock) {

  if (!is.null(big_n_structure)) {
    frmtted_vals <- .data |>
      dplyr::filter(
        !!param %in% big_n_structure$param_val
      ) |>
      apply_frmt.frmt(
        big_n_structure$n_frmt,
        .data = _,
        value = value,
        mock = mock
      )

    if (big_n_structure$by_page) {
      frmtted_vals <-  frmtted_vals |>
        dplyr::select(
          !!!columns,
          !!value,
          tidyselect::where(
            ~sum(is.na(.x)) == 0
          ),
          -!!param
        )
    } else {
      frmtted_vals <-  frmtted_vals |>
        dplyr::select(
          !!!columns,
          !!value
        )
    }

    # Test for missing big n's
    if (nrow(frmtted_vals) == 0) {
      warning(
        "Unable to add big n's as there are no matching parameter values in the given ARD"
      )
    }

    # Test for too many big n's
    grp_vars <- setdiff(names(frmtted_vals), as_label(value))
    multi_test <- frmtted_vals |>
      dplyr::group_by(
        dplyr::across(
          tidyselect::all_of(
            grp_vars
          )
        )
      ) |>
      dplyr::summarise(n = n()) |>
      dplyr::filter(n > 1)
    if (nrow(multi_test) > 0) {

      warn_df <- multi_test |>
        dplyr::select(-"n")

      warning(
        c(
          "The following columns have multiple Big N's associated with them:\n",
          warn_df
        ),
        call. = FALSE
      )
    }

    by_var <- setdiff(grp_vars, purrr::map_chr(columns, as_label))

    data_out <- frmtted_vals |>
      dplyr::mutate(
        `_tfrmt______id` = row_number()
      ) |>
      tidyr::pivot_longer(
        -c(
          "_tfrmt______id",
          !!value,
          tidyselect::all_of(by_var)
        ),
        names_to = "__tfrmt_big_n_names__",
        values_to = "__tfrmt_big_n_values__"
      ) |>
      dplyr::filter(
        !is.na(.data$`__tfrmt_big_n_values__`) &
          .data$`__tfrmt_big_n_values__` != ""
      ) |>
      dplyr::group_by(.data$`_tfrmt______id`) |>
      dplyr::mutate(
        exp = paste0(
          .data$`__tfrmt_big_n_names__`,
          "=='",
          .data$`__tfrmt_big_n_values__`,
          "'",
          collapse = "&"
        ),
        `__tfrmt_big_n_names__` = paste0(
          "__tfrmt_new_name__",
          .data$`__tfrmt_big_n_names__`
        )
      ) |>
      dplyr::slice_tail() |>
      dplyr::ungroup()|>
      dplyr::select(-"_tfrmt______id")

    if (big_n_structure$by_page ){
      if (is_empty(by_var)){
        data_out <- data_out |>
          dplyr::group_split()
      } else {
        data_out <- data_out |>
          tidyr::unite(
            "..tfrmt_big_n_order..",
            tidyselect::all_of(by_var),
            remove = FALSE
          ) |>
          dplyr::mutate(
            `..tfrmt_big_n_order..` = forcats::fct_inorder(
              .data$`..tfrmt_big_n_order..`
            )
          ) |>
          dplyr::group_by(.data$`..tfrmt_big_n_order..`) |>
          dplyr::group_split() |>
          purrr::map(
            ~dplyr::select(.x, -"..tfrmt_big_n_order..")
          )
      }
    }
  } else {
    data_out <- NULL
  }
  data_out
}

#' Table Format
#'
#' tfrmt, or "table format" is a way to pre-define the non-data components of
#' your tables, and how the data will be handled once added: i.e. title,
#' footers, headers, span headers, and cell formats. In addition, tfrmt's can be
#' layered, building from one table format to the next. For cases where only one
#' value can be used, the newly defined tfrmt accepts the latest tfrmt
#'
#' @param tfrmt_obj a tfrmt object to base this new format off of
#' @param group what are the grouping vars of the input dataset
#' @param label what is the label column of the input dataset
#' @param param what is the param column of the input dataset
#' @param value what is the value column of the input dataset
#' @param column what is the column names column in the input dataset
#' @param title title of the table
#' @param subtitle subtitle of the table
#' @param row_grp_plan plan of the row groups blocking. Takes a [row_grp_plan()]
#' @param body_plan combination and formatting of the input data. Takes a [body_plan()]
#' @param col_style_plan how to style columns including alignment (left, right, character) and width. Takes a [col_style_plan()]
#' @param col_plan a col_plan object which is used to select, rename, and nest
#'   columns. Takes a [col_plan()]
#' @param sorting_cols which columns determine sorting of output
#' @param big_n how to format subject totals ("big Ns") for inclusion in the column labels. Takes a [big_n_structure()]
#' @param footnote_plan footnotes to be added to the table. Takes a [footnote_plan()]
#' @param page_plan pagination splits to be applied to the table. Takes a [page_plan()]
#' @param ... These dots are for future extensions and must be empty.
#'
#' @return tfrmt object
#'
#' @details
#'
#'  ## NSE and Argument Evaluation
#'
#'   - tfrmt allows users to pass `vars`, `quo`, and unquoted expressions to a
#'   variety of arguments, such as `group`, `label`, `param`, `value`,
#'   `column`, and `sorting_cols`. Users accustomed to tidyverse semantics
#'   should be familiar with this behaviour. However, there is an important
#'   behaviour difference between tfrmt and normal tidyverse functions. Because
#'   the data are not a part of tfrmt, it does not know when a value being
#'   passed to it is intended to be an unquoted expression representing a column
#'   name or an object from the environment. As such, it preferentially uses the
#'   value from the environment over preserving the entry as an expression. For
#'   example, if you have an object "my_object" in your
#'   environment with the value "Hello world", and try to create a tfrmt as
#'   `tfrmt(column = my_object)`, it will take the value of "my_object" over
#'   assuming the column argument is an unquoted expression and view the entry
#'   to `column` as "Hello World". To pass "my_object" to tfrmt as a column name, use
#'   quotes around the value: `tfrmt(columnn = "my_object")`.
#'
#'   - Additionally, unquoted expressions that match `tfrmt`'s other
#'   argument names can cause unexpected results. It is recommended
#'   to put quotes around the value as such:
#'   `tfrmt(label = "group")`. In this case, the quoting will prevent `tfrmt`
#'   from assigning its `group` input value to the `label` value.
#'
#' @seealso \href{https://gsk-biostatistics.github.io/tfrmt/articles/building_blocks.html}{Link to related article}
#'
#' @section Images:
#' Here are some example outputs:
#' \if{html}{\out{
#' `r "<img src=\"https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/main/images/tfrmt-groups-three-cols-cropped.jpg\" alt =\"Example showing no groups, one group and two groups\" style=\"width:100\\%;\">"`
#' }}
#'
#' @rdname tfrmt
#'
#' @importFrom rlang caller_env
#'
#' @export
#' @examples
#'
#' tfrmt_spec <- tfrmt(
#'   label = label,
#'   column = column,
#'   param = param,
#'   value=value)
#'
#' tfrmt_spec <- tfrmt(
#'   label = label,
#'   column = column,
#'   param = param,
#'   value=value,
#' # Set the formatting for values
#'   body_plan = body_plan(
#'     frmt_structure(
#'       group_val = ".default",
#'       label_val = ".default",
#'       frmt_combine("{n} {pct}",
#'            n = frmt("xxx"),
#'            pct = frmt_when(
#'                 "==100" ~ "(100%)",
#'                 "==0" ~ "",
#'                 TRUE ~ frmt("(xx.x %)")
#'                 )
#'            )
#'     )
#'   ),
#' # Specify column styling plan
#'   col_style_plan = col_style_plan(
#'     col_style_structure(col = vars(everything()), align = c(".",","," "))
#'   ))
#'
#' tfrmt_spec <- tfrmt(
#'   group = group,
#'   label = label,
#'   column = column,
#'   param = param,
#'   value=value,
#'   sorting_cols = c(ord1, ord2),
#'   # specify value formatting
#'   body_plan = body_plan(
#'     frmt_structure(
#'       group_val = ".default",
#'       label_val = ".default",
#'       frmt_combine("{n} {pct}",
#'            n = frmt("xxx"),
#'            pct = frmt_when(
#'                 "==100" ~ "(100%)",
#'                 "==0" ~ "",
#'                 TRUE ~ frmt("(xx.x %)")
#'                 )
#'            )
#'     ),
#'     frmt_structure(
#'         group_val = ".default",
#'         label_val = "n",
#'         frmt("xxx")
#'    ),
#'     frmt_structure(
#'         group_val = ".default",
#'         label_val = c("Mean", "Median", "Min","Max"),
#'         frmt("xxx.x")
#'      ),
#'     frmt_structure(
#'         group_val = ".default",
#'         label_val = "SD",
#'         frmt("xxx.xx")
#'     ),
#'     frmt_structure(
#'         group_val = ".default",
#'         label_val = ".default",
#'         p = frmt("")
#'     ),
#'     frmt_structure(
#'         group_val = ".default",
#'         label_val = c("n","<65 yrs","<12 months","<25"),
#'         p = frmt_when(
#'             ">0.99" ~ ">0.99",
#'             "<0.001" ~ "<0.001",
#'             TRUE ~ frmt("x.xxx", missing = "")
#'         )
#'     )
#'   ),
#'   # remove extra cols
#'   col_plan = col_plan(-grp,
#'                       -starts_with("ord") ),
#'   # Specify column styling plan
#'   col_style_plan = col_style_plan(
#'     col_style_structure(col = vars(everything()), align = c(".",","," "))
#'   ),
#'
#'   # Specify row group plan
#'   row_grp_plan = row_grp_plan(
#'     row_grp_structure(
#'          group_val = ".default",
#'          element_block(post_space = " ")
#'     ),
#'     label_loc = element_row_grp_loc(location = "column")
#'   )
#'
#' )
#'
#'
tfrmt <- function(
  tfrmt_obj,
  group = vars(),
  label = quo(),
  param = quo(),
  value = quo(),
  column = vars(),
  title,
  subtitle,
  row_grp_plan, #the style between blocking
  body_plan,
  col_style_plan,
  col_plan,
  sorting_cols,
  big_n,
  footnote_plan,
  page_plan,
  ...
){

  tfrmt_el <- tfrmt_find_args(..., env = environment(), parent_env = caller_env())

  new_tfrmt <- structure(
    tfrmt_el,
    class = c("tfrmt")
  )

  # check non-null plan parameters are supplied with plan function
  check_plan(new_tfrmt, "body_plan")
  check_plan(new_tfrmt, "col_plan")
  check_plan(new_tfrmt, "col_style_plan")
  check_plan(new_tfrmt, "row_grp_plan")
  check_plan(new_tfrmt, "footnote_plan")
  check_plan(new_tfrmt, "page_plan")

  # check non-null big_n is supplied a big_n_structure
  check_big_n(new_tfrmt)


  if(!missing(tfrmt_obj)){
    new_tfrmt <- layer_tfrmt(
      tfrmt_obj,
      new_tfrmt
    )
  }


  ## check to confirm user has not defined multiple columns and
  ## any span_structures in col_plan
  check_column_and_col_plan(new_tfrmt)
  check_group_var_consistency(new_tfrmt)
  check_col_style_row_grp_consistency(new_tfrmt)
  check_footnote_plan(new_tfrmt)

  new_tfrmt

}

is_tfrmt <- function(x){
  inherits(x, "tfrmt")
}

tfrmt_find_args <- function(..., env = parent.frame(), parent_env = parent.env(env)){

  ## get args of parent function
  arg_parent <- names(formals(sys.function(sys.parent(1))))
  ## don't try to get the tftmt obj
  args <- setdiff(arg_parent,c("tfrmt_obj","..."))

  ## get the values from the parent env. turn the
  ## as_var_args call into vars
  ## and as_quo_args into length one quo's
  vals <- quo_get(
    args,
    as_var_args = c("group","column","sorting_cols"),
    as_quo_args = c("label","param","value"),
    envir = env,
    parent_env = parent_env
  )

  ## remove the "missing" values from vals
  vals <- vals[!sapply(vals, is_missing)]

  dot_subs <- as.list(substitute(substitute(...)))[-1]
  for(dot_name in names(dot_subs)){
    compare_dot_args_against_formals(dot_arg = dot_name, formals = args)
    vals[[dot_name]] <- tryCatch(
      eval(dot_subs[[dot_name]], envir = env, enclos = parent_env),
      error = function(e){
        stop(e$message,call. = FALSE)
      }
    )
  }

  vals
}

#' @importFrom rlang abort frame_call is_quosure
#' @importFrom dplyr vars
#' @importFrom purrr safely
quo_get <- function(args, as_var_args = c(), as_quo_args = c(), envir = parent.frame(), parent_env = parent.env(envir), allow_tidy_select = FALSE){

  arg_set <- lapply(args, function(arg){


    ## try to get arg call
    arg_call <- do.call('substitute',list(as.symbol(arg)), envir = envir)

    if(missing(arg_call)){

      ## args not defined can quietly return empty expressions.
      return(quote(expr = ))

    }else{

      if(identical(arg_call, quo()) | identical(arg_call, vars())){
        return(arg_call)
      }

      # don't try to eval quosures if it is intended to be a quosure
      if(is_quosure(arg_call) & arg %in% c(as_quo_args)){
        arg_call_results <- list(result = arg_call, error = NULL)
      }else{
        # try to safely evaluate arg call
        arg_call_results_envir <-  safely(eval_tidy)(arg_call, env = envir)
        arg_call_results_parent_env <-  safely(eval_tidy)(arg_call, env = parent_env)


        if(is.null(arg_call_results_parent_env$error)){
          arg_call_results <- arg_call_results_parent_env
        }else{
          arg_call_results <- arg_call_results_envir
        }
      }


      ## if expression evaluation was successful, move forward to confirming is correct class and returning the value
      if(is.null(arg_call_results$error)){

        if(arg %in% c(as_var_args, as_quo_args)){
          ## for arg_var_args, we expect not a function. this means arguments can be
          ## entered such as `col`. convert into final forms respectively
          if(!(is.function(arg_call_results$result) | is_basic_list(arg_call_results$result))){
            if(arg %in% as_var_args){
              return(as_vars(arg_call_results$result))
            }else{
              return(as_length_one_quo(arg_call_results$result, arg = as.character(arg)))
            }
          }
        }else{
          ## return value as normal if not a var or quo arg
          return(arg_call_results$result)
        }
      }

      ## Now assume expression evaluation was a failure/returned incorrect value

      ## if not a var or quo and failed, return informative error message
      if(arg %in% c(as_var_args, as_quo_args)){

        arg_call <- trim_vars_quo_c(arg_call)

        ## check if argcall is tidyselect call, give feedback that is invalid if so
        if(any(map_lgl(arg_call,is_valid_tidyselect_call))){
          if(!allow_tidy_select){
            abort(
              message = "Tidyselect selection helpers are not acceptable to use in this context. Please provide a specific column to use.",
              class = "invalid_tidyselect_use"
            )
          }
        }

        if(arg %in% as_var_args){
          check_var_arg_call_valid(arg_call, arg, allow_tidy_select = allow_tidy_select)
          arg_val <- as_vars(do.call('vars',arg_call, envir = envir))

        }else{
          arg_val <-as_length_one_quo(do.call('vars', arg_call, envir = envir), arg = as.character(arg))
        }

        return(arg_val)

      }else{
        abort(
          paste0(
            "Error in evaluating argument `",
            arg,
            "`:\n",
            paste0(" ", arg_call_results$error, collapse = "")
          ),
          call = frame_call(frame = envir)
        )
      }
    }

  })

  names(arg_set) <- args
  arg_set
}


check_var_arg_call_valid <- function(var_list, arg, allow_tidy_select = FALSE){

  var_list_is_name <- sapply(var_list, is.name)
  var_list_is_tidyselect <- sapply(var_list, is_valid_tidyselect_call)

  if(!all(var_list_is_name | (var_list_is_tidyselect & allow_tidy_select))){

    new_arg_call <- paste0(
      "vars(",paste(sapply(var_list, as.character),collapse = ","),")"
    )

    abort(
      paste0(
        "Entries for `",
        arg,
        "` argument must be vars(), a character vector, or unquoted column name.\n",
        "  Consider updating the argument input to `",
        arg,
        "` to:\n\t",
        new_arg_call
      ),
      class = c("group_vars_error")
    )
  }
}


trim_vars_quo_c <- function(x){
  x_list <- as.list(x)
  if(as.character(x_list[[1]]) %in% c("c","quo","vars")){
    x_list[-1]
  }else{
    list(x)
  }
}



#' @importFrom rlang is_quosures
is_basic_list <- function(x){
  is.list(x) & !is_quosures(x)
}

is_missing <- function(x){
  identical(x, quote(expr = ))
}

as_length_one_quo <- function(x, ...){
  UseMethod("as_length_one_quo",x)
}

#' @export
#' @keywords internal
as_length_one_quo.quosure <- function(x, ...){
  x
}

#' @export
#' @keywords internal
#' @importFrom rlang warn
as_length_one_quo.quosures <- function(x, ..., arg = NULL){
  if(length(x) == 0){
    quo()
  }else{
    if(length(x) > 1){
      warn(
        paste0(
          "Passed more than one quosure to the argument `",
          arg,
          "`. Selecting the first entry."
        ),
        class = "quo_greater_length_one"
      )
    }
    x[[1]]
  }
}

#' @export
#' @keywords internal
as_length_one_quo.character <- function(x, ...){
  quo(!!sym(x))
}

as_vars <-  function(x){
  UseMethod("as_vars",x)
}

#' @export
#' @keywords internal
as_vars.quosures <- function(x){
  x
}

#' @export
#' @keywords internal
as_vars.quosure <- function(x){
  vars(!!x)
}

#' @export
#' @keywords internal
as_vars.character <- function(x){
  do.call(vars,lapply(x,function(x){ quo(!!sym(x))}))
}

#' @importFrom rlang inform
compare_dot_args_against_formals <- function(dot_arg, formals){
  arg_message <- paste0("Argument '",dot_arg,"' passed to tfrmt is not a recognized argument.")
  fuzzy_arg_match <- agrep(dot_arg, formals, ignore.case = TRUE, value = TRUE, max.distance = .25)
  if(length(fuzzy_arg_match)){
    arg_message <- paste0(arg_message,"\n","Did you intend to use the argument `",fuzzy_arg_match[[1]],"`?")
  }
  inform(arg_message, class = "tfrmt_unrecognized_argument_inform")
}

test_that("turn length one unquoted vals into args", {

  temp_function <- function(x){
    quo_get(
      args = "x",
      as_var_args = "x"
    )
  }

  expect_equal(
    temp_function(bare_arg_val)$x,
    vars(bare_arg_val),
    ignore_attr = TRUE
  )

})

test_that("turn length 2 unquoted vals into args", {

  temp_function <- function(x){
    quo_get(
      args = "x",
      as_var_args = "x"
    )
  }

  expect_equal(
    temp_function(c(bare_arg_val1, bare_arg_val2))$x,
    vars(bare_arg_val1, bare_arg_val2),
    ignore_attr = TRUE
  )

})

test_that("turn length one unquoted quo into args", {

  temp_function <- function(x){
    quo_get(
      args = "x",
      as_quo_args = "x"
    )
  }

  expect_equal(
    temp_function(bare_arg_val)$x,
    quo(bare_arg_val),
    ignore_attr = TRUE
  )

})

test_that("turn length two unquoted quo into args", {

  temp_function <- function(x){
    quo_get(
      args = "x",
      as_quo_args = "x"
    )
  }

  warning_res <- capture_warnings({
    tempfunc_res <-
      temp_function(c(bare_arg_val1, bare_arg_val2))
  })

  expect_equal(
    tempfunc_res$x,
    quo(bare_arg_val1),
    ignore_attr = TRUE
  )

})


test_that("throw error when input argument errors out", {

  temp_function_1 <- function(x, ...){
    val <- temp_function_2(...)
    val
  }

  temp_function_2 <- function(..., env = parent.frame()){

    arg_parent <- names(formals(sys.function(sys.parent(1))))

    args <- setdiff(arg_parent,c("..."))

    val <- quo_get(
      args,
      envir = env
    )

    val
  }

  fail_function <- function(...){

    dot_list <- as.list(substitute(substitute(...)))[-1]

    if(length(dot_list) == 0){
      stop("I HAVE FAILED")
    }else{
      dot_list
    }
  }

  expect_silent(
    temp_function_1(x = fail_function("test value"))
  )

  expect_error(
    temp_function_1(x = fail_function()),
    "Error in evaluating argument `x`:\n Error in fail_function(): I HAVE FAILED",
    fixed = TRUE
  )

})

# strip_env_label() helper works

    Code
      cat(strip_env_label(
        "<list_of<quosure>>\n\n[[1]]\n<quosure>\nexpr: ^grp2\nenv:  0x72db165b8\n\n[[2]]\n<quosure>\nexpr: ^lbl\nenv:  0x72db31118"))
    Output
      <list_of<quosure>>
      
      [[1]]
      <quosure>
      expr: ^grp2
      env:  <env-address>
      
      [[2]]
      <quosure>
      expr: ^lbl
      env:  <env-address>

# shuffle/trim works

    Code
      ard_simple_shuffled
    Output
                          ARM         AGE    context stat_variable stat_name
      1               Placebo Overall AGE continuous           AGE         N
      2               Placebo Overall AGE continuous           AGE      mean
      3               Placebo Overall AGE continuous           AGE        sd
      4               Placebo Overall AGE continuous           AGE    median
      5               Placebo Overall AGE continuous           AGE       p25
      6               Placebo Overall AGE continuous           AGE       p75
      7               Placebo Overall AGE continuous           AGE       min
      8               Placebo Overall AGE continuous           AGE       max
      9  Xanomeline High Dose Overall AGE continuous           AGE         N
      10 Xanomeline High Dose Overall AGE continuous           AGE      mean
      11 Xanomeline High Dose Overall AGE continuous           AGE        sd
      12 Xanomeline High Dose Overall AGE continuous           AGE    median
      13 Xanomeline High Dose Overall AGE continuous           AGE       p25
      14 Xanomeline High Dose Overall AGE continuous           AGE       p75
      15 Xanomeline High Dose Overall AGE continuous           AGE       min
      16 Xanomeline High Dose Overall AGE continuous           AGE       max
      17  Xanomeline Low Dose Overall AGE continuous           AGE         N
      18  Xanomeline Low Dose Overall AGE continuous           AGE      mean
      19  Xanomeline Low Dose Overall AGE continuous           AGE        sd
      20  Xanomeline Low Dose Overall AGE continuous           AGE    median
      21  Xanomeline Low Dose Overall AGE continuous           AGE       p25
      22  Xanomeline Low Dose Overall AGE continuous           AGE       p75
      23  Xanomeline Low Dose Overall AGE continuous           AGE       min
      24  Xanomeline Low Dose Overall AGE continuous           AGE       max
         stat_label      stat fmt_fun warning error
      1           N 86.000000       0    NULL  NULL
      2        Mean 75.209302       1    NULL  NULL
      3          SD  8.590167       1    NULL  NULL
      4      Median 76.000000       1    NULL  NULL
      5          Q1 69.000000       1    NULL  NULL
      6          Q3 82.000000       1    NULL  NULL
      7         Min 52.000000       1    NULL  NULL
      8         Max 89.000000       1    NULL  NULL
      9           N 84.000000       0    NULL  NULL
      10       Mean 74.380952       1    NULL  NULL
      11         SD  7.886094       1    NULL  NULL
      12     Median 76.000000       1    NULL  NULL
      13         Q1 70.500000       1    NULL  NULL
      14         Q3 80.000000       1    NULL  NULL
      15        Min 56.000000       1    NULL  NULL
      16        Max 88.000000       1    NULL  NULL
      17          N 84.000000       0    NULL  NULL
      18       Mean 75.666667       1    NULL  NULL
      19         SD  8.286051       1    NULL  NULL
      20     Median 77.500000       1    NULL  NULL
      21         Q1 71.000000       1    NULL  NULL
      22         Q3 82.000000       1    NULL  NULL
      23        Min 51.000000       1    NULL  NULL
      24        Max 88.000000       1    NULL  NULL

---

    Code
      ard_shuffled[1:5, ]
    Output
                         ARM  AGE AGEGR1     context stat_variable stat_name
      1              Placebo <NA>   <NA> categorical           ARM         n
      2              Placebo <NA>   <NA> categorical           ARM         N
      3              Placebo <NA>   <NA> categorical           ARM         p
      4 Xanomeline High Dose <NA>   <NA> categorical           ARM         n
      5 Xanomeline High Dose <NA>   <NA> categorical           ARM         N
        stat_label        stat
      1          n  86.0000000
      2          N 254.0000000
      3          %   0.3385827
      4          n  84.0000000
      5          N 254.0000000

---

    Code
      ard_shuff_trim[1:5, ]
    Output
                         ARM  AGE AGEGR1     context stat_variable stat_name
      1              Placebo <NA>   <NA> categorical           ARM         n
      2              Placebo <NA>   <NA> categorical           ARM         N
      3              Placebo <NA>   <NA> categorical           ARM         p
      4 Xanomeline High Dose <NA>   <NA> categorical           ARM         n
      5 Xanomeline High Dose <NA>   <NA> categorical           ARM         N
        stat_label        stat
      1          n  86.0000000
      2          N 254.0000000
      3          %   0.3385827
      4          n  84.0000000
      5          N 254.0000000

# shuffle_card notifies user about warnings/errors before dropping

    Code
      shuffle_card(cards::ard_continuous(cards::ADSL, variables = AGEGR1))
    Message
      "warning" column contains messages that will be removed.
    Output
      # A tibble: 8 x 6
        AGEGR1         context    stat_variable stat_name stat_label stat 
        <chr>          <chr>      <chr>         <chr>     <chr>      <chr>
      1 Overall AGEGR1 continuous AGEGR1        N         N          254  
      2 Overall AGEGR1 continuous AGEGR1        mean      Mean       <NA> 
      3 Overall AGEGR1 continuous AGEGR1        sd        SD         <NA> 
      4 Overall AGEGR1 continuous AGEGR1        median    Median     <NA> 
      5 Overall AGEGR1 continuous AGEGR1        p25       Q1         65-80
      6 Overall AGEGR1 continuous AGEGR1        p75       Q3         >80  
      7 Overall AGEGR1 continuous AGEGR1        min       Min        65-80
      8 Overall AGEGR1 continuous AGEGR1        max       Max        >80  

# shuffle_card correctly handles a combined ARD when by is explicitly supplied

    Code
      shuffle_card(dplyr::filter(cards::bind_ard(cards::ard_continuous(cards::ADSL,
      by = "ARM", variables = "AGE", statistic = ~ cards::continuous_summary_fns(
        "mean")), dplyr::tibble(group1 = "ARM", variable = "AGE", stat_name = "p",
        stat_label = "p", stat = list(0.05))), dplyr::row_number() <= 5L))
    Message
      The `by` argument was not supplied and cannot reliably be inferred from objects of class <bind_ard>.
      * A `by` value of "ARM" was found in the input object's attributes.
      * To use it as a grouping variable, pass it explicitly: `shuffle_card(by = "ARM")`.
    Output
      # A tibble: 4 x 7
        ARM                  AGE      context stat_variable stat_name stat_label  stat
        <chr>                <chr>    <chr>   <chr>         <chr>     <chr>      <dbl>
      1 Placebo              Overall~ contin~ AGE           mean      Mean       75.2 
      2 Xanomeline High Dose Overall~ contin~ AGE           mean      Mean       74.4 
      3 Xanomeline Low Dose  Overall~ contin~ AGE           mean      Mean       75.7 
      4 Overall ARM          Overall~ <NA>    AGE           p         p           0.05

---

    Code
      shuffle_card(dplyr::filter(cards::bind_ard(cards::ard_continuous(cards::ADSL,
      variables = "AGE", statistic = ~ cards::continuous_summary_fns("mean")), dplyr::tibble(
        group1 = "ARM", variable = "AGE", stat_name = "p", stat_label = "p", stat = list(
          0.05))), dplyr::row_number() <= 5L), by = "ARM")
    Output
      # A tibble: 2 x 7
        ARM         AGE         context    stat_variable stat_name stat_label  stat
        <chr>       <chr>       <chr>      <chr>         <chr>     <chr>      <dbl>
      1 <NA>        Overall AGE continuous AGE           mean      Mean       75.1 
      2 Overall ARM Overall AGE <NA>       AGE           p         p           0.05

---

    Code
      as.data.frame(shuffle_card(cards::bind_ard(dplyr::slice(cards::ard_categorical(
        cards::ADSL, by = ARM, variables = AGEGR1), 1), dplyr::slice(cards::ard_categorical(
        cards::ADSL, variables = AGEGR1), 1), dplyr::slice(cards::ard_continuous(
        cards::ADSL, by = SEX, variables = AGE), 1), dplyr::slice(cards::ard_continuous(
        cards::ADSL, variables = AGE), 1)), by = c("ARM", "SEX")))
    Output
                ARM         SEX AGEGR1         AGE     context stat_variable
      1     Placebo        <NA>  65-80        <NA> categorical        AGEGR1
      2 Overall ARM        <NA>  65-80        <NA> categorical        AGEGR1
      3        <NA> Overall SEX   <NA> Overall AGE  continuous           AGE
      4        <NA>           F   <NA> Overall AGE  continuous           AGE
        stat_name stat_label stat
      1         n          n   42
      2         n          n  144
      3         N          N  254
      4         N          N  143

---

    Code
      as.data.frame(shuffle_card(cards::bind_ard(dplyr::slice(cards::ard_categorical(
        cards::ADSL, by = ARM, variables = AGEGR1), 1), dplyr::slice(cards::ard_categorical(
        cards::ADSL, variables = AGEGR1), 1), dplyr::slice(cards::ard_continuous(
        cards::ADSL, by = SEX, variables = AGE), 1), dplyr::slice(cards::ard_continuous(
        cards::ADSL, variables = AGE), 1)), by = c("ARM", "SEX"), fill_overall = "{colname}"))
    Output
            ARM  SEX AGEGR1  AGE     context stat_variable stat_name stat_label stat
      1 Placebo <NA>  65-80 <NA> categorical        AGEGR1         n          n   42
      2     ARM <NA>  65-80 <NA> categorical        AGEGR1         n          n  144
      3    <NA>  SEX   <NA>  AGE  continuous           AGE         N          N  254
      4    <NA>    F   <NA>  AGE  continuous           AGE         N          N  143

---

    Code
      shuffle_card(cards::bind_ard(dplyr::slice(cards::ard_categorical(cards::ADSL,
      by = c(ARM, SEX), variables = AGEGR1), 1), dplyr::slice(cards::ard_categorical(
        cards::ADSL, by = SEX, variables = AGEGR1), 1), dplyr::slice(cards::ard_categorical(
        cards::ADSL, variables = AGEGR1), 1)), by = c("ARM", "SEX"))
    Output
      # A tibble: 3 x 8
        ARM         SEX        AGEGR1 context stat_variable stat_name stat_label  stat
        <chr>       <chr>      <chr>  <chr>   <chr>         <chr>     <chr>      <int>
      1 Placebo     F          65-80  catego~ AGEGR1        n         n             22
      2 Overall ARM F          65-80  catego~ AGEGR1        n         n             78
      3 Overall ARM Overall S~ 65-80  catego~ AGEGR1        n         n            144

---

    Code
      shuffle_card(cards::bind_ard(dplyr::slice(cards::ard_categorical(cards::ADSL,
      by = c(ARM, SEX), variables = AGEGR1), 1), dplyr::slice(cards::ard_categorical(
        cards::ADSL, by = SEX, variables = AGEGR1), 1), dplyr::slice(cards::ard_categorical(
        cards::ADSL, variables = AGEGR1), 1)), by = c("ARM", "SEX"), fill_overall = "total")
    Output
      # A tibble: 3 x 8
        ARM     SEX   AGEGR1 context     stat_variable stat_name stat_label  stat
        <chr>   <chr> <chr>  <chr>       <chr>         <chr>     <chr>      <int>
      1 Placebo F     65-80  categorical AGEGR1        n         n             22
      2 total   F     65-80  categorical AGEGR1        n         n             78
      3 total   total 65-80  categorical AGEGR1        n         n            144

---

    Code
      shuffle_card(cards::bind_ard(dplyr::slice(cards::ard_categorical(cards::ADSL,
      by = c(ARM, SEX), variables = AGEGR1), 1), dplyr::slice(cards::ard_categorical(
        cards::ADSL, by = SEX, variables = AGEGR1), 1), dplyr::slice(cards::ard_categorical(
        cards::ADSL, variables = AGEGR1), 1)), by = c("ARM", "SEX"), fill_overall = NA)
    Output
      # A tibble: 3 x 8
        ARM     SEX   AGEGR1 context     stat_variable stat_name stat_label  stat
        <chr>   <chr> <chr>  <chr>       <chr>         <chr>     <chr>      <int>
      1 Placebo F     65-80  categorical AGEGR1        n         n             22
      2 <NA>    F     65-80  categorical AGEGR1        n         n             78
      3 <NA>    <NA>  65-80  categorical AGEGR1        n         n            144

---

    Code
      shuffle_card(cards::bind_ard(cards::ard_continuous(adsl_new, variables = "AGE",
        statistic = ~ cards::continuous_summary_fns("mean")), cards::ard_continuous(
        adsl_new, by = "ARM", variables = "AGE", statistic = ~ cards::continuous_summary_fns(
          "mean"))), by = "ARM")
    Message
      i "Overall ARM" already exists in the `ARM` column. Using "Overall ARM.1".
    Output
      # A tibble: 4 x 7
        ARM                  AGE      context stat_variable stat_name stat_label  stat
        <chr>                <chr>    <chr>   <chr>         <chr>     <chr>      <dbl>
      1 Overall ARM.1        Overall~ contin~ AGE           mean      Mean        75.1
      2 Overall ARM          Overall~ contin~ AGE           mean      Mean        75.2
      3 Xanomeline High Dose Overall~ contin~ AGE           mean      Mean        74.4
      4 Xanomeline Low Dose  Overall~ contin~ AGE           mean      Mean        75.7

# shuffle_card fills missing group levels if the group is meaningful for cardx output

    Code
      as.data.frame(shuffle_card(ard_cardx))
    Output
                ARM         SEX         AGEGR1          context stat_variable
      1 Overall ARM        <NA> Overall AGEGR1 stats_chisq_test        AGEGR1
      2 Overall ARM        <NA> Overall AGEGR1 stats_chisq_test        AGEGR1
      3        <NA> Overall SEX Overall AGEGR1 stats_chisq_test        AGEGR1
      4        <NA> Overall SEX Overall AGEGR1 stats_chisq_test        AGEGR1
        stat_name          stat_label       stat
      1 statistic X-squared Statistic 5.07944167
      2   p.value             p-value 0.07888842
      3 statistic X-squared Statistic 1.03944200
      4   p.value             p-value 0.59468644

# shuffle_card() messages about 'Overall <var>' or 'Any <var>'

    Code
      dplyr::mutate(test_data, dplyr::across(ARM:TRTA, ~ .derive_overall_labels(.x,
        fill_overall = "Overall {colname}", fill_hierarchical_overall = "Any {colname}")))
    Message
      i "Overall ARM" already exists in the `ARM` column. Using "Overall ARM.1".
    Output
      # A tibble: 5 x 2
        ARM           TRTA    
        <chr>         <chr>   
      1 Overall ARM.1 <NA>    
      2 Overall ARM   <NA>    
      3 <NA>          Any TRTA
      4 BB            C       
      5 <NA>          C       

---

    Code
      shuffled_ard <- shuffle_card(ard)
    Message
      i "Overall TRTA" already exists in the `TRTA` column. Using "Overall TRTA.1".
      i "Any AESOC" already exists in the `AESOC` column. Using"Any AESOC.1".

# shuffle_card() preserves the attributes of a `card` object

    Code
      shuffled_ard <- shuffle_card(ard)
    Message
      The `by` argument was not supplied and cannot reliably be inferred from objects of class <bind_ard>.
      * A `by` value of "ARM" was found in the input object's attributes.
      * To use it as a grouping variable, pass it explicitly: `shuffle_card(by = "ARM")`.

---

    Code
      shuffle_card(ard_no_by_attr)
    Message
      The `by` argument was not supplied and cannot reliably be inferred from objects of class <bind_ard>.
      * If you want to use a grouping variable, pass it explicitly via the `by` argument.
    Output
      # A tibble: 15 x 7
         AGEGR1 SEX   context  stat_variable stat_name stat_label    stat
         <chr>  <chr> <chr>    <chr>         <chr>     <chr>        <dbl>
       1 65-80  <NA>  tabulate AGEGR1        n         n          144    
       2 65-80  <NA>  tabulate AGEGR1        N         N          254    
       3 65-80  <NA>  tabulate AGEGR1        p         %            0.567
       4 <65    <NA>  tabulate AGEGR1        n         n           33    
       5 <65    <NA>  tabulate AGEGR1        N         N          254    
       6 <65    <NA>  tabulate AGEGR1        p         %            0.130
       7 >80    <NA>  tabulate AGEGR1        n         n           77    
       8 >80    <NA>  tabulate AGEGR1        N         N          254    
       9 >80    <NA>  tabulate AGEGR1        p         %            0.303
      10 <NA>   F     tabulate SEX           n         n          143    
      11 <NA>   F     tabulate SEX           N         N          254    
      12 <NA>   F     tabulate SEX           p         %            0.563
      13 <NA>   M     tabulate SEX           n         n          111    
      14 <NA>   M     tabulate SEX           N         N          254    
      15 <NA>   M     tabulate SEX           p         %            0.437

# json basic tfrmt

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": []
      } 

---

    Code
      .
    Output
      {
        "group": ["group"],
        "label": ["label"],
        "param": ["parm"],
        "value": ["val"],
        "column": ["span2", "span1", "my_col"]
      } 

# json Titles and subtitle

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "title": ["Test Title"],
        "subtitle": ["Also a test"]
      } 

# json row group plans

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "row_grp_plan": {
          "struct_list": [
            {
              "group_val": ["A", "C"],
              "block_to_apply": {
                "post_space": ["---"],
                "fill": [true]
              }
            },
            {
              "group_val": ["B"],
              "block_to_apply": {
                "post_space": [" "],
                "fill": [true]
              }
            }
          ],
          "label_loc": {
            "location": ["column"],
            "indent": ["  "]
          }
        }
      } 

---

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "row_grp_plan": {
          "struct_list": [
            {
              "group_val": {
                "grp1": ["A"],
                "grp2": ["b"]
              },
              "block_to_apply": {
                "post_space": [" "],
                "fill": [true]
              }
            }
          ],
          "label_loc": {
            "location": ["spanning"],
            "indent": ["  "]
          }
        }
      } 

# json body plan

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "body_plan": [
          {
            "group_val": ["group1"],
            "label_val": [".default"],
            "param_val": [".default"],
            "frmt": {
              "expression": ["XXX"],
              "missing": {},
              "scientific": {},
              "transform": {}
            }
          }
        ]
      } 

---

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "body_plan": [
          {
            "group_val": {
              "grp_col1": ["group1"],
              "grp_col2": ["subgroup"]
            },
            "label_val": [".default"],
            "param_val": ["test"],
            "frmt": {
              "expression": ["XXX"],
              "missing": {},
              "scientific": {},
              "transform": {}
            }
          }
        ]
      } 

---

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "body_plan": [
          {
            "group_val": [".default"],
            "label_val": [".default"],
            "param_val": [".default"],
            "frmt_when": {
              "frmt_ls": {
                ">3": {
                  "frmt": {
                    "expression": ["(X.X%)"],
                    "missing": {},
                    "scientific": {},
                    "transform": {}
                  }
                },
                "<=3": {
                  "frmt": {
                    "expression": ["Undetectable"],
                    "missing": {},
                    "scientific": {},
                    "transform": {}
                  }
                }
              },
              "missing": {}
            }
          }
        ]
      } 

---

    Code
      frmt_when_scalar %>% as_json()
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "body_plan": [
          {
            "group_val": [".default"],
            "label_val": [".default"],
            "param_val": [".default"],
            "frmt_when": {
              "frmt_ls": {
                "<1": ["<1"],
                "==100": [""],
                "==0": [""],
                "TRUE": {
                  "frmt": {
                    "expression": ["(xx)"],
                    "missing": {},
                    "scientific": {},
                    "transform": {}
                  }
                }
              },
              "missing": {}
            }
          }
        ]
      } 

---

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "body_plan": [
          {
            "group_val": [".default"],
            "label_val": [".default"],
            "param_val": ["param1", "param2"],
            "frmt_combine": {
              "expression": ["{param1} {param2}"],
              "frmt_ls": {
                "param1": {
                  "frmt": {
                    "expression": ["XXX %"],
                    "missing": {},
                    "scientific": {},
                    "transform": {}
                  }
                },
                "param2": {
                  "frmt": {
                    "expression": ["XX.XXX"],
                    "missing": {},
                    "scientific": {},
                    "transform": {}
                  }
                }
              },
              "missing": {}
            }
          }
        ]
      } 

---

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "body_plan": [
          {
            "group_val": ["group1"],
            "label_val": [".default"],
            "param_val": [".default"],
            "frmt": {
              "expression": ["xx.xx"],
              "missing": {},
              "scientific": ["x10^xx"],
              "transform": {}
            }
          }
        ]
      } 

---

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "body_plan": [
          {
            "group_val": [".default"],
            "label_val": [".default"],
            "param_val": ["param1", "param2"],
            "frmt_combine": {
              "expression": ["{param1} {param2}"],
              "frmt_ls": {
                "param1": {
                  "frmt": {
                    "expression": ["XXX %"],
                    "missing": {},
                    "scientific": {},
                    "transform": {}
                  }
                },
                "param2": {
                  "frmt_when": {
                    "frmt_ls": {
                      ">3": {
                        "frmt": {
                          "expression": ["(X.X%)"],
                          "missing": {},
                          "scientific": {},
                          "transform": {}
                        }
                      },
                      "<=3": {
                        "frmt": {
                          "expression": ["Undetectable"],
                          "missing": {},
                          "scientific": {},
                          "transform": {}
                        }
                      }
                    },
                    "missing": {}
                  }
                }
              },
              "missing": [""]
            }
          },
          {
            "group_val": ["test1"],
            "label_val": [".default"],
            "param_val": ["foo"],
            "frmt": {
              "expression": ["xx.x"],
              "missing": ["--"],
              "scientific": {},
              "transform": {}
            }
          },
          {
            "group_val": [".default"],
            "label_val": [".default"],
            "param_val": [".default"],
            "frmt_when": {
              "frmt_ls": {
                ">0.4": {
                  "frmt": {
                    "expression": ["(X.X%)"],
                    "missing": [""],
                    "scientific": {},
                    "transform": {}
                  }
                },
                "<=0.4": {
                  "frmt_combine": {
                    "expression": ["[{par2m1}-{param2}]"],
                    "frmt_ls": {
                      "par2m1": {
                        "frmt": {
                          "expression": ["XXX"],
                          "missing": {},
                          "scientific": {},
                          "transform": {}
                        }
                      },
                      "param2": {
                        "frmt": {
                          "expression": ["XXX"],
                          "missing": {},
                          "scientific": {},
                          "transform": {}
                        }
                      }
                    },
                    "missing": {}
                  }
                }
              },
              "missing": {}
            }
          }
        ]
      } 

# json big n

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "big_n": {
          "param_val": ["bigN"],
          "n_frmt": {
            "expression": ["\nN = xx"],
            "missing": {},
            "scientific": {},
            "transform": {}
          },
          "by_page": [false]
        }
      } 

# json footnote plan

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "footnote_plan": {
          "struct_list": [
            {
              "column_val": {},
              "group_val": {},
              "label_val": {},
              "footnote_text": ["Source Note"]
            },
            {
              "column_val": ["PL"],
              "group_val": {},
              "label_val": {},
              "footnote_text": ["Placebo"]
            }
          ],
          "marks": ["standard"],
          "order": ["marks_first"]
        }
      } 

---

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "footnote_plan": {
          "struct_list": [
            {
              "column_val": {
                "column": ["T1", "T2", "T1&T2"]
              },
              "group_val": {},
              "label_val": {},
              "footnote_text": ["All Treatments"]
            }
          ],
          "marks": ["numbers"],
          "order": ["marks_first"]
        }
      } 

---

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "footnote_plan": {
          "struct_list": [
            {
              "column_val": {},
              "group_val": ["group 1"],
              "label_val": ["label 1"],
              "footnote_text": ["Footnote goes here"]
            }
          ],
          "marks": ["numbers"],
          "order": ["marks_first"]
        }
      } 

---

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "footnote_plan": {
          "struct_list": [
            {
              "column_val": {
                "span": ["Treatment"],
                "column": ["T1&T2"]
              },
              "group_val": {},
              "label_val": {},
              "footnote_text": ["Footnote goes here"]
            }
          ],
          "marks": ["numbers"],
          "order": ["marks_first"]
        }
      } 

# json col_plan

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "col_plan": {
          "col_plan": {
            "dots": [
              ["col1"],
              ["col2"],
              ["col3"]
            ],
            ".drop": [false]
          }
        }
      } 

---

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "col_plan": {
          "col_plan": {
            "dots": {
              "foo": ["col1"],
              "": ["col2"],
              "": ["col3"]
            },
            ".drop": [false]
          }
        }
      } 

---

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "col_plan": {
          "col_plan": {
            "dots": [
              ["starts_with(\"col\")"]
            ],
            ".drop": [false]
          }
        }
      } 

---

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": ["span1", "col"],
        "col_plan": {
          "col_plan": {
            "dots": [
              {
                "span_structure": {
                  "span1": ["col 4"]
                }
              }
            ],
            ".drop": [false]
          }
        }
      } 

---

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": ["span2", "span1", "my_col"],
        "col_plan": {
          "col_plan": {
            "dots": {
              "": ["group"],
              "": ["label"],
              "": {
                "span_structure": {
                  "span1": ["col 4"]
                }
              },
              "": {
                "span_structure": {
                  "span1": ["cols 1,2"],
                  "my_col": ["col2", "col1"]
                }
              },
              "": {
                "span_structure": {
                  "span2": ["Top Label Level 1"],
                  "span1": ["Second Label Level 1.1"],
                  "my_col": ["col_3", "col_4"]
                }
              },
              "": {
                "span_structure": {
                  "span2": ["Top Label Level 1"],
                  "span1": ["col2", "col1"],
                  "my_col": ["starts_with(\"B\")"]
                }
              },
              "": {
                "span_structure": {
                  "span2": ["Top Label Level 1"],
                  "my_col": ["col_5"]
                }
              },
              "": {
                "span_structure": {
                  "span2": ["Top Label Level 2"],
                  "my_col": ["col_6", "col_7"]
                }
              },
              "": ["everything()"],
              "new_col_3": ["mycol3"],
              "": ["-mycol5"]
            },
            ".drop": [false]
          }
        }
      } 

# json col_style_plan

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "col_style_plan": [
          {
            "cols": [
              ["my_var"]
            ],
            "align": ["left"],
            "type": ["char"],
            "width": [100]
          },
          {
            "cols": [
              ["four"]
            ],
            "align": ["right"],
            "type": ["char"],
            "width": {}
          },
          {
            "cols": [
              ["two"],
              ["three"]
            ],
            "align": [".", ",", " "],
            "type": ["char"],
            "width": {}
          },
          {
            "cols": [
              {
                "span_structure": {
                  "span": ["value"],
                  "col": ["val2"]
                }
              }
            ],
            "align": {},
            "type": ["char"],
            "width": [25]
          }
        ]
      } 

---

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "col_style_plan": [
          {
            "cols": [
              ["my var"]
            ],
            "align": ["left"],
            "type": ["char"],
            "width": [100]
          }
        ]
      } 

# json page plan

    Code
      as_json(pp)
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "page_plan": {
          "struct_list": [
            {
              "group_val": [".default"],
              "label_val": {}
            },
            {
              "group_val": {},
              "label_val": ["A"]
            }
          ],
          "note_loc": ["source_note"],
          "max_rows": {},
          "transform": {}
        }
      } 

---

    Code
      .
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "page_plan": {
          "struct_list": [],
          "note_loc": ["noprint"],
          "max_rows": [5],
          "transform": {}
        }
      } 

# page_plan() roundtrip to JSON with transform

    Code
      as_json(page_plan_lambda)
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "page_plan": {
          "struct_list": [
            {
              "group_val": [".default"],
              "label_val": {}
            },
            {
              "group_val": {},
              "label_val": ["A"]
            }
          ],
          "note_loc": ["source_note"],
          "max_rows": {},
          "transform": ["~stringr::str_replace(.x, \"foo\", \"bar\")"]
        }
      } 

---

    Code
      as_json(page_plan_function)
    Output
      {
        "group": [],
        "label": {},
        "param": {},
        "value": {},
        "column": [],
        "page_plan": {
          "struct_list": [
            {
              "group_val": [".default"],
              "label_val": {}
            },
            {
              "group_val": {},
              "label_val": ["A"]
            }
          ],
          "note_loc": ["source_note"],
          "max_rows": {},
          "transform": ["function (x) \n{\n    stringr::str_replace(x, \"foo\", \"bar\")\n}"]
        }
      } 

# create_stub_head() works

    Code
      create_stub_head(col_plan_vars = rlang::quos(-ord, grp2, lbl, `1`), group = rlang::quos(
        grp2), label = rlang::quos(lbl), row_grp_plan_label_loc = "indented")
    Output
      [1] ""

---

    Code
      create_stub_head(col_plan_vars = rlang::quos(-ord, grp2, lbl, `1`), group = rlang::quos(
        grp2), label = rlang::quos(lbl), row_grp_plan_label_loc = "column")
    Output
      [1] "" ""

# apply_col_plan() works

    Code
      apply_col_plan(df1, col_selection = "-ord", grp_lbl = rlang::quos(grp2, lbl))
    Output
      # A tibble: 7 x 5
        grp2  lbl   prm   column   val
        <chr> <chr> <chr>  <dbl> <dbl>
      1 c     n     n          1   1  
      2 c     n     n_2        1   1.1
      3 b     m     n          1   2  
      4 b     m     n_2        1   2.1
      5 v     s     n          1   3  
      6 v     s     n_3        1   3.3
      7 p     e     n          1   4  

# apply_col_plan() group and label vars are excluded from renaming

    Code
      apply_col_plan(df1, col_selection = c("-ord", "lbl"), grp_lbl = rlang::quos(
        grp2, lbl))
    Output
      # A tibble: 7 x 5
        grp2  lbl   prm   column   val
        <chr> <chr> <chr>  <dbl> <dbl>
      1 c     n     n          1   1  
      2 c     n     n_2        1   1.1
      3 b     m     n          1   2  
      4 b     m     n_2        1   2.1
      5 v     s     n          1   3  
      6 v     s     n_3        1   3.3
      7 p     e     n          1   4  

# create_col_order() works

    Code
      create_col_order(data_names = c("grp2", "lbl", "ord", "1"), columns = rlang::quos(
        column), cp = col_plan(-ord))
    Output
      <list_of<quosure>>
      
      [[1]]
      <quosure>
      expr: ^-ord
      env:  <env-address>
      
      [[2]]
      <quosure>
      expr: ^grp2
      env:  <env-address>
      
      [[3]]
      <quosure>
      expr: ^lbl
      env:  <env-address>
      
      [[4]]
      <quosure>
      expr: ^`1`
      env:  <env-address>
      

# create_col_order() with NULL col plan

    Code
      create_col_order(data_names = c("grp2", "lbl", "ord", "1"), columns = rlang::quos(
        column), cp = NULL)
    Output
      <list_of<quosure>>
      
      [[1]]
      <quosure>
      expr: ^grp2
      env:  <env-address>
      
      [[2]]
      <quosure>
      expr: ^lbl
      env:  <env-address>
      
      [[3]]
      <quosure>
      expr: ^ord
      env:  <env-address>
      
      [[4]]
      <quosure>
      expr: ^`1`
      env:  <env-address>
      

# create_col_order() with empty columns arg & cp not NULL

    Code
      create_col_order(data_names = c("grp2", "lbl", "ord", "1"), columns = rlang::quos(),
      cp = col_plan(-ord))
    Output
      <list_of<quosure>>
      
      [[1]]
      <quosure>
      expr: ^-ord
      env:  <env-address>
      
      [[2]]
      <quosure>
      expr: ^grp2
      env:  <env-address>
      
      [[3]]
      <quosure>
      expr: ^lbl
      env:  <env-address>
      
      [[4]]
      <quosure>
      expr: ^`1`
      env:  <env-address>
      

# create_col_order() with span_structure()

    Code
      create_col_order(data_names = c("group", "label",
        "cols 1,2___tlang_delim___col1", "cols 1,2___tlang_delim___col2", "mycol3",
        "col 4___tlang_delim___col4", "mycol5"), columns = rlang::quos(span1, my_col),
      cp = col_plan(span_structure(span1 = c(`first cols` = "cols 1,2")), group,
      label, starts_with("col"), new_col_3 = mycol3, -mycol5))
    Output
      <list_of<quosure>>
      
      [[1]]
      <quosure>
      expr: ^group
      env:  <env-address>
      
      [[2]]
      <quosure>
      expr: ^label
      env:  <env-address>
      
      $`first cols___tlang_delim___col1`
      <quosure>
      expr: ^`cols 1,2___tlang_delim___col1`
      env:  <env-address>
      
      $`first cols___tlang_delim___col2`
      <quosure>
      expr: ^`cols 1,2___tlang_delim___col2`
      env:  <env-address>
      
      [[5]]
      <quosure>
      expr: ^`col 4___tlang_delim___col4`
      env:  <env-address>
      
      $new_col_3
      <quosure>
      expr: ^mycol3
      env:  <env-address>
      
      [[7]]
      <quosure>
      expr: ^-mycol5
      env:  <env-address>
      

# split_data_names_to_df() works

    Code
      split_data_names_to_df(data_names = c("grp2", "lbl", "ord", "1"),
      preselected_cols = NULL, column_names = "column")
    Output
      # A tibble: 4 x 3
        column `__tfrmt_new_name__column` subtraction_status
        <chr>  <chr>                      <lgl>             
      1 grp2   grp2                       FALSE             
      2 lbl    lbl                        FALSE             
      3 ord    ord                        FALSE             
      4 1      1                          FALSE             

---

    Code
      split_data_names_to_df(data_names = c("grp2", "lbl", "ord", "1"),
      preselected_cols = c(order = "ord"), column_names = "column")
    Output
      # A tibble: 4 x 3
        column `__tfrmt_new_name__column` subtraction_status
        <chr>  <chr>                      <lgl>             
      1 ord    order                      FALSE             
      2 grp2   grp2                       FALSE             
      3 lbl    lbl                        FALSE             
      4 1      1                          FALSE             

---

    Code
      split_data_names_to_df(data_names = c("grp2", "lbl", "ord", "1"),
      preselected_cols = c(order = "ord"), column_names = c("foo", "bar"))
    Output
      # A tibble: 4 x 5
        foo   bar   `__tfrmt_new_name__foo` `__tfrmt_new_name__bar` subtraction_status
        <chr> <chr> <chr>                   <chr>                   <lgl>             
      1 <NA>  ord   <NA>                    order                   FALSE             
      2 <NA>  grp2  <NA>                    grp2                    FALSE             
      3 <NA>  lbl   <NA>                    lbl                     FALSE             
      4 <NA>  1     <NA>                    1                       FALSE             

# print_to_gt() works

    Code
      print_to_gt(tfrmt_plan, .data = test_data)[["_data"]]
    Output
      # A tibble: 2 x 8
        label     span01___tlang_delim~1 span01___tlang_delim~2 span01___tlang_delim~3
        <chr>     <chr>                  <chr>                  <chr>                 
      1 "mygrp"   <NA>                   <NA>                   <NA>                  
      2 "  mylbl" 1.00                   1.00                   1.00                  
      # i abbreviated names:
      #   1: span01___tlang_delim___span1___tlang_delim___lower1_a,
      #   2: span01___tlang_delim___span1___tlang_delim___lower1_b,
      #   3: span01___tlang_delim___span2___tlang_delim___lower2_a
      # i 4 more variables:
      #   span01___tlang_delim___span2___tlang_delim___lower2_b <chr>,
      #   span02___tlang_delim___span3___tlang_delim___lower2_a <chr>, ...

# print_to_gt() complains with incorrect inputs

    Code
      print_to_gt(mtcars)
    Condition
      Error in `print_to_gt()`:
      ! Requires a tfrmt object

---

    Code
      print_to_gt(tfrmt_plan, "foo")
    Condition
      Error in `print_to_gt()`:
      ! Requires data, if not available please use `print_mock_gt()`

# print_mock_gt() messages when tfrmt$param is missing

    Code
      print_mock_gt(tfrmt_plan_no_param)[["_data"]]
    Message
      `tfrmt` will need a `param` value to `print_to_gt` when data is available
    Output
      # A tibble: 3 x 5
        label   column1     column2     column3     ..tfrmt_row_grp_lbl
        <chr>   <chr>       <chr>       <chr>       <lgl>              
      1 label_1 xxx (xx.x%) xxx (xx.x%) xxx (xx.x%) FALSE              
      2 label_2 xxx (xx.x%) xxx (xx.x%) xxx (xx.x%) FALSE              
      3 label_3 xxx (xx.x%) xxx (xx.x%) xxx (xx.x%) FALSE              

# print_mock_gt() messages when tfrmt$column is missing

    Code
      print_mock_gt(tfrmt_plan_no_column)[["_data"]]
    Message
      `tfrmt` will need `column` value(s) to `print_to_gt` when data is available
    Output
      # A tibble: 3 x 5
        label   `__tfrmt__column1` `__tfrmt__column2` `__tfrmt__column3`
        <chr>   <chr>              <chr>              <chr>             
      1 label_1 xxx (xx.x%)        xxx (xx.x%)        xxx (xx.x%)       
      2 label_2 xxx (xx.x%)        xxx (xx.x%)        xxx (xx.x%)       
      3 label_3 xxx (xx.x%)        xxx (xx.x%)        xxx (xx.x%)       
      # i 1 more variable: ..tfrmt_row_grp_lbl <lgl>

# print_mock_gt() messages when tfrmt$value is missing

    Code
      print_mock_gt(tfrmt_plan_no_value)[["_data"]]
    Message
      Message: `tfrmt` will need `value` value to `print_to_gt` when data is available
    Output
      # A tibble: 3 x 5
        label   column1     column2     column3     ..tfrmt_row_grp_lbl
        <chr>   <chr>       <chr>       <chr>       <lgl>              
      1 label_1 xxx (xx.x%) xxx (xx.x%) xxx (xx.x%) FALSE              
      2 label_2 xxx (xx.x%) xxx (xx.x%) xxx (xx.x%) FALSE              
      3 label_3 xxx (xx.x%) xxx (xx.x%) xxx (xx.x%) FALSE              

# print_mock_gt() with missing body_plan

    Code
      print_mock_gt(tfrmt_plan_no_body_plan)[["_data"]]
    Output
      # A tibble: 3 x 5
        label   column1 column2 column3 ..tfrmt_row_grp_lbl
        <chr>   <chr>   <chr>   <chr>   <lgl>              
      1 label_1 X.X     X.X     X.X     FALSE              
      2 label_2 X.X     X.X     X.X     FALSE              
      3 label_3 X.X     X.X     X.X     FALSE              

# print_mock_data() removes `value` when it exists in the input data

    Code
      print_mock_gt(plan, data)["_data"]
    Message
       Removing `value_to_remove` from input data for mocking.
    Output
      $`_data`
      # A tibble: 5 x 6
        label                `Placebo\n(N=XX)` `Total\n(N=XXX)` Xanomeline\nHigh Dos~1
        <chr>                <chr>             <chr>            <chr>                 
      1 Complete Study       xx (xxx%)         xx (xxx%)        xx (xxx%)             
      2 Complete Week 24     xx (xxx%)         xx (xxx%)        xx (xxx%)             
      3 Efficacy             xx (xxx%)         xx (xxx%)        xx (xxx%)             
      4 Intent-To-Treat (IT~ xx (xxx%)         xx (xxx%)        xx (xxx%)             
      5 Safety               xx (xxx%)         xx (xxx%)        xx (xxx%)             
      # i abbreviated name: 1: `Xanomeline\nHigh Dose\n(N=XX)`
      # i 2 more variables: `Xanomeline\nLow Dose\n(N=XX)` <chr>,
      #   ..tfrmt_row_grp_lbl <lgl>
      

# cleaned_data_to_gt() works

    Code
      cleaned_data_to_gt(test_data, tfrmt, .unicode_ws = TRUE)[["_data"]]
    Output
      # A tibble: 3 x 7
        label   group `0`   `1000` `2000` `3000` ..tfrmt_row_grp_lbl
        <chr>   <chr> <chr> <chr>  <chr>  <chr>  <lgl>              
      1 Obs     A     630   372    256    11     FALSE              
      2 Lev     A     620   360    266    8      FALSE              
      3 Lev+5FU A     608   425    328    14     FALSE              

# cleaned_data_to_gt.list() works

    Code
      cleaned_tables$gt_tbls$gt_tbl[[1]][["_data"]]
    Output
      # A tibble: 3 x 7
        label   group `0`   `1000` `2000` `3000` ..tfrmt_row_grp_lbl
        <chr>   <chr> <chr> <chr>  <chr>  <chr>  <lgl>              
      1 Obs     A     630   372    256    11     FALSE              
      2 Lev     A     620   360    266    8      FALSE              
      3 Lev+5FU A     608   425    328    14     FALSE              

---

    Code
      cleaned_tables$gt_tbls$gt_tbl[[2]][["_data"]]
    Output
      # A tibble: 3 x 7
        label   group `0`   `1000` `2000` `3000` ..tfrmt_row_grp_lbl
        <chr>   <chr> <chr> <chr>  <chr>  <chr>  <lgl>              
      1 Obs2    B     631   373    257    12     FALSE              
      2 Lev2    B     621   361    267    9      FALSE              
      3 Lev+5FU B     609   426    329    15     FALSE              

# cleaned_data_to_gt() with col_style_plan

    Code
      purrr::pluck(cleaned_data_to_gt(apply_tfrmt(test_data, plan, mock = FALSE),
      plan, .unicode_ws = TRUE), "_data")
    Output
      # A tibble: 11 x 6
         g1    one          trt1         trt2            four    ..tfrmt_row_grp_lbl
         <chr> <chr>        <chr>        <chr>           <chr>   <lgl>              
       1 " G1" "        g3"  <NA>         <NA>            <NA>   TRUE               
       2 " G1" "     n (%)" "12 (34.0%)" "24\n(58.0%)"   ""      FALSE              
       3 " G1" "      mean" "          " "          "    "<.001" FALSE              
       4 " G1" "  --------" "----------" "----------"    "-----" FALSE              
       5 "G2_" "        g3"  <NA>         <NA>            <NA>   TRUE               
       6 "G2_" "      mean" "12.30     " "     15.40"    ""      FALSE              
       7 "G2_" "        sd" " 4.34     " "      8.25"    ""      FALSE              
       8 "G2_" "    median" "14.00     " "     16.00"    ""      FALSE              
       9 "G2_" "  --------" "----------" "----------"    "-----" FALSE              
      10 " G3" "        g3"  <NA>         <NA>            <NA>   TRUE               
      11 " G3" "  (q1, q3)" "(10, 20)  " "  (22,\n  22)" ""      FALSE              

# cleaned_data_to_gt() with row_grp_plan and location = 'column'

    Code
      purrr::pluck(cleaned_data_to_gt(apply_tfrmt(test_data, tfrmt_plan, mock = FALSE),
      tfrmt_plan, .unicode_ws = TRUE), "_data")
    Output
      # A tibble: 4 x 5
        grp1  grp2  lbl   `1`   ..tfrmt_row_grp_lbl
        <chr> <chr> <chr> <chr> <lgl>              
      1 d     c     n     1     FALSE              
      2 a     b     m     2     FALSE              
      3 q     v     s     3     FALSE              
      4 b     p     e     4     FALSE              

# prep_...() pipe with demographic data

    Code
      gt::as_raw_html(b)
    Output
      <div id="stripped_id" style="padding-left:0px;padding-right:0px;padding-top:10px;padding-bottom:10px;overflow-x:auto;overflow-y:auto;width:auto;height:auto;">
        
        <table class="gt_table" data-quarto-disable-processing="false" data-quarto-bootstrap="false" style="-webkit-font-smoothing: antialiased; -moz-osx-font-smoothing: grayscale; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; display: table; border-collapse: collapse; line-height: normal; margin-left: auto; margin-right: auto; color: #333333; font-size: 14px; font-weight: normal; font-style: normal; background-color: #FFFFFF; width: auto; border-top-style: solid; border-top-width: 2px; border-top-color: #A8A8A8; border-right-style: none; border-right-width: 2px; border-right-color: #D3D3D3; border-bottom-style: solid; border-bottom-width: 2px; border-bottom-color: #A8A8A8; border-left-style: none; border-left-width: 2px; border-left-color: #D3D3D3;" bgcolor="#FFFFFF">
        <thead style="border-style: none;">
          <tr class="gt_heading" style="border-style: none; background-color: #FFFFFF; text-align: center; border-bottom-color: #FFFFFF; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3;" bgcolor="#FFFFFF" align="center">
            <td colspan="5" class="gt_heading gt_title gt_font_normal gt_bottom_border" style="border-style: none; color: #333333; font-size: 125%; padding-top: 4px; padding-bottom: 4px; padding-left: 5px; padding-right: 5px; background-color: #FFFFFF; text-align: center; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; border-bottom-style: solid; border-bottom-width: 2px; border-bottom-color: #D3D3D3; font-weight: normal;" bgcolor="#FFFFFF" align="center">Summary of Demographic Characteristics</td>
          </tr>
          
          <tr class="gt_col_headings" style="border-style: none; border-top-style: solid; border-top-width: 2px; border-top-color: #D3D3D3; border-bottom-style: solid; border-bottom-width: 2px; border-bottom-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3;">
            <th class="gt_col_heading gt_columns_bottom_border gt_left" rowspan="1" colspan="1" scope="col" id="a::stub" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: left;" bgcolor="#FFFFFF" valign="bottom" align="left"><span class="gt_from_md"></span></th>
            <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 0px; border-top-style: solid; border-top-color: transparent;" scope="col" id="Placebo-(N=86)" align="center" bgcolor="#FFFFFF" valign="bottom"><span class="gt_from_md">Placebo
      (N=86)</span></th>
            <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 0px; border-top-style: solid; border-top-color: transparent;" scope="col" id="Xanomeline-High-Dose-(N=84)" align="center" bgcolor="#FFFFFF" valign="bottom"><span class="gt_from_md">Xanomeline High Dose
      (N=84)</span></th>
            <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 0px; border-top-style: solid; border-top-color: transparent;" scope="col" id="Xanomeline-Low-Dose-(N=84)" align="center" bgcolor="#FFFFFF" valign="bottom"><span class="gt_from_md">Xanomeline Low Dose
      (N=84)</span></th>
            <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 0px; border-top-style: solid; border-top-color: transparent;" scope="col" id="Overall-ARM-(N=254)" align="center" bgcolor="#FFFFFF" valign="bottom"><span class="gt_from_md">Overall ARM
      (N=254)</span></th>
          </tr>
        </thead>
        <tbody class="gt_table_body" style="border-style: none; border-top-style: solid; border-top-width: 2px; border-top-color: #D3D3D3; border-bottom-style: solid; border-bottom-width: 2px; border-bottom-color: #D3D3D3;">
          <tr style="border-style: none;"><th id="stub_1_1" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">SEX                                        </th>
      <td headers="stub_1_1 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td>
      <td headers="stub_1_1 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td>
      <td headers="stub_1_1 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td>
      <td headers="stub_1_1 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td></tr>
          <tr style="border-style: none;"><th id="stub_1_2" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  n                                        </th>
      <td headers="stub_1_2 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 86      </td>
      <td headers="stub_1_2 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 84      </td>
      <td headers="stub_1_2 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 84      </td>
      <td headers="stub_1_2 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">254      </td></tr>
          <tr style="border-style: none;"><th id="stub_1_3" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  F                                        </th>
      <td headers="stub_1_3 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 53 (62%)</td>
      <td headers="stub_1_3 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 40 (48%)</td>
      <td headers="stub_1_3 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 50 (60%)</td>
      <td headers="stub_1_3 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">143 (56%)</td></tr>
          <tr style="border-style: none;"><th id="stub_1_4" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  M                                        </th>
      <td headers="stub_1_4 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 33 (38%)</td>
      <td headers="stub_1_4 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 44 (52%)</td>
      <td headers="stub_1_4 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 34 (40%)</td>
      <td headers="stub_1_4 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">111 (44%)</td></tr>
          <tr style="border-style: none;"><th id="stub_1_5" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">                                           </th>
      <td headers="stub_1_5 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">         </td>
      <td headers="stub_1_5 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">         </td>
      <td headers="stub_1_5 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">         </td>
      <td headers="stub_1_5 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">         </td></tr>
          <tr style="border-style: none;"><th id="stub_1_6" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">AGE                                        </th>
      <td headers="stub_1_6 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td>
      <td headers="stub_1_6 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td>
      <td headers="stub_1_6 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td>
      <td headers="stub_1_6 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td></tr>
          <tr style="border-style: none;"><th id="stub_1_7" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  n                                        </th>
      <td headers="stub_1_7 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 86      </td>
      <td headers="stub_1_7 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 84      </td>
      <td headers="stub_1_7 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 84      </td>
      <td headers="stub_1_7 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">254      </td></tr>
          <tr style="border-style: none;"><th id="stub_1_8" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  Mean                                     </th>
      <td headers="stub_1_8 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 75      </td>
      <td headers="stub_1_8 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 74      </td>
      <td headers="stub_1_8 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 76      </td>
      <td headers="stub_1_8 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 75      </td></tr>
          <tr style="border-style: none;"><th id="stub_1_9" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  SD                                       </th>
      <td headers="stub_1_9 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  9      </td>
      <td headers="stub_1_9 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  8      </td>
      <td headers="stub_1_9 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  8      </td>
      <td headers="stub_1_9 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  8      </td></tr>
          <tr style="border-style: none;"><th id="stub_1_10" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  Min                                      </th>
      <td headers="stub_1_10 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 52      </td>
      <td headers="stub_1_10 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 56      </td>
      <td headers="stub_1_10 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 51      </td>
      <td headers="stub_1_10 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 51      </td></tr>
          <tr style="border-style: none;"><th id="stub_1_11" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  Max                                      </th>
      <td headers="stub_1_11 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 89      </td>
      <td headers="stub_1_11 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 88      </td>
      <td headers="stub_1_11 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 88      </td>
      <td headers="stub_1_11 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 89      </td></tr>
          <tr style="border-style: none;"><th id="stub_1_12" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">                                           </th>
      <td headers="stub_1_12 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">         </td>
      <td headers="stub_1_12 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">         </td>
      <td headers="stub_1_12 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">         </td>
      <td headers="stub_1_12 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">         </td></tr>
          <tr style="border-style: none;"><th id="stub_1_13" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">AGEGR1                                     </th>
      <td headers="stub_1_13 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td>
      <td headers="stub_1_13 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td>
      <td headers="stub_1_13 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td>
      <td headers="stub_1_13 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td></tr>
          <tr style="border-style: none;"><th id="stub_1_14" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  n                                        </th>
      <td headers="stub_1_14 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 86      </td>
      <td headers="stub_1_14 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 84      </td>
      <td headers="stub_1_14 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 84      </td>
      <td headers="stub_1_14 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">254      </td></tr>
          <tr style="border-style: none;"><th id="stub_1_15" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  &lt;=18 years                               </th>
      <td headers="stub_1_15 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td>
      <td headers="stub_1_15 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td>
      <td headers="stub_1_15 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td>
      <td headers="stub_1_15 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td></tr>
          <tr style="border-style: none;"><th id="stub_1_16" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  Between 18 and 65 years                  </th>
      <td headers="stub_1_16 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 14 (16%)</td>
      <td headers="stub_1_16 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 11 (13%)</td>
      <td headers="stub_1_16 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  8 (10%)</td>
      <td headers="stub_1_16 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 33 (13%)</td></tr>
          <tr style="border-style: none;"><th id="stub_1_17" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  &gt;=65 years                               </th>
      <td headers="stub_1_17 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 72 (84%)</td>
      <td headers="stub_1_17 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 73 (87%)</td>
      <td headers="stub_1_17 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 76 (90%)</td>
      <td headers="stub_1_17 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">221 (87%)</td></tr>
          <tr style="border-style: none;"><th id="stub_1_18" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">                                           </th>
      <td headers="stub_1_18 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">         </td>
      <td headers="stub_1_18 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">         </td>
      <td headers="stub_1_18 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">         </td>
      <td headers="stub_1_18 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">         </td></tr>
          <tr style="border-style: none;"><th id="stub_1_19" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">ETHNIC                                     </th>
      <td headers="stub_1_19 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td>
      <td headers="stub_1_19 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td>
      <td headers="stub_1_19 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td>
      <td headers="stub_1_19 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td></tr>
          <tr style="border-style: none;"><th id="stub_1_20" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  n                                        </th>
      <td headers="stub_1_20 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 86      </td>
      <td headers="stub_1_20 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 84      </td>
      <td headers="stub_1_20 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 84      </td>
      <td headers="stub_1_20 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">254      </td></tr>
          <tr style="border-style: none;"><th id="stub_1_21" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  HISPANIC OR LATINO                       </th>
      <td headers="stub_1_21 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  3 ( 3%)</td>
      <td headers="stub_1_21 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  3 ( 4%)</td>
      <td headers="stub_1_21 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  6 ( 7%)</td>
      <td headers="stub_1_21 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 12 ( 5%)</td></tr>
          <tr style="border-style: none;"><th id="stub_1_22" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  NOT HISPANIC OR LATINO                   </th>
      <td headers="stub_1_22 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 83 (97%)</td>
      <td headers="stub_1_22 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 81 (96%)</td>
      <td headers="stub_1_22 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 78 (93%)</td>
      <td headers="stub_1_22 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">242 (95%)</td></tr>
          <tr style="border-style: none;"><th id="stub_1_23" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  NOT REPORTED                             </th>
      <td headers="stub_1_23 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td>
      <td headers="stub_1_23 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td>
      <td headers="stub_1_23 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td>
      <td headers="stub_1_23 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td></tr>
          <tr style="border-style: none;"><th id="stub_1_24" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">                                           </th>
      <td headers="stub_1_24 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">         </td>
      <td headers="stub_1_24 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">         </td>
      <td headers="stub_1_24 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">         </td>
      <td headers="stub_1_24 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">         </td></tr>
          <tr style="border-style: none;"><th id="stub_1_25" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">RACE                                       </th>
      <td headers="stub_1_25 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td>
      <td headers="stub_1_25 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td>
      <td headers="stub_1_25 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td>
      <td headers="stub_1_25 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"><br></td></tr>
          <tr style="border-style: none;"><th id="stub_1_26" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  n                                        </th>
      <td headers="stub_1_26 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 86      </td>
      <td headers="stub_1_26 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 84      </td>
      <td headers="stub_1_26 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 84      </td>
      <td headers="stub_1_26 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">254      </td></tr>
          <tr style="border-style: none;"><th id="stub_1_27" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  AMERICAN INDIAN OR ALASKA NATIVE         </th>
      <td headers="stub_1_27 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td>
      <td headers="stub_1_27 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  1 ( 1%)</td>
      <td headers="stub_1_27 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td>
      <td headers="stub_1_27 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  1 ( 0%)</td></tr>
          <tr style="border-style: none;"><th id="stub_1_28" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  ASIAN                                    </th>
      <td headers="stub_1_28 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td>
      <td headers="stub_1_28 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td>
      <td headers="stub_1_28 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td>
      <td headers="stub_1_28 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td></tr>
          <tr style="border-style: none;"><th id="stub_1_29" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  BLACK OR AFRICAN AMERICAN                </th>
      <td headers="stub_1_29 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  8 ( 9%)</td>
      <td headers="stub_1_29 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  9 (11%)</td>
      <td headers="stub_1_29 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  6 ( 7%)</td>
      <td headers="stub_1_29 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 23 ( 9%)</td></tr>
          <tr style="border-style: none;"><th id="stub_1_30" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  NATIVE HAWAIIAN OR OTHER PACIFIC ISLANDER</th>
      <td headers="stub_1_30 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td>
      <td headers="stub_1_30 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td>
      <td headers="stub_1_30 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td>
      <td headers="stub_1_30 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">  0 ( 0%)</td></tr>
          <tr style="border-style: none;"><th id="stub_1_31" scope="row" class="gt_row gt_left gt_stub" style="border-style: none; padding-top: 1px; padding-bottom: 1px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: initial; text-transform: inherit; border-right-style: none; border-right-width: 0px; border-right-color: rgba(255, 255, 255, 0); padding-left: 5px; padding-right: 5px; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; text-align: left; white-space: pre-wrap;" align="left" valign="middle" bgcolor="#FFFFFF">  WHITE                                    </th>
      <td headers="stub_1_31 Placebo
      (N=86)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 78 (91%)</td>
      <td headers="stub_1_31 Xanomeline High Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 74 (88%)</td>
      <td headers="stub_1_31 Xanomeline Low Dose
      (N=84)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle"> 78 (93%)</td>
      <td headers="stub_1_31 Overall ARM
      (N=254)" class="gt_row gt_right" style="border-style: none; padding-top: 1px; padding-bottom: 1px; padding-left: 5px; padding-right: 5px; margin: 10px; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; font-variant-numeric: tabular-nums; text-align: center; white-space: pre-wrap; font-family: Courier, system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; border-top-width: 1px; border-top-style: solid; border-top-color: transparent; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: transparent;" align="center" valign="middle">230 (91%)</td></tr>
        </tbody>
        
      </table>
      </div>

# prep_combine_vars() works

    Code
      prep_combine_vars(df, vars = c("b", "c", "d", "e", "f", "g"))
    Output
      # A tibble: 6 x 3
            a context     variable_level
        <int> <chr>       <chr>         
      1     1 categorical a             
      2     2 categorical b             
      3     3 categorical c             
      4     4 categorical d             
      5     5 categorical e             
      6     6 categorical f             

---

    Code
      prep_combine_vars(df, vars = c("b", "c", "d", "e", "f"), remove = FALSE)
    Output
      # A tibble: 6 x 9
            a context     variable_level b     c     d     e     f     g    
        <int> <chr>       <chr>          <chr> <chr> <chr> <chr> <chr> <chr>
      1     1 categorical a              a     <NA>  <NA>  <NA>  <NA>  <NA> 
      2     2 categorical b              <NA>  b     <NA>  <NA>  <NA>  <NA> 
      3     3 categorical c              <NA>  <NA>  c     <NA>  <NA>  <NA> 
      4     4 categorical d              <NA>  <NA>  <NA>  d     <NA>  <NA> 
      5     5 categorical e              <NA>  <NA>  <NA>  <NA>  e     <NA> 
      6     6 categorical <NA>           <NA>  <NA>  <NA>  <NA>  <NA>  f    

# prep_combine_vars() returns the input when context hierarchical

    Code
      prep_combine_vars(df, vars = c("b", "c", "d", "e", "f", "g"))
    Message
      i The `context` column indicates data comes from a hierarchical `ard` stack.
      * Unable to apply `prep_combine_vars()`.
    Output
      # A tibble: 6 x 8
            a context      b     c     d     e     f     g    
        <int> <chr>        <chr> <chr> <chr> <chr> <chr> <chr>
      1     1 hierarchical a     <NA>  <NA>  <NA>  <NA>  <NA> 
      2     2 hierarchical <NA>  b     <NA>  <NA>  <NA>  <NA> 
      3     3 hierarchical <NA>  <NA>  c     <NA>  <NA>  <NA> 
      4     4 hierarchical <NA>  <NA>  <NA>  d     <NA>  <NA> 
      5     5 hierarchical <NA>  <NA>  <NA>  <NA>  e     <NA> 
      6     6 hierarchical <NA>  <NA>  <NA>  <NA>  <NA>  f    

# prep_combine_vars() return input unchanged when length(vars)=1

    Code
      prep_combine_vars(df, vars = "b")
    Message
      i You supplied a single column in `vars`.
      * Unable to apply `prep_combine_vars()`.
    Output
      # A tibble: 6 x 8
            a context     b     c     d     e     f     g    
        <int> <chr>       <chr> <chr> <chr> <chr> <chr> <chr>
      1     1 categorical a     <NA>  <NA>  <NA>  <NA>  <NA> 
      2     2 categorical <NA>  b     <NA>  <NA>  <NA>  <NA> 
      3     3 categorical <NA>  <NA>  c     <NA>  <NA>  <NA> 
      4     4 categorical <NA>  <NA>  <NA>  d     <NA>  <NA> 
      5     5 categorical <NA>  <NA>  <NA>  <NA>  e     <NA> 
      6     6 categorical <NA>  <NA>  <NA>  <NA>  <NA>  f    

# prep_combine_vars() does not over unite

    Code
      prep_combine_vars(df, vars = c("b", "c", "d", "e", "f", "g"))
    Message
      i Combining the columns listed in `vars` would result in a loss of information.
      * Unable to apply `prep_combine_vars()`.
    Output
      # A tibble: 6 x 8
            a context     b     c     d     e     f     g    
        <int> <chr>       <chr> <chr> <chr> <chr> <chr> <chr>
      1     1 categorical a     <NA>  <NA>  <NA>  <NA>  <NA> 
      2     2 categorical <NA>  b     b     b     <NA>  <NA> 
      3     3 categorical <NA>  <NA>  <NA>  <NA>  <NA>  <NA> 
      4     4 categorical <NA>  <NA>  <NA>  <NA>  <NA>  <NA> 
      5     5 categorical <NA>  <NA>  <NA>  <NA>  e     <NA> 
      6     6 categorical <NA>  <NA>  <NA>  <NA>  <NA>  f    

# prep_combine_vars() informs when the context col is missing

    Code
      prep_combine_vars(df, vars = c("b", "c", "d", "e", "f", "g"))
    Message
      i Required column (`context`) not present in the input data.
      * Unable to apply `prep_combine_vars()`.
    Output
      # A tibble: 6 x 7
            a b     c     d     e     f     g    
        <int> <chr> <chr> <chr> <chr> <chr> <chr>
      1     1 a     <NA>  <NA>  <NA>  <NA>  <NA> 
      2     2 <NA>  b     b     b     <NA>  <NA> 
      3     3 <NA>  <NA>  <NA>  <NA>  <NA>  <NA> 
      4     4 <NA>  <NA>  <NA>  <NA>  <NA>  <NA> 
      5     5 <NA>  <NA>  <NA>  <NA>  e     <NA> 
      6     6 <NA>  <NA>  <NA>  <NA>  <NA>  f    

# prep_combine_vars() errors when `vars` is not character

    Code
      prep_combine_vars(df, vars = 1:3)
    Condition
      Error in `prep_combine_vars()`:
      ! `vars` must be a character vector. You have supplied an integer vector.

---

    Code
      prep_combine_vars(df, vars = c(TRUE, FALSE))
    Condition
      Error in `prep_combine_vars()`:
      ! `vars` must be a character vector. You have supplied a logical vector.

# prep_big_n() works

    Code
      prep_big_n(df, vars = c("b", "c"))
    Output
      # A tibble: 6 x 3
        stat_name context      stat_variable
        <chr>     <chr>        <chr>        
      1 n         continuous   a            
      2 max       continuous   a            
      3 min       continuous   a            
      4 bigN      hierarchical b            
      5 bigN      categorical  c            
      6 bigN      total_n      d            

---

    Code
      prep_big_n(df, vars = "b")
    Output
      # A tibble: 8 x 3
        stat_name context      stat_variable
        <chr>     <chr>        <chr>        
      1 n         continuous   a            
      2 max       continuous   a            
      3 min       continuous   a            
      4 bigN      hierarchical b            
      5 n         categorical  c            
      6 N         categorical  c            
      7 p         categorical  c            
      8 bigN      total_n      d            

# prep_big_n() informs when required columns are missing

    Code
      prep_big_n(df, vars = "a")
    Message
      i Required column (`stat_name`) not present in the input data.
      * Unable to apply `prep_big_n()`.
    Output
      # A tibble: 9 x 3
        a     context      stat_variable
        <chr> <chr>        <chr>        
      1 n     continuous   a            
      2 max   continuous   a            
      3 min   continuous   a            
      4 n     hierarchical b            
      5 N     hierarchical b            
      6 p     hierarchical b            
      7 n     categorical  c            
      8 N     categorical  c            
      9 p     categorical  c            

# prep_big_n() errors when `vars` is not character

    Code
      prep_big_n(df, vars = 1)
    Condition
      Error in `prep_big_n()`:
      ! `vars` must be a character vector. You have supplied a number.

---

    Code
      prep_big_n(df, vars = TRUE)
    Condition
      Error in `prep_big_n()`:
      ! `vars` must be a character vector. You have supplied `TRUE`.

# prep_label() works

    Code
      prep_label(df)
    Output
      # A tibble: 9 x 5
        variable_level stat_label stat_name context      label
        <chr>          <chr>      <chr>     <chr>        <chr>
      1 a              n          n         categorical  a    
      2 a              N          N         categorical  a    
      3 a              %          p         categorical  a    
      4 b              N          N         continuous   N    
      5 b              Mean       mean      continuous   Mean 
      6 b              SD         sd        continuous   SD   
      7 c              n          n         hierarchical c    
      8 c              N          N         hierarchical c    
      9 c              %          p         hierarchical c    

# prep_label() returns the input when the required cols are missing

    Code
      prep_label(df)
    Message
      i Required columns (`variable_level` and `stat_name`) not present in the input data.
      * Unable to apply `prep_label()`.
    Output
      # A tibble: 3 x 3
        x     stat_label context     
        <chr> <chr>      <chr>       
      1 d     a          categorical 
      2 e     b          continuous  
      3 f     c          hierarchical

---

    Code
      prep_label(df2)
    Message
      i Required columns (`stat_label` and `stat_name`) not present in the input data.
      * Unable to apply `prep_label()`.
    Output
      # A tibble: 3 x 3
        variable_level y     context     
        <chr>          <chr> <chr>       
      1 d              a     categorical 
      2 e              b     continuous  
      3 f              c     hierarchical

# prep_hierarchical_fill() returns input when `length(vars) < 2`

    Code
      prep_hierarchical_fill(df, vars = "y")
    Message
      i At least 2 columns must be supplied to `vars`.
      * Unable to apply `prep_hierarchical_fill()`.
    Output
      # A tibble: 3 x 3
            x y     z    
        <dbl> <chr> <lgl>
      1     1 a     NA   
      2     2 <NA>  NA   
      3    NA b     NA   

# prep_hierarchical_fill() errors when `vars` is not character

    Code
      prep_hierarchical_fill(df, vars = 1:3)
    Condition
      Error in `prep_hierarchical_fill()`:
      ! `vars` must be a character vector. You have supplied an integer vector.

---

    Code
      prep_hierarchical_fill(df, vars = c(TRUE, FALSE))
    Condition
      Error in `prep_hierarchical_fill()`:
      ! `vars` must be a character vector. You have supplied a logical vector.

# prep_hierarchical_fill() errors when `fill` is not character

    Code
      prep_hierarchical_fill(df, vars = c("x", "y", "z"), fill = 2)
    Condition
      Error in `prep_hierarchical_fill()`:
      ! `fill` must be a character vector of length 1.

# generate_pairs() complains

    Code
      generate_pairs(1:3)
    Condition
      Error:
      ! `x` must be a character vector. You have supplied an integer vector.

---

    Code
      generate_pairs("foo")
    Condition
      Error:
      ! `x` must contain at least 2 column names. It contains 1.

# replace_na_pairwise() complains

    Code
      replace_na_pairwise(tibble::tibble(x = c(1, 2, NA), y = c("a", NA, "b"), z = rep(
        NA, 3)), pair = 1:2)
    Condition
      Error:
      ! `pair` must be a character vector. You have supplied an integer vector.

---

    Code
      replace_na_pairwise(tibble::tibble(x = c(1, 2, NA), y = c("a", NA, "b"), z = rep(
        NA, 3)), pair = c("x", "y", "z"))
    Condition
      Error:
      ! `pair` must contain exactly 2 elements. The one you supplied has 3.

---

    Code
      replace_na_pairwise(tibble::tibble(x = c(1, 2, NA), y = c("a", NA, "b"), z = rep(
        NA, "3")), pair = c("y", "z"), fill = 2)
    Condition
      Error:
      ! `fill` must be a character vector of length 1.

# check_order_vars() messages when order variables cause mismatching rows

    Code
      check_order_vars(test_data, tfrmt_plan)
    Message
      Note: Some row labels have values printed over more than 1 line.
       This could be due to incorrect sorting variables. Each row in your output table should have only one sorting var combination assigned to it.

---

    Code
      check_order_vars(test_data2, tfrmt_plan2)
    Message
      Note: Some row labels have values printed over more than 1 line.
       This could be due to incorrect sorting variables. Each row in your output table should have only one sorting var combination assigned to it.

# big Ns constant by page

    Code
      auto <- apply_tfrmt(.data = data, tfrmt = mytfrmt, mock = FALSE)
    Message
      Mismatch between big Ns and page_plan. For varying big N's by page (`by_page` = TRUE in `big_n_structure`), data must contain 1 big N value per unique grouping variable/value set to ".default" in `page_plan` and bigNs must be in order of the data

---

    The following columns have multiple Big N's associated with them:
    c("Placebo", "Total", "Treatment")

# not enough big Ns by page

    Code
      apply_tfrmt(.data = data, tfrmt = mytfrmt, mock = FALSE)
    Message
      Mismatch between big Ns and page_plan. For varying big N's by page (`by_page` = TRUE in `big_n_structure`), data must contain 1 big N value per unique grouping variable/value set to ".default" in `page_plan` and bigNs must be in order of the data
    Output
      [[1]]
      # A tibble: 2 x 5
        Label     `Placebo\nN = 12` `Treatment\nN = 14` `Total\nN = 31`
      * <chr>     <chr>             <chr>               <chr>          
      1 "Age (y)" <NA>              <NA>                <NA>           
      2 "  n"     12                14                  31             
      # i 1 more variable: ..tfrmt_row_grp_lbl <lgl>
      
      [[2]]
      # A tibble: 2 x 5
        Label `Placebo\nN = 12` `Treatment\nN = 14` `Total\nN = 31`
      * <chr> <chr>             <chr>               <chr>          
      1 "Sex" <NA>              <NA>                <NA>           
      2 "  n" 20                32                  18             
      # i 1 more variable: ..tfrmt_row_grp_lbl <lgl>
      

# Paging (group) variable is sorted non-alphabetically

    Code
      apply_tfrmt(.data = data, tfrmt = mytfrmt, mock = FALSE)
    Message
      Mismatch between big Ns and page_plan. For varying big N's by page (`by_page` = TRUE in `big_n_structure`), data must contain 1 big N value per unique grouping variable/value set to ".default" in `page_plan` and bigNs must be in order of the data
    Output
      [[1]]
      # A tibble: 2 x 5
        Label `Placebo\nN = 12` `Treatment\nN = 14` `Total\nN = 31`
      * <chr> <chr>             <chr>               <chr>          
      1 "Sex" <NA>              <NA>                <NA>           
      2 "  n" 20                32                  18             
      # i 1 more variable: ..tfrmt_row_grp_lbl <lgl>
      
      [[2]]
      # A tibble: 2 x 5
        Label     `Placebo\nN = 20` `Treatment\nN = 32` `Total\nN = 18`
      * <chr>     <chr>             <chr>               <chr>          
      1 "Age (y)" <NA>              <NA>                <NA>           
      2 "  n"     12                14                  31             
      # i 1 more variable: ..tfrmt_row_grp_lbl <lgl>
      

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_card.R
\name{prep_hierarchical_fill}
\alias{prep_hierarchical_fill}
\title{Fill missing values in hierarchical variables}
\usage{
prep_hierarchical_fill(
  df,
  vars,
  fill = "Any {colname}",
  fill_from_left = FALSE
)
}
\arguments{
\item{df}{(data.frame)}

\item{vars}{(character) a vector of variables to generate pairs from.}

\item{fill}{(character) value to replace with. Defaults to \code{"Any {colname}"},
in which case \code{colname} will be replaced with the name of the column.}

\item{fill_from_left}{(logical) indicating whether to fill from the left
(first) column in the pair. Defaults to \code{FALSE}. If \code{TRUE} it takes
precedence over \code{fill}.}
}
\value{
a data.frame with the same columns as the input, but in which some
the desired columns have been filled pairwise.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Replace \code{NA} values in one column conditional on the same row having a
non-NA value in a different column.

The user supplies a vector of columns from which the pairs will be extracted
with a rolling window. For example \code{vars <- c("A", "B", "C")} will generate
2 pairs \verb{("A", "B")} and \verb{("B", "C")}. Therefore the order of the variables
matters.

In each pair the second column \code{B} will be filled if \code{A} is not missing. One
can choose the value to fill with:
\itemize{
\item \code{"Any {colname}"}, in this case evaluating to \code{"Any B"} is the default.
\item Any other value. For example \code{"Any event"} for an adverse effects table.
\item the value of pair's first column. In this case, the value of \code{A}.
}
}
\examples{
df <- data.frame(
  x = c(1, 2, NA),
  y = c("a", NA, "b"),
  z = rep(NA, 3)
)

prep_hierarchical_fill(
  df,
  vars = c("x", "y")
)

prep_hierarchical_fill(
  df,
  vars = c("x", "y"),
  fill = "foo"
)

prep_hierarchical_fill(
  df,
  vars = c("x", "y", "z"),
  fill_from_left = TRUE
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JSON.R
\name{tfrmt_to_json}
\alias{tfrmt_to_json}
\title{Print to JSON}
\usage{
tfrmt_to_json(tfrmt, path = NULL)
}
\arguments{
\item{tfrmt}{tfrmt to print}

\item{path}{file path to save JSON to. If not provided the JSON will just print to the console}
}
\value{
JSON
}
\description{
Print to JSON
}
\examples{
tfrmt(
  label = label,
  column = column,
  param = param,
  value=value) |>
  tfrmt_to_json()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_to_gt.R
\name{cleaned_data_to_gt}
\alias{cleaned_data_to_gt}
\alias{cleaned_data_to_gt.list}
\alias{cleaned_data_to_gt.default}
\title{Do all the formatting for the GT}
\usage{
cleaned_data_to_gt(.data, tfrmt, .unicode_ws)

\method{cleaned_data_to_gt}{list}(.data, tfrmt, .unicode_ws)

\method{cleaned_data_to_gt}{default}(.data, tfrmt, .unicode_ws)
}
\arguments{
\item{.data}{cleaned dataset}

\item{tfrmt}{tfrmt}

\item{.unicode_ws}{Whether to convert white space to unicode in preparation for output}
}
\value{
gt_group object

GT object
}
\description{
Do all the formatting for the GT

Apply formatting to a list of tables

Apply formatting to a single table
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tfrmt.R
\name{tfrmt}
\alias{tfrmt}
\title{Table Format}
\usage{
tfrmt(
  tfrmt_obj,
  group = vars(),
  label = quo(),
  param = quo(),
  value = quo(),
  column = vars(),
  title,
  subtitle,
  row_grp_plan,
  body_plan,
  col_style_plan,
  col_plan,
  sorting_cols,
  big_n,
  footnote_plan,
  page_plan,
  ...
)
}
\arguments{
\item{tfrmt_obj}{a tfrmt object to base this new format off of}

\item{group}{what are the grouping vars of the input dataset}

\item{label}{what is the label column of the input dataset}

\item{param}{what is the param column of the input dataset}

\item{value}{what is the value column of the input dataset}

\item{column}{what is the column names column in the input dataset}

\item{title}{title of the table}

\item{subtitle}{subtitle of the table}

\item{row_grp_plan}{plan of the row groups blocking. Takes a \code{\link[=row_grp_plan]{row_grp_plan()}}}

\item{body_plan}{combination and formatting of the input data. Takes a \code{\link[=body_plan]{body_plan()}}}

\item{col_style_plan}{how to style columns including alignment (left, right, character) and width. Takes a \code{\link[=col_style_plan]{col_style_plan()}}}

\item{col_plan}{a col_plan object which is used to select, rename, and nest
columns. Takes a \code{\link[=col_plan]{col_plan()}}}

\item{sorting_cols}{which columns determine sorting of output}

\item{big_n}{how to format subject totals ("big Ns") for inclusion in the column labels. Takes a \code{\link[=big_n_structure]{big_n_structure()}}}

\item{footnote_plan}{footnotes to be added to the table. Takes a \code{\link[=footnote_plan]{footnote_plan()}}}

\item{page_plan}{pagination splits to be applied to the table. Takes a \code{\link[=page_plan]{page_plan()}}}

\item{...}{These dots are for future extensions and must be empty.}
}
\value{
tfrmt object
}
\description{
tfrmt, or "table format" is a way to pre-define the non-data components of
your tables, and how the data will be handled once added: i.e. title,
footers, headers, span headers, and cell formats. In addition, tfrmt's can be
layered, building from one table format to the next. For cases where only one
value can be used, the newly defined tfrmt accepts the latest tfrmt
}
\details{
\subsection{NSE and Argument Evaluation}{
\itemize{
\item tfrmt allows users to pass \code{vars}, \code{quo}, and unquoted expressions to a
variety of arguments, such as \code{group}, \code{label}, \code{param}, \code{value},
\code{column}, and \code{sorting_cols}. Users accustomed to tidyverse semantics
should be familiar with this behaviour. However, there is an important
behaviour difference between tfrmt and normal tidyverse functions. Because
the data are not a part of tfrmt, it does not know when a value being
passed to it is intended to be an unquoted expression representing a column
name or an object from the environment. As such, it preferentially uses the
value from the environment over preserving the entry as an expression. For
example, if you have an object "my_object" in your
environment with the value "Hello world", and try to create a tfrmt as
\code{tfrmt(column = my_object)}, it will take the value of "my_object" over
assuming the column argument is an unquoted expression and view the entry
to \code{column} as "Hello World". To pass "my_object" to tfrmt as a column name, use
quotes around the value: \code{tfrmt(columnn = "my_object")}.
\item Additionally, unquoted expressions that match \code{tfrmt}'s other
argument names can cause unexpected results. It is recommended
to put quotes around the value as such:
\code{tfrmt(label = "group")}. In this case, the quoting will prevent \code{tfrmt}
from assigning its \code{group} input value to the \code{label} value.
}
}
}
\section{Images}{

Here are some example outputs:
\if{html}{\out{
<img src="https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/main/images/tfrmt-groups-three-cols-cropped.jpg" alt ="Example showing no groups, one group and two groups" style="width:100\%;">
}}
}

\examples{

tfrmt_spec <- tfrmt(
  label = label,
  column = column,
  param = param,
  value=value)

tfrmt_spec <- tfrmt(
  label = label,
  column = column,
  param = param,
  value=value,
# Set the formatting for values
  body_plan = body_plan(
    frmt_structure(
      group_val = ".default",
      label_val = ".default",
      frmt_combine("{n} {pct}",
           n = frmt("xxx"),
           pct = frmt_when(
                "==100" ~ "(100\%)",
                "==0" ~ "",
                TRUE ~ frmt("(xx.x \%)")
                )
           )
    )
  ),
# Specify column styling plan
  col_style_plan = col_style_plan(
    col_style_structure(col = vars(everything()), align = c(".",","," "))
  ))

tfrmt_spec <- tfrmt(
  group = group,
  label = label,
  column = column,
  param = param,
  value=value,
  sorting_cols = c(ord1, ord2),
  # specify value formatting
  body_plan = body_plan(
    frmt_structure(
      group_val = ".default",
      label_val = ".default",
      frmt_combine("{n} {pct}",
           n = frmt("xxx"),
           pct = frmt_when(
                "==100" ~ "(100\%)",
                "==0" ~ "",
                TRUE ~ frmt("(xx.x \%)")
                )
           )
    ),
    frmt_structure(
        group_val = ".default",
        label_val = "n",
        frmt("xxx")
   ),
    frmt_structure(
        group_val = ".default",
        label_val = c("Mean", "Median", "Min","Max"),
        frmt("xxx.x")
     ),
    frmt_structure(
        group_val = ".default",
        label_val = "SD",
        frmt("xxx.xx")
    ),
    frmt_structure(
        group_val = ".default",
        label_val = ".default",
        p = frmt("")
    ),
    frmt_structure(
        group_val = ".default",
        label_val = c("n","<65 yrs","<12 months","<25"),
        p = frmt_when(
            ">0.99" ~ ">0.99",
            "<0.001" ~ "<0.001",
            TRUE ~ frmt("x.xxx", missing = "")
        )
    )
  ),
  # remove extra cols
  col_plan = col_plan(-grp,
                      -starts_with("ord") ),
  # Specify column styling plan
  col_style_plan = col_style_plan(
    col_style_structure(col = vars(everything()), align = c(".",","," "))
  ),

  # Specify row group plan
  row_grp_plan = row_grp_plan(
    row_grp_structure(
         group_val = ".default",
         element_block(post_space = " ")
    ),
    label_loc = element_row_grp_loc(location = "column")
  )

)


}
\seealso{
\href{https://gsk-biostatistics.github.io/tfrmt/articles/building_blocks.html}{Link to related article}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tfrmt_sigdig.R
\name{tfrmt_sigdig}
\alias{tfrmt_sigdig}
\title{Create tfrmt object from significant digits spec}
\usage{
tfrmt_sigdig(
  sigdig_df,
  group = vars(),
  label = quo(),
  param_defaults = param_set(),
  missing = NULL,
  tfrmt_obj = NULL,
  ...
)
}
\arguments{
\item{sigdig_df}{data frame containing significant digits formatting spec.
Has 1 record per group/label value, and columns for relevant group and/or
label variables, as well as a numeric column \code{sigdig} containing the
significant digits rounding to be applied in addition to the default. If
unique group/label values are represented in multiple rows, this will
result in only one of the \code{sigdig} values being carried through in
implementation.}

\item{group}{what are the grouping vars of the input dataset}

\item{label}{what is the label column of the input dataset}

\item{param_defaults}{Option to override or add to default parameters.}

\item{missing}{missing option to be included in all \code{frmt}s}

\item{tfrmt_obj}{an optional tfrmt object to layer}

\item{...}{These dots are for future extensions and must be empty.}
}
\value{
\code{tfrmt} object with a \code{body_plan} constructed based on the
significant digits data spec and param-level significant digits defaults.
}
\description{
This function creates a tfrmt based on significant digits specifications for
group/label values. The input data spec provided to \code{sigdig_df} will contain
group/label value specifications. \code{tfrmt_sigdig} assumes that these columns
are group columns unless otherwise specified. The user may optionally choose
to pass the names of the group and/or label columns as arguments to the
function.
}
\details{
\subsection{Formats covered}{

Currently covers specifications for \code{frmt} and
\code{frmt_combine}. \code{frmt_when} not supported and must be supplied in additional
\code{tfrmt} that is layered on.
}

\subsection{Group/label variables}{

If the group/label variables are not provided to the arguments, the body_plan
will be constructed from the input data with the following behaviour:
\itemize{
\item If no group or label are supplied, it will be assumed that all columns in the input
data are group columns.
\item If a label variable is provided, but nothing is
specified for group, any leftover columns (i.e. not matching \code{sigdig} or the
supplied label variable name) in the input data will be assumed to be group
columns.
\item If any group variable is provided, any leftover columns (i.e. not
matching \code{sigdig} or the supplied group/label variable) will be disregarded.
}
}
}
\section{Examples}{


\if{html}{\out{<div class="sourceCode r">}}\preformatted{sig_input <- tibble::tribble(
  ~group1,   ~group2, ~sigdig,
  "CHEMISTRY",   ".default", 3,
  "CHEMISTRY",   "ALBUMIN",  1,
  "CHEMISTRY",   "CALCIUM",   1,
  ".default",    ".default",  2
)

# Subset data for the example
data <- dplyr::filter(data_labs, group2 == "BASOPHILS", col1 \%in\% c("Placebo", "Xanomeline Low Dose"))
tfrmt_sigdig(sigdig_df = sig_input,
             group = vars(group1, group2),
             label = rowlbl,
             param_defaults = param_set("[\{n\}]" = NA)) |>
  tfrmt(column = vars(col1, col2),
        param = param,
        value = value,
        sorting_cols = vars(ord1, ord2, ord3),
        col_plan = col_plan(-starts_with("ord"))) |>
  print_to_gt(.data = data)
}\if{html}{\out{</div>}}

\if{html}{\out{
<img src="https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/master/images/example_sigdig.png" alt="Table of Hematology, which are rounded for visits baseline to week 26" style="width:100\%;">
}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_element.R
\name{element_block}
\alias{element_block}
\title{Element block}
\usage{
element_block(post_space = c(NULL, " ", "-"), fill = TRUE)
}
\arguments{
\item{post_space}{Values to show in a new line created after the group block}

\item{fill}{Whether to recycle the value of \code{post_space} to match width of the data. Defaults to \code{TRUE}}
}
\value{
element block object
}
\description{
Element block
}
\examples{

tfrmt_spec <- tfrmt(
  group = grp1,
  label = label,
  param = param,
  value = value,
  column = column,
  row_grp_plan = row_grp_plan(
    row_grp_structure(group_val = ".default", element_block(post_space = "   "))
  ),
  body_plan = body_plan(
    frmt_structure(group_val = ".default", label_val = ".default", frmt("xx"))
  )
)
}
\seealso{
\code{\link[=row_grp_plan]{row_grp_plan()}} for more details on how to group row group
structures, \code{\link[=row_grp_structure]{row_grp_structure()}} for more details on how to specify row group
structures, \code{\link[=element_row_grp_loc]{element_row_grp_loc()}} for more details on how to
specify whether row group titles span the entire table or collapse.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/page_plan.R
\name{page_structure}
\alias{page_structure}
\title{Page structure}
\usage{
page_structure(group_val = NULL, label_val = NULL)
}
\arguments{
\item{group_val}{string or a named list of strings which represent the value of group to split after.
Set to ".default" if the split should occur after every unique value of the variable.}

\item{label_val}{string which represents the value of label to split after.
Set to ".default" if the split should occur after every unique value of
the variable.}
}
\value{
page structure object
}
\description{
Page structure
}
\examples{
# split page after every unique level of the grouping variable
 page_structure(group_val = ".default", label_val = NULL)

 # split page after specific levels
 page_structure(group_val = "grp1", label_val = "lbl3")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/row_group_plan.R
\name{row_grp_plan}
\alias{row_grp_plan}
\title{Row Group Plan}
\usage{
row_grp_plan(..., label_loc = element_row_grp_loc(location = "indented"))
}
\arguments{
\item{...}{Row group structure objects separated by commas}

\item{label_loc}{\code{\link[=element_row_grp_loc]{element_row_grp_loc()}} object specifying location}
}
\value{
row_grp_plan object
}
\description{
Define the look of the table groups on the output. This function allows you to
add spaces after blocks and allows you to control how the groups are viewed
whether they span the entire table or are nested as a column.
}
\examples{


  ## single grouping variable example
  sample_grp_plan <- row_grp_plan(
    row_grp_structure(group_val = c("A","C"), element_block(post_space = "---")),
    row_grp_structure(group_val = c("B"), element_block(post_space = " ")),
    label_loc = element_row_grp_loc(location = "column")
  )

  ## example with multiple grouping variables
  sample_grp_plan <- row_grp_plan(
     row_grp_structure(group_val = list(grp1 = "A", grp2 = "b"), element_block(post_space = " ")),
     label_loc = element_row_grp_loc(location = "spanning")
     )

}
\seealso{
\code{\link[=row_grp_structure]{row_grp_structure()}} for more details on how to specify row group
structures, \code{\link[=element_block]{element_block()}} for more details on how to specify spacing
between each group, \code{\link[=element_row_grp_loc]{element_row_grp_loc()}} for more details on how to
specify whether row group titles span the entire table or collapse.

\href{https://gsk-biostatistics.github.io/tfrmt/articles/row_grp_plan.html}{Link to related article}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/display_insights.R
\name{display_val_frmts}
\alias{display_val_frmts}
\title{Display formatted values}
\usage{
display_val_frmts(tfrmt, .data, mock = FALSE, col = NULL)
}
\arguments{
\item{tfrmt}{tfrmt object to apply to the data}

\item{.data}{Data to apply the tfrmt to}

\item{mock}{Mock table? TRUE or FALSE (default)}

\item{col}{Column value to align on from \code{column} variable. May be a quoted
or unquoted column name, a tidyselect semantic, or a span_structure.}
}
\value{
text representing character vector of formatted values to be copied and modified in the col_style_plan
}
\description{
A helper for creating positional-alignment specifications for the col_style_plan.
Returns all unique formatted values to appear in the column(s) specified. Numeric values are represented by x's.
}
\examples{
 tf_spec <- tfrmt(
 group = c(rowlbl1,grp),
 label = rowlbl2,
 column = column,
 param = param,
 value = value,
 sorting_cols = c(ord1, ord2),
 body_plan = body_plan(
 frmt_structure(group_val = ".default", label_val = ".default", frmt_combine("{n} ({pct} \%)",
                                                                             n = frmt("xxx"),
                                                                             pct = frmt("xx.x"))),
 frmt_structure(group_val = ".default", label_val = "n", frmt("xxx")),
 frmt_structure(group_val = ".default", label_val = c("Mean", "Median", "Min","Max"),
                             frmt("xxx.x")),
 frmt_structure(group_val = ".default", label_val = "SD", frmt("xxx.xx")),
 frmt_structure(group_val = ".default", label_val = ".default",
                            p = frmt_when(">0.99" ~ ">0.99",
                                          "<0.15" ~ "<0.15",
                                          TRUE ~ frmt("x.xxx", missing = "")))
))

 display_val_frmts(tf_spec, data_demog, col = vars(everything()))
 display_val_frmts(tf_spec, data_demog, col = "p-value")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_to_gt.R
\name{print_mock_gt}
\alias{print_mock_gt}
\title{Print mock table to GT}
\usage{
print_mock_gt(
  tfrmt,
  .data = NULL,
  .default = 1:3,
  n_cols = NULL,
  .unicode_ws = TRUE
)
}
\arguments{
\item{tfrmt}{tfrmt the mock table will be based off of}

\item{.data}{Optional data. If this is missing, group values, labels values
and parameter values will be estimated based on the tfrmt}

\item{.default}{sequence to replace the default values if a dataset isn't
provided}

\item{n_cols}{the number of columns. This will only be used if mock data isn't
provided. If not supplied, it will default to using the \code{col_plan} from the
\code{tfrmt}. If neither are available it will use 3.}

\item{.unicode_ws}{Whether to convert white space to unicode in preparation for output}
}
\value{
a stylized gt object
}
\description{
Print mock table to GT
}
\section{Examples}{


\if{html}{\out{<div class="sourceCode r">}}\preformatted{
  # Create tfrmt specification
  tfrmt_spec <- tfrmt( label = label, column =
  column, param = param, body_plan = body_plan( frmt_structure(group_val =
  ".default", label_val = ".default", frmt_combine( "\{count\} \{percent\}",
  count = frmt("xxx"), percent = frmt_when("==100"~ frmt(""), "==0"~ "",
  "TRUE" ~ frmt("(xx.x\%)")))) ))

  # Print mock table using default
  print_mock_gt(tfrmt = tfrmt_spec)

}\if{html}{\out{</div>}}

\if{html}{\out{ <img src="https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/master/images/example_print_mock_gt1.png" alt = "Simple 3 by 3 table without values" style="width:50\%;"> }}

\if{html}{\out{<div class="sourceCode r">}}\preformatted{  # Create mock data
  df <- tidyr::crossing(label = c("label 1", "label 2",
  "label 3"), column = c("placebo", "trt1", "trt2"), param = c("count",
  "percent"))

  # Print mock table using mock data
  print_mock_gt(tfrmt_spec, df)

}\if{html}{\out{</div>}}

\if{html}{\out{ <img src="https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/master/images/example_print_mock_gt2.png" alt = "Simple 3 by 3 table without values, but with column names" style="width:50\%;"> }}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/body_plan.R
\name{body_plan}
\alias{body_plan}
\title{Table Body Plan}
\usage{
body_plan(...)
}
\arguments{
\item{...}{list of frmt_structures defining the body formatting}
}
\value{
body_plan object
}
\description{
Define the formatting of the body contents of the table through a series of
frmt_structures. Structures get applied in order from bottom up, so the last
added structure is the first applied.
}
\examples{

  tfrmt_spec<- tfrmt(
      title = "Table Title",
      body_plan = body_plan(
        frmt_structure(
          group_val = c("group1"),
          label_val = ".default",
          frmt("XXX")
        )
      )
     )

}
\seealso{
\code{\link[=frmt_structure]{frmt_structure()}} defines which rows the formats will be applied
to, and \code{\link[=frmt]{frmt()}}, \code{\link[=frmt_combine]{frmt_combine()}}, and \code{\link[=frmt_when]{frmt_when()}} define the format
semantics.

\href{https://gsk-biostatistics.github.io/tfrmt/articles/body_plan.html}{Link to related article}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/col_style_plan.R
\name{col_style_structure}
\alias{col_style_structure}
\title{Column Style Structure}
\usage{
col_style_structure(
  col,
  align = NULL,
  type = c("char", "pos"),
  width = NULL,
  ...
)
}
\arguments{
\item{col}{Column value to align on from \code{column} variable. May be a quoted
or unquoted column name, a tidyselect semantic, or a span_structure.}

\item{align}{Alignment to be applied to column. Defaults to \code{left} alignment. See details for acceptable values.}

\item{type}{Type of alignment: "char" or "pos", for character alignment (default), and positional alignment, respectively. Positional alignment allows for aligning over multiple positions in the column.}

\item{width}{Width to apply to the column in number of characters. Acceptable values include a
numeric value, or a character string of a number.}

\item{...}{These dots are for future extensions and must be empty}
}
\value{
col_style_structure object
}
\description{
Column Style Structure
}
\details{
Supports alignment and width setting of data value columns (values found in the \code{column} column). Row group and label columns are left-aligned by default. Acceptable input values for \code{align} differ by type = "char" or "pos":
\subsection{Character alignment (type = "char"):}{
\itemize{
\item "left" for left alignment
\item "right" for right alignment"
\item supply a vector of character(s) to align on. If more than
one character is provided, alignment will be based on the first occurrence
of any of the characters. For alignment based on white space, leading white
spaces will be ignored.
}
}

\subsection{Positional alignment (type = "pos"):}{

supply a vector of strings covering all formatted cell values, with numeric values represented as x's. These values can be created manually or obtained by utilizing the helper \code{display_val_frmts()}. Alignment positions will be represented by vertical bars. For example, with starting values: c("12.3", "(5\%)", "2.35 (10.23)") we can align all of the first sets of decimals and parentheses by providing align = c("xx|.x", "||(x\%)", "x|.xx |")
}
}
\examples{

 plan <- col_style_plan(
    col_style_structure(col = "my_var",
                        align = c("xx| |(xx\%)",
                                  "xx|.x |(xx.x - xx.x)"),
                        type = "pos", width = 100),
    col_style_structure(col = vars(four), align = "right", width = 200),
    col_style_structure(col = vars(two, three), align = c(".", ",", " ")),
    col_style_structure(col = c(two, three), width = 25),
    col_style_structure(col = two, width = 25),
    col_style_structure(col = span_structure(span = value, col = val2),
                        width = 25)
   )

}
\seealso{
\code{\link[=col_style_plan]{col_style_plan()}} for more information on how to combine
col_style_structure()'s together to form a plan.

\href{https://gsk-biostatistics.github.io/tfrmt/articles/col_style_plan.html}{Link to related article}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/display_insights.R
\name{display_row_frmts}
\alias{display_row_frmts}
\title{Display formatting applied to each row}
\usage{
display_row_frmts(tfrmt, .data, convert_to_txt = TRUE)
}
\arguments{
\item{tfrmt}{tfrmt object to apply to the data}

\item{.data}{Data to apply the tfrmt to}

\item{convert_to_txt}{Logical value converting formatting to text, by default
\code{TRUE}}
}
\value{
formatted tibble
}
\description{
Used when debugging formatting, it is an easy way to allow you to see which
formats are applied to each row in your dataset.
}
\examples{
 library(dplyr)
 library(tidyr)

 tfrmt_spec <- tfrmt(
 label = label,
 column = column,
 param = param,
 value=value,
 body_plan = body_plan(
   frmt_structure(group_val = ".default", label_val = ".default",
                  frmt_combine(
                    "{count} {percent}",
                    count = frmt("xxx"),
                    percent = frmt_when("==100"~ frmt(""),
                                        "==0"~ "",
                                        "TRUE" ~ frmt("(xx.x\%)"))))
 ))

 # Create data
 df <- tidyr::crossing(label = c("label 1", "label 2"),
                column = c("placebo", "trt1"),
                param = c("count", "percent")) |>
   dplyr::mutate(value=c(24,19,2400/48,1900/38,5,1,500/48,100/38))

 display_row_frmts(tfrmt_spec,df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_data.R
\name{extract_data}
\alias{extract_data}
\title{Extract underlying data from tfrmt output}
\usage{
extract_data(x, col_delim = "_")
}
\arguments{
\item{x}{A \code{gt_tbl} or \code{gt_group} object (usually the output of \code{print_to_gt()}).}

\item{col_delim}{Character string to replace the internal "tlang_delim"
separator in column names only for tables with spanning headers. Defaults to "_".}
}
\value{
If \code{gt_tbl}, a single data frame. If \code{gt_group}, a list of data frames.
}
\description{
Following a call to \code{print_to_gt}, this function extracts the underlying
data frame(s) from the resulting \code{gt} or \code{gt_group} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/col_plan.R
\name{col_plan}
\alias{col_plan}
\alias{span_structure}
\title{Define the Column Plan & Span Structures}
\usage{
col_plan(..., .drop = FALSE)

span_structure(...)
}
\arguments{
\item{...}{For a col_plan and span_structure,
<\code{\link[dplyr:dplyr_tidy_select]{tidy-select}}> arguments, unquoted expressions
separated by commas, and span_structures. span_structures must have the
arguments named to match the name the column in the input data has to identify the correct columns. See the examples}

\item{.drop}{Boolean. Should un-listed columns be dropped from the data.
Defaults to FALSE.}
}
\value{
col_plan object

span_structure object
}
\description{
Using <\code{\link[dplyr:dplyr_tidy_select]{tidy-select}}> expressions and a series
span_structures, define the order of the columns. The selection follows "last
selected" principals, meaning columns are moved to the \emph{last} selection as
opposed to preserving the first location.
}
\details{
\subsection{Column Selection}{

When col_plan gets applied and is used to create the output table, the
underlying logic sorts out which column specifically is being selected. If a column
is selected twice, the \emph{last} instance in which the column is selected will be
the location it gets rendered.

Avoid beginning the \code{col_plan()} column selection with a deselection (i.e.
\code{col_plan(-col1)}, \code{col_plan(-starts_with("value")))}. This will
result in the table preserving all columns not "de-selected" in the
statement, and the order of the columns not changed. It is preferred when
creating the \code{col_plan()} to identify all the columns planned on
preserving in the order they are wished to appear, or if
<\code{\link[dplyr:dplyr_tidy_select]{tidy-select}}> arguments - such as
\code{\link[dplyr]{everything}}- are used, identify the de-selection after
the positive-selection.

Alternatively, once the gt table is produced, use the
\code{\link[gt]{cols_hide}} function to remove un-wanted columns.
}
}
\section{Images}{

Here are some example outputs:

\if{html}{\out{
<img src="https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/main/images/tfrmt-span_structure-cropped.jpg" alt ="Example of a dataset being turned into a table with spanning columns" style="width:100\%;">
}}
}

\examples{

library(dplyr)

## select col_1 as the first column, remove col_last, then create spanning
## structures that have multiple levels
##
## examples also assume the tfrmt has the column argument set to c(c1, c2, c3)
##
spanning_col_plan_ex <- col_plan(
 col_1,
 -col_last,
 span_structure(
   c1 = "Top Label Level 1",
   c2 = "Second Label Level 1.1",
   c3 = c(col_3, col_4)
 ),
 span_structure(
   c1 = "Top Label Level 1",
   c2 = "Second Label Level 1.2",
   c3 = starts_with("B")
   ),
 span_structure(
   c1 = "Top Label Level 1",
   c3 = col_5
 ),
 span_structure(
   c2 = "Top Label Level 2",
   c3 = c(col_6, col_7)
 )
)

## select my_col_1 as the first column, then
## rename col_2 to new_col_1 and put as the
## second column, then select the rest of the columns
renaming_col_plan_ex <- col_plan(
   my_col_1,
   new_col_1 = col_2,
   everything()
 )

renaming_col_plan_ex2 <- col_plan(
   my_col_1,
   new_col_1 = col_2,
   span_structure(
    c1 = c(`My Favorite span name` = "Top Label Level 1"),
    c3 = c(`the results column` = col_5)
   )
 )

## To one or more stub headers rename the corresponding group/label variables in the column plan.
## If multiple group variables exist, any of them can be renamed.
## Note: Multiple stub headers are only possible if the `row_grp_plan` label
## location is set to "column". Otherwise, if more than one group/label column is renamed,
## {tfrmt} will use the highest level group name available.

 renaming_group <- col_plan(
    my_grp = group, # rename group
    label,
    starts_with("col")
  )

}
\seealso{
\href{https://gsk-biostatistics.github.io/tfrmt/articles/col_plan.html}{Link to related article}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tfrmt_n_pct.R
\name{tfrmt_n_pct}
\alias{tfrmt_n_pct}
\title{N Percent Template}
\usage{
tfrmt_n_pct(
  n = "n",
  pct = "pct",
  pct_frmt_when = frmt_when("==100" ~ frmt(""), ">99" ~ frmt("(>99\%)"), "==0" ~ "", "<1"
    ~ frmt("(<1\%)"), "TRUE" ~ frmt("(xx.x\%)")),
  tfrmt_obj = NULL
)
}
\arguments{
\item{n}{name of count (n) value in the parameter column}

\item{pct}{name of percent (pct) value in the parameter column}

\item{pct_frmt_when}{formatting to be used on the the percent values}

\item{tfrmt_obj}{an optional tfrmt object to layer}
}
\value{
tfrmt object
}
\description{
This function creates an tfrmt for an n \% table, so count based table. The
parameter values for n and percent can be provided (by default it will assume
\code{n} and \code{pct}). Additionally the \code{frmt_when} for formatting the percent can
be specified. By default 100\% and 0\% will not appear and everything between
99\% and 100\% and 0\% and 1\% will be rounded using greater than (>) and less
than (<) signs respectively.
}
\section{Examples}{


\if{html}{\out{<div class="sourceCode r">}}\preformatted{print_mock_gt(tfrmt_n_pct())
}\if{html}{\out{</div>}}

\if{html}{\out{
<img src="https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/master/images/example_n_percent.png" alt="3 by 3 table" style="width:50\%;">
}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/footnote_plan.R
\name{footnote_structure}
\alias{footnote_structure}
\title{Footnote Structure}
\usage{
footnote_structure(
  footnote_text,
  column_val = NULL,
  group_val = NULL,
  label_val = NULL
)
}
\arguments{
\item{footnote_text}{string with text for footnote}

\item{column_val}{string or a named list of strings which represent the column to apply the footnote to}

\item{group_val}{string or a named list of strings which represent the value of group to apply the footnote to}

\item{label_val}{string which represents the value of label to apply the footnote to}
}
\value{
footnote structure object
}
\description{
Footnote Structure
}
\examples{

# Adds a source note aka a footnote without a symbol in the table
footnote_structure <- footnote_structure(footnote_text = "Source Note")

# Adds a footnote to the 'Placebo' column
footnote_structure <- footnote_structure(footnote_text = "Text",
                               column_val = "Placebo")

# Adds a footnote to either 'Placebo' or 'Treatment groups' depending on which
# which is last to appear in the column vector
footnote_structure <- footnote_structure(footnote_text = "Text",
      column_val = list(col1 = "Placebo", col2= "Treatment groups"))

# Adds a footnote to the 'Adverse Event' label
footnote_structure <- footnote_structure("Text", label_val = "Adverse Event")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shuffle_card.R
\name{shuffle_card}
\alias{shuffle_card}
\title{Shuffle \code{cards}}
\usage{
shuffle_card(
  x,
  by = NULL,
  trim = TRUE,
  order_rows = TRUE,
  fill_overall = "Overall {colname}",
  fill_hierarchical_overall = "Any {colname}"
)
}
\arguments{
\item{x}{an ARD data frame of class 'card'}

\item{by}{Grouping variable(s) used in calculations. Defaults to \code{NULL}. If
available (i.e. if \code{x} comes from a stacking function), \code{attributes(x)$by}
will be used instead of \code{by}.}

\item{trim}{logical representing whether or not to trim away \code{fmt_fun},
\code{error}, and \code{warning} columns}

\item{order_rows}{logical representing whether or not to apply
\code{cards::tidy_ard_row_order()} to sort the rows}

\item{fill_overall}{scalar to fill missing grouping or variable levels. If a
character is passed, then it is processed with \code{\link[glue:glue]{glue::glue()}} where the
colname element is available to inject into the string,
e.g. \verb{Overall \{colname\}} may resolve to \code{"Overall AGE"} for an \code{AGE}
column. Default is \verb{Overall \{colname\}}. If \code{NA} then no fill will occur.}

\item{fill_hierarchical_overall}{scalar to fill variable levels for overall
hierarchical calculations. If a character is passed, then it is processed
with \code{\link[glue:glue]{glue::glue()}} where the colname element is available to inject into
the string, e.g. \verb{Any \{colname\}} may resolve to \code{"Any AESOC"} for an
\code{AESOC} column. Default is \verb{Any \{colname\}}. If \code{NA} then no fill will
occur.}
}
\value{
a tibble
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

This function ingests an ARD object of class \code{card} and shuffles the information to prepare
for analysis. Helpful for streamlining across multiple ARDs.
}
\examples{
\dontrun{
cards::bind_ard(
  cards::ard_categorical(cards::ADSL, by = "ARM", variables = "AGEGR1"),
  cards::ard_categorical(cards::ADSL, variables = "ARM")
) |>
  shuffle_card()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tfrmt_sigdig.R
\name{param_set}
\alias{param_set}
\title{Set custom parameter-level significant digits rounding}
\usage{
param_set(...)
}
\arguments{
\item{...}{Series of name-value pairs, optionally formatted using
\code{glue::glue()} syntax (note \code{glue} syntax is required for combined
parameters).The name represents the parameter and the value represents the number of places to round the parameter to.
For combined parameters (e.g., \code{"{min}, {max}"}), value should
be a vector of the same length (e.g., c(1,1)).}
}
\value{
list of default parameter-level significant digits rounding
}
\description{
Set custom parameter-level significant digits rounding
}
\details{
Type \code{param_set()} in console to view package defaults. Use of the
function will add to the defaults and/or override included defaults of the
same name. For values that are integers, use \code{NA} so no decimal places will
be added.
}
\examples{
# View included defaults
param_set()

# Update the defaults
param_set("{mean} ({sd})" = c(2,3), "pct" = 1)

# Separate mean and SD to different lines
param_set("mean" = 2, "sd" = 3)

# Add formatting using the glue syntax
param_set("{pct} \%" = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_card.R
\name{prep_combine_vars}
\alias{prep_combine_vars}
\title{Combine variables}
\usage{
prep_combine_vars(df, vars, remove = TRUE)
}
\arguments{
\item{df}{(data.frame)}

\item{vars}{(character) a vector of variables to unite. If a single variable
is supplied, the input is returned unchanged.}

\item{remove}{If \code{TRUE}, remove input columns from output data frame.}
}
\value{
a data.frame with an additional column, called \code{variable_level} or
the input unchanged.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

A wrapper around \code{tidyr::unite()} which pastes several columns into one.
In addition it checks the output is identical to \code{dplyr::coalesce()}. If not
identical, the input data.frame is returned unchanged. Useful for uniting
sparsely populated columns, for example when processing an ard that was
created with \code{\link[cards:ard_stack]{cards::ard_stack()}} then shuffled with \verb{[shuffle_card()]}.

If the data is the result of a hierarchical ard stack (with
\code{\link[cards:ard_stack_hierarchical]{cards::ard_stack_hierarchical()}} or
\code{\link[cards:ard_stack_hierarchical]{cards::ard_stack_hierarchical_count()}}), the input is returned unchanged.
This is assessed from the information in the \code{context} column which needs to
be present. If the input data does not have a \code{context} column, the input
will be returned unmodified.
}
\examples{
df <- data.frame(
  a = 1:6,
  context = rep("categorical", 6),
  b = c("a", rep(NA, 5)),
  c = c(NA, "b", rep(NA, 4)),
  d = c(NA, NA, "c", rep(NA, 3)),
  e = c(NA, NA, NA, "d", rep(NA, 2)),
  f = c(NA, NA, NA, NA, "e", NA),
  g = c(rep(NA, 5), "f")
)

prep_combine_vars(
  df,
  vars = c("b", "c", "d", "e", "f", "g")
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tfrmt_layer.R
\name{update_group}
\alias{update_group}
\title{Remap group values in a tfrmt}
\usage{
update_group(tfrmt, ...)
}
\arguments{
\item{tfrmt}{a \code{tfrmt}}

\item{...}{Use new_name = old_name to rename selected variables}
}
\value{
A \code{tfrmt} with the \code{group} variables updated in all places

tfrmt object with updated groups#'
}
\description{
Remap group values in a tfrmt
}
\examples{

tfrmt_spec <- tfrmt(
    group = c(group1, group2),
    body_plan  = body_plan(
      frmt_structure(
         group_val = list(group2 = "value"),
         label_val = ".default",
         frmt("XXX")
         ),
     frmt_structure(
         group_val = list(group1 = "value", group2 = "value"),
         label_val = ".default",
         frmt("XXX")
       )
    ))

tfrmt_spec \%>\%
  update_group(New_Group = group1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_element.R
\name{element_row_grp_loc}
\alias{element_row_grp_loc}
\title{Element Row Group Location}
\usage{
element_row_grp_loc(
  location = c("indented", "spanning", "column", "noprint", "gtdefault"),
  indent = "  "
)
}
\arguments{
\item{location}{Location of the row group labels. Specifying 'indented'
combines all group and label variables into a single column with each
sub-group indented under its parent. 'spanning' and 'column' retain the
highest level group variable in its own column and combine all remaining
group and label variables into a single column with sub-groups indented. The
highest level group column will either be printed as a spanning header or in
its own column in the gt. The 'noprint' option allows the user to suppress
group values from being printed. Finally, the 'gtdefault' option allows
users to use the 'gt' defaults for styling multiple group columns.}

\item{indent}{A string of the number of spaces you want to indent}
}
\value{
element_row_grp_loc object
}
\description{
Element Row Group Location
}
\section{Images}{
 Here are some example outputs:

\if{html}{\out{ <img src="https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/main/images/tfrmt-row_group_plan-cropped.jpg" alt="Examples showing the difference between the row group locations" style="width:100\%;"> }}
}

\examples{

tfrmt_spec <- tfrmt(
  group = c(grp1, grp2),
  label = label,
  param = param,
  value = value,
  column = column,
  row_grp_plan = row_grp_plan(label_loc = element_row_grp_loc(location = "noprint")),
  body_plan = body_plan(
    frmt_structure(group_val = ".default", label_val = ".default", frmt("xx"))
  )
)

}
\seealso{
\code{\link[=row_grp_plan]{row_grp_plan()}} for more details on how to group row group
structures, \code{\link[=row_grp_structure]{row_grp_structure()}} for more details on how to specify row
group structures, \code{\link[=element_block]{element_block()}} for more details on how to specify
spacing between each group.

\href{https://gsk-biostatistics.github.io/tfrmt/articles/row_grp_plan.html}{Link
to related article}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/col_style_plan.R
\name{col_style_plan}
\alias{col_style_plan}
\title{Column Style Plan}
\usage{
col_style_plan(...)
}
\arguments{
\item{...}{series of col_style_structure objects}
}
\value{
col_style_plan object
}
\description{
Define how the columns of the table body should be aligned, whether left,
right or on a specific character(s).
}
\examples{

 plan <- col_style_plan(
    col_style_structure(col = "my_var", align = "left", width = 100),
    col_style_structure(col = vars(four), align = "right"),
    col_style_structure(col = vars(two, three), align = c(".", ",", " "))
   )



}
\seealso{
\code{\link[=col_style_structure]{col_style_structure()}} for more information on how to specify how to and which columns to align.

\href{https://gsk-biostatistics.github.io/tfrmt/articles/col_style_plan.html}{Link to related article}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tfrmt_layer.R
\name{layer_tfrmt}
\alias{layer_tfrmt}
\title{Layer tfrmt objects together}
\usage{
layer_tfrmt(x, y, ..., join_body_plans = TRUE)
}
\arguments{
\item{x, y}{tfrmt objects that need to be combined}

\item{...}{arguments passed to layer_tfrmt_arg functions for combining different tfrmt elements}

\item{join_body_plans}{should the \code{body_plans} be combined, or just keep styling in y. See details: join_body_plans for more details.}
}
\value{
tfrmt object
}
\description{
Provide utility for layering tfrmt objects together. If both tfrmt's have
values, it will preferentially choose the second tfrmt by default. This is an
alternative to piping together tfrmt's
}
\details{
\subsection{join_body_plan}{

When combining two body_plans, the body plans will stack together, first the
body plan from x tfrmt then y tfrmt. This means that frmt_structures in y
will take priority over those in x.

Combining two tfrmt with large body_plans can lead to slow table evaluation.
Consider setting \code{join_body_plan} to \code{FALSE}. Only the y \code{body_plan} will be
preserved.
}
}
\examples{

tfrmt_1 <- tfrmt(title = "title1")

tfrmt_2 <- tfrmt(title = "title2",subtitle = "subtitle2")

layered_table_format <- layer_tfrmt(tfrmt_1, tfrmt_2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/footnote_plan.R
\name{footnote_plan}
\alias{footnote_plan}
\title{Footnote Plan}
\usage{
footnote_plan(
  ...,
  marks = c("numbers", "letters", "standard", "extended"),
  order = c("marks_first", "preserve_order", "marks_last")
)
}
\arguments{
\item{...}{a series of \code{\link[=footnote_structure]{footnote_structure()}} separated by commas}

\item{marks}{type of marks required for footnotes, properties inherited from
tab_footnote in 'gt'. Available options are "numbers", "letters",
"standard" and "extended" (standard for a traditional set of 4 symbols,
extended for 6 symbols). The default option is set to "numbers".}

\item{order}{how to order footnotes in the footer. Options are "marks_first" (anchored notes appear first, default),
"preserve_order" or "marks_last" (general notes appear first).}
}
\value{
footnote plan object
}
\description{
Defining the location and content of footnotes with a series of footnote
structures. Each structure is a footnote and can be applied in multiple locations.
}
\examples{

# Adds a footnote indicated by letters rather than numbers to Group 1
footnote_plan <- footnote_plan(
    footnote_structure(footnote_text = "footnote", group_val = "Group 1"),
    marks = "letters")

# Adds a footnote to the 'Placebo' column
footnote_plan <- footnote_plan(
    footnote_structure(footnote_text = "footnote", column_val = "Placebo"),
    marks = "numbers")

# Preserve order of footnotes
footnote_plan <- footnote_plan(
    footnote_structure(footnote_text = "footnote 1", group_val = "Group 1"),
    footnote_structure(footnote_text = "footnote 2", column_val = "Placebo"),
    order = "marks_first")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/frmt_plans.R
\name{frmt_structure}
\alias{frmt_structure}
\title{Format Structure Object}
\usage{
frmt_structure(group_val = ".default", label_val = ".default", ...)
}
\arguments{
\item{group_val}{A string or a named list of strings which represent the
value of group should be when the given frmt is implemented}

\item{label_val}{A string which represent the value of label should be when
the given frmt is implemented}

\item{...}{either a \code{\link[=frmt]{frmt()}}, \code{\link[=frmt_combine]{frmt_combine()}}, or a \code{\link[=frmt_when]{frmt_when()}} object.
This can be named to also specify the parameter value}
}
\value{
frmt_structure object
}
\description{
Function needed to create a frmt_structure object, which is a building block
of \code{\link[=body_plan]{body_plan()}}. This specifies the rows the format will be applied to.
}
\section{Images}{

Here are some example outputs:
\if{html}{\out{
<img src="https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/main/images/tfrmt-frmts.jpg" alt = "Example comparing fmt, frmt_combine, and frmt_when" style="width:100\%;">
}}
}

\examples{

sample_structure <- frmt_structure(
          group_val = c("group1"),
          label_val = ".default",
          frmt("XXX")
        )
## multiple group columns
sample_structure <- frmt_structure(
          group_val = list(grp_col1 = "group1", grp_col2 = "subgroup3"),
          label_val = ".default",
          frmt("XXX")
        )

}
\seealso{
\code{\link[=body_plan]{body_plan()}} combines the frmt_structures to be applied to the
table body, and \code{\link[=frmt]{frmt()}}, \code{\link[=frmt_combine]{frmt_combine()}}, and \code{\link[=frmt_when]{frmt_when()}} define the
format semantics.

\href{https://gsk-biostatistics.github.io/tfrmt/articles/body_plan.html}{Link to related article}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/big_n.R
\name{big_n_structure}
\alias{big_n_structure}
\title{Big N Structure}
\usage{
big_n_structure(param_val, n_frmt = frmt("\\nN = xx"), by_page = FALSE)
}
\arguments{
\item{param_val}{row value(s) of the parameter column for which the values
are big N's}

\item{n_frmt}{\code{\link[=frmt]{frmt()}} to control the formatting of the big N's}

\item{by_page}{Option to include different big N's for each group-defined set
of pages (defined by any variables set to \code{".default"} in the \code{page_plan}).
Default is \code{FALSE}, meaning only the overall Ns are applied}
}
\value{
big_n_structure object
}
\description{
Big N structure allows you to specify which values should become the subject
totals ("big N" values) and how they should be formatted in the table's
column labels. Values are specified by providing the value(s) of the \code{param}
column for which the values are big N's. This will remove these from the
body of the table and place them into columns matching the values in the
column column(s). The default formatting is \code{N = xx}, on its own line, but
that can be changed by providing a different \code{frmt()} to \code{n_frmt}.
}
\seealso{
\href{https://gsk-biostatistics.github.io/tfrmt/articles/big_ns.html}{Link to related article}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apply_frmt_methods.R
\name{apply_frmt}
\alias{apply_frmt}
\alias{apply_frmt.frmt}
\alias{apply_frmt.frmt_combine}
\alias{apply_frmt.frmt_when}
\title{Apply formatting}
\usage{
apply_frmt(frmt_def, .data, value, mock = FALSE, ...)

\method{apply_frmt}{frmt}(frmt_def, .data, value, mock = FALSE, ...)

\method{apply_frmt}{frmt_combine}(
  frmt_def,
  .data,
  value,
  mock = FALSE,
  param,
  column,
  label,
  group,
  ...
)

\method{apply_frmt}{frmt_when}(frmt_def, .data, value, mock = FALSE, ...)
}
\arguments{
\item{frmt_def}{formatting to be applied}

\item{.data}{data, but only what is getting changed}

\item{value}{value symbol should only be one}

\item{mock}{Logical value is this is for a mock or not. By default \code{FALSE}}

\item{...}{additional arguments for methods}

\item{param}{param column as a quosure}

\item{column}{column columns as a list of quosures}

\item{label}{label column as a quosure}

\item{group}{group column as a list of quosures}
}
\value{
formatted dataset
}
\description{
Apply formatting
}
\examples{

library(tibble)
library(dplyr)
# Set up data
df <- tibble(x = c(20.12,34.54,12.34))

apply_frmt(
 frmt_def = frmt("XX.X"),
 .data=df,
 value=quo(x))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JSON.R
\name{json_to_tfrmt}
\alias{json_to_tfrmt}
\title{json to tfrmt}
\usage{
json_to_tfrmt(path = NULL, json = NULL)
}
\arguments{
\item{path}{location of the json file to read in}

\item{json}{json object to read in. By default this is null. This function
will read in json object preferentially. So if both a path and a json
object are supplied the json object will be read in.}
}
\description{
Reader to read JSON files/objects into tfrmt objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_efficacy}
\alias{data_efficacy}
\title{Efficacy Analysis Results Data}
\format{
A data frame with 70 rows and 7 variables:
\describe{
\item{group}{highest level row labels}
\item{label}{more specific row labels}
\item{column}{column names}
\item{param}{parameter to explain each value}
\item{value}{values to put in a table}
\item{ord1}{controls ordering}
\item{ord2}{more ordering controls}
}
}
\usage{
data_efficacy
}
\description{
A dataset containing the results needed for an Efficacy table. Using the
CDISC pilot data for ADAS-Cog(11).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/page_plan.R
\name{page_plan}
\alias{page_plan}
\title{Page Plan}
\usage{
page_plan(
  ...,
  note_loc = c("noprint", "preheader", "subtitle", "source_note"),
  max_rows = NULL,
  transform = NULL
)
}
\arguments{
\item{...}{a series of \code{\link[=page_structure]{page_structure()}} separated by commas}

\item{note_loc}{Location of the note describing each table's subset value(s).
Useful if the \code{page_structure} contains only ".default" values (meaning the
table is split by every unique level of a grouping variable), and that
variable is dropped in the col_plan. \code{preheader} only available for rtf output.}

\item{max_rows}{Option to set a maximum number of rows per page. Takes a
numeric value.}

\item{transform}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}} optional, a function
or formula to transform the page label.

It should take a character vector as input and return a modified character
vector as output.

A \strong{function} is used as is, e.g. \code{function(x) {stringr::str_replace(x, "grp", "Group")}}

A \strong{formula}, e.g. \code{~ stringr::str_remove_all(.x, "_")} is converted to a
function. This allows you to create more compact anonymous functions
(lambdas).}
}
\value{
page_plan object
}
\description{
Defining the location and/or frequency of page splits with a series of
page_structure's and the row_every_n argument, respectively.
}
\examples{
 # use of page_struct
 page_plan(
   page_structure(
     group_val = "grp1",
     label_val = "lbl1"
   )
 )

 # use of #  rows
 page_plan(
   max_rows = 5
 )

 # use of transform with a formula
 page_plan(
   page_structure(group_val = ".default"),
   transform = ~ stringr::str_replace(.x, "grp", "Group")
 )

 # use of transform with a function
 transformation_function <- function(x) {
   stringr::str_replace(x, "grp", "Group")
 }

 page_plan(
   page_structure(group_val = ".default"),
   transform = transformation_function
 )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tfrmt_layer.R
\name{reset_component}
\alias{reset_component}
\title{Reset or Remove a Component from a tfrmt Object}
\usage{
reset_component(tfrmt_obj, component_name)
}
\arguments{
\item{tfrmt_obj}{A \code{tfrmt} object to be modified.}

\item{component_name}{A character string specifying the name of the component
to reset/remove (e.g., "body_plan").}
}
\value{
A modified \code{tfrmt} object with the specified component removed.
}
\description{
This utility function allows users to remove a specific component (e.g., \code{body_plan},
\code{row_grp_plan}, \code{col_plan}) from a \code{tfrmt} object by setting it to \code{NULL}.
}
\examples{
\dontrun{
  my_tfrmt <- tfrmt(column = "col1", label = "label1")
  # Remove the column component
  reset_tfrmt <- reset_component(my_tfrmt, "column")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_ae}
\alias{data_ae}
\title{Adverse Events Analysis Results Data}
\format{
A data frame with 2,794 rows and 8 variables:
\describe{
\item{AEBODSYS}{highest level row labels: System Organ Class}
\item{AETERM}{more specific row labels: Preferred Term}
\item{col2}{higher level column names (spanners)}
\item{col1}{lower level column names}
\item{param}{parameter to explain each value}
\item{value}{values to put in a table}
\item{ord1}{controls ordering}
\item{ord2}{more ordering controls}
}
}
\usage{
data_ae
}
\description{
A dataset containing the results needed for an AE table. Using the
CDISC pilot data.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_to_gt.R
\name{print_to_gt}
\alias{print_to_gt}
\title{Print to gt}
\usage{
print_to_gt(tfrmt, .data, .unicode_ws = TRUE)
}
\arguments{
\item{tfrmt}{tfrmt object that will dictate the structure of the table}

\item{.data}{Data to style in order to make the table}

\item{.unicode_ws}{Whether to convert white space to unicode in preparation for output}
}
\value{
a stylized gt object
}
\description{
Print to gt
}
\section{Examples}{


\if{html}{\out{<div class="sourceCode r">}}\preformatted{library(dplyr)
# Create tfrmt specification
tfrmt_spec <- tfrmt(
  label = label,
  column = column,
  param = param,
  value=value,
  body_plan = body_plan(
    frmt_structure(group_val = ".default", label_val = ".default",
                   frmt_combine(
                     "\{count\} \{percent\}",
                     count = frmt("xxx"),
                     percent = frmt_when("==100"~ frmt(""),
                                         "==0"~ "",
                                         "TRUE" ~ frmt("(xx.x\%)"))))
  ))

# Create data
df <- tidyr::crossing(label = c("label 1", "label 2"),
               column = c("placebo", "trt1"),
               param = c("count", "percent")) |>
      dplyr::mutate(value=c(24,19,2400/48,1900/38,5,1,500/48,100/38))

print_to_gt(tfrmt_spec,df)

}\if{html}{\out{</div>}}

\if{html}{\out{
<img src="https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/master/images/example_print_to_gt.png" alt = "2 by 2 table with labels down the side and placebo and trt1 across the top" style="width:50\%;">
}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/frmt_utils.R
\name{is_frmt}
\alias{is_frmt}
\alias{is_frmt_combine}
\alias{is_frmt_when}
\alias{is_frmt_structure}
\alias{is_row_grp_structure}
\title{Check if input is a frmt}
\usage{
is_frmt(x)

is_frmt_combine(x)

is_frmt_when(x)

is_frmt_structure(x)

is_row_grp_structure(x)
}
\arguments{
\item{x}{Object to check}
}
\value{
'TRUE' if yes, 'FALSE' if no
}
\description{
Check if input is a frmt

Check if input is a frmt_combine

Check if input is a frmt_when

Check if input is a frmt_structure

Check if input is a row_grp_structure
}
\examples{
x1 <- frmt("XXX.XX")
is_frmt(x1)

x2 <- frmt_combine("XXX \%","XX,XXX")
is_frmt_combine(x2)

x2 <- frmt_when(
">3" ~ frmt("(X.X\%)"),
"<=3" ~ frmt("Undetectable")
)
is_frmt_when(x2)

x3 <- frmt_structure(
 group_val = c("group1"),
 label_val = ".default",
frmt("XXX")
)
is_frmt_structure(x3)

x4 <- row_grp_structure(group_val = c("A","C"), element_block(post_space = "---"))
is_row_grp_structure(x4)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mock_tbl.R
\name{make_mock_data}
\alias{make_mock_data}
\title{Make mock data for display shells}
\usage{
make_mock_data(tfrmt, .default = 1:3, n_cols = NULL)
}
\arguments{
\item{tfrmt}{tfrmt object}

\item{.default}{Number of unique levels to create for group/label values set
to ".default"}

\item{n_cols}{Number of columns in the output table (not including
group/label variables). If not supplied it will default to using the
\code{col_plan} from the \code{tfrmt}. If neither are available it will use 3.}
}
\value{
tibble containing mock data
}
\description{
Make mock data for display shells
}
\examples{

tfrmt_spec <- tfrmt(
  label = label,
  column = column,
  param = param,
  value=value,
  body_plan = body_plan(
    frmt_structure(group_val=".default", label_val=".default", frmt("xx.x"))
    )
  )

make_mock_data(tfrmt_spec)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_card.R
\name{prep_label}
\alias{prep_label}
\title{Prepare label}
\usage{
prep_label(df)
}
\arguments{
\item{df}{(data.frame)}
}
\value{
a data.frame with a \code{label} column (if the input has the required
columns) or the input unchanged.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Adds a \code{label} column which is a combination of \code{stat_label} (for continuous
variables) and \code{variable_level} (for categorical ones) if these 2 columns are
present in the input data frame.
}
\examples{
df <- data.frame(
  variable_level = c("d", "e", "f"),
  stat_label = c("a", "b", "c"),
  stat_name = c("n", "N", "n"),
  context = c("categorical", "continuous", "hierarchical")
)

prep_label(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_card.R
\name{prep_big_n}
\alias{prep_big_n}
\title{Prepare \code{bigN} stat variables}
\usage{
prep_big_n(df, vars)
}
\arguments{
\item{df}{(data.frame)}

\item{vars}{(character) a vector of variables to prepare \code{bigN} for.}
}
\value{
a data.frame with the same columns as the input. The \code{stat_name}
column is modified.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

\code{prep_big_n()}:
\itemize{
\item recodes the \code{"n"} \code{stat_name} into \code{bigN} for the desired variables,
and
\item drops all other \code{stat_names} for the same variables.
}

If your \code{tfrmt} contains a \code{\link[=big_n_structure]{big_n_structure()}} you pass the tfrmt \code{column} to
\code{prep_big_n()} via \code{vars}.
}
\examples{
df <- data.frame(
  stat_name = c("n", "max", "min", rep(c("n", "N", "p"), times = 2)),
  context = rep(c("continuous", "hierarchical", "categorical"), each = 3),
  stat_variable = rep(c("a", "b", "c"), each = 3)
) |>
  dplyr::bind_rows(
    data.frame(
      stat_name = "n",
      context = "total_n",
      stat_variable = "d"
    )
  )

prep_big_n(
  df,
  vars = c("b", "c")
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_to_ggplot.R
\name{print_to_ggplot}
\alias{print_to_ggplot}
\title{Print to ggplot}
\usage{
print_to_ggplot(tfrmt, .data, ...)
}
\arguments{
\item{tfrmt}{tfrmt object that will dictate the structure of the ggplot object}

\item{.data}{Data to style in order to make the ggplot object}

\item{...}{Inputs to geom_text to modify the style of the table body}
}
\value{
a stylized ggplot object
}
\description{
Print to ggplot
}
\section{Examples}{


\if{html}{\out{<div class="sourceCode r">}}\preformatted{
# Create data
risk<-tibble(time=c(rep(c(0,1000,2000,3000),3)),
            label=c(rep("Obs",4),rep("Lev",4),rep("Lev+5FU",4)),
            value=c(630,372,256,11,620,360,266,8,608,425,328,14),
            param=rep("n",12))

table<-tfrmt(
 label = label ,
 column = time,
 param = param,
 value = value) |>
  print_to_ggplot(risk)

table

}\if{html}{\out{</div>}}

\if{html}{\out{
<img src="https://raw.githubusercontent.com/GSK-Biostatistics/tfrmt/master/images/example_print_to_ggplot.png" alt="Simple table to stack with a KM-plot" style="width:100\%;">
}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JSON.R
\name{as_json}
\alias{as_json}
\title{Convert tfrmt elements}
\usage{
as_json(x)
}
\arguments{
\item{x}{tfrmt or tfrmt element}
}
\value{
as_json.tfrmt() will return a json object all other methods will
return a list that is cleaned so it can be directly converted to a json
object
}
\description{
This collection of functions is used to create a json or json ready objects.
While primarily intended for internal use these functions can be used externally
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_labs}
\alias{data_labs}
\title{Labs Analysis Results Data}
\format{
A data frame with 4,950 rows and 7 variables:
\describe{
\item{group1}{highest level row labels: Lab value class}
\item{group2}{more specific row labels: Lab parameter}
\item{rowlbl}{most specific row labels: Study visit}
\item{col1}{higher level column names (spanners)}
\item{col2}{lower level column names}
\item{param}{parameter to explain each value}
\item{value}{values to put in a table}
\item{ord1}{controls ordering}
\item{ord2}{more ordering controls}
\item{ord3}{more ordering controls}
}
}
\usage{
data_labs
}
\description{
A dataset containing the results needed for an labs results table. Using the
CDISC pilot data.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_demog}
\alias{data_demog}
\title{Demography Analysis Results Data}
\format{
A data frame with 386 rows and 7 variables:
\describe{
\item{rowlbl1}{highest level row labels}
\item{rowlbl2}{more specific row labels}
\item{param}{parameter to explain each value}
\item{grp}{grouping column used to distinguish continuous and categorical}
\item{ord1}{controls ordering}
\item{ord2}{more ordering controls}
\item{column}{column names}
\item{value}{values to put in a table}
}
}
\usage{
data_demog
}
\description{
A dataset containing the results needed for a demography table. Using the
CDISC pilot data.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/frmt_plans.R
\name{frmt}
\alias{frmt}
\alias{frmt_combine}
\alias{frmt_when}
\title{Table Value Formatting}
\usage{
frmt(expression, missing = NULL, scientific = NULL, transform = NULL, ...)

frmt_combine(expression, ..., missing = NULL)

frmt_when(..., missing = NULL)
}
\arguments{
\item{expression}{this is the string representing the intended format. See details: expression for more a detailed description.}

\item{missing}{when a value is missing that is intended to be formatted, what value to place. See details: missing for more a detailed description.}

\item{scientific}{a string representing the intended scientific notation to be appended to the expression. Ex. "e^XX" or " x10^XX".}

\item{transform}{this is what should happen to the value prior to formatting,
It should be a formula or function. Ex. \code{~.*100}if you want to convert a
percent from a decimal prior to rounding}

\item{...}{See details: \code{...} for a detailed description.}
}
\value{
frmt object
}
\description{
These functions provide an abstracted way to approach to define formatting of table
contents. By defining in this way, the formats can be
layered to be more specific and general cell styling can be done first.

\code{frmt()} is the base definition of a format. This defines spacing, rounding,
and missing behaviour.

\code{frmt_combine()} is used when two or more rows need to be combined into a
single cell in the table. Each of the rows needs to have a defined \code{frmt()}
and need to share a label.

\code{frmt_when()} is used when a rows format behaviour is dependent on the value itself and is written similarly to \code{\link[dplyr:case_when]{dplyr::case_when()}}.
The left hand side of the equation is a \code{"TRUE"}for the default case or the right hand side of a boolean expression \code{">50"}.
}
\details{
\subsection{expression}{
\itemize{
\item \code{frmt()} All numbers are represented by "x". Any additional character are
printed as-is. If additional X's present to the left of the decimal point
than the value, they will be represented as spaces.
\item \code{frmt_combine()} defines how the parameters will be combined as a
\code{glue::glue()} statement. Parameters need to be equal to the values in the
param column and defined in the expression as \code{"{param1} {param2}"}.
}
}

\subsection{missing}{
\itemize{
\item \code{frmt()} Value to enter when the value is missing. When NULL, the value
is "".
\item \code{frmt_combine()} defines how when all values to be combined are missing.
When NULL the value is "".
}
}

\subsection{...}{
\itemize{
\item \code{frmt()} These dots are for future extensions and must be
empty.
\item \code{frmt_combine()} accepts named arguments defining the \code{frmt()} to
be applied to which parameters before being combined.
\item \code{frmt_when()}accepts a series of equations separated by commas, similar
to \code{\link[dplyr:case_when]{dplyr::case_when()}}. The left hand side of the equation is a \code{"TRUE"}for the
default case or the right hand side of a boolean expression \code{">50"}. The
right hand side of the equation is the \code{frmt()} to apply when the left
side evaluates to \code{TRUE}.
}
}
}
\examples{

frmt("XXX \%")

frmt("XX.XXX")

frmt("xx.xx", scientific = "x10^xx")

frmt_combine(
 "{param1} {param2}",
 param1 = frmt("XXX \%"),
 param2 = frmt("XX.XXX")
)

frmt_when(
  ">3" ~ frmt("(X.X\%)"),
  "<=3" ~ frmt("Undetectable")
  )

frmt_when(
  "==100"~ frmt(""),
  "==0"~ "",
  "TRUE" ~ frmt("(XXX.X\%)")
  )

}
\seealso{
\code{\link[=body_plan]{body_plan()}} combines the frmt_structures to be applied to the
table body, and \code{\link[=frmt_structure]{frmt_structure()}} defines which rows the formats will be applied
to.

\href{https://gsk-biostatistics.github.io/tfrmt/articles/body_plan.html}{Link to related article}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/row_group_plan.R
\name{row_grp_structure}
\alias{row_grp_structure}
\title{Row Group Structure Object}
\usage{
row_grp_structure(group_val = ".default", element_block)
}
\arguments{
\item{group_val}{A string or a named list of strings which represent the
value of group should be when the given frmt is implemented}

\item{element_block}{element_block() object to define the block styling}
}
\value{
row_grp_structure object
}
\description{
Function needed to create a row_grp_structure object, which is a building block
of \code{\link[=row_grp_plan]{row_grp_plan()}}
}
\examples{

## single grouping variable example
row_grp_structure(group_val = c("A","C"), element_block(post_space = "---"))

## example with multiple grouping variables
row_grp_structure(group_val = list(grp1 = "A", grp2 = "b"), element_block(post_space = " "))

}
\seealso{
\code{\link[=row_grp_plan]{row_grp_plan()}} for more details on how to group row group
structures, \code{\link[=element_block]{element_block()}} for more details on how to specify spacing
between each group.

\href{https://gsk-biostatistics.github.io/tfrmt/articles/row_grp_plan.html}{Link to related article}
}
